@@ -15,7 +15,7 @@
  * The Original Code is mozilla.org code.
  *
  * The Initial Developer of the Original Code is
- * Mozilla Corporation.
+ * Mozilla Foundation.
  * Portions created by the Initial Developer are Copyright (C) 2009
  * the Initial Developer. All Rights Reserved.
  *
@@ -38,8 +38,15 @@
 
 #include "nsARIAGridAccessible.h"
 
+#include "AccIterator.h"
+#include "nsAccUtils.h"
+#include "States.h"
+
+#include "nsIMutableArray.h"
 #include "nsComponentManagerUtils.h"
 
+using namespace mozilla::a11y;
+
 ////////////////////////////////////////////////////////////////////////////////
 // nsARIAGridAccessible
 ////////////////////////////////////////////////////////////////////////////////
@@ -48,9 +55,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 // Constructor
 
-nsARIAGridAccessible::nsARIAGridAccessible(nsIDOMNode* aDomNode,
-                                           nsIWeakReference* aShell) :
-  nsAccessibleWrap(aDomNode, aShell)
+nsARIAGridAccessible::
+  nsARIAGridAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsAccessibleWrap(aContent, aShell)
 {
 }
 
@@ -98,9 +105,15 @@ nsARIAGridAccessible::GetColumnCount(PRInt32 *acolumnCount)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row = GetNextRow();
-  nsCOMPtr<nsIAccessible> cell;
-  while ((cell = GetNextCellInRow(row, cell)))
+  AccIterator rowIter(this, filters::GetRow);
+  nsAccessible* row = rowIter.Next();
+  if (!row)
+    return NS_OK;
+
+  AccIterator cellIter(row, filters::GetCell);
+  nsAccessible *cell = nsnull;
+
+  while ((cell = cellIter.Next()))
     (*acolumnCount)++;
 
   return NS_OK;
@@ -115,8 +128,8 @@ nsARIAGridAccessible::GetRowCount(PRInt32 *arowCount)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row;
-  while ((row = GetNextRow(row)))
+  AccIterator rowIter(this, filters::GetRow);
+  while (rowIter.Next())
     (*arowCount)++;
 
   return NS_OK;
@@ -132,10 +145,10 @@ nsARIAGridAccessible::GetCellAt(PRInt32 aRowIndex, PRInt32 aColumnIndex,
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row = GetRowAt(aRowIndex);
+  nsAccessible *row = GetRowAt(aRowIndex);
   NS_ENSURE_ARG(row);
 
-  nsCOMPtr<nsIAccessible> cell = GetCellInRowAt(row, aColumnIndex);
+  nsAccessible *cell = GetCellInRowAt(row, aColumnIndex);
   NS_ENSURE_ARG(cell);
 
   NS_ADDREF(*aAccessible = cell);
@@ -213,6 +226,34 @@ nsARIAGridAccessible::GetRowIndexAt(PRInt32 aCellIndex, PRInt32 *aRowIndex)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsARIAGridAccessible::GetRowAndColumnIndicesAt(PRInt32 aCellIndex,
+                                               PRInt32* aRowIndex,
+                                               PRInt32* aColumnIndex)
+{
+  NS_ENSURE_ARG_POINTER(aRowIndex);
+  *aRowIndex = -1;
+  NS_ENSURE_ARG_POINTER(aColumnIndex);
+  *aColumnIndex = -1;
+
+  if (IsDefunct())
+    return NS_ERROR_FAILURE;
+
+  NS_ENSURE_ARG(aCellIndex >= 0);
+
+  PRInt32 rowCount = 0;
+  GetRowCount(&rowCount);
+
+  PRInt32 colsCount = 0;
+  GetColumnCount(&colsCount);
+
+  NS_ENSURE_ARG(aCellIndex < rowCount * colsCount);
+
+  *aColumnIndex = aCellIndex % colsCount;
+  *aRowIndex = aCellIndex / colsCount;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsARIAGridAccessible::GetColumnExtentAt(PRInt32 aRow, PRInt32 aColumn,
                                         PRInt32 *aExtentCount)
@@ -285,20 +326,21 @@ nsARIAGridAccessible::IsColumnSelected(PRInt32 aColumn, PRBool *aIsSelected)
 
   NS_ENSURE_ARG(IsValidColumn(aColumn));
 
-  nsCOMPtr<nsIAccessible> row = GetNextRow();
+  AccIterator rowIter(this, filters::GetRow);
+  nsAccessible *row = rowIter.Next();
   if (!row)
     return NS_OK;
 
   do {
     if (!nsAccUtils::IsARIASelected(row)) {
-      nsCOMPtr<nsIAccessible> cell = GetCellInRowAt(row, aColumn);
+      nsAccessible *cell = GetCellInRowAt(row, aColumn);
       if (!cell) // Do not fail due to wrong markup
         return NS_OK;
       
       if (!nsAccUtils::IsARIASelected(cell))
         return NS_OK;
     }
-  } while ((row = GetNextRow(row)));
+  } while ((row = rowIter.Next()));
 
   *aIsSelected = PR_TRUE;
   return NS_OK;
@@ -313,12 +355,13 @@ nsARIAGridAccessible::IsRowSelected(PRInt32 aRow, PRBool *aIsSelected)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row = GetRowAt(aRow);
+  nsAccessible *row = GetRowAt(aRow);
   NS_ENSURE_ARG(row);
 
   if (!nsAccUtils::IsARIASelected(row)) {
-    nsCOMPtr<nsIAccessible> cell;
-    while ((cell = GetNextCellInRow(row, cell))) {
+    AccIterator cellIter(row, filters::GetCell);
+    nsAccessible *cell = nsnull;
+    while ((cell = cellIter.Next())) {
       if (!nsAccUtils::IsARIASelected(cell))
         return NS_OK;
     }
@@ -338,11 +381,11 @@ nsARIAGridAccessible::IsCellSelected(PRInt32 aRow, PRInt32 aColumn,
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row(GetRowAt(aRow));
+  nsAccessible *row = GetRowAt(aRow);
   NS_ENSURE_ARG(row);
 
   if (!nsAccUtils::IsARIASelected(row)) {
-    nsCOMPtr<nsIAccessible> cell(GetCellInRowAt(row, aColumn));
+    nsAccessible *cell = GetCellInRowAt(row, aColumn);
     NS_ENSURE_ARG(cell);
 
     if (!nsAccUtils::IsARIASelected(cell))
@@ -364,16 +407,20 @@ nsARIAGridAccessible::GetSelectedCellCount(PRUint32* aCount)
 
   PRInt32 colCount = 0;
   GetColumnCount(&colCount);
-  
-  nsCOMPtr<nsIAccessible> row;
-  while ((row = GetNextRow(row))) {
+
+  AccIterator rowIter(this, filters::GetRow);
+
+  nsAccessible *row = nsnull;
+  while ((row = rowIter.Next())) {
     if (nsAccUtils::IsARIASelected(row)) {
       (*aCount) += colCount;
       continue;
     }
 
-    nsCOMPtr<nsIAccessible> cell;
-    while ((cell = GetNextCellInRow(row, cell))) {
+    AccIterator cellIter(row, filters::GetCell);
+    nsAccessible *cell = nsnull;
+
+    while ((cell = cellIter.Next())) {
       if (nsAccUtils::IsARIASelected(cell))
         (*aCount)++;
     }
@@ -397,14 +444,17 @@ nsARIAGridAccessible::GetSelectedRowCount(PRUint32* aCount)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row;
-  while ((row = GetNextRow(row))) {
+  AccIterator rowIter(this, filters::GetRow);
+
+  nsAccessible *row = nsnull;
+  while ((row = rowIter.Next())) {
     if (nsAccUtils::IsARIASelected(row)) {
       (*aCount)++;
       continue;
     }
 
-    nsCOMPtr<nsIAccessible> cell = GetNextCellInRow(row);
+    AccIterator cellIter(row, filters::GetCell);
+    nsAccessible *cell = cellIter.Next();
     if (!cell)
       continue;
 
@@ -414,7 +464,7 @@ nsARIAGridAccessible::GetSelectedRowCount(PRUint32* aCount)
         isRowSelected = PR_FALSE;
         break;
       }
-    } while ((cell = GetNextCellInRow(row, cell)));
+    } while ((cell = cellIter.Next()));
 
     if (isRowSelected)
       (*aCount)++;
@@ -437,20 +487,23 @@ nsARIAGridAccessible::GetSelectedCells(nsIArray **aCells)
     do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIAccessible> row;
-  while (row = GetNextRow(row)) {
+  AccIterator rowIter(this, filters::GetRow);
+
+  nsAccessible *row = nsnull;
+  while ((row = rowIter.Next())) {
+    AccIterator cellIter(row, filters::GetCell);
+    nsAccessible *cell = nsnull;
+
     if (nsAccUtils::IsARIASelected(row)) {
-      nsCOMPtr<nsIAccessible> cell;
-      while (cell = GetNextCellInRow(row, cell))
-        selCells->AppendElement(cell, PR_FALSE);
+      while ((cell = cellIter.Next()))
+        selCells->AppendElement(static_cast<nsIAccessible *>(cell), PR_FALSE);
 
       continue;
     }
 
-    nsCOMPtr<nsIAccessible> cell;
-    while (cell = GetNextCellInRow(row, cell)) {
+    while ((cell = cellIter.Next())) {
       if (nsAccUtils::IsARIASelected(cell))
-        selCells->AppendElement(cell, PR_FALSE);
+        selCells->AppendElement(static_cast<nsIAccessible *>(cell), PR_FALSE);
     }
   }
 
@@ -478,8 +531,10 @@ nsARIAGridAccessible::GetSelectedCellIndices(PRUint32 *aCellsCount,
 
   nsTArray<PRInt32> selCells(rowCount * colCount);
 
-  nsCOMPtr<nsIAccessible> row;
-  for (PRInt32 rowIdx = 0; row = GetNextRow(row); rowIdx++) {
+  AccIterator rowIter(this, filters::GetRow);
+
+  nsAccessible *row = nsnull;
+  for (PRInt32 rowIdx = 0; (row = rowIter.Next()); rowIdx++) {
     if (nsAccUtils::IsARIASelected(row)) {
       for (PRInt32 colIdx = 0; colIdx < colCount; colIdx++)
         selCells.AppendElement(rowIdx * colCount + colIdx);
@@ -487,8 +542,10 @@ nsARIAGridAccessible::GetSelectedCellIndices(PRUint32 *aCellsCount,
       continue;
     }
 
-    nsCOMPtr<nsIAccessible> cell;
-    for (PRInt32 colIdx = 0; cell = GetNextCellInRow(row, cell); colIdx++) {
+    AccIterator cellIter(row, filters::GetCell);
+    nsAccessible *cell = nsnull;
+
+    for (PRInt32 colIdx = 0; (cell = cellIter.Next()); colIdx++) {
       if (nsAccUtils::IsARIASelected(cell))
         selCells.AppendElement(rowIdx * colCount + colIdx);
     }
@@ -534,14 +591,17 @@ nsARIAGridAccessible::GetSelectedRowIndices(PRUint32 *arowCount,
 
   nsTArray<PRInt32> selRows(rowCount);
 
-  nsCOMPtr<nsIAccessible> row;
-  for (PRInt32 rowIdx = 0; row = GetNextRow(row); rowIdx++) {
+  AccIterator rowIter(this, filters::GetRow);
+
+  nsAccessible *row = nsnull;
+  for (PRInt32 rowIdx = 0; (row = rowIter.Next()); rowIdx++) {
     if (nsAccUtils::IsARIASelected(row)) {
       selRows.AppendElement(rowIdx);
       continue;
     }
 
-    nsCOMPtr<nsIAccessible> cell = GetNextCellInRow(row);
+    AccIterator cellIter(row, filters::GetCell);
+    nsAccessible *cell = cellIter.Next();
     if (!cell)
       continue;
 
@@ -551,7 +611,7 @@ nsARIAGridAccessible::GetSelectedRowIndices(PRUint32 *arowCount,
         isRowSelected = PR_FALSE;
         break;
       }
-    } while ((cell = GetNextCellInRow(row, cell)));
+    } while ((cell = cellIter.Next()));
 
     if (isRowSelected)
       selRows.AppendElement(rowIdx);
@@ -577,8 +637,10 @@ nsARIAGridAccessible::SelectRow(PRInt32 aRow)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row;
-  for (PRInt32 rowIdx = 0; row = GetNextRow(row); rowIdx++) {
+  AccIterator rowIter(this, filters::GetRow);
+
+  nsAccessible *row = nsnull;
+  for (PRInt32 rowIdx = 0; (row = rowIter.Next()); rowIdx++) {
     nsresult rv = SetARIASelected(row, rowIdx == aRow);
     NS_ENSURE_SUCCESS(rv, rv);
   }
@@ -594,14 +656,16 @@ nsARIAGridAccessible::SelectColumn(PRInt32 aColumn)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row;
-  while ((row = GetNextRow(row))) {
+  AccIterator rowIter(this, filters::GetRow);
+
+  nsAccessible *row = nsnull;
+  while ((row = rowIter.Next())) {
     // Unselect all cells in the row.
     nsresult rv = SetARIASelected(row, PR_FALSE);
     NS_ENSURE_SUCCESS(rv, rv);
 
     // Select cell at the column index.
-    nsCOMPtr<nsIAccessible> cell = GetCellInRowAt(row, aColumn);
+    nsAccessible *cell = GetCellInRowAt(row, aColumn);
     if (cell) {
       rv = SetARIASelected(cell, PR_TRUE);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -617,7 +681,7 @@ nsARIAGridAccessible::UnselectRow(PRInt32 aRow)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row = GetRowAt(aRow);
+  nsAccessible *row = GetRowAt(aRow);
   NS_ENSURE_ARG(row);
 
   return SetARIASelected(row, PR_FALSE);
@@ -631,9 +695,11 @@ nsARIAGridAccessible::UnselectColumn(PRInt32 aColumn)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row;
-  while ((row = GetNextRow(row))) {
-    nsCOMPtr<nsIAccessible> cell = GetCellInRowAt(row, aColumn);
+  AccIterator rowIter(this, filters::GetRow);
+
+  nsAccessible *row = nsnull;
+  while ((row = rowIter.Next())) {
+    nsAccessible *cell = GetCellInRowAt(row, aColumn);
     if (cell) {
       nsresult rv = SetARIASelected(cell, PR_FALSE);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -693,92 +759,47 @@ nsARIAGridAccessible::IsValidRowNColumn(PRInt32 aRow, PRInt32 aColumn)
   return aColumn < colCount;
 }
 
-already_AddRefed<nsIAccessible>
+nsAccessible*
 nsARIAGridAccessible::GetRowAt(PRInt32 aRow)
 {
   PRInt32 rowIdx = aRow;
-  nsCOMPtr<nsIAccessible> row(GetNextRow());
-  while (rowIdx != 0 && (row = GetNextRow(row)))
+
+  AccIterator rowIter(this, filters::GetRow);
+
+  nsAccessible *row = rowIter.Next();
+  while (rowIdx != 0 && (row = rowIter.Next()))
     rowIdx--;
 
-  return row.forget();
+  return row;
 }
 
-already_AddRefed<nsIAccessible>
-nsARIAGridAccessible::GetCellInRowAt(nsIAccessible *aRow, PRInt32 aColumn)
+nsAccessible*
+nsARIAGridAccessible::GetCellInRowAt(nsAccessible *aRow, PRInt32 aColumn)
 {
   PRInt32 colIdx = aColumn;
-  nsCOMPtr<nsIAccessible> cell(GetNextCellInRow(aRow));
-  while (colIdx != 0 && (cell = GetNextCellInRow(aRow, cell)))
+
+  AccIterator cellIter(aRow, filters::GetCell);
+  nsAccessible *cell = cellIter.Next();
+  while (colIdx != 0 && (cell = cellIter.Next()))
     colIdx--;
 
-  return cell.forget();
-}
-
-already_AddRefed<nsIAccessible>
-nsARIAGridAccessible::GetNextRow(nsIAccessible *aRow)
-{
-  nsCOMPtr<nsIAccessible> nextRow, tmpAcc;
-  if (!aRow)
-    GetFirstChild(getter_AddRefs(nextRow));
-  else
-    aRow->GetNextSibling(getter_AddRefs(nextRow));
-
-  while (nextRow) {
-    if (nsAccUtils::Role(nextRow) == nsIAccessibleRole::ROLE_ROW)
-      return nextRow.forget();
-
-    nextRow->GetNextSibling(getter_AddRefs(tmpAcc));
-    tmpAcc.swap(nextRow);
-  }
-
-  return nsnull;
-}
-
-already_AddRefed<nsIAccessible>
-nsARIAGridAccessible::GetNextCellInRow(nsIAccessible *aRow, nsIAccessible *aCell)
-{
-  if (!aRow)
-    return nsnull;
-
-  nsCOMPtr<nsIAccessible> nextCell, tmpAcc;
-  if (!aCell)
-    aRow->GetFirstChild(getter_AddRefs(nextCell));
-  else
-    aCell->GetNextSibling(getter_AddRefs(nextCell));
-
-  while (nextCell) {
-    PRUint32 role = nsAccUtils::Role(nextCell);
-    if (role == nsIAccessibleRole::ROLE_GRID_CELL ||
-        role == nsIAccessibleRole::ROLE_ROWHEADER ||
-        role == nsIAccessibleRole::ROLE_COLUMNHEADER)
-      return nextCell.forget();
-
-    nextCell->GetNextSibling(getter_AddRefs(tmpAcc));
-    tmpAcc.swap(nextCell);
-  }
-
-  return nsnull;
+  return cell;
 }
 
 nsresult
-nsARIAGridAccessible::SetARIASelected(nsIAccessible *aAccessible,
+nsARIAGridAccessible::SetARIASelected(nsAccessible *aAccessible,
                                       PRBool aIsSelected, PRBool aNotify)
 {
-  nsRefPtr<nsAccessible> acc = nsAccUtils::QueryAccessible(aAccessible);
-  nsCOMPtr<nsIDOMNode> node;
-  acc->GetDOMNode(getter_AddRefs(node));
-  NS_ENSURE_STATE(node);
-
-  nsCOMPtr<nsIContent> content(do_QueryInterface(node));
+  nsIContent *content = aAccessible->GetContent();
+  NS_ENSURE_STATE(content);
 
   nsresult rv = NS_OK;
   if (aIsSelected)
-    rv = content->SetAttr(kNameSpaceID_None, nsAccessibilityAtoms::aria_selected,
+    rv = content->SetAttr(kNameSpaceID_None, nsGkAtoms::aria_selected,
                           NS_LITERAL_STRING("true"), aNotify);
   else
     rv = content->UnsetAttr(kNameSpaceID_None,
-                            nsAccessibilityAtoms::aria_selected, aNotify);
+                            nsGkAtoms::aria_selected, aNotify);
 
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -792,13 +813,15 @@ nsARIAGridAccessible::SetARIASelected(nsIAccessible *aAccessible,
   if (aIsSelected)
     return NS_OK;
 
-  PRUint32 role = nsAccUtils::Role(aAccessible);
+  PRUint32 role = aAccessible->Role();
 
   // If the given accessible is row that was unselected then remove
   // aria-selected from cell accessible.
   if (role == nsIAccessibleRole::ROLE_ROW) {
-    nsCOMPtr<nsIAccessible> cell;
-    while ((cell = GetNextCellInRow(aAccessible, cell))) {
+    AccIterator cellIter(aAccessible, filters::GetCell);
+    nsAccessible *cell = nsnull;
+
+    while ((cell = cellIter.Next())) {
       rv = SetARIASelected(cell, PR_FALSE, PR_FALSE);
       NS_ENSURE_SUCCESS(rv, rv);
     }
@@ -811,16 +834,16 @@ nsARIAGridAccessible::SetARIASelected(nsIAccessible *aAccessible,
   if (role == nsIAccessibleRole::ROLE_GRID_CELL ||
       role == nsIAccessibleRole::ROLE_ROWHEADER ||
       role == nsIAccessibleRole::ROLE_COLUMNHEADER) {
-    nsCOMPtr<nsIAccessible> row;
-    aAccessible->GetParent(getter_AddRefs(row));
+    nsAccessible* row = aAccessible->Parent();
 
-    if (nsAccUtils::Role(row) == nsIAccessibleRole::ROLE_ROW &&
+    if (row && row->Role() == nsIAccessibleRole::ROLE_ROW &&
         nsAccUtils::IsARIASelected(row)) {
       rv = SetARIASelected(row, PR_FALSE, PR_FALSE);
       NS_ENSURE_SUCCESS(rv, rv);
 
-      nsCOMPtr<nsIAccessible> cell;
-      while ((cell = GetNextCellInRow(row, cell))) {
+      AccIterator cellIter(row, filters::GetCell);
+      nsAccessible *cell = nsnull;
+      while ((cell = cellIter.Next())) {
         if (cell != aAccessible) {
           rv = SetARIASelected(cell, PR_TRUE, PR_FALSE);
           NS_ENSURE_SUCCESS(rv, rv);
@@ -844,7 +867,8 @@ nsARIAGridAccessible::GetSelectedColumnsArray(PRUint32 *acolumnCount,
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row = GetNextRow();
+  AccIterator rowIter(this, filters::GetRow);
+  nsAccessible *row = rowIter.Next();
   if (!row)
     return NS_OK;
 
@@ -865,15 +889,17 @@ nsARIAGridAccessible::GetSelectedColumnsArray(PRUint32 *acolumnCount,
       continue;
 
     PRInt32 colIdx = 0;
-    nsCOMPtr<nsIAccessible> cell;
-    for (colIdx = 0; cell = GetNextCellInRow(row, cell); colIdx++) {
+
+    AccIterator cellIter(row, filters::GetCell);
+    nsAccessible *cell = nsnull;
+    for (colIdx = 0; (cell = cellIter.Next()); colIdx++) {
       if (isColSelArray.SafeElementAt(colIdx, PR_FALSE) &&
           !nsAccUtils::IsARIASelected(cell)) {
         isColSelArray[colIdx] = PR_FALSE;
         selColCount--;
       }
     }
-  } while ((row = GetNextRow(row)));
+  } while ((row = rowIter.Next()));
 
   if (!selColCount)
     return NS_OK;
@@ -905,9 +931,9 @@ nsARIAGridAccessible::GetSelectedColumnsArray(PRUint32 *acolumnCount,
 ////////////////////////////////////////////////////////////////////////////////
 // Constructor
 
-nsARIAGridCellAccessible::nsARIAGridCellAccessible(nsIDOMNode* aDomNode,
-                                                   nsIWeakReference* aShell) :
-  nsHyperTextAccessibleWrap(aDomNode, aShell)
+nsARIAGridCellAccessible::
+  nsARIAGridCellAccessible(nsIContent *aContent, nsIWeakReference *aShell) :
+  nsHyperTextAccessibleWrap(aContent, aShell)
 {
 }
 
@@ -927,15 +953,17 @@ nsARIAGridCellAccessible::GetTable(nsIAccessibleTable **aTable)
   NS_ENSURE_ARG_POINTER(aTable);
   *aTable = nsnull;
 
-  nsCOMPtr<nsIAccessible> thisRow;
-  GetParent(getter_AddRefs(thisRow));
-  if (nsAccUtils::Role(thisRow) != nsIAccessibleRole::ROLE_ROW)
+  nsAccessible* thisRow = Parent();
+  if (!thisRow || thisRow->Role() != nsIAccessibleRole::ROLE_ROW)
     return NS_OK;
 
-  nsCOMPtr<nsIAccessible> table;
-  thisRow->GetParent(getter_AddRefs(table));
-  if (nsAccUtils::Role(table) != nsIAccessibleRole::ROLE_TABLE &&
-      nsAccUtils::Role(table) != nsIAccessibleRole::ROLE_TREE_TABLE)
+  nsAccessible* table = thisRow->Parent();
+  if (!table)
+    return NS_OK;
+
+  PRUint32 tableRole = table->Role();
+  if (tableRole != nsIAccessibleRole::ROLE_TABLE &&
+      tableRole != nsIAccessibleRole::ROLE_TREE_TABLE)
     return NS_OK;
 
   CallQueryInterface(table, aTable);
@@ -951,20 +979,20 @@ nsARIAGridCellAccessible::GetColumnIndex(PRInt32 *aColumnIndex)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
+  nsAccessible* row = Parent();
+  if (!row)
+    return NS_OK;
+
   *aColumnIndex = 0;
 
-  nsCOMPtr<nsIAccessible> prevCell, tmpAcc;
-  GetPreviousSibling(getter_AddRefs(prevCell));
-
-  while (prevCell) {
-    PRUint32 role = nsAccUtils::Role(prevCell);
+  PRInt32 indexInRow = IndexInParent();
+  for (PRInt32 idx = 0; idx < indexInRow; idx++) {
+    nsAccessible* cell = row->GetChildAt(idx);
+    PRUint32 role = cell->Role();
     if (role == nsIAccessibleRole::ROLE_GRID_CELL ||
         role == nsIAccessibleRole::ROLE_ROWHEADER ||
         role == nsIAccessibleRole::ROLE_COLUMNHEADER)
       (*aColumnIndex)++;
-
-    prevCell->GetPreviousSibling(getter_AddRefs(tmpAcc));
-    tmpAcc.swap(prevCell);
   }
 
   return NS_OK;
@@ -979,15 +1007,21 @@ nsARIAGridCellAccessible::GetRowIndex(PRInt32 *aRowIndex)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row, prevRow;
-  GetParent(getter_AddRefs(row));
+  nsAccessible* row = Parent();
+  if (!row)
+    return NS_OK;
 
-  while (row) {
-    if (nsAccUtils::Role(row) == nsIAccessibleRole::ROLE_ROW)
+  nsAccessible* table = row->Parent();
+  if (!table)
+    return NS_OK;
+
+  *aRowIndex = 0;
+
+  PRInt32 indexInTable = row->IndexInParent();
+  for (PRInt32 idx = 0; idx < indexInTable; idx++) {
+    row = table->GetChildAt(idx);
+    if (row->Role() == nsIAccessibleRole::ROLE_ROW)
       (*aRowIndex)++;
-
-    row->GetPreviousSibling(getter_AddRefs(prevRow));
-    row.swap(prevRow);
   }
 
   return NS_OK;
@@ -1066,9 +1100,8 @@ nsARIAGridCellAccessible::IsSelected(PRBool *aIsSelected)
   if (IsDefunct())
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIAccessible> row;
-  GetParent(getter_AddRefs(row));
-  if (nsAccUtils::Role(row) != nsIAccessibleRole::ROLE_ROW)
+  nsAccessible* row = Parent();
+  if (!row || row->Role() != nsIAccessibleRole::ROLE_ROW)
     return NS_OK;
 
   if (!nsAccUtils::IsARIASelected(row) && !nsAccUtils::IsARIASelected(this))
@@ -1081,39 +1114,27 @@ nsARIAGridCellAccessible::IsSelected(PRBool *aIsSelected)
 ////////////////////////////////////////////////////////////////////////////////
 // nsAccessible
 
-nsresult
-nsARIAGridCellAccessible::GetARIAState(PRUint32 *aState, PRUint32 *aExtraState)
+void
+nsARIAGridCellAccessible::ApplyARIAState(PRUint64* aState)
 {
-  nsresult rv = nsHyperTextAccessibleWrap::GetARIAState(aState, aExtraState);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsHyperTextAccessibleWrap::ApplyARIAState(aState);
 
   // Return if the gridcell has aria-selected="true".
-  if (*aState & nsIAccessibleStates::STATE_SELECTED)
-    return NS_OK;
+  if (*aState & states::SELECTED)
+    return;
 
   // Check aria-selected="true" on the row.
-  nsCOMPtr<nsIAccessible> row;
-  GetParent(getter_AddRefs(row));
-  if (nsAccUtils::Role(row) != nsIAccessibleRole::ROLE_ROW)
-    return NS_OK;
+  nsAccessible* row = Parent();
+  if (!row || row->Role() != nsIAccessibleRole::ROLE_ROW)
+    return;
 
-  nsRefPtr<nsAccessible> acc = nsAccUtils::QueryAccessible(row);
-  nsCOMPtr<nsIDOMNode> rowNode;
-  acc->GetDOMNode(getter_AddRefs(rowNode));
-  NS_ENSURE_STATE(rowNode);
-
-  nsCOMPtr<nsIContent> rowContent(do_QueryInterface(rowNode));
+  nsIContent *rowContent = row->GetContent();
   if (nsAccUtils::HasDefinedARIAToken(rowContent,
-                                      nsAccessibilityAtoms::aria_selected) &&
+                                      nsGkAtoms::aria_selected) &&
       !rowContent->AttrValueIs(kNameSpaceID_None,
-                               nsAccessibilityAtoms::aria_selected,
-                               nsAccessibilityAtoms::_false, eCaseMatters)) {
-
-    *aState |= nsIAccessibleStates::STATE_SELECTABLE |
-      nsIAccessibleStates::STATE_SELECTED;
-  }
-
-  return NS_OK;
+                               nsGkAtoms::aria_selected,
+                               nsGkAtoms::_false, eCaseMatters))
+    *aState |= states::SELECTABLE | states::SELECTED;
 }
 
 nsresult
@@ -1127,49 +1148,49 @@ nsARIAGridCellAccessible::GetAttributesInternal(nsIPersistentProperties *aAttrib
 
   // Expose "table-cell-index" attribute.
 
-  nsCOMPtr<nsIAccessible> thisRow;
-  GetParent(getter_AddRefs(thisRow));
-  if (nsAccUtils::Role(thisRow) != nsIAccessibleRole::ROLE_ROW)
+  nsAccessible* thisRow = Parent();
+  if (!thisRow || thisRow->Role() != nsIAccessibleRole::ROLE_ROW)
     return NS_OK;
 
   PRInt32 colIdx = 0, colCount = 0;
-  nsCOMPtr<nsIAccessible> child, nextChild;
-  thisRow->GetFirstChild(getter_AddRefs(child));
-  while (child) {
+  PRInt32 childCount = thisRow->GetChildCount();
+  for (PRInt32 childIdx = 0; childIdx < childCount; childIdx++) {
+    nsAccessible *child = thisRow->GetChildAt(childIdx);
     if (child == this)
       colIdx = colCount;
 
-    PRUint32 role = nsAccUtils::Role(child);
+    PRUint32 role = child->Role();
     if (role == nsIAccessibleRole::ROLE_GRID_CELL ||
         role == nsIAccessibleRole::ROLE_ROWHEADER ||
         role == nsIAccessibleRole::ROLE_COLUMNHEADER)
       colCount++;
-
-    child->GetNextSibling(getter_AddRefs(nextChild));
-    child.swap(nextChild);
   }
 
-  nsCOMPtr<nsIAccessible> table;
-  thisRow->GetParent(getter_AddRefs(table));
-  if (nsAccUtils::Role(table) != nsIAccessibleRole::ROLE_TABLE &&
-      nsAccUtils::Role(table) != nsIAccessibleRole::ROLE_TREE_TABLE)
+  nsAccessible* table = thisRow->Parent();
+  if (!table)
+    return NS_OK;
+
+  PRUint32 tableRole = table->Role();
+  if (tableRole != nsIAccessibleRole::ROLE_TABLE &&
+      tableRole != nsIAccessibleRole::ROLE_TREE_TABLE)
     return NS_OK;
 
   PRInt32 rowIdx = 0;
-  table->GetFirstChild(getter_AddRefs(child));
-  while (child && child != thisRow) {
-    if (nsAccUtils::Role(child) == nsIAccessibleRole::ROLE_ROW)
-      rowIdx++;
+  childCount = table->GetChildCount();
+  for (PRInt32 childIdx = 0; childIdx < childCount; childIdx++) {
+    nsAccessible *child = table->GetChildAt(childIdx);
+    if (child == thisRow)
+      break;
 
-    child->GetNextSibling(getter_AddRefs(nextChild));
-    child.swap(nextChild);
+    if (child->Role() == nsIAccessibleRole::ROLE_ROW)
+      rowIdx++;
   }
 
   PRInt32 idx = rowIdx * colCount + colIdx;
 
   nsAutoString stringIdx;
   stringIdx.AppendInt(idx);
-  nsAccUtils::SetAccAttr(aAttributes, nsAccessibilityAtoms::tableCellIndex,
+  nsAccUtils::SetAccAttr(aAttributes, nsGkAtoms::tableCellIndex,
                          stringIdx);
 
   return NS_OK;
