@@ -32,6 +32,9 @@ function createGroupItemWithTabs(win, width, height, padding, urls, animate) {
     ok(newItem.container, "Created element "+t+":"+newItem.container);
     ++t;
   });
+  // to set one of tabItem to be active since we load tabs into a group 
+  // in a non-standard flow.
+  contentWindow.UI.setActive(groupItem);
   return groupItem;
 }
 
@@ -45,15 +48,16 @@ function createGroupItemWithBlankTabs(win, width, height, padding, numNewTabs, a
 
 // ----------
 function closeGroupItem(groupItem, callback) {
-  groupItem.addSubscriber(groupItem, "close", function () {
-    groupItem.removeSubscriber(groupItem, "close");
-    if ("function" == typeof callback)
+  if (callback) {
+    groupItem.addSubscriber("close", function onClose() {
+      groupItem.removeSubscriber("close", onClose);
       executeSoon(callback);
-  });
+    });
+  }
 
   if (groupItem.getChildren().length) {
-    groupItem.addSubscriber(groupItem, "groupHidden", function () {
-      groupItem.removeSubscriber(groupItem, "groupHidden");
+    groupItem.addSubscriber("groupHidden", function onHide() {
+      groupItem.removeSubscriber("groupHidden", onHide);
       groupItem.closeHidden();
     });
   }
@@ -83,9 +87,11 @@ function newWindowWithTabView(shownCallback, loadCallback, width, height) {
                               ",width=" + winWidth);
 
   whenWindowLoaded(win, function () {
-    if (typeof loadCallback == "function")
+    if (loadCallback)
       loadCallback(win);
+  });
 
+  whenDelayedStartupFinished(win, function () {
     showTabView(function () shownCallback(win), win);
   });
 }
@@ -131,11 +137,14 @@ function showTabView(callback, win) {
   win = win || window;
 
   if (win.TabView.isVisible()) {
-    callback();
+    waitForFocus(callback, win);
     return;
   }
 
-  whenTabViewIsShown(callback, win);
+  whenTabViewIsShown(function () {
+    waitForFocus(callback, win);
+  }, win);
+
   win.TabView.show();
 }
 
@@ -144,11 +153,14 @@ function hideTabView(callback, win) {
   win = win || window;
 
   if (!win.TabView.isVisible()) {
-    callback();
+    if (callback)
+      callback();
     return;
   }
 
-  whenTabViewIsHidden(callback, win);
+  if (callback)
+    whenTabViewIsHidden(callback, win);
+
   win.TabView.hide();
 }
 
@@ -161,8 +173,8 @@ function whenTabViewIsHidden(callback, win) {
     return;
   }
 
-  win.addEventListener('tabviewhidden', function () {
-    win.removeEventListener('tabviewhidden', arguments.callee, false);
+  win.addEventListener('tabviewhidden', function onHidden() {
+    win.removeEventListener('tabviewhidden', onHidden, false);
     callback();
   }, false);
 }
@@ -176,37 +188,26 @@ function whenTabViewIsShown(callback, win) {
     return;
   }
 
-  win.addEventListener('tabviewshown', function () {
-    win.removeEventListener('tabviewshown', arguments.callee, false);
+  win.addEventListener('tabviewshown', function onShown() {
+    win.removeEventListener('tabviewshown', onShown, false);
     callback();
   }, false);
 }
 
-// ----------
-function showSearch(callback, win) {
-  win = win || window;
-
-  let contentWindow = win.TabView.getContentWindow();
-  if (contentWindow.isSearchEnabled()) {
-    callback();
-    return;
-  }
-
-  whenSearchIsEnabled(callback, win);
-  contentWindow.performSearch();
-}
-
 // ----------
 function hideSearch(callback, win) {
   win = win || window;
 
   let contentWindow = win.TabView.getContentWindow();
   if (!contentWindow.isSearchEnabled()) {
-    callback();
+    if (callback)
+      callback();
     return;
   }
 
-  whenSearchIsDisabled(callback, win);
+  if (callback)
+    whenSearchIsDisabled(callback, win);
+
   contentWindow.hideSearch();
 }
 
@@ -220,8 +221,8 @@ function whenSearchIsEnabled(callback, win) {
     return;
   }
 
-  contentWindow.addEventListener("tabviewsearchenabled", function () {
-    contentWindow.removeEventListener("tabviewsearchenabled", arguments.callee, false);
+  contentWindow.addEventListener("tabviewsearchenabled", function onSearchEnabled() {
+    contentWindow.removeEventListener("tabviewsearchenabled", onSearchEnabled, false);
     callback();
   }, false);
 }
@@ -236,8 +237,8 @@ function whenSearchIsDisabled(callback, win) {
     return;
   }
 
-  contentWindow.addEventListener("tabviewsearchdisabled", function () {
-    contentWindow.removeEventListener("tabviewsearchdisabled", arguments.callee, false);
+  contentWindow.addEventListener("tabviewsearchdisabled", function onSearchDisabled() {
+    contentWindow.removeEventListener("tabviewsearchdisabled", onSearchDisabled, false);
     callback();
   }, false);
 }
@@ -246,28 +247,36 @@ function whenSearchIsDisabled(callback, win) {
 // ----------
 function hideGroupItem(groupItem, callback) {
   if (groupItem.hidden) {
-    callback();
+    if (callback)
+      callback();
     return;
   }
 
-  groupItem.addSubscriber(groupItem, "groupHidden", function () {
-    groupItem.removeSubscriber(groupItem, "groupHidden");
-    callback();
-  });
+  if (callback) {
+    groupItem.addSubscriber("groupHidden", function onHide() {
+      groupItem.removeSubscriber("groupHidden", onHide);
+      callback();
+    });
+  }
+
   groupItem.closeAll();
 }
 
 // ----------
 function unhideGroupItem(groupItem, callback) {
   if (!groupItem.hidden) {
-    callback();
+    if (callback)
+      callback();
     return;
   }
 
-  groupItem.addSubscriber(groupItem, "groupShown", function () {
-    groupItem.removeSubscriber(groupItem, "groupShown");
-    callback();
-  });
+  if (callback) {
+    groupItem.addSubscriber("groupShown", function onShown() {
+      groupItem.removeSubscriber("groupShown", onShown);
+      callback();
+    });
+  }
+
   groupItem._unhide();
 }
 
@@ -340,8 +349,26 @@ function restoreTab(callback, index, win) {
     return;
   }
 
-  tab._tabViewTabItem.addSubscriber(tab, "reconnected", function onReconnected() {
-    tab._tabViewTabItem.removeSubscriber(tab, "reconnected");
+  tab._tabViewTabItem.addSubscriber("reconnected", function onReconnected() {
+    tab._tabViewTabItem.removeSubscriber("reconnected", onReconnected);
     finalize();
   });
 }
+
+// ----------
+function togglePrivateBrowsing(callback) {
+  let topic = "private-browsing-transition-complete";
+
+  Services.obs.addObserver(function observe() {
+    Services.obs.removeObserver(observe, topic);
+
+    // use executeSoon() to let Panorama load its group data from the session
+    // before we call afterAllTabsLoaded()
+    executeSoon(function () afterAllTabsLoaded(callback));
+  }, topic, false);
+
+  let pb = Cc["@mozilla.org/privatebrowsing;1"].
+           getService(Ci.nsIPrivateBrowsingService);
+
+  pb.privateBrowsingEnabled = !pb.privateBrowsingEnabled;
+}
