@@ -92,18 +92,28 @@ Site.prototype = {
    *        A callback function that takes a favicon image URL as a parameter.
    */
   getFavicon: function Site_getFavicon(aCallback) {
+    let callbackExecuted = false;
     function faviconDataCallback(aURI, aDataLen, aData, aMimeType) {
+      // We don't need a second callback, so we can ignore it to avoid making
+      // a second database query for the favicon data.
+      if (callbackExecuted) {
+        return;
+      }
       try {
-        aCallback(aURI.spec);
+        // Use getFaviconLinkForIcon to get image data from the database instead
+        // of using the favicon URI to fetch image data over the network.
+        aCallback(gFaviconService.getFaviconLinkForIcon(aURI).spec);
+        callbackExecuted = true;
       } catch (e) {
         Cu.reportError("AboutPermissions: " + e);
       }
     }
 
     // Try to find favicion for both URIs. Callback will only be called if a
-    // favicon URI is found, so this means we'll always prefer the https favicon.
-    gFaviconService.getFaviconURLForPage(this.httpURI, faviconDataCallback);
+    // favicon URI is found. We'll ignore the second callback if it is called,
+    // so this means we'll always prefer the https favicon.
     gFaviconService.getFaviconURLForPage(this.httpsURI, faviconDataCallback);
+    gFaviconService.getFaviconURLForPage(this.httpURI, faviconDataCallback);
   },
 
   /**
@@ -234,10 +244,8 @@ Site.prototype = {
    * @return An array of the logins stored for the site.
    */
   get logins() {
-    // There could be more logins for different schemes/ports, but this covers
-    // the vast majority of cases.
-    let httpLogins = Services.logins.findLogins({}, this.httpURI.prePath, "", null);
-    let httpsLogins = Services.logins.findLogins({}, this.httpsURI.prePath, "", null);
+    let httpLogins = Services.logins.findLogins({}, this.httpURI.prePath, "", "");
+    let httpsLogins = Services.logins.findLogins({}, this.httpsURI.prePath, "", "");
     return httpLogins.concat(httpsLogins);
   },
 
