@@ -27,6 +27,7 @@
  *  Asaf Romano    <mano@mozilla.com>
  *  Ryan Jones     <sciguyryan@gmail.com>
  *  Paul O'Shannessy <paul@oshannessy.com>
+ *  Jim Mathies    <jmathies@mozilla.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -63,6 +64,8 @@
 #include "nsDirectoryServiceDefs.h"
 #include "nsIWindowsRegKey.h"
 #include "nsUnicharUtils.h"
+#include "nsIWinTaskbar.h"
+#include "nsISupportsPrimitives.h"
 
 #include "windows.h"
 #include "shellapi.h"
@@ -86,11 +89,9 @@
 #define REG_FAILED(val) \
   (val != ERROR_SUCCESS)
 
-#ifndef WINCE
+#define NS_TASKBAR_CONTRACTID "@mozilla.org/windows-taskbar;1"
+
 NS_IMPL_ISUPPORTS2(nsWindowsShellService, nsIWindowsShellService, nsIShellService)
-#else
-NS_IMPL_ISUPPORTS1(nsWindowsShellService, nsIShellService)
-#endif
 
 static nsresult
 OpenKeyForReading(HKEY aKeyRoot, const nsAString& aKeyName, HKEY* aKey)
@@ -110,33 +111,6 @@ OpenKeyForReading(HKEY aKeyRoot, const nsAString& aKeyName, HKEY* aKey)
   return NS_OK;
 }
 
-#ifdef WINCE
-static nsresult
-OpenKeyForWriting(HKEY aStartKey, const nsAString& aKeyName, HKEY* aKey)
-{
-  const nsString &flatName = PromiseFlatString(aKeyName);
-
-  DWORD dwDisp = 0;
-  DWORD res = ::RegCreateKeyExW(aStartKey, flatName.get(), 0, NULL,
-                                0, KEY_READ | KEY_WRITE, NULL, aKey,
-                                &dwDisp);
-  switch (res) {
-  case ERROR_SUCCESS:
-    break;
-  case ERROR_ACCESS_DENIED:
-    return NS_ERROR_FILE_ACCESS_DENIED;
-  case ERROR_FILE_NOT_FOUND:
-    res = ::RegCreateKeyExW(aStartKey, flatName.get(), 0, NULL,
-                            0, KEY_READ | KEY_WRITE, NULL, aKey,
-                            NULL);
-    if (res != ERROR_SUCCESS)
-      return NS_ERROR_FILE_ACCESS_DENIED;
-  }
-
-  return NS_OK;
-}
-#endif
-
 ///////////////////////////////////////////////////////////////////////////////
 // Default Browser Registry Settings
 //
@@ -212,23 +186,15 @@ typedef struct {
   char* valueData;
 } SETTING;
 
-#ifndef WINCE
 #define APP_REG_NAME L"Firefox"
 #define CLS_HTML "FirefoxHTML"
 #define CLS_URL "FirefoxURL"
 #define CPL_DESKTOP L"Control Panel\\Desktop"
 #define VAL_OPEN "\"%APPPATH%\" -requestPending -osint -url \"%1\""
 #define VAL_FILE_ICON "%APPPATH%,1"
-#else
-#define CPL_DESKTOP L"ControlPanel\\Desktop"
-#define VAL_OPEN "\"%APPPATH%\" -osint -url \"%1\""
-#define VAL_FILE_ICON "%APPPATH%,-2"
-#endif
-
 #define DI "\\DefaultIcon"
 #define SOP "\\shell\\open\\command"
 
-
 #define MAKE_KEY_NAME1(PREFIX, MID) \
   PREFIX MID
 
@@ -238,28 +204,11 @@ typedef struct {
 // Handlers. see http://msdn2.microsoft.com/en-us/library/aa969357.aspx for
 // more info.
 static SETTING gSettings[] = {
-#ifndef WINCE
   // File Handler Class
   { MAKE_KEY_NAME1(CLS_HTML, SOP), "", VAL_OPEN },
 
   // Protocol Handler Class - for Vista and above
   { MAKE_KEY_NAME1(CLS_URL, SOP), "", VAL_OPEN },
-#else
-  { MAKE_KEY_NAME1("FTP", DI), "", VAL_FILE_ICON },
-  { MAKE_KEY_NAME1("FTP", SOP), "", VAL_OPEN },
-
-  // File handlers for Windows CE
-  { MAKE_KEY_NAME1("bmpfile", DI), "", VAL_FILE_ICON },
-  { MAKE_KEY_NAME1("bmpfile", SOP), "", VAL_OPEN },
-  { MAKE_KEY_NAME1("giffile", DI), "", VAL_FILE_ICON },
-  { MAKE_KEY_NAME1("giffile", SOP), "", VAL_OPEN },
-  { MAKE_KEY_NAME1("jpegfile", DI), "", VAL_FILE_ICON },
-  { MAKE_KEY_NAME1("jpegfile", SOP), "", VAL_OPEN },
-  { MAKE_KEY_NAME1("pngfile", DI), "", VAL_FILE_ICON },
-  { MAKE_KEY_NAME1("pngfile", SOP), "", VAL_OPEN },
-  { MAKE_KEY_NAME1("htmlfile", DI), "", VAL_FILE_ICON },
-  { MAKE_KEY_NAME1("htmlfile", SOP), "", VAL_OPEN },
-#endif
 
   // Protocol Handlers
   { MAKE_KEY_NAME1("HTTP", DI),    "", VAL_FILE_ICON },
@@ -268,7 +217,120 @@ static SETTING gSettings[] = {
   { MAKE_KEY_NAME1("HTTPS", SOP),  "", VAL_OPEN }
 };
 
-#ifndef WINCE
+nsresult
+GetHelperPath(nsAutoString& aPath)
+{
+  nsresult rv;
+  nsCOMPtr<nsIProperties> directoryService = 
+    do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsILocalFile> appHelper;
+  rv = directoryService->Get(NS_XPCOM_CURRENT_PROCESS_DIR,
+                             NS_GET_IID(nsILocalFile),
+                             getter_AddRefs(appHelper));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  rv = appHelper->AppendNative(NS_LITERAL_CSTRING("uninstall"));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  rv = appHelper->AppendNative(NS_LITERAL_CSTRING("helper.exe"));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return appHelper->GetPath(aPath);
+}
+
+nsresult
+LaunchHelper(nsAutoString& aPath)
+{
+  STARTUPINFOW si = {sizeof(si), 0};
+  PROCESS_INFORMATION pi = {0};
+
+  BOOL ok = CreateProcessW(NULL, (LPWSTR)aPath.get(), NULL, NULL,
+                           FALSE, 0, NULL, NULL, &si, &pi);
+
+  if (!ok)
+    return NS_ERROR_FAILURE;
+
+  CloseHandle(pi.hProcess);
+  CloseHandle(pi.hThread);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsWindowsShellService::ShortcutMaintenance()
+{
+  nsresult rv;
+
+  // Launch helper.exe so it can update the application user model ids on
+  // shortcuts in the user's taskbar and start menu. This keeps older pinned
+  // shortcuts grouped correctly after major updates. Note, we also do this
+  // through the upgrade installer script, however, this is the only place we
+  // have a chance to trap links created by users who do control the install/
+  // update process of the browser.
+
+  nsCOMPtr<nsIWinTaskbar> taskbarInfo =
+    do_GetService(NS_TASKBAR_CONTRACTID);
+  if (!taskbarInfo) // If we haven't built with win7 sdk features, this fails.
+    return NS_OK;
+
+  // Avoid if this isn't Win7+
+  PRBool isSupported = PR_FALSE;
+  taskbarInfo->GetAvailable(&isSupported);
+  if (!isSupported)
+    return NS_OK;
+
+  nsAutoString appId;
+  if (NS_FAILED(taskbarInfo->GetDefaultGroupId(appId)))
+    return NS_ERROR_UNEXPECTED;
+
+  NS_NAMED_LITERAL_CSTRING(prefName, "browser.taskbar.lastgroupid");
+  nsCOMPtr<nsIPrefService> prefs =
+    do_GetService(NS_PREFSERVICE_CONTRACTID);
+  if (!prefs)
+    return NS_ERROR_UNEXPECTED;
+
+  nsCOMPtr<nsIPrefBranch> prefBranch;
+  prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+  if (!prefBranch)
+    return NS_ERROR_UNEXPECTED;
+
+  nsCOMPtr<nsISupportsString> prefString;
+  rv = prefBranch->GetComplexValue(prefName.get(),
+                                   NS_GET_IID(nsISupportsString),
+                                   getter_AddRefs(prefString));
+  if (NS_SUCCEEDED(rv)) {
+    nsAutoString version;
+    prefString->GetData(version);
+    if (!version.IsEmpty() && version.Equals(appId)) {
+      // We're all good, get out of here.
+      return NS_OK;
+    }
+  }
+  // Update the version in prefs
+  prefString =
+    do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, &rv);
+  if (NS_FAILED(rv))
+    return rv;
+
+  prefString->SetData(appId);
+  rv = prefBranch->SetComplexValue(prefName.get(),
+                                   NS_GET_IID(nsISupportsString),
+                                   prefString);
+  if (NS_FAILED(rv)) {
+    NS_WARNING("Couldn't set last user model id!");
+    return NS_ERROR_UNEXPECTED;
+  }
+
+  nsAutoString appHelperPath;
+  if (NS_FAILED(GetHelperPath(appHelperPath)))
+    return NS_ERROR_UNEXPECTED;
+
+  appHelperPath.AppendLiteral(" /UpdateShortcutAppUserModelIds");
+
+  return LaunchHelper(appHelperPath);
+}
+
 PRBool
 nsWindowsShellService::IsDefaultBrowserVista(PRBool* aIsDefaultBrowser)
 {
@@ -282,9 +344,11 @@ nsWindowsShellService::IsDefaultBrowserVista(PRBool* aIsDefaultBrowser)
                                 (void**)&pAAR);
 
   if (SUCCEEDED(hr)) {
+    BOOL res;
     hr = pAAR->QueryAppIsDefaultAll(AL_EFFECTIVE,
                                     APP_REG_NAME,
-                                    aIsDefaultBrowser);
+                                    &res);
+    *aIsDefaultBrowser = res;
 
     pAAR->Release();
     return PR_TRUE;
@@ -292,7 +356,6 @@ nsWindowsShellService::IsDefaultBrowserVista(PRBool* aIsDefaultBrowser)
 #endif  
   return PR_FALSE;
 }
-#endif
 
 NS_IMETHODIMP
 nsWindowsShellService::IsDefaultBrowser(PRBool aStartupCheck,
@@ -313,12 +376,10 @@ nsWindowsShellService::IsDefaultBrowser(PRBool aStartupCheck,
   if (!::GetModuleFileNameW(0, exePath, MAX_BUF))
     return NS_ERROR_FAILURE;
 
-#ifndef WINCE
   // Convert the path to a long path since GetModuleFileNameW returns the path
   // that was used to launch Firefox which is not necessarily a long path.
   if (!::GetLongPathNameW(exePath, exePath, MAX_BUF))
     return NS_ERROR_FAILURE;
-#endif
 
   nsAutoString appLongPath(exePath);
 
@@ -352,12 +413,10 @@ nsWindowsShellService::IsDefaultBrowser(PRBool aStartupCheck,
     }
   }
 
-#ifndef WINCE
   // Only check if Firefox is the default browser on Vista if the previous
   // checks show that Firefox is the default browser.
   if (*aIsDefaultBrowser)
     IsDefaultBrowserVista(aIsDefaultBrowser);
-#endif
 
   return NS_OK;
 }
@@ -365,25 +424,9 @@ nsWindowsShellService::IsDefaultBrowser(PRBool aStartupCheck,
 NS_IMETHODIMP
 nsWindowsShellService::SetDefaultBrowser(PRBool aClaimAllTypes, PRBool aForAllUsers)
 {
-#ifndef WINCE
-  nsresult rv;
-  nsCOMPtr<nsIProperties> directoryService = 
-    do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsILocalFile> appHelper;
-  rv = directoryService->Get(NS_XPCOM_CURRENT_PROCESS_DIR, NS_GET_IID(nsILocalFile), getter_AddRefs(appHelper));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = appHelper->AppendNative(NS_LITERAL_CSTRING("uninstall"));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = appHelper->AppendNative(NS_LITERAL_CSTRING("helper.exe"));
-  NS_ENSURE_SUCCESS(rv, rv);
-
   nsAutoString appHelperPath;
-  rv = appHelper->GetPath(appHelperPath);
-  NS_ENSURE_SUCCESS(rv, rv);
+  if (NS_FAILED(GetHelperPath(appHelperPath)))
+    return NS_ERROR_FAILURE;
 
   if (aForAllUsers) {
     appHelperPath.AppendLiteral(" /SetAsDefaultAppGlobal");
@@ -391,102 +434,9 @@ nsWindowsShellService::SetDefaultBrowser(PRBool aClaimAllTypes, PRBool aForAllUs
     appHelperPath.AppendLiteral(" /SetAsDefaultAppUser");
   }
 
-  STARTUPINFOW si = {sizeof(si), 0};
-  PROCESS_INFORMATION pi = {0};
-
-  BOOL ok = CreateProcessW(NULL, (LPWSTR)appHelperPath.get(), NULL, NULL,
-                           FALSE, 0, NULL, NULL, &si, &pi);
-
-  if (!ok)
-    return NS_ERROR_FAILURE;
-
-  CloseHandle(pi.hProcess);
-  CloseHandle(pi.hThread);
-#else
-  SETTING* settings;
-  SETTING* end = gSettings + sizeof(gSettings)/sizeof(SETTING);
-
-  PRUnichar exePath[MAX_BUF];
-  if (!::GetModuleFileNameW(0, exePath, MAX_BUF))
-    return NS_ERROR_FAILURE;
-
-  nsAutoString appLongPath(exePath);
-
-  // The .png registry key isn't present by default so also add Content Type.
-  SetRegKey(NS_LITERAL_STRING(".png"), EmptyString(),
-            NS_LITERAL_STRING("pngfile"));
-  SetRegKey(NS_LITERAL_STRING(".png"), NS_LITERAL_STRING("Content Type"),
-            NS_LITERAL_STRING("image/png"));
-
-  // Set these keys to their default value for a clean install in case another
-  // app has changed these keys.
-  SetRegKey(NS_LITERAL_STRING(".htm"), EmptyString(),
-            NS_LITERAL_STRING("htmlfile"));
-  SetRegKey(NS_LITERAL_STRING(".html"), EmptyString(),
-            NS_LITERAL_STRING("htmlfile"));
-  SetRegKey(NS_LITERAL_STRING(".bmp"), EmptyString(),
-            NS_LITERAL_STRING("bmpfile"));
-  SetRegKey(NS_LITERAL_STRING(".gif"), EmptyString(),
-            NS_LITERAL_STRING("giffile"));
-  SetRegKey(NS_LITERAL_STRING(".jpe"), EmptyString(),
-            NS_LITERAL_STRING("jpegfile"));
-  SetRegKey(NS_LITERAL_STRING(".jpg"), EmptyString(),
-            NS_LITERAL_STRING("jpegfile"));
-  SetRegKey(NS_LITERAL_STRING(".jpeg"), EmptyString(),
-            NS_LITERAL_STRING("jpegfile"));
-
-  for (settings = gSettings; settings < end; ++settings) {
-    NS_ConvertUTF8toUTF16 dataLongPath(settings->valueData);
-    NS_ConvertUTF8toUTF16 key(settings->keyName);
-    NS_ConvertUTF8toUTF16 value(settings->valueName);
-    PRInt32 offset = dataLongPath.Find("%APPPATH%");
-    dataLongPath.Replace(offset, 9, appLongPath);
-    SetRegKey(key, value, dataLongPath);
-  }
-  // On Windows CE RegFlushKey can negatively impact performance if there are a
-  // lot of pending writes to the HKEY_CLASSES_ROOT registry hive but it is
-  // necessary to save the values in the case where the user performs a hard
-  // power off of the device.
-  ::RegFlushKey(HKEY_CLASSES_ROOT);
-#endif
-
-  return NS_OK;
+  return LaunchHelper(appHelperPath);
 }
 
-#ifdef WINCE
-void
-nsWindowsShellService::SetRegKey(const nsString& aKeyName,
-                                 const nsString& aValueName,
-                                 const nsString& aValue)
-{
-  PRUnichar buf[MAX_BUF];
-  DWORD len = sizeof buf;
-
-  HKEY theKey;
-  nsresult rv = OpenKeyForWriting(HKEY_CLASSES_ROOT, aKeyName, &theKey);
-  if (NS_FAILED(rv))
-    return;
-
-  // Get the current value.
-  DWORD res = ::RegQueryValueExW(theKey, PromiseFlatString(aValueName).get(),
-                                 NULL, NULL, (LPBYTE)buf, &len);
-
-  // Set the new value if it doesn't exist or it is different than the current
-  // value.
-  nsAutoString current(buf);
-  if (REG_FAILED(res) || !current.Equals(aValue)) {
-    const nsString &flatValue = PromiseFlatString(aValue);
-
-    ::RegSetValueExW(theKey, PromiseFlatString(aValueName).get(),
-                     0, REG_SZ, (const BYTE *)flatValue.get(),
-                     (flatValue.Length() + 1) * sizeof(PRUnichar));
-  }
-
-  // Close the key we opened.
-  ::RegCloseKey(theKey);
-}
-#endif
-
 NS_IMETHODIMP
 nsWindowsShellService::GetShouldCheckDefaultBrowser(PRBool* aResult)
 {
@@ -524,7 +474,9 @@ static nsresult
 WriteBitmap(nsIFile* aFile, imgIContainer* aImage)
 {
   nsRefPtr<gfxImageSurface> image;
-  nsresult rv = aImage->CopyCurrentFrame(getter_AddRefs(image));
+  nsresult rv = aImage->CopyFrame(imgIContainer::FRAME_FIRST,
+                                  imgIContainer::FLAG_SYNC_DECODE,
+                                  getter_AddRefs(image));
   NS_ENSURE_SUCCESS(rv, rv);
 
   PRInt32 width = image->Width();
@@ -602,6 +554,7 @@ nsWindowsShellService::SetDesktopBackground(nsIDOMElement* aElement,
   nsCOMPtr<nsIDOMHTMLImageElement> imgElement(do_QueryInterface(aElement));
   if (!imgElement) {
     // XXX write background loading stuff!
+    return NS_ERROR_NOT_AVAILABLE;
   } 
   else {
     nsCOMPtr<nsIImageLoadingContent> imageContent =
@@ -664,7 +617,6 @@ nsWindowsShellService::SetDesktopBackground(nsIDOMElement* aElement,
                                    0, NULL, REG_OPTION_NON_VOLATILE,
                                    KEY_WRITE, NULL, &key, &dwDisp);
     if (REG_SUCCEEDED(res)) {
-#ifndef WINCE
       PRUnichar tile[2], style[2];
       switch (aPosition) {
         case BACKGROUND_TILE:
@@ -691,25 +643,9 @@ nsWindowsShellService::SetDesktopBackground(nsIDOMElement* aElement,
                        0, REG_SZ, (const BYTE *)style, size);
       ::SystemParametersInfoW(SPI_SETDESKWALLPAPER, 0, (PVOID)path.get(),
                               SPIF_UPDATEINIFILE | SPIF_SENDCHANGE);
-#else
-      DWORD tile = (aPosition == BACKGROUND_TILE);
-      ::RegSetValueExW(key, L"Tile",
-                       0, REG_DWORD, (const BYTE *)&tile, sizeof(DWORD));
-      // On WinCE SPI_SETDESKWALLPAPER isn't available, so set the registry
-      // entry ourselves and then broadcast UI change
-      PRInt32 size = (path.Length() + 1) * sizeof(PRUnichar);
-      ::RegSetValueExW(key, L"Wallpaper",
-                       0, REG_SZ, (const BYTE *)path.get(), size);
-      ::SendMessage(HWND_BROADCAST, WM_SETTINGCHANGE, NULL, 0);
-#endif
 
       // Close the key we opened.
       ::RegCloseKey(key);
-
-#ifdef WINCE
-      // Ensure that the writes are flushed in case of hard reboot
-      ::RegFlushKey(HKEY_CURRENT_USER);
-#endif
     }
   }
   return rv;
@@ -835,9 +771,6 @@ nsWindowsShellService::SetDesktopBackgroundColor(PRUint32 aColor)
 
   ::SetSysColors(sizeof(aParameters) / sizeof(int), aParameters, colors);
 
-  // SetSysColors is persisting across sessions on Windows CE, so no need to
-  // write to registry
-#ifndef WINCE
   PRBool result = PR_FALSE;
   DWORD  dwDisp = 0;
   HKEY   key;
@@ -859,11 +792,9 @@ nsWindowsShellService::SetDesktopBackgroundColor(PRUint32 aColor)
   
   // Close the key we opened.
   ::RegCloseKey(key);
-#endif
   return NS_OK;
 }
 
-#ifndef WINCE
 NS_IMETHODIMP
 nsWindowsShellService::GetUnreadMailCount(PRUint32* aCount)
 {
@@ -923,7 +854,6 @@ nsWindowsShellService::GetMailAccountKey(HKEY* aResult)
   ::RegCloseKey(mailKey);
   return PR_FALSE;
 }
-#endif
 
 NS_IMETHODIMP
 nsWindowsShellService::OpenApplicationWithURI(nsILocalFile* aApplication,
