@@ -51,6 +51,7 @@
 #include "nsMappedAttributes.h"
 #include "nsUnicharUtils.h"
 #include "nsAutoPtr.h"
+#include "nsContentUtils.h" // nsAutoScriptBlocker
 
 /*
 CACHE_POINTER_SHIFT indicates how many steps to downshift the |this| pointer.
@@ -146,9 +147,11 @@ nsAttrAndChildArray::GetSafeChildAt(PRUint32 aPos) const
 }
 
 nsIContent * const *
-nsAttrAndChildArray::GetChildArray() const
+nsAttrAndChildArray::GetChildArray(PRUint32* aChildCount) const
 {
-  if (!mImpl) {
+  *aChildCount = ChildCount();
+  
+  if (!*aChildCount) {
     return nsnull;
   }
   
@@ -173,8 +176,7 @@ nsAttrAndChildArray::InsertChildAt(nsIContent* aChild, PRUint32 aPos)
     if (childCount != aPos) {
       memmove(pos + 1, pos, (childCount - aPos) * sizeof(nsIContent*));
     }
-    *pos = aChild;
-    NS_ADDREF(aChild);
+    SetChildAtPos(pos, aChild, aPos, childCount);
 
     SetChildCount(childCount + 1);
 
@@ -189,10 +191,9 @@ nsAttrAndChildArray::InsertChildAt(nsIContent* aChild, PRUint32 aPos)
     void** newStart = mImpl->mBuffer + attrCount * ATTRSIZE;
     void** oldStart = mImpl->mBuffer + offset;
     memmove(newStart, oldStart, aPos * sizeof(nsIContent*));
-    newStart[aPos] = aChild;
     memmove(&newStart[aPos + 1], &oldStart[aPos],
             (childCount - aPos) * sizeof(nsIContent*));
-    NS_ADDREF(aChild);
+    SetChildAtPos(newStart + aPos, aChild, aPos, childCount);
 
     SetAttrSlotAndChildCount(attrCount, childCount + 1);
 
@@ -208,8 +209,7 @@ nsAttrAndChildArray::InsertChildAt(nsIContent* aChild, PRUint32 aPos)
   if (childCount != aPos) {
     memmove(pos + 1, pos, (childCount - aPos) * sizeof(nsIContent*));
   }
-  *pos = aChild;
-  NS_ADDREF(aChild);
+  SetChildAtPos(pos, aChild, aPos, childCount);
 
   SetChildCount(childCount + 1);
   
@@ -218,15 +218,32 @@ nsAttrAndChildArray::InsertChildAt(nsIContent* aChild, PRUint32 aPos)
 
 void
 nsAttrAndChildArray::RemoveChildAt(PRUint32 aPos)
+{
+  // Just store the return value of TakeChildAt in an nsCOMPtr to
+  // trigger a release.
+  nsCOMPtr<nsIContent> child = TakeChildAt(aPos);
+}
+
+already_AddRefed<nsIContent>
+nsAttrAndChildArray::TakeChildAt(PRUint32 aPos)
 {
   NS_ASSERTION(aPos < ChildCount(), "out-of-bounds");
 
   PRUint32 childCount = ChildCount();
   void** pos = mImpl->mBuffer + AttrSlotsSize() + aPos;
   nsIContent* child = static_cast<nsIContent*>(*pos);
-  NS_RELEASE(child);
+  if (child->mPreviousSibling) {
+    child->mPreviousSibling->mNextSibling = child->mNextSibling;
+  }
+  if (child->mNextSibling) {
+    child->mNextSibling->mPreviousSibling = child->mPreviousSibling;
+  }
+  child->mPreviousSibling = child->mNextSibling = nsnull;
+
   memmove(pos, pos + 1, (childCount - aPos - 1) * sizeof(nsIContent*));
   SetChildCount(childCount - 1);
+
+  return child;
 }
 
 PRInt32
@@ -314,7 +331,7 @@ nsAttrAndChildArray::GetAttr(nsIAtom* aLocalName, PRInt32 aNamespaceID) const
   PRUint32 i, slotCount = AttrSlotCount();
   if (aNamespaceID == kNameSpaceID_None) {
     // This should be the common case so lets make an optimized loop
-    for (i = 0; i < slotCount && mImpl->mBuffer[i * ATTRSIZE]; ++i) {
+    for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
       if (ATTRS(mImpl)[i].mName.Equals(aLocalName)) {
         return &ATTRS(mImpl)[i].mValue;
       }
@@ -325,7 +342,7 @@ nsAttrAndChildArray::GetAttr(nsIAtom* aLocalName, PRInt32 aNamespaceID) const
     }
   }
   else {
-    for (i = 0; i < slotCount && mImpl->mBuffer[i * ATTRSIZE]; ++i) {
+    for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
       if (ATTRS(mImpl)[i].mName.Equals(aLocalName, aNamespaceID)) {
         return &ATTRS(mImpl)[i].mValue;
       }
@@ -353,7 +370,7 @@ nsresult
 nsAttrAndChildArray::SetAttr(nsIAtom* aLocalName, const nsAString& aValue)
 {
   PRUint32 i, slotCount = AttrSlotCount();
-  for (i = 0; i < slotCount && mImpl->mBuffer[i * ATTRSIZE]; ++i) {
+  for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
     if (ATTRS(mImpl)[i].mName.Equals(aLocalName)) {
       ATTRS(mImpl)[i].mValue.SetTo(aValue);
 
@@ -378,7 +395,7 @@ nsresult
 nsAttrAndChildArray::SetAndTakeAttr(nsIAtom* aLocalName, nsAttrValue& aValue)
 {
   PRUint32 i, slotCount = AttrSlotCount();
-  for (i = 0; i < slotCount && mImpl->mBuffer[i * ATTRSIZE]; ++i) {
+  for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
     if (ATTRS(mImpl)[i].mName.Equals(aLocalName)) {
       ATTRS(mImpl)[i].mValue.Reset();
       ATTRS(mImpl)[i].mValue.SwapValueWith(aValue);
@@ -411,7 +428,7 @@ nsAttrAndChildArray::SetAndTakeAttr(nsINodeInfo* aName, nsAttrValue& aValue)
   }
 
   PRUint32 i, slotCount = AttrSlotCount();
-  for (i = 0; i < slotCount && mImpl->mBuffer[i * ATTRSIZE]; ++i) {
+  for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
     if (ATTRS(mImpl)[i].mName.Equals(localName, namespaceID)) {
       ATTRS(mImpl)[i].mName.SetTo(aName);
       ATTRS(mImpl)[i].mValue.Reset();
@@ -511,10 +528,10 @@ nsAttrAndChildArray::GetSafeAttrNameAt(PRUint32 aPos) const
 }
 
 const nsAttrName*
-nsAttrAndChildArray::GetExistingAttrNameFromQName(const nsACString& aName) const
+nsAttrAndChildArray::GetExistingAttrNameFromQName(const nsAString& aName) const
 {
   PRUint32 i, slotCount = AttrSlotCount();
-  for (i = 0; i < slotCount && mImpl->mBuffer[i * ATTRSIZE]; ++i) {
+  for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
     if (ATTRS(mImpl)[i].mName.QualifiedNameEquals(aName)) {
       return &ATTRS(mImpl)[i].mName;
     }
@@ -543,14 +560,14 @@ nsAttrAndChildArray::IndexOfAttr(nsIAtom* aLocalName, PRInt32 aNamespaceID) cons
   PRUint32 slotCount = AttrSlotCount();
   if (aNamespaceID == kNameSpaceID_None) {
     // This should be the common case so lets make an optimized loop
-    for (i = 0; i < slotCount && mImpl->mBuffer[i * ATTRSIZE]; ++i) {
+    for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
       if (ATTRS(mImpl)[i].mName.Equals(aLocalName)) {
         return i + mapped;
       }
     }
   }
   else {
-    for (i = 0; i < slotCount && mImpl->mBuffer[i * ATTRSIZE]; ++i) {
+    for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
       if (ATTRS(mImpl)[i].mName.Equals(aLocalName, aNamespaceID)) {
         return i + mapped;
       }
@@ -567,7 +584,13 @@ nsAttrAndChildArray::SetAndTakeMappedAttr(nsIAtom* aLocalName,
                                           nsHTMLStyleSheet* aSheet)
 {
   nsRefPtr<nsMappedAttributes> mapped;
-  nsresult rv = GetModifiableMapped(aContent, aSheet, PR_TRUE,
+
+  PRBool willAdd = PR_TRUE;
+  if (mImpl && mImpl->mMappedAttrs) {
+    willAdd = mImpl->mMappedAttrs->GetAttr(aLocalName) == nsnull;
+  }
+
+  nsresult rv = GetModifiableMapped(aContent, aSheet, willAdd,
                                     getter_AddRefs(mapped));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -598,7 +621,7 @@ nsAttrAndChildArray::SetMappedAttrStyleSheet(nsHTMLStyleSheet* aSheet)
 void
 nsAttrAndChildArray::WalkMappedAttributeStyleRules(nsRuleWalker* aRuleWalker)
 {
-  if (mImpl && mImpl->mMappedAttrs && aRuleWalker) {
+  if (mImpl && mImpl->mMappedAttrs) {
     aRuleWalker->Forward(mImpl->mMappedAttrs);
   }
 }
@@ -648,7 +671,7 @@ nsAttrAndChildArray::Clear()
   }
 
   PRUint32 i, slotCount = AttrSlotCount();
-  for (i = 0; i < slotCount && mImpl->mBuffer[i * ATTRSIZE]; ++i) {
+  for (i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
     ATTRS(mImpl)[i].~InternalAttr();
   }
 
@@ -659,6 +682,18 @@ nsAttrAndChildArray::Clear()
     // making this PR_FALSE so tree teardown doesn't end up being
     // O(N*D) (number of nodes times average depth of tree).
     child->UnbindFromTree(PR_FALSE); // XXX is it better to let the owner do this?
+    // Make sure to unlink our kids from each other, since someone
+    // else could stil be holding references to some of them.
+
+    // XXXbz We probably can't push this assignment down into the |aNullParent|
+    // case of UnbindFromTree because we still need the assignment in
+    // RemoveChildAt.  In particular, ContentRemoved fires between
+    // RemoveChildAt and UnbindFromTree, and in ContentRemoved the sibling
+    // chain needs to be correct.  Though maybe we could set the prev and next
+    // to point to each other but keep the kid being removed pointing to them
+    // through ContentRemoved so consumers can find where it used to be in the
+    // list?
+    child->mPreviousSibling = child->mNextSibling = nsnull;
     NS_RELEASE(child);
   }
 
@@ -766,16 +801,14 @@ nsAttrAndChildArray::GrowBy(PRUint32 aGrowSize)
     size = PR_BIT(PR_CeilingLog2(minSize));
   }
 
-  Impl* newImpl = static_cast<Impl*>
-                             (mImpl ? PR_Realloc(mImpl, size * sizeof(void*)) :
-              PR_Malloc(size * sizeof(void*)));
+  PRBool needToInitialize = !mImpl;
+  Impl* newImpl = static_cast<Impl*>(PR_Realloc(mImpl, size * sizeof(void*)));
   NS_ENSURE_TRUE(newImpl, PR_FALSE);
 
-  Impl* oldImpl = mImpl;
   mImpl = newImpl;
 
   // Set initial counts if we didn't have a buffer before
-  if (!oldImpl) {
+  if (needToInitialize) {
     mImpl->mMappedAttrs = nsnull;
     SetAttrSlotAndChildCount(0, 0);
   }
@@ -809,3 +842,48 @@ nsAttrAndChildArray::AddAttrSlot()
 
   return PR_TRUE;
 }
+
+inline void
+nsAttrAndChildArray::SetChildAtPos(void** aPos, nsIContent* aChild,
+                                   PRUint32 aIndex, PRUint32 aChildCount)
+{
+  NS_PRECONDITION(!aChild->GetNextSibling(), "aChild with next sibling?");
+  NS_PRECONDITION(!aChild->GetPreviousSibling(), "aChild with prev sibling?");
+
+  *aPos = aChild;
+  NS_ADDREF(aChild);
+  if (aIndex != 0) {
+    nsIContent* previous = static_cast<nsIContent*>(*(aPos - 1));
+    aChild->mPreviousSibling = previous;
+    previous->mNextSibling = aChild;
+  }
+  if (aIndex != aChildCount) {
+    nsIContent* next = static_cast<nsIContent*>(*(aPos + 1));
+    aChild->mNextSibling = next;
+    next->mPreviousSibling = aChild;
+  }
+}
+
+PRInt64
+nsAttrAndChildArray::SizeOf() const
+{
+  PRInt64 size = sizeof(*this);
+
+  if (mImpl) {
+    // Don't add the size taken by *mMappedAttrs because it's shared.
+
+    // mBuffer cointains InternalAttr and nsIContent* (even if it's void**)
+    // so, we just have to compute the size of *mBuffer given that this object
+    // doesn't own the children list.
+    size += mImpl->mBufferSize * sizeof(*(mImpl->mBuffer)) + NS_IMPL_EXTRA_SIZE;
+
+    PRUint32 slotCount = AttrSlotCount();
+    for (PRUint32 i = 0; i < slotCount && AttrSlotIsTaken(i); ++i) {
+      nsAttrValue* value = &ATTRS(mImpl)[i].mValue;
+      size += value->SizeOf() - sizeof(*value);
+    }
+  }
+
+  return size;
+}
+
