@@ -49,12 +49,17 @@
 #include "nsIDragService.h"
 #include "nsIScriptableRegion.h"
 #include "nsContentUtils.h"
+#include "nsIContent.h"
+#include "nsCRT.h"
+#include "nsIScriptObjectPrincipal.h"
 
 NS_IMPL_CYCLE_COLLECTION_2(nsDOMDataTransfer, mDragTarget, mDragImage)
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(nsDOMDataTransfer)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(nsDOMDataTransfer)
 
+DOMCI_DATA(DataTransfer, nsDOMDataTransfer)
+
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsDOMDataTransfer)
   NS_INTERFACE_MAP_ENTRY(nsIDOMDataTransfer)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNSDataTransfer)
@@ -80,25 +85,23 @@ nsDOMDataTransfer::nsDOMDataTransfer()
 {
 }
 
-nsDOMDataTransfer::nsDOMDataTransfer(PRUint32 aEventType, PRUint32 aAction)
+nsDOMDataTransfer::nsDOMDataTransfer(PRUint32 aEventType)
   : mEventType(aEventType),
     mDropEffect(nsIDragService::DRAGDROP_ACTION_NONE),
+    mEffectAllowed(nsIDragService::DRAGDROP_ACTION_UNINITIALIZED),
+    mCursorState(PR_FALSE),
     mReadOnly(PR_TRUE),
     mIsExternal(PR_TRUE),
     mUserCancelled(PR_FALSE),
     mDragImageX(0),
     mDragImageY(0)
 {
-  mEffectAllowed = aAction &
-                   (nsIDragService::DRAGDROP_ACTION_COPY |
-                    nsIDragService::DRAGDROP_ACTION_LINK |
-                    nsIDragService::DRAGDROP_ACTION_MOVE);
-
   CacheExternalFormats();
 }
 
 nsDOMDataTransfer::nsDOMDataTransfer(PRUint32 aEventType,
                                      const PRUint32 aEffectAllowed,
+                                     PRBool aCursorState,
                                      PRBool aIsExternal,
                                      PRBool aUserCancelled,
                                      nsTArray<nsTArray<TransferItem> >& aItems,
@@ -108,6 +111,7 @@ nsDOMDataTransfer::nsDOMDataTransfer(PRUint32 aEventType,
   : mEventType(aEventType),
     mDropEffect(nsIDragService::DRAGDROP_ACTION_NONE),
     mEffectAllowed(aEffectAllowed),
+    mCursorState(aCursorState),
     mReadOnly(PR_TRUE),
     mIsExternal(aIsExternal),
     mUserCancelled(aUserCancelled),
@@ -251,8 +255,7 @@ nsDOMDataTransfer::GetFiles(nsIDOMFileList** aFileList)
       if (!file)
         continue;
 
-      nsRefPtr<nsDOMFile> domFile = new nsDOMFile(file);
-      NS_ENSURE_TRUE(domFile, NS_ERROR_OUT_OF_MEMORY);
+      nsRefPtr<nsDOMFileFile> domFile = new nsDOMFileFile(file);
 
       if (!mFiles->Append(domFile))
         return NS_ERROR_FAILURE;
@@ -324,7 +327,7 @@ nsDOMDataTransfer::GetData(const nsAString& aFormat, nsAString& aData)
             aData.Assign(Substring(stringdata, lastidx));
           else
             aData.Assign(Substring(stringdata, lastidx, idx - lastidx));
-          aData = nsContentUtils::TrimWhitespace(aData, PR_TRUE);
+          aData = nsContentUtils::TrimWhitespace<nsCRT::IsAsciiSpace>(aData, PR_TRUE);
           return NS_OK;
         }
         lastidx = idx + 1;
@@ -383,6 +386,24 @@ nsDOMDataTransfer::SetMozCursor(const nsAString& aCursorState)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsDOMDataTransfer::GetMozSourceNode(nsIDOMNode** aSourceNode)
+{
+  *aSourceNode = nsnull;
+
+  nsCOMPtr<nsIDragSession> dragSession = nsContentUtils::GetDragSession();
+  if (!dragSession)
+    return NS_OK;
+
+  nsCOMPtr<nsIDOMNode> sourceNode;
+  dragSession->GetSourceNode(getter_AddRefs(sourceNode));
+  if (sourceNode && !nsContentUtils::CanCallerAccess(sourceNode))
+    return NS_OK;
+
+  sourceNode.swap(*aSourceNode);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsDOMDataTransfer::MozTypesAt(PRUint32 aIndex, nsIDOMDOMStringList** aTypes)
 {
@@ -427,8 +448,11 @@ nsDOMDataTransfer::MozGetDataAt(const nsAString& aFormat,
   // data from the same principal.
   nsIPrincipal* principal = nsnull;
   if (mEventType != NS_DRAGDROP_DROP && mEventType != NS_DRAGDROP_DRAGDROP &&
-      !nsContentUtils::IsCallerTrustedForCapability("UniversalBrowserRead"))
-    principal = GetCurrentPrincipal();
+      !nsContentUtils::IsCallerTrustedForCapability("UniversalBrowserRead")) {
+    nsresult rv = NS_OK;
+    principal = GetCurrentPrincipal(&rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
 
   PRUint32 count = item.Length();
   for (PRUint32 i = 0; i < count; i++) {
@@ -439,8 +463,29 @@ nsDOMDataTransfer::MozGetDataAt(const nsAString& aFormat,
           (NS_FAILED(principal->Subsumes(formatitem.mPrincipal, &subsumes)) || !subsumes))
         return NS_ERROR_DOM_SECURITY_ERR;
 
-      if (!formatitem.mData)
+      if (!formatitem.mData) {
         FillInExternalDragData(formatitem, aIndex);
+      } else {
+        nsCOMPtr<nsISupports> data;
+        formatitem.mData->GetAsISupports(getter_AddRefs(data));
+        // Make sure the code that is calling us is same-origin with the data.
+        nsCOMPtr<nsIDOMEventTarget> pt = do_QueryInterface(data);
+        if (pt) {
+          nsresult rv = NS_OK;
+          nsIScriptContext* c = pt->GetContextForEventHandlers(&rv);
+          NS_ENSURE_TRUE(c && NS_SUCCEEDED(rv), NS_ERROR_DOM_SECURITY_ERR);
+          nsIScriptObjectPrincipal* sp = c->GetObjectPrincipal();
+          NS_ENSURE_TRUE(sp, NS_ERROR_DOM_SECURITY_ERR);
+          nsIPrincipal* dataPrincipal = sp->GetPrincipal();
+          NS_ENSURE_TRUE(dataPrincipal, NS_ERROR_DOM_SECURITY_ERR);
+          NS_ENSURE_TRUE(principal || (principal = GetCurrentPrincipal(&rv)),
+                         NS_ERROR_DOM_SECURITY_ERR);
+          NS_ENSURE_SUCCESS(rv, rv);
+          PRBool equals = PR_FALSE;
+          NS_ENSURE_TRUE(NS_SUCCEEDED(principal->Equals(dataPrincipal, &equals)) && equals,
+                         NS_ERROR_DOM_SECURITY_ERR);
+        }
+      }
       *aData = formatitem.mData;
       NS_IF_ADDREF(*aData);
       return NS_OK;
@@ -476,7 +521,10 @@ nsDOMDataTransfer::MozSetDataAt(const nsAString& aFormat,
     return NS_ERROR_DOM_SECURITY_ERR;
   }
 
-  return SetDataWithPrincipal(aFormat, aData, aIndex, GetCurrentPrincipal());
+  nsresult rv = NS_OK;
+  nsIPrincipal* principal = GetCurrentPrincipal(&rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+  return SetDataWithPrincipal(aFormat, aData, aIndex, principal);
 }
 
 NS_IMETHODIMP
@@ -491,7 +539,9 @@ nsDOMDataTransfer::MozClearDataAt(const nsAString& aFormat, PRUint32 aIndex)
   nsAutoString format;
   GetRealFormat(aFormat, format);
 
-  nsIPrincipal* principal = GetCurrentPrincipal();
+  nsresult rv = NS_OK;
+  nsIPrincipal* principal = GetCurrentPrincipal(&rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // if the format is empty, clear all formats
   PRBool clearall = format.IsEmpty();
@@ -530,6 +580,10 @@ nsDOMDataTransfer::SetDragImage(nsIDOMElement* aImage, PRInt32 aX, PRInt32 aY)
   if (mReadOnly)
     return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
 
+  if (aImage) {
+    nsCOMPtr<nsIContent> content = do_QueryInterface(aImage);
+    NS_ENSURE_TRUE(content, NS_ERROR_INVALID_ARG);
+  }
   mDragImage = aImage;
   mDragImageX = aX;
   mDragImageY = aY;
@@ -554,8 +608,9 @@ nsDOMDataTransfer::Clone(PRUint32 aEventType, PRBool aUserCancelled,
                          nsIDOMDataTransfer** aNewDataTransfer)
 {
   nsDOMDataTransfer* newDataTransfer =
-    new nsDOMDataTransfer(aEventType, mEffectAllowed, mIsExternal, aUserCancelled,
-                          mItems, mDragImage, mDragImageX, mDragImageY);
+    new nsDOMDataTransfer(aEventType, mEffectAllowed, mCursorState,
+                          mIsExternal, aUserCancelled, mItems,
+                          mDragImage, mDragImageX, mDragImageY);
   NS_ENSURE_TRUE(newDataTransfer, NS_ERROR_OUT_OF_MEMORY);
 
   *aNewDataTransfer = newDataTransfer;
@@ -669,16 +724,19 @@ nsDOMDataTransfer::ConvertFromVariant(nsIVariant* aVariant,
   }
 
   PRUnichar* chrs;
-  nsresult rv = aVariant->GetAsWString(&chrs);
+  PRUint32 len = 0;
+  nsresult rv = aVariant->GetAsWStringWithSize(&len, &chrs);
   if (NS_FAILED(rv))
     return PR_FALSE;
 
+  nsAutoString str;
+  str.Adopt(chrs, len);
+
   nsCOMPtr<nsISupportsString>
     strSupports(do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID));
   if (!strSupports)
     return PR_FALSE;
 
-  nsAutoString str(chrs);
   strSupports->SetData(str);
 
   *aSupports = strSupports;
@@ -749,12 +807,14 @@ nsDOMDataTransfer::SetDataWithPrincipal(const nsAString& aFormat,
 }
 
 nsIPrincipal*
-nsDOMDataTransfer::GetCurrentPrincipal()
+nsDOMDataTransfer::GetCurrentPrincipal(nsresult* rv)
 {
   nsIScriptSecurityManager* ssm = nsContentUtils::GetSecurityManager();
 
   nsCOMPtr<nsIPrincipal> currentPrincipal;
-  ssm->GetSubjectPrincipal(getter_AddRefs(currentPrincipal));
+  *rv = ssm->GetSubjectPrincipal(getter_AddRefs(currentPrincipal));
+  NS_ENSURE_SUCCESS(*rv, nsnull);
+
   if (!currentPrincipal)
     ssm->GetSystemPrincipal(getter_AddRefs(currentPrincipal));
 
@@ -782,13 +842,7 @@ nsDOMDataTransfer::CacheExternalFormats()
   // asked for, as it may be time consuming for the source application to
   // generate it.
 
-  nsCOMPtr<nsIDragService> dragService =
-    do_GetService("@mozilla.org/widget/dragservice;1");
-  if (!dragService)
-    return;
-
-  nsCOMPtr<nsIDragSession> dragSession;
-  dragService->GetCurrentSession(getter_AddRefs(dragSession));
+  nsCOMPtr<nsIDragSession> dragSession = nsContentUtils::GetDragSession();
   if (!dragSession)
     return;
 
@@ -846,13 +900,7 @@ nsDOMDataTransfer::FillInExternalDragData(TransferItem& aItem, PRUint32 aIndex)
     else if (strcmp(format, "text/uri-list") == 0)
       format = kURLDataMime;
 
-    nsCOMPtr<nsIDragService> dragService =
-      do_GetService("@mozilla.org/widget/dragservice;1");
-    if (!dragService)
-      return;
-
-    nsCOMPtr<nsIDragSession> dragSession;
-    dragService->GetCurrentSession(getter_AddRefs(dragSession));
+    nsCOMPtr<nsIDragSession> dragSession = nsContentUtils::GetDragSession();
     if (!dragSession)
       return;
 
