@@ -37,6 +37,8 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "base/basictypes.h"
+#include "IPC/IPCMessageUtils.h"
 #include "nsCOMPtr.h"
 #include "nsDOMEvent.h"
 #include "nsEventStateManager.h"
@@ -44,8 +46,6 @@
 #include "nsIContent.h"
 #include "nsIPresShell.h"
 #include "nsIDocument.h"
-#include "nsIPrivateCompositionEvent.h"
-#include "nsIDOMEventTarget.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "prmem.h"
@@ -55,13 +55,20 @@
 #include "nsIURI.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsIScriptError.h"
+#include "nsDOMPopStateEvent.h"
+#include "mozilla/Preferences.h"
+
+using namespace mozilla;
 
 static const char* const sEventNames[] = {
   "mousedown", "mouseup", "click", "dblclick", "mouseover",
-  "mouseout", "mousemove", "contextmenu", "keydown", "keyup", "keypress",
-  "focus", "blur", "load", "beforeunload", "unload", "hashchange", "abort", "error",
-  "submit", "reset", "change", "select", "input", "paint" ,"text",
-  "compositionstart", "compositionend", "popupshowing", "popupshown",
+  "mouseout", "MozMouseHittest", "mousemove", "contextmenu", "keydown", "keyup", "keypress",
+  "focus", "blur", "load", "popstate", "beforescriptexecute",
+  "afterscriptexecute", "beforeunload", "unload",
+  "hashchange", "readystatechange", "abort", "error",
+  "submit", "reset", "change", "select", "input", "invalid", "text",
+  "compositionstart", "compositionend", "compositionupdate",
+  "popupshowing", "popupshown",
   "popuphiding", "popuphidden", "close", "command", "broadcast", "commandupdate",
   "dragenter", "dragover", "dragexit", "dragdrop", "draggesture",
   "drag", "dragend", "dragstart", "dragleave", "drop", "resize",
@@ -71,18 +78,22 @@ static const char* const sEventNames[] = {
   "DOMAttrModified", "DOMCharacterDataModified",
   "DOMActivate", "DOMFocusIn", "DOMFocusOut",
   "pageshow", "pagehide", "DOMMouseScroll", "MozMousePixelScroll",
-  "offline", "online", "copy", "cut", "paste",
-#ifdef MOZ_SVG
+  "offline", "online", "copy", "cut", "paste", "open", "message", "show",
   "SVGLoad", "SVGUnload", "SVGAbort", "SVGError", "SVGResize", "SVGScroll",
   "SVGZoom",
-#endif // MOZ_SVG
+#ifdef MOZ_SMIL
+  "beginEvent", "endEvent", "repeatEvent",
+#endif // MOZ_SMIL
 #ifdef MOZ_MEDIA
   "loadstart", "progress", "suspend", "emptied", "stalled", "play", "pause",
   "loadedmetadata", "loadeddata", "waiting", "playing", "canplay",
   "canplaythrough", "seeking", "seeked", "timeupdate", "ended", "ratechange",
-  "durationchange", "volumechange",
+  "durationchange", "volumechange", "MozAudioAvailable",
 #endif // MOZ_MEDIA
   "MozAfterPaint",
+  "MozBeforePaint",
+  "MozBeforeResize",
+  "mozfullscreenchange",
   "MozSwipeGesture",
   "MozMagnifyGestureStart",
   "MozMagnifyGestureUpdate",
@@ -92,7 +103,16 @@ static const char* const sEventNames[] = {
   "MozRotateGesture",
   "MozTapGesture",
   "MozPressTapGesture",
-  "MozScrolledAreaChanged"
+  "MozTouchDown",
+  "MozTouchMove",
+  "MozTouchUp",
+  "MozScrolledAreaChanged",
+  "transitionend",
+  "animationstart",
+  "animationend",
+  "animationiteration",
+  "devicemotion",
+  "deviceorientation"
 };
 
 static char *sPopupAllowedEvents;
@@ -139,13 +159,15 @@ nsDOMEvent::nsDOMEvent(nsPresContext* aPresContext, nsEvent* aEvent)
 
   // Get the explicit original target (if it's anonymous make it null)
   {
-    mExplicitOriginalTarget = GetTargetFromFrame();
-    mTmpRealOriginalTarget = mExplicitOriginalTarget;
-    nsCOMPtr<nsIContent> content = do_QueryInterface(mExplicitOriginalTarget);
+    nsCOMPtr<nsIContent> content = GetTargetFromFrame();
+    mTmpRealOriginalTarget = do_QueryInterface(content);
+    mExplicitOriginalTarget = mTmpRealOriginalTarget;
     if (content && content->IsInAnonymousSubtree()) {
       mExplicitOriginalTarget = nsnull;
     }
   }
+
+  NS_ASSERTION(mEvent->message != NS_PAINT, "Trying to create a DOM paint event!");
 }
 
 nsDOMEvent::~nsDOMEvent() 
@@ -159,12 +181,14 @@ nsDOMEvent::~nsDOMEvent()
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(nsDOMEvent)
 
+DOMCI_DATA(Event, nsDOMEvent)
+
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsDOMEvent)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIDOMEvent)
   NS_INTERFACE_MAP_ENTRY(nsIDOMEvent)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNSEvent)
   NS_INTERFACE_MAP_ENTRY(nsIPrivateDOMEvent)
-  NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(Event)
+  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(Event)
 NS_INTERFACE_MAP_END
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(nsDOMEvent)
@@ -179,6 +203,7 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsDOMEvent)
       case NS_MOUSE_EVENT:
       case NS_MOUSE_SCROLL_EVENT:
       case NS_SIMPLE_GESTURE_EVENT:
+      case NS_MOZTOUCH_EVENT:
         static_cast<nsMouseEvent_base*>(tmp->mEvent)->relatedTarget = nsnull;
         break;
       case NS_DRAG_EVENT:
@@ -194,7 +219,9 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsDOMEvent)
   }
   NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mPresContext);
   NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mTmpRealOriginalTarget)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mExplicitOriginalTarget)
+  // Always set mExplicitOriginalTarget to null, when 
+  // mTmpRealOriginalTarget doesn't point to any object!
+  tmp->mExplicitOriginalTarget = nsnull;
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsDOMEvent)
@@ -206,6 +233,7 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsDOMEvent)
       case NS_MOUSE_EVENT:
       case NS_MOUSE_SCROLL_EVENT:
       case NS_SIMPLE_GESTURE_EVENT:
+      case NS_MOZTOUCH_EVENT:
         NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "mEvent->relatedTarget");
         cb.NoteXPCOMChild(
           static_cast<nsMouseEvent_base*>(tmp->mEvent)->relatedTarget);
@@ -227,41 +255,42 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsDOMEvent)
         break;
     }
   }
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mPresContext)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NATIVE_MEMBER(mPresContext.get(), nsPresContext)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mTmpRealOriginalTarget)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mExplicitOriginalTarget)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 // nsIDOMEventInterface
 NS_METHOD nsDOMEvent::GetType(nsAString& aType)
 {
+  if (!mCachedType.IsEmpty()) {
+    aType = mCachedType;
+    return NS_OK;
+  }
   const char* name = GetEventName(mEvent->message);
 
   if (name) {
     CopyASCIItoUTF16(name, aType);
+    mCachedType = aType;
     return NS_OK;
   } else if (mEvent->message == NS_USER_DEFINED_EVENT && mEvent->userType) {
-    nsAutoString name;
-    mEvent->userType->ToString(name);
-    aType = Substring(name, 2, name.Length() - 2); // Remove "on"
+    aType = Substring(nsDependentAtomString(mEvent->userType), 2); // Remove "on"
+    mCachedType = aType;
     return NS_OK;
   }
-  
-  return NS_ERROR_FAILURE;
+
+  aType.Truncate();
+  return NS_OK;
 }
 
 static nsresult
-GetDOMEventTarget(nsISupports* aTarget,
+GetDOMEventTarget(nsIDOMEventTarget* aTarget,
                   nsIDOMEventTarget** aDOMTarget)
 {
-  nsCOMPtr<nsPIDOMEventTarget> piTarget = do_QueryInterface(aTarget);
-  nsISupports* realTarget =
-    piTarget ? piTarget->GetTargetForDOMEvent() : aTarget;
-  if (realTarget) {
-    return CallQueryInterface(realTarget, aDOMTarget);
-  }
+  nsIDOMEventTarget* realTarget =
+    aTarget ? aTarget->GetTargetForDOMEvent() : aTarget;
+
+  NS_IF_ADDREF(*aDOMTarget = realTarget);
 
-  *aDOMTarget = nsnull;
   return NS_OK;
 }
 
@@ -280,25 +309,19 @@ nsDOMEvent::GetCurrentTarget(nsIDOMEventTarget** aCurrentTarget)
 //
 // Get the actual event target node (may have been retargeted for mouse events)
 //
-already_AddRefed<nsIDOMEventTarget>
+already_AddRefed<nsIContent>
 nsDOMEvent::GetTargetFromFrame()
 {
   if (!mPresContext) { return nsnull; }
 
   // Get the target frame (have to get the ESM first)
-  nsIFrame* targetFrame = nsnull;
-  mPresContext->EventStateManager()->GetEventTarget(&targetFrame);
+  nsIFrame* targetFrame = mPresContext->EventStateManager()->GetEventTarget();
   if (!targetFrame) { return nsnull; }
 
   // get the real content
   nsCOMPtr<nsIContent> realEventContent;
   targetFrame->GetContentForEvent(mPresContext, mEvent, getter_AddRefs(realEventContent));
-  if (!realEventContent) { return nsnull; }
-
-  // Finally, we have the real content.  QI it and return.
-  nsIDOMEventTarget* target = nsnull;
-  CallQueryInterface(realEventContent, &target);
-  return target;
+  return realEventContent.forget();
 }
 
 NS_IMETHODIMP
@@ -383,7 +406,7 @@ nsDOMEvent::GetCancelable(PRBool* aCancelable)
 NS_IMETHODIMP
 nsDOMEvent::GetTimeStamp(PRUint64* aTimeStamp)
 {
-  LL_UI2L(*aTimeStamp, mEvent->time);
+  *aTimeStamp = mEvent->time;
   return NS_OK;
 }
 
@@ -420,10 +443,10 @@ ReportUseOfDeprecatedMethod(nsEvent* aEvent, nsIDOMEvent* aDOMEvent,
   nsContentUtils::ReportToConsole(nsContentUtils::eDOM_PROPERTIES,
                                   aWarning,
                                   strings, NS_ARRAY_LENGTH(strings),
-                                  doc ? doc->GetDocumentURI() : nsnull,
+                                  nsnull,
                                   EmptyString(), 0, 0,
                                   nsIScriptError::warningFlag,
-                                  "DOM Events");
+                                  "DOM Events", doc);
 }
 
 NS_IMETHODIMP
@@ -476,230 +499,9 @@ nsDOMEvent::PreventDefault()
 nsresult
 nsDOMEvent::SetEventType(const nsAString& aEventTypeArg)
 {
-  nsCOMPtr<nsIAtom> atom = do_GetAtom(NS_LITERAL_STRING("on") + aEventTypeArg);
-  mEvent->message = NS_USER_DEFINED_EVENT;
-
-  if (mEvent->eventStructType == NS_MOUSE_EVENT) {
-    if (atom == nsGkAtoms::onmousedown)
-      mEvent->message = NS_MOUSE_BUTTON_DOWN;
-    else if (atom == nsGkAtoms::onmouseup)
-      mEvent->message = NS_MOUSE_BUTTON_UP;
-    else if (atom == nsGkAtoms::onclick)
-      mEvent->message = NS_MOUSE_CLICK;
-    else if (atom == nsGkAtoms::ondblclick)
-      mEvent->message = NS_MOUSE_DOUBLECLICK;
-    else if (atom == nsGkAtoms::onmouseover)
-      mEvent->message = NS_MOUSE_ENTER_SYNTH;
-    else if (atom == nsGkAtoms::onmouseout)
-      mEvent->message = NS_MOUSE_EXIT_SYNTH;
-    else if (atom == nsGkAtoms::onmousemove)
-      mEvent->message = NS_MOUSE_MOVE;
-    else if (atom == nsGkAtoms::oncontextmenu)
-      mEvent->message = NS_CONTEXTMENU;
-  } else if (mEvent->eventStructType == NS_MOUSE_SCROLL_EVENT) {
-    if (atom == nsGkAtoms::onDOMMouseScroll)
-      mEvent->message = NS_MOUSE_SCROLL;
-    else if (atom == nsGkAtoms::onMozMousePixelScroll)
-      mEvent->message = NS_MOUSE_PIXEL_SCROLL;
-  } else if (mEvent->eventStructType == NS_DRAG_EVENT) {
-    if (atom == nsGkAtoms::ondragstart)
-      mEvent->message = NS_DRAGDROP_START;
-    else if (atom == nsGkAtoms::ondraggesture)
-      mEvent->message = NS_DRAGDROP_GESTURE;
-    else if (atom == nsGkAtoms::ondragenter)
-      mEvent->message = NS_DRAGDROP_ENTER;
-    else if (atom == nsGkAtoms::ondragover)
-      mEvent->message = NS_DRAGDROP_OVER_SYNTH;
-    else if (atom == nsGkAtoms::ondragleave)
-      mEvent->message = NS_DRAGDROP_LEAVE_SYNTH;
-    else if (atom == nsGkAtoms::ondragexit)
-      mEvent->message = NS_DRAGDROP_EXIT;
-    else if (atom == nsGkAtoms::ondrag)
-      mEvent->message = NS_DRAGDROP_DRAG;
-    else if (atom == nsGkAtoms::ondrop)
-      mEvent->message = NS_DRAGDROP_DROP;
-    else if (atom == nsGkAtoms::ondragdrop)
-      mEvent->message = NS_DRAGDROP_DRAGDROP;
-    else if (atom == nsGkAtoms::ondragend)
-      mEvent->message = NS_DRAGDROP_END;
-  } else if (mEvent->eventStructType == NS_KEY_EVENT) {
-    if (atom == nsGkAtoms::onkeydown)
-      mEvent->message = NS_KEY_DOWN;
-    else if (atom == nsGkAtoms::onkeyup)
-      mEvent->message = NS_KEY_UP;
-    else if (atom == nsGkAtoms::onkeypress)
-      mEvent->message = NS_KEY_PRESS;
-  } else if (mEvent->eventStructType == NS_COMPOSITION_EVENT) {
-    if (atom == nsGkAtoms::oncompositionstart)
-      mEvent->message = NS_COMPOSITION_START;
-    else if (atom == nsGkAtoms::oncompositionend)
-      mEvent->message = NS_COMPOSITION_END;
-  } else if (mEvent->eventStructType == NS_EVENT) {
-    if (atom == nsGkAtoms::onMozAfterPaint)
-      mEvent->message = NS_AFTERPAINT;
-    if (atom == nsGkAtoms::onfocus)
-      mEvent->message = NS_FOCUS_CONTENT;
-    else if (atom == nsGkAtoms::onblur)
-      mEvent->message = NS_BLUR_CONTENT;
-    else if (atom == nsGkAtoms::onsubmit)
-      mEvent->message = NS_FORM_SUBMIT;
-    else if (atom == nsGkAtoms::onreset)
-      mEvent->message = NS_FORM_RESET;
-    else if (atom == nsGkAtoms::onchange)
-      mEvent->message = NS_FORM_CHANGE;
-    else if (atom == nsGkAtoms::onselect)
-      mEvent->message = NS_FORM_SELECTED;
-    else if (atom == nsGkAtoms::onload)
-      mEvent->message = NS_LOAD;
-    else if (atom == nsGkAtoms::onunload)
-      mEvent->message = NS_PAGE_UNLOAD;
-    else if (atom == nsGkAtoms::onbeforeunload)
-      mEvent->message = NS_BEFORE_PAGE_UNLOAD;
-    else if (atom == nsGkAtoms::onabort)
-      mEvent->message = NS_IMAGE_ABORT;
-    else if (atom == nsGkAtoms::onerror)
-      mEvent->message = NS_LOAD_ERROR;
-    else if (atom == nsGkAtoms::onoffline)
-      mEvent->message = NS_OFFLINE;
-    else if (atom == nsGkAtoms::ononline)
-      mEvent->message = NS_ONLINE;
-    else if (atom == nsGkAtoms::oncopy)
-      mEvent->message = NS_COPY;
-    else if (atom == nsGkAtoms::oncut)
-      mEvent->message = NS_CUT;
-    else if (atom == nsGkAtoms::onpaste)
-      mEvent->message = NS_PASTE;
-    else if (atom == nsGkAtoms::onpageshow)
-      mEvent->message = NS_PAGE_SHOW;
-    else if (atom == nsGkAtoms::onpagehide)
-      mEvent->message = NS_PAGE_HIDE;
-    else if (atom == nsGkAtoms::onhashchange)
-      mEvent->message = NS_HASHCHANGE;
-  } else if (mEvent->eventStructType == NS_MUTATION_EVENT) {
-    if (atom == nsGkAtoms::onDOMAttrModified)
-      mEvent->message = NS_MUTATION_ATTRMODIFIED;
-    else if (atom == nsGkAtoms::onDOMCharacterDataModified)
-      mEvent->message = NS_MUTATION_CHARACTERDATAMODIFIED;
-    else if (atom == nsGkAtoms::onDOMNodeInserted)
-      mEvent->message = NS_MUTATION_NODEINSERTED;
-    else if (atom == nsGkAtoms::onDOMNodeRemoved)
-      mEvent->message = NS_MUTATION_NODEREMOVED;
-    else if (atom == nsGkAtoms::onDOMNodeInsertedIntoDocument)
-      mEvent->message = NS_MUTATION_NODEINSERTEDINTODOCUMENT;
-    else if (atom == nsGkAtoms::onDOMNodeRemovedFromDocument)
-      mEvent->message = NS_MUTATION_NODEREMOVEDFROMDOCUMENT;
-    else if (atom == nsGkAtoms::onDOMSubtreeModified)
-      mEvent->message = NS_MUTATION_SUBTREEMODIFIED;
-  } else if (mEvent->eventStructType == NS_UI_EVENT) {
-    if (atom == nsGkAtoms::onDOMActivate)
-      mEvent->message = NS_UI_ACTIVATE;
-    else if (atom == nsGkAtoms::onDOMFocusIn)
-      mEvent->message = NS_UI_FOCUSIN;
-    else if (atom == nsGkAtoms::onDOMFocusOut)
-      mEvent->message = NS_UI_FOCUSOUT;
-    else if (atom == nsGkAtoms::oninput)
-      mEvent->message = NS_FORM_INPUT;
-  } else if (mEvent->eventStructType == NS_SCROLLAREA_EVENT) {
-    if (atom == nsGkAtoms::onMozScrolledAreaChanged)
-      mEvent->message = NS_SCROLLEDAREACHANGED;
-  } else if (mEvent->eventStructType == NS_INPUT_EVENT) {
-    if (atom == nsGkAtoms::oncommand)
-      mEvent->message = NS_XUL_COMMAND;
-  }
-#ifdef MOZ_SVG
-  else if (mEvent->eventStructType == NS_SVG_EVENT) {
-    if (atom == nsGkAtoms::onSVGLoad)
-      mEvent->message = NS_SVG_LOAD;
-    else if (atom == nsGkAtoms::onSVGUnload)
-      mEvent->message = NS_SVG_UNLOAD;
-    else if (atom == nsGkAtoms::onSVGAbort)
-      mEvent->message = NS_SVG_ABORT;
-    else if (atom == nsGkAtoms::onSVGError)
-      mEvent->message = NS_SVG_ERROR;
-    else if (atom == nsGkAtoms::onSVGResize)
-      mEvent->message = NS_SVG_RESIZE;
-    else if (atom == nsGkAtoms::onSVGScroll)
-      mEvent->message = NS_SVG_SCROLL;
-  } else if (mEvent->eventStructType == NS_SVGZOOM_EVENT) {
-    if (atom == nsGkAtoms::onSVGZoom)
-      mEvent->message = NS_SVG_ZOOM;
-  }
-#endif // MOZ_SVG
-
-#ifdef MOZ_MEDIA
-  else if (mEvent->eventStructType == NS_MEDIA_EVENT) {
-    if (atom == nsGkAtoms::onloadstart)
-      mEvent->message = NS_LOADSTART;
-    else if (atom == nsGkAtoms::onprogress)
-      mEvent->message = NS_PROGRESS;
-    else if (atom == nsGkAtoms::onsuspend)
-      mEvent->message = NS_SUSPEND;
-    else if (atom == nsGkAtoms::onemptied)
-      mEvent->message = NS_EMPTIED;
-    else if (atom == nsGkAtoms::onstalled)
-      mEvent->message = NS_STALLED;
-    else if (atom == nsGkAtoms::onplay)
-      mEvent->message = NS_PLAY;
-    else if (atom == nsGkAtoms::onpause)
-      mEvent->message = NS_PAUSE;
-    else if (atom == nsGkAtoms::onloadedmetadata)
-      mEvent->message = NS_LOADEDMETADATA;
-    else if (atom == nsGkAtoms::onloadeddata)
-      mEvent->message = NS_LOADEDDATA;
-    else if (atom == nsGkAtoms::onwaiting)
-      mEvent->message = NS_WAITING;
-    else if (atom == nsGkAtoms::onplaying)
-      mEvent->message = NS_PLAYING;
-    else if (atom == nsGkAtoms::oncanplay)
-      mEvent->message = NS_CANPLAY;
-    else if (atom == nsGkAtoms::oncanplaythrough)
-      mEvent->message = NS_CANPLAYTHROUGH;
-    else if (atom == nsGkAtoms::onseeking)
-      mEvent->message = NS_SEEKING;
-    else if (atom == nsGkAtoms::onseeked)
-      mEvent->message = NS_SEEKED;
-    else if (atom == nsGkAtoms::ontimeupdate)
-      mEvent->message = NS_TIMEUPDATE;
-    else if (atom == nsGkAtoms::onended)
-      mEvent->message = NS_ENDED;
-    else if (atom == nsGkAtoms::onratechange)
-      mEvent->message = NS_RATECHANGE;
-    else if (atom == nsGkAtoms::ondurationchange)
-      mEvent->message = NS_DURATIONCHANGE;
-    else if (atom == nsGkAtoms::onvolumechange)
-      mEvent->message = NS_VOLUMECHANGE;
-    else if (atom == nsGkAtoms::onload)
-      mEvent->message = NS_LOAD;
-    else if (atom == nsGkAtoms::onabort)
-      mEvent->message = NS_MEDIA_ABORT;
-    else if (atom == nsGkAtoms::onerror)
-      mEvent->message = NS_MEDIA_ERROR;
-  }
-#endif // MOZ_MEDIA
-  else if (mEvent->eventStructType == NS_SIMPLE_GESTURE_EVENT) {
-    if (atom == nsGkAtoms::onMozSwipeGesture)
-      mEvent->message = NS_SIMPLE_GESTURE_SWIPE;
-    else if (atom == nsGkAtoms::onMozMagnifyGestureStart)
-      mEvent->message = NS_SIMPLE_GESTURE_MAGNIFY_START;
-    else if (atom == nsGkAtoms::onMozMagnifyGestureUpdate)
-      mEvent->message = NS_SIMPLE_GESTURE_MAGNIFY_UPDATE;
-    else if (atom == nsGkAtoms::onMozMagnifyGesture)
-      mEvent->message = NS_SIMPLE_GESTURE_MAGNIFY;
-    else if (atom == nsGkAtoms::onMozRotateGestureStart)
-      mEvent->message = NS_SIMPLE_GESTURE_ROTATE_START;
-    else if (atom == nsGkAtoms::onMozRotateGestureUpdate)
-      mEvent->message = NS_SIMPLE_GESTURE_ROTATE_UPDATE;
-    else if (atom == nsGkAtoms::onMozRotateGesture)
-      mEvent->message = NS_SIMPLE_GESTURE_ROTATE;
-    else if (atom == nsGkAtoms::onMozPressTapGesture)
-      mEvent->message = NS_SIMPLE_GESTURE_PRESSTAP;
-    else if (atom == nsGkAtoms::onMozTapGesture)
-      mEvent->message = NS_SIMPLE_GESTURE_TAP;
-  }
-
-  if (mEvent->message == NS_USER_DEFINED_EVENT)
-    mEvent->userType = atom;
-
+  mEvent->userType =
+    nsContentUtils::GetEventIdAndAtom(aEventTypeArg, mEvent->eventStructType,
+                                      &(mEvent->message));
   return NS_OK;
 }
 
@@ -739,7 +541,7 @@ nsDOMEvent::InitEvent(const nsAString& aEventTypeArg, PRBool aCanBubbleArg, PRBo
   // re-dispatching it.
   mEvent->target = nsnull;
   mEvent->originalTarget = nsnull;
-
+  mCachedType = aEventTypeArg;
   return NS_OK;
 }
 
@@ -797,11 +599,6 @@ NS_METHOD nsDOMEvent::DuplicatePrivateData()
       newEvent = zLevelEvent;
       break;
     }
-    case NS_PAINT_EVENT:
-    {
-      newEvent = new nsPaintEvent(PR_FALSE, msg, nsnull);
-      break;
-    }
     case NS_SCROLLBAR_EVENT:
     {
       newEvent = new nsScrollbarEvent(PR_FALSE, msg, nsnull);
@@ -841,6 +638,7 @@ NS_METHOD nsDOMEvent::DuplicatePrivateData()
       mouseEvent->relatedTarget = oldMouseEvent->relatedTarget;
       mouseEvent->button = oldMouseEvent->button;
       mouseEvent->pressure = oldMouseEvent->pressure;
+      mouseEvent->inputSource = oldMouseEvent->inputSource;
       newEvent = mouseEvent;
       break;
     }
@@ -856,17 +654,11 @@ NS_METHOD nsDOMEvent::DuplicatePrivateData()
       dragEvent->acceptActivation = oldDragEvent->acceptActivation;
       dragEvent->relatedTarget = oldDragEvent->relatedTarget;
       dragEvent->button = oldDragEvent->button;
+      static_cast<nsMouseEvent*>(dragEvent)->inputSource =
+        static_cast<nsMouseEvent*>(oldDragEvent)->inputSource;
       newEvent = dragEvent;
       break;
     }
-    case NS_MENU_EVENT:
-    {
-      newEvent = new nsMenuEvent(PR_FALSE, msg, nsnull);
-      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
-      static_cast<nsMenuEvent*>(newEvent)->mCommand =
-        static_cast<nsMenuEvent*>(mEvent)->mCommand;
-      break;
-    }
     case NS_SCRIPT_ERROR_EVENT:
     {
       newEvent = new nsScriptErrorEvent(PR_FALSE, msg);
@@ -883,8 +675,12 @@ NS_METHOD nsDOMEvent::DuplicatePrivateData()
     }
     case NS_COMPOSITION_EVENT:
     {
-      newEvent = new nsCompositionEvent(PR_FALSE, msg, nsnull);
-      isInputEvent = PR_TRUE;
+      nsCompositionEvent* compositionEvent =
+        new nsCompositionEvent(PR_FALSE, msg, nsnull);
+      nsCompositionEvent* oldCompositionEvent =
+        static_cast<nsCompositionEvent*>(mEvent);
+      compositionEvent->data = oldCompositionEvent->data;
+      newEvent = compositionEvent;
       break;
     }
     case NS_MOUSE_SCROLL_EVENT:
@@ -899,6 +695,8 @@ NS_METHOD nsDOMEvent::DuplicatePrivateData()
       mouseScrollEvent->delta = oldMouseScrollEvent->delta;
       mouseScrollEvent->relatedTarget = oldMouseScrollEvent->relatedTarget;
       mouseScrollEvent->button = oldMouseScrollEvent->button;
+      static_cast<nsMouseEvent_base*>(mouseScrollEvent)->inputSource =
+        static_cast<nsMouseEvent_base*>(oldMouseScrollEvent)->inputSource;
       newEvent = mouseScrollEvent;
       break;
     }
@@ -934,17 +732,30 @@ NS_METHOD nsDOMEvent::DuplicatePrivateData()
       newEvent = mutationEvent;
       break;
     }
+#ifdef ACCESSIBILITY
     case NS_ACCESSIBLE_EVENT:
     {
       newEvent = new nsAccessibleEvent(PR_FALSE, msg, nsnull);
       isInputEvent = PR_TRUE;
       break;
     }
+#endif
     case NS_FORM_EVENT:
     {
       newEvent = new nsFormEvent(PR_FALSE, msg);
       break;
     }
+    case NS_FOCUS_EVENT:
+    {
+      nsFocusEvent* newFocusEvent = new nsFocusEvent(PR_FALSE, msg);
+      NS_ENSURE_TRUE(newFocusEvent, NS_ERROR_OUT_OF_MEMORY);
+      nsFocusEvent* oldFocusEvent = static_cast<nsFocusEvent*>(mEvent);
+      newFocusEvent->fromRaise = oldFocusEvent->fromRaise;
+      newFocusEvent->isRefocus = oldFocusEvent->isRefocus;
+      newEvent = newFocusEvent;
+      break;
+    }
+
     case NS_POPUP_EVENT:
     {
       newEvent = new nsInputEvent(PR_FALSE, msg, nsnull);
@@ -966,7 +777,6 @@ NS_METHOD nsDOMEvent::DuplicatePrivateData()
                                static_cast<nsUIEvent*>(mEvent)->detail);
       break;
     }
-#ifdef MOZ_SVG
     case NS_SVG_EVENT:
     {
       newEvent = new nsEvent(PR_FALSE, msg);
@@ -981,7 +791,15 @@ NS_METHOD nsDOMEvent::DuplicatePrivateData()
       newEvent->eventStructType = NS_SVGZOOM_EVENT;
       break;
     }
-#endif // MOZ_SVG
+#ifdef MOZ_SMIL
+    case NS_SMIL_TIME_EVENT:
+    {
+      newEvent = new nsUIEvent(PR_FALSE, msg, 0);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      newEvent->eventStructType = NS_SMIL_TIME_EVENT;
+      break;
+    }
+#endif // MOZ_SMIL
     case NS_SIMPLE_GESTURE_EVENT:
     {
       nsSimpleGestureEvent* oldSimpleGestureEvent = static_cast<nsSimpleGestureEvent*>(mEvent);
@@ -994,6 +812,34 @@ NS_METHOD nsDOMEvent::DuplicatePrivateData()
       newEvent = simpleGestureEvent;
       break;
     }
+    case NS_TRANSITION_EVENT:
+    {
+      nsTransitionEvent* oldTransitionEvent =
+        static_cast<nsTransitionEvent*>(mEvent);
+      newEvent = new nsTransitionEvent(PR_FALSE, msg,
+                                       oldTransitionEvent->propertyName,
+                                       oldTransitionEvent->elapsedTime);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      break;
+    }
+    case NS_ANIMATION_EVENT:
+    {
+      nsAnimationEvent* oldAnimationEvent =
+        static_cast<nsAnimationEvent*>(mEvent);
+      newEvent = new nsAnimationEvent(PR_FALSE, msg,
+                                      oldAnimationEvent->animationName,
+                                      oldAnimationEvent->elapsedTime);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      break;
+    }
+    case NS_MOZTOUCH_EVENT:
+    {
+      newEvent = new nsMozTouchEvent(PR_FALSE, msg, nsnull,
+                                     static_cast<nsMozTouchEvent*>(mEvent)->streamId);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      isInputEvent = PR_TRUE;
+      break;
+    }
     default:
     {
       NS_WARNING("Unknown event type!!!");
@@ -1039,7 +885,7 @@ NS_METHOD nsDOMEvent::SetTarget(nsIDOMEventTarget* aTarget)
   }
 #endif
 
-  mEvent->target = aTarget;
+  mEvent->target = do_QueryInterface(aTarget);
   return NS_OK;
 }
 
@@ -1241,8 +1087,7 @@ nsDOMEvent::PopupAllowedEventsChanged()
     nsMemory::Free(sPopupAllowedEvents);
   }
 
-  nsAdoptingCString str =
-    nsContentUtils::GetCharPref("dom.popup_allowed_events");
+  nsAdoptingCString str = Preferences::GetCString("dom.popup_allowed_events");
 
   // We'll want to do this even if str is empty to avoid looking up
   // this pref all the time if it's not set.
@@ -1276,6 +1121,8 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
     return sEventNames[eDOMEvents_mouseover];
   case NS_MOUSE_EXIT_SYNTH:
     return sEventNames[eDOMEvents_mouseout];
+  case NS_MOUSE_MOZHITTEST:
+    return sEventNames[eDOMEvents_MozMouseHittest];
   case NS_MOUSE_MOVE:
     return sEventNames[eDOMEvents_mousemove];
   case NS_KEY_UP:
@@ -1286,6 +1133,8 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
     return sEventNames[eDOMEvents_keypress];
   case NS_COMPOSITION_START:
     return sEventNames[eDOMEvents_compositionstart];
+  case NS_COMPOSITION_UPDATE:
+    return sEventNames[eDOMEvents_compositionupdate];
   case NS_COMPOSITION_END:
     return sEventNames[eDOMEvents_compositionend];
   case NS_FOCUS_CONTENT:
@@ -1296,12 +1145,20 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
     return sEventNames[eDOMEvents_close];
   case NS_LOAD:
     return sEventNames[eDOMEvents_load];
+  case NS_POPSTATE:
+    return sEventNames[eDOMEvents_popstate];
+  case NS_BEFORE_SCRIPT_EXECUTE:
+    return sEventNames[eDOMEvents_beforescriptexecute];
+  case NS_AFTER_SCRIPT_EXECUTE:
+    return sEventNames[eDOMEvents_afterscriptexecute];
   case NS_BEFORE_PAGE_UNLOAD:
     return sEventNames[eDOMEvents_beforeunload];
   case NS_PAGE_UNLOAD:
     return sEventNames[eDOMEvents_unload];
   case NS_HASHCHANGE:
     return sEventNames[eDOMEvents_hashchange];
+  case NS_READYSTATECHANGE:
+    return sEventNames[eDOMEvents_readystatechange];
   case NS_IMAGE_ABORT:
     return sEventNames[eDOMEvents_abort];
   case NS_LOAD_ERROR:
@@ -1316,8 +1173,8 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
     return sEventNames[eDOMEvents_select];
   case NS_FORM_INPUT:
     return sEventNames[eDOMEvents_input];
-  case NS_PAINT:
-    return sEventNames[eDOMEvents_paint];
+  case NS_FORM_INVALID:
+    return sEventNames[eDOMEvents_invalid];
   case NS_RESIZE_EVENT:
     return sEventNames[eDOMEvents_resize];
   case NS_SCROLL_EVENT:
@@ -1404,7 +1261,12 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
     return sEventNames[eDOMEvents_cut];
   case NS_PASTE:
     return sEventNames[eDOMEvents_paste];
-#ifdef MOZ_SVG
+  case NS_OPEN:
+    return sEventNames[eDOMEvents_open];
+  case NS_MESSAGE:
+    return sEventNames[eDOMEvents_message];
+  case NS_SHOW_EVENT:
+    return sEventNames[eDOMEvents_show];
   case NS_SVG_LOAD:
     return sEventNames[eDOMEvents_SVGLoad];
   case NS_SVG_UNLOAD:
@@ -1419,7 +1281,14 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
     return sEventNames[eDOMEvents_SVGScroll];
   case NS_SVG_ZOOM:
     return sEventNames[eDOMEvents_SVGZoom];
-#endif // MOZ_SVG
+#ifdef MOZ_SMIL
+  case NS_SMIL_BEGIN:
+    return sEventNames[eDOMEvents_beginEvent];
+  case NS_SMIL_END:
+    return sEventNames[eDOMEvents_endEvent];
+  case NS_SMIL_REPEAT:
+    return sEventNames[eDOMEvents_repeatEvent];
+#endif // MOZ_SMIL
 #ifdef MOZ_MEDIA
   case NS_LOADSTART:
     return sEventNames[eDOMEvents_loadstart];
@@ -1461,9 +1330,15 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
     return sEventNames[eDOMEvents_durationchange];
   case NS_VOLUMECHANGE:
     return sEventNames[eDOMEvents_volumechange];
+  case NS_MOZAUDIOAVAILABLE:
+    return sEventNames[eDOMEvents_mozaudioavailable];
 #endif
   case NS_AFTERPAINT:
     return sEventNames[eDOMEvents_afterpaint];
+  case NS_BEFOREPAINT:
+    return sEventNames[eDOMEvents_beforepaint];
+  case NS_BEFORERESIZE_EVENT:
+    return sEventNames[eDOMEvents_beforeresize];
   case NS_SIMPLE_GESTURE_SWIPE:
     return sEventNames[eDOMEvents_MozSwipeGesture];
   case NS_SIMPLE_GESTURE_MAGNIFY_START:
@@ -1482,8 +1357,28 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
     return sEventNames[eDOMEvents_MozTapGesture];
   case NS_SIMPLE_GESTURE_PRESSTAP:
     return sEventNames[eDOMEvents_MozPressTapGesture];
+  case NS_MOZTOUCH_DOWN:
+    return sEventNames[eDOMEvents_MozTouchDown];
+  case NS_MOZTOUCH_MOVE:
+    return sEventNames[eDOMEvents_MozTouchMove];
+  case NS_MOZTOUCH_UP:
+    return sEventNames[eDOMEvents_MozTouchUp];
   case NS_SCROLLEDAREACHANGED:
     return sEventNames[eDOMEvents_MozScrolledAreaChanged];
+  case NS_TRANSITION_END:
+    return sEventNames[eDOMEvents_transitionend];
+  case NS_ANIMATION_START:
+    return sEventNames[eDOMEvents_animationstart];
+  case NS_ANIMATION_END:
+    return sEventNames[eDOMEvents_animationend];
+  case NS_ANIMATION_ITERATION:
+    return sEventNames[eDOMEvents_animationiteration];
+  case NS_DEVICE_MOTION:
+    return sEventNames[eDOMEvents_devicemotion];
+  case NS_DEVICE_ORIENTATION:
+    return sEventNames[eDOMEvents_deviceorientation];
+  case NS_FULLSCREENCHANGE:
+    return sEventNames[eDOMEvents_mozfullscreenchange];
   default:
     break;
   }
@@ -1495,25 +1390,6 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
   return nsnull;
 }
 
-nsresult
-nsDOMEvent::ReportWrongPropertyAccessWarning(const char* aPropertyName)
-{
-  nsCOMPtr<nsIDocument> doc(GetDocumentForReport(mEvent));
-
-  nsAutoString propertyName, type;
-  GetType(type);
-  propertyName.AssignASCII(aPropertyName);
-  const PRUnichar *strings[] = { propertyName.get(), type.get() };
-
-  return nsContentUtils::ReportToConsole(nsContentUtils::eDOM_PROPERTIES,
-                                         "WrongEventPropertyAccessWarning",
-                                         strings, NS_ARRAY_LENGTH(strings),
-                                         doc ? doc->GetDocumentURI() : nsnull,
-                                         EmptyString(), 0, 0,
-                                         nsIScriptError::warningFlag,
-                                         "DOM Events");
-}
-
 NS_IMETHODIMP
 nsDOMEvent::GetPreventDefault(PRBool* aReturn)
 {
@@ -1522,6 +1398,61 @@ nsDOMEvent::GetPreventDefault(PRBool* aReturn)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsDOMEvent::GetDefaultPrevented(PRBool* aReturn)
+{
+  return GetPreventDefault(aReturn);
+}
+
+void
+nsDOMEvent::Serialize(IPC::Message* aMsg, PRBool aSerializeInterfaceType)
+{
+  if (aSerializeInterfaceType) {
+    IPC::WriteParam(aMsg, NS_LITERAL_STRING("event"));
+  }
+
+  nsString type;
+  GetType(type);
+  IPC::WriteParam(aMsg, type);
+
+  PRBool bubbles = PR_FALSE;
+  GetBubbles(&bubbles);
+  IPC::WriteParam(aMsg, bubbles);
+
+  PRBool cancelable = PR_FALSE;
+  GetCancelable(&cancelable);
+  IPC::WriteParam(aMsg, cancelable);
+
+  PRBool trusted = PR_FALSE;
+  GetIsTrusted(&trusted);
+  IPC::WriteParam(aMsg, trusted);
+
+  // No timestamp serialization for now!
+}
+
+PRBool
+nsDOMEvent::Deserialize(const IPC::Message* aMsg, void** aIter)
+{
+  nsString type;
+  NS_ENSURE_TRUE(IPC::ReadParam(aMsg, aIter, &type), PR_FALSE);
+
+  PRBool bubbles = PR_FALSE;
+  NS_ENSURE_TRUE(IPC::ReadParam(aMsg, aIter, &bubbles), PR_FALSE);
+
+  PRBool cancelable = PR_FALSE;
+  NS_ENSURE_TRUE(IPC::ReadParam(aMsg, aIter, &cancelable), PR_FALSE);
+
+  PRBool trusted = PR_FALSE;
+  NS_ENSURE_TRUE(IPC::ReadParam(aMsg, aIter, &trusted), PR_FALSE);
+
+  nsresult rv = InitEvent(type, bubbles, cancelable);
+  NS_ENSURE_SUCCESS(rv, PR_FALSE);
+  SetTrusted(trusted);
+
+  return PR_TRUE;
+}
+
+
 nsresult NS_NewDOMEvent(nsIDOMEvent** aInstancePtrResult,
                         nsPresContext* aPresContext,
                         nsEvent *aEvent) 
