@@ -34,11 +34,14 @@
  * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
+
+/**
+ * Implementation of HTML <label> elements.
+ */
+#include "nsHTMLLabelElement.h"
 #include "nsCOMPtr.h"
-#include "nsIDOMHTMLLabelElement.h"
 #include "nsIDOMHTMLFormElement.h"
 #include "nsIDOMEventTarget.h"
-#include "nsGenericHTMLElement.h"
 #include "nsGkAtoms.h"
 #include "nsStyleConsts.h"
 #include "nsPresContext.h"
@@ -47,78 +50,13 @@
 #include "nsIDOMDocument.h"
 #include "nsIDocument.h"
 #include "nsGUIEvent.h"
-#include "nsIEventStateManager.h"
 #include "nsEventDispatcher.h"
 #include "nsPIDOMWindow.h"
 #include "nsFocusManager.h"
 
-class nsHTMLLabelElement : public nsGenericHTMLFormElement,
-                           public nsIDOMHTMLLabelElement
-{
-public:
-  nsHTMLLabelElement(already_AddRefed<nsINodeInfo> aNodeInfo);
-  virtual ~nsHTMLLabelElement();
-
-  // nsISupports
-  NS_DECL_ISUPPORTS_INHERITED
-
-  // nsIDOMNode
-  NS_FORWARD_NSIDOMNODE(nsGenericHTMLFormElement::)
-
-  // nsIDOMElement
-  NS_FORWARD_NSIDOMELEMENT(nsGenericHTMLFormElement::)
-
-  // nsIDOMHTMLLabelElement
-  NS_DECL_NSIDOMHTMLLABELELEMENT
-
-  // Forward nsIDOMHTMLElement -- We don't override Click()
-  NS_FORWARD_NSIDOMHTMLELEMENT_NOFOCUSCLICK(nsGenericHTMLFormElement::)
-  NS_IMETHOD Click() { 
-    return nsGenericHTMLFormElement::Click(); 
-  }
-  NS_IMETHOD Focus();
-
-  // nsIFormControl
-  NS_IMETHOD_(PRUint32) GetType() const { return NS_FORM_LABEL; }
-  NS_IMETHOD Reset();
-  NS_IMETHOD SubmitNamesValues(nsFormSubmission* aFormSubmission);
-
-  virtual bool IsDisabled() const { return PR_FALSE; }
-
-  // nsIContent
-  virtual nsresult BindToTree(nsIDocument* aDocument, nsIContent* aParent,
-                              nsIContent* aBindingParent,
-                              PRBool aCompileEventHandlers);
-  virtual void UnbindFromTree(PRBool aDeep = PR_TRUE,
-                              PRBool aNullParent = PR_TRUE);
-
-  virtual nsresult PostHandleEvent(nsEventChainPostVisitor& aVisitor);
-
-  nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                   const nsAString& aValue, PRBool aNotify)
-  {
-    return SetAttr(aNameSpaceID, aName, nsnull, aValue, aNotify);
-  }
-  virtual nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                           nsIAtom* aPrefix, const nsAString& aValue,
-                           PRBool aNotify);
-  virtual nsresult UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute,
-                             PRBool aNotify);
-  virtual void PerformAccesskey(PRBool aKeyCausesActivation,
-                                PRBool aIsTrustedEvent);
-  virtual nsresult Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const;
-
-  virtual nsXPCClassInfo* GetClassInfo();
-protected:
-  already_AddRefed<nsIContent> GetControlContent();
-  already_AddRefed<nsIContent> GetFirstFormControl(nsIContent *current);
-
-  // XXX It would be nice if we could use an event flag instead.
-  PRPackedBool mHandlingEvent;
-};
-
 // construction, destruction
 
+using namespace mozilla::dom;
 
 NS_IMPL_NS_NEW_HTML_ELEMENT(Label)
 
@@ -168,8 +106,7 @@ nsHTMLLabelElement::GetControl(nsIDOMHTMLElement** aElement)
 {
   *aElement = nsnull;
 
-  nsCOMPtr<nsIContent> content = GetControlContent();
-  nsCOMPtr<nsIDOMHTMLElement> element = do_QueryInterface(content);
+  nsCOMPtr<nsIDOMHTMLElement> element = do_QueryInterface(GetLabeledElement());
 
   element.swap(*aElement);
   return NS_OK;
@@ -184,9 +121,7 @@ nsHTMLLabelElement::Focus()
   // retarget the focus method at the for content
   nsIFocusManager* fm = nsFocusManager::GetFocusManager();
   if (fm) {
-    nsCOMPtr<nsIContent> content = GetControlContent();
-
-    nsCOMPtr<nsIDOMElement> elem = do_QueryInterface(content);
+    nsCOMPtr<nsIDOMElement> elem = do_QueryInterface(GetLabeledElement());
     if (elem)
       fm->SetFocus(elem, 0);
   }
@@ -246,11 +181,14 @@ nsHTMLLabelElement::PostHandleEvent(nsEventChainPostVisitor& aVisitor)
       (!NS_IS_MOUSE_LEFT_CLICK(aVisitor.mEvent) &&
        aVisitor.mEvent->message != NS_MOUSE_BUTTON_DOWN) ||
       aVisitor.mEventStatus == nsEventStatus_eConsumeNoDefault ||
-      !aVisitor.mPresContext) {
+      !aVisitor.mPresContext ||
+      // Don't handle the event if it's already been handled by another label
+      (aVisitor.mEvent->flags & NS_EVENT_FLAG_PREVENT_MULTIPLE_ACTIONS)) {
     return NS_OK;
   }
 
-  nsCOMPtr<nsIContent> content = GetControlContent();
+  // Strong ref because event dispatch is going to happen.
+  nsRefPtr<Element> content = GetLabeledElement();
 
   if (content && !EventTargetIn(aVisitor.mEvent, content, this)) {
     mHandlingEvent = PR_TRUE;
@@ -320,8 +258,11 @@ nsHTMLLabelElement::PostHandleEvent(nsEventChainPostVisitor& aVisitor)
           // will actually create a new event.
           DispatchClickEvent(aVisitor.mPresContext,
                              static_cast<nsInputEvent*>(aVisitor.mEvent),
-                             content, PR_FALSE, &status);
+                             content, PR_FALSE,
+                             NS_EVENT_FLAG_PREVENT_MULTIPLE_ACTIONS, &status);
           // Do we care about the status this returned?  I don't think we do...
+          // Don't run another <label> off of this click
+          aVisitor.mEvent->flags |= NS_EVENT_FLAG_PREVENT_MULTIPLE_ACTIONS;
         }
         break;
     }
@@ -362,9 +303,9 @@ nsHTMLLabelElement::PerformAccesskey(PRBool aKeyCausesActivation,
                                      PRBool aIsTrustedEvent)
 {
   if (!aKeyCausesActivation) {
-    nsCOMPtr<nsIContent> content = GetControlContent();
-    if (content)
-      content->PerformAccesskey(aKeyCausesActivation, aIsTrustedEvent);
+    nsRefPtr<Element> element = GetLabeledElement();
+    if (element)
+      element->PerformAccesskey(aKeyCausesActivation, aIsTrustedEvent);
   } else {
     nsPresContext *presContext = GetPresContext();
     if (!presContext)
@@ -373,7 +314,7 @@ nsHTMLLabelElement::PerformAccesskey(PRBool aKeyCausesActivation,
     // Click on it if the users prefs indicate to do so.
     nsMouseEvent event(aIsTrustedEvent, NS_MOUSE_CLICK,
                        nsnull, nsMouseEvent::eReal);
-    event.inputSource = nsIDOMNSMouseEvent::MOZ_SOURCE_KEYBOARD;
+    event.inputSource = nsIDOMMouseEvent::MOZ_SOURCE_KEYBOARD;
 
     nsAutoPopupStatePusher popupStatePusher(aIsTrustedEvent ?
                                             openAllowed : openAbused);
@@ -383,15 +324,15 @@ nsHTMLLabelElement::PerformAccesskey(PRBool aKeyCausesActivation,
   }
 }
 
-already_AddRefed<nsIContent>
-nsHTMLLabelElement::GetControlContent()
+Element*
+nsHTMLLabelElement::GetLabeledElement()
 {
   nsAutoString elementId;
 
   if (!GetAttr(kNameSpaceID_None, nsGkAtoms::_for, elementId)) {
     // No @for, so we are a label for our first form control element.
     // Do a depth-first traversal to look for the first form control element.
-    return GetFirstFormControl(this);
+    return GetFirstDescendantFormControl();
   }
 
   // We have a @for. The id has to be linked to an element in the same document
@@ -401,41 +342,31 @@ nsHTMLLabelElement::GetControlContent()
     return nsnull;
   }
 
-  nsIContent* content = doc->GetElementById(elementId);
-  if (!content) {
+  Element* element = doc->GetElementById(elementId);
+  if (!element) {
     return nsnull;
   }
 
-  nsCOMPtr<nsIFormControl> element = do_QueryInterface(content);
-  if (element && element->IsLabelableControl()) {
+  nsCOMPtr<nsIFormControl> controlElement = do_QueryInterface(element);
+  if (controlElement && controlElement->IsLabelableControl()) {
     // Transfer the reference count of element to the returned value.
-    element.forget();
-    return content;
+    return element;
   }
 
   return nsnull;
 }
 
-already_AddRefed<nsIContent>
-nsHTMLLabelElement::GetFirstFormControl(nsIContent *current)
+Element*
+nsHTMLLabelElement::GetFirstDescendantFormControl()
 {
-  PRUint32 numNodes = current->GetChildCount();
-
-  for (PRUint32 i = 0; i < numNodes; i++) {
-    nsIContent *child = current->GetChildAt(i);
-    if (!child) {
-      continue;
-    }
-
-    nsCOMPtr<nsIFormControl> element = do_QueryInterface(child);
+  // Have to cast do disambiguate GetFirstChild from the DOM method of that name
+  for (nsINode* cur = static_cast<nsINode*>(this)->GetFirstChild();
+       cur;
+       cur = cur->GetNextNode(this)) {
+    nsCOMPtr<nsIFormControl> element = do_QueryInterface(cur);
     if (element && element->IsLabelableControl()) {
-      NS_ADDREF(child);
-      return child;
-    }
-
-    nsIContent* content = GetFirstFormControl(child).get();
-    if (content) {
-      return content;
+      NS_ASSERTION(cur->IsElement(), "How did that happen?");
+      return cur->AsElement();
     }
   }
 
