@@ -43,12 +43,12 @@
 #include "nsSVGString.h"
 #include "nsCOMPtr.h"
 #include "nsGkAtoms.h"
-
+#include "nsContentUtils.h"
 
 nsSVGElement::StringInfo nsSVGAElement::sStringInfo[2] =
 {
-  { &nsGkAtoms::href, kNameSpaceID_XLink },
-  { &nsGkAtoms::target, kNameSpaceID_None }
+  { &nsGkAtoms::href, kNameSpaceID_XLink, PR_TRUE },
+  { &nsGkAtoms::target, kNameSpaceID_None, PR_TRUE }
 };
 
 NS_IMPL_NS_NEW_SVG_ELEMENT(A)
@@ -60,19 +60,27 @@ NS_IMPL_NS_NEW_SVG_ELEMENT(A)
 NS_IMPL_ADDREF_INHERITED(nsSVGAElement, nsSVGAElementBase)
 NS_IMPL_RELEASE_INHERITED(nsSVGAElement, nsSVGAElementBase)
 
+DOMCI_NODE_DATA(SVGAElement, nsSVGAElement)
+
 NS_INTERFACE_TABLE_HEAD(nsSVGAElement)
-  NS_NODE_INTERFACE_TABLE5(nsSVGAElement, nsIDOMNode, nsIDOMElement,
-                           nsIDOMSVGElement, nsIDOMSVGAElement, nsILink)
-  NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(SVGAElement)
+  NS_NODE_INTERFACE_TABLE7(nsSVGAElement,
+                           nsIDOMNode,
+                           nsIDOMElement,
+                           nsIDOMSVGElement,
+                           nsIDOMSVGAElement,
+                           nsIDOMSVGURIReference,
+                           nsILink,
+                           Link)
+  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(SVGAElement)
 NS_INTERFACE_MAP_END_INHERITING(nsSVGAElementBase)
 
 
 //----------------------------------------------------------------------
 // Implementation
 
-nsSVGAElement::nsSVGAElement(nsINodeInfo *aNodeInfo)
+nsSVGAElement::nsSVGAElement(already_AddRefed<nsINodeInfo> aNodeInfo)
   : nsSVGAElementBase(aNodeInfo),
-    mLinkState(eLinkState_Unknown)
+    Link(this)
 {
 }
 
@@ -122,27 +130,71 @@ nsSVGAElement::GetTarget(nsIDOMSVGAnimatedString * *aTarget)
 //----------------------------------------------------------------------
 // nsIContent methods
 
-nsLinkState
-nsSVGAElement::GetLinkState() const
+nsresult
+nsSVGAElement::BindToTree(nsIDocument *aDocument, nsIContent *aParent,
+                          nsIContent *aBindingParent,
+                          PRBool aCompileEventHandlers)
 {
-  return mLinkState;
+  Link::ResetLinkState(false);
+
+  nsresult rv = nsSVGAElementBase::BindToTree(aDocument, aParent,
+                                              aBindingParent,
+                                              aCompileEventHandlers);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_OK;
 }
 
 void
-nsSVGAElement::SetLinkState(nsLinkState aState)
+nsSVGAElement::UnbindFromTree(PRBool aDeep, PRBool aNullParent)
 {
-  mLinkState = aState;
+  // If this link is ever reinserted into a document, it might
+  // be under a different xml:base, so forget the cached state now.
+  Link::ResetLinkState(false);
+
+  nsSVGAElementBase::UnbindFromTree(aDeep, aNullParent);
+}
+
+nsLinkState
+nsSVGAElement::GetLinkState() const
+{
+  return Link::GetLinkState();
+}
+
+void
+nsSVGAElement::RequestLinkStateUpdate()
+{
+  UpdateLinkState(Link::LinkState());
 }
 
 already_AddRefed<nsIURI>
 nsSVGAElement::GetHrefURI() const
 {
-  return nsnull; // XXX GetHrefURIForAnchors();
+  nsCOMPtr<nsIURI> hrefURI;
+  return IsLink(getter_AddRefs(hrefURI)) ? hrefURI.forget() : nsnull;
 }
 
 
+NS_IMETHODIMP_(PRBool)
+nsSVGAElement::IsAttributeMapped(const nsIAtom* name) const
+{
+  static const MappedAttributeEntry* const map[] = {
+    sFEFloodMap,
+    sFiltersMap,
+    sFontSpecificationMap,
+    sGradientStopMap,
+    sLightingEffectsMap,
+    sMarkersMap,
+    sTextContentElementsMap,
+    sViewportsMap
+  };
+
+  return FindAttributeDependence(name, map, NS_ARRAY_LENGTH(map)) ||
+    nsSVGAElementBase::IsAttributeMapped(name);
+}
+
 PRBool
-nsSVGAElement::IsFocusable(PRInt32 *aTabIndex)
+nsSVGAElement::IsFocusable(PRInt32 *aTabIndex, PRBool aWithMouse)
 {
   nsCOMPtr<nsIURI> uri;
   if (IsLink(getter_AddRefs(uri))) {
@@ -232,6 +284,50 @@ nsSVGAElement::GetLinkTarget(nsAString& aTarget)
   }
 }
 
+nsEventStates
+nsSVGAElement::IntrinsicState() const
+{
+  return Link::LinkState() | nsSVGAElementBase::IntrinsicState();
+}
+
+nsresult
+nsSVGAElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                       nsIAtom* aPrefix, const nsAString& aValue,
+                       PRBool aNotify)
+{
+  nsresult rv = nsSVGAElementBase::SetAttr(aNameSpaceID, aName, aPrefix,
+                                           aValue, aNotify);
+
+  // The ordering of the parent class's SetAttr call and Link::ResetLinkState
+  // is important here!  The attribute is not set until SetAttr returns, and
+  // we will need the updated attribute value because notifying the document
+  // that content states have changed will call IntrinsicState, which will try
+  // to get updated information about the visitedness from Link.
+  if (aName == nsGkAtoms::href && aNameSpaceID == kNameSpaceID_XLink) {
+    Link::ResetLinkState(!!aNotify);
+  }
+
+  return rv;
+}
+
+nsresult
+nsSVGAElement::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttr,
+                         PRBool aNotify)
+{
+  nsresult rv = nsSVGAElementBase::UnsetAttr(aNameSpaceID, aAttr, aNotify);
+
+  // The ordering of the parent class's UnsetAttr call and Link::ResetLinkState
+  // is important here!  The attribute is not unset until UnsetAttr returns, and
+  // we will need the updated attribute value because notifying the document
+  // that content states have changed will call IntrinsicState, which will try
+  // to get updated information about the visitedness from Link.
+  if (aAttr == nsGkAtoms::href && aNameSpaceID == kNameSpaceID_XLink) {
+    Link::ResetLinkState(!!aNotify);
+  }
+
+  return rv;
+}
+
 //----------------------------------------------------------------------
 // nsSVGElement methods
 
@@ -241,4 +337,3 @@ nsSVGAElement::GetStringInfo()
   return StringAttributesInfo(mStringAttributes, sStringInfo,
                               NS_ARRAY_LENGTH(sStringInfo));
 }
-
