@@ -38,10 +38,10 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsSVGGraphicElement.h"
-#include "nsSVGTransformList.h"
-#include "nsSVGAnimatedTransformList.h"
+#include "nsSVGSVGElement.h"
+#include "DOMSVGAnimatedTransformList.h"
+#include "DOMSVGMatrix.h"
 #include "nsGkAtoms.h"
-#include "nsSVGMatrix.h"
 #include "nsIDOMEventTarget.h"
 #include "nsIFrame.h"
 #include "nsISVGChildFrame.h"
@@ -49,6 +49,9 @@
 #include "nsSVGUtils.h"
 #include "nsDOMError.h"
 #include "nsSVGRect.h"
+#include "nsContentUtils.h"
+
+using namespace mozilla;
 
 //----------------------------------------------------------------------
 // nsISupports methods
@@ -82,7 +85,7 @@ NS_IMETHODIMP nsSVGGraphicElement::GetNearestViewportElement(nsIDOMSVGElement *
 /* readonly attribute nsIDOMSVGElement farthestViewportElement; */
 NS_IMETHODIMP nsSVGGraphicElement::GetFarthestViewportElement(nsIDOMSVGElement * *aFarthestViewportElement)
 {
-  *aFarthestViewportElement = nsSVGUtils::GetFarthestViewportElement(this).get();
+  NS_IF_ADDREF(*aFarthestViewportElement = nsSVGUtils::GetOuterSVGElement(this));
   return NS_OK;
 }
 
@@ -107,7 +110,8 @@ NS_IMETHODIMP nsSVGGraphicElement::GetBBox(nsIDOMSVGRect **_retval)
 NS_IMETHODIMP nsSVGGraphicElement::GetCTM(nsIDOMSVGMatrix * *aCTM)
 {
   gfxMatrix m = nsSVGUtils::GetCTM(this, PR_FALSE);
-  *aCTM = m.IsSingular() ? nsnull : NS_NewSVGMatrix(m).get();
+  *aCTM = m.IsSingular() ? nsnull : new DOMSVGMatrix(m);
+  NS_IF_ADDREF(*aCTM);
   return NS_OK;
 }
 
@@ -115,7 +119,8 @@ NS_IMETHODIMP nsSVGGraphicElement::GetCTM(nsIDOMSVGMatrix * *aCTM)
 NS_IMETHODIMP nsSVGGraphicElement::GetScreenCTM(nsIDOMSVGMatrix * *aCTM)
 {
   gfxMatrix m = nsSVGUtils::GetCTM(this, PR_TRUE);
-  *aCTM = m.IsSingular() ? nsnull : NS_NewSVGMatrix(m).get();
+  *aCTM = m.IsSingular() ? nsnull : new DOMSVGMatrix(m);
+  NS_IF_ADDREF(*aCTM);
   return NS_OK;
 }
 
@@ -147,13 +152,12 @@ NS_IMETHODIMP nsSVGGraphicElement::GetTransformToElement(nsIDOMSVGElement *eleme
 // nsIDOMSVGTransformable methods
 /* readonly attribute nsIDOMSVGAnimatedTransformList transform; */
 
-NS_IMETHODIMP nsSVGGraphicElement::GetTransform(nsIDOMSVGAnimatedTransformList * *aTransform)
+NS_IMETHODIMP nsSVGGraphicElement::GetTransform(
+    nsIDOMSVGAnimatedTransformList **aTransform)
 {
-  if (!mTransforms && NS_FAILED(CreateTransformList()))
-    return NS_ERROR_OUT_OF_MEMORY;
-      
-  *aTransform = mTransforms;
-  NS_ADDREF(*aTransform);
+  *aTransform =
+    DOMSVGAnimatedTransformList::GetDOMWrapper(GetAnimatedTransformList(), this)
+    .get();
   return NS_OK;
 }
 
@@ -183,7 +187,7 @@ nsSVGGraphicElement::IsEventName(nsIAtom* aName)
 }
 
 gfxMatrix
-nsSVGGraphicElement::PrependLocalTransformTo(const gfxMatrix &aMatrix)
+nsSVGGraphicElement::PrependLocalTransformTo(const gfxMatrix &aMatrix) const
 {
   gfxMatrix result(aMatrix);
 
@@ -195,17 +199,7 @@ nsSVGGraphicElement::PrependLocalTransformTo(const gfxMatrix &aMatrix)
   }
 
   if (mTransforms) {
-    nsresult rv;
-    nsCOMPtr<nsIDOMSVGTransformList> transforms;
-    rv = mTransforms->GetAnimVal(getter_AddRefs(transforms));
-    NS_ENSURE_SUCCESS(rv, aMatrix);
-    PRUint32 count;
-    transforms->GetNumberOfItems(&count);
-    if (count > 0) {
-      nsCOMPtr<nsIDOMSVGMatrix> matrix =
-        nsSVGTransformList::GetConsolidationMatrix(transforms);
-      result.PreMultiply(nsSVGUtils::ConvertSVGMatrixToThebes(matrix));
-    }
+    result.PreMultiply(mTransforms->GetAnimValue().GetConsolidationMatrix());
   }
 
   return result;
@@ -215,40 +209,14 @@ void
 nsSVGGraphicElement::SetAnimateMotionTransform(const gfxMatrix* aMatrix)
 {
   mAnimateMotionTransform = aMatrix ? new gfxMatrix(*aMatrix) : nsnull;
-  DidAnimateTransform();
+  DidAnimateTransformList();
 }
 
-nsresult
-nsSVGGraphicElement::BeforeSetAttr(PRInt32 aNamespaceID, nsIAtom* aName,
-                                   const nsAString* aValue, PRBool aNotify)
+SVGAnimatedTransformList*
+nsSVGGraphicElement::GetAnimatedTransformList()
 {
-  if (aNamespaceID == kNameSpaceID_None &&
-      aName == nsGkAtoms::transform &&
-      !mTransforms &&
-      NS_FAILED(CreateTransformList()))
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  return nsSVGGraphicElementBase::BeforeSetAttr(aNamespaceID, aName,
-                                                aValue, aNotify);
-}
-
-nsresult
-nsSVGGraphicElement::CreateTransformList()
-{
-  nsresult rv;
-
-  // DOM property: transform, #IMPLIED attrib: transform
-  nsCOMPtr<nsIDOMSVGTransformList> transformList;
-  rv = nsSVGTransformList::Create(getter_AddRefs(transformList));
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = NS_NewSVGAnimatedTransformList(getter_AddRefs(mTransforms),
-                                      transformList);
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = AddMappedSVGValue(nsGkAtoms::transform, mTransforms);
-  if (NS_FAILED(rv)) {
-    mTransforms = nsnull;
-    return rv;
+  if (!mTransforms) {
+    mTransforms = new SVGAnimatedTransformList();
   }
-
-  return NS_OK;
+  return mTransforms;
 }
