@@ -53,6 +53,8 @@
 
 #include "nsXULTemplateBuilder.h"
 #include "nsXULTemplateResultStorage.h"
+#include "nsXULContentUtils.h"
+#include "nsXULSortService.h"
 
 #include "mozIStorageService.h"
 
@@ -96,7 +98,7 @@ nsXULTemplateResultSetStorage::HasMoreElements(PRBool *aResult)
     // Because the nsXULTemplateResultSetStorage is owned by many nsXULTemplateResultStorage objects,
     // it could live longer than it needed to get results.
     // So we destroy the statement to free resources when all results are fetched
-    if (*aResult == PR_FALSE) {
+    if (!*aResult) {
         mStatement = nsnull;
     }
     return NS_OK;
@@ -248,7 +250,10 @@ nsXULTemplateQueryProcessorStorage::GetDatasource(nsIArray* aDataSources,
         NS_ENSURE_SUCCESS(rv, rv);
 
         nsCOMPtr<nsIFileChannel> fileChannel = do_QueryInterface(channel, &rv);
-        NS_ENSURE_SUCCESS(rv, rv); // if it fails, not a file url
+        if (NS_FAILED(rv)) { // if it fails, not a file url
+            nsXULContentUtils::LogTemplateError(ERROR_TEMPLATE_STORAGE_BAD_URI);
+            return rv;
+        }
 
         nsCOMPtr<nsIFile> file;
         rv = fileChannel->GetFile(getter_AddRefs(databaseFile));
@@ -258,7 +263,10 @@ nsXULTemplateQueryProcessorStorage::GetDatasource(nsIArray* aDataSources,
     // ok now we have an URI of a sqlite file
     nsCOMPtr<mozIStorageConnection> connection;
     rv = storage->OpenDatabase(databaseFile, getter_AddRefs(connection));
-    NS_ENSURE_SUCCESS(rv, rv);
+    if (NS_FAILED(rv)) {
+        nsXULContentUtils::LogTemplateError(ERROR_TEMPLATE_STORAGE_CANNOT_OPEN_DATABASE);
+        return rv;
+    }
 
     NS_ADDREF(*aReturn = connection);
     return NS_OK;
@@ -314,13 +322,15 @@ nsXULTemplateQueryProcessorStorage::CompileQuery(nsIXULTemplateBuilder* aBuilder
 
     nsresult rv = mStorageConnection->CreateStatement(NS_ConvertUTF16toUTF8(sqlQuery),
                                                               getter_AddRefs(statement));
-    NS_ENSURE_SUCCESS(rv, rv);
+    if (NS_FAILED(rv)) {
+        nsXULContentUtils::LogTemplateError(ERROR_TEMPLATE_STORAGE_BAD_QUERY);
+        return rv;
+    }
 
     PRUint32 parameterCount = 0;
-    PRUint32 count = queryContent->GetChildCount();
-
-    for (PRUint32 i = 0; i < count; ++i) {
-        nsIContent *child = queryContent->GetChildAt(i);
+    for (nsIContent* child = queryContent->GetFirstChild();
+         child;
+         child = child->GetNextSibling()) {
 
         if (child->NodeInfo()->Equals(nsGkAtoms::param, kNameSpaceID_XUL)) {
             nsAutoString value;
@@ -332,7 +342,10 @@ nsXULTemplateQueryProcessorStorage::CompileQuery(nsIXULTemplateBuilder* aBuilder
             if (child->GetAttr(kNameSpaceID_None, nsGkAtoms::name, name)) {
                 rv = statement->GetParameterIndex(NS_ConvertUTF16toUTF8(name),
                                                   &index);
-                NS_ENSURE_SUCCESS(rv, rv);
+                if (NS_FAILED(rv)) {
+                    nsXULContentUtils::LogTemplateError(ERROR_TEMPLATE_STORAGE_UNKNOWN_QUERY_PARAMETER);
+                    return rv;
+                }
                 parameterCount++;
             }
             else if (child->GetAttr(kNameSpaceID_None, nsGkAtoms::index, indexValue)) {
@@ -348,7 +361,8 @@ nsXULTemplateQueryProcessorStorage::CompileQuery(nsIXULTemplateBuilder* aBuilder
                 { &nsGkAtoms::int32, &nsGkAtoms::integer, &nsGkAtoms::int64,
                   &nsGkAtoms::null, &nsGkAtoms::double_, &nsGkAtoms::string, nsnull };
 
-            PRInt32 typeError, typeValue = child->FindAttrValueIn(kNameSpaceID_None, nsGkAtoms::type,
+            PRInt32 typeError = 1;
+            PRInt32 typeValue = child->FindAttrValueIn(kNameSpaceID_None, nsGkAtoms::type,
                                                        sTypeValues, eCaseMatters);
             rv = NS_ERROR_ILLEGAL_VALUE;
             PRInt32 valInt32 = 0;
@@ -360,27 +374,38 @@ nsXULTemplateQueryProcessorStorage::CompileQuery(nsIXULTemplateBuilder* aBuilder
               case 1:
                 typeError = PR_sscanf(NS_ConvertUTF16toUTF8(value).get(),"%d",&valInt32);
                 if (typeError > 0)
-                    rv = statement->BindInt32Parameter(index, valInt32);
+                    rv = statement->BindInt32ByIndex(index, valInt32);
                 break;
               case 2:
                 typeError = PR_sscanf(NS_ConvertUTF16toUTF8(value).get(),"%lld",&valInt64);
                 if (typeError > 0)
-                    rv = statement->BindInt64Parameter(index, valInt64);
+                    rv = statement->BindInt64ByIndex(index, valInt64);
                 break;
               case 3:
-                rv = statement->BindNullParameter(index);
+                rv = statement->BindNullByIndex(index);
                 break;
               case 4:
                 typeError = PR_sscanf(NS_ConvertUTF16toUTF8(value).get(),"%lf",&valFloat);
                 if (typeError > 0)
-                    rv = statement->BindDoubleParameter(index, valFloat);
+                    rv = statement->BindDoubleByIndex(index, valFloat);
                 break;
               case 5:
               case nsIContent::ATTR_MISSING:
-                rv = statement->BindStringParameter(index, value);
+                rv = statement->BindStringByIndex(index, value);
                 break;
+              default:
+                typeError = 0;
+            }
+
+            if (typeError <= 0) {
+                nsXULContentUtils::LogTemplateError(ERROR_TEMPLATE_STORAGE_WRONG_TYPE_QUERY_PARAMETER);
+                return rv;
+            }
+
+            if (NS_FAILED(rv)) {
+                nsXULContentUtils::LogTemplateError(ERROR_TEMPLATE_STORAGE_QUERY_PARAMETER_NOT_BOUND);
+                return rv;
             }
-            NS_ENSURE_SUCCESS(rv, rv);
         }
     }
 
@@ -443,6 +468,7 @@ NS_IMETHODIMP
 nsXULTemplateQueryProcessorStorage::CompareResults(nsIXULTemplateResult* aLeft,
                                                    nsIXULTemplateResult* aRight,
                                                    nsIAtom* aVar,
+                                                   PRUint32 aSortHints,
                                                    PRInt32* aResult)
 {
     *aResult = 0;
@@ -499,14 +525,12 @@ nsXULTemplateQueryProcessorStorage::CompareResults(nsIXULTemplateResult* aLeft,
     // Values are not integers or floats, so we just compare them as simple strings
     nsAutoString leftVal;
     if (aLeft)
-      aLeft->GetBindingFor(aVar, leftVal);
+        aLeft->GetBindingFor(aVar, leftVal);
 
     nsAutoString rightVal;
     if (aRight)
-      aRight->GetBindingFor(aVar, rightVal);
+        aRight->GetBindingFor(aVar, rightVal);
 
-    *aResult = Compare(nsDependentString(leftVal),
-                       nsDependentString(rightVal),
-                       nsCaseInsensitiveStringComparator());
+    *aResult = XULSortServiceImpl::CompareValues(leftVal, rightVal, aSortHints);
     return NS_OK;
 }
