@@ -49,6 +49,13 @@
 
 class nsIDocShellTreeItem;
 class nsPIDOMWindow;
+
+namespace mozilla {
+namespace dom {
+  class TabParent;
+}
+}
+
 struct nsDelayedBlurOrFocusEvent;
 
 /**
@@ -424,6 +431,12 @@ protected:
                               PRBool aIsForDocNavigation,
                               PRBool aCheckVisibility);
 
+  /**
+   * Get the TabParent associated with aContent if it is a remote browser,
+   * or null in all other cases.
+   */
+  mozilla::dom::TabParent* GetRemoteForContent(nsIContent* aContent);
+
   /**
    * Get the last docshell child of aItem and return it in aResult.
    */
@@ -467,6 +480,19 @@ protected:
                            nsIContent* aEndSelection,
                            nsIContent** aFocusedContent);
 
+private:
+  // Notify that the focus state of aContent has changed.  Note that
+  // we need to pass in whether the window should show a focus ring
+  // before the SetFocusedNode call on it happened when losing focus
+  // and after the SetFocusedNode call when gaining focus, which is
+  // why that information needs to be an explicit argument instead of
+  // just passing in the window and asking it whether it should show
+  // focus rings: in the losing focus case that information could be
+  // wrong..
+  static void NotifyFocusStateChange(nsIContent* aContent,
+                                     PRBool aWindowShouldShowFocusRing,
+                                     PRBool aGettingFocus);
+
   // the currently active and front-most top-most window
   nsCOMPtr<nsPIDOMWindow> mActiveWindow;
 
