@@ -43,6 +43,7 @@
 #include "nsISupports.h"
 #include "nsCOMPtr.h"
 #include "nsIProgrammingLanguage.h"
+#include "jspubtd.h"
 
 class nsIScriptGlobalObject;
 class nsIScriptSecurityManager;
@@ -53,13 +54,28 @@ class nsIVariant;
 class nsIObjectInputStream;
 class nsIObjectOutputStream;
 class nsScriptObjectHolder;
+class nsIScriptObjectPrincipal;
 
 typedef void (*nsScriptTerminationFunc)(nsISupports* aRef);
 
-// 87482b5e-e019-4df5-9bc2-b2a51b1f2d28
+#define NS_ISCRIPTCONTEXTPRINCIPAL_IID \
+  { 0xd012cdb3, 0x8f1e, 0x4440, \
+    { 0x8c, 0xbd, 0x32, 0x7f, 0x98, 0x1d, 0x37, 0xb4 } }
+
+class nsIScriptContextPrincipal : public nsISupports
+{
+public:
+  NS_DECLARE_STATIC_IID_ACCESSOR(NS_ISCRIPTCONTEXTPRINCIPAL_IID)
+
+  virtual nsIScriptObjectPrincipal* GetObjectPrincipal() = 0;
+};
+
+NS_DEFINE_STATIC_IID_ACCESSOR(nsIScriptContextPrincipal,
+                              NS_ISCRIPTCONTEXTPRINCIPAL_IID)
+
 #define NS_ISCRIPTCONTEXT_IID \
-{ 0x87482b5e, 0xe019, 0x4df5, \
-  { 0x9b, 0xc2, 0xb2, 0xa5, 0x1b, 0x1f, 0x2d, 0x28 } }
+{ 0x827d1e82, 0x5aab, 0x4e3a, \
+  { 0x88, 0x76, 0x53, 0xf7, 0xed, 0x1e, 0x3f, 0xbe } }
 
 /* This MUST match JSVERSION_DEFAULT.  This version stuff if we don't
    know what language we have is a little silly... */
@@ -68,11 +84,8 @@ typedef void (*nsScriptTerminationFunc)(nsISupports* aRef);
 /**
  * It is used by the application to initialize a runtime and run scripts.
  * A script runtime would implement this interface.
- * <P><I>It does have a bit too much java script information now, that
- * should be removed in a short time. Ideally this interface will be
- * language neutral</I>
  */
-class nsIScriptContext : public nsISupports
+class nsIScriptContext : public nsIScriptContextPrincipal
 {
 public:
   NS_DECLARE_STATIC_IID_ACCESSOR(NS_ISCRIPTCONTEXT_IID)
@@ -217,41 +230,30 @@ public:
                                     nsIArray *argv, nsIVariant **rval) = 0;
 
   /**
-   * Bind an already-compiled event handler function to a name in the given
-   * scope object.  The same restrictions on aName (lowercase ASCII, not too
-   * long) applies here as for CompileEventHandler.  Scripting languages with
-   * static scoping must re-bind the scope chain for aHandler to begin (after
-   * the activation scope for aHandler itself, typically) with aTarget's scope.
+   * Bind an already-compiled event handler function to the given
+   * target.  Scripting languages with static scoping must re-bind the
+   * scope chain for aHandler to begin (after the activation scope for
+   * aHandler itself, typically) with aTarget's scope.
    *
-   * Logically, this 'bind' operation is more of a 'copy' - it simply
-   * stashes/associates the event handler function with the event target, so
-   * it can be fetched later with GetBoundEventHandler().
+   * The result of the bind operation is a new handler object, with
+   * principals now set and scope set as above.  This is returned in
+   * aBoundHandler.  When this function is called, aBoundHandler is
+   * expected to not be holding an object.
    *
    * @param aTarget an object telling the scope in which to bind the compiled
    *        event handler function.  The context will presumably associate
    *        this nsISupports with a native script object.
-   * @param aName an nsIAtom pointer naming the function; it must be lowercase
-   *        and ASCII, and should not be longer than 63 chars.  This bound on
-   *        length is enforced only by assertions, so caveat caller!
-   * @param aHandler the function object to name, created by an earlier call to
+   * @param aScope the scope in which the script object for aTarget should be
+   *        looked for.
+   * @param aHandler the function object to bind, created by an earlier call to
    *        CompileEventHandler
-   * @return NS_OK if the function was successfully bound to the name
-   *
-   * XXXmarkh - fold this in with SetProperty?  Exactly the same concept!
+   * @param aBoundHandler [out] the result of the bind operation.
+   * @return NS_OK if the function was successfully bound
    */
-  virtual nsresult BindCompiledEventHandler(nsISupports* aTarget, void *aScope,
-                                            nsIAtom* aName,
-                                            void* aHandler) = 0;
-
-  /**
-   * Lookup a previously bound event handler for the specified target.  This
-   * will return an object equivilent to the one passed to
-   * BindCompiledEventHandler (although the pointer may not be the same).
-   *
-   */
-  virtual nsresult GetBoundEventHandler(nsISupports* aTarget, void *aScope,
-                                        nsIAtom* aName,
-                                        nsScriptObjectHolder &aHandler) = 0;
+  virtual nsresult BindCompiledEventHandler(nsISupports* aTarget,
+                                            void *aScope,
+                                            void* aHandler,
+                                            nsScriptObjectHolder& aBoundHandler) = 0;
 
   /**
    * Compile a function that isn't used as an event handler.
@@ -288,7 +290,7 @@ public:
    * Return the native script context
    *
    **/
-  virtual void *GetNativeContext() = 0;
+  virtual JSContext* GetNativeContext() = 0;
 
   /**
    * Return the native global object for this context.
@@ -304,6 +306,7 @@ public:
   virtual nsresult CreateNativeGlobalForInner(
                                       nsIScriptGlobalObject *aNewInner,
                                       PRBool aIsChrome,
+                                      nsIPrincipal *aPrincipal,
                                       void **aNativeGlobal,
                                       nsISupports **aHolder) = 0;
 
@@ -317,17 +320,28 @@ public:
 
 
   /**
-   * Init this context ready for use.  If aGlobalObject is not NULL, this
-   * function may initialize based on this global (for example, using the
-   * global to locate a chrome window, create a new 'scope' for this
-   * global, etc)
-   *
-   * @param aGlobalObject the gobal object, which may be nsnull.
-   *
-   * @return NS_OK if context initialization was successful
-   *
+   * Initialize the context generally. Does not create a global object.
    **/
-  virtual nsresult InitContext(nsIScriptGlobalObject *aGlobalObject) = 0;
+  virtual nsresult InitContext() = 0;
+
+  /**
+   * Creates the outer window for this context.
+   *
+   * @param aGlobalObject The script global object to use as our global.
+   */
+  virtual nsresult CreateOuterObject(nsIScriptGlobalObject *aGlobalObject,
+                                     nsIScriptGlobalObject *aCurrentInner) = 0;
+
+  /**
+   * Given an outer object, updates this context with that outer object.
+   */
+  virtual nsresult SetOuterObject(void *aOuterObject) = 0;
+
+  /**
+   * Prepares this context for use with the current inner window for the
+   * context's global object. This must be called after CreateOuterObject.
+   */
+  virtual nsresult InitOuterWindow() = 0;
 
   /**
    * Check to see if context is as yet intialized. Used to prevent
@@ -402,6 +416,11 @@ public:
   virtual PRBool GetProcessingScriptTag() = 0;
   virtual void SetProcessingScriptTag(PRBool aResult) = 0;
 
+  /**
+   * Called to find out if this script context might be executing script.
+   */
+  virtual PRBool GetExecutingScript() = 0;
+
   /**
    * Tell the context whether or not to GC when destroyed.  An optimization
    * used when the window is a [i]frame, so GC will happen anyway.
@@ -461,25 +480,7 @@ public:
   virtual void LeaveModalState() = 0;
 };
 
-// 9531D11E-97F8-4455-9DB1-5008F10BDBD6
-#define NS_ISCRIPTCONTEXT_1_9_2_IID \
-{ 0x9531D11E, 0x97F8, 0x4455, \
-  { 0x9D, 0xB1, 0x50, 0x08, 0xF1, 0x0B, 0xDB, 0xD6 } }
-
-class nsIScriptContext_1_9_2 : public nsIScriptContext
-{
-public:
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_ISCRIPTCONTEXT_1_9_2_IID)
-
-  /**
-   * Called to find out if this script context might be executing script.
-   */
-  virtual PRBool GetExecutingScript() = 0;
-};
-
-
 NS_DEFINE_STATIC_IID_ACCESSOR(nsIScriptContext, NS_ISCRIPTCONTEXT_IID)
-NS_DEFINE_STATIC_IID_ACCESSOR(nsIScriptContext_1_9_2, NS_ISCRIPTCONTEXT_1_9_2_IID)
 
 #endif // nsIScriptContext_h__
 
