@@ -40,7 +40,7 @@
 
 #include "nsIDOMKeyEvent.h"
 #include "nsIJSEventListener.h"
-#include "nsIDOMMouseListener.h"
+#include "nsIDOMEventListener.h"
 #include "jsapi.h"
 #include "nsCOMPtr.h"
 #include "nsIAtom.h"
@@ -49,12 +49,11 @@
 
 // nsJSEventListener interface
 // misnamed - JS no longer has exclusive rights over this interface!
-class nsJSEventListener : public nsIDOMEventListener,
-                          public nsIJSEventListener
+class nsJSEventListener : public nsIJSEventListener
 {
 public:
   nsJSEventListener(nsIScriptContext *aContext, void *aScopeObject,
-                    nsISupports* aObject);
+                    nsISupports* aTarget, nsIAtom* aType, void *aHandler);
   virtual ~nsJSEventListener();
 
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
@@ -62,23 +61,17 @@ public:
   // nsIDOMEventListener interface
   NS_DECL_NSIDOMEVENTLISTENER
 
-  // nsIJSEventListener interface
-  virtual void SetEventName(nsIAtom* aName);
+  // nsIJSEventListener
+  virtual void SetHandler(void *aHandler);
+  
+  virtual PRInt64 SizeOf() const
+  {
+    return sizeof(*this);
+  }
 
-  virtual nsresult GetJSVal(const nsAString& aEventName, jsval* aJSVal);
-
-  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS_AMBIGUOUS(nsJSEventListener,
-                                                         nsIDOMEventListener)
+  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(nsJSEventListener)
 protected:
   nsCOMPtr<nsIAtom> mEventName;
-  
-  enum nsReturnResult {
-     nsReturnResult_eNotSet,
-     nsReturnResult_eReverseReturnResult,
-     nsReturnResult_eDoNotReverseReturnResult
-  };
-
-  nsReturnResult mReturnResult;
 };
 
 #endif //nsJSEventListener_h__
