@@ -47,14 +47,11 @@
 #include "nsIDOMDocument.h"
 #include "nsIDocument.h"
 #include "nsIDOMEventTarget.h" 
-#include "nsIDOM3EventTarget.h" 
 #include "nsIDOMKeyEvent.h"
-#include "nsIDOMMouseListener.h"
 #include "nsISelection.h"
 #include "nsISelectionPrivate.h"
 #include "nsISelectionController.h"
 #include "nsGUIEvent.h"
-#include "nsIDOMEventGroup.h"
 #include "nsCRT.h"
 
 #include "nsIEnumerator.h"
@@ -71,14 +68,13 @@
 
 // Misc
 #include "nsEditorUtils.h"  // nsAutoEditBatch, nsAutoRules
-#include "nsIPrefBranch.h"
-#include "nsIPrefService.h"
 #include "nsUnicharUtils.h"
 #include "nsContentCID.h"
 #include "nsInternetCiter.h"
 #include "nsEventDispatcher.h"
 #include "nsGkAtoms.h"
 #include "nsDebug.h"
+#include "mozilla/Preferences.h"
 
 // Drag & Drop, Clipboard
 #include "nsIClipboard.h"
@@ -87,6 +83,8 @@
 
 #include "mozilla/FunctionTimer.h"
 
+using namespace mozilla;
+
 nsPlaintextEditor::nsPlaintextEditor()
 : nsEditor()
 , mIgnoreSpuriousDragEvent(PR_FALSE)
@@ -171,10 +169,11 @@ static int
 EditorPrefsChangedCallback(const char *aPrefName, void *)
 {
   if (nsCRT::strcmp(aPrefName, "editor.singleLine.pasteNewlines") == 0) {
-    sNewlineHandlingPref = nsContentUtils::GetIntPref("editor.singleLine.pasteNewlines",
-                                                      nsIPlaintextEditor::eNewlinesPasteToFirst);
+    sNewlineHandlingPref =
+      Preferences::GetInt("editor.singleLine.pasteNewlines",
+                          nsIPlaintextEditor::eNewlinesPasteToFirst);
   } else if (nsCRT::strcmp(aPrefName, "layout.selection.caret_style") == 0) {
-    sCaretStylePref = nsContentUtils::GetIntPref("layout.selection.caret_style",
+    sCaretStylePref = Preferences::GetInt("layout.selection.caret_style",
 #ifdef XP_WIN
                                                  1);
     if (sCaretStylePref == 0)
@@ -192,13 +191,11 @@ nsPlaintextEditor::GetDefaultEditorPrefs(PRInt32 &aNewlineHandling,
                                          PRInt32 &aCaretStyle)
 {
   if (sNewlineHandlingPref == -1) {
-    nsContentUtils::RegisterPrefCallback("editor.singleLine.pasteNewlines",
-                                         EditorPrefsChangedCallback,
-                                         nsnull);
+    Preferences::RegisterCallback(EditorPrefsChangedCallback,
+                                  "editor.singleLine.pasteNewlines");
     EditorPrefsChangedCallback("editor.singleLine.pasteNewlines", nsnull);
-    nsContentUtils::RegisterPrefCallback("layout.selection.caret_style",
-                                         EditorPrefsChangedCallback,
-                                         nsnull);
+    Preferences::RegisterCallback(EditorPrefsChangedCallback,
+                                  "layout.selection.caret_style");
     EditorPrefsChangedCallback("layout.selection.caret_style", nsnull);
   }
 
@@ -1141,12 +1138,8 @@ nsPlaintextEditor::SetWrapWidth(PRInt32 aWrapColumn)
   // We may reset mWrapToWindow here, based on the pref's current value.
   if (IsMailEditor())
   {
-    nsresult rv;
-    nsCOMPtr<nsIPrefBranch> prefBranch =
-      do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
-    if (NS_SUCCEEDED(rv))
-      prefBranch->GetBoolPref("mail.compose.wrap_to_window_width",
-                              &mWrapToWindow);
+    mWrapToWindow =
+      Preferences::GetBool("mail.compose.wrap_to_window_width", mWrapToWindow);
   }
 
   // and now we're ready to set the new whitespace/wrapping style.
@@ -1220,6 +1213,7 @@ nsPlaintextEditor::Undo(PRUint32 aCount)
     result = mRules->DidDoAction(selection, &ruleInfo, result);
   } 
    
+  NotifyEditorObservers();
   return result;
 }
 
@@ -1249,6 +1243,7 @@ nsPlaintextEditor::Redo(PRUint32 aCount)
     result = mRules->DidDoAction(selection, &ruleInfo, result);
   } 
    
+  NotifyEditorObservers();
   return result;
 }
 
@@ -1696,14 +1691,34 @@ nsPlaintextEditor::SelectEntireDocument(nsISelection *aSelection)
     return aSelection->Collapse(rootElement, 0);
   }
 
-  return nsEditor::SelectEntireDocument(aSelection);
+  nsresult rv = nsEditor::SelectEntireDocument(aSelection);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // Don't select the trailing BR node if we have one
+  PRInt32 selOffset;
+  nsCOMPtr<nsIDOMNode> selNode;
+  rv = GetEndNodeAndOffset(aSelection, getter_AddRefs(selNode), &selOffset);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIDOMNode> childNode = GetChildAt(selNode, selOffset - 1);
+
+  if (childNode && nsTextEditUtils::IsMozBR(childNode)) {
+    nsCOMPtr<nsIDOMNode> parentNode;
+    PRInt32 parentOffset;
+    rv = GetNodeLocation(childNode, address_of(parentNode), &parentOffset);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    return aSelection->Extend(parentNode, parentOffset);
+  }
+
+  return NS_OK;
 }
 
-already_AddRefed<nsPIDOMEventTarget>
-nsPlaintextEditor::GetPIDOMEventTarget()
+already_AddRefed<nsIDOMEventTarget>
+nsPlaintextEditor::GetDOMEventTarget()
 {
-  NS_IF_ADDREF(mEventTarget);
-  return mEventTarget.get();
+  nsCOMPtr<nsIDOMEventTarget> copy = mEventTarget;
+  return copy.forget();
 }
 
 
