@@ -37,6 +37,9 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "mozilla/dom/ContentParent.h"
+#include "mozilla/dom/ContentChild.h"
+#include "mozilla/unused.h"
 #include "nsPermissionManager.h"
 #include "nsPermission.h"
 #include "nsCRT.h"
@@ -54,6 +57,51 @@
 #include "mozIStorageConnection.h"
 #include "mozStorageHelper.h"
 #include "mozStorageCID.h"
+#include "nsXULAppAPI.h"
+
+static nsPermissionManager *gPermissionManager = nsnull;
+
+using mozilla::dom::ContentParent;
+using mozilla::dom::ContentChild;
+using mozilla::unused; // ha!
+
+static PRBool
+IsChildProcess()
+{
+  return XRE_GetProcessType() == GeckoProcessType_Content;
+}
+
+/**
+ * @returns The child process object, or if we are not in the child
+ *          process, nsnull.
+ */
+static ContentChild*
+ChildProcess()
+{
+  if (IsChildProcess()) {
+    ContentChild* cpc = ContentChild::GetSingleton();
+    if (!cpc)
+      NS_RUNTIMEABORT("Content Process is NULL!");
+    return cpc;
+  }
+
+  return nsnull;
+}
+
+
+#define ENSURE_NOT_CHILD_PROCESS_(onError) \
+  PR_BEGIN_MACRO \
+  if (IsChildProcess()) { \
+    NS_ERROR("Cannot perform action in content process!"); \
+    onError \
+  } \
+  PR_END_MACRO
+
+#define ENSURE_NOT_CHILD_PROCESS \
+  ENSURE_NOT_CHILD_PROCESS_({ return NS_ERROR_NOT_AVAILABLE; })
+
+#define ENSURE_NOT_CHILD_PROCESS_NORET \
+  ENSURE_NOT_CHILD_PROCESS_()
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -95,7 +143,7 @@ nsHostEntry::nsHostEntry(const nsHostEntry& toCopy)
 // nsPermissionManager Implementation
 
 static const char kPermissionsFileName[] = "permissions.sqlite";
-#define HOSTS_SCHEMA_VERSION 1
+#define HOSTS_SCHEMA_VERSION 2
 
 static const char kHostpermFileName[] = "hostperm.1";
 
@@ -111,6 +159,33 @@ nsPermissionManager::nsPermissionManager()
 nsPermissionManager::~nsPermissionManager()
 {
   RemoveAllFromMemory();
+  gPermissionManager = nsnull;
+}
+
+// static
+nsIPermissionManager*
+nsPermissionManager::GetXPCOMSingleton()
+{
+  if (gPermissionManager) {
+    NS_ADDREF(gPermissionManager);
+    return gPermissionManager;
+  }
+
+  // Create a new singleton nsPermissionManager.
+  // We AddRef only once since XPCOM has rules about the ordering of module
+  // teardowns - by the time our module destructor is called, it's too late to
+  // Release our members, since GC cycles have already been completed and
+  // would result in serious leaks.
+  // See bug 209571.
+  gPermissionManager = new nsPermissionManager();
+  if (gPermissionManager) {
+    NS_ADDREF(gPermissionManager);
+    if (NS_FAILED(gPermissionManager->Init())) {
+      NS_RELEASE(gPermissionManager);
+    }
+  }
+
+  return gPermissionManager;
 }
 
 nsresult
@@ -122,17 +197,32 @@ nsPermissionManager::Init()
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  // ignore failure here, since it's non-fatal (we can run fine without
-  // persistent storage - e.g. if there's no profile).
-  // XXX should we tell the user about this?
-  InitDB(PR_FALSE);
-
   mObserverService = do_GetService("@mozilla.org/observer-service;1", &rv);
   if (NS_SUCCEEDED(rv)) {
     mObserverService->AddObserver(this, "profile-before-change", PR_TRUE);
     mObserverService->AddObserver(this, "profile-do-change", PR_TRUE);
   }
 
+  if (IsChildProcess()) {
+    // Get the permissions from the parent process
+    InfallibleTArray<IPC::Permission> perms;
+    ChildProcess()->SendReadPermissions(&perms);
+
+    for (PRUint32 i = 0; i < perms.Length(); i++) {
+      const IPC::Permission &perm = perms[i];
+      AddInternal(perm.host, perm.type, perm.capability, 0, perm.expireType,
+                  perm.expireTime, eNotify, eNoDBOperation);
+    }
+
+    // Stop here; we don't need the DB in the child process
+    return NS_OK;
+  }
+
+  // ignore failure here, since it's non-fatal (we can run fine without
+  // persistent storage - e.g. if there's no profile).
+  // XXX should we tell the user about this?
+  InitDB(PR_FALSE);
+
   return NS_OK;
 }
 
@@ -183,8 +273,8 @@ nsPermissionManager::InitDB(PRBool aRemoveFile)
   PRBool tableExists = PR_FALSE;
   mDBConn->TableExists(NS_LITERAL_CSTRING("moz_hosts"), &tableExists);
   if (!tableExists) {
-      rv = CreateTable();
-      NS_ENSURE_SUCCESS(rv, rv);
+    rv = CreateTable();
+    NS_ENSURE_SUCCESS(rv, rv);
 
   } else {
     // table already exists; check the schema version before reading
@@ -198,6 +288,24 @@ nsPermissionManager::InitDB(PRBool aRemoveFile)
     // the upgrading code from the previous version to the new one.
     // fall through to current version
 
+    case 1:
+      {
+        // previous non-expiry version of database.  Upgrade it by adding the
+        // expiration columns
+        rv = mDBConn->ExecuteSimpleSQL(NS_LITERAL_CSTRING(
+              "ALTER TABLE moz_hosts ADD expireType INTEGER"));
+        NS_ENSURE_SUCCESS(rv, rv);
+
+        rv = mDBConn->ExecuteSimpleSQL(NS_LITERAL_CSTRING(
+              "ALTER TABLE moz_hosts ADD expireTime INTEGER"));
+        NS_ENSURE_SUCCESS(rv, rv);
+
+        rv = mDBConn->SetSchemaVersion(HOSTS_SCHEMA_VERSION);
+        NS_ENSURE_SUCCESS(rv, rv);
+      }
+
+      // fall through to the next upgrade
+      
     // current version.
     case HOSTS_SCHEMA_VERSION:
       break;
@@ -227,7 +335,8 @@ nsPermissionManager::InitDB(PRBool aRemoveFile)
         // check if all the expected columns exist
         nsCOMPtr<mozIStorageStatement> stmt;
         rv = mDBConn->CreateStatement(NS_LITERAL_CSTRING(
-          "SELECT host, type, permission FROM moz_hosts"), getter_AddRefs(stmt));
+          "SELECT host, type, permission, expireType, expireTime FROM moz_hosts"),
+          getter_AddRefs(stmt));
         if (NS_SUCCEEDED(rv))
           break;
 
@@ -248,8 +357,8 @@ nsPermissionManager::InitDB(PRBool aRemoveFile)
   // cache frequently used statements (for insertion, deletion, and updating)
   rv = mDBConn->CreateStatement(NS_LITERAL_CSTRING(
     "INSERT INTO moz_hosts "
-    "(id, host, type, permission) "
-    "VALUES (?1, ?2, ?3, ?4)"), getter_AddRefs(mStmtInsert));
+    "(id, host, type, permission, expireType, expireTime) "
+    "VALUES (?1, ?2, ?3, ?4, ?5, ?6)"), getter_AddRefs(mStmtInsert));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = mDBConn->CreateStatement(NS_LITERAL_CSTRING(
@@ -259,7 +368,8 @@ nsPermissionManager::InitDB(PRBool aRemoveFile)
 
   rv = mDBConn->CreateStatement(NS_LITERAL_CSTRING(
     "UPDATE moz_hosts "
-    "SET permission = ?2 WHERE id = ?1"), getter_AddRefs(mStmtUpdate));
+    "SET permission = ?2, expireType= ?3, expireTime = ?4 WHERE id = ?1"),
+    getter_AddRefs(mStmtUpdate));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // check whether to import or just read in the db
@@ -278,30 +388,48 @@ nsPermissionManager::CreateTable()
   if (NS_FAILED(rv)) return rv;
 
   // create the table
+  // SQL also lives in automation.py.in. If you change this SQL change that
+  // one too.
   return mDBConn->ExecuteSimpleSQL(NS_LITERAL_CSTRING(
     "CREATE TABLE moz_hosts ("
       " id INTEGER PRIMARY KEY"
       ",host TEXT"
       ",type TEXT"
       ",permission INTEGER"
+      ",expireType INTEGER"
+      ",expireTime INTEGER"
     ")"));
 }
 
 NS_IMETHODIMP
 nsPermissionManager::Add(nsIURI     *aURI,
                          const char *aType,
-                         PRUint32    aPermission)
+                         PRUint32    aPermission,
+                         PRUint32    aExpireType,
+                         PRInt64     aExpireTime)
 {
+  ENSURE_NOT_CHILD_PROCESS;
+
   NS_ENSURE_ARG_POINTER(aURI);
   NS_ENSURE_ARG_POINTER(aType);
+  NS_ENSURE_TRUE(aExpireType == nsIPermissionManager::EXPIRE_NEVER ||
+                 aExpireType == nsIPermissionManager::EXPIRE_TIME ||
+                 aExpireType == nsIPermissionManager::EXPIRE_SESSION,
+                 NS_ERROR_INVALID_ARG);
 
   nsresult rv;
 
+  // Skip addition if the permission is already expired.
+  if (aExpireType == nsIPermissionManager::EXPIRE_TIME &&
+      aExpireTime <= PR_Now() / 1000)
+    return NS_OK;
+
   nsCAutoString host;
   rv = GetHost(aURI, host);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  return AddInternal(host, nsDependentCString(aType), aPermission, 0, eNotify, eWriteToDB);
+  return AddInternal(host, nsDependentCString(aType), aPermission, 0, 
+                     aExpireType, aExpireTime, eNotify, eWriteToDB);
 }
 
 nsresult
@@ -309,9 +437,25 @@ nsPermissionManager::AddInternal(const nsAFlatCString &aHost,
                                  const nsAFlatCString &aType,
                                  PRUint32              aPermission,
                                  PRInt64               aID,
+                                 PRUint32              aExpireType,
+                                 PRInt64               aExpireTime,
                                  NotifyOperationType   aNotifyOperation,
                                  DBOperationType       aDBOperation)
 {
+  if (!IsChildProcess()) {
+    IPC::Permission permission((aHost),
+                               (aType),
+                               aPermission, aExpireType, aExpireTime);
+
+    nsTArray<ContentParent*> cplist;
+    ContentParent::GetAll(cplist);
+    for (PRUint32 i = 0; i < cplist.Length(); ++i) {
+      ContentParent* cp = cplist[i];
+      if (cp->NeedsPermissionsUpdate())
+        unused << cp->SendAddPermission(permission);
+    }
+  }
+
   if (!gHostArena) {
     gHostArena = new PLArenaPool;
     if (!gHostArena)
@@ -335,7 +479,6 @@ nsPermissionManager::AddInternal(const nsAFlatCString &aHost,
   // figure out the transaction type, and get any existing permission value
   OperationType op;
   PRInt32 index = entry->GetPermissionIndex(typeIndex);
-  PRUint32 oldPermission;
   if (index == -1) {
     if (aPermission == nsIPermissionManager::UNKNOWN_ACTION)
       op = eOperationNone;
@@ -343,9 +486,17 @@ nsPermissionManager::AddInternal(const nsAFlatCString &aHost,
       op = eOperationAdding;
 
   } else {
-    oldPermission = entry->GetPermissions()[index].mPermission;
+    nsPermissionEntry oldPermissionEntry = entry->GetPermissions()[index];
 
-    if (aPermission == oldPermission)
+    // remove the permission if the permission is UNKNOWN, update the
+    // permission if its value or expire type have changed OR if the time has
+    // changed and the expire type is time, otherwise, don't modify.  There's
+    // no need to modify a permission that doesn't expire with time when the
+    // only thing changed is the expire time.
+    if (aPermission == oldPermissionEntry.mPermission && 
+        aExpireType == oldPermissionEntry.mExpireType &&
+        (aExpireType != nsIPermissionManager::EXPIRE_TIME || 
+         aExpireTime == oldPermissionEntry.mExpireTime))
       op = eOperationNone;
     else if (aPermission == nsIPermissionManager::UNKNOWN_ACTION)
       op = eOperationRemoving;
@@ -373,15 +524,17 @@ nsPermissionManager::AddInternal(const nsAFlatCString &aHost,
         id = aID;
       }
 
-      entry->GetPermissions().AppendElement(nsPermissionEntry(typeIndex, aPermission, id));
+      entry->GetPermissions().AppendElement(nsPermissionEntry(typeIndex, aPermission, id, aExpireType, aExpireTime));
 
-      if (aDBOperation == eWriteToDB)
-        UpdateDB(op, mStmtInsert, id, aHost, aType, aPermission);
+      if (aDBOperation == eWriteToDB && aExpireType != nsIPermissionManager::EXPIRE_SESSION)
+        UpdateDB(op, mStmtInsert, id, aHost, aType, aPermission, aExpireType, aExpireTime);
 
       if (aNotifyOperation == eNotify) {
         NotifyObserversWithPermission(aHost,
                                       mTypeArray[typeIndex],
                                       aPermission,
+                                      aExpireType,
+                                      aExpireTime,
                                       NS_LITERAL_STRING("added").get());
       }
 
@@ -390,7 +543,8 @@ nsPermissionManager::AddInternal(const nsAFlatCString &aHost,
 
   case eOperationRemoving:
     {
-      id = entry->GetPermissions()[index].mID;
+      nsPermissionEntry oldPermissionEntry = entry->GetPermissions()[index];
+      id = oldPermissionEntry.mID;
       entry->GetPermissions().RemoveElementAt(index);
 
       // If no more types are present, remove the entry
@@ -398,12 +552,15 @@ nsPermissionManager::AddInternal(const nsAFlatCString &aHost,
         mHostTable.RawRemoveEntry(entry);
 
       if (aDBOperation == eWriteToDB)
-        UpdateDB(op, mStmtDelete, id, EmptyCString(), EmptyCString(), 0);
+        UpdateDB(op, mStmtDelete, id, EmptyCString(), EmptyCString(), 0, 
+                 nsIPermissionManager::EXPIRE_NEVER, 0);
 
       if (aNotifyOperation == eNotify) {
         NotifyObserversWithPermission(aHost,
                                       mTypeArray[typeIndex],
-                                      oldPermission,
+                                      oldPermissionEntry.mPermission,
+                                      oldPermissionEntry.mExpireType,
+                                      oldPermissionEntry.mExpireTime,
                                       NS_LITERAL_STRING("deleted").get());
       }
 
@@ -415,13 +572,15 @@ nsPermissionManager::AddInternal(const nsAFlatCString &aHost,
       id = entry->GetPermissions()[index].mID;
       entry->GetPermissions()[index].mPermission = aPermission;
 
-      if (aDBOperation == eWriteToDB)
-        UpdateDB(op, mStmtUpdate, id, EmptyCString(), EmptyCString(), aPermission);
+      if (aDBOperation == eWriteToDB && aExpireType != nsIPermissionManager::EXPIRE_SESSION)
+        UpdateDB(op, mStmtUpdate, id, EmptyCString(), EmptyCString(), aPermission, aExpireType, aExpireTime);
 
       if (aNotifyOperation == eNotify) {
         NotifyObserversWithPermission(aHost,
                                       mTypeArray[typeIndex],
                                       aPermission,
+                                      aExpireType,
+                                      aExpireTime,
                                       NS_LITERAL_STRING("changed").get());
       }
 
@@ -436,6 +595,8 @@ NS_IMETHODIMP
 nsPermissionManager::Remove(const nsACString &aHost,
                             const char       *aType)
 {
+  ENSURE_NOT_CHILD_PROCESS;
+
   NS_ENSURE_ARG_POINTER(aType);
 
   // AddInternal() handles removal, just let it do the work
@@ -443,6 +604,8 @@ nsPermissionManager::Remove(const nsACString &aHost,
                      nsDependentCString(aType),
                      nsIPermissionManager::UNKNOWN_ACTION,
                      0,
+                     nsIPermissionManager::EXPIRE_NEVER,
+                     0,
                      eNotify,
                      eWriteToDB);
 }
@@ -450,6 +613,8 @@ nsPermissionManager::Remove(const nsACString &aHost,
 NS_IMETHODIMP
 nsPermissionManager::RemoveAll()
 {
+  ENSURE_NOT_CHILD_PROCESS;
+
   nsresult rv = RemoveAllInternal();
   NotifyObservers(nsnull, NS_LITERAL_STRING("cleared").get());
   return rv;
@@ -506,8 +671,19 @@ nsPermissionManager::CommonTestPermission(nsIURI     *aURI,
 
   nsCAutoString host;
   nsresult rv = GetHost(aURI, host);
-  // no host doesn't mean an error. just return the default
-  if (NS_FAILED(rv)) return NS_OK;
+  // No host doesn't mean an error. Just return the default. Unless this is
+  // a file uri. In that case use a magic host.
+  if (NS_FAILED(rv)) {
+    PRBool isFile;
+    rv = aURI->SchemeIs("file", &isFile);
+    NS_ENSURE_SUCCESS(rv, rv);
+    if (isFile) {
+      host.AssignLiteral("<file>");
+    }
+    else {
+      return NS_OK;
+    }
+  }
   
   PRInt32 typeIndex = GetTypeIndex(aType, PR_FALSE);
   // If type == -1, the type isn't known,
@@ -516,7 +692,7 @@ nsPermissionManager::CommonTestPermission(nsIURI     *aURI,
 
   nsHostEntry *entry = GetHostEntry(host, typeIndex, aExactHostMatch);
   if (entry)
-    *aPermission = entry->GetPermission(typeIndex);
+    *aPermission = entry->GetPermission(typeIndex).mPermission;
 
   return NS_OK;
 }
@@ -524,6 +700,8 @@ nsPermissionManager::CommonTestPermission(nsIURI     *aURI,
 // Get hostentry for given host string and permission type.
 // walk up the domain if needed.
 // return null if nothing found.
+// Also accepts host on the format "<foo>". This will perform an exact match
+// lookup as the string doesn't contain any dots.
 nsHostEntry *
 nsPermissionManager::GetHostEntry(const nsAFlatCString &aHost,
                                   PRUint32              aType,
@@ -531,10 +709,18 @@ nsPermissionManager::GetHostEntry(const nsAFlatCString &aHost,
 {
   PRUint32 offset = 0;
   nsHostEntry *entry;
+  PRInt64 now = PR_Now() / 1000;
+
   do {
     entry = mHostTable.GetEntry(aHost.get() + offset);
     if (entry) {
-      if (entry->GetPermission(aType) != nsIPermissionManager::UNKNOWN_ACTION)
+      nsPermissionEntry permEntry = entry->GetPermission(aType);
+
+      // if the entry is expired, remove and keep looking for others.
+      if (permEntry.mExpireType == nsIPermissionManager::EXPIRE_TIME &&
+          permEntry.mExpireTime <= now)
+        Remove(aHost, mTypeArray[aType].get());
+      else if (permEntry.mPermission != nsIPermissionManager::UNKNOWN_ACTION)
         break;
 
       // reset entry, to be able to return null on failure
@@ -572,7 +758,9 @@ AddPermissionsToList(nsHostEntry *entry, void *arg)
 
     nsPermission *perm = new nsPermission(entry->GetHost(), 
                                           data->types->ElementAt(permEntry.mType),
-                                          permEntry.mPermission);
+                                          permEntry.mPermission,
+                                          permEntry.mExpireType,
+                                          permEntry.mExpireTime);
 
     data->array->AppendObject(perm);
   }
@@ -593,6 +781,8 @@ NS_IMETHODIMP nsPermissionManager::GetEnumerator(nsISimpleEnumerator **aEnum)
 
 NS_IMETHODIMP nsPermissionManager::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *someData)
 {
+  ENSURE_NOT_CHILD_PROCESS;
+
   if (!nsCRT::strcmp(aTopic, "profile-before-change")) {
     // The profile is about to change,
     // or is going away because the application is shutting down.
@@ -602,7 +792,7 @@ NS_IMETHODIMP nsPermissionManager::Observe(nsISupports *aSubject, const char *aT
     } else {
       RemoveAllFromMemory();
     }
-  }  
+  }
   else if (!nsCRT::strcmp(aTopic, "profile-do-change")) {
     // the profile has already changed; init the db from the new location
     InitDB(PR_FALSE);
@@ -653,15 +843,18 @@ nsPermissionManager::GetTypeIndex(const char *aType,
   return mTypeArray.Length() - 1;
 }
 
-// wrapper function for mangling (host,type,perm) triplet into an nsIPermission.
+// wrapper function for mangling (host,type,perm,expireType,expireTime)
+// set into an nsIPermission.
 void
 nsPermissionManager::NotifyObserversWithPermission(const nsACString &aHost,
                                                    const nsCString  &aType,
                                                    PRUint32          aPermission,
+                                                   PRUint32          aExpireType,
+                                                   PRInt64           aExpireTime,
                                                    const PRUnichar  *aData)
 {
   nsCOMPtr<nsIPermission> permission =
-    new nsPermission(aHost, aType, aPermission);
+    new nsPermission(aHost, aType, aPermission, aExpireType, aExpireTime);
   if (permission)
     NotifyObservers(permission, aData);
 }
@@ -685,17 +878,41 @@ nsPermissionManager::NotifyObservers(nsIPermission   *aPermission,
 nsresult
 nsPermissionManager::Read()
 {
+  ENSURE_NOT_CHILD_PROCESS;
+
   nsresult rv;
 
+  // delete expired permissions before we read in the db
+  {
+    // this deletion has its own scope so the write lock is released when done.
+    nsCOMPtr<mozIStorageStatement> stmtDeleteExpired;
+    rv = mDBConn->CreateStatement(NS_LITERAL_CSTRING(
+          "DELETE FROM moz_hosts WHERE expireType = ?1 AND expireTime <= ?2"),
+          getter_AddRefs(stmtDeleteExpired));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = stmtDeleteExpired->BindInt32ByIndex(0, nsIPermissionManager::EXPIRE_TIME);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = stmtDeleteExpired->BindInt64ByIndex(1, PR_Now() / 1000);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    PRBool hasResult;
+    rv = stmtDeleteExpired->ExecuteStep(&hasResult);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
   nsCOMPtr<mozIStorageStatement> stmt;
   rv = mDBConn->CreateStatement(NS_LITERAL_CSTRING(
-    "SELECT id, host, type, permission "
+    "SELECT id, host, type, permission, expireType, expireTime "
     "FROM moz_hosts"), getter_AddRefs(stmt));
   NS_ENSURE_SUCCESS(rv, rv);
 
   PRInt64 id;
   nsCAutoString host, type;
   PRUint32 permission;
+  PRUint32 expireType;
+  PRInt64 expireTime;
   PRBool hasResult;
   while (NS_SUCCEEDED(stmt->ExecuteStep(&hasResult)) && hasResult) {
     // explicitly set our entry id counter for use in AddInternal(),
@@ -711,8 +928,13 @@ nsPermissionManager::Read()
     NS_ENSURE_SUCCESS(rv, rv);
 
     permission = stmt->AsInt32(3);
+    expireType = stmt->AsInt32(4);
 
-    rv = AddInternal(host, type, permission, id, eDontNotify, eNoDBOperation);
+    // convert into PRInt64 value (milliseconds)
+    expireTime = stmt->AsInt64(5);
+
+    rv = AddInternal(host, type, permission, id, expireType, expireTime,
+                     eDontNotify, eNoDBOperation);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -724,6 +946,8 @@ static const char kMatchTypeHost[] = "host";
 nsresult
 nsPermissionManager::Import()
 {
+  ENSURE_NOT_CHILD_PROCESS;
+
   nsresult rv;
 
   nsCOMPtr<nsIFile> permissionsFile;
@@ -779,7 +1003,8 @@ nsPermissionManager::Import()
           continue;
       }
 
-      rv = AddInternal(lineArray[3], lineArray[1], permission, 0, eDontNotify, eWriteToDB);
+      rv = AddInternal(lineArray[3], lineArray[1], permission, 0, 
+                       nsIPermissionManager::EXPIRE_NEVER, 0, eDontNotify, eWriteToDB);
       NS_ENSURE_SUCCESS(rv, rv);
     }
   }
@@ -823,8 +1048,12 @@ nsPermissionManager::UpdateDB(OperationType         aOp,
                               PRInt64               aID,
                               const nsACString     &aHost,
                               const nsACString     &aType,
-                              PRUint32              aPermission)
+                              PRUint32              aPermission,
+                              PRUint32              aExpireType,
+                              PRInt64               aExpireTime)
 {
+  ENSURE_NOT_CHILD_PROCESS_NORET;
+
   nsresult rv;
 
   // no statement is ok - just means we don't have a profile
@@ -834,31 +1063,43 @@ nsPermissionManager::UpdateDB(OperationType         aOp,
   switch (aOp) {
   case eOperationAdding:
     {
-      rv = aStmt->BindInt64Parameter(0, aID);
+      rv = aStmt->BindInt64ByIndex(0, aID);
       if (NS_FAILED(rv)) break;
 
-      rv = aStmt->BindUTF8StringParameter(1, aHost);
+      rv = aStmt->BindUTF8StringByIndex(1, aHost);
       if (NS_FAILED(rv)) break;
       
-      rv = aStmt->BindUTF8StringParameter(2, aType);
+      rv = aStmt->BindUTF8StringByIndex(2, aType);
       if (NS_FAILED(rv)) break;
 
-      rv = aStmt->BindInt32Parameter(3, aPermission);
+      rv = aStmt->BindInt32ByIndex(3, aPermission);
+      if (NS_FAILED(rv)) break;
+
+      rv = aStmt->BindInt32ByIndex(4, aExpireType);
+      if (NS_FAILED(rv)) break;
+
+      rv = aStmt->BindInt64ByIndex(5, aExpireTime);
       break;
     }
 
   case eOperationRemoving:
     {
-      rv = aStmt->BindInt64Parameter(0, aID);
+      rv = aStmt->BindInt64ByIndex(0, aID);
       break;
     }
 
   case eOperationChanging:
     {
-      rv = aStmt->BindInt64Parameter(0, aID);
+      rv = aStmt->BindInt64ByIndex(0, aID);
       if (NS_FAILED(rv)) break;
 
-      rv = aStmt->BindInt32Parameter(1, aPermission);
+      rv = aStmt->BindInt32ByIndex(1, aPermission);
+      if (NS_FAILED(rv)) break;
+
+      rv = aStmt->BindInt32ByIndex(2, aExpireType);
+      if (NS_FAILED(rv)) break;
+
+      rv = aStmt->BindInt64ByIndex(3, aExpireTime);
       break;
     }
 
