@@ -66,7 +66,7 @@
 #include "nsAutoPtr.h"
 #include "nsThreadUtils.h"
 
-#if defined(MOZ_PLATFORM_MAEMO) || defined(ANDROID)
+#if defined(MOZ_PLATFORM_MAEMO) || defined(ANDROID) || defined(MOZ_EGL_XRENDER_COMPOSITE)
 #define USE_GLES2 1
 #endif
 
@@ -75,8 +75,12 @@ typedef char realGLboolean;
 #include "GLContextSymbols.h"
 
 namespace mozilla {
-namespace gl {
+  namespace layers {
+    class LayerManagerOGL;
+    class ColorTextureLayerProgram;
+  };
 
+namespace gl {
 class GLContext;
 
 class LibrarySymbolLoader
@@ -156,6 +160,13 @@ class TextureImage
 {
     NS_INLINE_DECL_REFCOUNTING(TextureImage)
 public:
+    enum TextureState
+    {
+      Created, // Texture created, but has not had glTexImage called to initialize it.
+      Allocated,  // Texture memory exists, but contents are invalid.
+      Valid  // Texture fully ready to use.
+    };
+
     typedef gfxASurface::gfxContentType ContentType;
 
     virtual ~TextureImage() {}
@@ -182,6 +193,15 @@ public:
      * followed by EndUpdate().
      */
     virtual gfxASurface* BeginUpdate(nsIntRegion& aRegion) = 0;
+    /**
+     * Retrieves the region that will require updating, given a
+     * region that needs to be updated. This can be used for
+     * making decisions about updating before calling BeginUpdate().
+     *
+     * |aRegion| is an inout param.
+     */
+    virtual void GetUpdateRegion(nsIntRegion& aForRegion) {
+    };
     /**
      * Finish the active update and synchronize with the server, if
      * necessary.
@@ -191,6 +211,27 @@ public:
      */
     virtual void EndUpdate() = 0;
 
+    /**
+     * The Image may contain several textures for different regions (tiles).
+     * These functions iterate over each sub texture image tile.
+     */
+    virtual void BeginTileIteration() {
+    };
+
+    virtual PRBool NextTile() {
+        return PR_FALSE;
+    };
+
+    virtual nsIntRect GetTileRect() {
+        return nsIntRect(nsIntPoint(0,0), mSize);
+    };
+
+    virtual GLuint GetTextureID() = 0;
+
+    virtual PRUint32 GetTileCount() {
+        return 1;
+    };
+
     /**
      * Set this TextureImage's size, and ensure a texture has been
      * allocated.  Must not be called between BeginUpdate and EndUpdate.
@@ -206,7 +247,12 @@ public:
         EndUpdate();
     }
 
-    virtual bool DirectUpdate(gfxASurface *aSurf, const nsIntRegion& aRegion) =0;
+    /**
+     * aSurf - the source surface to update from
+     * aRegion - the region in this image to update
+     * aFrom - offset in the source to update from
+     */
+    virtual bool DirectUpdate(gfxASurface *aSurf, const nsIntRegion& aRegion, const nsIntPoint& aFrom = nsIntPoint(0,0)) = 0;
 
     virtual void BindTexture(GLenum aTextureUnit) = 0;
     virtual void ReleaseTexture() {};
@@ -233,16 +279,6 @@ public:
         TextureImage *mTexture;
     };
 
-    /**
-     * Return this TextureImage's texture ID for use with GL APIs.
-     * Callers are responsible for properly binding the texture etc.
-     *
-     * The texture is only texture complete after either Resize
-     * or a matching pair of BeginUpdate/EndUpdate have been called.
-     * Otherwise, a texture ID may be returned, but the texture
-     * may not be texture complete.
-     */
-    GLuint Texture() { return mTexture; }
 
     /**
      * Returns the shader program type that should be used to render
@@ -279,17 +315,15 @@ protected:
      * TextureImage from GLContext::CreateTextureImage().  That is,
      * clients must not be given partially-constructed TextureImages.
      */
-    TextureImage(GLuint aTexture, const nsIntSize& aSize,
+    TextureImage(const nsIntSize& aSize,
                  GLenum aWrapMode, ContentType aContentType,
                  PRBool aIsRGB = PR_FALSE)
-        : mTexture(aTexture)
-        , mSize(aSize)
+        : mSize(aSize)
         , mWrapMode(aWrapMode)
         , mContentType(aContentType)
         , mIsRGBFormat(aIsRGB)
     {}
 
-    GLuint mTexture;
     nsIntSize mSize;
     GLenum mWrapMode;
     ContentType mContentType;
@@ -318,25 +352,20 @@ public:
                       GLenum aWrapMode,
                       ContentType aContentType,
                       GLContext* aContext)
-        : TextureImage(aTexture, aSize, aWrapMode, aContentType)
+        : TextureImage(aSize, aWrapMode, aContentType)
+        , mTexture(aTexture)
         , mTextureState(Created)
         , mGLContext(aContext)
         , mUpdateOffset(0, 0)
     {}
 
-    enum TextureState
-    {
-      Created, // Texture created, but has not had glTexImage called to initialize it.
-      Allocated,  // Texture memory exists, but contents are invalid.
-      Valid  // Texture fully ready to use.
-    };
-    
     virtual void BindTexture(GLenum aTextureUnit);
 
     virtual gfxASurface* BeginUpdate(nsIntRegion& aRegion);
+    virtual void GetUpdateRegion(nsIntRegion& aForRegion);
     virtual void EndUpdate();
-    virtual bool DirectUpdate(gfxASurface *aSurf, const nsIntRegion& aRegion);
-
+    virtual bool DirectUpdate(gfxASurface* aSurf, const nsIntRegion& aRegion, const nsIntPoint& aFrom = nsIntPoint(0,0));
+    virtual GLuint GetTextureID() { return mTexture; };
     // Returns a surface to draw into
     virtual already_AddRefed<gfxASurface>
       GetSurfaceForUpdate(const gfxIntSize& aSize, ImageFormat aFmt);
@@ -354,6 +383,7 @@ public:
     virtual void Resize(const nsIntSize& aSize);
 protected:
 
+    GLuint mTexture;
     TextureState mTextureState;
     GLContext* mGLContext;
     nsRefPtr<gfxASurface> mUpdateSurface;
@@ -363,6 +393,49 @@ protected:
     nsIntPoint mUpdateOffset;
 };
 
+/**
+ * A container class that complements many sub TextureImages into a big TextureImage.
+ * Aims to behave just like the real thing.
+ */
+
+class TiledTextureImage
+    : public TextureImage
+{
+public:
+    TiledTextureImage(GLContext* aGL, nsIntSize aSize,
+        TextureImage::ContentType, PRBool aUseNearestFilter = PR_FALSE);
+    ~TiledTextureImage();
+    void DumpDiv();
+    virtual gfxASurface* BeginUpdate(nsIntRegion& aRegion);
+    virtual void GetUpdateRegion(nsIntRegion& aForRegion);
+    virtual void EndUpdate();
+    virtual void Resize(const nsIntSize& aSize);
+    virtual PRUint32 GetTileCount();
+    virtual void BeginTileIteration();
+    virtual PRBool NextTile();
+    virtual nsIntRect GetTileRect();
+    virtual GLuint GetTextureID() {
+        return mImages[mCurrentImage]->GetTextureID();
+    };
+    virtual bool DirectUpdate(gfxASurface* aSurf, const nsIntRegion& aRegion, const nsIntPoint& aFrom = nsIntPoint(0,0));
+    virtual PRBool InUpdate() const { return mInUpdate; };
+    virtual void BindTexture(GLenum);
+protected:
+    unsigned int mCurrentImage;
+    nsTArray< nsRefPtr<TextureImage> > mImages;
+    bool mInUpdate;
+    nsIntSize mSize;
+    unsigned int mTileSize;
+    unsigned int mRows, mColumns;
+    GLContext* mGL;
+    PRBool mUseNearestFilter;
+    // A temporary surface to faciliate cross-tile updates.
+    nsRefPtr<gfxASurface> mUpdateSurface;
+    // The region of update requested
+    nsIntRegion mUpdateRegion;
+    TextureState mTextureState;
+};
+
 struct THEBES_API ContextFormat
 {
     static const ContextFormat BasicRGBA32Format;
@@ -496,6 +569,8 @@ public:
 
     virtual void WindowDestroyed() {}
 
+    virtual void ReleaseSurface() {}
+
     void *GetUserData(void *aKey) {
         void *result = nsnull;
         mUserData.Get(aKey, &result);
@@ -681,6 +756,15 @@ public:
         return mOffscreenTexture;
     }
 
+#if defined(MOZ_X11) && defined(MOZ_EGL_XRENDER_COMPOSITE)
+    virtual gfxASurface* GetOffscreenPixmapSurface()
+    {
+      return 0;
+    };
+    
+    virtual PRBool WaitNative() { return PR_FALSE; }
+#endif
+
     virtual PRBool TextureImageSupportsGetBackingSurface() {
         return PR_FALSE;
     }
@@ -706,6 +790,21 @@ public:
                        GLenum aWrapMode,
                        PRBool aUseNearestFilter=PR_FALSE);
 
+    /**
+     * In EGL we want to use Tiled Texture Images, which we return
+     * from CreateTextureImage above.
+     * Inside TiledTextureImage we need to create actual images and to
+     * prevent infinite recursion we need to differentiate the two
+     * functions.
+     **/
+    virtual already_AddRefed<TextureImage>
+    TileGenFunc(const nsIntSize& aSize,
+                TextureImage::ContentType aContentType,
+                PRBool aUseNearestFilter = PR_FALSE)
+    {
+        return nsnull;
+    };
+
     /**
      * Read the image data contained in aTexture, and return it as an ImageSurface.
      * If GL_RGBA is given as the format, a ImageFormatARGB32 surface is returned.
@@ -791,6 +890,19 @@ public:
                                              const nsIntPoint& aSrcPoint = nsIntPoint(0, 0),
                                              bool aPixelBuffer = PR_FALSE);
 
+    
+    void TexImage2D(GLenum target, GLint level, GLint internalformat, 
+                    GLsizei width, GLsizei height, GLsizei stride,
+                    GLint pixelsize, GLint border, GLenum format, 
+                    GLenum type, const GLvoid *pixels);
+
+
+    void TexSubImage2D(GLenum target, GLint level, 
+                       GLint xoffset, GLint yoffset, 
+                       GLsizei width, GLsizei height, GLsizei stride,
+                       GLint pixelsize, GLenum format, 
+                       GLenum type, const GLvoid* pixels);
+
     /** Helper for DecomposeIntoNoRepeatTriangles
      */
     struct RectTriangles {
@@ -866,6 +978,9 @@ public:
         ARB_texture_non_power_of_two,
         ARB_pixel_buffer_object,
         ARB_ES2_compatibility,
+        OES_texture_float,
+        ARB_texture_float,
+        EXT_unpack_subimage,
         Extensions_Max
     };
 
@@ -995,6 +1110,21 @@ protected:
     GLint mMaxRenderbufferSize;
 
 public:
+ 
+    /** \returns the first GL error, and guarantees that all GL error flags are cleared,
+      * i.e. that a subsequent GetError call will return NO_ERROR
+      */
+    GLenum GetAndClearError() {
+        // the first error is what we want to return
+        GLenum error = fGetError();
+        
+        if (error) {
+            // clear all pending errors
+            while(fGetError()) {}
+        }
+        
+        return error;
+    }
 
 #ifdef DEBUG
 
@@ -1012,6 +1142,7 @@ protected:
     GLenum mGLError;
 
 public:
+
     void BeforeGLCall(const char* glFunction) {
         if (mDebugMode) {
             // since the static member variable sCurrentGLContext is not thread-local as it should,
