@@ -64,14 +64,18 @@
 #include "jsstaticcheck.h"
 #include "jswrapper.h"
 
+#include "vm/GlobalObject.h"
+
+#include "jsinferinlines.h"
 #include "jsobjinlines.h"
 
 #include "vm/Stack-inl.h"
 
 using namespace js;
 using namespace js::gc;
+using namespace js::types;
 
-/* Forward declarations for js_ErrorClass's initializer. */
+/* Forward declarations for ErrorClass's initializer. */
 static JSBool
 Exception(JSContext *cx, uintN argc, Value *vp);
 
@@ -85,17 +89,17 @@ static JSBool
 exn_resolve(JSContext *cx, JSObject *obj, jsid id, uintN flags,
             JSObject **objp);
 
-Class js_ErrorClass = {
+Class js::ErrorClass = {
     js_Error_str,
     JSCLASS_HAS_PRIVATE | JSCLASS_NEW_RESOLVE |
     JSCLASS_HAS_CACHED_PROTO(JSProto_Error),
-    PropertyStub,         /* addProperty */
-    PropertyStub,         /* delProperty */
-    PropertyStub,         /* getProperty */
-    StrictPropertyStub,   /* setProperty */
-    EnumerateStub,
+    JS_PropertyStub,         /* addProperty */
+    JS_PropertyStub,         /* delProperty */
+    JS_PropertyStub,         /* getProperty */
+    JS_StrictPropertyStub,   /* setProperty */
+    JS_EnumerateStub,
     (JSResolveOp)exn_resolve,
-    ConvertStub,
+    JS_ConvertStub,
     exn_finalize,
     NULL,                 /* reserved0   */
     NULL,                 /* checkAccess */
@@ -120,6 +124,7 @@ typedef struct JSExnPrivate {
     JSString            *filename;
     uintN               lineno;
     size_t              stackDepth;
+    intN                exnType;
     JSStackTraceElem    stackElems[1];
 } JSExnPrivate;
 
@@ -258,112 +263,97 @@ GetStackTraceValueBuffer(JSExnPrivate *priv)
     return (jsval *)(priv->stackElems + priv->stackDepth);
 }
 
-static JSBool
-InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
-               JSString *filename, uintN lineno, JSErrorReport *report)
+struct SuppressErrorsGuard
 {
-    JSSecurityCallbacks *callbacks;
-    CheckAccessOp checkAccess;
-    JSErrorReporter older;
-    JSExceptionState *state;
-    jsid callerid;
-    size_t stackDepth, valueCount, size;
-    JSBool overflow;
-    JSExnPrivate *priv;
-    JSStackTraceElem *elem;
-    jsval *values;
+    JSContext *cx;
+    JSErrorReporter prevReporter;
+    JSExceptionState *prevState;
 
-    JS_ASSERT(exnObject->getClass() == &js_ErrorClass);
+    SuppressErrorsGuard(JSContext *cx)
+      : cx(cx),
+        prevReporter(JS_SetErrorReporter(cx, NULL)),
+        prevState(JS_SaveExceptionState(cx))
+    {}
 
-    /*
-     * Prepare stack trace data.
-     *
-     * Set aside any error reporter for cx and save its exception state
-     * so we can suppress any checkAccess failures.  Such failures should stop
-     * the backtrace procedure, not result in a failure of this constructor.
-     */
-    callbacks = JS_GetSecurityCallbacks(cx);
-    checkAccess = callbacks
-                  ? Valueify(callbacks->checkObjectAccess)
-                  : NULL;
-    older = JS_SetErrorReporter(cx, NULL);
-    state = JS_SaveExceptionState(cx);
+    ~SuppressErrorsGuard()
+    {
+        JS_RestoreExceptionState(cx, prevState);
+        JS_SetErrorReporter(cx, prevReporter);
+    }
+};
 
-    callerid = ATOM_TO_JSID(cx->runtime->atomState.callerAtom);
-    stackDepth = 0;
-    valueCount = 0;
+struct AppendArg {
+    Vector<Value> &values;
+    AppendArg(Vector<Value> &values) : values(values) {}
+    bool operator()(uintN, Value *vp) {
+        return values.append(*vp);
+    }
+};
 
-    FrameRegsIter firstPass(cx);
-    for (; !firstPass.done(); ++firstPass) {
-        StackFrame *fp = firstPass.fp();
-        if (fp->compartment() != cx->compartment)
-            break;
-        if (fp->isNonEvalFunctionFrame()) {
-            Value v = NullValue();
-            if (checkAccess &&
-                !checkAccess(cx, &fp->callee(), callerid, JSACC_READ, &v)) {
+static bool
+InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
+               JSString *filename, uintN lineno, JSErrorReport *report, intN exnType)
+{
+    JS_ASSERT(exnObject->isError());
+    JS_ASSERT(!exnObject->getPrivate());
+
+    JSSecurityCallbacks *callbacks = JS_GetSecurityCallbacks(cx);
+    JSCheckAccessOp checkAccess = callbacks ? callbacks->checkObjectAccess : NULL;
+
+    Vector<JSStackTraceElem> frames(cx);
+    Vector<Value> values(cx);
+    {
+        SuppressErrorsGuard seg(cx);
+        for (FrameRegsIter i(cx); !i.done(); ++i) {
+            /*
+             * An exception object stores stack values from 'fp' which may be
+             * in a different compartment from 'exnObject'. Engine compartment
+             * invariants require such values to be wrapped. A simpler solution
+             * is to just cut off the backtrace at compartment boundaries.
+             * Also, avoid exposing values from different security principals.
+             */
+            StackFrame *fp = i.fp();
+            if (fp->compartment() != cx->compartment)
                 break;
+            if (checkAccess && fp->isNonEvalFunctionFrame()) {
+                Value v = NullValue();
+                jsid callerid = ATOM_TO_JSID(cx->runtime->atomState.callerAtom);
+                if (!checkAccess(cx, &fp->callee(), callerid, JSACC_READ, &v))
+                    break;
             }
-            valueCount += fp->numActualArgs();
-        }
-        ++stackDepth;
-    }
-    JS_RestoreExceptionState(cx, state);
-    JS_SetErrorReporter(cx, older);
 
-    size = offsetof(JSExnPrivate, stackElems);
-    overflow = (stackDepth > ((size_t)-1 - size) / sizeof(JSStackTraceElem));
-    size += stackDepth * sizeof(JSStackTraceElem);
-    overflow |= (valueCount > ((size_t)-1 - size) / sizeof(jsval));
-    size += valueCount * sizeof(jsval);
-    if (overflow) {
-        js_ReportAllocationOverflow(cx);
-        return JS_FALSE;
+            if (!frames.growBy(1))
+                return false;
+            JSStackTraceElem &frame = frames.back();
+            if (fp->isNonEvalFunctionFrame()) {
+                frame.funName = fp->fun()->atom ? fp->fun()->atom : cx->runtime->emptyString;
+                frame.argc = fp->numActualArgs();
+                if (!fp->forEachCanonicalActualArg(AppendArg(values)))
+                    return false;
+            } else {
+                frame.funName = NULL;
+                frame.argc = 0;
+            }
+            if (fp->isScriptFrame()) {
+                frame.filename = fp->script()->filename;
+                frame.ulineno = js_FramePCToLineNumber(cx, fp, i.pc());
+            } else {
+                frame.ulineno = 0;
+                frame.filename = NULL;
+            }
+        }
     }
-    priv = (JSExnPrivate *)cx->malloc_(size);
+
+    /* Do not need overflow check: the vm stack is already bigger. */
+    JS_STATIC_ASSERT(sizeof(JSStackTraceElem) <= sizeof(StackFrame));
+
+    size_t nbytes = offsetof(JSExnPrivate, stackElems) +
+                    frames.length() * sizeof(JSStackTraceElem) +
+                    values.length() * sizeof(Value);
+
+    JSExnPrivate *priv = (JSExnPrivate *)cx->malloc_(nbytes);
     if (!priv)
-        return JS_FALSE;
-
-    /*
-     * We initialize errorReport with a copy of report after setting the
-     * private slot, to prevent GC accessing a junk value we clear the field
-     * here.
-     */
-    priv->errorReport = NULL;
-    priv->message = message;
-    priv->filename = filename;
-    priv->lineno = lineno;
-    priv->stackDepth = stackDepth;
-
-    values = GetStackTraceValueBuffer(priv);
-    elem = priv->stackElems;
-    for (FrameRegsIter iter(cx); iter != firstPass; ++iter) {
-        StackFrame *fp = iter.fp();
-        if (fp->compartment() != cx->compartment)
-            break;
-        if (!fp->isNonEvalFunctionFrame()) {
-            elem->funName = NULL;
-            elem->argc = 0;
-        } else {
-            elem->funName = fp->fun()->atom
-                            ? fp->fun()->atom
-                            : cx->runtime->emptyString;
-            elem->argc = fp->numActualArgs();
-            fp->forEachCanonicalActualArg(CopyTo(Valueify(values)));
-            values += elem->argc;
-        }
-        elem->ulineno = 0;
-        elem->filename = NULL;
-        if (fp->isScriptFrame()) {
-            elem->filename = fp->script()->filename;
-            elem->ulineno = js_FramePCToLineNumber(cx, fp, iter.pc());
-        }
-        ++elem;
-    }
-    JS_ASSERT(priv->stackElems + stackDepth == elem);
-    JS_ASSERT(GetStackTraceValueBuffer(priv) + valueCount == values);
-
-    exnObject->setPrivate(priv);
+        return false;
 
     if (report) {
         /*
@@ -374,17 +364,34 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
          */
         priv->errorReport = CopyErrorReport(cx, report);
         if (!priv->errorReport) {
-            /* The finalizer realeases priv since it is in the private slot. */
-            return JS_FALSE;
+            cx->free_(priv);
+            return false;
         }
+    } else {
+        priv->errorReport = NULL;
     }
 
-    return JS_TRUE;
+    priv->message = message;
+    priv->filename = filename;
+    priv->lineno = lineno;
+    priv->stackDepth = frames.length();
+    priv->exnType = exnType;
+
+    JSStackTraceElem *framesDest = priv->stackElems;
+    Value *valuesDest = reinterpret_cast<Value *>(framesDest + frames.length());
+    JS_ASSERT(valuesDest == GetStackTraceValueBuffer(priv));
+
+    PodCopy(framesDest, frames.begin(), frames.length());
+    PodCopy(valuesDest, values.begin(), values.length());
+
+    exnObject->setPrivate(priv);
+    return true;
 }
 
 static inline JSExnPrivate *
-GetExnPrivate(JSContext *cx, JSObject *obj)
+GetExnPrivate(JSObject *obj)
 {
+    JS_ASSERT(obj->isError());
     return (JSExnPrivate *) obj->getPrivate();
 }
 
@@ -396,7 +403,7 @@ exn_trace(JSTracer *trc, JSObject *obj)
     size_t vcount, i;
     jsval *vp, v;
 
-    priv = GetExnPrivate(trc->context, obj);
+    priv = GetExnPrivate(obj);
     if (priv) {
         if (priv->message)
             MarkString(trc, priv->message, "exception message");
@@ -422,12 +429,9 @@ exn_trace(JSTracer *trc, JSObject *obj)
 static void
 exn_finalize(JSContext *cx, JSObject *obj)
 {
-    JSExnPrivate *priv;
-
-    priv = GetExnPrivate(cx, obj);
-    if (priv) {
-        if (priv->errorReport)
-            cx->free_(priv->errorReport);
+    if (JSExnPrivate *priv = GetExnPrivate(obj)) {
+        if (JSErrorReport *report = priv->errorReport)
+            cx->free_(report);
         cx->free_(priv);
     }
 }
@@ -445,7 +449,7 @@ exn_resolve(JSContext *cx, JSObject *obj, jsid id, uintN flags,
     uintN attrs;
 
     *objp = NULL;
-    priv = GetExnPrivate(cx, obj);
+    priv = GetExnPrivate(obj);
     if (priv && JSID_IS_ATOM(id)) {
         str = JSID_TO_STRING(id);
 
@@ -514,33 +518,34 @@ js_ErrorFromException(JSContext *cx, jsval exn)
     if (JSVAL_IS_PRIMITIVE(exn))
         return NULL;
     obj = JSVAL_TO_OBJECT(exn);
-    if (obj->getClass() != &js_ErrorClass)
+    if (!obj->isError())
         return NULL;
-    priv = GetExnPrivate(cx, obj);
+    priv = GetExnPrivate(obj);
     if (!priv)
         return NULL;
     return priv->errorReport;
 }
 
 static JSString *
-ValueToShortSource(JSContext *cx, jsval v)
+ValueToShortSource(JSContext *cx, const Value &v)
 {
     JSString *str;
 
     /* Avoid toSource bloat and fallibility for object types. */
-    if (JSVAL_IS_PRIMITIVE(v))
-        return js_ValueToSource(cx, Valueify(v));
+    if (!v.isObject())
+        return js_ValueToSource(cx, v);
 
-    AutoCompartment ac(cx, JSVAL_TO_OBJECT(v));
+    JSObject *obj = &v.toObject();
+    AutoCompartment ac(cx, obj);
     if (!ac.enter())
         return NULL;
 
-    if (VALUE_IS_FUNCTION(cx, v)) {
+    if (obj->isFunction()) {
         /*
          * XXX Avoid function decompilation bloat for now.
          */
-        str = JS_GetFunctionId(JS_ValueToFunction(cx, v));
-        if (!str && !(str = js_ValueToSource(cx, Valueify(v)))) {
+        str = JS_GetFunctionId(obj->getFunctionPrivate());
+        if (!str && !(str = js_ValueToSource(cx, v))) {
             /*
              * Continue to soldier on if the function couldn't be
              * converted into a string.
@@ -554,8 +559,7 @@ ValueToShortSource(JSContext *cx, jsval v)
          * memory, for too many classes (see Mozilla bug 166743).
          */
         char buf[100];
-        JS_snprintf(buf, sizeof buf, "[object %s]",
-                    JSVAL_TO_OBJECT(v)->getClass()->name);
+        JS_snprintf(buf, sizeof buf, "[object %s]", obj->getClass()->name);
         str = JS_NewStringCopyZ(cx, buf);
     }
 
@@ -690,9 +694,15 @@ FilenameToString(JSContext *cx, const char *filename)
     return JS_NewStringCopyZ(cx, filename);
 }
 
+enum {
+    JSSLOT_ERROR_EXNTYPE = 0
+};
+
 static JSBool
 Exception(JSContext *cx, uintN argc, Value *vp)
 {
+    CallArgs args = CallArgsFromVp(argc, vp);
+
     /*
      * ECMA ed. 3, 15.11.1 requires Error, etc., to construct even when
      * called as functions, without operator new.  But as we do not give
@@ -700,36 +710,27 @@ Exception(JSContext *cx, uintN argc, Value *vp)
      * NewNativeClassInstance to find the class prototype, we must get the
      * class prototype ourselves.
      */
-    JSObject &callee = vp[0].toObject();
     Value protov;
-    if (!callee.getProperty(cx, ATOM_TO_JSID(cx->runtime->atomState.classPrototypeAtom), &protov))
-        return JS_FALSE;
+    if (!args.callee().getProperty(cx, cx->runtime->atomState.classPrototypeAtom, &protov))
+        return false;
 
     if (!protov.isObject()) {
         JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, JSMSG_BAD_PROTOTYPE, "Error");
-        return JS_FALSE;
+        return false;
     }
 
     JSObject *errProto = &protov.toObject();
-    JSObject *obj = NewNativeClassInstance(cx, &js_ErrorClass, errProto, errProto->getParent());
+    JSObject *obj = NewNativeClassInstance(cx, &ErrorClass, errProto, errProto->getParent());
     if (!obj)
-        return JS_FALSE;
-
-    /*
-     * If it's a new object of class Exception, then null out the private
-     * data so that the finalizer doesn't attempt to free it.
-     */
-    if (obj->getClass() == &js_ErrorClass)
-        obj->setPrivate(NULL);
+        return false;
 
     /* Set the 'message' property. */
-    Value *argv = vp + 2;
     JSString *message;
-    if (argc != 0 && !argv[0].isUndefined()) {
-        message = js_ValueToString(cx, argv[0]);
+    if (args.argc() != 0 && !args[0].isUndefined()) {
+        message = js_ValueToString(cx, args[0]);
         if (!message)
-            return JS_FALSE;
-        argv[0].setString(message);
+            return false;
+        args[0].setString(message);
     } else {
         message = NULL;
     }
@@ -741,16 +742,16 @@ Exception(JSContext *cx, uintN argc, Value *vp)
 
     /* Set the 'fileName' property. */
     JSString *filename;
-    if (argc > 1) {
-        filename = js_ValueToString(cx, argv[1]);
+    if (args.argc() > 1) {
+        filename = js_ValueToString(cx, args[1]);
         if (!filename)
-            return JS_FALSE;
-        argv[1].setString(filename);
+            return false;
+        args[1].setString(filename);
     } else {
         if (!iter.done()) {
             filename = FilenameToString(cx, iter.fp()->script()->filename);
             if (!filename)
-                return JS_FALSE;
+                return false;
         } else {
             filename = cx->runtime->emptyString;
         }
@@ -758,20 +759,19 @@ Exception(JSContext *cx, uintN argc, Value *vp)
 
     /* Set the 'lineNumber' property. */
     uint32_t lineno;
-    if (argc > 2) {
-        if (!ValueToECMAUint32(cx, argv[2], &lineno))
-            return JS_FALSE;
+    if (args.argc() > 2) {
+        if (!ValueToECMAUint32(cx, args[2], &lineno))
+            return false;
     } else {
         lineno = iter.done() ? 0 : js_FramePCToLineNumber(cx, iter.fp(), iter.pc());
     }
 
-    if (obj->getClass() == &js_ErrorClass &&
-        !InitExnPrivate(cx, obj, message, filename, lineno, NULL)) {
-        return JS_FALSE;
-    }
+    intN exnType = args.callee().getReservedSlot(JSSLOT_ERROR_EXNTYPE).toInt32();
+    if (!InitExnPrivate(cx, obj, message, filename, lineno, NULL, exnType))
+        return false;
 
-    vp->setObject(*obj);
-    return JS_TRUE;
+    args.rval().setObject(*obj);
+    return true;
 }
 
 /*
@@ -792,7 +792,7 @@ exn_toString(JSContext *cx, uintN argc, Value *vp)
     JSObject *obj = ToObject(cx, &vp[1]);
     if (!obj)
         return JS_FALSE;
-    if (!obj->getProperty(cx, ATOM_TO_JSID(cx->runtime->atomState.nameAtom), Valueify(&v)))
+    if (!obj->getProperty(cx, cx->runtime->atomState.nameAtom, &v))
         return JS_FALSE;
     name = JSVAL_IS_STRING(v) ? JSVAL_TO_STRING(v) : cx->runtime->emptyString;
     vp->setString(name);
@@ -853,7 +853,7 @@ exn_toSource(JSContext *cx, uintN argc, Value *vp)
     JSObject *obj = ToObject(cx, &vp[1]);
     if (!obj)
         return false;
-    if (!obj->getProperty(cx, ATOM_TO_JSID(cx->runtime->atomState.nameAtom), vp))
+    if (!obj->getProperty(cx, cx->runtime->atomState.nameAtom, vp))
         return false;
     name = js_ValueToString(cx, *vp);
     if (!name)
@@ -861,19 +861,19 @@ exn_toSource(JSContext *cx, uintN argc, Value *vp)
     vp->setString(name);
 
     {
-        AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(localroots), Valueify(localroots));
+        AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(localroots), localroots);
 
 #ifdef __GNUC__
         message = filename = NULL;
 #endif
         if (!JS_GetProperty(cx, obj, js_message_str, &localroots[0]) ||
-            !(message = js_ValueToSource(cx, Valueify(localroots[0])))) {
+            !(message = js_ValueToSource(cx, localroots[0]))) {
             return false;
         }
         localroots[0] = STRING_TO_JSVAL(message);
 
         if (!JS_GetProperty(cx, obj, js_fileName_str, &localroots[1]) ||
-            !(filename = js_ValueToSource(cx, Valueify(localroots[1])))) {
+            !(filename = js_ValueToSource(cx, localroots[1]))) {
             return false;
         }
         localroots[1] = STRING_TO_JSVAL(filename);
@@ -881,11 +881,11 @@ exn_toSource(JSContext *cx, uintN argc, Value *vp)
         if (!JS_GetProperty(cx, obj, js_lineNumber_str, &localroots[2]))
             return false;
         uint32_t lineno;
-        if (!ValueToECMAUint32(cx, Valueify(localroots[2]), &lineno))
+        if (!ValueToECMAUint32(cx, localroots[2], &lineno))
             return false;
 
         if (lineno != 0) {
-            lineno_as_str = js_ValueToString(cx, Valueify(localroots[2]));
+            lineno_as_str = js_ValueToString(cx, localroots[2]);
             if (!lineno_as_str)
                 return false;
             lineno_length = lineno_as_str->length();
@@ -999,64 +999,80 @@ GetExceptionProtoKey(intN exn)
 {
     JS_ASSERT(JSEXN_ERR <= exn);
     JS_ASSERT(exn < JSEXN_LIMIT);
-    return (JSProtoKey) (JSProto_Error + exn);
+    return JSProtoKey(JSProto_Error + exn);
 }
 
-JSObject *
-js_InitExceptionClasses(JSContext *cx, JSObject *obj)
+static JSObject *
+InitErrorClass(JSContext *cx, GlobalObject *global, intN type, JSObject &proto)
 {
-    /*
-     * If lazy class initialization occurs for any Error subclass, then all
-     * classes are initialized, starting with Error.  To avoid reentry and
-     * redundant initialization, we must not pass a null proto parameter to
-     * NewNonFunction below, when called for the Error superclass.  We need to
-     * ensure that Object.prototype is the proto of Error.prototype.
-     *
-     * See the equivalent code to ensure that parent_proto is non-null when
-     * js_InitClass calls NewObject, in jsobj.cpp.
-     */
-    JSObject *obj_proto;
-    if (!js_GetClassPrototype(cx, obj, JSProto_Object, &obj_proto))
+    JSProtoKey key = GetExceptionProtoKey(type);
+    JSAtom *name = cx->runtime->atomState.classAtoms[key];
+    JSObject *errorProto = global->createBlankPrototypeInheriting(cx, &ErrorClass, proto);
+    if (!errorProto)
         return NULL;
 
-    /* Define all error constructors. */
     Value empty = StringValue(cx->runtime->emptyString);
     jsid nameId = ATOM_TO_JSID(cx->runtime->atomState.nameAtom);
     jsid messageId = ATOM_TO_JSID(cx->runtime->atomState.messageAtom);
     jsid fileNameId = ATOM_TO_JSID(cx->runtime->atomState.fileNameAtom);
     jsid lineNumberId = ATOM_TO_JSID(cx->runtime->atomState.lineNumberAtom);
-    JSObject *error_proto = NULL;
-    for (intN i = JSEXN_ERR; i != JSEXN_LIMIT; i++) {
-        JSProtoKey protoKey = GetExceptionProtoKey(i);
-        JSAtom *atom = cx->runtime->atomState.classAtoms[protoKey];
-        JSObject *proto =
-            DefineConstructorAndPrototype(cx, obj, protoKey, atom,
-                                          (i == JSEXN_ERR) ? obj_proto : error_proto,
-                                          &js_ErrorClass, Exception, 1,
-                                          NULL, (i == JSEXN_ERR) ? exception_methods : NULL,
-                                          NULL, NULL);
-        if (!proto)
-            return NULL;
-        JS_ASSERT(proto->privateData == NULL);
-
-        if (i == JSEXN_ERR)
-            error_proto = proto;
-
-        /* Add properties to the prototype. */
-        JSAutoResolveFlags rf(cx, JSRESOLVE_QUALIFIED | JSRESOLVE_DECLARING);
-        if (!DefineNativeProperty(cx, proto, nameId, StringValue(atom),
-                                  PropertyStub, StrictPropertyStub, 0, 0, 0) ||
-            !DefineNativeProperty(cx, proto, messageId, empty,
-                                  PropertyStub, StrictPropertyStub, 0, 0, 0) ||
-            !DefineNativeProperty(cx, proto, fileNameId, empty,
-                                  PropertyStub, StrictPropertyStub, JSPROP_ENUMERATE, 0, 0) ||
-            !DefineNativeProperty(cx, proto, lineNumberId, Valueify(JSVAL_ZERO),
-                                  PropertyStub, StrictPropertyStub, JSPROP_ENUMERATE, 0, 0)) {
-            return NULL;
-        }
+    if (!DefineNativeProperty(cx, errorProto, nameId, StringValue(name),
+                              JS_PropertyStub, JS_StrictPropertyStub, 0, 0, 0) ||
+        !DefineNativeProperty(cx, errorProto, messageId, empty,
+                              JS_PropertyStub, JS_StrictPropertyStub, 0, 0, 0) ||
+        !DefineNativeProperty(cx, errorProto, fileNameId, empty,
+                              JS_PropertyStub, JS_StrictPropertyStub, JSPROP_ENUMERATE, 0, 0) ||
+        !DefineNativeProperty(cx, errorProto, lineNumberId, Int32Value(0),
+                              JS_PropertyStub, JS_StrictPropertyStub, JSPROP_ENUMERATE, 0, 0))
+    {
+        return NULL;
     }
 
-    return error_proto;
+    /* Create the corresponding constructor. */
+    JSFunction *ctor = global->createConstructor(cx, Exception, &ErrorClass, name, 1);
+    if (!ctor)
+        return NULL;
+    ctor->setReservedSlot(JSSLOT_ERROR_EXNTYPE, Int32Value(int32(type)));
+
+    if (!LinkConstructorAndPrototype(cx, ctor, errorProto))
+        return NULL;
+
+    if (!DefineConstructorAndPrototype(cx, global, key, ctor, errorProto))
+        return NULL;
+
+    JS_ASSERT(!errorProto->getPrivate());
+
+    return errorProto;
+}
+
+JSObject *
+js_InitExceptionClasses(JSContext *cx, JSObject *obj)
+{
+    JS_ASSERT(obj->isGlobal());
+    JS_ASSERT(obj->isNative());
+
+    GlobalObject *global = obj->asGlobal();
+
+    JSObject *objectProto;
+    if (!js_GetClassPrototype(cx, global, JSProto_Object, &objectProto))
+        return NULL;
+
+    /* Initialize the base Error class first. */
+    JSObject *errorProto = InitErrorClass(cx, global, JSEXN_ERR, *objectProto);
+    if (!errorProto)
+        return NULL;
+
+    /* |Error.prototype| alone has method properties. */
+    if (!DefinePropertiesAndBrand(cx, errorProto, NULL, exception_methods))
+        return NULL;
+
+    /* Define all remaining *Error constructors. */
+    for (intN i = JSEXN_ERR + 1; i < JSEXN_LIMIT; i++) {
+        if (!InitErrorClass(cx, global, i, *errorProto))
+            return NULL;
+    }
+
+    return errorProto;
 }
 
 const JSErrorFormatString*
@@ -1140,7 +1156,7 @@ js_ErrorToException(JSContext *cx, const char *message, JSErrorReport *reportp,
 
     /* Protect the newly-created strings below from nesting GCs. */
     PodArrayZero(tv);
-    AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(tv), Valueify(tv));
+    AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(tv), tv);
 
     /*
      * Try to get an appropriate prototype by looking up the corresponding
@@ -1152,7 +1168,7 @@ js_ErrorToException(JSContext *cx, const char *message, JSErrorReport *reportp,
         goto out;
     tv[0] = OBJECT_TO_JSVAL(errProto);
 
-    errObject = NewNativeClassInstance(cx, &js_ErrorClass, errProto, errProto->getParent());
+    errObject = NewNativeClassInstance(cx, &ErrorClass, errProto, errProto->getParent());
     if (!errObject) {
         ok = JS_FALSE;
         goto out;
@@ -1174,7 +1190,7 @@ js_ErrorToException(JSContext *cx, const char *message, JSErrorReport *reportp,
     tv[3] = STRING_TO_JSVAL(filenameStr);
 
     ok = InitExnPrivate(cx, errObject, messageStr, filenameStr,
-                        reportp->lineno, reportp);
+                        reportp->lineno, reportp, exn);
     if (!ok)
         goto out;
 
@@ -1205,7 +1221,7 @@ js_ReportUncaughtException(JSContext *cx)
         return false;
 
     PodArrayZero(roots);
-    AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(roots), Valueify(roots));
+    AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(roots), roots);
 
     /*
      * Because js_ValueToString below could error and an exception object
@@ -1224,7 +1240,7 @@ js_ReportUncaughtException(JSContext *cx)
     reportp = js_ErrorFromException(cx, exn);
 
     /* XXX L10N angels cry once again (see also jsemit.c, /L10N gaffes/) */
-    str = js_ValueToString(cx, Valueify(exn));
+    str = js_ValueToString(cx, exn);
     JSAutoByteString bytesStorage;
     if (!str) {
         bytes = "unknown (can't convert to string)";
@@ -1236,7 +1252,7 @@ js_ReportUncaughtException(JSContext *cx)
     }
 
     JSAutoByteString filename;
-    if (!reportp && exnObject && exnObject->getClass() == &js_ErrorClass) {
+    if (!reportp && exnObject && exnObject->isError()) {
         if (!JS_GetProperty(cx, exnObject, js_message_str, &roots[2]))
             return false;
         if (JSVAL_IS_STRING(roots[2])) {
@@ -1248,14 +1264,14 @@ js_ReportUncaughtException(JSContext *cx)
 
         if (!JS_GetProperty(cx, exnObject, js_fileName_str, &roots[3]))
             return false;
-        str = js_ValueToString(cx, Valueify(roots[3]));
+        str = js_ValueToString(cx, roots[3]);
         if (!str || !filename.encode(cx, str))
             return false;
 
         if (!JS_GetProperty(cx, exnObject, js_lineNumber_str, &roots[4]))
             return false;
         uint32_t lineno;
-        if (!ValueToECMAUint32(cx, Valueify(roots[4]), &lineno))
+        if (!ValueToECMAUint32(cx, roots[4], &lineno))
             return false;
 
         reportp = &report;
@@ -1285,3 +1301,63 @@ js_ReportUncaughtException(JSContext *cx)
 
     return true;
 }
+
+extern JSObject *
+js_CopyErrorObject(JSContext *cx, JSObject *errobj, JSObject *scope)
+{
+    assertSameCompartment(cx, scope);
+    JSExnPrivate *priv = GetExnPrivate(errobj);
+
+    uint32 stackDepth = priv->stackDepth;
+    size_t valueCount = 0;
+    for (uint32 i = 0; i < stackDepth; i++)
+        valueCount += priv->stackElems[i].argc;
+
+    size_t size = offsetof(JSExnPrivate, stackElems) +
+                  stackDepth * sizeof(JSStackTraceElem) +
+                  valueCount * sizeof(jsval);
+
+    JSExnPrivate *copy = (JSExnPrivate *)cx->malloc_(size);
+    if (!copy)
+        return NULL;
+
+    struct AutoFree {
+        JSContext *cx;
+        JSExnPrivate *p;
+        ~AutoFree() {
+            if (p) {
+                cx->free_(p->errorReport);
+                cx->free_(p);
+            }
+        }
+    } autoFree = {cx, copy};
+
+    // Copy each field. Don't bother copying the stack elements.
+    if (priv->errorReport) {
+        copy->errorReport = CopyErrorReport(cx, priv->errorReport);
+        if (!copy->errorReport)
+            return NULL;
+    } else {
+        copy->errorReport = NULL;
+    }
+    copy->message = priv->message;
+    if (!cx->compartment->wrap(cx, &copy->message))
+        return NULL;
+    JS::Anchor<JSString *> messageAnchor(copy->message);
+    copy->filename = priv->filename;
+    if (!cx->compartment->wrap(cx, &copy->filename))
+        return NULL;
+    JS::Anchor<JSString *> filenameAnchor(copy->filename);
+    copy->lineno = priv->lineno;
+    copy->stackDepth = 0;
+    copy->exnType = priv->exnType;
+
+    // Create the Error object.
+    JSObject *proto;
+    if (!js_GetClassPrototype(cx, scope->getGlobal(), GetExceptionProtoKey(copy->exnType), &proto))
+        return NULL;
+    JSObject *copyobj = NewNativeClassInstance(cx, &ErrorClass, proto, proto->getParent());
+    copyobj->setPrivate(copy);
+    autoFree.p = NULL;
+    return copyobj;
+}
