@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  *
  * ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
@@ -45,6 +45,9 @@
 #include "jsobjinlines.h"
 #include "jsscopeinlines.h"
 
+#include "vm/String-inl.h"
+#include "methodjit/MethodJIT.h"
+
 /*
  * There are two mostly separate mark paths. The first is a fast path used
  * internally in the GC. The second is a slow path used for root marking and
@@ -63,7 +66,7 @@
  * MarkString, etc. These functions check if an object is in the compartment
  * currently being GCed. If it is, they call PushMarkStack. Roots are pushed
  * this way as well as pointers traversed inside trace hooks (for things like
- * js_IteratorClass). It it always valid to call a MarkX function instead of
+ * IteratorClass). It it always valid to call a MarkX function instead of
  * PushMarkStack, although it may be slower.
  *
  * The MarkX functions also handle non-GC object traversal. In this case, they
@@ -89,6 +92,9 @@ PushMarkStack(GCMarker *gcmarker, JSObject *thing);
 static inline void
 PushMarkStack(GCMarker *gcmarker, JSFunction *thing);
 
+static inline void
+PushMarkStack(GCMarker *gcmarker, JSScript *thing);
+
 static inline void
 PushMarkStack(GCMarker *gcmarker, const Shape *thing);
 
@@ -98,20 +104,34 @@ PushMarkStack(GCMarker *gcmarker, JSShortString *thing);
 static inline void
 PushMarkStack(GCMarker *gcmarker, JSString *thing);
 
+static inline void
+PushMarkStack(GCMarker *gcmarker, types::TypeObject *thing);
+
+template<typename T>
+static inline void
+CheckMarkedThing(JSTracer *trc, T *thing)
+{
+    JS_ASSERT(thing);
+    JS_ASSERT(trc->debugPrinter || trc->debugPrintArg);
+    JS_ASSERT_IF(trc->context->runtime->gcCurrentCompartment, IS_GC_MARKING_TRACER(trc));
+
+    JS_ASSERT(thing->isAligned());
+
+    JS_ASSERT(thing->compartment());
+    JS_ASSERT(thing->compartment()->rt == trc->context->runtime);
+}
+
 template<typename T>
 void
 Mark(JSTracer *trc, T *thing)
 {
-    JS_ASSERT(thing);
-    JS_ASSERT(JS_IS_VALID_TRACE_KIND(GetGCThingTraceKind(thing)));
-    JS_ASSERT(trc->debugPrinter || trc->debugPrintArg);
-
-    JS_ASSERT(!JSAtom::isStatic(thing));
-    JS_ASSERT(thing->asFreeCell()->isAligned());
+    CheckMarkedThing(trc, thing);
 
     JSRuntime *rt = trc->context->runtime;
-    JS_ASSERT(thing->arenaHeader()->compartment);
-    JS_ASSERT(thing->arenaHeader()->compartment->rt == rt);
+
+    JS_OPT_ASSERT_IF(rt->gcCheckCompartment,
+                     thing->compartment() == rt->gcCheckCompartment ||
+                     thing->compartment() == rt->atomsCompartment);
 
     /*
      * Don't mark things outside a compartment if we are in a per-compartment
@@ -134,8 +154,6 @@ void
 MarkString(JSTracer *trc, JSString *str)
 {
     JS_ASSERT(str);
-    if (str->isStaticAtom())
-        return;
     Mark(trc, str);
 }
 
@@ -156,6 +174,16 @@ MarkObject(JSTracer *trc, JSObject &obj, const char *name)
     Mark(trc, &obj);
 }
 
+void
+MarkCrossCompartmentObject(JSTracer *trc, JSObject &obj, const char *name)
+{
+    JSRuntime *rt = trc->context->runtime;
+    if (rt->gcCurrentCompartment && rt->gcCurrentCompartment != obj.compartment())
+        return;
+
+    MarkObject(trc, obj, name);
+}
+
 void
 MarkObjectWithPrinter(JSTracer *trc, JSObject &obj, JSTraceNamePrinter printer,
 		      const void *arg, size_t index)
@@ -166,6 +194,15 @@ MarkObjectWithPrinter(JSTracer *trc, JSObject &obj, JSTraceNamePrinter printer,
     Mark(trc, &obj);
 }
 
+void
+MarkScript(JSTracer *trc, JSScript *script, const char *name)
+{
+    JS_ASSERT(trc);
+    JS_ASSERT(script);
+    JS_SET_TRACING_NAME(trc, name);
+    Mark(trc, script);
+}
+
 void
 MarkShape(JSTracer *trc, const Shape *shape, const char *name)
 {
@@ -175,6 +212,30 @@ MarkShape(JSTracer *trc, const Shape *shape, const char *name)
     Mark(trc, shape);
 }
 
+void
+MarkTypeObject(JSTracer *trc, types::TypeObject *type, const char *name)
+{
+    JS_ASSERT(trc);
+    JS_ASSERT(type);
+    JS_SET_TRACING_NAME(trc, name);
+    if (type == &types::emptyTypeObject)
+        return;
+    Mark(trc, type);
+
+    /*
+     * Mark parts of a type object skipped by ScanTypeObject. ScanTypeObject is
+     * only used for marking tracers; for tracers with a callback, if we
+     * reenter through JS_TraceChildren then MarkChildren will *not* skip these
+     * members, and we don't need to handle them here.
+     */
+    if (IS_GC_MARKING_TRACER(trc)) {
+        if (type->singleton)
+            MarkObject(trc, *type->singleton, "type_singleton");
+        if (type->interpretedFunction)
+            MarkObject(trc, *type->interpretedFunction, "type_function");
+    }
+}
+
 #if JS_HAS_XML_SUPPORT
 void
 MarkXML(JSTracer *trc, JSXML *xml, const char *name)
@@ -189,8 +250,8 @@ MarkXML(JSTracer *trc, JSXML *xml, const char *name)
 void
 PushMarkStack(GCMarker *gcmarker, JSXML *thing)
 {
-    JS_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
-                 thing->compartment() == gcmarker->context->runtime->gcCurrentCompartment);
+    JS_OPT_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
+                     thing->compartment() == gcmarker->context->runtime->gcCurrentCompartment);
 
     if (thing->markIfUnmarked(gcmarker->getMarkColor()))
         gcmarker->pushXML(thing);
@@ -199,8 +260,8 @@ PushMarkStack(GCMarker *gcmarker, JSXML *thing)
 void
 PushMarkStack(GCMarker *gcmarker, JSObject *thing)
 {
-    JS_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
-                 thing->compartment() == gcmarker->context->runtime->gcCurrentCompartment);
+    JS_OPT_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
+                     thing->compartment() == gcmarker->context->runtime->gcCurrentCompartment);
 
     if (thing->markIfUnmarked(gcmarker->getMarkColor()))
         gcmarker->pushObject(thing);
@@ -209,44 +270,68 @@ PushMarkStack(GCMarker *gcmarker, JSObject *thing)
 void
 PushMarkStack(GCMarker *gcmarker, JSFunction *thing)
 {
-    JS_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
-                 thing->compartment() == gcmarker->context->runtime->gcCurrentCompartment);
+    JS_OPT_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
+                     thing->compartment() == gcmarker->context->runtime->gcCurrentCompartment);
 
     if (thing->markIfUnmarked(gcmarker->getMarkColor()))
         gcmarker->pushObject(thing);
 }
 
 void
-PushMarkStack(GCMarker *gcmarker, JSShortString *thing)
+PushMarkStack(GCMarker *gcmarker, types::TypeObject *thing)
 {
     JS_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
                  thing->compartment() == gcmarker->context->runtime->gcCurrentCompartment);
 
+    if (thing->markIfUnmarked(gcmarker->getMarkColor()))
+        gcmarker->pushType(thing);
+}
+
+void
+PushMarkStack(GCMarker *gcmarker, JSShortString *thing)
+{
+    JS_OPT_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
+                     thing->compartment() == gcmarker->context->runtime->gcCurrentCompartment);
+
     (void) thing->markIfUnmarked(gcmarker->getMarkColor());
 }
 
+void
+PushMarkStack(GCMarker *gcmarker, JSScript *thing)
+{
+    JS_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
+                 thing->compartment() == gcmarker->context->runtime->gcCurrentCompartment);
+
+    /*
+     * We mark scripts directly rather than pushing on the stack as they can
+     * refer to other scripts only indirectly (like via nested functions) and
+     * we cannot get to deep recursion.
+     */
+    if (thing->markIfUnmarked(gcmarker->getMarkColor()))
+        MarkChildren(gcmarker, thing);
+}
+
 static void
 ScanShape(GCMarker *gcmarker, const Shape *shape);
 
 void
 PushMarkStack(GCMarker *gcmarker, const Shape *thing)
 {
-    JS_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
-                 thing->compartment() == gcmarker->context->runtime->gcCurrentCompartment);
+    JS_OPT_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
+                     thing->compartment() == gcmarker->context->runtime->gcCurrentCompartment);
 
     /* We mark shapes directly rather than pushing on the stack. */
     if (thing->markIfUnmarked(gcmarker->getMarkColor()))
         ScanShape(gcmarker, thing);
 }
 
-void
+static void
 MarkAtomRange(JSTracer *trc, size_t len, JSAtom **vec, const char *name)
 {
     for (uint32 i = 0; i < len; i++) {
         if (JSAtom *atom = vec[i]) {
             JS_SET_TRACING_INDEX(trc, name, i);
-            if (!atom->isStaticAtom())
-                Mark(trc, atom);
+            Mark(trc, atom);
         }
     }
 }
@@ -276,13 +361,10 @@ MarkXMLRange(JSTracer *trc, size_t len, JSXML **vec, const char *name)
 void
 MarkId(JSTracer *trc, jsid id)
 {
-    if (JSID_IS_STRING(id)) {
-        JSString *str = JSID_TO_STRING(id);
-        if (!str->isStaticAtom())
-            Mark(trc, str);
-    } else if (JS_UNLIKELY(JSID_IS_OBJECT(id))) {
+    if (JSID_IS_STRING(id))
+        Mark(trc, JSID_TO_STRING(id));
+    else if (JS_UNLIKELY(JSID_IS_OBJECT(id)))
         Mark(trc, JSID_TO_OBJECT(id));
-    }
 }
 
 void
@@ -308,27 +390,31 @@ MarkIdRange(JSTracer *trc, size_t len, jsid *vec, const char *name)
 }
 
 void
-MarkKind(JSTracer *trc, void *thing, uint32 kind)
+MarkKind(JSTracer *trc, void *thing, JSGCTraceKind kind)
 {
     JS_ASSERT(thing);
     JS_ASSERT(kind == GetGCThingTraceKind(thing));
     switch (kind) {
-        case JSTRACE_OBJECT:
-            Mark(trc, reinterpret_cast<JSObject *>(thing));
-            break;
-        case JSTRACE_STRING:
-            MarkString(trc, reinterpret_cast<JSString *>(thing));
-            break;
-        case JSTRACE_SHAPE:
-            Mark(trc, reinterpret_cast<Shape *>(thing));
-            break;
+      case JSTRACE_OBJECT:
+        Mark(trc, reinterpret_cast<JSObject *>(thing));
+        break;
+      case JSTRACE_STRING:
+        MarkString(trc, reinterpret_cast<JSString *>(thing));
+        break;
+      case JSTRACE_SCRIPT:
+        Mark(trc, static_cast<JSScript *>(thing));
+        break;
+      case JSTRACE_SHAPE:
+        Mark(trc, reinterpret_cast<Shape *>(thing));
+        break;
+      case JSTRACE_TYPE_OBJECT:
+        MarkTypeObject(trc, reinterpret_cast<types::TypeObject *>(thing), "type_stack");
+        break;
 #if JS_HAS_XML_SUPPORT
-        case JSTRACE_XML:
-            Mark(trc, reinterpret_cast<JSXML *>(thing));
-            break;
+      case JSTRACE_XML:
+        Mark(trc, static_cast<JSXML *>(thing));
+        break;
 #endif
-        default:
-            JS_ASSERT(false);
     }
 }
 
@@ -350,16 +436,29 @@ MarkValue(JSTracer *trc, const js::Value &v, const char *name)
 }
 
 void
-MarkValueRange(JSTracer *trc, Value *beg, Value *end, const char *name)
+MarkCrossCompartmentValue(JSTracer *trc, const js::Value &v, const char *name)
 {
-    for (Value *vp = beg; vp < end; ++vp) {
+    if (v.isMarkable()) {
+        js::gc::Cell *cell = (js::gc::Cell *)v.toGCThing();
+        JSRuntime *rt = trc->context->runtime;
+        if (rt->gcCurrentCompartment && cell->compartment() != rt->gcCurrentCompartment)
+            return;
+
+        MarkValue(trc, v, name);
+    }
+}
+
+void
+MarkValueRange(JSTracer *trc, const Value *beg, const Value *end, const char *name)
+{
+    for (const Value *vp = beg; vp < end; ++vp) {
         JS_SET_TRACING_INDEX(trc, name, vp - beg);
         MarkValueRaw(trc, *vp);
     }
 }
 
 void
-MarkValueRange(JSTracer *trc, size_t len, Value *vec, const char *name)
+MarkValueRange(JSTracer *trc, size_t len, const Value *vec, const char *name)
 {
     MarkValueRange(trc, vec, vec + len, name);
 }
@@ -381,7 +480,7 @@ MarkShapeRange(JSTracer *trc, size_t len, const Shape **vec, const char *name)
 
 /* N.B. Assumes JS_SET_TRACING_NAME/INDEX has already been called. */
 void
-MarkGCThing(JSTracer *trc, void *thing, uint32 kind)
+MarkGCThing(JSTracer *trc, void *thing, JSGCTraceKind kind)
 {
     if (!thing)
         return;
@@ -412,7 +511,7 @@ MarkGCThing(JSTracer *trc, void *thing, const char *name, size_t index)
 }
 
 void
-Mark(JSTracer *trc, void *thing, uint32 kind, const char *name)
+Mark(JSTracer *trc, void *thing, JSGCTraceKind kind, const char *name)
 {
     JS_ASSERT(thing);
     JS_SET_TRACING_NAME(trc, name);
@@ -431,12 +530,24 @@ MarkRoot(JSTracer *trc, JSString *thing, const char *name)
     MarkString(trc, thing, name);
 }
 
+void
+MarkRoot(JSTracer *trc, JSScript *thing, const char *name)
+{
+    MarkScript(trc, thing, name);
+}
+
 void
 MarkRoot(JSTracer *trc, const Shape *thing, const char *name)
 {
     MarkShape(trc, thing, name);
 }
 
+void
+MarkRoot(JSTracer *trc, types::TypeObject *thing, const char *name)
+{
+    MarkTypeObject(trc, thing, name);
+}
+
 void
 MarkRoot(JSTracer *trc, JSXML *thing, const char *name)
 {
@@ -481,19 +592,12 @@ static inline void
 ScanValue(GCMarker *gcmarker, const Value &v)
 {
     if (v.isMarkable()) {
-        switch (v.gcKind()) {
-          case JSTRACE_STRING: {
-            JSString *str = (JSString *)v.toGCThing();
-            if (!str->isStaticAtom())
-                PushMarkStack(gcmarker, str);
-            break;
-          }
-          case JSTRACE_OBJECT:
-            PushMarkStack(gcmarker, (JSObject *)v.toGCThing());
-            break;
-          case JSTRACE_XML:
-            PushMarkStack(gcmarker, (JSXML *)v.toGCThing());
-            break;
+        JSGCTraceKind kind = v.gcKind();
+        if (kind == JSTRACE_STRING) {
+            PushMarkStack(gcmarker, v.toString());
+        } else {
+            JS_ASSERT(kind == JSTRACE_OBJECT);
+            PushMarkStack(gcmarker, &v.toObject());
         }
     }
 }
@@ -506,13 +610,10 @@ restart:
     if (rt->gcRegenShapes)
         shape->shapeid = js_RegenerateShapeForGC(rt);
 
-    if (JSID_IS_STRING(shape->propid)) {
-        JSString *str = JSID_TO_STRING(shape->propid);
-        if (!str->isStaticAtom())
-            PushMarkStack(gcmarker, str);
-    } else if (JS_UNLIKELY(JSID_IS_OBJECT(shape->propid))) {
+    if (JSID_IS_STRING(shape->propid))
+        PushMarkStack(gcmarker, JSID_TO_STRING(shape->propid));
+    else if (JS_UNLIKELY(JSID_IS_OBJECT(shape->propid)))
         PushMarkStack(gcmarker, JSID_TO_OBJECT(shape->propid));
-    }
 
     if (shape->hasGetterValue() && shape->getter())
         PushMarkStack(gcmarker, shape->getterObject());
@@ -527,14 +628,48 @@ restart:
         goto restart;
 }
 
+static inline void
+ScanRope(GCMarker *gcmarker, JSRope *rope)
+{
+    JS_OPT_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
+                     rope->compartment() == gcmarker->context->runtime->gcCurrentCompartment
+                     || rope->compartment() == gcmarker->context->runtime->atomsCompartment);
+    JS_ASSERT(rope->isMarked());
+
+    JSString *leftChild = NULL;
+    do {
+        JSString *rightChild = rope->rightChild();
+
+        if (rightChild->isRope()) {
+            if (rightChild->markIfUnmarked())
+                gcmarker->pushRope(&rightChild->asRope());
+        } else {
+            rightChild->asLinear().mark(gcmarker);
+        }
+        leftChild = rope->leftChild();
+
+        if (leftChild->isLinear()) {
+            leftChild->asLinear().mark(gcmarker);
+            return;
+        }
+        rope = &leftChild->asRope();
+    } while (leftChild->markIfUnmarked());
+}
+
 static inline void
 PushMarkStack(GCMarker *gcmarker, JSString *str)
 {
-    JS_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
-                 str->compartment() == gcmarker->context->runtime->gcCurrentCompartment
-                 || str->compartment() == gcmarker->context->runtime->atomsCompartment);
+    JS_OPT_ASSERT_IF(gcmarker->context->runtime->gcCurrentCompartment,
+                     str->compartment() == gcmarker->context->runtime->gcCurrentCompartment
+                     || str->compartment() == gcmarker->context->runtime->atomsCompartment);
 
-    str->mark(gcmarker);
+    if (str->isLinear()) {
+        str->asLinear().mark(gcmarker);
+    } else {
+        JS_ASSERT(str->isRope());
+        if (str->markIfUnmarked())
+            ScanRope(gcmarker, &str->asRope());
+    }
 }
 
 static const uintN LARGE_OBJECT_CHUNK_SIZE = 2048;
@@ -545,35 +680,37 @@ ScanObject(GCMarker *gcmarker, JSObject *obj)
     if (obj->isNewborn())
         return;
 
+    types::TypeObject *type = obj->typeFromGC();
+    if (type != &types::emptyTypeObject)
+        PushMarkStack(gcmarker, type);
+
     if (JSObject *parent = obj->getParent())
         PushMarkStack(gcmarker, parent);
-    if (JSObject *proto = obj->getProto())
-        PushMarkStack(gcmarker, proto);
 
+    /*
+     * Call the trace hook if necessary, and check for a newType on objects
+     * which are not dense arrays (dense arrays have trace hooks).
+     */
     Class *clasp = obj->getClass();
     if (clasp->trace) {
-        if (obj->isDenseArray() && obj->getDenseArrayCapacity() > LARGE_OBJECT_CHUNK_SIZE) {
-            if (!gcmarker->largeStack.push(LargeMarkItem(obj))) {
+        if (clasp == &ArrayClass) {
+            if (obj->getDenseArrayInitializedLength() > LARGE_OBJECT_CHUNK_SIZE) {
+                if (!gcmarker->largeStack.push(LargeMarkItem(obj)))
+                    clasp->trace(gcmarker, obj);
+            } else {
                 clasp->trace(gcmarker, obj);
             }
         } else {
+            if (obj->newType)
+                PushMarkStack(gcmarker, obj->newType);
             clasp->trace(gcmarker, obj);
         }
-    }
-
-    if (obj->emptyShapes) {
-        int count = FINALIZE_OBJECT_LAST - FINALIZE_OBJECT0 + 1;
-        for (int i = 0; i < count; i++) {
-            if (obj->emptyShapes[i])
-                PushMarkStack(gcmarker, obj->emptyShapes[i]);
-        }
+    } else {
+        if (obj->newType)
+            PushMarkStack(gcmarker, obj->newType);
     }
 
     if (obj->isNative()) {
-#ifdef JS_DUMP_SCOPE_METERS
-        js::MeterEntryCount(obj->propertyCount);
-#endif
-
         js::Shape *shape = obj->lastProp;
         PushMarkStack(gcmarker, shape);
 
@@ -590,15 +727,11 @@ ScanObject(GCMarker *gcmarker, JSObject *obj)
         uint32 nslots = obj->slotSpan();
         JS_ASSERT(obj->slotSpan() <= obj->numSlots());
         if (nslots > LARGE_OBJECT_CHUNK_SIZE) {
-            if (!gcmarker->largeStack.push(LargeMarkItem(obj))) {
-                for (uint32 i = nslots; i > 0; i--)
-                    ScanValue(gcmarker, obj->getSlot(i-1));
-            }
-        } else {
-            for (uint32 i = nslots; i > 0; i--) {
-                ScanValue(gcmarker, obj->getSlot(i-1));
-            }
+            if (gcmarker->largeStack.push(LargeMarkItem(obj)))
+                return;
         }
+
+        obj->scanSlots(gcmarker);
     }
 }
 
@@ -607,19 +740,22 @@ ScanLargeObject(GCMarker *gcmarker, LargeMarkItem &item)
 {
     JSObject *obj = item.obj;
 
+    uintN start = item.markpos;
+    uintN stop;
     uint32 capacity;
     if (obj->isDenseArray()) {
-        capacity = obj->getDenseArrayCapacity();
+        capacity = obj->getDenseArrayInitializedLength();
+        stop = JS_MIN(start + LARGE_OBJECT_CHUNK_SIZE, capacity);
+        for (uintN i=stop; i>start; i--)
+            ScanValue(gcmarker, obj->getDenseArrayElement(i-1));
     } else {
         JS_ASSERT(obj->isNative());
         capacity = obj->slotSpan();
+        stop = JS_MIN(start + LARGE_OBJECT_CHUNK_SIZE, capacity);
+        for (uintN i=stop; i>start; i--)
+            ScanValue(gcmarker, obj->nativeGetSlot(i-1));
     }
 
-    uintN start = item.markpos;
-    uintN stop = JS_MIN(start + LARGE_OBJECT_CHUNK_SIZE, capacity);
-    for (uintN i=stop; i>start; i--)
-        ScanValue(gcmarker, obj->getSlot(i-1));
-
     if (stop == capacity)
         return true;
 
@@ -627,18 +763,6 @@ ScanLargeObject(GCMarker *gcmarker, LargeMarkItem &item)
     return false;
 }
 
-static void
-MarkObjectSlots(JSTracer *trc, JSObject *obj)
-{
-    JS_ASSERT(obj->slotSpan() <= obj->numSlots());
-    uint32 nslots = obj->slotSpan();
-    for (uint32 i = 0; i != nslots; ++i) {
-        const Value &v = obj->getSlot(i);
-        JS_SET_TRACING_DETAILS(trc, js_PrintObjectSlotName, obj, i);
-        MarkValueRaw(trc, v);
-    }
-}
-
 void
 MarkChildren(JSTracer *trc, JSObject *obj)
 {
@@ -646,33 +770,27 @@ MarkChildren(JSTracer *trc, JSObject *obj)
     if (obj->isNewborn())
         return;
 
+    MarkTypeObject(trc, obj->typeFromGC(), "type");
+
     /* Trace universal (ops-independent) members. */
-    if (JSObject *proto = obj->getProto())
-        MarkObject(trc, *proto, "proto");
+    if (!obj->isDenseArray() && obj->newType)
+        MarkTypeObject(trc, obj->newType, "new_type");
     if (JSObject *parent = obj->getParent())
         MarkObject(trc, *parent, "parent");
 
-    if (obj->emptyShapes) {
-        int count = FINALIZE_OBJECT_LAST - FINALIZE_OBJECT0 + 1;
-        for (int i = 0; i < count; i++) {
-            if (obj->emptyShapes[i])
-                MarkShape(trc, obj->emptyShapes[i], "emptyShape");
-        }
-    }
-
     Class *clasp = obj->getClass();
     if (clasp->trace)
         clasp->trace(trc, obj);
 
     if (obj->isNative()) {
-#ifdef JS_DUMP_SCOPE_METERS
-        js::MeterEntryCount(obj->propertyCount);
-#endif
-
         MarkShape(trc, obj->lastProp, "shape");
 
-        if (obj->slotSpan() > 0)
-            MarkObjectSlots(trc, obj);
+        JS_ASSERT(obj->slotSpan() <= obj->numSlots());
+        uint32 nslots = obj->slotSpan();
+        for (uint32 i = 0; i < nslots; i++) {
+            JS_SET_TRACING_DETAILS(trc, js_PrintObjectSlotName, obj, i);
+            MarkValueRaw(trc, obj->nativeGetSlot(i));
+        }
     }
 }
 
@@ -688,6 +806,53 @@ MarkChildren(JSTracer *trc, JSString *str)
     }
 }
 
+
+void
+MarkChildren(JSTracer *trc, JSScript *script)
+{
+    CheckScript(script, NULL);
+
+#ifdef JS_CRASH_DIAGNOSTICS
+    JSRuntime *rt = trc->context->runtime;
+    JS_OPT_ASSERT_IF(rt->gcCheckCompartment, script->compartment() == rt->gcCheckCompartment);
+#endif
+    
+    MarkAtomRange(trc, script->natoms, script->atoms, "atoms");
+
+    if (JSScript::isValidOffset(script->objectsOffset)) {
+        JSObjectArray *objarray = script->objects();
+        MarkObjectRange(trc, objarray->length, objarray->vector, "objects");
+    }
+
+    if (JSScript::isValidOffset(script->regexpsOffset)) {
+        JSObjectArray *objarray = script->regexps();
+        MarkObjectRange(trc, objarray->length, objarray->vector, "objects");
+    }
+
+    if (JSScript::isValidOffset(script->constOffset)) {
+        JSConstArray *constarray = script->consts();
+        MarkValueRange(trc, constarray->length, constarray->vector, "consts");
+    }
+
+    if (!script->isCachedEval && script->u.object)
+        MarkObject(trc, *script->u.object, "object");
+
+    if (IS_GC_MARKING_TRACER(trc) && script->filename)
+        js_MarkScriptFilename(script->filename);
+
+    script->bindings.trace(trc);
+
+    if (script->types)
+        script->types->trace(trc);
+
+#ifdef JS_METHODJIT
+    if (script->jitNormal)
+        script->jitNormal->trace(trc);
+    if (script->jitCtor)
+        script->jitCtor->trace(trc);
+#endif
+}
+
 void
 MarkChildren(JSTracer *trc, const Shape *shape)
 {
@@ -707,6 +872,78 @@ restart:
         goto restart;
 }
 
+static void
+ScanTypeObject(GCMarker *gcmarker, types::TypeObject *type)
+{
+    if (!type->singleton) {
+        unsigned count = type->getPropertyCount();
+        for (unsigned i = 0; i < count; i++) {
+            types::Property *prop = type->getProperty(i);
+            if (prop && JSID_IS_STRING(prop->id))
+                PushMarkStack(gcmarker, JSID_TO_STRING(prop->id));
+        }
+    }
+
+    if (type->emptyShapes) {
+        int count = FINALIZE_OBJECT_LAST - FINALIZE_OBJECT0 + 1;
+        for (int i = 0; i < count; i++) {
+            if (type->emptyShapes[i])
+                PushMarkStack(gcmarker, type->emptyShapes[i]);
+        }
+    }
+
+    if (type->proto)
+        PushMarkStack(gcmarker, type->proto);
+
+    if (type->newScript) {
+        PushMarkStack(gcmarker, type->newScript->fun);
+        PushMarkStack(gcmarker, type->newScript->shape);
+    }
+
+    /*
+     * Don't need to trace singleton or functionScript, an object with this
+     * type must have already been traced and it will also hold a reference
+     * on the script (singleton and functionScript types cannot be the newType
+     * of another object). Attempts to mark type objects directly must use
+     * MarkTypeObject, which will itself mark these extra bits.
+     */
+}
+
+void
+MarkChildren(JSTracer *trc, types::TypeObject *type)
+{
+    if (!type->singleton) {
+        unsigned count = type->getPropertyCount();
+        for (unsigned i = 0; i < count; i++) {
+            types::Property *prop = type->getProperty(i);
+            if (prop)
+                MarkId(trc, prop->id, "type_prop");
+        }
+    }
+
+    if (type->emptyShapes) {
+        int count = FINALIZE_OBJECT_LAST - FINALIZE_OBJECT0 + 1;
+        for (int i = 0; i < count; i++) {
+            if (type->emptyShapes[i])
+                MarkShape(trc, type->emptyShapes[i], "empty_shape");
+        }
+    }
+
+    if (type->proto)
+        MarkObject(trc, *type->proto, "type_proto");
+
+    if (type->singleton)
+        MarkObject(trc, *type->singleton, "type_singleton");
+
+    if (type->newScript) {
+        MarkObject(trc, *type->newScript->fun, "type_new_function");
+        MarkShape(trc, type->newScript->shape, "type_new_shape");
+    }
+
+    if (type->interpretedFunction)
+        MarkObject(trc, *type->interpretedFunction, "type_function");
+}
+
 #ifdef JS_HAS_XML_SUPPORT
 void
 MarkChildren(JSTracer *trc, JSXML *xml)
@@ -720,10 +957,19 @@ MarkChildren(JSTracer *trc, JSXML *xml)
 void
 GCMarker::drainMarkStack()
 {
+    JSRuntime *rt = context->runtime;
+    rt->gcCheckCompartment = rt->gcCurrentCompartment;
+
     while (!isMarkStackEmpty()) {
+        while (!ropeStack.isEmpty())
+            ScanRope(this, ropeStack.pop());
+
         while (!objStack.isEmpty())
             ScanObject(this, objStack.pop());
 
+        while (!typeStack.isEmpty())
+            ScanTypeObject(this, typeStack.pop());
+
         while (!xmlStack.isEmpty())
             MarkChildren(this, xmlStack.pop());
 
@@ -741,30 +987,67 @@ GCMarker::drainMarkStack()
             markDelayedChildren();
         }
     }
+
+    rt->gcCheckCompartment = NULL;
 }
 
 } /* namespace js */
 
 JS_PUBLIC_API(void)
-JS_TraceChildren(JSTracer *trc, void *thing, uint32 kind)
+JS_TraceChildren(JSTracer *trc, void *thing, JSGCTraceKind kind)
 {
     switch (kind) {
       case JSTRACE_OBJECT:
-	MarkChildren(trc, (JSObject *)thing);
+	MarkChildren(trc, static_cast<JSObject *>(thing));
         break;
 
       case JSTRACE_STRING:
-	MarkChildren(trc, (JSString *)thing);
+	MarkChildren(trc, static_cast<JSString *>(thing));
+        break;
+
+      case JSTRACE_SCRIPT:
+	MarkChildren(trc, static_cast<JSScript *>(thing));
         break;
 
       case JSTRACE_SHAPE:
-	MarkChildren(trc, (js::Shape *)thing);
+	MarkChildren(trc, static_cast<Shape *>(thing));
+        break;
+
+      case JSTRACE_TYPE_OBJECT:
+        MarkChildren(trc, (types::TypeObject *)thing);
         break;
 
 #if JS_HAS_XML_SUPPORT
       case JSTRACE_XML:
-        MarkChildren(trc, (JSXML *)thing);
+        MarkChildren(trc, static_cast<JSXML *>(thing));
         break;
 #endif
     }
 }
+
+inline void
+JSObject::scanSlots(GCMarker *gcmarker)
+{
+    /*
+     * Scan the fixed slots and the dynamic slots separately, to avoid
+     * branching inside nativeGetSlot().
+     */
+    JS_ASSERT(slotSpan() <= numSlots());
+    unsigned i, nslots = slotSpan();
+    if (slots) {
+        unsigned nfixed = numFixedSlots();
+        if (nslots > nfixed) {
+            Value *vp = fixedSlots();
+            for (i = 0; i < nfixed; i++, vp++)
+                ScanValue(gcmarker, *vp);
+            vp = slots;
+            for (; i < nslots; i++, vp++)
+                ScanValue(gcmarker, *vp);
+            return;
+        }
+    }
+    JS_ASSERT(nslots <= numFixedSlots());
+    Value *vp = fixedSlots();
+    for (i = 0; i < nslots; i++, vp++)
+        ScanValue(gcmarker, *vp);
+}
