@@ -59,7 +59,7 @@
 #include "jsiter.h"
 #include "jsbool.h"
 #include "jsval.h"
-#include "jsvalue.h"
+#include "jsinferinlines.h"
 #include "jsobjinlines.h"
 #include "jsobj.h"
 #include "jsarray.h"
@@ -231,11 +231,11 @@ class NodeBuilder
             if (!newNodeLoc(pos, &loc))
                 return false;
             Value argv[] = { loc };
-            return ExternalInvoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
+            return Invoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
         }
 
         Value argv[] = { NullValue() }; /* no zero-length arrays allowed! */
-        return ExternalInvoke(cx, userv, fun, 0, argv, dst);
+        return Invoke(cx, userv, fun, 0, argv, dst);
     }
 
     bool callback(Value fun, Value v1, TokenPos *pos, Value *dst) {
@@ -244,11 +244,11 @@ class NodeBuilder
             if (!newNodeLoc(pos, &loc))
                 return false;
             Value argv[] = { v1, loc };
-            return ExternalInvoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
+            return Invoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
         }
 
         Value argv[] = { v1 };
-        return ExternalInvoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
+        return Invoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
     }
 
     bool callback(Value fun, Value v1, Value v2, TokenPos *pos, Value *dst) {
@@ -257,11 +257,11 @@ class NodeBuilder
             if (!newNodeLoc(pos, &loc))
                 return false;
             Value argv[] = { v1, v2, loc };
-            return ExternalInvoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
+            return Invoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
         }
 
         Value argv[] = { v1, v2 };
-        return ExternalInvoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
+        return Invoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
     }
 
     bool callback(Value fun, Value v1, Value v2, Value v3, TokenPos *pos, Value *dst) {
@@ -270,11 +270,11 @@ class NodeBuilder
             if (!newNodeLoc(pos, &loc))
                 return false;
             Value argv[] = { v1, v2, v3, loc };
-            return ExternalInvoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
+            return Invoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
         }
 
         Value argv[] = { v1, v2, v3 };
-        return ExternalInvoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
+        return Invoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
     }
 
     bool callback(Value fun, Value v1, Value v2, Value v3, Value v4, TokenPos *pos, Value *dst) {
@@ -283,11 +283,11 @@ class NodeBuilder
             if (!newNodeLoc(pos, &loc))
                 return false;
             Value argv[] = { v1, v2, v3, v4, loc };
-            return ExternalInvoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
+            return Invoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
         }
 
         Value argv[] = { v1, v2, v3, v4 };
-        return ExternalInvoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
+        return Invoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
     }
 
     bool callback(Value fun, Value v1, Value v2, Value v3, Value v4, Value v5,
@@ -297,11 +297,11 @@ class NodeBuilder
             if (!newNodeLoc(pos, &loc))
                 return false;
             Value argv[] = { v1, v2, v3, v4, v5, loc };
-            return ExternalInvoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
+            return Invoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
         }
 
         Value argv[] = { v1, v2, v3, v4, v5 };
-        return ExternalInvoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
+        return Invoke(cx, userv, fun, JS_ARRAY_LENGTH(argv), argv, dst);
     }
 
     Value opt(Value v) {
@@ -322,7 +322,7 @@ class NodeBuilder
     }
 
     bool newObject(JSObject **dst) {
-        JSObject *nobj = NewNonFunction<WithProto::Class>(cx, &js_ObjectClass, NULL, NULL);
+        JSObject *nobj = NewNonFunction<WithProto::Class>(cx, &ObjectClass, NULL, NULL);
         if (!nobj)
             return false;
 
@@ -632,7 +632,7 @@ NodeBuilder::newNode(ASTType type, TokenPos *pos, JSObject **dst)
 
     Value tv;
 
-    JSObject *node = NewNonFunction<WithProto::Class>(cx, &js_ObjectClass, NULL, NULL);
+    JSObject *node = NewNonFunction<WithProto::Class>(cx, &ObjectClass, NULL, NULL);
     if (!node ||
         !setNodeLoc(node, pos) ||
         !atomValue(nodeTypeNames[type], &tv) ||
@@ -907,29 +907,20 @@ bool
 NodeBuilder::tryStatement(Value body, NodeVector &catches, Value finally,
                           TokenPos *pos, Value *dst)
 {
-    Value handler;
+    Value handlers;
 
     Value cb = callbacks[AST_TRY_STMT];
     if (!cb.isNull()) {
-        return newArray(catches, &handler) &&
-               callback(cb, body, handler, opt(finally), pos, dst);
+        return newArray(catches, &handlers) &&
+               callback(cb, body, handlers, opt(finally), pos, dst);
     }
 
-    switch (catches.length()) {
-      case 0:
-        handler.setNull();
-        break;
-      case 1:
-        handler = catches[0];
-        break;
-      default:
-        if (!newArray(catches, &handler))
-            return false;
-    }
+    if (!newArray(catches, &handlers))
+        return false;
 
     return newNode(AST_TRY_STMT, pos,
                    "block", body,
-                   "handler", handler,
+                   "handlers", handlers,
                    "finalizer", finally,
                    dst);
 }
@@ -1624,7 +1615,6 @@ class ASTSerializer
     bool expressions(JSParseNode *pn, NodeVector &elts);
     bool xmls(JSParseNode *pn, NodeVector &elts);
     bool leftAssociate(JSParseNode *pn, Value *dst);
-    bool binaryOperands(JSParseNode *pn, NodeVector &elts);
     bool functionArgs(JSParseNode *pn, JSParseNode *pnargs, JSParseNode *pndestruct,
                       JSParseNode *pnbody, NodeVector &args);
 
@@ -1832,7 +1822,7 @@ ASTSerializer::binop(TokenKind tk, JSOp op)
 bool
 ASTSerializer::statements(JSParseNode *pn, NodeVector &elts)
 {
-    JS_ASSERT(PN_TYPE(pn) == TOK_LC && pn->pn_arity == PN_LIST);
+    JS_ASSERT(pn->isKind(TOK_LC) && pn->isArity(PN_LIST));
 
     if (!elts.reserve(pn->pn_count))
         return false;
@@ -1882,7 +1872,7 @@ ASTSerializer::xmls(JSParseNode *pn, NodeVector &elts)
 bool
 ASTSerializer::blockStatement(JSParseNode *pn, Value *dst)
 {
-    JS_ASSERT(PN_TYPE(pn) == TOK_LC);
+    JS_ASSERT(pn->isKind(TOK_LC));
 
     NodeVector stmts(cx);
     return statements(pn, stmts) &&
@@ -1892,10 +1882,7 @@ ASTSerializer::blockStatement(JSParseNode *pn, Value *dst)
 bool
 ASTSerializer::program(JSParseNode *pn, Value *dst)
 {
-    JS_ASSERT(pn);
-
-    /* Workaround for bug 588061: parser's reported start position is always 0:0. */
-    pn->pn_pos.begin.lineno = lineno;
+    JS_ASSERT(pn->pn_pos.begin.lineno == lineno);
 
     NodeVector stmts(cx);
     return statements(pn, stmts) &&
@@ -1912,11 +1899,9 @@ ASTSerializer::sourceElement(JSParseNode *pn, Value *dst)
 bool
 ASTSerializer::declaration(JSParseNode *pn, Value *dst)
 {
-    JS_ASSERT(PN_TYPE(pn) == TOK_FUNCTION ||
-              PN_TYPE(pn) == TOK_VAR ||
-              PN_TYPE(pn) == TOK_LET);
+    JS_ASSERT(pn->isKind(TOK_FUNCTION) || pn->isKind(TOK_VAR) || pn->isKind(TOK_LET));
 
-    switch (PN_TYPE(pn)) {
+    switch (pn->getKind()) {
       case TOK_FUNCTION:
         return function(pn, AST_FUNC_DECL, dst);
 
@@ -1924,7 +1909,7 @@ ASTSerializer::declaration(JSParseNode *pn, Value *dst)
         return variableDeclaration(pn, false, dst);
 
       default:
-        JS_ASSERT(PN_TYPE(pn) == TOK_LET);
+        JS_ASSERT(pn->isKind(TOK_LET));
         return variableDeclaration(pn, true, dst);
     }
 }
@@ -1932,7 +1917,7 @@ ASTSerializer::declaration(JSParseNode *pn, Value *dst)
 bool
 ASTSerializer::variableDeclaration(JSParseNode *pn, bool let, Value *dst)
 {
-    JS_ASSERT(let ? PN_TYPE(pn) == TOK_LET : PN_TYPE(pn) == TOK_VAR);
+    JS_ASSERT(let ? pn->isKind(TOK_LET) : pn->isKind(TOK_VAR));
 
     /* Later updated to VARDECL_CONST if we find a PND_CONST declarator. */
     VarDeclKind kind = let ? VARDECL_LET : VARDECL_VAR;
@@ -1964,16 +1949,16 @@ bool
 ASTSerializer::variableDeclarator(JSParseNode *pn, VarDeclKind *pkind, Value *dst)
 {
     /* A destructuring declarator is always a TOK_ASSIGN. */
-    JS_ASSERT(PN_TYPE(pn) == TOK_NAME || PN_TYPE(pn) == TOK_ASSIGN);
+    JS_ASSERT(pn->isKind(TOK_NAME) || pn->isKind(TOK_ASSIGN));
 
     JSParseNode *pnleft;
     JSParseNode *pnright;
 
-    if (PN_TYPE(pn) == TOK_NAME) {
+    if (pn->isKind(TOK_NAME)) {
         pnleft = pn;
-        pnright = pn->pn_expr;
+        pnright = pn->isUsed() ? NULL : pn->pn_expr;
     } else {
-        JS_ASSERT(PN_TYPE(pn) == TOK_ASSIGN);
+        JS_ASSERT(pn->isKind(TOK_ASSIGN));
         pnleft = pn->pn_left;
         pnright = pn->pn_right;
     }
@@ -2029,7 +2014,7 @@ ASTSerializer::switchStatement(JSParseNode *pn, Value *dst)
     JSParseNode *listNode;
     bool lexical;
 
-    if (PN_TYPE(pn->pn_right) == TOK_LEXICALSCOPE) {
+    if (pn->pn_right->isKind(TOK_LEXICALSCOPE)) {
         listNode = pn->pn_right->pn_expr;
         lexical = true;
     } else {
@@ -2098,9 +2083,9 @@ ASTSerializer::forInit(JSParseNode *pn, Value *dst)
         return true;
     }
 
-    return (PN_TYPE(pn) == TOK_VAR)
+    return pn->isKind(TOK_VAR)
            ? variableDeclaration(pn, false, dst)
-           : (PN_TYPE(pn) == TOK_LET)
+           : pn->isKind(TOK_LET)
            ? variableDeclaration(pn, true, dst)
            : expression(pn, dst);
 }
@@ -2108,14 +2093,15 @@ ASTSerializer::forInit(JSParseNode *pn, Value *dst)
 bool
 ASTSerializer::statement(JSParseNode *pn, Value *dst)
 {
-    switch (PN_TYPE(pn)) {
+    JS_CHECK_RECURSION(cx, return false);
+    switch (pn->getKind()) {
       case TOK_FUNCTION:
       case TOK_VAR:
       case TOK_LET:
         return declaration(pn, dst);
 
       case TOK_NAME:
-        LOCAL_ASSERT(pn->pn_used);
+        LOCAL_ASSERT(pn->isUsed());
         return statement(pn->pn_lexdef, dst);
 
       case TOK_SEMI:
@@ -2128,7 +2114,7 @@ ASTSerializer::statement(JSParseNode *pn, Value *dst)
 
       case TOK_LEXICALSCOPE:
         pn = pn->pn_expr;
-        if (PN_TYPE(pn) == TOK_LET) {
+        if (pn->isKind(TOK_LET)) {
             NodeVector dtors(cx);
             Value stmt;
 
@@ -2137,7 +2123,7 @@ ASTSerializer::statement(JSParseNode *pn, Value *dst)
                    builder.letStatement(dtors, stmt, &pn->pn_pos, dst);
         }
 
-        if (PN_TYPE(pn) != TOK_LC)
+        if (!pn->isKind(TOK_LC))
             return statement(pn, dst);
         /* FALL THROUGH */
 
@@ -2167,7 +2153,7 @@ ASTSerializer::statement(JSParseNode *pn, Value *dst)
 
         return expression(pn->pn_left, &expr) &&
                statement(pn->pn_right, &stmt) &&
-               (PN_TYPE(pn) == TOK_WITH)
+               pn->isKind(TOK_WITH)
                ? builder.withStatement(expr, stmt, &pn->pn_pos, dst)
                : builder.whileStatement(expr, stmt, &pn->pn_pos, dst);
       }
@@ -2191,15 +2177,15 @@ ASTSerializer::statement(JSParseNode *pn, Value *dst)
 
         bool isForEach = pn->pn_iflags & JSITER_FOREACH;
 
-        if (PN_TYPE(head) == TOK_IN) {
+        if (head->isKind(TOK_IN)) {
             Value var, expr;
 
-            return (PN_TYPE(head->pn_left) == TOK_VAR
-                    ? variableDeclaration(head->pn_left, false, &var)
-                    : PN_TYPE(head->pn_left) == TOK_LET
-                    ? variableDeclaration(head->pn_left, true, &var)
-                    : pattern(head->pn_left, NULL, &var)) &&
-                   expression(head->pn_right, &expr) &&
+            return (!head->pn_kid1
+                    ? pattern(head->pn_kid2, NULL, &var)
+                    : variableDeclaration(head->pn_kid1,
+                                          head->pn_kid1->isKind(TOK_LET),
+                                          &var)) &&
+                   expression(head->pn_kid3, &expr) &&
                    builder.forInStatement(var, expr, stmt, isForEach, &pn->pn_pos, dst);
         }
 
@@ -2217,49 +2203,22 @@ ASTSerializer::statement(JSParseNode *pn, Value *dst)
         LOCAL_ASSERT(pn->pn_count == 2);
 
         JSParseNode *prelude = pn->pn_head;
-        JSParseNode *body = prelude->pn_next;
+        JSParseNode *loop = prelude->pn_next;
 
-        LOCAL_ASSERT((PN_TYPE(prelude) == TOK_VAR && PN_TYPE(body) == TOK_FOR) ||
-                     (PN_TYPE(prelude) == TOK_SEMI && PN_TYPE(body) == TOK_LEXICALSCOPE));
+        LOCAL_ASSERT(prelude->isKind(TOK_VAR) && loop->isKind(TOK_FOR));
 
-        JSParseNode *loop;
         Value var;
-
-        if (PN_TYPE(prelude) == TOK_VAR) {
-            loop = body;
-
-            if (!variableDeclaration(prelude, false, &var))
-                return false;
-        } else {
-            loop = body->pn_expr;
-
-            LOCAL_ASSERT(PN_TYPE(loop->pn_left) == TOK_IN &&
-                         PN_TYPE(loop->pn_left->pn_left) == TOK_LET &&
-                         loop->pn_left->pn_left->pn_count == 1);
-
-            JSParseNode *pnlet = loop->pn_left->pn_left;
-
-            VarDeclKind kind = VARDECL_LET;
-            NodeVector dtors(cx);
-            Value patt, init, dtor;
-
-            if (!pattern(pnlet->pn_head, &kind, &patt) ||
-                !expression(prelude->pn_kid, &init) ||
-                !builder.variableDeclarator(patt, init, &pnlet->pn_pos, &dtor) ||
-                !dtors.append(dtor) ||
-                !builder.variableDeclaration(dtors, kind, &pnlet->pn_pos, &var)) {
-                return false;
-            }
-        }
+        if (!variableDeclaration(prelude, false, &var))
+            return false;
 
         JSParseNode *head = loop->pn_left;
-        JS_ASSERT(PN_TYPE(head) == TOK_IN);
+        JS_ASSERT(head->isKind(TOK_IN));
 
         bool isForEach = loop->pn_iflags & JSITER_FOREACH;
 
         Value expr, stmt;
 
-        return expression(head->pn_right, &expr) &&
+        return expression(head->pn_kid3, &expr) &&
                statement(loop->pn_right, &stmt) &&
                builder.forInStatement(var, expr, stmt, isForEach, &pn->pn_pos, dst);
       }
@@ -2270,7 +2229,7 @@ ASTSerializer::statement(JSParseNode *pn, Value *dst)
         Value label;
 
         return optIdentifier(pn->pn_atom, NULL, &label) &&
-               (PN_TYPE(pn) == TOK_BREAK
+               (pn->isKind(TOK_BREAK)
                 ? builder.breakStatement(label, &pn->pn_pos, dst)
                 : builder.continueStatement(label, &pn->pn_pos, dst));
       }
@@ -2290,7 +2249,7 @@ ASTSerializer::statement(JSParseNode *pn, Value *dst)
         Value arg;
 
         return optExpression(pn->pn_kid, &arg) &&
-               (PN_TYPE(pn) == TOK_THROW
+               (pn->isKind(TOK_THROW)
                 ? builder.throwStatement(arg, &pn->pn_pos, dst)
                 : builder.returnStatement(arg, &pn->pn_pos, dst));
       }
@@ -2301,7 +2260,7 @@ ASTSerializer::statement(JSParseNode *pn, Value *dst)
 #if JS_HAS_XML_SUPPORT
       case TOK_DEFAULT:
       {
-        LOCAL_ASSERT(pn->pn_arity == PN_UNARY);
+        LOCAL_ASSERT(pn->isArity(PN_UNARY));
 
         Value ns;
 
@@ -2318,10 +2277,10 @@ ASTSerializer::statement(JSParseNode *pn, Value *dst)
 bool
 ASTSerializer::leftAssociate(JSParseNode *pn, Value *dst)
 {
-    JS_ASSERT(pn->pn_arity == PN_LIST);
+    JS_ASSERT(pn->isArity(PN_LIST));
     JS_ASSERT(pn->pn_count >= 1);
 
-    TokenKind tk = PN_TYPE(pn);
+    TokenKind tk = pn->getKind();
     bool lor = tk == TOK_OR;
     bool logop = lor || (tk == TOK_AND);
 
@@ -2340,7 +2299,7 @@ ASTSerializer::leftAssociate(JSParseNode *pn, Value *dst)
             if (!builder.logicalExpression(lor, left, right, &subpos, &left))
                 return false;
         } else {
-            BinaryOperator op = binop(PN_TYPE(pn), PN_OP(pn));
+            BinaryOperator op = binop(pn->getKind(), pn->getOp());
             LOCAL_ASSERT(op > BINOP_ERR && op < BINOP_LIMIT);
 
             if (!builder.binaryExpression(op, left, right, &subpos, &left))
@@ -2352,49 +2311,32 @@ ASTSerializer::leftAssociate(JSParseNode *pn, Value *dst)
     return true;
 }
 
-bool
-ASTSerializer::binaryOperands(JSParseNode *pn, NodeVector &elts)
-{
-    if (pn->pn_arity == PN_BINARY) {
-        Value left, right;
-
-        return expression(pn->pn_left, &left) &&
-               elts.append(left) &&
-               expression(pn->pn_right, &right) &&
-               elts.append(right);
-    }
-
-    LOCAL_ASSERT(pn->pn_arity == PN_LIST);
-
-    return expressions(pn, elts);
-}
-
 bool
 ASTSerializer::comprehensionBlock(JSParseNode *pn, Value *dst)
 {
-    LOCAL_ASSERT(pn->pn_arity == PN_BINARY);
+    LOCAL_ASSERT(pn->isArity(PN_BINARY));
 
     JSParseNode *in = pn->pn_left;
 
-    LOCAL_ASSERT(in && PN_TYPE(in) == TOK_IN);
+    LOCAL_ASSERT(in && in->isKind(TOK_IN));
 
     bool isForEach = pn->pn_iflags & JSITER_FOREACH;
 
     Value patt, src;
-    return pattern(in->pn_left, NULL, &patt) &&
-           expression(in->pn_right, &src) &&
+    return pattern(in->pn_kid2, NULL, &patt) &&
+           expression(in->pn_kid3, &src) &&
            builder.comprehensionBlock(patt, src, isForEach, &in->pn_pos, dst);
 }
 
 bool
 ASTSerializer::comprehension(JSParseNode *pn, Value *dst)
 {
-    LOCAL_ASSERT(PN_TYPE(pn) == TOK_FOR);
+    LOCAL_ASSERT(pn->isKind(TOK_FOR));
 
     NodeVector blocks(cx);
 
     JSParseNode *next = pn;
-    while (PN_TYPE(next) == TOK_FOR) {
+    while (next->isKind(TOK_FOR)) {
         Value block;
         if (!comprehensionBlock(next, &block) || !blocks.append(block))
             return false;
@@ -2403,17 +2345,17 @@ ASTSerializer::comprehension(JSParseNode *pn, Value *dst)
 
     Value filter = MagicValue(JS_SERIALIZE_NO_NODE);
 
-    if (PN_TYPE(next) == TOK_IF) {
+    if (next->isKind(TOK_IF)) {
         if (!optExpression(next->pn_kid1, &filter))
             return false;
         next = next->pn_kid2;
-    } else if (PN_TYPE(next) == TOK_LC && next->pn_count == 0) {
+    } else if (next->isKind(TOK_LC) && next->pn_count == 0) {
         /* js_FoldConstants optimized away the push. */
         NodeVector empty(cx);
         return builder.arrayExpression(empty, &pn->pn_pos, dst);
     }
 
-    LOCAL_ASSERT(PN_TYPE(next) == TOK_ARRAYPUSH);
+    LOCAL_ASSERT(next->isKind(TOK_ARRAYPUSH));
 
     Value body;
 
@@ -2424,12 +2366,12 @@ ASTSerializer::comprehension(JSParseNode *pn, Value *dst)
 bool
 ASTSerializer::generatorExpression(JSParseNode *pn, Value *dst)
 {
-    LOCAL_ASSERT(PN_TYPE(pn) == TOK_FOR);
+    LOCAL_ASSERT(pn->isKind(TOK_FOR));
 
     NodeVector blocks(cx);
 
     JSParseNode *next = pn;
-    while (PN_TYPE(next) == TOK_FOR) {
+    while (next->isKind(TOK_FOR)) {
         Value block;
         if (!comprehensionBlock(next, &block) || !blocks.append(block))
             return false;
@@ -2438,14 +2380,14 @@ ASTSerializer::generatorExpression(JSParseNode *pn, Value *dst)
 
     Value filter = MagicValue(JS_SERIALIZE_NO_NODE);
 
-    if (PN_TYPE(next) == TOK_IF) {
+    if (next->isKind(TOK_IF)) {
         if (!optExpression(next->pn_kid1, &filter))
             return false;
         next = next->pn_kid2;
     }
 
-    LOCAL_ASSERT(PN_TYPE(next) == TOK_SEMI &&
-                 PN_TYPE(next->pn_kid) == TOK_YIELD &&
+    LOCAL_ASSERT(next->isKind(TOK_SEMI) &&
+                 next->pn_kid->isKind(TOK_YIELD) &&
                  next->pn_kid->pn_kid);
 
     Value body;
@@ -2457,7 +2399,8 @@ ASTSerializer::generatorExpression(JSParseNode *pn, Value *dst)
 bool
 ASTSerializer::expression(JSParseNode *pn, Value *dst)
 {
-    switch (PN_TYPE(pn)) {
+    JS_CHECK_RECURSION(cx, return false);
+    switch (pn->getKind()) {
       case TOK_FUNCTION:
         return function(pn, AST_FUNC_EXPR, dst);
 
@@ -2481,11 +2424,11 @@ ASTSerializer::expression(JSParseNode *pn, Value *dst)
       case TOK_OR:
       case TOK_AND:
       {
-        if (pn->pn_arity == PN_BINARY) {
+        if (pn->isArity(PN_BINARY)) {
             Value left, right;
             return expression(pn->pn_left, &left) &&
                    expression(pn->pn_right, &right) &&
-                   builder.logicalExpression(PN_TYPE(pn) == TOK_OR, left, right, &pn->pn_pos, dst);
+                   builder.logicalExpression(pn->isKind(TOK_OR), left, right, &pn->pn_pos, dst);
         }
         return leftAssociate(pn, dst);
       }
@@ -2493,8 +2436,8 @@ ASTSerializer::expression(JSParseNode *pn, Value *dst)
       case TOK_INC:
       case TOK_DEC:
       {
-        bool incr = PN_TYPE(pn) == TOK_INC;
-        bool prefix = PN_OP(pn) >= JSOP_INCNAME && PN_OP(pn) <= JSOP_DECELEM;
+        bool incr = pn->isKind(TOK_INC);
+        bool prefix = pn->getOp() >= JSOP_INCNAME && pn->getOp() <= JSOP_DECELEM;
 
         Value expr;
         return expression(pn->pn_kid, &expr) &&
@@ -2503,7 +2446,7 @@ ASTSerializer::expression(JSParseNode *pn, Value *dst)
 
       case TOK_ASSIGN:
       {
-        AssignmentOperator op = aop(PN_OP(pn));
+        AssignmentOperator op = aop(pn->getOp());
         LOCAL_ASSERT(op > AOP_ERR && op < AOP_LIMIT);
 
         Value lhs, rhs;
@@ -2525,8 +2468,8 @@ ASTSerializer::expression(JSParseNode *pn, Value *dst)
       case TOK_IN:
       case TOK_INSTANCEOF:
       case TOK_DBLDOT:
-        if (pn->pn_arity == PN_BINARY) {
-            BinaryOperator op = binop(PN_TYPE(pn), PN_OP(pn));
+        if (pn->isArity(PN_BINARY)) {
+            BinaryOperator op = binop(pn->getKind(), pn->getOp());
             LOCAL_ASSERT(op > BINOP_ERR && op < BINOP_LIMIT);
 
             Value left, right;
@@ -2539,14 +2482,14 @@ ASTSerializer::expression(JSParseNode *pn, Value *dst)
       case TOK_DELETE:
       case TOK_UNARYOP:
 #if JS_HAS_XML_SUPPORT
-        if (PN_OP(pn) == JSOP_XMLNAME ||
-            PN_OP(pn) == JSOP_SETXMLNAME ||
-            PN_OP(pn) == JSOP_BINDXMLNAME)
+        if (pn->isOp(JSOP_XMLNAME) ||
+            pn->isOp(JSOP_SETXMLNAME) ||
+            pn->isOp(JSOP_BINDXMLNAME))
             return expression(pn->pn_kid, dst);
 #endif
 
       {
-        UnaryOperator op = unop(PN_TYPE(pn), PN_OP(pn));
+        UnaryOperator op = unop(pn->getKind(), pn->getOp());
         LOCAL_ASSERT(op > UNOP_ERR && op < UNOP_LIMIT);
 
         Value expr;
@@ -2579,7 +2522,7 @@ ASTSerializer::expression(JSParseNode *pn, Value *dst)
             args.infallibleAppend(arg);
         }
 
-        return PN_TYPE(pn) == TOK_NEW
+        return pn->isKind(TOK_NEW)
                ? builder.newExpression(callee, args, &pn->pn_pos, dst)
                : builder.callExpression(callee, args, &pn->pn_pos, dst);
       }
@@ -2597,9 +2540,9 @@ ASTSerializer::expression(JSParseNode *pn, Value *dst)
         Value left, right;
         bool computed = true;
 #ifdef JS_HAS_XML_SUPPORT
-        computed = (PN_TYPE(pn->pn_right) != TOK_DBLCOLON &&
-                    PN_TYPE(pn->pn_right) != TOK_ANYNAME &&
-                    PN_TYPE(pn->pn_right) != TOK_AT);
+        computed = (!pn->pn_right->isKind(TOK_DBLCOLON) &&
+                    !pn->pn_right->isKind(TOK_ANYNAME) &&
+                    !pn->pn_right->isKind(TOK_AT));
 #endif
         return expression(pn->pn_left, &left) &&
                expression(pn->pn_right, &right) &&
@@ -2613,7 +2556,7 @@ ASTSerializer::expression(JSParseNode *pn, Value *dst)
             return false;
 
         for (JSParseNode *next = pn->pn_head; next; next = next->pn_next) {
-            if (PN_TYPE(next) == TOK_COMMA) {
+            if (next->isKind(TOK_COMMA)) {
                 elts.infallibleAppend(MagicValue(JS_SERIALIZE_NO_NODE));
             } else {
                 Value expr;
@@ -2649,7 +2592,7 @@ ASTSerializer::expression(JSParseNode *pn, Value *dst)
       case TOK_REGEXP:
       case TOK_NUMBER:
       case TOK_PRIMARY:
-        return PN_OP(pn) == JSOP_THIS ? builder.thisExpression(&pn->pn_pos, dst) : literal(pn, dst);
+        return pn->isOp(JSOP_THIS) ? builder.thisExpression(&pn->pn_pos, dst) : literal(pn, dst);
 
       case TOK_YIELD:
       {
@@ -2671,7 +2614,7 @@ ASTSerializer::expression(JSParseNode *pn, Value *dst)
       case TOK_ARRAYCOMP:
         /* NB: it's no longer the case that pn_count could be 2. */
         LOCAL_ASSERT(pn->pn_count == 1);
-        LOCAL_ASSERT(PN_TYPE(pn->pn_head) == TOK_LEXICALSCOPE);
+        LOCAL_ASSERT(pn->pn_head->isKind(TOK_LEXICALSCOPE));
 
         return comprehension(pn->pn_head->pn_expr, dst);
 
@@ -2695,25 +2638,25 @@ ASTSerializer::expression(JSParseNode *pn, Value *dst)
       {
         Value right;
 
-        LOCAL_ASSERT(pn->pn_arity == PN_NAME || pn->pn_arity == PN_BINARY);
+        LOCAL_ASSERT(pn->isArity(PN_NAME) || pn->isArity(PN_BINARY));
 
         JSParseNode *pnleft;
         bool computed;
 
-        if (pn->pn_arity == PN_BINARY) {
+        if (pn->isArity(PN_BINARY)) {
             computed = true;
             pnleft = pn->pn_left;
             if (!expression(pn->pn_right, &right))
                 return false;
         } else {
-            JS_ASSERT(pn->pn_arity == PN_NAME);
+            JS_ASSERT(pn->isArity(PN_NAME));
             computed = false;
             pnleft = pn->pn_expr;
             if (!identifier(pn->pn_atom, NULL, &right))
                 return false;
         }
 
-        if (PN_TYPE(pnleft) == TOK_FUNCTION)
+        if (pnleft->isKind(TOK_FUNCTION))
             return builder.xmlFunctionQualifiedIdentifier(right, computed, &pn->pn_pos, dst);
 
         Value left;
@@ -2725,9 +2668,9 @@ ASTSerializer::expression(JSParseNode *pn, Value *dst)
       {
         Value expr;
         JSParseNode *kid = pn->pn_kid;
-        bool computed = ((PN_TYPE(kid) != TOK_NAME || PN_OP(kid) != JSOP_QNAMEPART) &&
-                         PN_TYPE(kid) != TOK_DBLCOLON &&
-                         PN_TYPE(kid) != TOK_ANYNAME);
+        bool computed = ((!kid->isKind(TOK_NAME) || !kid->isOp(JSOP_QNAMEPART)) &&
+                         !kid->isKind(TOK_DBLCOLON) &&
+                         !kid->isKind(TOK_ANYNAME));
         return expression(kid, &expr) &&
             builder.xmlAttributeSelector(expr, computed, &pn->pn_pos, dst);
       }
@@ -2753,7 +2696,8 @@ ASTSerializer::expression(JSParseNode *pn, Value *dst)
 bool
 ASTSerializer::xml(JSParseNode *pn, Value *dst)
 {
-    switch (PN_TYPE(pn)) {
+    JS_CHECK_RECURSION(cx, return false);
+    switch (pn->getKind()) {
 #ifdef JS_HAS_XML_SUPPORT
       case TOK_LC:
       {
@@ -2807,10 +2751,10 @@ ASTSerializer::xml(JSParseNode *pn, Value *dst)
         return builder.xmlText(atomContents(pn->pn_atom), &pn->pn_pos, dst);
 
       case TOK_XMLNAME:
-        if (pn->pn_arity == PN_NULLARY)
+        if (pn->isArity(PN_NULLARY))
             return builder.xmlName(atomContents(pn->pn_atom), &pn->pn_pos, dst);
 
-        LOCAL_ASSERT(pn->pn_arity == PN_LIST);
+        LOCAL_ASSERT(pn->isArity(PN_LIST));
 
         {
             NodeVector elts(cx);
@@ -2845,10 +2789,10 @@ ASTSerializer::xml(JSParseNode *pn, Value *dst)
 bool
 ASTSerializer::propertyName(JSParseNode *pn, Value *dst)
 {
-    if (PN_TYPE(pn) == TOK_NAME)
+    if (pn->isKind(TOK_NAME))
         return identifier(pn, dst);
 
-    LOCAL_ASSERT(PN_TYPE(pn) == TOK_STRING || PN_TYPE(pn) == TOK_NUMBER);
+    LOCAL_ASSERT(pn->isKind(TOK_STRING) || pn->isKind(TOK_NUMBER));
 
     return literal(pn, dst);
 }
@@ -2857,7 +2801,7 @@ bool
 ASTSerializer::property(JSParseNode *pn, Value *dst)
 {
     PropKind kind;
-    switch (PN_OP(pn)) {
+    switch (pn->getOp()) {
       case JSOP_INITPROP:
         kind = PROP_INIT;
         break;
@@ -2884,7 +2828,7 @@ bool
 ASTSerializer::literal(JSParseNode *pn, Value *dst)
 {
     Value val;
-    switch (PN_TYPE(pn)) {
+    switch (pn->getKind()) {
       case TOK_STRING:
         val.setString(pn->pn_atom);
         break;
@@ -2911,10 +2855,10 @@ ASTSerializer::literal(JSParseNode *pn, Value *dst)
         break;
 
       case TOK_PRIMARY:
-        if (PN_OP(pn) == JSOP_NULL)
+        if (pn->isOp(JSOP_NULL))
             val.setNull();
         else
-            val.setBoolean(PN_OP(pn) == JSOP_TRUE);
+            val.setBoolean(pn->isOp(JSOP_TRUE));
         break;
 
       default:
@@ -2927,14 +2871,14 @@ ASTSerializer::literal(JSParseNode *pn, Value *dst)
 bool
 ASTSerializer::arrayPattern(JSParseNode *pn, VarDeclKind *pkind, Value *dst)
 {
-    JS_ASSERT(PN_TYPE(pn) == TOK_RB);
+    JS_ASSERT(pn->isKind(TOK_RB));
 
     NodeVector elts(cx);
     if (!elts.reserve(pn->pn_count))
         return false;
 
     for (JSParseNode *next = pn->pn_head; next; next = next->pn_next) {
-        if (PN_TYPE(next) == TOK_COMMA) {
+        if (next->isKind(TOK_COMMA)) {
             elts.infallibleAppend(MagicValue(JS_SERIALIZE_NO_NODE));
         } else {
             Value patt;
@@ -2950,14 +2894,14 @@ ASTSerializer::arrayPattern(JSParseNode *pn, VarDeclKind *pkind, Value *dst)
 bool
 ASTSerializer::objectPattern(JSParseNode *pn, VarDeclKind *pkind, Value *dst)
 {
-    JS_ASSERT(PN_TYPE(pn) == TOK_RC);
+    JS_ASSERT(pn->isKind(TOK_RC));
 
     NodeVector elts(cx);
     if (!elts.reserve(pn->pn_count))
         return false;
 
     for (JSParseNode *next = pn->pn_head; next; next = next->pn_next) {
-        LOCAL_ASSERT(PN_OP(next) == JSOP_INITPROP);
+        LOCAL_ASSERT(next->isOp(JSOP_INITPROP));
 
         Value key, patt, prop;
         if (!propertyName(next->pn_left, &key) ||
@@ -2975,7 +2919,8 @@ ASTSerializer::objectPattern(JSParseNode *pn, VarDeclKind *pkind, Value *dst)
 bool
 ASTSerializer::pattern(JSParseNode *pn, VarDeclKind *pkind, Value *dst)
 {
-    switch (PN_TYPE(pn)) {
+    JS_CHECK_RECURSION(cx, return false);
+    switch (pn->getKind()) {
       case TOK_RC:
         return objectPattern(pn, pkind, dst);
 
@@ -3001,7 +2946,7 @@ ASTSerializer::identifier(JSAtom *atom, TokenPos *pos, Value *dst)
 bool
 ASTSerializer::identifier(JSParseNode *pn, Value *dst)
 {
-    LOCAL_ASSERT(pn->pn_arity == PN_NAME || pn->pn_arity == PN_NULLARY);
+    LOCAL_ASSERT(pn->isArity(PN_NAME) || pn->isArity(PN_NULLARY));
     LOCAL_ASSERT(pn->pn_atom);
 
     return identifier(pn->pn_atom, &pn->pn_pos, dst);
@@ -3032,7 +2977,7 @@ ASTSerializer::function(JSParseNode *pn, ASTType type, Value *dst)
 
     NodeVector args(cx);
 
-    JSParseNode *argsAndBody = (PN_TYPE(pn->pn_body) == TOK_UPVARS)
+    JSParseNode *argsAndBody = pn->pn_body->isKind(TOK_UPVARS)
                                ? pn->pn_body->pn_tree
                                : pn->pn_body;
 
@@ -3048,7 +2993,7 @@ ASTSerializer::functionArgsAndBody(JSParseNode *pn, NodeVector &args, Value *bod
     JSParseNode *pnbody;
 
     /* Extract the args and body separately. */
-    if (PN_TYPE(pn) == TOK_ARGSBODY) {
+    if (pn->isKind(TOK_ARGSBODY)) {
         pnargs = pn;
         pnbody = pn->last();
     } else {
@@ -3059,18 +3004,18 @@ ASTSerializer::functionArgsAndBody(JSParseNode *pn, NodeVector &args, Value *bod
     JSParseNode *pndestruct;
 
     /* Extract the destructuring assignments. */
-    if (pnbody->pn_arity == PN_LIST && (pnbody->pn_xflags & PNX_DESTRUCT)) {
+    if (pnbody->isArity(PN_LIST) && (pnbody->pn_xflags & PNX_DESTRUCT)) {
         JSParseNode *head = pnbody->pn_head;
-        LOCAL_ASSERT(head && PN_TYPE(head) == TOK_SEMI);
+        LOCAL_ASSERT(head && head->isKind(TOK_SEMI));
 
         pndestruct = head->pn_kid;
-        LOCAL_ASSERT(pndestruct && PN_TYPE(pndestruct) == TOK_VAR);
+        LOCAL_ASSERT(pndestruct && pndestruct->isKind(TOK_VAR));
     } else {
         pndestruct = NULL;
     }
 
     /* Serialize the arguments and body. */
-    switch (PN_TYPE(pnbody)) {
+    switch (pnbody->getKind()) {
       case TOK_RETURN: /* expression closure, no destructured args */
         return functionArgs(pn, pnargs, NULL, pnbody, args) &&
                expression(pnbody->pn_kid, body);
@@ -3078,7 +3023,7 @@ ASTSerializer::functionArgsAndBody(JSParseNode *pn, NodeVector &args, Value *bod
       case TOK_SEQ:    /* expression closure with destructured args */
       {
         JSParseNode *pnstart = pnbody->pn_head->pn_next;
-        LOCAL_ASSERT(pnstart && PN_TYPE(pnstart) == TOK_RETURN);
+        LOCAL_ASSERT(pnstart && pnstart->isKind(TOK_RETURN));
 
         return functionArgs(pn, pnargs, pndestruct, pnbody, args) &&
                expression(pnstart->pn_kid, body);
@@ -3162,20 +3107,6 @@ ASTSerializer::functionBody(JSParseNode *pn, TokenPos *pos, Value *dst)
 
 } /* namespace js */
 
-/* Reflect class */
-
-Class js_ReflectClass = {
-    js_Reflect_str,
-    JSCLASS_HAS_CACHED_PROTO(JSProto_Reflect),
-    PropertyStub,
-    PropertyStub,
-    PropertyStub,
-    StrictPropertyStub,
-    EnumerateStub,
-    ResolveStub,
-    ConvertStub
-};
-
 static JSBool
 reflect_parse(JSContext *cx, uint32 argc, jsval *vp)
 {
@@ -3185,7 +3116,7 @@ reflect_parse(JSContext *cx, uint32 argc, jsval *vp)
         return JS_FALSE;
     }
 
-    JSString *src = js_ValueToString(cx, Valueify(JS_ARGV(cx, vp)[0]));
+    JSString *src = js_ValueToString(cx, JS_ARGV(cx, vp)[0]);
     if (!src)
         return JS_FALSE;
 
@@ -3196,7 +3127,7 @@ reflect_parse(JSContext *cx, uint32 argc, jsval *vp)
 
     JSObject *builder = NULL;
 
-    Value arg = argc > 1 ? Valueify(JS_ARGV(cx, vp)[1]) : UndefinedValue();
+    Value arg = argc > 1 ? JS_ARGV(cx, vp)[1] : UndefinedValue();
 
     if (!arg.isNullOrUndefined()) {
         if (!arg.isObject()) {
@@ -3234,7 +3165,7 @@ reflect_parse(JSContext *cx, uint32 argc, jsval *vp)
                 if (!chars)
                     return JS_FALSE;
 
-                filename = js_DeflateString(cx, chars, length);
+                filename = DeflateString(cx, chars, length);
                 if (!filename)
                     return JS_FALSE;
                 filenamep.reset(filename);
@@ -3274,7 +3205,7 @@ reflect_parse(JSContext *cx, uint32 argc, jsval *vp)
     if (!chars)
         return JS_FALSE;
 
-    Parser parser(cx);
+    Parser parser(cx, NULL, NULL, false);
 
     if (!parser.init(chars, length, filename, lineno, cx->findVersion()))
         return JS_FALSE;
@@ -3289,7 +3220,7 @@ reflect_parse(JSContext *cx, uint32 argc, jsval *vp)
         return JS_FALSE;
     }
 
-    JS_SET_RVAL(cx, vp, Jsvalify(val));
+    JS_SET_RVAL(cx, vp, val);
     return JS_TRUE;
 }
 
@@ -3299,14 +3230,16 @@ static JSFunctionSpec static_methods[] = {
 };
 
 
-JSObject *
-js_InitReflectClass(JSContext *cx, JSObject *obj)
+JS_BEGIN_EXTERN_C
+
+JS_PUBLIC_API(JSObject *)
+JS_InitReflect(JSContext *cx, JSObject *obj)
 {
-    JSObject *Reflect = NewNonFunction<WithProto::Class>(cx, &js_ReflectClass, NULL, obj);
-    if (!Reflect)
+    JSObject *Reflect = NewNonFunction<WithProto::Class>(cx, &ObjectClass, NULL, obj);
+    if (!Reflect || !Reflect->setSingletonType(cx))
         return NULL;
 
-    if (!JS_DefineProperty(cx, obj, js_Reflect_str, OBJECT_TO_JSVAL(Reflect),
+    if (!JS_DefineProperty(cx, obj, "Reflect", OBJECT_TO_JSVAL(Reflect),
                            JS_PropertyStub, JS_StrictPropertyStub, 0)) {
         return NULL;
     }
@@ -3314,7 +3247,7 @@ js_InitReflectClass(JSContext *cx, JSObject *obj)
     if (!JS_DefineFunctions(cx, Reflect, static_methods))
         return NULL;
 
-    MarkStandardClassInitializedNoProto(obj, &js_ReflectClass);
-
     return Reflect;
 }
+
+JS_END_EXTERN_C
