@@ -53,17 +53,28 @@
 namespace js {
 namespace mjit {
 
+/*
+ * Patch for storing call site and rejoin site return addresses at, for
+ * redirecting the return address in InvariantFailure.
+ */
+struct InvariantCodePatch {
+    bool hasPatch;
+    JSC::MacroAssembler::DataLabelPtr codePatch;
+    InvariantCodePatch() : hasPatch(false) {}
+};
+
 class Compiler : public BaseCompiler
 {
     friend class StubCompiler;
 
     struct BranchPatch {
-        BranchPatch(const Jump &j, jsbytecode *pc)
-          : jump(j), pc(pc)
+        BranchPatch(const Jump &j, jsbytecode *pc, uint32 inlineIndex)
+          : jump(j), pc(pc), inlineIndex(inlineIndex)
         { }
 
         Jump jump;
         jsbytecode *pc;
+        uint32 inlineIndex;
     };
 
 #if defined JS_MONOIC
@@ -109,6 +120,8 @@ class Compiler : public BaseCompiler
         MaybeJump jumpToStub;
         Label fallThrough;
         jsbytecode *jumpTarget;
+        bool trampoline;
+        Label trampolineStart;
         ValueRemat lvr, rvr;
         Assembler::Condition cond;
         JSC::MacroAssembler::RegisterID tempReg;
@@ -119,6 +132,8 @@ class Compiler : public BaseCompiler
         Label stubEntry;
         DataLabelPtr addrLabel;
         jsbytecode *jumpTarget;
+        bool fastTrampoline;
+        Label trampolineStart;
         Jump traceHint;
         MaybeJump slowTraceHint;
 
@@ -128,13 +143,11 @@ class Compiler : public BaseCompiler
     /* InlineFrameAssembler wants to see this. */
   public:
     struct CallGenInfo {
-        CallGenInfo(jsbytecode *pc) : pc(pc) {}
-
         /*
          * These members map to members in CallICInfo. See that structure for
          * more comments.
          */
-        jsbytecode   *pc;
+        uint32       callIndex;
         DataLabelPtr funGuard;
         Jump         funJump;
         Jump         hotJump;
@@ -150,6 +163,7 @@ class Compiler : public BaseCompiler
         RegisterID   funObjReg;
         RegisterID   funPtrReg;
         FrameSize    frameSize;
+        bool         typeMonitored;
     };
 
   private:
@@ -160,16 +174,17 @@ class Compiler : public BaseCompiler
      * absolute address of the join point is known.
      */
     struct CallPatchInfo {
-        CallPatchInfo() : hasFastNcode(false), hasSlowNcode(false) {}
+        CallPatchInfo() : hasFastNcode(false), hasSlowNcode(false), joinSlow(false) {}
         Label joinPoint;
         DataLabelPtr fastNcodePatch;
         DataLabelPtr slowNcodePatch;
         bool hasFastNcode;
         bool hasSlowNcode;
+        bool joinSlow;
     };
 
     struct BaseICInfo {
-        BaseICInfo(JSOp op) : op(op)
+        BaseICInfo(JSOp op) : op(op), canCallHook(false), forcedTypeBarrier(false)
         { }
         Label fastPathStart;
         Label fastPathRejoin;
@@ -177,12 +192,16 @@ class Compiler : public BaseCompiler
         Call slowPathCall;
         DataLabelPtr paramAddr;
         JSOp op;
+        bool canCallHook;
+        bool forcedTypeBarrier;
 
         void copyTo(ic::BaseIC &to, JSC::LinkBuffer &full, JSC::LinkBuffer &stub) {
             to.fastPathStart = full.locationOf(fastPathStart);
             to.fastPathRejoin = full.locationOf(fastPathRejoin);
             to.slowPathStart = stub.locationOf(slowPathStart);
             to.slowPathCall = stub.locationOf(slowPathCall);
+            to.canCallHook = canCallHook;
+            to.forcedTypeBarrier = forcedTypeBarrier;
             to.op = op;
             JS_ASSERT(to.op == op);
         }
@@ -213,7 +232,7 @@ class Compiler : public BaseCompiler
 
     struct PICGenInfo : public BaseICInfo {
         PICGenInfo(ic::PICInfo::Kind kind, JSOp op, bool usePropCache)
-          : BaseICInfo(op), kind(kind), usePropCache(usePropCache)
+          : BaseICInfo(op), kind(kind), usePropCache(usePropCache), typeMonitored(false)
         { }
         ic::PICInfo::Kind kind;
         Label typeCheck;
@@ -225,6 +244,8 @@ class Compiler : public BaseCompiler
         jsbytecode *pc;
         JSAtom *atom;
         bool hasTypeCheck;
+        bool typeMonitored;
+        types::TypeSet *rhsTypes;
         ValueRemat vr;
 #ifdef JS_HAS_IC_LABELS
         union {
@@ -247,7 +268,8 @@ class Compiler : public BaseCompiler
             return bindNameLabels_;
         }
         ic::ScopeNameLabels &scopeNameLabels() {
-            JS_ASSERT(kind == ic::PICInfo::NAME || kind == ic::PICInfo::XNAME);
+            JS_ASSERT(kind == ic::PICInfo::NAME || kind == ic::PICInfo::CALLNAME ||
+                      kind == ic::PICInfo::XNAME);
             return scopeNameLabels_;
         }
 #else
@@ -264,7 +286,8 @@ class Compiler : public BaseCompiler
             return ic::PICInfo::bindNameLabels_;
         }
         ic::ScopeNameLabels &scopeNameLabels() {
-            JS_ASSERT(kind == ic::PICInfo::NAME || kind == ic::PICInfo::XNAME);
+            JS_ASSERT(kind == ic::PICInfo::NAME || kind == ic::PICInfo::CALLNAME ||
+                      kind == ic::PICInfo::XNAME);
             return ic::PICInfo::scopeNameLabels_;
         }
 #endif
@@ -281,6 +304,8 @@ class Compiler : public BaseCompiler
                 ic.u.get.typeReg = typeReg;
                 ic.u.get.hasTypeCheck = hasTypeCheck;
             }
+            ic.typeMonitored = typeMonitored;
+            ic.rhsTypes = rhsTypes;
 #ifdef JS_HAS_IC_LABELS
             if (ic.isGet())
                 ic.setLabels(getPropLabels());
@@ -304,14 +329,20 @@ class Compiler : public BaseCompiler
 
     struct InternalCallSite {
         uint32 returnOffset;
-        jsbytecode *pc;
-        uint32 id;
-        bool call;
+        DataLabelPtr inlinePatch;
+        uint32 inlineIndex;
+        jsbytecode *inlinepc;
+        RejoinState rejoin;
         bool ool;
+        Label loopJumpLabel;
+        InvariantCodePatch loopPatch;
 
-        InternalCallSite(uint32 returnOffset, jsbytecode *pc, uint32 id,
-                         bool call, bool ool)
-          : returnOffset(returnOffset), pc(pc), id(id), call(call), ool(ool)
+        InternalCallSite(uint32 returnOffset,
+                         uint32 inlineIndex, jsbytecode *inlinepc,
+                         RejoinState rejoin, bool ool)
+          : returnOffset(returnOffset),
+            inlineIndex(inlineIndex), inlinepc(inlinepc),
+            rejoin(rejoin), ool(ool)
         { }
     };
 
@@ -326,18 +357,85 @@ class Compiler : public BaseCompiler
         size_t offsetIndex;
     };
 
-    StackFrame *fp;
-    JSScript *script;
-    JSObject *scopeChain;
-    JSObject *globalObj;
-    JSFunction *fun;
+    struct LoopEntry {
+        uint32 pcOffset;
+        Label label;
+    };
+
+    struct VarType {
+        JSValueType type;
+        types::TypeSet *types;
+    };
+
+    struct SlotType
+    {
+        uint32 slot;
+        VarType vt;
+        SlotType(uint32 slot, VarType vt) : slot(slot), vt(vt) {}
+    };
+
+    JSScript *outerScript;
     bool isConstructing;
-    analyze::Script *analysis;
-    Label *jumpMap;
-    bool *savedTraps;
-    jsbytecode *PC;
+
+    /* SSA information for the outer script and all frames we will be inlining. */
+    analyze::CrossScriptSSA ssa;
+
+    GlobalObject *globalObj;
+    const Value *globalSlots;  /* Original slots pointer. */
+
     Assembler masm;
     FrameState frame;
+
+    /*
+     * State for the current stack frame, and links to its parents going up to
+     * the outermost script.
+     */
+
+    struct ActiveFrame {
+        ActiveFrame *parent;
+        jsbytecode *parentPC;
+        JSScript *script;
+        Label *jumpMap;
+
+        /*
+         * Index into inlineFrames or OUTER_FRAME, matches this frame's index
+         * in the cross script SSA.
+         */
+        uint32 inlineIndex;
+
+        /* Current types for non-escaping vars in the script. */
+        VarType *varTypes;
+
+        /* State for managing return from inlined frames. */
+        bool needReturnValue;          /* Return value will be used. */
+        bool syncReturnValue;          /* Return value should be fully synced. */
+        bool returnValueDouble;        /* Return value should be a double. */
+        bool returnSet;                /* Whether returnRegister is valid. */
+        AnyRegisterID returnRegister;  /* Register holding return value. */
+        const FrameEntry *returnEntry; /* Entry copied by return value. */
+        Vector<Jump, 4, CompilerAllocPolicy> *returnJumps;
+
+        /*
+         * Snapshot of the heap state to use after the call, in case
+         * there are multiple return paths the inlined frame could take.
+         */
+        RegisterAllocation *exitState;
+
+        ActiveFrame(JSContext *cx);
+        ~ActiveFrame();
+    };
+    ActiveFrame *a;
+    ActiveFrame *outer;
+
+    JSScript *script;
+    analyze::ScriptAnalysis *analysis;
+    jsbytecode *PC;
+
+    LoopState *loop;
+
+    /* State spanning all stack frames. */
+
+    js::Vector<ActiveFrame*, 4, CompilerAllocPolicy> inlineFrames;
     js::Vector<BranchPatch, 64, CompilerAllocPolicy> branchPatches;
 #if defined JS_MONOIC
     js::Vector<GetGlobalNameICInfo, 16, CompilerAllocPolicy> getGlobalNames;
@@ -354,42 +452,83 @@ class Compiler : public BaseCompiler
     js::Vector<CallPatchInfo, 64, CompilerAllocPolicy> callPatches;
     js::Vector<InternalCallSite, 64, CompilerAllocPolicy> callSites;
     js::Vector<DoublePatch, 16, CompilerAllocPolicy> doubleList;
+    js::Vector<uint32> fixedIntToDoubleEntries;
+    js::Vector<uint32> fixedDoubleToAnyEntries;
     js::Vector<JumpTable, 16> jumpTables;
     js::Vector<uint32, 16> jumpTableOffsets;
+    js::Vector<LoopEntry, 16> loopEntries;
+    js::Vector<JSObject *, 0, CompilerAllocPolicy> rootedObjects;
     StubCompiler stubcc;
     Label invokeLabel;
     Label arityLabel;
+    Label argsCheckLabel;
+#ifdef JS_MONOIC
+    Label argsCheckStub;
+    Label argsCheckFallthrough;
+    Jump argsCheckJump;
+#endif
     bool debugMode_;
     bool addTraceHints;
+    bool inlining_;
+    bool hasGlobalReallocation;
     bool oomInVector;       // True if we have OOM'd appending to a vector. 
+    bool overflowICSpace;   // True if we added a constant pool in a reserved space.
+    uint32 gcNumber;
     enum { NoApplyTricks, LazyArgsObj } applyTricks;
-#ifdef DEBUG
-    int *pcProfile;
-#endif
+    PCLengthEntry *pcLengths;
 
     Compiler *thisFromCtor() { return this; }
 
     friend class CompilerAllocPolicy;
   public:
-    // Special atom index used to indicate that the atom is 'length'. This
-    // follows interpreter usage in JSOP_LENGTH.
-    enum { LengthAtomIndex = uint32(-2) };
-
-    Compiler(JSContext *cx, StackFrame *fp);
+    Compiler(JSContext *cx, JSScript *outerScript, bool isConstructing);
     ~Compiler();
 
     CompileStatus compile();
 
-    jsbytecode *getPC() { return PC; }
     Label getLabel() { return masm.label(); }
     bool knownJump(jsbytecode *pc);
-    Label labelOf(jsbytecode *target);
-    void *findCallSite(const CallSite &callSite);
+    Label labelOf(jsbytecode *target, uint32 inlineIndex);
     void addCallSite(const InternalCallSite &callSite);
-    void addReturnSite(Label joinPoint, uint32 id);
-    bool loadOldTraps(const Vector<CallSite> &site);
+    void addReturnSite();
+    void inlineStubCall(void *stub, RejoinState rejoin, Uses uses);
 
     bool debugMode() { return debugMode_; }
+    bool inlining() { return inlining_; }
+    bool constructing() { return isConstructing; }
+
+    jsbytecode *outerPC() {
+        if (a == outer)
+            return PC;
+        ActiveFrame *scan = a;
+        while (scan && scan->parent != outer)
+            scan = scan->parent;
+        return scan->parentPC;
+    }
+
+    jsbytecode *inlinePC() { return PC; }
+    uint32 inlineIndex() { return a->inlineIndex; }
+
+    Assembler &getAssembler(bool ool) { return ool ? stubcc.masm : masm; }
+
+    InvariantCodePatch *getInvariantPatch(unsigned index) {
+        return &callSites[index].loopPatch;
+    }
+    jsbytecode *getInvariantPC(unsigned index) {
+        return callSites[index].inlinepc;
+    }
+
+    bool arrayPrototypeHasIndexedProperty();
+
+    bool activeFrameHasMultipleExits() {
+        ActiveFrame *na = a;
+        while (na->parent) {
+            if (na->exitState)
+                return true;
+            na = na->parent;
+        }
+        return false;
+    }
 
   private:
     CompileStatus performCompilation(JITScript **jitp);
@@ -397,21 +536,61 @@ class Compiler : public BaseCompiler
     CompileStatus generateMethod();
     CompileStatus generateEpilogue();
     CompileStatus finishThisUp(JITScript **jitp);
+    CompileStatus pushActiveFrame(JSScript *script, uint32 argc);
+    void popActiveFrame();
+    void updatePCCounters(jsbytecode *pc, Label *start, bool *updated);
+
+    /* Analysis helpers. */
+    CompileStatus prepareInferenceTypes(JSScript *script, ActiveFrame *a);
+    void ensureDoubleArguments();
+    void fixDoubleTypes(jsbytecode *target);
+    void watchGlobalReallocation();
+    void updateVarType();
+    void updateJoinVarTypes();
+    void restoreVarType();
+    JSValueType knownPushedType(uint32 pushed);
+    bool mayPushUndefined(uint32 pushed);
+    types::TypeSet *pushedTypeSet(uint32 which);
+    bool monitored(jsbytecode *pc);
+    bool hasTypeBarriers(jsbytecode *pc);
+    bool testSingletonProperty(JSObject *obj, jsid id);
+    bool testSingletonPropertyTypes(FrameEntry *top, jsid id, bool *testObject);
+    CompileStatus addInlineFrame(JSScript *script, uint32 depth, uint32 parent, jsbytecode *parentpc);
+    CompileStatus scanInlineCalls(uint32 index, uint32 depth);
+    CompileStatus checkAnalysis(JSScript *script);
+
+    struct BarrierState {
+        MaybeJump jump;
+        RegisterID typeReg;
+        RegisterID dataReg;
+    };
+
+    MaybeJump trySingleTypeTest(types::TypeSet *types, RegisterID typeReg);
+    Jump addTypeTest(types::TypeSet *types, RegisterID typeReg, RegisterID dataReg);
+    BarrierState pushAddressMaybeBarrier(Address address, JSValueType type, bool reuseBase,
+                                         bool testUndefined = false);
+    BarrierState testBarrier(RegisterID typeReg, RegisterID dataReg,
+                             bool testUndefined = false, bool testReturn = false,
+                             bool force = false);
+    void finishBarrier(const BarrierState &barrier, RejoinState rejoin, uint32 which);
+
+    void testPushedType(RejoinState rejoin, int which, bool ool = true);
 
     /* Non-emitting helpers. */
+    void pushSyncedEntry(uint32 pushed);
     uint32 fullAtomIndex(jsbytecode *pc);
     bool jumpInScript(Jump j, jsbytecode *pc);
     bool compareTwoValues(JSContext *cx, JSOp op, const Value &lhs, const Value &rhs);
     bool canUseApplyTricks();
 
     /* Emitting helpers. */
-    void restoreFrameRegs(Assembler &masm);
+    bool constantFoldBranch(jsbytecode *target, bool taken);
     bool emitStubCmpOp(BoolStub stub, jsbytecode *target, JSOp fused);
     bool iter(uintN flags);
-    void iterNext();
-    bool iterMore();
+    void iterNext(ptrdiff_t offset);
+    bool iterMore(jsbytecode *target);
     void iterEnd();
-    MaybeJump loadDouble(FrameEntry *fe, FPRegisterID fpReg);
+    MaybeJump loadDouble(FrameEntry *fe, FPRegisterID *fpReg, bool *allocated);
 #ifdef JS_POLYIC
     void passICAddress(BaseICInfo *ic);
 #endif
@@ -419,12 +598,23 @@ class Compiler : public BaseCompiler
     void passMICAddress(GlobalNameICInfo &mic);
 #endif
     bool constructThis();
+    void ensureDouble(FrameEntry *fe);
+
+    /*
+     * Ensure fe is an integer, truncating from double if necessary, or jump to
+     * the slow path per uses.
+     */
+    void ensureInteger(FrameEntry *fe, Uses uses);
+
+    /* Convert fe from a double to integer (per ValueToECMAInt32) in place. */
+    void truncateDoubleToInt32(FrameEntry *fe, Uses uses);
 
     /* Opcode handlers. */
-    bool jumpAndTrace(Jump j, jsbytecode *target, Jump *slow = NULL);
+    bool jumpAndTrace(Jump j, jsbytecode *target, Jump *slow = NULL, bool *trampoline = NULL);
+    bool startLoop(jsbytecode *head, Jump entry, jsbytecode *entryTarget);
+    bool finishLoop(jsbytecode *head);
     void jsop_bindname(JSAtom *atom, bool usePropCache);
     void jsop_setglobal(uint32 index);
-    void jsop_getglobal(uint32 index);
     void jsop_getprop_slow(JSAtom *atom, bool usePropCache = true);
     void jsop_getarg(uint32 slot);
     void jsop_setarg(uint32 slot, bool popped);
@@ -433,56 +623,55 @@ class Compiler : public BaseCompiler
     void emitFinalReturn(Assembler &masm);
     void loadReturnValue(Assembler *masm, FrameEntry *fe);
     void emitReturnValue(Assembler *masm, FrameEntry *fe);
+    void emitInlineReturnValue(FrameEntry *fe);
     void dispatchCall(VoidPtrStubUInt32 stub, uint32 argc);
     void interruptCheckHelper();
+    void recompileCheckHelper();
     void emitUncachedCall(uint32 argc, bool callingNew);
     void checkCallApplySpeculation(uint32 callImmArgc, uint32 speculatedArgc,
                                    FrameEntry *origCallee, FrameEntry *origThis,
                                    MaybeRegisterID origCalleeType, RegisterID origCalleeData,
                                    MaybeRegisterID origThisType, RegisterID origThisData,
                                    Jump *uncachedCallSlowRejoin, CallPatchInfo *uncachedCallPatch);
-    void inlineCallHelper(uint32 argc, bool callingNew);
+    bool inlineCallHelper(uint32 argc, bool callingNew, FrameSize &callFrameSize);
     void fixPrimitiveReturn(Assembler *masm, FrameEntry *fe);
-    void jsop_gnameinc(JSOp op, VoidStubAtom stub, uint32 index);
-    bool jsop_nameinc(JSOp op, VoidStubAtom stub, uint32 index);
-    bool jsop_propinc(JSOp op, VoidStubAtom stub, uint32 index);
-    void jsop_eleminc(JSOp op, VoidStub);
     void jsop_getgname(uint32 index);
     void jsop_getgname_slow(uint32 index);
     void jsop_callgname_epilogue();
-    void jsop_setgname(JSAtom *atom, bool usePropertyCache);
+    void jsop_setgname(JSAtom *atom, bool usePropertyCache, bool popGuaranteed);
     void jsop_setgname_slow(JSAtom *atom, bool usePropertyCache);
     void jsop_bindgname();
     void jsop_setelem_slow();
     void jsop_getelem_slow();
     void jsop_callelem_slow();
     void jsop_unbrand();
-    bool jsop_getprop(JSAtom *atom, bool typeCheck = true, bool usePropCache = true);
-    bool jsop_length();
-    bool jsop_setprop(JSAtom *atom, bool usePropCache = true);
+    bool jsop_getprop(JSAtom *atom, JSValueType type,
+                      bool typeCheck = true, bool usePropCache = true);
+    bool jsop_setprop(JSAtom *atom, bool usePropCache, bool popGuaranteed);
     void jsop_setprop_slow(JSAtom *atom, bool usePropCache = true);
     bool jsop_callprop_slow(JSAtom *atom);
     bool jsop_callprop(JSAtom *atom);
     bool jsop_callprop_obj(JSAtom *atom);
     bool jsop_callprop_str(JSAtom *atom);
     bool jsop_callprop_generic(JSAtom *atom);
+    bool jsop_callprop_dispatch(JSAtom *atom);
     bool jsop_instanceof();
-    void jsop_name(JSAtom *atom);
+    void jsop_name(JSAtom *atom, JSValueType type, bool isCall);
     bool jsop_xname(JSAtom *atom);
     void enterBlock(JSObject *obj);
     void leaveBlock();
     void emitEval(uint32 argc);
-    void jsop_arguments();
+    void jsop_arguments(RejoinState rejoin);
     bool jsop_tableswitch(jsbytecode *pc);
-    void jsop_forprop(JSAtom *atom);
-    void jsop_forname(JSAtom *atom);
-    void jsop_forgname(JSAtom *atom);
 
     /* Fast arithmetic. */
-    void jsop_binary(JSOp op, VoidStub stub);
-    void jsop_binary_full(FrameEntry *lhs, FrameEntry *rhs, JSOp op, VoidStub stub);
-    void jsop_binary_full_simple(FrameEntry *fe, JSOp op, VoidStub stub);
-    void jsop_binary_double(FrameEntry *lhs, FrameEntry *rhs, JSOp op, VoidStub stub);
+    bool jsop_binary(JSOp op, VoidStub stub, JSValueType type, types::TypeSet *typeSet);
+    void jsop_binary_full(FrameEntry *lhs, FrameEntry *rhs, JSOp op, VoidStub stub,
+                          JSValueType type, bool cannotOverflow, bool ignoreOverflow);
+    void jsop_binary_full_simple(FrameEntry *fe, JSOp op, VoidStub stub,
+                                 JSValueType type);
+    void jsop_binary_double(FrameEntry *lhs, FrameEntry *rhs, JSOp op, VoidStub stub,
+                            JSValueType type);
     void slowLoadConstantDouble(Assembler &masm, FrameEntry *fe,
                                 FPRegisterID fpreg);
     void maybeJumpIfNotInt32(Assembler &masm, MaybeJump &mj, FrameEntry *fe,
@@ -490,9 +679,9 @@ class Compiler : public BaseCompiler
     void maybeJumpIfNotDouble(Assembler &masm, MaybeJump &mj, FrameEntry *fe,
                               MaybeRegisterID &mreg);
     bool jsop_relational(JSOp op, BoolStub stub, jsbytecode *target, JSOp fused);
-    bool jsop_relational_self(JSOp op, BoolStub stub, jsbytecode *target, JSOp fused);
     bool jsop_relational_full(JSOp op, BoolStub stub, jsbytecode *target, JSOp fused);
     bool jsop_relational_double(JSOp op, BoolStub stub, jsbytecode *target, JSOp fused);
+    bool jsop_relational_int(JSOp op, jsbytecode *target, JSOp fused);
 
     void emitLeftDoublePath(FrameEntry *lhs, FrameEntry *rhs, FrameState::BinaryAlloc &regs,
                             MaybeJump &lhsNotDouble, MaybeJump &rhsNotNumber,
@@ -500,21 +689,11 @@ class Compiler : public BaseCompiler
     void emitRightDoublePath(FrameEntry *lhs, FrameEntry *rhs, FrameState::BinaryAlloc &regs,
                              MaybeJump &rhsNotNumber2);
     bool tryBinaryConstantFold(JSContext *cx, FrameState &frame, JSOp op,
-                               FrameEntry *lhs, FrameEntry *rhs);
+                               FrameEntry *lhs, FrameEntry *rhs, Value *vp);
 
     /* Fast opcodes. */
     void jsop_bitop(JSOp op);
-    void jsop_rsh();
-    RegisterID rightRegForShift(FrameEntry *rhs);
-    void jsop_rsh_int_int(FrameEntry *lhs, FrameEntry *rhs);
-    void jsop_rsh_const_int(FrameEntry *lhs, FrameEntry *rhs);
-    void jsop_rsh_int_const(FrameEntry *lhs, FrameEntry *rhs);
-    void jsop_rsh_int_unknown(FrameEntry *lhs, FrameEntry *rhs);
-    void jsop_rsh_const_const(FrameEntry *lhs, FrameEntry *rhs);
-    void jsop_rsh_const_unknown(FrameEntry *lhs, FrameEntry *rhs);
-    void jsop_rsh_unknown_const(FrameEntry *lhs, FrameEntry *rhs);
-    void jsop_rsh_unknown_any(FrameEntry *lhs, FrameEntry *rhs);
-    void jsop_mod();
+    bool jsop_mod();
     void jsop_neg();
     void jsop_bitnot();
     void jsop_not();
@@ -522,46 +701,105 @@ class Compiler : public BaseCompiler
     bool booleanJumpScript(JSOp op, jsbytecode *target);
     bool jsop_ifneq(JSOp op, jsbytecode *target);
     bool jsop_andor(JSOp op, jsbytecode *target);
-    void jsop_arginc(JSOp op, uint32 slot, bool popped);
-    void jsop_localinc(JSOp op, uint32 slot, bool popped);
-    void jsop_newinit();
+    bool jsop_arginc(JSOp op, uint32 slot);
+    bool jsop_localinc(JSOp op, uint32 slot);
+    bool jsop_newinit();
     void jsop_initmethod();
     void jsop_initprop();
     void jsop_initelem();
+    void jsop_setelem_dense();
+#ifdef JS_METHODJIT_TYPED_ARRAY
+    void jsop_setelem_typed(int atype);
+    void convertForTypedArray(int atype, ValueRemat *vr, bool *allocated);
+#endif
     bool jsop_setelem(bool popGuaranteed);
     bool jsop_getelem(bool isCall);
+    void jsop_getelem_dense(bool isPacked);
+    void jsop_getelem_args();
+#ifdef JS_METHODJIT_TYPED_ARRAY
+    bool jsop_getelem_typed(int atype);
+#endif
+    void jsop_toid();
     bool isCacheableBaseAndIndex(FrameEntry *obj, FrameEntry *id);
     void jsop_stricteq(JSOp op);
     bool jsop_equality(JSOp op, BoolStub stub, jsbytecode *target, JSOp fused);
     bool jsop_equality_int_string(JSOp op, BoolStub stub, jsbytecode *target, JSOp fused);
     void jsop_pos();
 
-   
+    static inline Assembler::Condition
+    GetCompareCondition(JSOp op, JSOp fused)
+    {
+        bool ifeq = fused == JSOP_IFEQ;
+        switch (op) {
+          case JSOP_GT:
+            return ifeq ? Assembler::LessThanOrEqual : Assembler::GreaterThan;
+          case JSOP_GE:
+            return ifeq ? Assembler::LessThan : Assembler::GreaterThanOrEqual;
+          case JSOP_LT:
+            return ifeq ? Assembler::GreaterThanOrEqual : Assembler::LessThan;
+          case JSOP_LE:
+            return ifeq ? Assembler::GreaterThan : Assembler::LessThanOrEqual;
+          case JSOP_EQ:
+            return ifeq ? Assembler::NotEqual : Assembler::Equal;
+          case JSOP_NE:
+            return ifeq ? Assembler::Equal : Assembler::NotEqual;
+          default:
+            JS_NOT_REACHED("unrecognized op");
+            return Assembler::Equal;
+        }
+    }
+
+    static inline Assembler::Condition
+    GetStubCompareCondition(JSOp fused)
+    {
+        return (fused == JSOP_IFEQ) ? Assembler::Zero : Assembler::NonZero;
+    }
+
+    /* Fast builtins. */
+    JSObject *pushedSingleton(unsigned pushed);
+    CompileStatus callArrayBuiltin(uint32 argc, bool callingNew);
+    CompileStatus inlineNativeFunction(uint32 argc, bool callingNew);
+    CompileStatus inlineScriptedFunction(uint32 argc, bool callingNew);
+    CompileStatus compileMathAbsInt(FrameEntry *arg);
+    CompileStatus compileMathAbsDouble(FrameEntry *arg);
+    CompileStatus compileMathSqrt(FrameEntry *arg);
+    CompileStatus compileMathMinMaxDouble(FrameEntry *arg1, FrameEntry *arg2, 
+                                          Assembler::DoubleCondition cond); 
+    CompileStatus compileMathMinMaxInt(FrameEntry *arg1, FrameEntry *arg2, 
+                                       Assembler::Condition cond);                                       
+    CompileStatus compileMathPowSimple(FrameEntry *arg1, FrameEntry *arg2);
+    CompileStatus compileArrayPush(FrameEntry *thisv, FrameEntry *arg);
+    CompileStatus compileArrayPop(FrameEntry *thisv, bool isPacked);
+    CompileStatus compileArrayWithLength(uint32 argc);
+    CompileStatus compileArrayWithArgs(uint32 argc);
+
+    enum RoundingMode { Floor, Round };
+    CompileStatus compileRound(FrameEntry *arg, RoundingMode mode);
+
+    enum GetCharMode { GetChar, GetCharCode };
+    CompileStatus compileGetChar(FrameEntry *thisValue, FrameEntry *arg, GetCharMode mode);
+
     void prepareStubCall(Uses uses);
-    Call emitStubCall(void *ptr);
+    Call emitStubCall(void *ptr, DataLabelPtr *pinline);
 };
 
-// Given a stub call, emits the call into the inline assembly path. If
-// debug mode is on, adds the appropriate instrumentation for recompilation.
-#define INLINE_STUBCALL(stub)                                               \
-    do {                                                                    \
-        Call cl = emitStubCall(JS_FUNC_TO_DATA_PTR(void *, (stub)));        \
-        if (debugMode()) {                                                  \
-            InternalCallSite site(masm.callReturnOffset(cl), PC, __LINE__,  \
-                                  true, false);                             \
-            addCallSite(site);                                              \
-        }                                                                   \
-    } while (0)                                                             \
+// Given a stub call, emits the call into the inline assembly path. rejoin
+// indicates how to rejoin should this call trigger expansion/discarding.
+#define INLINE_STUBCALL(stub, rejoin)                                       \
+    inlineStubCall(JS_FUNC_TO_DATA_PTR(void *, (stub)), rejoin, Uses(0))
+#define INLINE_STUBCALL_USES(stub, rejoin, uses)                            \
+    inlineStubCall(JS_FUNC_TO_DATA_PTR(void *, (stub)), rejoin, uses)
 
-// Given a stub call, emits the call into the out-of-line assembly path. If
-// debug mode is on, adds the appropriate instrumentation for recompilation.
+// Given a stub call, emits the call into the out-of-line assembly path.
 // Unlike the INLINE_STUBCALL variant, this returns the Call offset.
-#define OOL_STUBCALL(stub)                                                      \
-    stubcc.emitStubCall(JS_FUNC_TO_DATA_PTR(void *, (stub)), __LINE__)          \
+#define OOL_STUBCALL(stub, rejoin)                                          \
+    stubcc.emitStubCall(JS_FUNC_TO_DATA_PTR(void *, (stub)), rejoin, Uses(0))
+#define OOL_STUBCALL_USES(stub, rejoin, uses)                               \
+    stubcc.emitStubCall(JS_FUNC_TO_DATA_PTR(void *, (stub)), rejoin, uses)
 
 // Same as OOL_STUBCALL, but specifies a slot depth.
-#define OOL_STUBCALL_LOCAL_SLOTS(stub, slots)                                   \
-    stubcc.emitStubCall(JS_FUNC_TO_DATA_PTR(void *, (stub)), (slots), __LINE__) \
+#define OOL_STUBCALL_LOCAL_SLOTS(stub, rejoin, slots)                       \
+    stubcc.emitStubCall(JS_FUNC_TO_DATA_PTR(void *, (stub)), rejoin, Uses(0), (slots))
 
 } /* namespace js */
 } /* namespace mjit */
