@@ -1,4 +1,4 @@
-/* -*- Mode: java; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2005
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Brendan Eich
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,22 +35,26 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+//-----------------------------------------------------------------------------
+var BUGNUMBER = 460886;
+var summary = 'Do not crash @ js_NewStringCopy';
+var actual = '';
+var expect = '';
 
-var BUGNUMBER = 311157;
-var summary = 'Comment-hiding compromise left E4X parsing/scanning inconsistent';
-var actual = 'No Crash';
-var expect = 'No Crash';
 
-printBugNumber(BUGNUMBER);
-printStatus (summary);
+//-----------------------------------------------------------------------------
+test();
+//-----------------------------------------------------------------------------
 
-try
+function test()
 {
-  eval('var x = <hi> <![CDATA[ duh ]]>\n     there </hi>');
-}
-catch(e)
-{
-}
+  enterFunc ('test');
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
 
-reportCompare(expect, actual, summary);
+  for (var j = 0; j < 5; ++j) { "".substring(-60000); } 
+ 
+  reportCompare(expect, actual, summary);
 
+  exitFunc ('test');
+}
