@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,12 +36,11 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 455464;
-var summary = 'Do not assert with JIT: !TRACE_RECORDER(cx) ^ (jumpTable == recordingJumpTable)';
+var BUGNUMBER = 355052;
+var summary = 'Do not crash with valueOf:gc and __iterator__';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
-a=b=c=d=0; this.__defineGetter__('g', gc); for each (y in this);
 
 //-----------------------------------------------------------------------------
 test();
@@ -53,13 +52,24 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  jit(true);
+  expect = /TypeError: NaN is not a function/;
+  actual = 'No Error';
+  try
+  {
+    var obj = {valueOf: gc };
 
-  a=b=c=d=0; this.__defineGetter__('g', gc); for each (y in this);
+    function f() {
+      ( obj * [].a )();
+    }
 
-  jit(false);
+    f();
+  }
+  catch(ex)
+  {
+    actual = ex + '';
+  }
 
-  reportCompare(expect, actual, summary);
+  reportMatch(expect, actual, summary);
 
   exitFunc ('test');
 }
