@@ -46,7 +46,11 @@
 
 #include "Stack.h"
 
+#include "jsscriptinlines.h"
 #include "ArgumentsObject-inl.h"
+#include "CallObject-inl.h"
+
+#include "methodjit/MethodJIT.h"
 
 namespace js {
 
@@ -57,12 +61,14 @@ StackFrame::initPrev(JSContext *cx)
     if (FrameRegs *regs = cx->maybeRegs()) {
         prev_ = regs->fp();
         prevpc_ = regs->pc;
+        prevInline_ = regs->inlined();
         JS_ASSERT_IF(!prev_->isDummyFrame() && !prev_->hasImacropc(),
                      uint32(prevpc_ - prev_->script()->code) < prev_->script()->length);
     } else {
         prev_ = NULL;
 #ifdef DEBUG
         prevpc_ = (jsbytecode *)0xbadc;
+        prevInline_ = (JSInlinedSite *)0xbadc;
 #endif
     }
 }
@@ -74,11 +80,34 @@ StackFrame::resetGeneratorPrev(JSContext *cx)
     initPrev(cx);
 }
 
+inline void
+StackFrame::initInlineFrame(JSFunction *fun, StackFrame *prevfp, jsbytecode *prevpc)
+{
+    /*
+     * Note: no need to ensure the scopeChain is instantiated for inline
+     * frames. Functions which use the scope chain are never inlined.
+     */
+    flags_ = StackFrame::FUNCTION;
+    exec.fun = fun;
+    resetInlinePrev(prevfp, prevpc);
+}
+
+inline void
+StackFrame::resetInlinePrev(StackFrame *prevfp, jsbytecode *prevpc)
+{
+    JS_ASSERT_IF(flags_ & StackFrame::HAS_PREVPC, prevInline_);
+    flags_ |= StackFrame::HAS_PREVPC;
+    prev_ = prevfp;
+    prevpc_ = prevpc;
+    prevInline_ = NULL;
+}
+
 inline void
 StackFrame::initCallFrame(JSContext *cx, JSObject &callee, JSFunction *fun,
                           JSScript *script, uint32 nactual, StackFrame::Flags flagsArg)
 {
     JS_ASSERT((flagsArg & ~(CONSTRUCTING |
+                            LOWERED_CALL_APPLY |
                             OVERFLOW_ARGS |
                             UNDERFLOW_ARGS)) == 0);
     JS_ASSERT(fun == callee.getFunctionPrivate());
@@ -87,8 +116,9 @@ StackFrame::initCallFrame(JSContext *cx, JSObject &callee, JSFunction *fun,
     /* Initialize stack frame members. */
     flags_ = FUNCTION | HAS_PREVPC | HAS_SCOPECHAIN | flagsArg;
     exec.fun = fun;
-    args.nactual = nactual;  /* only need to write if over/under-flow */
+    args.nactual = nactual;
     scopeChain_ = callee.getParent();
+    ncode_ = NULL;
     initPrev(cx);
     JS_ASSERT(!hasImacropc());
     JS_ASSERT(!hasHookData());
@@ -103,11 +133,6 @@ StackFrame::resetCallFrame(JSScript *script)
 {
     JS_ASSERT(script == this->script());
 
-    /* Undo changes to frame made during execution; see also initCallFrame */
-
-    putActivationObjects();
-    markActivationObjectsAsPut();
-
     if (flags_ & UNDERFLOW_ARGS)
         SetValueRangeToUndefined(formalArgs() + numActualArgs(), formalArgsEnd());
 
@@ -120,17 +145,8 @@ StackFrame::resetCallFrame(JSScript *script)
                            HAS_SCOPECHAIN |
                            HAS_ANNOTATION |
                            HAS_HOOK_DATA |
-                           HAS_CALL_OBJ |
-                           HAS_ARGS_OBJ |
-                           FINISHED_IN_INTERP)));
-
-    /*
-     * Since the stack frame is usually popped after PutActivationObjects,
-     * these bits aren't cleared. The activation objects must have actually
-     * been put, though.
-     */
-    JS_ASSERT_IF(flags_ & HAS_CALL_OBJ, callObj().getPrivate() == NULL);
-    JS_ASSERT_IF(flags_ & HAS_ARGS_OBJ, argsObj().getPrivate() == NULL);
+                           FINISHED_IN_INTERP |
+                           DOWN_FRAMES_EXPANDED)));
 
     flags_ &= FUNCTION |
               OVERFLOW_ARGS |
@@ -144,16 +160,16 @@ StackFrame::resetCallFrame(JSScript *script)
 }
 
 inline void
-StackFrame::initJitFrameCallerHalf(JSContext *cx, StackFrame::Flags flags,
-                                    void *ncode)
+StackFrame::initJitFrameCallerHalf(StackFrame *prev, StackFrame::Flags flags, void *ncode)
 {
     JS_ASSERT((flags & ~(CONSTRUCTING |
+                         LOWERED_CALL_APPLY |
                          FUNCTION |
                          OVERFLOW_ARGS |
                          UNDERFLOW_ARGS)) == 0);
 
     flags_ = FUNCTION | flags;
-    prev_ = cx->fp();
+    prev_ = prev;
     ncode_ = ncode;
 }
 
@@ -165,8 +181,7 @@ inline void
 StackFrame::initJitFrameEarlyPrologue(JSFunction *fun, uint32 nactual)
 {
     exec.fun = fun;
-    if (flags_ & (OVERFLOW_ARGS | UNDERFLOW_ARGS))
-        args.nactual = nactual;
+    args.nactual = nactual;
 }
 
 /*
@@ -176,17 +191,14 @@ StackFrame::initJitFrameEarlyPrologue(JSFunction *fun, uint32 nactual)
 inline bool
 StackFrame::initJitFrameLatePrologue(JSContext *cx, Value **limit)
 {
-    uintN nvals = script()->nslots + VALUES_PER_STACK_FRAME;
-    Value *required = (Value *)this + nvals;
-    if (required >= *limit) {
-        ContextStack &stack = cx->stack;
-        if (!stack.space().tryBumpLimit(NULL, slots(), nvals, limit)) {
-            stack.popFrameAfterOverflow();
-            js_ReportOverRecursed(cx);
-            return false;
-        }
+    *limit = cx->stack.space().getStackLimit(cx, DONT_REPORT_ERROR);
+    if (!*limit) {
+        cx->stack.popFrameAfterOverflow();
+        js_ReportOverRecursed(cx);
+        return false;
     }
 
+    scopeChain();
     SetValueRangeToUndefined(slots(), script()->nfixed);
     return true;
 }
@@ -264,6 +276,14 @@ struct CopyTo
 inline uintN
 StackFrame::numActualArgs() const
 {
+    /*
+     * args.nactual is always coherent, except for method JIT frames where the
+     * callee does not access its arguments and the number of actual arguments
+     * matches the number of formal arguments. The JIT requires that all frames
+     * which do not have an arguments object and use their arguments have a
+     * coherent args.nactual (even though the below code may not use it), as
+     * JIT code may access the field directly.
+     */
     JS_ASSERT(hasArgs());
     if (JS_UNLIKELY(flags_ & (OVERFLOW_ARGS | UNDERFLOW_ARGS)))
         return hasArgsObj() ? argsObj().initialLength() : args.nactual;
@@ -322,30 +342,65 @@ StackFrame::setScopeChainNoCallObj(JSObject &obj)
 }
 
 inline void
-StackFrame::setScopeChainWithOwnCallObj(JSObject &obj)
+StackFrame::setScopeChainWithOwnCallObj(CallObject &obj)
 {
     JS_ASSERT(&obj != NULL);
-    JS_ASSERT(!hasCallObj() && obj.isCall() && obj.getPrivate() == this);
+    JS_ASSERT(!hasCallObj() && obj.maybeStackFrame() == this);
     scopeChain_ = &obj;
     flags_ |= HAS_SCOPECHAIN | HAS_CALL_OBJ;
 }
 
-inline JSObject &
+inline CallObject &
 StackFrame::callObj() const
 {
     JS_ASSERT_IF(isNonEvalFunctionFrame() || isStrictEvalFrame(), hasCallObj());
 
     JSObject *pobj = &scopeChain();
-    while (JS_UNLIKELY(pobj->getClass() != &js_CallClass)) {
+    while (JS_UNLIKELY(!pobj->isCall())) {
         JS_ASSERT(IsCacheableNonGlobalScope(pobj) || pobj->isWith());
         pobj = pobj->getParent();
     }
-    return *pobj;
+    return pobj->asCall();
+}
+
+inline bool
+StackFrame::maintainNestingState() const
+{
+    /*
+     * Whether to invoke the nesting epilogue/prologue to maintain active
+     * frame counts and check for reentrant outer functions.
+     */
+    return isNonEvalFunctionFrame() && !isGeneratorFrame() && script()->nesting();
+}
+
+inline bool
+StackFrame::functionPrologue(JSContext *cx)
+{
+    JS_ASSERT(isNonEvalFunctionFrame());
+
+    JSFunction *fun = this->fun();
+
+    if (fun->isHeavyweight()) {
+        if (!CreateFunCallObject(cx, this))
+            return false;
+    } else {
+        /* Force instantiation of the scope chain, for JIT frames. */
+        scopeChain();
+    }
+
+    if (script()->nesting()) {
+        JS_ASSERT(maintainNestingState());
+        types::NestingPrologue(cx, this);
+    }
+
+    return true;
 }
 
 inline void
-StackFrame::putActivationObjects()
+StackFrame::functionEpilogue()
 {
+    JS_ASSERT(isNonEvalFunctionFrame());
+
     if (flags_ & (HAS_ARGS_OBJ | HAS_CALL_OBJ)) {
         /* NB: there is an ordering dependency here. */
         if (hasCallObj())
@@ -353,17 +408,20 @@ StackFrame::putActivationObjects()
         else if (hasArgsObj())
             js_PutArgsObject(this);
     }
+
+    if (maintainNestingState())
+        types::NestingEpilogue(this);
 }
 
 inline void
-StackFrame::markActivationObjectsAsPut()
+StackFrame::markFunctionEpilogueDone()
 {
     if (flags_ & (HAS_ARGS_OBJ | HAS_CALL_OBJ)) {
-        if (hasArgsObj() && !argsObj().getPrivate()) {
+        if (hasArgsObj() && !argsObj().maybeStackFrame()) {
             args.nactual = args.obj->initialLength();
             flags_ &= ~HAS_ARGS_OBJ;
         }
-        if (hasCallObj() && !callObj().getPrivate()) {
+        if (hasCallObj() && !callObj().maybeStackFrame()) {
             /*
              * For function frames, the call object may or may not have have an
              * enclosing DeclEnv object, so we use the callee's parent, since
@@ -377,103 +435,72 @@ StackFrame::markActivationObjectsAsPut()
             flags_ &= ~HAS_CALL_OBJ;
         }
     }
+
+    /*
+     * For outer/inner function frames, undo the active frame balancing so that
+     * when we redo it in the epilogue we get the right final value. The other
+     * nesting epilogue changes (update active args/vars) are idempotent.
+     */
+    if (maintainNestingState())
+        script()->nesting()->activeFrames++;
 }
 
 /*****************************************************************************/
 
 #ifdef JS_TRACER
 JS_ALWAYS_INLINE bool
-StackSpace::ensureEnoughSpaceToEnterTrace()
+StackSpace::ensureEnoughSpaceToEnterTrace(JSContext *cx)
 {
     ptrdiff_t needed = TraceNativeStorage::MAX_NATIVE_STACK_SLOTS +
                        TraceNativeStorage::MAX_CALL_STACK_ENTRIES * VALUES_PER_STACK_FRAME;
-#ifdef XP_WIN
-    return ensureSpace(NULL, firstUnused(), needed);
-#else
-    return end_ - firstUnused() > needed;
-#endif
+    return ensureSpace(cx, DONT_REPORT_ERROR, firstUnused(), needed);
 }
 #endif
 
 STATIC_POSTCONDITION(!return || ubound(from) >= nvals)
 JS_ALWAYS_INLINE bool
-StackSpace::ensureSpace(JSContext *maybecx, Value *from, ptrdiff_t nvals) const
+StackSpace::ensureSpace(JSContext *cx, MaybeReportError report, Value *from, ptrdiff_t nvals,
+                        JSCompartment *dest) const
 {
+    assertInvariants();
     JS_ASSERT(from >= firstUnused());
 #ifdef XP_WIN
     JS_ASSERT(from <= commitEnd_);
-    if (commitEnd_ - from < nvals)
-        return bumpCommit(maybecx, from, nvals);
-    return true;
-#else
-    if (end_ - from < nvals) {
-        js_ReportOverRecursed(maybecx);
-        return false;
-    }
-    return true;
 #endif
+    if (JS_UNLIKELY(conservativeEnd_ - from < nvals))
+        return ensureSpaceSlow(cx, report, from, nvals, dest);
+    return true;
 }
 
 inline Value *
-StackSpace::getStackLimit(JSContext *cx)
+StackSpace::getStackLimit(JSContext *cx, MaybeReportError report)
 {
-    Value *limit;
-#ifdef XP_WIN
-    limit = commitEnd_;
-#else
-    limit = end_;
-#endif
-
-    /* See getStackLimit comment in Stack.h. */
     FrameRegs &regs = cx->regs();
-    uintN minSpace = regs.fp()->numSlots() + VALUES_PER_STACK_FRAME;
-    if (regs.sp + minSpace > limit) {
-        js_ReportOverRecursed(cx);
-        return NULL;
-    }
-
-    return limit;
+    uintN nvals = regs.fp()->numSlots() + STACK_JIT_EXTRA;
+    return ensureSpace(cx, report, regs.sp, nvals)
+           ? conservativeEnd_
+           : NULL;
 }
 
 /*****************************************************************************/
 
-JS_ALWAYS_INLINE bool
-OOMCheck::operator()(JSContext *cx, StackSpace &space, Value *from, uintN nvals)
-{
-    return space.ensureSpace(cx, from, nvals);
-}
-
-JS_ALWAYS_INLINE bool
-LimitCheck::operator()(JSContext *cx, StackSpace &space, Value *from, uintN nvals)
-{
-    /*
-     * Include an extra sizeof(StackFrame) to satisfy the method-jit
-     * stackLimit invariant.
-     */
-    nvals += VALUES_PER_STACK_FRAME;
-    JS_ASSERT(from < *limit);
-    if (*limit - from >= ptrdiff_t(nvals))
-        return true;
-    return space.tryBumpLimit(cx, from, nvals, limit);
-}
-
-template <class Check>
 JS_ALWAYS_INLINE StackFrame *
-ContextStack::getCallFrame(JSContext *cx, const CallArgs &args,
-                           JSFunction *fun, JSScript *script,
-                           StackFrame::Flags *flags, Check check) const
+ContextStack::getCallFrame(JSContext *cx, MaybeReportError report, const CallArgs &args,
+                           JSFunction *fun, JSScript *script, StackFrame::Flags *flags) const
 {
     JS_ASSERT(fun->script() == script);
-    JS_ASSERT(space().firstUnused() == args.end());
+    uintN nformal = fun->nargs;
 
     Value *firstUnused = args.end();
-    uintN nvals = VALUES_PER_STACK_FRAME + script->nslots;
-    uintN nformal = fun->nargs;
+    JS_ASSERT(firstUnused == space().firstUnused());
+
+    /* Include extra space to satisfy the method-jit stackLimit invariant. */
+    uintN nvals = VALUES_PER_STACK_FRAME + script->nslots + StackSpace::STACK_JIT_EXTRA;
 
     /* Maintain layout invariant: &formalArgs[0] == ((Value *)fp) - nformal. */
 
     if (args.argc() == nformal) {
-        if (JS_UNLIKELY(!check(cx, space(), firstUnused, nvals)))
+        if (!space().ensureSpace(cx, report, firstUnused, nvals))
             return NULL;
         return reinterpret_cast<StackFrame *>(firstUnused);
     }
@@ -481,7 +508,7 @@ ContextStack::getCallFrame(JSContext *cx, const CallArgs &args,
     if (args.argc() < nformal) {
         *flags = StackFrame::Flags(*flags | StackFrame::UNDERFLOW_ARGS);
         uintN nmissing = nformal - args.argc();
-        if (JS_UNLIKELY(!check(cx, space(), firstUnused, nmissing + nvals)))
+        if (!space().ensureSpace(cx, report, firstUnused, nmissing + nvals))
             return NULL;
         SetValueRangeToUndefined(firstUnused, nmissing);
         return reinterpret_cast<StackFrame *>(firstUnused + nmissing);
@@ -489,58 +516,71 @@ ContextStack::getCallFrame(JSContext *cx, const CallArgs &args,
 
     *flags = StackFrame::Flags(*flags | StackFrame::OVERFLOW_ARGS);
     uintN ncopy = 2 + nformal;
-    if (JS_UNLIKELY(!check(cx, space(), firstUnused, ncopy + nvals)))
+    if (!space().ensureSpace(cx, report, firstUnused, ncopy + nvals))
         return NULL;
-
     Value *dst = firstUnused;
     Value *src = args.base();
     PodCopy(dst, src, ncopy);
     return reinterpret_cast<StackFrame *>(firstUnused + ncopy);
 }
 
-template <class Check>
 JS_ALWAYS_INLINE bool
 ContextStack::pushInlineFrame(JSContext *cx, FrameRegs &regs, const CallArgs &args,
                               JSObject &callee, JSFunction *fun, JSScript *script,
-                              MaybeConstruct construct, Check check)
+                              InitialFrameFlags initial)
 {
     JS_ASSERT(onTop());
-    JS_ASSERT(&regs == &seg_->regs());
     JS_ASSERT(regs.sp == args.end());
     /* Cannot assert callee == args.callee() since this is called from LeaveTree. */
     JS_ASSERT(callee.getFunctionPrivate() == fun);
     JS_ASSERT(fun->script() == script);
 
-    StackFrame::Flags flags = ToFrameFlags(construct);
-    StackFrame *fp = getCallFrame(cx, args, fun, script, &flags, check);
+    StackFrame::Flags flags = ToFrameFlags(initial);
+    StackFrame *fp = getCallFrame(cx, REPORT_ERROR, args, fun, script, &flags);
     if (!fp)
         return false;
 
     /* Initialize frame, locals, regs. */
     fp->initCallFrame(cx, callee, fun, script, args.argc(), flags);
+
+    /*
+     * N.B. regs may differ from the active registers, if the parent is about
+     * to repoint the active registers to regs. See UncachedInlineCall.
+     */
     regs.prepareToRun(*fp, script);
     return true;
 }
 
+JS_ALWAYS_INLINE bool
+ContextStack::pushInlineFrame(JSContext *cx, FrameRegs &regs, const CallArgs &args,
+                              JSObject &callee, JSFunction *fun, JSScript *script,
+                              InitialFrameFlags initial, Value **stackLimit)
+{
+    if (!pushInlineFrame(cx, regs, args, callee, fun, script, initial))
+        return false;
+    *stackLimit = space().conservativeEnd_;
+    return true;
+}
+
 JS_ALWAYS_INLINE StackFrame *
-ContextStack::getFixupFrame(JSContext *cx, FrameRegs &regs, const CallArgs &args,
-                            JSFunction *fun, JSScript *script, void *ncode,
-                            MaybeConstruct construct, LimitCheck check)
+ContextStack::getFixupFrame(JSContext *cx, MaybeReportError report,
+                            const CallArgs &args, JSFunction *fun, JSScript *script,
+                            void *ncode, InitialFrameFlags initial, Value **stackLimit)
 {
     JS_ASSERT(onTop());
-    JS_ASSERT(&regs == &cx->regs());
-    JS_ASSERT(regs.sp == args.end());
     JS_ASSERT(args.callee().getFunctionPrivate() == fun);
     JS_ASSERT(fun->script() == script);
 
-    StackFrame::Flags flags = ToFrameFlags(construct);
-    StackFrame *fp = getCallFrame(cx, args, fun, script, &flags, check);
+    StackFrame::Flags flags = ToFrameFlags(initial);
+    StackFrame *fp = getCallFrame(cx, report, args, fun, script, &flags);
     if (!fp)
         return NULL;
 
     /* Do not init late prologue or regs; this is done by jit code. */
-    fp->initJitFrameCallerHalf(cx, flags, ncode);
+    fp->initJitFrameCallerHalf(cx->fp(), flags, ncode);
     fp->initJitFrameEarlyPrologue(fun, args.argc());
+
+    *stackLimit = space().conservativeEnd_;
     return fp;
 }
 
@@ -551,7 +591,7 @@ ContextStack::popInlineFrame(FrameRegs &regs)
     JS_ASSERT(&regs == &seg_->regs());
 
     StackFrame *fp = regs.fp();
-    fp->putActivationObjects();
+    fp->functionEpilogue();
 
     Value *newsp = fp->actualArgs() - 1;
     JS_ASSERT(newsp >= fp->prev()->base());
@@ -569,6 +609,54 @@ ContextStack::popFrameAfterOverflow()
     regs.popFrame(fp->actualArgsEnd());
 }
 
+inline JSScript *
+ContextStack::currentScript(jsbytecode **ppc) const
+{
+    if (ppc)
+        *ppc = NULL;
+
+    FrameRegs *regs = maybeRegs();
+    StackFrame *fp = regs ? regs->fp() : NULL;
+    while (fp && fp->isDummyFrame())
+        fp = fp->prev();
+    if (!fp)
+        return NULL;
+
+#ifdef JS_METHODJIT
+    mjit::CallSite *inlined = regs->inlined();
+    if (inlined) {
+        JS_ASSERT(inlined->inlineIndex < fp->jit()->nInlineFrames);
+        mjit::InlineFrame *frame = &fp->jit()->inlineFrames()[inlined->inlineIndex];
+        JSScript *script = frame->fun->script();
+        if (script->compartment() != cx_->compartment)
+            return NULL;
+        if (ppc)
+            *ppc = script->code + inlined->pcOffset;
+        return script;
+    }
+#endif
+
+    JSScript *script = fp->script();
+    if (script->compartment() != cx_->compartment)
+        return NULL;
+
+    if (ppc) {
+        if (fp->hasImacropc())
+            *ppc = fp->imacropc();
+        else
+            *ppc = fp->pcQuadratic(*this);
+    }
+    return script;
+}
+
+inline JSObject *
+ContextStack::currentScriptedScopeChain() const
+{
+    return &fp()->scopeChain();
+}
+
+/*****************************************************************************/
+
 namespace detail {
 
 struct STATIC_SKIP_INFERENCE CopyNonHoleArgsTo
@@ -605,8 +693,7 @@ ArgumentsObject::getElement(uint32 i, Value *vp)
      * If this arguments object was created on trace the actual argument value
      * could be in a register or something, so we can't optimize.
      */
-    StackFrame *fp = reinterpret_cast<StackFrame *>(getPrivate());
-    if (fp == JS_ARGUMENTS_OBJECT_ON_TRACE)
+    if (onTrace())
         return false;
 
     /*
@@ -614,6 +701,7 @@ ArgumentsObject::getElement(uint32 i, Value *vp)
      * the canonical argument value.  Note that strict arguments objects do not
      * alias named arguments and never have a stack frame.
      */
+    StackFrame *fp = maybeStackFrame();
     JS_ASSERT_IF(isStrictArguments(), !fp);
     if (fp)
         *vp = fp->canonicalActualArg(i);
@@ -629,13 +717,14 @@ ArgumentsObject::getElements(uint32 start, uint32 count, Value *vp)
     if (start > length || start + count > length)
         return false;
 
-    StackFrame *fp = reinterpret_cast<StackFrame *>(getPrivate());
+    StackFrame *fp = maybeStackFrame();
 
     /* If there's no stack frame for this, argument values are in elements(). */
     if (!fp) {
-        Value *srcbeg = elements() + start;
-        Value *srcend = srcbeg + count;
-        for (Value *dst = vp, *src = srcbeg; src < srcend; ++dst, ++src) {
+        const Value *srcbeg = elements() + start;
+        const Value *srcend = srcbeg + count;
+        const Value *src = srcbeg;
+        for (Value *dst = vp; src < srcend; ++dst, ++src) {
             if (src->isMagic(JS_ARGS_HOLE))
                 return false;
             *dst = *src;
@@ -644,11 +733,11 @@ ArgumentsObject::getElements(uint32 start, uint32 count, Value *vp)
     }
 
     /* If we're on trace, there's no canonical location for elements: fail. */
-    if (fp == JS_ARGUMENTS_OBJECT_ON_TRACE)
+    if (onTrace())
         return false;
 
     /* Otherwise, element values are on the stack. */
-    JS_ASSERT(fp->numActualArgs() <= JS_ARGS_LENGTH_MAX);
+    JS_ASSERT(fp->numActualArgs() <= StackSpace::ARGS_LENGTH_MAX);
     return fp->forEachCanonicalActualArg(detail::CopyNonHoleArgsTo(this, vp), start, count);
 }
 
