@@ -62,45 +62,26 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(nsXPCJSContextStackIterator)
 NS_GENERIC_FACTORY_SINGLETON_CONSTRUCTOR(nsIXPConnect,
                                          nsXPConnect::GetSingleton)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsScriptError)
-NS_GENERIC_FACTORY_CONSTRUCTOR(nsXPCComponents_Interfaces)
 
 NS_GENERIC_FACTORY_CONSTRUCTOR(mozJSComponentLoader)
 NS_GENERIC_FACTORY_CONSTRUCTOR(mozJSSubScriptLoader)
 
-#ifdef XPC_IDISPATCH_SUPPORT
-NS_GENERIC_FACTORY_SINGLETON_CONSTRUCTOR(nsIDispatchSupport,
-                                         nsDispatchSupport::GetSingleton)
-#endif // XPC_IDISPATCH_SUPPORT
-
 NS_DEFINE_NAMED_CID(NS_JS_ID_CID);
 NS_DEFINE_NAMED_CID(NS_XPCONNECT_CID);
 NS_DEFINE_NAMED_CID(NS_XPCEXCEPTION_CID);
 NS_DEFINE_NAMED_CID(NS_SCRIPTERROR_CID);
-NS_DEFINE_NAMED_CID(SCRIPTABLE_INTERFACES_CID);
 NS_DEFINE_NAMED_CID(NS_XPC_JSCONTEXT_STACK_ITERATOR_CID);
 NS_DEFINE_NAMED_CID(MOZJSCOMPONENTLOADER_CID);
 NS_DEFINE_NAMED_CID(MOZ_JSSUBSCRIPTLOADER_CID);
-#ifdef XPC_IDISPATCH_SUPPORT
-NS_DEFINE_NAMED_CID(NS_IDISPATCH_SUPPORT_CID);
-#define XPCIDISPATCH_CIDS \
-  { &kNS_IDISPATCH_SUPPORT_CID, false, NULL, nsIDispatchSupportConstructor },
-#define XPCIDISPATCH_CONTRACTS \
-  { NS_IDISPATCH_SUPPORT_CONTRACTID, &kNS_IDISPATCH_SUPPORT_CID },
-#else
-#define XPCIDISPATCH_CIDS
-#define XPCIDISPATCH_CONTRACTS
-#endif
 
 #define XPCONNECT_CIDENTRIES \
   { &kNS_JS_ID_CID, false, NULL,  nsJSIDConstructor }, \
   { &kNS_XPCONNECT_CID, false, NULL,  nsIXPConnectConstructor }, \
   { &kNS_XPCEXCEPTION_CID, false, NULL, nsXPCExceptionConstructor }, \
   { &kNS_SCRIPTERROR_CID, false, NULL, nsScriptErrorConstructor }, \
-  { &kSCRIPTABLE_INTERFACES_CID, false, NULL, nsXPCComponents_InterfacesConstructor }, \
   { &kNS_XPC_JSCONTEXT_STACK_ITERATOR_CID, false, NULL, nsXPCJSContextStackIteratorConstructor }, \
   { &kMOZJSCOMPONENTLOADER_CID, false, NULL, mozJSComponentLoaderConstructor }, \
-  { &kMOZ_JSSUBSCRIPTLOADER_CID, false, NULL, mozJSSubScriptLoaderConstructor }, \
-  XPCIDISPATCH_CIDS
+  { &kMOZ_JSSUBSCRIPTLOADER_CID, false, NULL, mozJSSubScriptLoaderConstructor },
 
 #define XPCONNECT_CONTRACTS \
   { XPC_ID_CONTRACTID, &kNS_JS_ID_CID }, \
@@ -109,11 +90,9 @@ NS_DEFINE_NAMED_CID(NS_IDISPATCH_SUPPORT_CID);
   { XPC_RUNTIME_CONTRACTID, &kNS_XPCONNECT_CID }, \
   { XPC_EXCEPTION_CONTRACTID, &kNS_XPCEXCEPTION_CID }, \
   { NS_SCRIPTERROR_CONTRACTID, &kNS_SCRIPTERROR_CID }, \
-  { NS_SCRIPTABLE_INTERFACES_CONTRACTID, &kSCRIPTABLE_INTERFACES_CID }, \
   { XPC_JSCONTEXT_STACK_ITERATOR_CONTRACTID, &kNS_XPC_JSCONTEXT_STACK_ITERATOR_CID }, \
   { MOZJSCOMPONENTLOADER_CONTRACTID, &kMOZJSCOMPONENTLOADER_CID }, \
-  { MOZJSSUBSCRIPTLOADER_CONTRACTID, &kMOZ_JSSUBSCRIPTLOADER_CID }, \
-  XPCIDISPATCH_CONTRACTS
+  { MOZJSSUBSCRIPTLOADER_CONTRACTID, &kMOZ_JSSUBSCRIPTLOADER_CID },
 
 #define XPCONNECT_CATEGORIES \
   { "module-loader", "js", MOZJSCOMPONENTLOADER_CONTRACTID },
