@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+ * vim: set ts=8 sw=4 et tw=78:
  *
  * ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
@@ -45,18 +46,21 @@
 #include "nsArrayEnumerator.h"
 #include "nsWrapperCache.h"
 #include "XPCWrapper.h"
+#include "AccessCheck.h"
+#include "nsJSUtils.h"
 
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsXPCWrappedJSClass, nsIXPCWrappedJSClass)
 
 // the value of this variable is never used - we use its address as a sentinel
 static uint32 zero_methods_descriptor;
 
-void AutoScriptEvaluate::StartEvaluating(JSErrorReporter errorReporter)
+PRBool AutoScriptEvaluate::StartEvaluating(JSObject *scope, JSErrorReporter errorReporter)
 {
     NS_PRECONDITION(!mEvaluated, "AutoScriptEvaluate::Evaluate should only be called once");
 
-    if(!mJSContext)
-        return;
+    if (!mJSContext)
+        return PR_TRUE;
+
     mEvaluated = PR_TRUE;
     if(!mJSContext->errorReporter)
     {
@@ -67,6 +71,9 @@ void AutoScriptEvaluate::StartEvaluating(JSErrorReporter errorReporter)
     if (mContextHasThread)
         JS_BeginRequest(mJSContext);
 
+    if (!mEnterCompartment.enter(mJSContext, scope))
+        return PR_FALSE;
+
     // Saving the exception state keeps us from interfering with another script
     // that may also be running on this context.  This occurred first with the
     // js debugger, as described in
@@ -82,6 +89,8 @@ void AutoScriptEvaluate::StartEvaluating(JSErrorReporter errorReporter)
         mState = JS_SaveExceptionState(mJSContext);
         JS_ClearPendingException(mJSContext);
     }
+
+    return PR_TRUE;
 }
 
 AutoScriptEvaluate::~AutoScriptEvaluate()
@@ -157,8 +166,9 @@ nsXPCWrappedJSClass::GetNewOrUsed(XPCCallContext& ccx, REFNSIID aIID,
         ccx.GetXPConnect()->GetInfoForIID(&aIID, getter_AddRefs(info));
         if(info)
         {
-            PRBool canScript;
+            PRBool canScript, isBuiltin;
             if(NS_SUCCEEDED(info->IsScriptable(&canScript)) && canScript &&
+               NS_SUCCEEDED(info->IsBuiltinClass(&isBuiltin)) && !isBuiltin &&
                nsXPConnect::IsISupportsDescendant(info))
             {
                 clazz = new nsXPCWrappedJSClass(ccx, aIID, info);
@@ -253,24 +263,19 @@ nsXPCWrappedJSClass::CallQueryInterfaceOnJSObject(XPCCallContext& ccx,
     // interface (i.e. whether the interface is scriptable) and most content
     // objects don't have QI implementations anyway. Also see bug 503926.
     if(XPCPerThreadData::IsMainThread(ccx) &&
-       !JS_GetGlobalForObject(ccx, jsobj)->isSystem())
+       !xpc::AccessCheck::isChrome(jsobj->compartment()))
     {
-        nsCOMPtr<nsIPrincipal> objprin;
-        nsIScriptSecurityManager *ssm = XPCWrapper::GetSecurityManager();
-        if(ssm)
-        {
-            nsresult rv = ssm->GetObjectPrincipal(ccx, jsobj, getter_AddRefs(objprin));
-            NS_ENSURE_SUCCESS(rv, nsnull);
-
-            PRBool isSystem;
-            rv = ssm->IsSystemPrincipal(objprin, &isSystem);
-            NS_ENSURE_SUCCESS(rv, nsnull);
-
-            if(!isSystem)
-                return nsnull;
-        }
+        return nsnull;
     }
 
+    // OK, it looks like we'll be calling into JS code.
+    AutoScriptEvaluate scriptEval(cx);
+
+    // XXX we should install an error reporter that will send reports to
+    // the JS error console service.
+    if (!scriptEval.StartEvaluating(jsobj))
+        return nsnull;
+
     // check upfront for the existence of the function property
     funid = mRuntime->GetStringID(XPCJSRuntime::IDX_QUERY_INTERFACE);
     if(!JS_GetPropertyById(cx, jsobj, funid, &fun) || JSVAL_IS_PRIMITIVE(fun))
@@ -291,19 +296,12 @@ nsXPCWrappedJSClass::CallQueryInterfaceOnJSObject(XPCCallContext& ccx,
         ccx.GetXPConnect()->GetInfoForIID(&aIID, getter_AddRefs(info));
         if(!info)
             return nsnull;
-        PRBool canScript;
-        if(NS_FAILED(info->IsScriptable(&canScript)) || !canScript)
+        PRBool canScript, isBuiltin;
+        if(NS_FAILED(info->IsScriptable(&canScript)) || !canScript ||
+           NS_FAILED(info->IsBuiltinClass(&isBuiltin)) || isBuiltin)
             return nsnull;
     }
 
-    // OK, it looks like we'll be calling into JS code.
-
-    AutoScriptEvaluate scriptEval(cx);
-
-    // XXX we should install an error reporter that will send reports to
-    // the JS error console service.
-    scriptEval.StartEvaluating();
-
     id = xpc_NewIDObject(cx, jsobj, aIID);
     if(id)
     {
@@ -357,7 +355,7 @@ nsXPCWrappedJSClass::CallQueryInterfaceOnJSObject(XPCCallContext& ccx,
                 {
                     // JS often throws an nsresult.
                     if(JSVAL_IS_DOUBLE(jsexception))
-                        rv = (nsresult)(*JSVAL_TO_DOUBLE(jsexception));
+                        rv = (nsresult)(JSVAL_TO_DOUBLE(jsexception));
                     else
                         rv = (nsresult)(JSVAL_TO_INT(jsexception));
 
@@ -391,7 +389,10 @@ GetNamedPropertyAsVariantRaw(XPCCallContext& ccx,
     jsval val;
 
     return JS_GetPropertyById(ccx, aJSObj, aName, &val) &&
-           XPCConvert::JSData2Native(ccx, aResult, val, type, JS_FALSE, 
+           // Note that this always takes the T_INTERFACE path through
+           // JSData2Native, so the value passed for useAllocator
+           // doesn't really matter. We pass true for consistency.
+           XPCConvert::JSData2Native(ccx, aResult, val, type, JS_TRUE,
                                      &NS_GET_IID(nsIVariant), pErr);
 }
 
@@ -408,7 +409,8 @@ nsXPCWrappedJSClass::GetNamedPropertyAsVariant(XPCCallContext& ccx,
     nsresult rv = NS_ERROR_FAILURE;
 
     AutoScriptEvaluate scriptEval(cx);
-    scriptEval.StartEvaluating();
+    if (!scriptEval.StartEvaluating(aJSObj))
+        return NS_ERROR_FAILURE;
 
     ok = JS_ValueToId(cx, aName, &id) && 
          GetNamedPropertyAsVariantRaw(ccx, aJSObj, id, aResult, &rv);
@@ -431,7 +433,8 @@ nsXPCWrappedJSClass::BuildPropertyEnumerator(XPCCallContext& ccx,
 
     // Saved state must be restored, all exits through 'out'...
     AutoScriptEvaluate scriptEval(cx);
-    scriptEval.StartEvaluating();
+    if (!scriptEval.StartEvaluating(aJSObj))
+        return NS_ERROR_FAILURE;
 
     idArray = JS_Enumerate(cx, aJSObj);
     if(!idArray)
@@ -460,10 +463,13 @@ nsXPCWrappedJSClass::BuildPropertyEnumerator(XPCCallContext& ccx,
         if(!name)
             goto out;
 
+        size_t length;
+        const jschar *chars = JS_GetStringCharsAndLength(cx, name, &length);
+        if (!chars)
+            goto out;
+
         nsCOMPtr<nsIProperty> property = 
-            new xpcProperty((const PRUnichar*) JS_GetStringChars(name), 
-                            (PRUint32) JS_GetStringLength(name),
-                            value);
+            new xpcProperty(chars, (PRUint32) length, value);
         if(!property)
             goto out;
 
@@ -563,6 +569,10 @@ GetContextFromObject(JSObject *obj)
     XPCCallContext ccx(NATIVE_CALLER);
     if(!ccx.IsValid())
         return nsnull;
+
+    JSAutoEnterCompartment ac;
+    if(!ac.enter(ccx, obj))
+        return nsnull;
     XPCWrappedNativeScope* scope =
         XPCWrappedNativeScope::FindInJSObjectScope(ccx, obj);
     XPCContext *xpcc = scope->GetContext();
@@ -570,14 +580,13 @@ GetContextFromObject(JSObject *obj)
     if(xpcc)
     {
         JSContext *cx = xpcc->GetJSContext();
-        if(cx->thread->id == js_CurrentThreadId())
+        if(cx->thread()->id == js_CurrentThreadId())
             return cx;
     }
 
     return nsnull;
 }
 
-#ifndef XPCONNECT_STANDALONE
 class SameOriginCheckedComponent : public nsISecurityCheckedComponent
 {
 public:
@@ -636,8 +645,6 @@ SameOriginCheckedComponent::CanSetProperty(const nsIID * iid,
     return *_retval ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
 }
 
-#endif
-
 NS_IMETHODIMP
 nsXPCWrappedJSClass::DelegatedQueryInterface(nsXPCWrappedJS* self,
                                              REFNSIID aIID,
@@ -659,13 +666,6 @@ nsXPCWrappedJSClass::DelegatedQueryInterface(nsXPCWrappedJS* self,
         return NS_OK;
     }
 
-#ifdef XPC_IDISPATCH_SUPPORT
-    // If IDispatch is enabled and we're QI'ing to IDispatch
-    if(nsXPConnect::IsIDispatchEnabled() && aIID.Equals(NSID_IDISPATCH))
-    {
-        return XPCIDispatchExtension::IDispatchQIWrappedJS(self, aInstancePtr);
-    }
-#endif
     if(aIID.Equals(NS_GET_IID(nsIPropertyBag)))
     {
         // We only want to expose one implementation from our aggregate.
@@ -739,7 +739,6 @@ nsXPCWrappedJSClass::DelegatedQueryInterface(nsXPCWrappedJS* self,
 
     // else we do the more expensive stuff...
 
-#ifndef XPCONNECT_STANDALONE
     // Before calling out, ensure that we're not about to claim to implement
     // nsISecurityCheckedComponent for an untrusted object. Doing so causes
     // problems. See bug 352882.
@@ -779,7 +778,7 @@ nsXPCWrappedJSClass::DelegatedQueryInterface(nsXPCWrappedJS* self,
         PRBool isSystem;
         rv = secMan->IsSystemPrincipal(objPrin, &isSystem);
         if((NS_FAILED(rv) || !isSystem) &&
-           !IS_WRAPPER_CLASS(STOBJ_GET_CLASS(selfObj)))
+           !IS_WRAPPER_CLASS(selfObj->getClass()))
         {
             // A content object.
             nsRefPtr<SameOriginCheckedComponent> checked =
@@ -790,7 +789,6 @@ nsXPCWrappedJSClass::DelegatedQueryInterface(nsXPCWrappedJS* self,
             return NS_OK;
         }
     }
-#endif
 
     // check if the JSObject claims to implement this interface
     JSObject* jsobj = CallQueryInterfaceOnJSObject(ccx, self->GetJSObject(),
@@ -937,7 +935,7 @@ nsXPCWrappedJSClass::GetArraySizeFromParam(JSContext* cx,
     if(arg_type.IsPointer() || arg_type.TagPart() != nsXPTType::T_U32)
         return JS_FALSE;
 
-    if(arg_param.IsOut())
+    if(arg_param.IsIndirect())
         *result = *(JSUint32*)nativeParams[argnum].val.p;
     else
         *result = nativeParams[argnum].val.u32;
@@ -978,7 +976,7 @@ nsXPCWrappedJSClass::GetInterfaceTypeFromParam(JSContext* cx,
         if(arg_type.IsPointer() &&
            arg_type.TagPart() == nsXPTType::T_IID)
         {
-            if(arg_param.IsOut())
+            if(arg_param.IsIndirect())
             {
                 nsID** p = (nsID**) nativeParams[argnum].val.p;
                 if(!p || !*p)
@@ -1116,7 +1114,7 @@ nsXPCWrappedJSClass::CheckForException(XPCCallContext & ccx,
                     JSStackFrame * fp = nsnull;
                     while((fp = JS_FrameIterator(cx, &fp)))
                     {
-                        if(!JS_IsNativeFrame(cx, fp))
+                        if(JS_IsScriptFrame(cx, fp))
                         {
                             onlyNativeStackFrames = PR_FALSE;
                             break;
@@ -1216,11 +1214,14 @@ nsXPCWrappedJSClass::CheckForException(XPCCallContext & ccx,
                                     rv = location->GetFilename(getter_Copies(sourceName));
                                 }
 
-                                rv = scriptError->Init(newMessage.get(),
-                                                       NS_ConvertASCIItoUTF16(sourceName).get(),
-                                                       nsnull,
-                                                       lineNumber, 0, 0,
-                                                       "XPConnect JavaScript");
+                                nsCOMPtr<nsIScriptError2> scriptError2 =
+                                    do_QueryInterface(scriptError);
+                                rv = scriptError2->InitWithWindowID(newMessage.get(),
+                                                                    NS_ConvertASCIItoUTF16(sourceName).get(),
+                                                                    nsnull,
+                                                                    lineNumber, 0, 0,
+                                                                    "XPConnect JavaScript",
+                                                                    nsJSUtils::GetCurrentlyRunningCodeInnerWindowID(cx));
                                 if(NS_FAILED(rv))
                                     scriptError = nsnull;
                             }
@@ -1250,17 +1251,26 @@ nsXPCWrappedJSClass::CheckForException(XPCCallContext & ccx,
     return NS_ERROR_FAILURE;
 }
 
+class ContextPrincipalGuard
+{
+    nsIScriptSecurityManager *ssm;
+    XPCCallContext &ccx;
+  public:
+    ContextPrincipalGuard(XPCCallContext &ccx)
+      : ssm(nsnull), ccx(ccx) {}
+    void principalPushed(nsIScriptSecurityManager *ssm) { this->ssm = ssm; }
+    ~ContextPrincipalGuard() { if (ssm) ssm->PopContextPrincipal(ccx); }
+};
+
 NS_IMETHODIMP
 nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16 methodIndex,
                                 const XPTMethodDescriptor* info,
                                 nsXPTCMiniVariant* nativeParams)
 {
-    jsval* stackbase = nsnull;
     jsval* sp = nsnull;
+    jsval* argv = nsnull;
     uint8 i;
     uint8 argc=0;
-    uint8 stack_size;
-    jsval result;
     uint8 paramCount=0;
     nsresult retval = NS_ERROR_FAILURE;
     nsresult pending_result = NS_OK;
@@ -1270,13 +1280,10 @@ nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16 methodIndex,
     JSObject* obj;
     const char* name = info->name;
     jsval fval;
-    void* mark;
     JSBool foundDependentParam;
     XPCContext* xpcc;
     JSContext* cx;
     JSObject* thisObj;
-    JSBool popPrincipal = JS_FALSE;
-    nsIScriptSecurityManager_1_9_2* ssm = nsnull;
 
     // Make sure not to set the callee on ccx until after we've gone through
     // the whole nsIXPCFunctionThisTranslator bit.  That code uses ccx to
@@ -1284,65 +1291,75 @@ nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16 methodIndex,
     // to our real callee.
     JSContext *context = GetContextFromObject(wrapper->GetJSObject());
     XPCCallContext ccx(NATIVE_CALLER, context);
-    if(ccx.IsValid())
-    {
-        xpcc = ccx.GetXPCContext();
-        cx = ccx.GetJSContext();
-    }
-    else
-    {
-        xpcc = nsnull;
-        cx = nsnull;
-    }
+    if(!ccx.IsValid())
+        return retval;
 
-    AutoScriptEvaluate scriptEval(cx);
-#ifdef DEBUG_stats_jband
-    PRIntervalTime startTime = PR_IntervalNow();
-    PRIntervalTime endTime = 0;
-    static int totalTime = 0;
+    xpcc = ccx.GetXPCContext();
+    cx = ccx.GetJSContext();
 
-
-    static int count = 0;
-    static const int interval = 10;
-    if(0 == (++count % interval))
-        printf("<<<<<<<< %d calls on nsXPCWrappedJSs made.  (%d)\n", count, PR_IntervalToMilliseconds(totalTime));
-#endif
+    if(!cx || !xpcc || !IsReflectable(methodIndex))
+        return NS_ERROR_FAILURE;
 
     obj = thisObj = wrapper->GetJSObject();
 
+    JSAutoEnterCompartment ac;
+    if (!ac.enter(cx, obj))
+        return NS_ERROR_FAILURE;
+
+    ccx.SetScopeForNewJSObjects(obj);
+
+    js::AutoValueVector args(cx);
+    AutoScriptEvaluate scriptEval(cx);
+    ContextPrincipalGuard principalGuard(ccx);
+
     // XXX ASSUMES that retval is last arg. The xpidl compiler ensures this.
     paramCount = info->num_args;
     argc = paramCount -
         (paramCount && XPT_PD_IS_RETVAL(info->params[paramCount-1].flags) ? 1 : 0);
 
-    if(!cx || !xpcc || !IsReflectable(methodIndex))
+    if (!scriptEval.StartEvaluating(obj, xpcWrappedJSErrorReporter))
         goto pre_call_clean_up;
 
-    scriptEval.StartEvaluating(xpcWrappedJSErrorReporter);
-
     xpcc->SetPendingResult(pending_result);
     xpcc->SetException(nsnull);
     ccx.GetThreadData()->SetException(nsnull);
 
-    // We use js_AllocStack, js_Invoke, and js_FreeStack so that the gcthings
-    // we use as args will be rooted by the engine as we do conversions and
-    // prepare to do the function call. This adds a fair amount of complexity,
-    // but is a good optimization compared to calling JS_AddRoot for each item.
+    if(XPCPerThreadData::IsMainThread(ccx))
+    {
+        // TODO Remove me in favor of security wrappers.
+        nsIScriptSecurityManager *ssm = XPCWrapper::GetSecurityManager();
+        if(ssm)
+        {
+            nsIPrincipal *objPrincipal =
+                xpc::AccessCheck::getPrincipal(obj->compartment());
+            if(objPrincipal)
+            {
+                JSStackFrame* fp = nsnull;
+                nsresult rv =
+                    ssm->PushContextPrincipal(ccx, JS_FrameIterator(ccx, &fp),
+                                              objPrincipal);
+                if(NS_FAILED(rv))
+                {
+                    JS_ReportOutOfMemory(ccx);
+                    retval = NS_ERROR_OUT_OF_MEMORY;
+                    goto pre_call_clean_up;
+                }
 
-    js_LeaveTrace(cx);
+                principalGuard.principalPushed(ssm);
+            }
+        }
+    }
+
+    // We use js_Invoke so that the gcthings we use as args will be rooted by
+    // the engine as we do conversions and prepare to do the function call.
+
+    js::LeaveTrace(cx);
 
     // setup stack
 
     // if this isn't a function call then we don't need to push extra stuff
-    if(XPT_MD_IS_GETTER(info->flags) || XPT_MD_IS_SETTER(info->flags))
+    if (!(XPT_MD_IS_SETTER(info->flags) || XPT_MD_IS_GETTER(info->flags)))
     {
-        stack_size = argc;
-    }
-    else
-    {
-        // allocate extra space for function and 'this'
-        stack_size = argc + 2;
-
         // We get fval before allocating the stack to avoid gc badness that can
         // happen if the GetProperty call leaves our request and the gc runs
         // while the stack we allocate contains garbage.
@@ -1413,10 +1430,11 @@ nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16 methodIndex,
                             if(newThis)
                             {
                                 jsval v;
+                                xpcObjectHelper helper(newThis);
                                 JSBool ok =
                                   XPCConvert::NativeInterface2JSObject(ccx,
-                                        &v, nsnull, newThis, newWrapperIID,
-                                        nsnull, nsnull, obj, PR_FALSE, PR_FALSE,
+                                        &v, nsnull, helper, newWrapperIID,
+                                        nsnull, PR_FALSE, PR_FALSE,
                                         nsnull);
                                 if(newWrapperIID)
                                     nsMemory::Free(newWrapperIID);
@@ -1425,6 +1443,8 @@ nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16 methodIndex,
                                     goto pre_call_clean_up;
                                 }
                                 thisObj = JSVAL_TO_OBJECT(v);
+                                if(!JS_WrapObject(cx, &thisObj))
+                                    goto pre_call_clean_up;
                             }
                         }
                     }
@@ -1442,59 +1462,21 @@ nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16 methodIndex,
         }
     }
 
-    // if stack_size is zero then we won't be needing a stack
-    if(stack_size && !(stackbase = sp = js_AllocStack(cx, stack_size, &mark)))
+    if (!args.resize(argc))
     {
         retval = NS_ERROR_OUT_OF_MEMORY;
         goto pre_call_clean_up;
     }
 
-    NS_ASSERTION(XPT_MD_IS_GETTER(info->flags) || sp,
-                 "Only a getter needs no stack.");
-
-    // this is a function call, so push function and 'this'
-    if(stack_size != argc)
-    {
-        *sp++ = fval;
-        *sp++ = OBJECT_TO_JSVAL(thisObj);
-    }
-
-    // Figure out what our callee is
-    if(XPT_MD_IS_GETTER(info->flags) || XPT_MD_IS_SETTER(info->flags))
-    {
-        // Pull the getter or setter off of |obj|
-        uintN attrs;
-        JSBool found;
-        JSPropertyOp getter;
-        JSPropertyOp setter;
-        if(!JS_GetPropertyAttrsGetterAndSetter(cx, obj, name,
-                                               &attrs, &found,
-                                               &getter, &setter))
-        {
-            // XXX Do we want to report this exception?
-            JS_ClearPendingException(cx);
-            goto pre_call_clean_up;
-        }
-
-        if(XPT_MD_IS_GETTER(info->flags) && (attrs & JSPROP_GETTER))
-        {
-            // JSPROP_GETTER means the getter is actually a
-            // function object.
-            ccx.SetCallee(JS_FUNC_TO_DATA_PTR(JSObject*, getter));
-        }
-        else if(XPT_MD_IS_SETTER(info->flags) && (attrs & JSPROP_SETTER))
-        {
-            // JSPROP_SETTER means the setter is actually a
-            // function object.
-            ccx.SetCallee(JS_FUNC_TO_DATA_PTR(JSObject*, setter));
-        }
-    }
-    else if(JSVAL_IS_OBJECT(fval))
-    {
-        ccx.SetCallee(JSVAL_TO_OBJECT(fval));
-    }
+    argv = args.jsval_begin();
+    sp = argv;
 
     // build the args
+    // NB: This assignment *looks* wrong because we haven't yet called our
+    // function. However, we *have* already entered the compartmen that we're
+    // about to call, and that's the global that we want here. In other words:
+    // we're trusting the JS engine to come up with a good global to use for
+    // our object (whatever it was).
     for(i = 0; i < argc; i++)
     {
         const nsXPTParamInfo& param = info->params[i];
@@ -1530,7 +1512,7 @@ nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16 methodIndex,
         {
             nsXPTCMiniVariant* pv;
 
-            if(param.IsOut())
+            if(param.IsIndirect())
                 pv = (nsXPTCMiniVariant*) nativeParams[i].val.p;
             else
                 pv = &nativeParams[i];
@@ -1555,7 +1537,7 @@ nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16 methodIndex,
                 if(!XPCConvert::NativeArray2JS(lccx, &val,
                                                (const void**)&pv->val,
                                                datum_type, &param_iid,
-                                               array_count, obj, nsnull))
+                                               array_count, nsnull))
                     goto pre_call_clean_up;
             }
             else if(isSizedString)
@@ -1569,12 +1551,12 @@ nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16 methodIndex,
             else
             {
                 if(!XPCConvert::NativeData2JS(ccx, &val, &pv->val, type,
-                                              &param_iid, obj, nsnull))
+                                              &param_iid, nsnull))
                     goto pre_call_clean_up;
             }
         }
 
-        if(param.IsOut())
+        if(param.IsOut() || param.IsDipper())
         {
             // create an 'out' object
             JSObject* out_obj = NewOutObject(cx, obj);
@@ -1650,51 +1632,34 @@ pre_call_clean_up:
     // Make sure "this" doesn't get deleted during this call.
     nsCOMPtr<nsIXPCWrappedJSClass> kungFuDeathGrip(this);
 
-    result = JSVAL_NULL;
-    AUTO_MARK_JSVAL(ccx, &result);
-
     if(!readyToDoTheCall)
-        goto done;
+        return retval;
 
     // do the deed - note exceptions
 
     JS_ClearPendingException(cx);
 
-    if(XPCPerThreadData::IsMainThread(ccx))
-    {
-        ssm = XPCWrapper::GetSecurityManager();
-        if(ssm)
-        {
-            nsCOMPtr<nsIPrincipal> objPrincipal;
-            ssm->GetObjectPrincipal(ccx, obj, getter_AddRefs(objPrincipal));
-            if(objPrincipal)
-            {
-                JSStackFrame* fp = nsnull;
-                nsresult rv =
-                    ssm->PushContextPrincipal(ccx, JS_FrameIterator(ccx, &fp),
-                                              objPrincipal);
-                if(NS_FAILED(rv))
-                {
-                    JS_ReportOutOfMemory(ccx);
-                    retval = NS_ERROR_OUT_OF_MEMORY;
-                    goto done;
-                }
-
-                popPrincipal = JS_TRUE;
-            }
-        }
-    }
-
+    jsval rval;
     if(XPT_MD_IS_GETTER(info->flags))
-        success = JS_GetProperty(cx, obj, name, &result);
+    {
+        success = JS_GetProperty(cx, obj, name, argv);
+        rval = *argv;
+    }
     else if(XPT_MD_IS_SETTER(info->flags))
-        success = JS_SetProperty(cx, obj, name, sp-1);
+    {
+        success = JS_SetProperty(cx, obj, name, argv);
+        rval = *argv;
+    }
     else
     {
         if(!JSVAL_IS_PRIMITIVE(fval))
         {
-            success = js_Invoke(cx, argc, stackbase, 0);
-            result = *stackbase;
+            uint32 oldOpts = JS_GetOptions(cx);
+            JS_SetOptions(cx, oldOpts | JSOPTION_DONT_REPORT_UNCAUGHT);
+
+            success = JS_CallFunctionValue(cx, thisObj, fval, argc, argv, &rval);
+
+            JS_SetOptions(cx, oldOpts);
         }
         else
         {
@@ -1721,9 +1686,6 @@ pre_call_clean_up:
         }
     }
 
-    if(popPrincipal)
-        ssm->PopContextPrincipal(ccx);
-
     if(!success)
     {
         PRBool forceReport;
@@ -1733,8 +1695,7 @@ pre_call_clean_up:
         // May also want to check if we're moving from content->chrome and force
         // a report in that case.
 
-        retval = CheckForException(ccx, name, GetInterfaceName(), forceReport);
-        goto done;
+        return CheckForException(ccx, name, GetInterfaceName(), forceReport);
     }
 
     ccx.GetThreadData()->SetException(nsnull); // XXX necessary?
@@ -1750,6 +1711,7 @@ pre_call_clean_up:
     for(i = 0; i < paramCount; i++)
     {
         const nsXPTParamInfo& param = info->params[i];
+        NS_ABORT_IF_FALSE(!param.IsShared(), "[shared] implies [noscript]!");
         if(!param.IsOut() && !param.IsDipper())
             continue;
 
@@ -1762,7 +1724,6 @@ pre_call_clean_up:
 
         jsval val;
         uint8 type_tag = type.TagPart();
-        JSBool useAllocator = JS_FALSE;
         nsXPTCMiniVariant* pv;
 
         if(param.IsDipper())
@@ -1771,9 +1732,9 @@ pre_call_clean_up:
             pv = (nsXPTCMiniVariant*) nativeParams[i].val.p;
 
         if(param.IsRetval())
-            val = result;
-        else if(JSVAL_IS_PRIMITIVE(stackbase[i+2]) ||
-                !JS_GetPropertyById(cx, JSVAL_TO_OBJECT(stackbase[i+2]),
+            val = rval;
+        else if(JSVAL_IS_PRIMITIVE(argv[i]) ||
+                !JS_GetPropertyById(cx, JSVAL_TO_OBJECT(argv[i]),
                     mRuntime->GetStringID(XPCJSRuntime::IDX_VALUE),
                     &val))
             break;
@@ -1787,11 +1748,9 @@ pre_call_clean_up:
                                                   &param_iid)))
                 break;
         }
-        else if(type.IsPointer() && !param.IsShared() && !param.IsDipper())
-            useAllocator = JS_TRUE;
 
         if(!XPCConvert::JSData2Native(ccx, &pv->val, val, type,
-                                      useAllocator, &param_iid, nsnull))
+                                      !param.IsDipper(), &param_iid, nsnull))
             break;
     }
 
@@ -1811,7 +1770,6 @@ pre_call_clean_up:
             jsval val;
             nsXPTCMiniVariant* pv;
             nsXPTType datum_type;
-            JSBool useAllocator = JS_FALSE;
             JSUint32 array_count;
             PRBool isArray = type.IsArray();
             PRBool isSizedString = isArray ?
@@ -1822,8 +1780,8 @@ pre_call_clean_up:
             pv = (nsXPTCMiniVariant*) nativeParams[i].val.p;
 
             if(param.IsRetval())
-                val = result;
-            else if(!JS_GetPropertyById(cx, JSVAL_TO_OBJECT(stackbase[i+2]),
+                val = rval;
+            else if(!JS_GetPropertyById(cx, JSVAL_TO_OBJECT(argv[i]),
                         mRuntime->GetStringID(XPCJSRuntime::IDX_VALUE),
                         &val))
                 break;
@@ -1846,8 +1804,6 @@ pre_call_clean_up:
                                              &param_iid))
                    break;
             }
-            else if(type.IsPointer() && !param.IsShared())
-                useAllocator = JS_TRUE;
 
             if(isArray || isSizedString)
             {
@@ -1862,8 +1818,7 @@ pre_call_clean_up:
                 if(array_count &&
                    !XPCConvert::JSArray2Native(ccx, (void**)&pv->val, val,
                                                array_count, array_count,
-                                               datum_type,
-                                               useAllocator, &param_iid,
+                                               datum_type, &param_iid,
                                                nsnull))
                     break;
             }
@@ -1872,14 +1827,13 @@ pre_call_clean_up:
                 if(!XPCConvert::JSStringWithSize2Native(ccx,
                                                    (void*)&pv->val, val,
                                                    array_count, array_count,
-                                                   datum_type, useAllocator,
-                                                   nsnull))
+                                                   datum_type, nsnull))
                     break;
             }
             else
             {
                 if(!XPCConvert::JSData2Native(ccx, &pv->val, val, type,
-                                              useAllocator, &param_iid,
+                                              JS_TRUE, &param_iid,
                                               nsnull))
                     break;
             }
@@ -1935,15 +1889,6 @@ pre_call_clean_up:
         retval = pending_result;
     }
 
-done:
-    if(sp)
-        js_FreeStack(cx, mark);
-
-#ifdef DEBUG_stats_jband
-    endTime = PR_IntervalNow();
-    printf("%s::%s %d ( c->js ) \n", GetInterfaceName(), info->GetName(), PR_IntervalToMilliseconds(endTime-startTime));
-    totalTime += endTime-startTime;
-#endif
     return retval;
 }
 
