@@ -48,6 +48,7 @@
 #include "nsStubMutationObserver.h"
 #include "nsITextControlElement.h"
 #include "nsIStatefulFrame.h"
+#include "nsContentUtils.h" // nsAutoScriptBlocker
 
 class nsIEditor;
 class nsISelectionController;
@@ -76,7 +77,7 @@ public:
   virtual nsIScrollableFrame* GetScrollTargetFrame() {
     if (!IsScrollable())
       return nsnull;
-    return do_QueryFrame(GetFirstChild(nsnull));
+    return do_QueryFrame(GetFirstPrincipalChild());
   }
 
   virtual nscoord GetMinWidth(nsRenderingContext* aRenderingContext);
@@ -127,7 +128,7 @@ public:
 
   // Utility methods to set current widget state
 
-  NS_IMETHOD SetInitialChildList(nsIAtom*        aListName,
+  NS_IMETHOD SetInitialChildList(ChildListID     aListID,
                                  nsFrameList&    aChildList);
 
 //==== BEGIN NSIFORMCONTROLFRAME
@@ -145,8 +146,12 @@ public:
   NS_IMETHOD    CheckFireOnChange();
   NS_IMETHOD    SetSelectionStart(PRInt32 aSelectionStart);
   NS_IMETHOD    SetSelectionEnd(PRInt32 aSelectionEnd);
-  NS_IMETHOD    SetSelectionRange(PRInt32 aSelectionStart, PRInt32 aSelectionEnd);
-  NS_IMETHOD    GetSelectionRange(PRInt32* aSelectionStart, PRInt32* aSelectionEnd);
+  NS_IMETHOD    SetSelectionRange(PRInt32 aSelectionStart,
+                                  PRInt32 aSelectionEnd,
+                                  SelectionDirection aDirection = eNone);
+  NS_IMETHOD    GetSelectionRange(PRInt32* aSelectionStart,
+                                  PRInt32* aSelectionEnd,
+                                  SelectionDirection* aDirection = nsnull);
   NS_IMETHOD    GetOwnedSelectionController(nsISelectionController** aSelCon);
   virtual nsFrameSelection* GetOwnedFrameSelection();
 
@@ -180,6 +185,11 @@ public:
 
   NS_DECL_QUERYFRAME
 
+  // Temp reference to scriptrunner
+  // We could make these auto-Revoking via the "delete" entry for safety
+  NS_DECLARE_FRAME_PROPERTY(TextControlInitializer, nsnull)
+
+
 public: //for methods who access nsTextControlFrame directly
   void FireOnInput(PRBool aTrusted);
   void SetValueChanged(PRBool aValueChanged);
@@ -286,23 +296,37 @@ protected:
   class EditorInitializer : public nsRunnable {
   public:
     EditorInitializer(nsTextControlFrame* aFrame) :
-      mWeakFrame(aFrame),
       mFrame(aFrame) {}
 
     NS_IMETHOD Run() {
-      if (mWeakFrame) {
+      if (mFrame) {
+        // need to block script to avoid bug 669767
+        nsAutoScriptBlocker scriptBlocker;
+
         nsCOMPtr<nsIPresShell> shell =
-          mWeakFrame.GetFrame()->PresContext()->GetPresShell();
+          mFrame->PresContext()->GetPresShell();
         PRBool observes = shell->ObservesNativeAnonMutationsForPrint();
         shell->ObserveNativeAnonMutationsForPrint(PR_TRUE);
+        // This can cause the frame to be destroyed (and call Revoke())
         mFrame->EnsureEditorInitialized();
         shell->ObserveNativeAnonMutationsForPrint(observes);
+
+        // The frame can *still* be destroyed even though we have a scriptblocker
+        // Bug 682684
+        if (!mFrame)
+          return NS_ERROR_FAILURE;
+
+        mFrame->FinishedInitializer();
       }
       return NS_OK;
     }
 
+    // avoids use of nsWeakFrame
+    void Revoke() {
+      mFrame = nsnull;
+    }
+
   private:
-    nsWeakFrame mWeakFrame;
     nsTextControlFrame* mFrame;
   };
 
@@ -375,9 +399,11 @@ protected:
 private:
   //helper methods
   nsresult SetSelectionInternal(nsIDOMNode *aStartNode, PRInt32 aStartOffset,
-                                nsIDOMNode *aEndNode, PRInt32 aEndOffset);
+                                nsIDOMNode *aEndNode, PRInt32 aEndOffset,
+                                SelectionDirection aDirection = eNone);
   nsresult SelectAllOrCollapseToEndOfText(PRBool aSelect);
-  nsresult SetSelectionEndPoints(PRInt32 aSelStart, PRInt32 aSelEnd);
+  nsresult SetSelectionEndPoints(PRInt32 aSelStart, PRInt32 aSelEnd,
+                                 SelectionDirection aDirection = eNone);
 
   // accessors for the notify on input flag
   PRBool GetNotifyOnInput() const { return mNotifyOnInput; }
@@ -388,6 +414,10 @@ private:
    */
   nsresult GetRootNodeAndInitializeEditor(nsIDOMElement **aRootElement);
 
+  void FinishedInitializer() {
+    Properties().Delete(TextControlInitializer());
+  }
+
 private:
   // these packed bools could instead use the high order bits on mState, saving 4 bytes 
   PRPackedBool mUseEditor;
