@@ -40,20 +40,20 @@
 #include "nsCOMPtr.h"
 #include "nsInlineFrame.h"
 #include "nsBlockFrame.h"
+#include "nsPlaceholderFrame.h"
 #include "nsGkAtoms.h"
 #include "nsHTMLParts.h"
 #include "nsStyleContext.h"
 #include "nsIPresShell.h"
 #include "nsPresContext.h"
-#include "nsIRenderingContext.h"
-#include "nsIFontMetrics.h"
+#include "nsRenderingContext.h"
 #include "nsAbsoluteContainingBlock.h"
 #include "nsCSSAnonBoxes.h"
 #include "nsAutoPtr.h"
 #include "nsFrameManager.h"
 #ifdef ACCESSIBILITY
 #include "nsIServiceManager.h"
-#include "nsIAccessibilityService.h"
+#include "nsAccessibilityService.h"
 #endif
 #include "nsDisplayList.h"
 
@@ -93,18 +93,10 @@ nsInlineFrame::GetType() const
 }
 
 static inline PRBool
-IsPaddingZero(nsStyleUnit aUnit, const nsStyleCoord &aCoord)
+IsMarginZero(const nsStyleCoord &aCoord)
 {
-    return ((aUnit == eStyleUnit_Coord && aCoord.GetCoordValue() == 0) ||
-            (aUnit == eStyleUnit_Percent && aCoord.GetPercentValue() == 0.0));
-}
-
-static inline PRBool
-IsMarginZero(nsStyleUnit aUnit, const nsStyleCoord &aCoord)
-{
-    return (aUnit == eStyleUnit_Auto ||
-            (aUnit == eStyleUnit_Coord && aCoord.GetCoordValue() == 0) ||
-            (aUnit == eStyleUnit_Percent && aCoord.GetPercentValue() == 0.0));
+  return aCoord.GetUnit() == eStyleUnit_Auto ||
+         nsLayoutUtils::IsMarginZero(aCoord);
 }
 
 /* virtual */ PRBool
@@ -125,16 +117,12 @@ nsInlineFrame::IsSelfEmpty()
   // ZeroEffectiveSpanBox, anymore, so what should this really be?
   PRBool haveRight =
     border->GetActualBorderWidth(NS_SIDE_RIGHT) != 0 ||
-    !IsPaddingZero(padding->mPadding.GetRightUnit(),
-                   padding->mPadding.GetRight()) ||
-    !IsMarginZero(margin->mMargin.GetRightUnit(),
-                  margin->mMargin.GetRight());
+    !nsLayoutUtils::IsPaddingZero(padding->mPadding.GetRight()) ||
+    !IsMarginZero(margin->mMargin.GetRight());
   PRBool haveLeft =
     border->GetActualBorderWidth(NS_SIDE_LEFT) != 0 ||
-    !IsPaddingZero(padding->mPadding.GetLeftUnit(),
-                   padding->mPadding.GetLeft()) ||
-    !IsMarginZero(margin->mMargin.GetLeftUnit(),
-                  margin->mMargin.GetLeft());
+    !nsLayoutUtils::IsPaddingZero(padding->mPadding.GetLeft()) ||
+    !IsMarginZero(margin->mMargin.GetLeft());
   if (haveLeft || haveRight) {
     if (GetStateBits() & NS_FRAME_IS_SPECIAL) {
       PRBool haveStart, haveEnd;
@@ -153,8 +141,8 @@ nsInlineFrame::IsSelfEmpty()
       // avoid having to get it twice..
       nsIFrame* firstCont = GetFirstContinuation();
       return
-        (!haveStart || nsLayoutUtils::FrameIsInLastPartOfIBSplit(firstCont)) &&
-        (!haveEnd || nsLayoutUtils::FrameIsInFirstPartOfIBSplit(firstCont));
+        (!haveStart || nsLayoutUtils::FrameIsNonFirstInIBSplit(firstCont)) &&
+        (!haveEnd || nsLayoutUtils::FrameIsNonLastInIBSplit(firstCont));
     }
     return PR_FALSE;
   }
@@ -177,7 +165,8 @@ nsInlineFrame::IsEmpty()
 }
 
 PRBool
-nsInlineFrame::PeekOffsetCharacter(PRBool aForward, PRInt32* aOffset)
+nsInlineFrame::PeekOffsetCharacter(PRBool aForward, PRInt32* aOffset,
+                                   PRBool aRespectClusters)
 {
   // Override the implementation in nsFrame, to skip empty inline frames
   NS_ASSERTION (aOffset && *aOffset <= 1, "aOffset out of range");
@@ -205,7 +194,7 @@ nsInlineFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
   // normally don't have any size, but in Editor we use CSS to display
   // an image to represent this "hidden" element.
   if (!mFrames.FirstChild()) {
-    rv = DisplaySelectionOverlay(aBuilder, aLists);
+    rv = DisplaySelectionOverlay(aBuilder, aLists.Content());
   }
   return rv;
 }
@@ -214,21 +203,21 @@ nsInlineFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
 // Reflow methods
 
 /* virtual */ void
-nsInlineFrame::AddInlineMinWidth(nsIRenderingContext *aRenderingContext,
+nsInlineFrame::AddInlineMinWidth(nsRenderingContext *aRenderingContext,
                                  nsIFrame::InlineMinWidthData *aData)
 {
   DoInlineIntrinsicWidth(aRenderingContext, aData, nsLayoutUtils::MIN_WIDTH);
 }
 
 /* virtual */ void
-nsInlineFrame::AddInlinePrefWidth(nsIRenderingContext *aRenderingContext,
+nsInlineFrame::AddInlinePrefWidth(nsRenderingContext *aRenderingContext,
                                   nsIFrame::InlinePrefWidthData *aData)
 {
   DoInlineIntrinsicWidth(aRenderingContext, aData, nsLayoutUtils::PREF_WIDTH);
 }
 
 /* virtual */ nsSize
-nsInlineFrame::ComputeSize(nsIRenderingContext *aRenderingContext,
+nsInlineFrame::ComputeSize(nsRenderingContext *aRenderingContext,
                            nsSize aCBSize, nscoord aAvailableWidth,
                            nsSize aMargin, nsSize aBorder, nsSize aPadding,
                            PRBool aShrinkWrap)
@@ -241,8 +230,9 @@ nsRect
 nsInlineFrame::ComputeTightBounds(gfxContext* aContext) const
 {
   // be conservative
-  if (GetStyleContext()->HasTextDecorations())
-    return GetOverflowRect();
+  if (GetStyleContext()->HasTextDecorationLines()) {
+    return GetVisualOverflowRect();
+  }
   return ComputeSimpleTightBounds(aContext);
 }
 
@@ -278,7 +268,7 @@ nsInlineFrame::ReparentFloatsForInlineChild(nsIFrame* aOurLineContainer,
   nsBlockFrame* frameBlock = nsLayoutUtils::GetAsBlock(ancestor);
   NS_ASSERTION(frameBlock, "ancestor not a block");
 
-  nsFrameList blockChildren(ancestor->GetFirstChild(nsnull));
+  const nsFrameList& blockChildren(ancestor->PrincipalChildList());
   PRBool isOverflow = !blockChildren.ContainsFrame(ancestorBlockChild);
 
   while (PR_TRUE) {
@@ -302,6 +292,19 @@ nsInlineFrame::ReparentFloatsForInlineChild(nsIFrame* aOurLineContainer,
   }
 }
 
+static void
+ReparentChildListStyle(nsPresContext* aPresContext,
+                       const nsFrameList::Slice& aFrames,
+                       nsIFrame* aParentFrame)
+{
+  nsFrameManager *frameManager = aPresContext->FrameManager();
+
+  for (nsFrameList::Enumerator e(aFrames); !e.AtEnd(); e.Next()) {
+    NS_ASSERTION(e.get()->GetParent() == aParentFrame, "Bogus parentage");
+    frameManager->ReparentStyleContext(e.get());
+  }
+}
+
 NS_IMETHODIMP
 nsInlineFrame::Reflow(nsPresContext*          aPresContext,
                       nsHTMLReflowMetrics&     aMetrics,
@@ -326,9 +329,9 @@ nsInlineFrame::Reflow(nsPresContext*          aPresContext,
     if (prevOverflowFrames) {
       // When pushing and pulling frames we need to check for whether any
       // views need to be reparented.
-      nsHTMLContainerFrame::ReparentFrameViewList(aPresContext,
-                                                  *prevOverflowFrames,
-                                                  prevInFlow, this);
+      nsContainerFrame::ReparentFrameViewList(aPresContext,
+                                              *prevOverflowFrames,
+                                              prevInFlow, this);
 
       // Check if we should do the lazilySetParentPointer optimization.
       // Only do it in simple cases where we're being reflowed for the
@@ -352,7 +355,16 @@ nsInlineFrame::Reflow(nsPresContext*          aPresContext,
         }
         // Insert the new frames at the beginning of the child list
         // and set their parent pointer
-        mFrames.InsertFrames(this, nsnull, *prevOverflowFrames);
+        const nsFrameList::Slice& newFrames =
+          mFrames.InsertFrames(this, nsnull, *prevOverflowFrames);
+        // If our prev in flow was under the first continuation of a first-line
+        // frame then we need to reparent the style contexts to remove the
+        // the special first-line styling. In the lazilySetParentPointer case
+        // we reparent the style contexts when we set their parents in
+        // nsInlineFrame::ReflowFrames and nsInlineFrame::ReflowInlineFrame.
+        if (aReflowState.mLineLayout->GetInFirstLine()) {
+          ReparentChildListStyle(aPresContext, newFrames, this);
+        }
       }
     }
   }
@@ -380,17 +392,7 @@ nsInlineFrame::Reflow(nsPresContext*          aPresContext,
     }
   }
 
-  if (IsFrameTreeTooDeep(aReflowState, aMetrics)) {
-#ifdef DEBUG_kipp
-    {
-      extern char* nsPresShell_ReflowStackPointerTop;
-      char marker;
-      char* newsp = (char*) &marker;
-      printf("XXX: frame tree is too deep; approx stack size = %d\n",
-             nsPresShell_ReflowStackPointerTop - newsp);
-    }
-#endif
-    aStatus = NS_FRAME_COMPLETE;
+  if (IsFrameTreeTooDeep(aReflowState, aMetrics, aStatus)) {
     return NS_OK;
   }
 
@@ -399,6 +401,7 @@ nsInlineFrame::Reflow(nsPresContext*          aPresContext,
   InlineReflowState irs;
   irs.mPrevFrame = nsnull;
   irs.mLineContainer = lineContainer;
+  irs.mLineLayout = aReflowState.mLineLayout;
   irs.mNextInFlow = (nsInlineFrame*) GetNextInFlow();
   irs.mSetParentPointer = lazilySetParentPointer;
 
@@ -434,9 +437,9 @@ nsInlineFrame::PullOverflowsFromPrevInFlow()
     nsAutoPtr<nsFrameList> prevOverflowFrames(prevInFlow->StealOverflowFrames());
     if (prevOverflowFrames) {
       // Assume that our prev-in-flow has the same line container that we do.
-      nsHTMLContainerFrame::ReparentFrameViewList(PresContext(),
-                                                  *prevOverflowFrames,
-                                                  prevInFlow, this);
+      nsContainerFrame::ReparentFrameViewList(PresContext(),
+                                              *prevOverflowFrames,
+                                              prevInFlow, this);
       mFrames.InsertFrames(this, nsnull, *prevOverflowFrames);
     }
   }
@@ -453,12 +456,14 @@ nsInlineFrame::ReflowFrames(nsPresContext* aPresContext,
   aStatus = NS_FRAME_COMPLETE;
 
   nsLineLayout* lineLayout = aReflowState.mLineLayout;
+  PRBool inFirstLine = aReflowState.mLineLayout->GetInFirstLine();
+  nsFrameManager* frameManager = aPresContext->FrameManager();
   PRBool ltr = (NS_STYLE_DIRECTION_LTR == aReflowState.mStyleVisibility->mDirection);
   nscoord leftEdge = 0;
   // Don't offset by our start borderpadding if we have a prev continuation or
-  // if we're in the last part of an {ib} split.
+  // if we're in a part of an {ib} split other than the first one.
   if (!GetPrevContinuation() &&
-      !nsLayoutUtils::FrameIsInLastPartOfIBSplit(this)) {
+      !nsLayoutUtils::FrameIsNonFirstInIBSplit(this)) {
     leftEdge = ltr ? aReflowState.mComputedBorderPadding.left
                    : aReflowState.mComputedBorderPadding.right;
   }
@@ -469,7 +474,8 @@ nsInlineFrame::ReflowFrames(nsPresContext* aPresContext,
   availableWidth -= leftEdge;
   availableWidth -= ltr ? aReflowState.mComputedBorderPadding.right
                         : aReflowState.mComputedBorderPadding.left;
-  lineLayout->BeginSpan(this, &aReflowState, leftEdge, leftEdge + availableWidth);
+  lineLayout->BeginSpan(this, &aReflowState, leftEdge,
+                        leftEdge + availableWidth, &mBaseline);
 
   // First reflow our current children
   nsIFrame* frame = mFrames.FirstChild();
@@ -495,6 +501,9 @@ nsInlineFrame::ReflowFrames(nsPresContext* aPresContext,
         ReparentFloatsForInlineChild(irs.mLineContainer, frame, PR_FALSE);
       }
       frame->SetParent(this);
+      if (inFirstLine) {
+        frameManager->ReparentStyleContext(frame);
+      }
       // We also need to check if frame has a next-in-flow. If it does, then set
       // its parent frame pointer, too. Otherwise, if we reflow frame and it's
       // complete we'll fail when deleting its next-in-flow which is no longer
@@ -510,13 +519,16 @@ nsInlineFrame::ReflowFrames(nsPresContext* aPresContext,
           ReparentFloatsForInlineChild(irs.mLineContainer, nextInFlow, PR_FALSE);
         }
         nextInFlow->SetParent(this);
+        if (inFirstLine) {
+          frameManager->ReparentStyleContext(nextInFlow);
+        }
       }
 
       // Fix the parent pointer for ::first-letter child frame next-in-flows,
       // so nsFirstLetterFrame::Reflow can destroy them safely (bug 401042).
       nsIFrame* realFrame = nsPlaceholderFrame::GetRealFrameFor(frame);
       if (realFrame->GetType() == nsGkAtoms::letterFrame) {
-        nsIFrame* child = realFrame->GetFirstChild(nsnull);
+        nsIFrame* child = realFrame->GetFirstPrincipalChild();
         if (child) {
           NS_ASSERTION(child->GetType() == nsGkAtoms::textFrame,
                        "unexpected frame type");
@@ -526,6 +538,9 @@ nsInlineFrame::ReflowFrames(nsPresContext* aPresContext,
                          "unexpected frame type");
             if (mFrames.ContainsFrame(nextInFlow)) {
               nextInFlow->SetParent(this);
+              if (inFirstLine) {
+                frameManager->ReparentStyleContext(nextInFlow);
+              }
             }
             else {
 #ifdef DEBUG              
@@ -607,29 +622,31 @@ nsInlineFrame::ReflowFrames(nsPresContext* aPresContext,
   // Compute final width.
 
   // Make sure to not include our start border and padding if we have a prev
-  // continuation or if we're in the last part of an {ib} split.
+  // continuation or if we're in a part of an {ib} split other than the first
+  // one.
   if (!GetPrevContinuation() &&
-      !nsLayoutUtils::FrameIsInLastPartOfIBSplit(this)) {
+      !nsLayoutUtils::FrameIsNonFirstInIBSplit(this)) {
     aMetrics.width += ltr ? aReflowState.mComputedBorderPadding.left
                           : aReflowState.mComputedBorderPadding.right;
   }
 
   /*
    * We want to only apply the end border and padding if we're the last
-   * continuation and not in the first part of an {ib} split.  To be the last
-   * continuation we have to be complete (so that we won't get a next-in-flow)
-   * and have no non-fluid continuations on our continuation chain.
+   * continuation and either not in an {ib} split or the last part of it.  To
+   * be the last continuation we have to be complete (so that we won't get a
+   * next-in-flow) and have no non-fluid continuations on our continuation
+   * chain.
    */
   if (NS_FRAME_IS_COMPLETE(aStatus) &&
       !GetLastInFlow()->GetNextContinuation() &&
-      !nsLayoutUtils::FrameIsInFirstPartOfIBSplit(this)) {
+      !nsLayoutUtils::FrameIsNonLastInIBSplit(this)) {
     aMetrics.width += ltr ? aReflowState.mComputedBorderPadding.right
                           : aReflowState.mComputedBorderPadding.left;
   }
 
-  nsLayoutUtils::SetFontFromStyle(aReflowState.rendContext, mStyleContext);
-  nsCOMPtr<nsIFontMetrics> fm;
-  aReflowState.rendContext->GetFontMetrics(*getter_AddRefs(fm));
+  nsRefPtr<nsFontMetrics> fm;
+  nsLayoutUtils::GetFontMetricsForFrame(this, getter_AddRefs(fm));
+  aReflowState.rendContext->SetFont(fm);
 
   if (fm) {
     // Compute final height of the frame.
@@ -642,8 +659,8 @@ nsInlineFrame::ReflowFrames(nsPresContext* aPresContext,
     // The height of our box is the sum of our font size plus the top
     // and bottom border and padding. The height of children do not
     // affect our height.
-    fm->GetMaxAscent(aMetrics.ascent);
-    fm->GetHeight(aMetrics.height);
+    aMetrics.ascent = fm->MaxAscent();
+    aMetrics.height = fm->MaxHeight();
   } else {
     NS_WARNING("Cannot get font metrics - defaulting sizes to 0");
     aMetrics.ascent = aMetrics.height = 0;
@@ -654,7 +671,7 @@ nsInlineFrame::ReflowFrames(nsPresContext* aPresContext,
 
   // For now our overflow area is zero. The real value will be
   // computed in |nsLineLayout::RelativePositionFrames|.
-  aMetrics.mOverflowArea.SetRect(0, 0, 0, 0);
+  aMetrics.mOverflowAreas.Clear();
 
 #ifdef NOISY_FINAL_SIZE
   ListTag(stdout);
@@ -681,85 +698,77 @@ nsInlineFrame::ReflowInlineFrame(nsPresContext* aPresContext,
   if (NS_FAILED(rv)) {
     return rv;
   }
-  if (NS_INLINE_IS_BREAK(aStatus)) {
-    if (NS_INLINE_IS_BREAK_BEFORE(aStatus)) {
-      if (aFrame != mFrames.FirstChild()) {
-        // Change break-before status into break-after since we have
-        // already placed at least one child frame. This preserves the
-        // break-type so that it can be propagated upward.
-        aStatus = NS_FRAME_NOT_COMPLETE |
-          NS_INLINE_BREAK | NS_INLINE_BREAK_AFTER |
-          (aStatus & NS_INLINE_BREAK_TYPE_MASK);
-        PushFrames(aPresContext, aFrame, irs.mPrevFrame);
-      }
-      else {
-        // Preserve reflow status when breaking-before our first child
-        // and propagate it upward without modification.
-        // Note: if we're lazily setting the frame pointer for our child 
-        // frames, then we need to set it now. Don't return and leave the
-        // remaining child frames in our child list with the wrong parent
-        // frame pointer...
-        if (irs.mSetParentPointer) {
-          if (irs.mLineContainer && irs.mLineContainer->GetPrevContinuation()) {
-            ReparentFloatsForInlineChild(irs.mLineContainer, aFrame->GetNextSibling(),
-                                         PR_TRUE);
-          }
-          for (nsIFrame* f = aFrame->GetNextSibling(); f; f = f->GetNextSibling()) {
-            f->SetParent(this);
-          }
-        }
-      }
+
+  if (NS_INLINE_IS_BREAK_BEFORE(aStatus)) {
+    if (aFrame != mFrames.FirstChild()) {
+      // Change break-before status into break-after since we have
+      // already placed at least one child frame. This preserves the
+      // break-type so that it can be propagated upward.
+      aStatus = NS_FRAME_NOT_COMPLETE |
+        NS_INLINE_BREAK | NS_INLINE_BREAK_AFTER |
+        (aStatus & NS_INLINE_BREAK_TYPE_MASK);
+      PushFrames(aPresContext, aFrame, irs.mPrevFrame, irs);
     }
     else {
-      // Break-after
-      if (NS_FRAME_IS_NOT_COMPLETE(aStatus)) {
-        nsIFrame* newFrame;
-        rv = CreateNextInFlow(aPresContext, this, aFrame, newFrame);
-        if (NS_FAILED(rv)) {
-          return rv;
+      // Preserve reflow status when breaking-before our first child
+      // and propagate it upward without modification.
+      // Note: if we're lazily setting the frame pointer for our child 
+      // frames, then we need to set it now. Don't return and leave the
+      // remaining child frames in our child list with the wrong parent
+      // frame pointer...
+      if (irs.mSetParentPointer) {
+        if (irs.mLineContainer && irs.mLineContainer->GetPrevContinuation()) {
+          ReparentFloatsForInlineChild(irs.mLineContainer, aFrame->GetNextSibling(),
+                                       PR_TRUE);
         }
-      }
-      nsIFrame* nextFrame = aFrame->GetNextSibling();
-      if (nextFrame) {
-        NS_FRAME_SET_INCOMPLETE(aStatus);
-        PushFrames(aPresContext, nextFrame, aFrame);
-      }
-      else if (nsnull != GetNextInFlow()) {
-        // We must return an incomplete status if there are more child
-        // frames remaining in a next-in-flow that follows this frame.
-        nsInlineFrame* nextInFlow = (nsInlineFrame*) GetNextInFlow();
-        while (nsnull != nextInFlow) {
-          if (nextInFlow->mFrames.NotEmpty()) {
-            NS_FRAME_SET_INCOMPLETE(aStatus);
-            break;
+        for (nsIFrame* f = aFrame->GetNextSibling(); f; f = f->GetNextSibling()) {
+          f->SetParent(this);
+          if (lineLayout->GetInFirstLine()) {
+            aPresContext->FrameManager()->ReparentStyleContext(f);
           }
-          nextInFlow = (nsInlineFrame*) nextInFlow->GetNextInFlow();
         }
       }
     }
+    return NS_OK;
   }
-  else if (NS_FRAME_IS_NOT_COMPLETE(aStatus)) {
-    if (nsGkAtoms::placeholderFrame == aFrame->GetType()) {
-      nsBlockReflowState* blockRS = lineLayout->mBlockRS;
-      blockRS->mBlock->SplitPlaceholder(*blockRS, aFrame);
-      // Allow the parent to continue reflowing
-      aStatus = NS_FRAME_COMPLETE;
+
+  // Create a next-in-flow if needed.
+  if (!NS_FRAME_IS_FULLY_COMPLETE(aStatus)) {
+    nsIFrame* newFrame;
+    rv = CreateNextInFlow(aPresContext, aFrame, newFrame);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
+  }
+
+  if (NS_INLINE_IS_BREAK_AFTER(aStatus)) {
+    nsIFrame* nextFrame = aFrame->GetNextSibling();
+    if (nextFrame) {
+      NS_FRAME_SET_INCOMPLETE(aStatus);
+      PushFrames(aPresContext, nextFrame, aFrame, irs);
     }
     else {
-      nsIFrame* newFrame;
-      rv = CreateNextInFlow(aPresContext, this, aFrame, newFrame);
-      if (NS_FAILED(rv)) {
-        return rv;
-      }
-      if (!reflowingFirstLetter) {
-        nsIFrame* nextFrame = aFrame->GetNextSibling();
-        if (nextFrame) {
-          PushFrames(aPresContext, nextFrame, aFrame);
+      // We must return an incomplete status if there are more child
+      // frames remaining in a next-in-flow that follows this frame.
+      nsInlineFrame* nextInFlow = static_cast<nsInlineFrame*>(GetNextInFlow());
+      while (nextInFlow) {
+        if (nextInFlow->mFrames.NotEmpty()) {
+          NS_FRAME_SET_INCOMPLETE(aStatus);
+          break;
         }
+        nextInFlow = static_cast<nsInlineFrame*>(nextInFlow->GetNextInFlow());
       }
     }
+    return NS_OK;
   }
-  return rv;
+
+  if (!NS_FRAME_IS_FULLY_COMPLETE(aStatus) && !reflowingFirstLetter) {
+    nsIFrame* nextFrame = aFrame->GetNextSibling();
+    if (nextFrame) {
+      PushFrames(aPresContext, nextFrame, aFrame, irs);
+    }
+  }
+  return NS_OK;
 }
 
 nsIFrame*
@@ -773,6 +782,17 @@ nsInlineFrame::PullOneFrame(nsPresContext* aPresContext,
   nsInlineFrame* nextInFlow = irs.mNextInFlow;
   while (nsnull != nextInFlow) {
     frame = nextInFlow->mFrames.FirstChild();
+
+    if (!frame) {
+      // If the principal childlist has no frames, then try moving the overflow
+      // frames to it.
+      nsAutoPtr<nsFrameList> overflowFrames(nextInFlow->StealOverflowFrames());
+      if (overflowFrames) {
+        nextInFlow->mFrames.SetFrames(*overflowFrames);
+        frame = nextInFlow->mFrames.FirstChild();
+      }
+    }
+
     if (nsnull != frame) {
       // If our block has no next continuation, then any floats belonging to
       // the pulled frame must belong to our block already. This check ensures
@@ -784,9 +804,22 @@ nsInlineFrame::PullOneFrame(nsPresContext* aPresContext,
         ReparentFloatsForInlineChild(irs.mLineContainer, frame, PR_FALSE);
       }
       nextInFlow->mFrames.RemoveFirstChild();
+
+      // If we removed the last frame from the principal child list then move
+      // any overflow frames to it.
+      if (!nextInFlow->mFrames.FirstChild()) {
+        nsAutoPtr<nsFrameList> overflowFrames(nextInFlow->StealOverflowFrames());
+        if (overflowFrames) {
+          nextInFlow->mFrames.SetFrames(*overflowFrames);
+        }
+      }
+
       mFrames.InsertFrame(this, irs.mPrevFrame, frame);
       isComplete = PR_FALSE;
-      nsHTMLContainerFrame::ReparentFrameView(aPresContext, frame, nextInFlow, this);
+      if (irs.mLineLayout) {
+        irs.mLineLayout->SetDirtyNextLine();
+      }
+      nsContainerFrame::ReparentFrameView(aPresContext, frame, nextInFlow, this);
       break;
     }
     nextInFlow = (nsInlineFrame*) nextInFlow->GetNextInFlow();
@@ -800,22 +833,24 @@ nsInlineFrame::PullOneFrame(nsPresContext* aPresContext,
 void
 nsInlineFrame::PushFrames(nsPresContext* aPresContext,
                           nsIFrame* aFromChild,
-                          nsIFrame* aPrevSibling)
+                          nsIFrame* aPrevSibling,
+                          InlineReflowState& aState)
 {
-  NS_PRECONDITION(nsnull != aFromChild, "null pointer");
-  NS_PRECONDITION(nsnull != aPrevSibling, "pushing first child");
+  NS_PRECONDITION(aFromChild, "null pointer");
+  NS_PRECONDITION(aPrevSibling, "pushing first child");
   NS_PRECONDITION(aPrevSibling->GetNextSibling() == aFromChild, "bad prev sibling");
 
 #ifdef NOISY_PUSHING
-      printf("%p pushing aFromChild %p, disconnecting from prev sib %p\n", 
-             this, aFromChild, aPrevSibling);
+  printf("%p pushing aFromChild %p, disconnecting from prev sib %p\n", 
+         this, aFromChild, aPrevSibling);
 #endif
-  // Disconnect aFromChild from its previous sibling
-  aPrevSibling->SetNextSibling(nsnull);
 
   // Add the frames to our overflow list (let our next in flow drain
   // our overflow list when it is ready)
-  SetOverflowFrames(aPresContext, aFromChild);
+  SetOverflowFrames(aPresContext, mFrames.RemoveFramesAfter(aPrevSibling));
+  if (aState.mLineLayout) {
+    aState.mLineLayout->SetDirtyNextLine();
+  }
 }
 
 
@@ -853,21 +888,22 @@ nsInlineFrame::GetSkipSides() const
   }
 
   if (GetStateBits() & NS_FRAME_IS_SPECIAL) {
-    // The first part of an {ib} split should always skip the "end" side (as
-    // determined by this frame's direction) and the last part of such a split
-    // should alwas skip the "start" side.  But figuring out which part of the
-    // split we are involves getting our first continuation, which might be
+    // All but the last part of an {ib} split should skip the "end" side (as
+    // determined by this frame's direction) and all but the first part of such
+    // a split should skip the "start" side.  But figuring out which part of
+    // the split we are involves getting our first continuation, which might be
     // expensive.  So don't bother if we already have the relevant bits set.
     PRBool ltr = (NS_STYLE_DIRECTION_LTR == GetStyleVisibility()->mDirection);
     PRIntn startBit = (1 << (ltr ? NS_SIDE_LEFT : NS_SIDE_RIGHT));
     PRIntn endBit = (1 << (ltr ? NS_SIDE_RIGHT : NS_SIDE_LEFT));
     if (((startBit | endBit) & skip) != (startBit | endBit)) {
       // We're missing one of the skip bits, so check whether we need to set it.
-      if (nsLayoutUtils::FrameIsInFirstPartOfIBSplit(this)) {
+      // Only get the first continuation once, as an optimization.
+      nsIFrame* firstContinuation = GetFirstContinuation();
+      if (nsLayoutUtils::FrameIsNonLastInIBSplit(firstContinuation)) {
         skip |= endBit;
-      } else {
-        NS_ASSERTION(nsLayoutUtils::FrameIsInLastPartOfIBSplit(this),
-                     "How did that happen?");
+      }
+      if (nsLayoutUtils::FrameIsNonFirstInIBSplit(firstContinuation)) {
         skip |= startBit;
       }
     }
@@ -876,28 +912,38 @@ nsInlineFrame::GetSkipSides() const
   return skip;
 }
 
+nscoord
+nsInlineFrame::GetBaseline() const
+{
+  return mBaseline;
+}
+
 #ifdef ACCESSIBILITY
-NS_IMETHODIMP nsInlineFrame::GetAccessible(nsIAccessible** aAccessible)
+already_AddRefed<nsAccessible>
+nsInlineFrame::CreateAccessible()
 {
   // Broken image accessibles are created here, because layout
   // replaces the image or image control frame with an inline frame
-  *aAccessible = nsnull;
   nsIAtom *tagAtom = mContent->Tag();
   if ((tagAtom == nsGkAtoms::img || tagAtom == nsGkAtoms::input || 
-       tagAtom == nsGkAtoms::label) && mContent->IsNodeOfType(nsINode::eHTML)) {
+       tagAtom == nsGkAtoms::label) && mContent->IsHTML()) {
     // Only get accessibility service if we're going to use it
-    nsCOMPtr<nsIAccessibilityService> accService(do_GetService("@mozilla.org/accessibilityService;1"));
+
+    nsAccessibilityService* accService = nsIPresShell::AccService();
     if (!accService)
-      return NS_ERROR_FAILURE;
+      return nsnull;
     if (tagAtom == nsGkAtoms::input)  // Broken <input type=image ... />
-      return accService->CreateHTMLButtonAccessible(static_cast<nsIFrame*>(this), aAccessible);
+      return accService->CreateHTMLButtonAccessible(mContent,
+                                                    PresContext()->PresShell());
     else if (tagAtom == nsGkAtoms::img)  // Create accessible for broken <img>
-      return accService->CreateHTMLImageAccessible(static_cast<nsIFrame*>(this), aAccessible);
+      return accService->CreateHTMLImageAccessible(mContent,
+                                                   PresContext()->PresShell());
     else if (tagAtom == nsGkAtoms::label)  // Creat accessible for <label>
-      return accService->CreateHTMLLabelAccessible(static_cast<nsIFrame*>(this), aAccessible);
+      return accService->CreateHTMLLabelAccessible(mContent,
+                                                   PresContext()->PresShell());
   }
 
-  return NS_ERROR_FAILURE;
+  return nsnull;
 }
 #endif
 
@@ -905,19 +951,6 @@ NS_IMETHODIMP nsInlineFrame::GetAccessible(nsIAccessible** aAccessible)
 
 // nsLineFrame implementation
 
-static void
-ReParentChildListStyle(nsPresContext* aPresContext,
-                       const nsFrameList::Slice& aFrames,
-                       nsIFrame* aParentFrame)
-{
-  nsFrameManager *frameManager = aPresContext->FrameManager();
-
-  for (nsFrameList::Enumerator e(aFrames); !e.AtEnd(); e.Next()) {
-    NS_ASSERTION(e.get()->GetParent() == aParentFrame, "Bogus parentage");
-    frameManager->ReParentStyleContext(e.get());
-  }
-}
-
 nsIFrame*
 NS_NewFirstLineFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
@@ -940,18 +973,6 @@ nsFirstLineFrame::GetType() const
   return nsGkAtoms::lineFrame;
 }
 
-void
-nsFirstLineFrame::StealFramesFrom(nsIFrame* aFrame)
-{
-  nsIFrame* prevFrame = mFrames.GetPrevSiblingFor(aFrame);
-  if (prevFrame) {
-    prevFrame->SetNextSibling(nsnull);
-  }
-  else {
-    mFrames.SetFrames(nsnull);
-  }
-}
-
 nsIFrame*
 nsFirstLineFrame::PullOneFrame(nsPresContext* aPresContext, InlineReflowState& irs,
                                PRBool* aIsComplete)
@@ -961,7 +982,7 @@ nsFirstLineFrame::PullOneFrame(nsPresContext* aPresContext, InlineReflowState& i
     // We are a first-line frame. Fixup the child frames
     // style-context that we just pulled.
     NS_ASSERTION(frame->GetParent() == this, "Incorrect parent?");
-    aPresContext->FrameManager()->ReParentStyleContext(frame);
+    aPresContext->FrameManager()->ReparentStyleContext(frame);
   }
   return frame;
 }
@@ -991,7 +1012,7 @@ nsFirstLineFrame::Reflow(nsPresContext* aPresContext,
       }
       const nsFrameList::Slice& newFrames =
         mFrames.InsertFrames(this, nsnull, *prevOverflowFrames);
-      ReParentChildListStyle(aPresContext, newFrames, this);
+      ReparentChildListStyle(aPresContext, newFrames, this);
     }
   }
 
@@ -1002,7 +1023,7 @@ nsFirstLineFrame::Reflow(nsPresContext* aPresContext,
 
     const nsFrameList::Slice& newFrames =
       mFrames.AppendFrames(nsnull, *overflowFrames);
-    ReParentChildListStyle(aPresContext, newFrames, this);
+    ReparentChildListStyle(aPresContext, newFrames, this);
   }
 
   // Set our own reflow state (additional state above and beyond
@@ -1010,6 +1031,7 @@ nsFirstLineFrame::Reflow(nsPresContext* aPresContext,
   InlineReflowState irs;
   irs.mPrevFrame = nsnull;
   irs.mLineContainer = lineContainer;
+  irs.mLineLayout = aReflowState.mLineLayout;
   irs.mNextInFlow = (nsInlineFrame*) GetNextInFlow();
 
   nsresult rv;
@@ -1057,14 +1079,13 @@ nsFirstLineFrame::Reflow(nsPresContext* aPresContext,
         // of the parent frame.
         nsRefPtr<nsStyleContext> newSC;
         newSC = aPresContext->StyleSet()->
-          ResolvePseudoStyleFor(nsnull,
-                                nsCSSAnonBoxes::mozLineFrame, parentContext);
+          ResolveAnonymousBoxStyle(nsCSSAnonBoxes::mozLineFrame, parentContext);
         if (newSC) {
           // Switch to the new style context.
           SetStyleContext(newSC);
 
           // Re-resolve all children
-          ReParentChildListStyle(aPresContext, mFrames, this);
+          ReparentChildListStyle(aPresContext, mFrames, this);
         }
       }
     }
@@ -1091,7 +1112,7 @@ nsFirstLineFrame::PullOverflowsFromPrevInFlow()
       // Assume that our prev-in-flow has the same line container that we do.
       const nsFrameList::Slice& newFrames =
         mFrames.InsertFrames(this, nsnull, *prevOverflowFrames);
-      ReParentChildListStyle(PresContext(), newFrames, this);
+      ReparentChildListStyle(PresContext(), newFrames, this);
     }
   }
 }
@@ -1107,69 +1128,70 @@ NS_NewPositionedInlineFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 NS_IMPL_FRAMEARENA_HELPERS(nsPositionedInlineFrame)
 
 void
-nsPositionedInlineFrame::Destroy()
+nsPositionedInlineFrame::DestroyFrom(nsIFrame* aDestructRoot)
 {
-  mAbsoluteContainer.DestroyFrames(this);
-  nsInlineFrame::Destroy();
+  mAbsoluteContainer.DestroyFrames(this, aDestructRoot);
+  nsInlineFrame::DestroyFrom(aDestructRoot);
 }
 
 NS_IMETHODIMP
-nsPositionedInlineFrame::SetInitialChildList(nsIAtom*        aListName,
+nsPositionedInlineFrame::SetInitialChildList(ChildListID     aListID,
                                              nsFrameList&    aChildList)
 {
   nsresult  rv;
 
-  if (nsGkAtoms::absoluteList == aListName) {
-    rv = mAbsoluteContainer.SetInitialChildList(this, aListName, aChildList);
+  if (kAbsoluteList == aListID) {
+    rv = mAbsoluteContainer.SetInitialChildList(this, aListID, aChildList);
   } else {
-    rv = nsInlineFrame::SetInitialChildList(aListName, aChildList);
+    rv = nsInlineFrame::SetInitialChildList(aListID, aChildList);
   }
 
   return rv;
 }
 
 NS_IMETHODIMP
-nsPositionedInlineFrame::AppendFrames(nsIAtom*        aListName,
+nsPositionedInlineFrame::AppendFrames(ChildListID     aListID,
                                       nsFrameList&    aFrameList)
 {
   nsresult  rv;
   
-  if (nsGkAtoms::absoluteList == aListName) {
-    rv = mAbsoluteContainer.AppendFrames(this, aListName, aFrameList);
+  if (kAbsoluteList == aListID) {
+    rv = mAbsoluteContainer.AppendFrames(this, aListID, aFrameList);
   } else {
-    rv = nsInlineFrame::AppendFrames(aListName, aFrameList);
+    rv = nsInlineFrame::AppendFrames(aListID, aFrameList);
   }
 
   return rv;
 }
   
 NS_IMETHODIMP
-nsPositionedInlineFrame::InsertFrames(nsIAtom*        aListName,
+nsPositionedInlineFrame::InsertFrames(ChildListID     aListID,
                                       nsIFrame*       aPrevFrame,
                                       nsFrameList&    aFrameList)
 {
   nsresult  rv;
 
-  if (nsGkAtoms::absoluteList == aListName) {
-    rv = mAbsoluteContainer.InsertFrames(this, aListName, aPrevFrame,
+  if (kAbsoluteList == aListID) {
+    rv = mAbsoluteContainer.InsertFrames(this, aListID, aPrevFrame,
                                          aFrameList);
   } else {
-    rv = nsInlineFrame::InsertFrames(aListName, aPrevFrame, aFrameList);
+    rv = nsInlineFrame::InsertFrames(aListID, aPrevFrame, aFrameList);
   }
 
   return rv;
 }
   
 NS_IMETHODIMP
-nsPositionedInlineFrame::RemoveFrame(nsIAtom*        aListName,
+nsPositionedInlineFrame::RemoveFrame(ChildListID     aListID,
                                      nsIFrame*       aOldFrame)
 {
   nsresult  rv;
 
-  if (nsGkAtoms::absoluteList == aListName) {
-    rv = mAbsoluteContainer.RemoveFrame(this, aListName, aOldFrame);
+  if (kAbsoluteList == aListID) {
+    mAbsoluteContainer.RemoveFrame(this, aListID, aOldFrame);
+    rv = NS_OK;
   } else {
-    rv = nsInlineFrame::RemoveFrame(aListName, aOldFrame);
+    rv = nsInlineFrame::RemoveFrame(aListID, aOldFrame);
   }
 
   return rv;
@@ -1185,22 +1207,20 @@ nsPositionedInlineFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
   return nsHTMLContainerFrame::BuildDisplayList(aBuilder, aDirtyRect, aLists);
 }
 
-nsIAtom*
-nsPositionedInlineFrame::GetAdditionalChildListName(PRInt32 aIndex) const
-{
-  if (0 == aIndex) {
-    return nsGkAtoms::absoluteList;
-  }
-  return nsnull;
-}
-
 nsFrameList
-nsPositionedInlineFrame::GetChildList(nsIAtom* aListName) const
+nsPositionedInlineFrame::GetChildList(ChildListID aListID) const
 {
-  if (nsGkAtoms::absoluteList == aListName)
+  if (kAbsoluteList == aListID)
     return mAbsoluteContainer.GetChildList();
 
-  return nsInlineFrame::GetChildList(aListName);
+  return nsInlineFrame::GetChildList(aListID);
+}
+
+void
+nsPositionedInlineFrame::GetChildLists(nsTArray<ChildList>* aLists) const
+{
+  nsInlineFrame::GetChildLists(aLists);
+  mAbsoluteContainer.AppendChildList(aLists, kAbsoluteList);
 }
 
 nsIAtom*
@@ -1249,7 +1269,7 @@ nsPositionedInlineFrame::Reflow(nsPresContext*          aPresContext,
     rv = mAbsoluteContainer.Reflow(this, aPresContext, aReflowState, aStatus,
                                    containingBlockWidth, containingBlockHeight,
                                    PR_TRUE, PR_TRUE, PR_TRUE, // XXX could be optimized
-                                   &aDesiredSize.mOverflowArea);
+                                   &aDesiredSize.mOverflowAreas);
   }
 
   return rv;
