@@ -42,8 +42,6 @@
 #include "nsStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsINameSpaceManager.h"
-#include "nsIRenderingContext.h"
-#include "nsIFontMetrics.h"
 
 #include "nsCSSRendering.h"
 #include "prprf.h"         // For PR_snprintf()
@@ -56,7 +54,6 @@
 #include "nsIDOMElement.h"
 
 #include "nsIDOMEventTarget.h"
-#include "nsIDOMMouseListener.h"
 
 #include "nsMathMLmactionFrame.h"
 #include "nsAutoPtr.h"
@@ -72,7 +69,6 @@
 #define NS_MATHML_ACTION_TYPE_TOGGLE       1
 #define NS_MATHML_ACTION_TYPE_STATUSLINE   2
 #define NS_MATHML_ACTION_TYPE_TOOLTIP      3 // unsupported
-#define NS_MATHML_ACTION_TYPE_RESTYLE      4
 
 
 nsIFrame*
@@ -87,8 +83,14 @@ nsMathMLmactionFrame::~nsMathMLmactionFrame()
 {
   // unregister us as a mouse event listener ...
   //  printf("maction:%p unregistering as mouse event listener ...\n", this);
-  if (mListener)
-    mContent->RemoveEventListenerByIID(mListener, NS_GET_IID(nsIDOMMouseListener));
+  if (mListener) {
+    mContent->RemoveEventListener(NS_LITERAL_STRING("click"), mListener,
+                                  PR_FALSE);
+    mContent->RemoveEventListener(NS_LITERAL_STRING("mouseover"), mListener,
+                                  PR_FALSE);
+    mContent->RemoveEventListener(NS_LITERAL_STRING("mouseout"), mListener,
+                                  PR_FALSE);
+  }
 }
 
 NS_IMETHODIMP
@@ -125,37 +127,6 @@ nsMathMLmactionFrame::Init(nsIContent*      aContent,
         mActionType = NS_MATHML_ACTION_TYPE_STATUSLINE;
     }
 
-    if (NS_MATHML_ACTION_TYPE_NONE == mActionType) {
-      // expected restyle prefix (8ch)...
-      if (8 < value.Length() && 0 == value.Find("restyle#")) {
-        mActionType = NS_MATHML_ACTION_TYPE_RESTYLE;
-        mRestyle = value;
-
-        // Here is the situation:
-        // When the attribute [actiontype="restyle#id"] is set, the Style System has
-        // given us the associated style. But we want to start with our default style.
-
-        // So... first, remove the attribute actiontype="restyle#id"
-        // XXXbz this is pretty messed up, since this can change whether we
-        // should have a frame at all.  This really needs a better solution.
-        PRBool notify = PR_FALSE; // don't trigger a reflow yet!
-        aContent->UnsetAttr(kNameSpaceID_None, nsGkAtoms::actiontype_, notify);
-
-        // then, re-resolve our style
-        nsStyleContext* parentStyleContext = GetStyleContext()->GetParent();
-        newStyleContext = PresContext()->StyleSet()->
-          ResolveStyleFor(aContent->AsElement(), parentStyleContext);
-
-        if (!newStyleContext) 
-          mRestyle.Truncate();
-        else {
-          if (newStyleContext != GetStyleContext())
-            SetStyleContextWithoutNotification(newStyleContext);
-          else
-            mRestyle.Truncate();
-        }
-      }
-    }
   }
 
   // Let the base class do the rest
@@ -202,7 +173,8 @@ nsMathMLmactionFrame::GetSelectedFrame()
   nsAutoString value;
   PRInt32 selection; 
 
-  mContent->GetAttr(kNameSpaceID_None, nsGkAtoms::selection_, value);
+  GetAttribute(mContent, mPresentationData.mstyle, nsGkAtoms::selection_,
+               value);
   if (!value.IsEmpty()) {
     PRInt32 errorCode;
     selection = value.ToInteger(&errorCode);
@@ -242,10 +214,10 @@ nsMathMLmactionFrame::GetSelectedFrame()
 }
 
 NS_IMETHODIMP
-nsMathMLmactionFrame::SetInitialChildList(nsIAtom*        aListName,
+nsMathMLmactionFrame::SetInitialChildList(ChildListID     aListID,
                                           nsFrameList&    aChildList)
 {
-  nsresult rv = nsMathMLContainerFrame::SetInitialChildList(aListName, aChildList);
+  nsresult rv = nsMathMLContainerFrame::SetInitialChildList(aListID, aChildList);
 
   // This very first call to GetSelectedFrame() will cause us to be marked as an
   // embellished operator if the selected child is an embellished operator
@@ -256,7 +228,12 @@ nsMathMLmactionFrame::SetInitialChildList(nsIAtom*        aListName,
     // create mouse event listener and register it
     mListener = new nsMathMLmactionFrame::MouseListener(this);
     // printf("maction:%p registering as mouse event listener ...\n", this);
-    mContent->AddEventListenerByIID(mListener, NS_GET_IID(nsIDOMMouseListener));
+    mContent->AddEventListener(NS_LITERAL_STRING("click"), mListener,
+                               PR_FALSE, PR_FALSE);
+    mContent->AddEventListener(NS_LITERAL_STRING("mouseover"), mListener,
+                               PR_FALSE, PR_FALSE);
+    mContent->AddEventListener(NS_LITERAL_STRING("mouseout"), mListener,
+                               PR_FALSE, PR_FALSE);
   }
   return rv;
 }
@@ -297,7 +274,7 @@ nsMathMLmactionFrame::Reflow(nsPresContext*          aPresContext,
   aStatus = NS_FRAME_COMPLETE;
   aDesiredSize.width = aDesiredSize.height = 0;
   aDesiredSize.ascent = 0;
-  mBoundingMetrics.Clear();
+  mBoundingMetrics = nsBoundingMetrics();
   nsIFrame* childFrame = GetSelectedFrame();
   if (childFrame) {
     nsSize availSize(aReflowState.ComputedWidth(), NS_UNCONSTRAINEDSIZE);
@@ -316,13 +293,13 @@ nsMathMLmactionFrame::Reflow(nsPresContext*          aPresContext,
 
 // Only place the selected child ...
 /* virtual */ nsresult
-nsMathMLmactionFrame::Place(nsIRenderingContext& aRenderingContext,
+nsMathMLmactionFrame::Place(nsRenderingContext& aRenderingContext,
                             PRBool               aPlaceOrigin,
                             nsHTMLReflowMetrics& aDesiredSize)
 {
   aDesiredSize.width = aDesiredSize.height = 0;
   aDesiredSize.ascent = 0;
-  mBoundingMetrics.Clear();
+  mBoundingMetrics = nsBoundingMetrics();
   nsIFrame* childFrame = GetSelectedFrame();
   if (childFrame) {
     GetReflowAndBoundingMetricsFor(childFrame, aDesiredSize, mBoundingMetrics);
@@ -340,9 +317,8 @@ nsMathMLmactionFrame::Place(nsIRenderingContext& aRenderingContext,
 // Event handlers 
 // ################################################################
 
-NS_IMPL_ISUPPORTS2(nsMathMLmactionFrame::MouseListener,
-                   nsIDOMEventListener,
-                   nsIDOMMouseListener)
+NS_IMPL_ISUPPORTS1(nsMathMLmactionFrame::MouseListener,
+                   nsIDOMEventListener)
 
 
 // helper to show a msg on the status bar
@@ -367,9 +343,23 @@ ShowStatus(nsPresContext* aPresContext, nsString& aStatusMsg)
 }
 
 NS_IMETHODIMP
-nsMathMLmactionFrame::MouseListener::MouseOver(nsIDOMEvent* aMouseEvent)
+nsMathMLmactionFrame::MouseListener::HandleEvent(nsIDOMEvent* aEvent)
 {
-  mOwner->MouseOver();
+  nsAutoString eventType;
+  aEvent->GetType(eventType);
+  if (eventType.EqualsLiteral("mouseover")) {
+    mOwner->MouseOver();
+  }
+  else if (eventType.EqualsLiteral("click")) {
+    mOwner->MouseClick();
+  }
+  else if (eventType.EqualsLiteral("mouseout")) {
+    mOwner->MouseOut();
+  }
+  else {
+    NS_ABORT();
+  }
+
   return NS_OK;
 }
 
@@ -388,13 +378,6 @@ nsMathMLmactionFrame::MouseOver()
   }
 }
 
-NS_IMETHODIMP
-nsMathMLmactionFrame::MouseListener::MouseOut(nsIDOMEvent* aMouseEvent) 
-{
-  mOwner->MouseOut();
-  return NS_OK;
-}
-
 void
 nsMathMLmactionFrame::MouseOut()
 {
@@ -406,13 +389,6 @@ nsMathMLmactionFrame::MouseOut()
   }
 }
 
-NS_IMETHODIMP
-nsMathMLmactionFrame::MouseListener::MouseClick(nsIDOMEvent* aMouseEvent)
-{
-  mOwner->MouseClick();
-  return NS_OK;
-}
-
 void
 nsMathMLmactionFrame::MouseClick()
 {
@@ -432,21 +408,4 @@ nsMathMLmactionFrame::MouseClick()
                          NS_FRAME_IS_DIRTY);
     }
   }
-  else if (NS_MATHML_ACTION_TYPE_RESTYLE == mActionType) {
-    if (!mRestyle.IsEmpty()) {
-      nsCOMPtr<nsIDOMElement> node( do_QueryInterface(mContent) );
-      if (node.get()) {
-        if (nsContentUtils::HasNonEmptyAttr(mContent, kNameSpaceID_None,
-                                            nsGkAtoms::actiontype_))
-          node->RemoveAttribute(NS_LITERAL_STRING("actiontype"));
-        else
-          node->SetAttribute(NS_LITERAL_STRING("actiontype"), mRestyle);
-
-        // Trigger a style change reflow
-        PresContext()->PresShell()->
-          FrameNeedsReflow(mSelectedFrame, nsIPresShell::eStyleChange,
-                           NS_FRAME_IS_DIRTY);
-      }
-    }
-  }
 }
