@@ -51,18 +51,16 @@
 #include "mozilla/css/Loader.h"
 #include "nsIRunnable.h"
 #include "nsIUnicharStreamLoader.h"
+#include "nsSyncLoadService.h"
 #include "nsCOMPtr.h"
 #include "nsCOMArray.h"
 #include "nsString.h"
 #include "nsIContent.h"
-#include "nsIDOMNode.h"
-#include "nsIDOMWindow.h"
 #include "nsIDocument.h"
-#include "nsIDOMNSDocumentStyle.h"
-#include "nsIUnicharInputStream.h"
-#include "nsIConverterInputStream.h"
+#include "nsIDOMNode.h"
+#include "nsIDOMDocument.h"
+#include "nsIDOMWindow.h"
 #include "nsICharsetAlias.h"
-#include "nsUnicharUtils.h"
 #include "nsHashtable.h"
 #include "nsIURI.h"
 #include "nsIServiceManager.h"
@@ -71,12 +69,10 @@
 #include "nsCRT.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsContentPolicyUtils.h"
-#include "nsITimelineService.h"
 #include "nsIHttpChannel.h"
 #include "nsIScriptError.h"
 #include "nsMimeTypes.h"
 #include "nsIAtom.h"
-#include "nsIDOM3Node.h"
 #include "nsCSSStyleSheet.h"
 #include "nsIStyleSheetLinkingElement.h"
 #include "nsICSSLoaderObserver.h"
@@ -85,6 +81,7 @@
 #include "nsThreadUtils.h"
 #include "nsGkAtoms.h"
 #include "nsDocShellCID.h"
+#include "nsIThreadInternal.h"
 
 #ifdef MOZ_XUL
 #include "nsXULPrototypeCache.h"
@@ -136,7 +133,8 @@ namespace css {
  *********************************************/
 
 class SheetLoadData : public nsIRunnable,
-                      public nsIUnicharStreamLoaderObserver
+                      public nsIUnicharStreamLoaderObserver,
+                      public nsIThreadObserver
 {
 public:
   virtual ~SheetLoadData(void);
@@ -171,8 +169,11 @@ public:
 
   already_AddRefed<nsIURI> GetReferrerURI();
 
+  void ScheduleLoadEventIfNeeded(nsresult aStatus);
+
   NS_DECL_ISUPPORTS
   NS_DECL_NSIRUNNABLE
+  NS_DECL_NSITHREADOBSERVER
   NS_DECL_NSIUNICHARSTREAMLOADEROBSERVER
 
   // Hold a ref to the CSSLoader so we can call back to it to let it
@@ -222,13 +223,15 @@ public:
 
   // mIsCancelled is set to true when a sheet load is stopped by
   // Stop() or StopLoadingSheet() (which was removed in Bug 556446).
-  // SheetLoadData::OnStreamComplete() checks this to avoid parsing 
+  // SheetLoadData::OnStreamComplete() checks this to avoid parsing
   // sheets that have been cancelled and such.
   PRPackedBool               mIsCancelled : 1;
 
   // mMustNotify is true if the load data is being loaded async and
   // the original function call that started the load has returned.
-  // XXXbz sort our relationship with load/error events!
+  // This applies only to observer notifications; load/error events
+  // are fired for any SheetLoadData that has a non-null
+  // mOwningElement.
   PRPackedBool               mMustNotify : 1;
 
   // mWasAlternate is true if the sheet was an alternate when the load data was
@@ -244,8 +247,12 @@ public:
   // loads.
   PRPackedBool               mUseSystemPrincipal : 1;
 
+  // If true, this SheetLoadData is being used as a way to handle
+  // async observer notification for an already-complete sheet.
+  PRPackedBool               mSheetAlreadyComplete : 1;
+
   // This is the element that imported the sheet.  Needed to get the
-  // charset set on it.
+  // charset set on it and to fire load/error events.
   nsCOMPtr<nsIStyleSheetLinkingElement> mOwningElement;
 
   // The observer that wishes to be notified of load completion
@@ -257,6 +264,15 @@ public:
   // The charset to use if the transport and sheet don't indicate one.
   // May be empty.  Must be empty if mOwningElement is non-null.
   nsCString                             mCharsetHint;
+
+  // The status our load ended up with; this determines whether we
+  // should fire error events or load events.  This gets initialized
+  // by ScheduleLoadEventIfNeeded, and is only used after that has
+  // been called.
+  nsresult                              mStatus;
+
+private:
+  void FireLoadEvent(nsIThreadInternal* aThread);
 };
 
 #ifdef MOZ_LOGGING
@@ -308,7 +324,8 @@ static const char* const gStateStrings[] = {
 /********************************
  * SheetLoadData implementation *
  ********************************/
-NS_IMPL_ISUPPORTS2(SheetLoadData, nsIUnicharStreamLoaderObserver, nsIRunnable)
+NS_IMPL_ISUPPORTS3(SheetLoadData, nsIUnicharStreamLoaderObserver, nsIRunnable,
+                   nsIThreadObserver)
 
 SheetLoadData::SheetLoadData(Loader* aLoader,
                              const nsSubstring& aTitle,
@@ -334,6 +351,7 @@ SheetLoadData::SheetLoadData(Loader* aLoader,
     mWasAlternate(aIsAlternate),
     mAllowUnsafeRules(PR_FALSE),
     mUseSystemPrincipal(PR_FALSE),
+    mSheetAlreadyComplete(PR_FALSE),
     mOwningElement(aOwningElement),
     mObserver(aObserver),
     mLoaderPrincipal(aLoaderPrincipal)
@@ -363,6 +381,7 @@ SheetLoadData::SheetLoadData(Loader* aLoader,
     mWasAlternate(PR_FALSE),
     mAllowUnsafeRules(PR_FALSE),
     mUseSystemPrincipal(PR_FALSE),
+    mSheetAlreadyComplete(PR_FALSE),
     mOwningElement(nsnull),
     mObserver(aObserver),
     mLoaderPrincipal(aLoaderPrincipal)
@@ -406,6 +425,7 @@ SheetLoadData::SheetLoadData(Loader* aLoader,
     mWasAlternate(PR_FALSE),
     mAllowUnsafeRules(aAllowUnsafeRules),
     mUseSystemPrincipal(aUseSystemPrincipal),
+    mSheetAlreadyComplete(PR_FALSE),
     mOwningElement(nsnull),
     mObserver(aObserver),
     mLoaderPrincipal(aLoaderPrincipal),
@@ -432,6 +452,78 @@ SheetLoadData::Run()
   return NS_OK;
 }
 
+NS_IMETHODIMP
+SheetLoadData::OnDispatchedEvent(nsIThreadInternal* aThread)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+SheetLoadData::OnProcessNextEvent(nsIThreadInternal* aThread,
+                                  PRBool aMayWait,
+                                  PRUint32 aRecursionDepth)
+{
+  // We want to fire our load even before or after event processing,
+  // whichever comes first.
+  FireLoadEvent(aThread);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+SheetLoadData::AfterProcessNextEvent(nsIThreadInternal* aThread,
+                                     PRUint32 aRecursionDepth)
+{
+  // We want to fire our load even before or after event processing,
+  // whichever comes first.
+  FireLoadEvent(aThread);
+  return NS_OK;
+}
+
+void
+SheetLoadData::FireLoadEvent(nsIThreadInternal* aThread)
+{
+  
+  // First remove ourselves as a thread observer.  But we need to keep
+  // ourselves alive while doing that!
+  nsRefPtr<SheetLoadData> kungFuDeathGrip(this);
+  aThread->RemoveObserver(this);
+
+  // Now fire the event
+  nsCOMPtr<nsINode> node = do_QueryInterface(mOwningElement);
+  NS_ASSERTION(node, "How did that happen???");
+
+  nsContentUtils::DispatchTrustedEvent(node->GetOwnerDoc(),
+                                       node,
+                                       NS_SUCCEEDED(mStatus) ?
+                                         NS_LITERAL_STRING("load") :
+                                         NS_LITERAL_STRING("error"),
+                                       PR_FALSE, PR_FALSE);
+
+  // And unblock onload
+  if (mLoader->mDocument) {
+    mLoader->mDocument->UnblockOnload(PR_TRUE);
+  }  
+}
+
+void
+SheetLoadData::ScheduleLoadEventIfNeeded(nsresult aStatus)
+{
+  if (!mOwningElement) {
+    return;
+  }
+
+  mStatus = aStatus;
+
+  nsCOMPtr<nsIThread> thread = do_GetMainThread();
+  nsCOMPtr<nsIThreadInternal> internalThread = do_QueryInterface(thread);
+  if (NS_SUCCEEDED(internalThread->AddObserver(this))) {
+    // Make sure to block onload here
+    if (mLoader->mDocument) {
+      mLoader->mDocument->BlockOnload();
+    }
+  }
+}
+
 /*************************
  * Loader Implementation *
  *************************/
@@ -459,7 +551,7 @@ Loader::Loader(nsIDocument* aDocument)
   // We can just use the preferred set, since there are no sheets in the
   // document yet (if there are, how did they get there? _we_ load the sheets!)
   // and hence the selected set makes no sense at this time.
-  nsCOMPtr<nsIDOMNSDocumentStyle> domDoc(do_QueryInterface(mDocument));
+  nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(mDocument);
   if (domDoc) {
     domDoc->GetPreferredStyleSheetSet(mPreferredSheet);
   }
@@ -498,7 +590,7 @@ CollectNonAlternates(URIAndPrincipalHashKey *aKey,
 {
   NS_PRECONDITION(aData, "Must have a data");
   NS_PRECONDITION(aClosure, "Must have an array");
-  
+
   // Note that we don't want to affect what the selected style set is,
   // so use PR_TRUE for aHasAlternateRel.
   if (aData->mLoader->IsAlternate(aData->mTitle, PR_TRUE)) {
@@ -513,7 +605,7 @@ nsresult
 Loader::SetPreferredSheet(const nsAString& aTitle)
 {
 #ifdef DEBUG
-  nsCOMPtr<nsIDOMNSDocumentStyle> doc(do_QueryInterface(mDocument));
+  nsCOMPtr<nsIDOMDocument> doc = do_QueryInterface(mDocument);
   if (doc) {
     nsAutoString currentPreferred;
     doc->GetLastStyleSheetSet(currentPreferred);
@@ -521,10 +613,10 @@ Loader::SetPreferredSheet(const nsAString& aTitle)
       doc->GetPreferredStyleSheetSet(currentPreferred);
     }
     NS_ASSERTION(currentPreferred.Equals(aTitle),
-                 "Unexpected argument to SetPreferredSheet");    
+                 "Unexpected argument to SetPreferredSheet");
   }
 #endif
-  
+
   mPreferredSheet = aTitle;
 
   // start any pending alternates that aren't alternates anymore
@@ -613,7 +705,7 @@ static nsresult GetCharsetFromData(const unsigned char* aStyleSheetData,
     if (aStyleSheetData[pos] == '"') {
       break;
     }
-    
+
     // casting to avoid ambiguities
     charset.Append(char(aStyleSheetData[pos]));
     pos += step;
@@ -632,8 +724,7 @@ static nsresult GetCharsetFromData(const unsigned char* aStyleSheetData,
 NS_IMETHODIMP
 SheetLoadData::OnDetermineCharset(nsIUnicharStreamLoader* aLoader,
                                   nsISupports* aContext,
-                                  const char* aData,
-                                  PRUint32 aDataLength,
+                                  nsACString const& aSegment,
                                   nsACString& aCharset)
 {
   NS_PRECONDITION(!mOwningElement || mCharsetHint.IsEmpty(),
@@ -671,8 +762,8 @@ SheetLoadData::OnDetermineCharset(nsIUnicharStreamLoader* aLoader,
   if (aCharset.IsEmpty()) {
     //  We have no charset
     //  Try @charset rule and BOM
-    result = GetCharsetFromData((const unsigned char*)aData,
-                                aDataLength, aCharset);
+    result = GetCharsetFromData((const unsigned char*)aSegment.BeginReading(),
+                                aSegment.Length(), aCharset);
 #ifdef PR_LOGGING
     if (NS_SUCCEEDED(result)) {
       LOG(("  Setting from @charset rule or BOM: %s",
@@ -709,14 +800,14 @@ SheetLoadData::OnDetermineCharset(nsIUnicharStreamLoader* aLoader,
     }
 #endif
   }
-  
+
   if (aCharset.IsEmpty() && mLoader->mDocument) {
     // no useful data on charset.  Try the document charset.
     aCharset = mLoader->mDocument->GetDocumentCharacterSet();
 #ifdef PR_LOGGING
     LOG(("  Set from document: %s", PromiseFlatCString(aCharset).get()));
 #endif
-  }      
+  }
 
   if (aCharset.IsEmpty()) {
     NS_WARNING("Unable to determine charset for sheet, using ISO-8859-1!");
@@ -750,7 +841,7 @@ NS_IMETHODIMP
 SheetLoadData::OnStreamComplete(nsIUnicharStreamLoader* aLoader,
                                 nsISupports* aContext,
                                 nsresult aStatus,
-                                nsIUnicharInputStream* aDataStream)
+                                const nsAString& aBuffer)
 {
   LOG(("SheetLoadData::OnStreamComplete"));
   NS_ASSERTION(!mLoader->mSyncCallback, "Synchronous callback from necko");
@@ -761,14 +852,14 @@ SheetLoadData::OnStreamComplete(nsIUnicharStreamLoader* aLoader,
     // this data and a likely crash.
     return NS_OK;
   }
-  
+
   if (!mLoader->mDocument && !mIsNonDocumentSheet) {
     // Sorry, we don't care about this load anymore
     LOG_WARN(("  No document and not non-document sheet; dropping load"));
     mLoader->SheetComplete(this, NS_BINDING_ABORTED);
     return NS_OK;
   }
-  
+
   if (NS_FAILED(aStatus)) {
     LOG_WARN(("  Load failed: status 0x%x", aStatus));
     mLoader->SheetComplete(this, aStatus);
@@ -782,7 +873,7 @@ SheetLoadData::OnStreamComplete(nsIUnicharStreamLoader* aLoader,
     mLoader->SheetComplete(this, result);
     return NS_OK;
   }
-  
+
   nsCOMPtr<nsIURI> originalURI;
   channel->GetOriginalURI(getter_AddRefs(originalURI));
 
@@ -805,7 +896,11 @@ SheetLoadData::OnStreamComplete(nsIUnicharStreamLoader* aLoader,
   nsIScriptSecurityManager* secMan = nsContentUtils::GetSecurityManager();
   result = NS_ERROR_NOT_AVAILABLE;
   if (secMan) {  // Could be null if we already shut down
-    result = secMan->GetChannelPrincipal(channel, getter_AddRefs(principal));
+    if (mUseSystemPrincipal) {
+      result = secMan->GetSystemPrincipal(getter_AddRefs(principal));
+    } else {
+      result = secMan->GetChannelPrincipal(channel, getter_AddRefs(principal));
+    }
   }
 
   if (NS_FAILED(result)) {
@@ -813,14 +908,9 @@ SheetLoadData::OnStreamComplete(nsIUnicharStreamLoader* aLoader,
     mLoader->SheetComplete(this, result);
     return NS_OK;
   }
-  
+
   mSheet->SetPrincipal(principal);
-  
-#ifdef MOZ_TIMELINE
-  NS_TIMELINE_OUTDENT();
-  NS_TIMELINE_MARK_CHANNEL("SheetLoadData::OnStreamComplete(%s)", channel);
-#endif // MOZ_TIMELINE
-  
+
   // If it's an HTTP channel, we want to make sure this is not an
   // error document we got.
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(channel));
@@ -834,12 +924,6 @@ SheetLoadData::OnStreamComplete(nsIUnicharStreamLoader* aLoader,
     }
   }
 
-  if (!aDataStream) {
-    LOG_WARN(("  No data stream; bailing"));
-    mLoader->SheetComplete(this, NS_ERROR_NOT_AVAILABLE);
-    return NS_OK;
-  }
-
   nsCAutoString contentType;
   if (channel) {
     channel->GetContentType(contentType);
@@ -902,7 +986,9 @@ SheetLoadData::OnStreamComplete(nsIUnicharStreamLoader* aLoader,
   mSheet->SetURIs(channelURI, originalURI, channelURI);
 
   PRBool completed;
-  return mLoader->ParseSheet(aDataStream, this, completed);
+  result = mLoader->ParseSheet(aBuffer, this, completed);
+  NS_ASSERTION(completed || !mSyncLoad, "sync load did not complete");
+  return result;
 }
 
 #ifdef MOZ_XUL
@@ -926,7 +1012,7 @@ Loader::IsAlternate(const nsAString& aTitle, PRBool aHasAlternateRel)
   if (aTitle.IsEmpty()) {
     return PR_FALSE;
   }
-  
+
   if (!aHasAlternateRel && mDocument && mPreferredSheet.IsEmpty()) {
     // There's no preferred set yet, and we now have a sheet with a title.
     // Make that be the preferred set.
@@ -940,7 +1026,7 @@ Loader::IsAlternate(const nsAString& aTitle, PRBool aHasAlternateRel)
 
 /**
  * CheckLoadAllowed will return success if the load is allowed,
- * failure otherwise. 
+ * failure otherwise.
  *
  * @param aSourcePrincipal the principal of the node or document or parent
  *                         sheet loading the sheet
@@ -956,7 +1042,7 @@ Loader::CheckLoadAllowed(nsIPrincipal* aSourcePrincipal,
   LOG(("css::Loader::CheckLoadAllowed"));
 
   nsresult rv;
-  
+
   if (aSourcePrincipal) {
     // Check with the security manager
     nsIScriptSecurityManager *secMan = nsContentUtils::GetSecurityManager();
@@ -1015,11 +1101,11 @@ Loader::CreateSheet(nsIURI* aURI,
                    (mLoadingDatas.IsInitialized() || mLoadingDatas.Init()) &&
                    (mPendingDatas.IsInitialized() || mPendingDatas.Init()),
                  NS_ERROR_OUT_OF_MEMORY);
-  
+
   nsresult rv = NS_OK;
   *aSheet = nsnull;
   aSheetState = eSheetStateUnknown;
-  
+
   if (aURI) {
     aSheetState = eSheetComplete;
     nsRefPtr<nsCSSStyleSheet> sheet;
@@ -1040,11 +1126,11 @@ Loader::CreateSheet(nsIURI* aURI,
     if (!sheet) {
       // Then our per-document complete sheets.
       URIAndPrincipalHashKey key(aURI, aLoaderPrincipal);
-      
+
       mCompleteSheets.Get(&key, getter_AddRefs(sheet));
       LOG(("  From completed: %p", sheet.get()));
     }
-    
+
     if (sheet) {
       // This sheet came from the XUL cache or our per-document hashtable; it
       // better be a complete sheet.
@@ -1139,7 +1225,7 @@ Loader::CreateSheet(nsIURI* aURI,
   NS_ASSERTION(*aSheet, "We should have a sheet by now!");
   NS_ASSERTION(aSheetState != eSheetStateUnknown, "Have to set a state!");
   LOG(("  State: %s", gStateStrings[aSheetState]));
-  
+
   return NS_OK;
 }
 
@@ -1168,7 +1254,6 @@ Loader::PrepareSheet(nsCSSStyleSheet* aSheet,
     NS_ENSURE_TRUE(mediaList, NS_ERROR_OUT_OF_MEMORY);
 
     nsCSSParser mediumParser(this);
-    NS_ENSURE_TRUE(mediumParser, NS_ERROR_OUT_OF_MEMORY);
 
     // We have aMediaString only when linked from link elements, style
     // elements, or PIs, so pass PR_TRUE.
@@ -1185,7 +1270,7 @@ Loader::PrepareSheet(nsCSSStyleSheet* aSheet,
   if (aIsAlternate) {
     *aIsAlternate = alternate;
   }
-  return NS_OK;    
+  return NS_OK;
 }
 
 /**
@@ -1232,7 +1317,7 @@ Loader::InsertSheetInDoc(nsCSSStyleSheet* aSheet,
     domSheet->GetOwnerNode(getter_AddRefs(sheetOwner));
     if (sheetOwner && !aLinkingContent) {
       // Keep moving; all sheets with a sheetOwner come after all
-      // sheets without a linkingNode 
+      // sheets without a linkingNode
       continue;
     }
 
@@ -1255,7 +1340,7 @@ Loader::InsertSheetInDoc(nsCSSStyleSheet* aSheet,
   }
 
   ++insertionPoint; // adjust the index to the spot we want to insert in
-  
+
   // XXX <meta> elements do not implement nsIStyleSheetLinkingElement;
   // need to fix this for them to be ordered correctly.
   nsCOMPtr<nsIStyleSheetLinkingElement>
@@ -1294,7 +1379,7 @@ Loader::InsertChildSheet(nsCSSStyleSheet* aSheet,
   // child sheets should always start out enabled, even if they got
   // cloned off of top-level sheets which were disabled
   aSheet->SetEnabled(PR_TRUE);
-  
+
   aParentSheet->AppendStyleSheet(aSheet);
   aParentRule->SetSheet(aSheet); // This sets the ownerRule on the sheet
 
@@ -1332,8 +1417,8 @@ Loader::LoadSheet(SheetLoadData* aLoadData, StyleSheetState aSheetState)
 #endif
 
   LOG_URI("  Load from: '%s'", aLoadData->mURI);
-  
-  nsresult rv = NS_OK;  
+
+  nsresult rv = NS_OK;
 
   if (!mDocument && !aLoadData->mIsNonDocumentSheet) {
     // No point starting the load; just release all the data and such.
@@ -1348,6 +1433,17 @@ Loader::LoadSheet(SheetLoadData* aLoadData, StyleSheetState aSheetState)
     NS_ASSERTION(aSheetState == eSheetNeedsParser,
                  "Sync loads can't reuse existing async loads");
 
+    // Create a nsIUnicharStreamLoader instance to which we will feed
+    // the data from the sync load.  Do this before creating the
+    // channel to make error recovery simpler.
+    nsCOMPtr<nsIUnicharStreamLoader> streamLoader;
+    rv = NS_NewUnicharStreamLoader(getter_AddRefs(streamLoader), aLoadData);
+    if (NS_FAILED(rv)) {
+      LOG_ERROR(("  Failed to create stream loader for sync load"));
+      SheetComplete(aLoadData, rv);
+      return rv;
+    }
+
     // Just load it
     nsCOMPtr<nsIInputStream> stream;
     nsCOMPtr<nsIChannel> channel;
@@ -1361,54 +1457,19 @@ Loader::LoadSheet(SheetLoadData* aLoadData, StyleSheetState aSheetState)
     }
 
     NS_ASSERTION(channel, "NS_OpenURI lied?");
-    
-    // Get the principal for this sheet
-    nsCOMPtr<nsIPrincipal> principal;
-    if (aLoadData->mUseSystemPrincipal) {
-      rv = nsContentUtils::GetSecurityManager()->
-        GetSystemPrincipal(getter_AddRefs(principal));
-    } else {
-      rv = nsContentUtils::GetSecurityManager()->
-        GetChannelPrincipal(channel, getter_AddRefs(principal));
-    }
 
-    if (NS_FAILED(rv)) {
-      LOG_ERROR(("  Failed to get a principal for the sheet"));
-      SheetComplete(aLoadData, rv);
-      return rv;
-    }
+    // Force UA sheets to be UTF-8.
+    // XXX this is only necessary because the default in
+    // SheetLoadData::OnDetermineCharset is wrong (bug 521039).
+    channel->SetContentCharset(NS_LITERAL_CSTRING("UTF-8"));
 
-    aLoadData->mSheet->SetPrincipal(principal);
-
-    nsCOMPtr<nsIConverterInputStream> converterStream = 
-      do_CreateInstance("@mozilla.org/intl/converter-input-stream;1", &rv);
-    
-    if (NS_FAILED(rv)) {
-      LOG_ERROR(("  Failed to create converter stream"));
-      SheetComplete(aLoadData, rv);
-      return rv;
-    }
-
-    // This forces UA sheets to be UTF-8.  We should really look for
-    // @charset rules here via ReadSegments on the raw stream...
-
-    // 8192 is a nice magic number that happens to be what a lot of
-    // other things use for buffer sizes.
-    rv = converterStream->Init(stream, "UTF-8",
-                               8192,
-                               nsIConverterInputStream::
-                                    DEFAULT_REPLACEMENT_CHARACTER);
-    
-    if (NS_FAILED(rv)) {
-      LOG_ERROR(("  Failed to initialize converter stream"));
-      SheetComplete(aLoadData, rv);
-      return rv;
-    }
-
-    PRBool completed;
-    rv = ParseSheet(converterStream, aLoadData, completed);
-    NS_ASSERTION(completed, "sync load did not complete");
-    return rv;
+    // Manually feed the streamloader the contents of the stream we
+    // got from NS_OpenURI.  This will call back into OnStreamComplete
+    // and thence to ParseSheet.  Regardless of whether this fails,
+    // SheetComplete has been called.
+    return nsSyncLoadService::PushSyncStreamToListener(stream,
+                                                       streamLoader,
+                                                       channel);
   }
 
   SheetLoadData* existingData = nsnull;
@@ -1422,7 +1483,7 @@ Loader::LoadSheet(SheetLoadData* aLoadData, StyleSheetState aSheetState)
     mPendingDatas.Get(&key, &existingData);
     NS_ASSERTION(existingData, "CreateSheet lied about the state");
   }
-  
+
   if (existingData) {
     LOG(("  Glomming on to existing load"));
     SheetLoadData* data = existingData;
@@ -1470,17 +1531,12 @@ Loader::LoadSheet(SheetLoadData* aLoadData, StyleSheetState aSheetState)
     }
   }
 
-#ifdef MOZ_TIMELINE
-  NS_TIMELINE_MARK_URI("Loading style sheet: %s", aLoadData->mURI);
-  NS_TIMELINE_INDENT();
-#endif
-  
   nsCOMPtr<nsIChannel> channel;
   rv = NS_NewChannel(getter_AddRefs(channel),
                      aLoadData->mURI, nsnull, loadGroup, nsnull,
                      nsIChannel::LOAD_NORMAL | nsIChannel::LOAD_CLASSIFY_URI,
                      channelPolicy);
-  
+
   if (NS_FAILED(rv)) {
 #ifdef DEBUG
     mSyncCallback = PR_FALSE;
@@ -1544,9 +1600,9 @@ Loader::LoadSheet(SheetLoadData* aLoadData, StyleSheetState aSheetState)
     SheetComplete(aLoadData, NS_ERROR_OUT_OF_MEMORY);
     return NS_ERROR_OUT_OF_MEMORY;
   }
-  
+
   aLoadData->mIsLoading = PR_TRUE;
-  
+
   return NS_OK;
 }
 
@@ -1557,12 +1613,11 @@ Loader::LoadSheet(SheetLoadData* aLoadData, StyleSheetState aSheetState)
  * correctly.
  */
 nsresult
-Loader::ParseSheet(nsIUnicharInputStream* aStream,
+Loader::ParseSheet(const nsAString& aInput,
                    SheetLoadData* aLoadData,
                    PRBool& aCompleted)
 {
   LOG(("css::Loader::ParseSheet"));
-  NS_PRECONDITION(aStream, "Must have data to parse");
   NS_PRECONDITION(aLoadData, "Must have load data");
   NS_PRECONDITION(aLoadData->mSheet, "Must have sheet to parse into");
 
@@ -1575,20 +1630,15 @@ Loader::ParseSheet(nsIUnicharInputStream* aStream,
   aCompleted = PR_FALSE;
 
   nsCSSParser parser(this, aLoadData->mSheet);
-  if (!parser) {
-    LOG_ERROR(("  Failed to get CSS parser"));
-    SheetComplete(aLoadData, NS_ERROR_OUT_OF_MEMORY);
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
 
   // Push our load data on the stack so any kids can pick it up
   mParsingDatas.AppendElement(aLoadData);
   nsIURI* sheetURI = aLoadData->mSheet->GetSheetURI();
   nsIURI* baseURI = aLoadData->mSheet->GetBaseURI();
-  nsresult rv = parser.Parse(aStream, sheetURI, baseURI,
-                             aLoadData->mSheet->Principal(),
-                             aLoadData->mLineNumber,
-                             aLoadData->mAllowUnsafeRules);
+  nsresult rv = parser.ParseSheet(aInput, sheetURI, baseURI,
+                                  aLoadData->mSheet->Principal(),
+                                  aLoadData->mLineNumber,
+                                  aLoadData->mAllowUnsafeRules);
   mParsingDatas.RemoveElementAt(mParsingDatas.Length() - 1);
 
   if (NS_FAILED(rv)) {
@@ -1635,7 +1685,7 @@ Loader::SheetComplete(SheetLoadData* aLoadData, nsresult aStatus)
   mDatasToNotifyOn += count;
   for (PRUint32 i = 0; i < count; ++i) {
     --mDatasToNotifyOn;
-    
+
     SheetLoadData* data = datasToNotify[i];
     NS_ASSERTION(data && data->mMustNotify, "How did this data get here?");
     if (data->mObserver) {
@@ -1690,13 +1740,19 @@ Loader::DoSheetComplete(SheetLoadData* aLoadData, nsresult aStatus,
       aLoadData->mIsLoading = PR_FALSE;
     }
   }
-  
+
   // Go through and deal with the whole linked list.
   SheetLoadData* data = aLoadData;
   while (data) {
-    NS_ABORT_IF_FALSE(!data->mSheet->IsModified(),
-                      "should not get marked modified during parsing");
-    data->mSheet->SetComplete();
+    if (!data->mSheetAlreadyComplete) {
+      // If mSheetAlreadyComplete, then the sheet could well be modified between
+      // when we posted the async call to SheetComplete and now, since the sheet
+      // was page-accessible during that whole time.
+      NS_ABORT_IF_FALSE(!data->mSheet->IsModified(),
+                        "should not get marked modified during parsing");
+      data->mSheet->SetComplete();
+      data->ScheduleLoadEventIfNeeded(aStatus);
+    }
     if (data->mMustNotify && (data->mObserver || !mObservers.IsEmpty())) {
       // Don't notify here so we don't trigger script.  Remember the
       // info we need to notify, then do it later when it's safe.
@@ -1720,11 +1776,14 @@ Loader::DoSheetComplete(SheetLoadData* aLoadData, nsresult aStatus,
         !mParsingDatas.Contains(data->mParentData)) {
       DoSheetComplete(data->mParentData, aStatus, aDatasToNotify);
     }
-    
+
     data = data->mNext;
   }
 
-  // Now that it's marked complete, put the sheet in our cache
+  // Now that it's marked complete, put the sheet in our cache.
+  // If we ever start doing this for failure aStatus, we'll need to
+  // adjust the PostLoadEvent code that thinks anything already
+  // complete must have loaded succesfully.
   if (NS_SUCCEEDED(aStatus) && aLoadData->mURI) {
 #ifdef MOZ_XUL
     if (IsChromeURI(aLoadData->mURI)) {
@@ -1751,7 +1810,7 @@ Loader::DoSheetComplete(SheetLoadData* aLoadData, nsresult aStatus,
 
 nsresult
 Loader::LoadInlineStyle(nsIContent* aElement,
-                        nsIUnicharInputStream* aStream, 
+                        const nsAString& aBuffer,
                         PRUint32 aLineNumber,
                         const nsAString& aTitle,
                         const nsAString& aMedia,
@@ -1760,7 +1819,6 @@ Loader::LoadInlineStyle(nsIContent* aElement,
                         PRBool* aIsAlternate)
 {
   LOG(("css::Loader::LoadInlineStyle"));
-  NS_PRECONDITION(aStream, "Must have a stream to parse!");
   NS_ASSERTION(mParsingDatas.Length() == 0, "We're in the middle of a parse?");
 
   *aCompleted = PR_TRUE;
@@ -1774,7 +1832,6 @@ Loader::LoadInlineStyle(nsIContent* aElement,
 
   nsCOMPtr<nsIStyleSheetLinkingElement> owningElement(do_QueryInterface(aElement));
   NS_ASSERTION(owningElement, "Element is not a style linking element!");
-  
 
   // Since we're not planning to load a URI, no need to hand a principal to the
   // load data or to CreateSheet().
@@ -1789,28 +1846,23 @@ Loader::LoadInlineStyle(nsIContent* aElement,
   rv = PrepareSheet(sheet, aTitle, aMedia, nsnull, PR_FALSE,
                     aIsAlternate);
   NS_ENSURE_SUCCESS(rv, rv);
-  
+
   LOG(("  Sheet is alternate: %d", *aIsAlternate));
-  
+
   rv = InsertSheetInDoc(sheet, aElement, mDocument);
   NS_ENSURE_SUCCESS(rv, rv);
-  
+
   SheetLoadData* data = new SheetLoadData(this, aTitle, nsnull, sheet,
                                           owningElement, *aIsAlternate,
                                           aObserver, nsnull);
 
-  if (!data) {
-    sheet->SetComplete();
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
   // We never actually load this, so just set its principal directly
   sheet->SetPrincipal(aElement->NodePrincipal());
 
   NS_ADDREF(data);
   data->mLineNumber = aLineNumber;
   // Parse completion releases the load data
-  rv = ParseSheet(aStream, data, *aCompleted);
+  rv = ParseSheet(aBuffer, data, *aCompleted);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // If aCompleted is true, |data| may well be deleted by now.
@@ -1818,11 +1870,11 @@ Loader::LoadInlineStyle(nsIContent* aElement,
     data->mMustNotify = PR_TRUE;
   }
   return rv;
-}        
+}
 
 nsresult
 Loader::LoadStyleLink(nsIContent* aElement,
-                      nsIURI* aURL, 
+                      nsIURI* aURL,
                       const nsAString& aTitle,
                       const nsAString& aMedia,
                       PRBool aHasAlternateRel,
@@ -1842,7 +1894,7 @@ Loader::LoadStyleLink(nsIContent* aElement,
     LOG_WARN(("  Not enabled"));
     return NS_ERROR_NOT_AVAILABLE;
   }
-  
+
   NS_ENSURE_TRUE(mDocument, NS_ERROR_NOT_INITIALIZED);
 
   nsIPrincipal* principal =
@@ -1856,7 +1908,7 @@ Loader::LoadStyleLink(nsIContent* aElement,
   if (NS_FAILED(rv)) return rv;
 
   LOG(("  Passed load check"));
-  
+
   StyleSheetState state;
   nsRefPtr<nsCSSStyleSheet> sheet;
   rv = CreateSheet(aURL, aElement, principal, PR_FALSE, state,
@@ -1868,32 +1920,28 @@ Loader::LoadStyleLink(nsIContent* aElement,
   NS_ENSURE_SUCCESS(rv, rv);
 
   LOG(("  Sheet is alternate: %d", *aIsAlternate));
-  
+
   rv = InsertSheetInDoc(sheet, aElement, mDocument);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  nsCOMPtr<nsIStyleSheetLinkingElement> owningElement(do_QueryInterface(aElement));
+
   if (state == eSheetComplete) {
     LOG(("  Sheet already complete: 0x%p",
          static_cast<void*>(sheet.get())));
-    if (aObserver) {
-      rv = PostLoadEvent(aURL, sheet, aObserver, *aIsAlternate);
+    if (aObserver || !mObservers.IsEmpty() || owningElement) {
+      rv = PostLoadEvent(aURL, sheet, aObserver, *aIsAlternate,
+                         owningElement);
       return rv;
     }
 
     return NS_OK;
   }
 
-  nsCOMPtr<nsIStyleSheetLinkingElement> owningElement(do_QueryInterface(aElement));
-
   // Now we need to actually load it
   SheetLoadData* data = new SheetLoadData(this, aTitle, aURL, sheet,
                                           owningElement, *aIsAlternate,
                                           aObserver, principal);
-  if (!data) {
-    sheet->SetComplete();
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
   NS_ADDREF(data);
 
   // If we have to parse and it's an alternate non-inline, defer it
@@ -1917,9 +1965,39 @@ Loader::LoadStyleLink(nsIContent* aElement,
   return rv;
 }
 
+static PRBool
+HaveAncestorDataWithURI(SheetLoadData *aData, nsIURI *aURI)
+{
+  if (!aData->mURI) {
+    // Inline style; this won't have any ancestors
+    NS_ABORT_IF_FALSE(!aData->mParentData,
+                      "How does inline style have a parent?");
+    return PR_FALSE;
+  }
+
+  PRBool equal;
+  if (NS_FAILED(aData->mURI->Equals(aURI, &equal)) || equal) {
+    return PR_TRUE;
+  }
+
+  // Datas down the mNext chain have the same URI as aData, so we
+  // don't have to compare to them.  But they might have different
+  // parents, and we have to check all of those.
+  while (aData) {
+    if (aData->mParentData &&
+        HaveAncestorDataWithURI(aData->mParentData, aURI)) {
+      return PR_TRUE;
+    }
+
+    aData = aData->mNext;
+  }
+
+  return PR_FALSE;
+}
+
 nsresult
 Loader::LoadChildSheet(nsCSSStyleSheet* aParentSheet,
-                       nsIURI* aURL, 
+                       nsIURI* aURL,
                        nsMediaList* aMedia,
                        ImportRule* aParentRule)
 {
@@ -1931,7 +2009,7 @@ Loader::LoadChildSheet(nsCSSStyleSheet* aParentSheet,
     LOG_WARN(("  Not enabled"));
     return NS_ERROR_NOT_AVAILABLE;
   }
-  
+
   LOG_URI("  Child uri: '%s'", aURL);
 
   nsCOMPtr<nsIDOMNode> owningNode;
@@ -1962,7 +2040,7 @@ Loader::LoadChildSheet(nsCSSStyleSheet* aParentSheet,
   if (NS_FAILED(rv)) return rv;
 
   LOG(("  Passed load check"));
-  
+
   SheetLoadData* parentData = nsnull;
   nsCOMPtr<nsICSSLoaderObserver> observer;
 
@@ -1971,16 +2049,11 @@ Loader::LoadChildSheet(nsCSSStyleSheet* aParentSheet,
     LOG(("  Have a parent load"));
     parentData = mParsingDatas.ElementAt(count - 1);
     // Check for cycles
-    SheetLoadData* data = parentData;
-    while (data && data->mURI) {
-      PRBool equal;
-      if (NS_SUCCEEDED(data->mURI->Equals(aURL, &equal)) && equal) {
-        // Houston, we have a loop, blow off this child and pretend this never
-        // happened
-        LOG_ERROR(("  @import cycle detected, dropping load"));
-        return NS_OK;
-      }
-      data = data->mParentData;
+    if (HaveAncestorDataWithURI(parentData, aURL)) {
+      // Houston, we have a loop, blow off this child and pretend this never
+      // happened
+      LOG_ERROR(("  @import cycle detected, dropping load"));
+      return NS_OK;
     }
 
     NS_ASSERTION(parentData->mSheet == aParentSheet,
@@ -2004,10 +2077,10 @@ Loader::LoadChildSheet(nsCSSStyleSheet* aParentSheet,
   const nsSubstring& empty = EmptyString();
   rv = PrepareSheet(sheet, empty, empty, aMedia);
   NS_ENSURE_SUCCESS(rv, rv);
-  
+
   rv = InsertChildSheet(sheet, aParentSheet, aParentRule);
   NS_ENSURE_SUCCESS(rv, rv);
-  
+
   if (state == eSheetComplete) {
     LOG(("  Sheet already complete"));
     // We're completely done.  No need to notify, even, since the
@@ -2016,15 +2089,9 @@ Loader::LoadChildSheet(nsCSSStyleSheet* aParentSheet,
     return NS_OK;
   }
 
-  
   SheetLoadData* data = new SheetLoadData(this, aURL, sheet, parentData,
                                           observer, principal);
 
-  if (!data) {
-    sheet->SetComplete();
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
   NS_ADDREF(data);
   PRBool syncLoad = data->mSyncLoad;
 
@@ -2036,7 +2103,7 @@ Loader::LoadChildSheet(nsCSSStyleSheet* aParentSheet,
   if (!syncLoad) {
     data->mMustNotify = PR_TRUE;
   }
-  return rv;  
+  return rv;
 }
 
 nsresult
@@ -2077,7 +2144,7 @@ Loader::LoadSheet(nsIURI* aURL,
 }
 
 nsresult
-Loader::InternalLoadNonDocumentSheet(nsIURI* aURL, 
+Loader::InternalLoadNonDocumentSheet(nsIURI* aURL,
                                      PRBool aAllowUnsafeRules,
                                      PRBool aUseSystemPrincipal,
                                      nsIPrincipal* aOriginPrincipal,
@@ -2092,11 +2159,11 @@ Loader::InternalLoadNonDocumentSheet(nsIURI* aURL,
   NS_ASSERTION(mParsingDatas.Length() == 0, "We're in the middle of a parse?");
 
   LOG_URI("  Non-document sheet uri: '%s'", aURL);
-  
+
   if (aSheet) {
     *aSheet = nsnull;
   }
-  
+
   if (!mEnabled) {
     LOG_WARN(("  Not enabled"));
     return NS_ERROR_NOT_AVAILABLE;
@@ -2110,7 +2177,7 @@ Loader::InternalLoadNonDocumentSheet(nsIURI* aURL,
   StyleSheetState state;
   nsRefPtr<nsCSSStyleSheet> sheet;
   PRBool syncLoad = (aObserver == nsnull);
-  
+
   rv = CreateSheet(aURL, nsnull, aOriginPrincipal, syncLoad, state,
                    getter_AddRefs(sheet));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -2118,11 +2185,11 @@ Loader::InternalLoadNonDocumentSheet(nsIURI* aURL,
   const nsSubstring& empty = EmptyString();
   rv = PrepareSheet(sheet, empty, empty, nsnull);
   NS_ENSURE_SUCCESS(rv, rv);
-  
+
   if (state == eSheetComplete) {
     LOG(("  Sheet already complete"));
-    if (aObserver) {
-      rv = PostLoadEvent(aURL, sheet, aObserver, PR_FALSE);
+    if (aObserver || !mObservers.IsEmpty()) {
+      rv = PostLoadEvent(aURL, sheet, aObserver, PR_FALSE, nsnull);
     }
     if (aSheet) {
       sheet.swap(*aSheet);
@@ -2135,11 +2202,6 @@ Loader::InternalLoadNonDocumentSheet(nsIURI* aURL,
                       aUseSystemPrincipal, aCharset, aObserver,
                       aOriginPrincipal);
 
-  if (!data) {
-    sheet->SetComplete();
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-  
   NS_ADDREF(data);
   rv = LoadSheet(data, state);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -2158,17 +2220,19 @@ nsresult
 Loader::PostLoadEvent(nsIURI* aURI,
                       nsCSSStyleSheet* aSheet,
                       nsICSSLoaderObserver* aObserver,
-                      PRBool aWasAlternate)
+                      PRBool aWasAlternate,
+                      nsIStyleSheetLinkingElement* aElement)
 {
   LOG(("css::Loader::PostLoadEvent"));
   NS_PRECONDITION(aSheet, "Must have sheet");
-  NS_PRECONDITION(aObserver, "Must have observer");
+  NS_PRECONDITION(aObserver || !mObservers.IsEmpty() || aElement,
+                  "Must have observer or element");
 
   nsRefPtr<SheetLoadData> evt =
     new SheetLoadData(this, EmptyString(), // title doesn't matter here
                       aURI,
                       aSheet,
-                      nsnull,  // owning element doesn't matter here
+                      aElement,
                       aWasAlternate,
                       aObserver,
                       nsnull);
@@ -2190,6 +2254,13 @@ Loader::PostLoadEvent(nsIURI* aURI,
 
     // We want to notify the observer for this data.
     evt->mMustNotify = PR_TRUE;
+    evt->mSheetAlreadyComplete = PR_TRUE;
+
+    // If we get to this code, aSheet loaded correctly at some point, so
+    // we can just use NS_OK for the status.  Note that we do this here
+    // and not from inside our SheetComplete so that we don't end up
+    // running the load event async.
+    evt->ScheduleLoadEventIfNeeded(NS_OK);
   }
 
   return rv;
@@ -2202,6 +2273,12 @@ Loader::HandleLoadEvent(SheetLoadData* aEvent)
   // we're unblocking the parser
   // NS_ASSERTION(aEvent->mObserver, "Must have observer");
   NS_ASSERTION(aEvent->mSheet, "Must have sheet");
+
+  // Very important: this needs to come before the SheetComplete call
+  // below, so that HasPendingLoads() will test true as needed under
+  // notifications we send from that SheetComplete call.
+  mPostedEvents.RemoveElement(aEvent);
+
   if (!aEvent->mIsCancelled) {
     // SheetComplete will call Release(), so give it a reference to do
     // that with.
@@ -2209,8 +2286,6 @@ Loader::HandleLoadEvent(SheetLoadData* aEvent)
     SheetComplete(aEvent, NS_OK);
   }
 
-  mPostedEvents.RemoveElement(aEvent);
-
   if (mDocument) {
     mDocument->UnblockOnload(PR_TRUE);
   }
@@ -2226,7 +2301,7 @@ StopLoadingSheetCallback(URIAndPrincipalHashKey* aKey,
 
   aData->mIsLoading = PR_FALSE; // we will handle the removal right here
   aData->mIsCancelled = PR_TRUE;
-  
+
   static_cast<Loader::LoadDataArray*>(aClosure)->AppendElement(aData);
 
   return PL_DHASH_REMOVE;
@@ -2240,7 +2315,7 @@ Loader::Stop()
   PRUint32 loadingCount =
     mLoadingDatas.IsInitialized() ? mLoadingDatas.Count() : 0;
   LoadDataArray arr(pendingCount + loadingCount + mPostedEvents.Length());
-  
+
   if (pendingCount) {
     mPendingDatas.Enumerate(StopLoadingSheetCallback, &arr);
   }
