@@ -48,12 +48,12 @@ class nsCSSParser;
 class nsIURI;
 class nsIPrincipal;
 class nsIDocument;
-class nsICSSRule;
 
 namespace mozilla {
 namespace css {
 class Declaration;
 class Loader;
+class Rule;
 }
 }
 
@@ -67,6 +67,15 @@ public:
 
   NS_DECL_NSICSSDECLARATION
 
+  NS_IMETHOD GetMozPerspective(nsAString_internal&);
+  NS_IMETHOD SetMozPerspective(const nsAString_internal&);
+  NS_IMETHOD GetMozPerspectiveOrigin(nsAString_internal&);
+  NS_IMETHOD SetMozPerspectiveOrigin(const nsAString_internal&);
+  NS_IMETHOD GetMozBackfaceVisibility(nsAString_internal&);
+  NS_IMETHOD SetMozBackfaceVisibility(const nsAString_internal&);
+  NS_IMETHOD GetMozTransformStyle(nsAString_internal&);
+  NS_IMETHOD SetMozTransformStyle(const nsAString_internal&);
+
   // Require subclasses to implement |GetParentRule|.
   //NS_DECL_NSIDOMCSSSTYLEDECLARATION
   NS_IMETHOD GetCssText(nsAString & aCssText);
@@ -100,22 +109,29 @@ protected:
   // to it.
   virtual nsIDocument* DocToUpdate() = 0;
 
-  // This will only fail if it can't get a parser or a principal.
-  // This means it can return NS_OK without aURI or aCSSLoader being
-  // initialized.
-  virtual nsresult GetCSSParsingEnvironment(nsIURI** aSheetURI,
-                                            nsIURI** aBaseURI,
-                                            nsIPrincipal** aSheetPrincipal,
-                                            mozilla::css::Loader** aCSSLoader) = 0;
+  // Information neded to parse a declaration.  We need the mSheetURI
+  // for error reporting, mBaseURI to resolve relative URIs,
+  // mPrincipal for subresource loads, and mCSSLoader for determining
+  // whether we're in quirks mode.  mBaseURI needs to be a strong
+  // pointer because of xml:base possibly creating base URIs on the
+  // fly.  This is why we don't use CSSParsingEnvironment as a return
+  // value, to avoid multiple-refcounting of mBaseURI.
+  struct CSSParsingEnvironment {
+    nsIURI* mSheetURI;
+    nsCOMPtr<nsIURI> mBaseURI;
+    nsIPrincipal* mPrincipal;
+    mozilla::css::Loader* mCSSLoader;
+  };
+  
+  // On failure, mPrincipal should be set to null in aCSSParseEnv.
+  // If mPrincipal is null, the other members may not be set to
+  // anything meaningful.
+  virtual void GetCSSParsingEnvironment(CSSParsingEnvironment& aCSSParseEnv) = 0;
 
   // An implementation for GetCSSParsingEnvironment for callers wrapping
-  // an nsICSSRule.
-  static nsresult
-  GetCSSParsingEnvironmentForRule(nsICSSRule* aRule,
-                                  nsIURI** aSheetURI,
-                                  nsIURI** aBaseURI,
-                                  nsIPrincipal** aSheetPrincipal,
-                                  mozilla::css::Loader** aCSSLoader);
+  // an css::Rule.
+  static void GetCSSParsingEnvironmentForRule(mozilla::css::Rule* aRule,
+                                              CSSParsingEnvironment& aCSSParseEnv);
 
   nsresult ParsePropertyValue(const nsCSSProperty aPropID,
                               const nsAString& aPropValue,
