@@ -38,10 +38,13 @@
 #define nsStyleUtil_h___
 
 #include "nsCoord.h"
-#include "nsPresContext.h"
-#include "nsILinkHandler.h" // for nsLinkState
+#include "nsCSSProperty.h"
 
+class nsPresContext;
 struct nsStyleBackground;
+class nsString;
+class nsStringComparator;
+class nsIContent;
 
 enum nsFontSizeType {
   eFontSize_HTML = 1,
@@ -53,27 +56,20 @@ enum nsFontSizeType {
 class nsStyleUtil {
 public:
   
-  static float GetScalingFactor(PRInt32 aScaler);
-
   static nscoord CalcFontPointSize(PRInt32 aHTMLSize, PRInt32 aBasePointSize, 
-                                   float aScalingFactor, nsPresContext* aPresContext,
+                                   nsPresContext* aPresContext,
                                    nsFontSizeType aFontSizeType = eFontSize_HTML);
 
   static nscoord FindNextSmallerFontSize(nscoord aFontSize, PRInt32 aBasePointSize, 
-                                         float aScalingFactor, nsPresContext* aPresContext,
+                                         nsPresContext* aPresContext,
                                          nsFontSizeType aFontSizeType = eFontSize_HTML);
 
   static nscoord FindNextLargerFontSize(nscoord aFontSize, PRInt32 aBasePointSize, 
-                                        float aScalingFactor, nsPresContext* aPresContext,
+                                        nsPresContext* aPresContext,
                                         nsFontSizeType aFontSizeType = eFontSize_HTML);
 
   static PRInt32 ConstrainFontWeight(PRInt32 aWeight);
 
-  static PRBool IsHTMLLink(nsIContent *aContent, nsILinkHandler *aLinkHandler,
-                           nsLinkState *aState);
-  static PRBool IsLink(nsIContent *aContent, nsILinkHandler *aLinkHandler,
-                       nsLinkState *aState);
-
  static PRBool DashMatchCompare(const nsAString& aAttributeValue,
                                 const nsAString& aSelectorValue,
                                 const nsStringComparator& aComparator);
@@ -81,6 +77,18 @@ public:
   // Append a quoted (with "") and escaped version of aString to aResult.
   static void AppendEscapedCSSString(const nsString& aString,
                                      nsAString& aResult);
+  // Append the identifier given by |aIdent| to |aResult|, with
+  // appropriate escaping so that it can be reparsed to the same
+  // identifier.
+  static void AppendEscapedCSSIdent(const nsString& aIdent,
+                                    nsAString& aResult);
+
+  // Append a bitmask-valued property's value(s) (space-separated) to aResult.
+  static void AppendBitmaskCSSValue(nsCSSProperty aProperty,
+                                    PRInt32 aMaskedValue,
+                                    PRInt32 aFirstMask,
+                                    PRInt32 aLastMask,
+                                    nsAString& aResult);
 
   /*
    * Convert an author-provided floating point number to an integer (0
