@@ -169,31 +169,29 @@ void nsTableCellMap::InsertGroupCellMap(nsTableRowGroupFrame*  aNewGroup,
                                         nsTableRowGroupFrame*& aPrevGroup)
 {
   nsCellMap* newMap = new nsCellMap(aNewGroup, mBCInfo != nsnull);
-  if (newMap) {
-    nsCellMap* prevMap = nsnull;
-    nsCellMap* lastMap = mFirstMap;
-    if (aPrevGroup) {
-      nsCellMap* map = mFirstMap;
-      while (map) {
-        lastMap = map;
-        if (map->GetRowGroup() == aPrevGroup) {
-          prevMap = map;
-          break;
-        }
-        map = map->GetNextSibling();
+  nsCellMap* prevMap = nsnull;
+  nsCellMap* lastMap = mFirstMap;
+  if (aPrevGroup) {
+    nsCellMap* map = mFirstMap;
+    while (map) {
+      lastMap = map;
+      if (map->GetRowGroup() == aPrevGroup) {
+        prevMap = map;
+        break;
       }
+      map = map->GetNextSibling();
     }
-    if (!prevMap) {
-      if (aPrevGroup) {
-        prevMap = lastMap;
-        aPrevGroup = (prevMap) ? prevMap->GetRowGroup() : nsnull;
-      }
-      else {
-        aPrevGroup = nsnull;
-      }
-    }
-    InsertGroupCellMap(prevMap, *newMap);
   }
+  if (!prevMap) {
+    if (aPrevGroup) {
+      prevMap = lastMap;
+      aPrevGroup = (prevMap) ? prevMap->GetRowGroup() : nsnull;
+    }
+    else {
+      aPrevGroup = nsnull;
+    }
+  }
+  InsertGroupCellMap(prevMap, *newMap);
 }
 
 void nsTableCellMap::RemoveGroupCellMap(nsTableRowGroupFrame* aGroup)
@@ -270,7 +268,7 @@ void
 nsTableCellMap::Synchronize(nsTableFrame* aTableFrame)
 {
   nsTableFrame::RowGroupArray orderedRowGroups;
-  nsAutoTPtrArray<nsCellMap, 8> maps;
+  nsAutoTArray<nsCellMap*, 8> maps;
 
   aTableFrame->OrderRowGroups(orderedRowGroups);
   if (!orderedRowGroups.Length()) {
@@ -1177,14 +1175,11 @@ nsCellMap::~nsCellMap()
 }
 
 /* static */
-nsresult
+void
 nsCellMap::Init()
 {
-  NS_ASSERTION(!sEmptyRow, "How did that happen?");
+  NS_ABORT_IF_FALSE(!sEmptyRow, "How did that happen?");
   sEmptyRow = new nsCellMap::CellDataArray();
-  NS_ENSURE_TRUE(sEmptyRow, NS_ERROR_OUT_OF_MEMORY);
-
-  return NS_OK;
 }
 
 /* static */
@@ -1644,7 +1639,7 @@ PRBool nsCellMap::CellsSpanOut(nsTArray<nsTableRowFrame*>& aRows) const
   PRInt32 numNewRows = aRows.Length();
   for (PRInt32 rowX = 0; rowX < numNewRows; rowX++) {
     nsIFrame* rowFrame = (nsIFrame *) aRows.ElementAt(rowX);
-    nsIFrame* childFrame = rowFrame->GetFirstChild(nsnull);
+    nsIFrame* childFrame = rowFrame->GetFirstPrincipalChild();
     while (childFrame) {
       nsTableCellFrame *cellFrame = do_QueryFrame(childFrame);
       if (cellFrame) {
@@ -1818,7 +1813,7 @@ nsCellMap::ExpandWithRows(nsTableCellMap&             aMap,
   for (PRInt32 rowX = startRowIndex; rowX <= endRowIndex; rowX++) {
     nsTableRowFrame* rFrame = aRowFrames.ElementAt(newRowIndex);
     // append cells
-    nsIFrame* cFrame = rFrame->GetFirstChild(nsnull);
+    nsIFrame* cFrame = rFrame->GetFirstPrincipalChild();
     PRInt32 colIndex = 0;
     while (cFrame) {
       nsTableCellFrame *cellFrame = do_QueryFrame(cFrame);
@@ -2281,7 +2276,7 @@ nsCellMap::RebuildConsideringRows(nsTableCellMap&             aMap,
     PRInt32 numNewRows = aRowsToInsert->Length();
     for (PRInt32 newRowX = 0; newRowX < numNewRows; newRowX++) {
       nsTableRowFrame* rFrame = aRowsToInsert->ElementAt(newRowX);
-      nsIFrame* cFrame = rFrame->GetFirstChild(nsnull);
+      nsIFrame* cFrame = rFrame->GetFirstPrincipalChild();
       while (cFrame) {
         nsTableCellFrame *cellFrame = do_QueryFrame(cFrame);
         if (cellFrame) {
