@@ -58,6 +58,9 @@
 #include "nsISupportsPrimitives.h"
 #include "prtypes.h"
 #include "nsSupportsPrimitives.h"
+#include "mozilla/dom/Element.h"
+
+using namespace mozilla::dom;
 
 // Implementation /////////////////////////////////////////////////////////////////
 
@@ -65,18 +68,38 @@
 
 // Implement our nsISupports methods
 
+NS_IMPL_CYCLE_COLLECTION_CLASS(nsBoxObject)
+
+NS_IMPL_CYCLE_COLLECTING_ADDREF(nsBoxObject)
+NS_IMPL_CYCLE_COLLECTING_RELEASE(nsBoxObject)
+
+DOMCI_DATA(BoxObject, nsBoxObject)
+
 // QueryInterface implementation for nsBoxObject
-NS_INTERFACE_MAP_BEGIN(nsBoxObject)
+NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsBoxObject)
   NS_INTERFACE_MAP_ENTRY(nsIBoxObject)
   NS_INTERFACE_MAP_ENTRY(nsPIBoxObject)
   NS_INTERFACE_MAP_ENTRY(nsISupports)
   NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(BoxObject)
 NS_INTERFACE_MAP_END
 
+PR_STATIC_CALLBACK(PLDHashOperator)
+PropertyTraverser(const nsAString& aKey, nsISupports* aProperty, void* userArg)
+{
+  nsCycleCollectionTraversalCallback *cb = 
+    static_cast<nsCycleCollectionTraversalCallback*>(userArg);
 
-NS_IMPL_ADDREF(nsBoxObject)
-NS_IMPL_RELEASE(nsBoxObject)
+  cb->NoteXPCOMChild(aProperty);
 
+  return PL_DHASH_NEXT;
+}
+
+NS_IMPL_CYCLE_COLLECTION_UNLINK_0(nsBoxObject)
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsBoxObject)
+  if (tmp->mPropertyTable) {
+    tmp->mPropertyTable->EnumerateRead(PropertyTraverser, &cb);
+  }
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 // Constructors/Destructors
 nsBoxObject::nsBoxObject(void)
@@ -135,7 +158,12 @@ nsBoxObject::GetFrame(PRBool aFlushLayout)
     shell->FlushPendingNotifications(Flush_Frames);
   }
 
-  return shell->GetPrimaryFrameFor(mContent);
+  // The flush might have killed mContent.
+  if (!mContent) {
+    return nsnull;
+  }
+
+  return mContent->GetPrimaryFrame();
 }
 
 nsIPresShell*
@@ -154,7 +182,7 @@ nsBoxObject::GetPresShell(PRBool aFlushLayout)
     doc->FlushPendingNotifications(Flush_Layout);
   }
 
-  return doc->GetPrimaryShell();
+  return doc->GetShell();
 }
 
 nsresult 
@@ -172,7 +200,7 @@ nsBoxObject::GetOffsetRect(nsIntRect& aRect)
     nsPoint origin = frame->GetPositionIgnoringScrolling();
 
     // Find the frame parent whose content is the document element.
-    nsIContent *docElement = mContent->GetCurrentDoc()->GetRootContent();
+    Element *docElement = mContent->GetCurrentDoc()->GetRootElement();
     nsIFrame* parent = frame->GetParent();
     for (;;) {
       // If we've hit the document element, break here
@@ -401,7 +429,7 @@ nsBoxObject::GetFirstChild(nsIDOMElement * *aFirstVisibleChild)
   *aFirstVisibleChild = nsnull;
   nsIFrame* frame = GetFrame(PR_FALSE);
   if (!frame) return NS_OK;
-  nsIFrame* firstFrame = frame->GetFirstChild(nsnull);
+  nsIFrame* firstFrame = frame->GetFirstPrincipalChild();
   if (!firstFrame) return NS_OK;
   // get the content for the box and query to a dom element
   nsCOMPtr<nsIDOMElement> el = do_QueryInterface(firstFrame->GetContent());
@@ -448,7 +476,7 @@ nsBoxObject::GetPreviousSibling(nsIFrame* aParentFrame, nsIFrame* aFrame,
                                 nsIDOMElement** aResult)
 {
   *aResult = nsnull;
-  nsIFrame* nextFrame = aParentFrame->GetFirstChild(nsnull);
+  nsIFrame* nextFrame = aParentFrame->GetFirstPrincipalChild();
   nsIFrame* prevFrame = nsnull;
   while (nextFrame) {
     if (nextFrame == aFrame)
