@@ -44,7 +44,6 @@
 #include "nsContentUtils.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMNSEvent.h"
-#include "nsIDOMNSUIEvent.h"
 #include "nsIDOMXULElement.h"
 #include "nsIXULDocument.h"
 #include "nsIXULTemplateBuilder.h"
@@ -54,7 +53,6 @@
 #include "nsCSSFrameConstructor.h"
 #include "nsLayoutUtils.h"
 #include "nsIViewManager.h"
-#include "nsILookAndFeel.h"
 #include "nsIComponentManager.h"
 #include "nsITimer.h"
 #include "nsFocusManager.h"
@@ -70,6 +68,16 @@
 #include "nsPIDOMWindow.h"
 #include "nsPIWindowRoot.h"
 #include "nsFrameManager.h"
+#include "nsIObserverService.h"
+#include "mozilla/Services.h"
+#include "mozilla/LookAndFeel.h"
+
+using namespace mozilla;
+
+#define FLAG_ALT        0x01
+#define FLAG_CONTROL    0x02
+#define FLAG_SHIFT      0x04
+#define FLAG_META       0x08
 
 const nsNavigationDirection DirectionFromKeyCodeTable[2][6] = {
   {
@@ -130,19 +138,24 @@ void nsMenuChainItem::Detach(nsMenuChainItem** aRoot)
 }
 
 NS_IMPL_ISUPPORTS4(nsXULPopupManager,
-                   nsIDOMKeyListener,
                    nsIDOMEventListener,
                    nsIMenuRollup,
-                   nsITimerCallback)
+                   nsITimerCallback,
+                   nsIObserver)
 
 nsXULPopupManager::nsXULPopupManager() :
   mRangeOffset(0),
   mCachedMousePoint(0, 0),
+  mCachedModifiers(0),
   mActiveMenuBar(nsnull),
   mPopups(nsnull),
   mNoHidePanels(nsnull),
   mTimerMenu(nsnull)
 {
+  nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
+  if (obs) {
+    obs->AddObserver(this, "xpcom-shutdown", PR_FALSE);
+  }
 }
 
 nsXULPopupManager::~nsXULPopupManager() 
@@ -165,6 +178,29 @@ nsXULPopupManager::Shutdown()
   NS_IF_RELEASE(sInstance);
 }
 
+NS_IMETHODIMP
+nsXULPopupManager::Observe(nsISupports *aSubject,
+                           const char *aTopic,
+                           const PRUnichar *aData)
+{
+  if (!nsCRT::strcmp(aTopic, "xpcom-shutdown")) {
+    if (mKeyListener) {
+      mKeyListener->RemoveEventListener(NS_LITERAL_STRING("keypress"), this, PR_TRUE);
+      mKeyListener->RemoveEventListener(NS_LITERAL_STRING("keydown"), this, PR_TRUE);
+      mKeyListener->RemoveEventListener(NS_LITERAL_STRING("keyup"), this, PR_TRUE);
+      mKeyListener = nsnull;
+    }
+    mRangeParent = nsnull;
+    // mOpeningPopup is cleared explicitly soon after using it.
+    nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
+    if (obs) {
+      obs->RemoveObserver(this, "xpcom-shutdown");
+    }
+  }
+
+  return NS_OK;
+}
+
 nsXULPopupManager*
 nsXULPopupManager::GetInstance()
 {
@@ -433,7 +469,9 @@ nsXULPopupManager::InitTriggerEvent(nsIDOMEvent* aEvent, nsIContent* aPopup,
     }
   }
 
-  nsCOMPtr<nsIDOMNSUIEvent> uiEvent = do_QueryInterface(aEvent);
+  mCachedModifiers = 0;
+
+  nsCOMPtr<nsIDOMUIEvent> uiEvent = do_QueryInterface(aEvent);
   if (uiEvent) {
     uiEvent->GetRangeParent(getter_AddRefs(mRangeParent));
     uiEvent->GetRangeOffset(&mRangeOffset);
@@ -446,6 +484,22 @@ nsXULPopupManager::InitTriggerEvent(nsIDOMEvent* aEvent, nsIContent* aPopup,
       nsEvent* event;
       event = privateEvent->GetInternalNSEvent();
       if (event) {
+        if (event->eventStructType == NS_MOUSE_EVENT ||
+            event->eventStructType == NS_KEY_EVENT) {
+          nsInputEvent* inputEvent = static_cast<nsInputEvent*>(event);
+          if (inputEvent->isAlt) {
+            mCachedModifiers |= FLAG_ALT;
+          }
+          if (inputEvent->isControl) {
+            mCachedModifiers |= FLAG_CONTROL;
+          }
+          if (inputEvent->isShift) {
+            mCachedModifiers |= FLAG_SHIFT;
+          }
+          if (inputEvent->isMeta) {
+            mCachedModifiers |= FLAG_META;
+          }
+        }
         nsIDocument* doc = aPopup->GetCurrentDoc();
         if (doc) {
           nsIPresShell* presShell = doc->GetShell();
@@ -665,11 +719,9 @@ CheckCaretDrawingState() {
     if (!window)
       return;
 
-    nsCOMPtr<nsIDOMWindowInternal> windowInternal = do_QueryInterface(window);
-
     nsCOMPtr<nsIDOMDocument> domDoc;
     nsCOMPtr<nsIDocument> focusedDoc;
-    windowInternal->GetDocument(getter_AddRefs(domDoc));
+    window->GetDocument(getter_AddRefs(domDoc));
     focusedDoc = do_QueryInterface(domDoc);
     if (!focusedDoc)
       return;
@@ -966,9 +1018,8 @@ nsXULPopupManager::HidePopupAfterDelay(nsMenuPopupFrame* aPopup)
   // Kick off a close timer.
   KillMenuTimer();
 
-  PRInt32 menuDelay = 300;   // ms
-  aPopup->PresContext()->LookAndFeel()->
-    GetMetric(nsILookAndFeel::eMetric_SubmenuDelay, menuDelay);
+  PRInt32 menuDelay =
+    LookAndFeel::GetInt(LookAndFeel::eIntID_SubmenuDelay, 300); // ms
 
   // Kick off the timer.
   mCloseTimer = do_CreateInstance("@mozilla.org/timer;1");
@@ -1158,10 +1209,19 @@ nsXULPopupManager::FirePopupShowingEvent(nsIContent* aPopup,
   }
 
   event.refPoint = mCachedMousePoint;
+
+  event.isAlt = !!(mCachedModifiers & FLAG_ALT);
+  event.isControl = !!(mCachedModifiers & FLAG_CONTROL);
+  event.isShift = !!(mCachedModifiers & FLAG_SHIFT);
+  event.isMeta = !!(mCachedModifiers & FLAG_META);
+
   nsEventDispatcher::Dispatch(popup, presContext, &event, nsnull, &status);
+
   mCachedMousePoint = nsIntPoint(0, 0);
   mOpeningPopup = nsnull;
 
+  mCachedModifiers = 0;
+
   // if a panel, blur whatever has focus so that the panel can take the focus.
   // This is done after the popupshowing event in case that event is cancelled.
   // Using noautofocus="true" will disable this behaviour, which is needed for
@@ -1342,8 +1402,11 @@ nsXULPopupManager::GetVisiblePopups()
 
   item = mNoHidePanels;
   while (item) {
-    if (item->Frame()->PopupState() == ePopupOpenAndVisible)
+    // skip panels which are not open and visible as well as draggable popups,
+    // as those don't respond to events.
+    if (item->Frame()->PopupState() == ePopupOpenAndVisible && !item->Frame()->IsDragPopup()) {
       popups.AppendElement(static_cast<nsIFrame*>(item->Frame()));
+    }
     item = item->GetParent();
   }
 
@@ -1941,7 +2004,7 @@ nsXULPopupManager::GetNextMenuItem(nsIFrame* aParent,
   if (aStart)
     currFrame = aStart->GetNextSibling();
   else 
-    currFrame = immediateParent->GetFirstChild(nsnull);
+    currFrame = immediateParent->GetFirstPrincipalChild();
   
   while (currFrame) {
     // See if it's a menu item.
@@ -1952,7 +2015,7 @@ nsXULPopupManager::GetNextMenuItem(nsIFrame* aParent,
     currFrame = currFrame->GetNextSibling();
   }
 
-  currFrame = immediateParent->GetFirstChild(nsnull);
+  currFrame = immediateParent->GetFirstPrincipalChild();
 
   // Still don't have anything. Try cycling from the beginning.
   while (currFrame && currFrame != aStart) {
@@ -1981,7 +2044,7 @@ nsXULPopupManager::GetPreviousMenuItem(nsIFrame* aParent,
   if (!immediateParent)
     immediateParent = aParent;
 
-  const nsFrameList& frames(immediateParent->GetChildList(nsnull));
+  const nsFrameList& frames(immediateParent->PrincipalChildList());
 
   nsIFrame* currFrame = nsnull;
   if (aStart)
@@ -2032,9 +2095,9 @@ nsXULPopupManager::IsValidMenuItem(nsPresContext* aPresContext,
 
   PRBool skipNavigatingDisabledMenuItem = PR_TRUE;
   if (aOnPopup) {
-    aPresContext->LookAndFeel()->
-      GetMetric(nsILookAndFeel::eMetric_SkipNavigatingDisabledMenuItem,
-                skipNavigatingDisabledMenuItem);
+    skipNavigatingDisabledMenuItem =
+      LookAndFeel::GetInt(LookAndFeel::eIntID_SkipNavigatingDisabledMenuItem,
+                          0) != 0;
   }
 
   return !(skipNavigatingDisabledMenuItem &&
@@ -2043,7 +2106,30 @@ nsXULPopupManager::IsValidMenuItem(nsPresContext* aPresContext,
 }
 
 nsresult
-nsXULPopupManager::KeyUp(nsIDOMEvent* aKeyEvent)
+nsXULPopupManager::HandleEvent(nsIDOMEvent* aEvent)
+{
+  nsCOMPtr<nsIDOMKeyEvent> keyEvent = do_QueryInterface(aEvent);
+  NS_ENSURE_TRUE(keyEvent, NS_ERROR_UNEXPECTED);
+
+  nsAutoString eventType;
+  keyEvent->GetType(eventType);
+  if (eventType.EqualsLiteral("keyup")) {
+    return KeyUp(keyEvent);
+  }
+  if (eventType.EqualsLiteral("keydown")) {
+    return KeyDown(keyEvent);
+  }
+  if (eventType.EqualsLiteral("keypress")) {
+    return KeyPress(keyEvent);
+  }
+
+  NS_ABORT();
+
+  return NS_OK;
+}
+
+nsresult
+nsXULPopupManager::KeyUp(nsIDOMKeyEvent* aKeyEvent)
 {
   // don't do anything if a menu isn't open or a menubar isn't active
   if (!mActiveMenuBar) {
@@ -2059,7 +2145,7 @@ nsXULPopupManager::KeyUp(nsIDOMEvent* aKeyEvent)
 }
 
 nsresult
-nsXULPopupManager::KeyDown(nsIDOMEvent* aKeyEvent)
+nsXULPopupManager::KeyDown(nsIDOMKeyEvent* aKeyEvent)
 {
   nsMenuChainItem* item = GetTopVisibleMenu();
   if (item && item->Frame()->IsMenuLocked())
@@ -2077,22 +2163,21 @@ nsXULPopupManager::KeyDown(nsIDOMEvent* aKeyEvent)
   nsMenuBarListener::GetMenuAccessKey(&menuAccessKey);
   if (menuAccessKey) {
     PRUint32 theChar;
-    nsCOMPtr<nsIDOMKeyEvent> keyEvent = do_QueryInterface(aKeyEvent);
-    keyEvent->GetKeyCode(&theChar);
+    aKeyEvent->GetKeyCode(&theChar);
 
     if (theChar == (PRUint32)menuAccessKey) {
       PRBool ctrl = PR_FALSE;
       if (menuAccessKey != nsIDOMKeyEvent::DOM_VK_CONTROL)
-        keyEvent->GetCtrlKey(&ctrl);
+        aKeyEvent->GetCtrlKey(&ctrl);
       PRBool alt=PR_FALSE;
       if (menuAccessKey != nsIDOMKeyEvent::DOM_VK_ALT)
-        keyEvent->GetAltKey(&alt);
+        aKeyEvent->GetAltKey(&alt);
       PRBool shift=PR_FALSE;
       if (menuAccessKey != nsIDOMKeyEvent::DOM_VK_SHIFT)
-        keyEvent->GetShiftKey(&shift);
+        aKeyEvent->GetShiftKey(&shift);
       PRBool meta=PR_FALSE;
       if (menuAccessKey != nsIDOMKeyEvent::DOM_VK_META)
-        keyEvent->GetMetaKey(&meta);
+        aKeyEvent->GetMetaKey(&meta);
       if (!(ctrl || alt || shift || meta)) {
         // The access key just went down and no other
         // modifiers are already down.
@@ -2112,7 +2197,7 @@ nsXULPopupManager::KeyDown(nsIDOMEvent* aKeyEvent)
 }
 
 nsresult
-nsXULPopupManager::KeyPress(nsIDOMEvent* aKeyEvent)
+nsXULPopupManager::KeyPress(nsIDOMKeyEvent* aKeyEvent)
 {
   // Don't check prevent default flag -- menus always get first shot at key events.
   // When a menu is open, the prevent default flag on a keypress is always set, so
@@ -2134,6 +2219,7 @@ nsXULPopupManager::KeyPress(nsIDOMEvent* aKeyEvent)
     return NS_OK;
 
   nsCOMPtr<nsIDOMKeyEvent> keyEvent = do_QueryInterface(aKeyEvent);
+  NS_ENSURE_TRUE(keyEvent, NS_ERROR_UNEXPECTED);
   PRUint32 theChar;
   keyEvent->GetKeyCode(&theChar);
 
