@@ -55,7 +55,6 @@
 nsTreeColumn::nsTreeColumn(nsTreeColumns* aColumns, nsIContent* aContent)
   : mContent(aContent),
     mColumns(aColumns),
-    mNext(nsnull),
     mPrevious(nsnull)
 {
   NS_ASSERTION(aContent &&
@@ -70,15 +69,32 @@ nsTreeColumn::~nsTreeColumn()
 {
   if (mNext) {
     mNext->SetPrevious(nsnull);
-    NS_RELEASE(mNext);
   }
 }
 
+NS_IMPL_CYCLE_COLLECTION_CLASS(nsTreeColumn)
+NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsTreeColumn)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mContent)
+  if (tmp->mNext) {
+    tmp->mNext->SetPrevious(nsnull);
+    NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mNext)
+  }
+NS_IMPL_CYCLE_COLLECTION_UNLINK_END
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsTreeColumn)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mContent)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mNext)
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
+
+NS_IMPL_CYCLE_COLLECTING_ADDREF(nsTreeColumn)
+NS_IMPL_CYCLE_COLLECTING_RELEASE(nsTreeColumn)
+
+DOMCI_DATA(TreeColumn, nsTreeColumn)
+
 // QueryInterface implementation for nsTreeColumn
-NS_INTERFACE_MAP_BEGIN(nsTreeColumn)
+NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsTreeColumn)
   NS_INTERFACE_MAP_ENTRY(nsITreeColumn)
   NS_INTERFACE_MAP_ENTRY(nsISupports)
-  NS_INTERFACE_MAP_ENTRY_DOM_CLASSINFO(TreeColumn)
+  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(TreeColumn)
   if (aIID.Equals(NS_GET_IID(nsTreeColumn))) {
     AddRef();
     *aInstancePtr = this;
@@ -86,52 +102,31 @@ NS_INTERFACE_MAP_BEGIN(nsTreeColumn)
   }
   else
 NS_INTERFACE_MAP_END
-                                                                                
-NS_IMPL_ADDREF(nsTreeColumn)
-NS_IMPL_RELEASE(nsTreeColumn)
-
-nsIFrame*
-nsTreeColumn::GetFrame(nsTreeBodyFrame* aBodyFrame)
-{
-  NS_PRECONDITION(aBodyFrame, "null frame?");
-
-  nsIPresShell *shell = aBodyFrame->PresContext()->PresShell();
-  if (!shell)
-    return nsnull;
-
-  return shell->GetPrimaryFrameFor(mContent);
-}
 
 nsIFrame*
 nsTreeColumn::GetFrame()
 {
-  nsCOMPtr<nsIDocument> document = mContent->GetDocument();
-  if (!document)
-    return nsnull;
+  NS_ENSURE_TRUE(mContent, nsnull);
 
-  nsIPresShell *shell = document->GetPrimaryShell();
-  if (!shell)
-    return nsnull;
-
-  return shell->GetPrimaryFrameFor(mContent);
+  return mContent->GetPrimaryFrame();
 }
 
 PRBool
 nsTreeColumn::IsLastVisible(nsTreeBodyFrame* aBodyFrame)
 {
-  NS_ASSERTION(GetFrame(aBodyFrame), "should have checked for this already");
+  NS_ASSERTION(GetFrame(), "should have checked for this already");
 
   // cyclers are fixed width, don't adjust them
   if (IsCycler())
     return PR_FALSE;
 
   // we're certainly not the last visible if we're not visible
-  if (GetFrame(aBodyFrame)->GetRect().width == 0)
+  if (GetFrame()->GetRect().width == 0)
     return PR_FALSE;
 
   // try to find a visible successor
   for (nsTreeColumn *next = GetNext(); next; next = next->GetNext()) {
-    nsIFrame* frame = next->GetFrame(aBodyFrame);
+    nsIFrame* frame = next->GetFrame();
     if (frame && frame->GetRect().width > 0)
       return PR_FALSE;
   }
@@ -141,7 +136,7 @@ nsTreeColumn::IsLastVisible(nsTreeBodyFrame* aBodyFrame)
 nsresult
 nsTreeColumn::GetRect(nsTreeBodyFrame* aBodyFrame, nscoord aY, nscoord aHeight, nsRect* aResult)
 {
-  nsIFrame* frame = GetFrame(aBodyFrame);
+  nsIFrame* frame = GetFrame();
   if (!frame) {
     *aResult = nsRect();
     return NS_ERROR_FAILURE;
@@ -161,7 +156,7 @@ nsTreeColumn::GetRect(nsTreeBodyFrame* aBodyFrame, nscoord aY, nscoord aHeight,
 nsresult
 nsTreeColumn::GetXInTwips(nsTreeBodyFrame* aBodyFrame, nscoord* aResult)
 {
-  nsIFrame* frame = GetFrame(aBodyFrame);
+  nsIFrame* frame = GetFrame();
   if (!frame) {
     *aResult = 0;
     return NS_ERROR_FAILURE;
@@ -173,7 +168,7 @@ nsTreeColumn::GetXInTwips(nsTreeBodyFrame* aBodyFrame, nscoord* aResult)
 nsresult
 nsTreeColumn::GetWidthInTwips(nsTreeBodyFrame* aBodyFrame, nscoord* aResult)
 {
-  nsIFrame* frame = GetFrame(aBodyFrame);
+  nsIFrame* frame = GetFrame();
   if (!frame) {
     *aResult = 0;
     return NS_ERROR_FAILURE;
@@ -188,7 +183,11 @@ nsTreeColumn::GetWidthInTwips(nsTreeBodyFrame* aBodyFrame, nscoord* aResult)
 NS_IMETHODIMP
 nsTreeColumn::GetElement(nsIDOMElement** aElement)
 {
-  return CallQueryInterface(mContent, aElement);
+  if (mContent) {
+    return CallQueryInterface(mContent, aElement);
+  }
+  *aElement = nsnull;
+  return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
@@ -318,9 +317,15 @@ nsTreeColumn::Invalidate()
   const nsStyleText* textStyle = frame->GetStyleText();
 
   mTextAlignment = textStyle->mTextAlign;
-  if (mTextAlignment == 0 || mTextAlignment == 2) { // Left or Right
-    if (vis->mDirection == NS_STYLE_DIRECTION_RTL)
-      mTextAlignment = 2 - mTextAlignment; // Right becomes left, left becomes right.
+  // DEFAULT or END alignment sometimes means RIGHT
+  if ((mTextAlignment == NS_STYLE_TEXT_ALIGN_DEFAULT &&
+       vis->mDirection == NS_STYLE_DIRECTION_RTL) ||
+      (mTextAlignment == NS_STYLE_TEXT_ALIGN_END &&
+       vis->mDirection == NS_STYLE_DIRECTION_LTR)) {
+    mTextAlignment = NS_STYLE_TEXT_ALIGN_RIGHT;
+  } else if (mTextAlignment == NS_STYLE_TEXT_ALIGN_DEFAULT ||
+             mTextAlignment == NS_STYLE_TEXT_ALIGN_END) {
+    mTextAlignment = NS_STYLE_TEXT_ALIGN_LEFT;
   }
 
   // Figure out if we're the primary column (that has to have indentation
@@ -382,11 +387,13 @@ nsTreeColumns::~nsTreeColumns()
   nsTreeColumns::InvalidateColumns();
 }
 
+DOMCI_DATA(TreeColumns, nsTreeColumns)
+
 // QueryInterface implementation for nsTreeColumns
 NS_INTERFACE_MAP_BEGIN(nsTreeColumns)
   NS_INTERFACE_MAP_ENTRY(nsITreeColumns)
   NS_INTERFACE_MAP_ENTRY(nsISupports)
-  NS_INTERFACE_MAP_ENTRY_DOM_CLASSINFO(TreeColumns)
+  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(TreeColumns)
 NS_INTERFACE_MAP_END
                                                                                 
 NS_IMPL_ADDREF(nsTreeColumns)
@@ -453,7 +460,8 @@ nsTreeColumns::GetSortedColumn(nsITreeColumn** _retval)
   EnsureColumns();
   *_retval = nsnull;
   for (nsTreeColumn* currCol = mFirstColumn; currCol; currCol = currCol->GetNext()) {
-    if (nsContentUtils::HasNonEmptyAttr(currCol->mContent, kNameSpaceID_None,
+    if (currCol->mContent &&
+        nsContentUtils::HasNonEmptyAttr(currCol->mContent, kNameSpaceID_None,
                                         nsGkAtoms::sortDirection)) {
       NS_ADDREF(*_retval = currCol);
       return NS_OK;
@@ -475,7 +483,8 @@ nsTreeColumns::GetKeyColumn(nsITreeColumn** _retval)
 
   for (nsTreeColumn* currCol = mFirstColumn; currCol; currCol = currCol->GetNext()) {
     // Skip hidden columns.
-    if (currCol->mContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::hidden,
+    if (!currCol->mContent ||
+        currCol->mContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::hidden,
                                        nsGkAtoms::_true, eCaseMatters))
       continue;
 
@@ -585,8 +594,9 @@ nsTreeColumns::RestoreNaturalOrder()
   boxObject->GetElement(getter_AddRefs(element));
   nsCOMPtr<nsIContent> content = do_QueryInterface(element);
 
-  nsCOMPtr<nsIContent> colsContent;
-  nsTreeUtils::GetImmediateChild(content, nsGkAtoms::treecols, getter_AddRefs(colsContent));
+  // Strong ref, since we'll be setting attributes
+  nsCOMPtr<nsIContent> colsContent =
+    nsTreeUtils::GetImmediateChild(content, nsGkAtoms::treecols);
   if (!colsContent)
     return NS_OK;
 
@@ -626,22 +636,17 @@ nsTreeColumns::EnsureColumns()
     boxObject->GetElement(getter_AddRefs(treeElement));
     nsCOMPtr<nsIContent> treeContent = do_QueryInterface(treeElement);
 
-    nsCOMPtr<nsIContent> colsContent;
-    nsTreeUtils::GetDescendantChild(treeContent, nsGkAtoms::treecols, getter_AddRefs(colsContent));
+    nsIContent* colsContent =
+      nsTreeUtils::GetDescendantChild(treeContent, nsGkAtoms::treecols);
     if (!colsContent)
       return;
 
-    nsCOMPtr<nsIDocument> document = treeContent->GetDocument();
-    nsIPresShell *shell = document->GetPrimaryShell();
-    if (!shell)
-      return;
-
-    nsCOMPtr<nsIContent> colContent;
-    nsTreeUtils::GetDescendantChild(colsContent, nsGkAtoms::treecol, getter_AddRefs(colContent));
+    nsIContent* colContent =
+      nsTreeUtils::GetDescendantChild(colsContent, nsGkAtoms::treecol);
     if (!colContent)
       return;
 
-    nsIFrame* colFrame = shell->GetPrimaryFrameFor(colContent);
+    nsIFrame* colFrame = colContent->GetPrimaryFrame();
     if (!colFrame)
       return;
 
@@ -649,7 +654,7 @@ nsTreeColumns::EnsureColumns()
     if (!colFrame)
       return;
 
-    colFrame = colFrame->GetFirstChild(nsnull);
+    colFrame = colFrame->GetFirstPrincipalChild();
     if (!colFrame)
       return;
 
