@@ -47,6 +47,4 @@ cp $1/COPYING ./COPYING
 cp $1/README ./README
 cp $1/AUTHORS ./AUTHORS
 patch -p3 < ./alloca.diff
-patch -p3 <bug487519.patch
-patch -p3 <bug498855.patch
-patch -p3 <bug550184.patch
+patch -p3 < ./bug666672_gccWarnings.diff
