@@ -113,7 +113,7 @@ nsIconDecoder::WriteInternal(const char *aBuffer, PRUint32 aCount)
         }
 
         // Add the frame and signal
-        rv = mImage->AppendFrame(0, 0, mWidth, mHeight,
+        rv = mImage->EnsureFrame(0, 0, 0, mWidth, mHeight,
                                  gfxASurface::ImageFormatARGB32,
                                  &mImageData, &mPixBytesTotal);
         if (NS_FAILED(rv)) {
