@@ -46,6 +46,8 @@
 #include "nsString.h"
 #include "nsStreamUtils.h"
 
+using namespace mozilla;
+
 NS_IMPL_THREADSAFE_ISUPPORTS3(nsPNGEncoder, imgIEncoder, nsIInputStream, nsIAsyncInputStream)
 
 nsPNGEncoder::nsPNGEncoder() : mPNG(nsnull), mPNGinfo(nsnull),
@@ -55,7 +57,7 @@ nsPNGEncoder::nsPNGEncoder() : mPNG(nsnull), mPNGinfo(nsnull),
                                mImageBufferUsed(0), mImageBufferReadPoint(0),
                                mCallback(nsnull),
                                mCallbackTarget(nsnull), mNotifyThreshold(0),
-                               mMonitor("PNG Encoder Monitor")
+                               mReentrantMonitor("nsPNGEncoder.mReentrantMonitor")
 {
 }
 
@@ -87,6 +89,7 @@ NS_IMETHODIMP nsPNGEncoder::InitFromData(const PRUint8* aData,
                                          PRUint32 aInputFormat,
                                          const nsAString& aOutputOptions)
 {
+  NS_ENSURE_ARG(aData);
   nsresult rv;
 
   rv = StartImageEncode(aWidth, aHeight, aInputFormat, aOutputOptions);
@@ -203,6 +206,21 @@ NS_IMETHODIMP nsPNGEncoder::StartImageEncode(PRUint32 aWidth,
   return NS_OK;
 }
 
+// Returns the image buffer size
+NS_IMETHODIMP nsPNGEncoder::GetImageBufferSize(PRUint32 *aOutputSize)
+{
+  NS_ENSURE_ARG_POINTER(aOutputSize);
+  *aOutputSize = mImageBufferSize;
+  return NS_OK;
+}
+
+// Returns a pointer to the start of the image buffer
+NS_IMETHODIMP nsPNGEncoder::GetImageBuffer(char **aOutputBuffer)
+{
+  NS_ENSURE_ARG_POINTER(aOutputBuffer);
+  *aOutputBuffer = reinterpret_cast<char*>(mImageBuffer);
+  return NS_OK;
+}
 
 NS_IMETHODIMP nsPNGEncoder::AddImageFrame(const PRUint8* aData,
                                           PRUint32 aLength, // (unused,
@@ -272,6 +290,10 @@ NS_IMETHODIMP nsPNGEncoder::AddImageFrame(const PRUint8* aData,
     return NS_ERROR_INVALID_ARG;
   }
 
+#ifdef PNG_WRITE_FILTER_SUPPORTED
+  png_set_filter(mPNG, PNG_FILTER_TYPE_BASE, PNG_FILTER_VALUE_NONE);
+#endif
+
   // write each row: if we add more input formats, we may want to
   // generalize the conversions
   if (aInputFormat == INPUT_FORMAT_HOSTARGB) {
@@ -531,7 +553,7 @@ NS_IMETHODIMP nsPNGEncoder::ReadSegments(nsWriteSegmentFun aWriter,
                                          PRUint32 *_retval)
 {
   // Avoid another thread reallocing the buffer underneath us
-  mozilla::MonitorAutoEnter autoEnter(mMonitor);
+  ReentrantMonitorAutoEnter autoEnter(mReentrantMonitor);
 
   PRUint32 maxCount = mImageBufferUsed - mImageBufferReadPoint;
   if (maxCount == 0) {
@@ -670,7 +692,7 @@ nsPNGEncoder::WriteCallback(png_structp png, png_bytep data,
   if (that->mImageBufferUsed + size > that->mImageBufferSize) {
     // When we're reallocing the buffer we need to take the lock to ensure
     // that nobody is trying to read from the buffer we are destroying
-    mozilla::MonitorAutoEnter autoEnter(that->mMonitor);
+    ReentrantMonitorAutoEnter autoEnter(that->mReentrantMonitor);
 
     // expand buffer, just double each time
     that->mImageBufferSize *= 2;
@@ -697,7 +719,7 @@ nsPNGEncoder::NotifyListener()
   // AsyncWait and any that do encoding) so we lock to avoid notifying the
   // listener twice about the same data (which generally leads to a truncated
   // image).
-  mozilla::MonitorAutoEnter autoEnter(mMonitor);
+  ReentrantMonitorAutoEnter autoEnter(mReentrantMonitor);
 
   if (mCallback &&
       (mImageBufferUsed - mImageBufferReadPoint >= mNotifyThreshold ||
