@@ -49,22 +49,41 @@
 #include "nsReadableUtils.h"
 #include "nsCRT.h"
 
+#include "Image.h"
 #include "ImageErrors.h"
 #include "ImageLogging.h"
 
 #include "nspr.h"
 
+using namespace mozilla::imagelib;
 
-NS_IMPL_ISUPPORTS4(imgRequestProxy, imgIRequest, nsIRequest,
-                   nsISupportsPriority, nsISecurityInfoProvider)
+NS_IMPL_ADDREF(imgRequestProxy)
+NS_IMPL_RELEASE(imgRequestProxy)
+
+NS_INTERFACE_MAP_BEGIN(imgRequestProxy)
+  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, imgIRequest)
+  NS_INTERFACE_MAP_ENTRY(imgIRequest)
+  NS_INTERFACE_MAP_ENTRY(nsIRequest)
+  NS_INTERFACE_MAP_ENTRY(nsISupportsPriority)
+  NS_INTERFACE_MAP_ENTRY(nsISecurityInfoProvider)
+  NS_INTERFACE_MAP_ENTRY_CONDITIONAL(nsITimedChannel, TimedChannel() != nsnull)
+NS_INTERFACE_MAP_END
 
 imgRequestProxy::imgRequestProxy() :
   mOwner(nsnull),
+  mURI(nsnull),
+  mImage(nsnull),
+  mPrincipal(nsnull),
   mListener(nsnull),
   mLoadFlags(nsIRequest::LOAD_NORMAL),
+  mLockCount(0),
+  mAnimationConsumers(0),
   mCanceled(PR_FALSE),
   mIsInLoadGroup(PR_FALSE),
-  mListenerIsStrongRef(PR_FALSE)
+  mListenerIsStrongRef(PR_FALSE),
+  mDecodeRequested(PR_FALSE),
+  mDeferNotifications(PR_FALSE),
+  mSentStartContainer(PR_FALSE)
 {
   /* member initializers and constructor code */
 
@@ -75,6 +94,13 @@ imgRequestProxy::~imgRequestProxy()
   /* destructor code */
   NS_PRECONDITION(!mListener, "Someone forgot to properly cancel this request!");
 
+  // Unlock the image the proper number of times if we're holding locks on it.
+  // Note that UnlockImage() decrements mLockCount each time it's called.
+  while (mLockCount)
+    UnlockImage();
+
+  ClearAnimationConsumers();
+
   // Explicitly set mListener to null to ensure that the RemoveProxy
   // call below can't send |this| to an arbitrary listener while |this|
   // is being destroyed.  This is all belt-and-suspenders in view of the
@@ -98,15 +124,15 @@ imgRequestProxy::~imgRequestProxy()
   }
 }
 
-nsresult imgRequestProxy::Init(imgRequest *request, nsILoadGroup *aLoadGroup, imgIDecoderObserver *aObserver)
+nsresult imgRequestProxy::Init(imgRequest* request, nsILoadGroup* aLoadGroup, Image* aImage,
+                               nsIURI* aURI, imgIDecoderObserver* aObserver)
 {
   NS_PRECONDITION(!mOwner && !mListener, "imgRequestProxy is already initialized");
-  NS_PRECONDITION(request, "no request");
-  if (!request)
-    return NS_ERROR_NULL_POINTER;
 
   LOG_SCOPE_WITH_PARAM(gImgLog, "imgRequestProxy::Init", "request", request);
 
+  NS_ABORT_IF_FALSE(mAnimationConsumers == 0, "Cannot have animation before Init");
+
   mOwner = request;
   mListener = aObserver;
   // Make sure to addref mListener before the AddProxy call below, since
@@ -117,26 +143,74 @@ nsresult imgRequestProxy::Init(imgRequest *request, nsILoadGroup *aLoadGroup, im
     NS_ADDREF(mListener);
   }
   mLoadGroup = aLoadGroup;
+  mImage = aImage;
+  mURI = aURI;
 
   // Note: AddProxy won't send all the On* notifications immediately
-  request->AddProxy(this);
+  if (mOwner)
+    mOwner->AddProxy(this);
 
   return NS_OK;
 }
 
 nsresult imgRequestProxy::ChangeOwner(imgRequest *aNewOwner)
 {
-  if (mCanceled)
+  NS_PRECONDITION(mOwner, "Cannot ChangeOwner on a proxy without an owner!");
+
+  // If we're holding locks, unlock the old image.
+  // Note that UnlockImage decrements mLockCount each time it's called.
+  PRUint32 oldLockCount = mLockCount;
+  while (mLockCount)
+    UnlockImage();
+
+  // If we're holding animation requests, undo them.
+  PRUint32 oldAnimationConsumers = mAnimationConsumers;
+  ClearAnimationConsumers();
+
+  // Even if we are cancelled, we MUST change our image, because the image
+  // holds our status, and the status must always be correct.
+  mImage = aNewOwner->mImage;
+
+  // If we were locked, apply the locks here
+  for (PRUint32 i = 0; i < oldLockCount; i++)
+    LockImage();
+
+  if (mCanceled) {
+    // If we had animation requests, restore them before exiting
+    // (otherwise we restore them later below)
+    for (PRUint32 i = 0; i < oldAnimationConsumers; i++)
+      IncrementAnimationConsumers();
+
     return NS_OK;
+  }
+
+  // Were we decoded before?
+  PRBool wasDecoded = PR_FALSE;
+  if (mImage &&
+      (mImage->GetStatusTracker().GetImageStatus() &
+       imgIRequest::STATUS_FRAME_COMPLETE)) {
+    wasDecoded = PR_TRUE;
+  }
 
   // Passing false to aNotify means that mListener will still get
   // OnStopRequest, if needed.
   mOwner->RemoveProxy(this, NS_IMAGELIB_CHANGING_OWNER, PR_FALSE);
 
+  // If we had animation requests, restore them here. Note that we
+  // do this *after* RemoveProxy, which clears out animation consumers
+  // (see bug 601723).
+  for (PRUint32 i = 0; i < oldAnimationConsumers; i++)
+    IncrementAnimationConsumers();
+
   mOwner = aNewOwner;
 
   mOwner->AddProxy(this);
 
+  // If we were decoded, or if we'd previously requested a decode, request a
+  // decode on the new image
+  if (wasDecoded || mDecodeRequested)
+    mOwner->RequestDecode();
+
   return NS_OK;
 }
 
@@ -162,7 +236,7 @@ void imgRequestProxy::RemoveFromLoadGroup(PRBool releaseLoadGroup)
   */
   nsCOMPtr<imgIRequest> kungFuDeathGrip(this);
 
-  mLoadGroup->RemoveRequest(this, NS_OK, nsnull);
+  mLoadGroup->RemoveRequest(this, nsnull, NS_OK);
   mIsInLoadGroup = PR_FALSE;
 
   if (releaseLoadGroup) {
@@ -178,12 +252,10 @@ void imgRequestProxy::RemoveFromLoadGroup(PRBool releaseLoadGroup)
 NS_IMETHODIMP imgRequestProxy::GetName(nsACString &aName)
 {
   aName.Truncate();
-  if (mOwner) {
-    nsCOMPtr<nsIURI> uri;
-    mOwner->GetURI(getter_AddRefs(uri));
-    if (uri)
-      uri->GetSpec(aName);
-  }
+
+  if (mURI)
+    mURI->GetSpec(aName);
+
   return NS_OK;
 }
 
@@ -202,7 +274,7 @@ NS_IMETHODIMP imgRequestProxy::GetStatus(nsresult *aStatus)
 /* void cancel (in nsresult status); */
 NS_IMETHODIMP imgRequestProxy::Cancel(nsresult status)
 {
-  if (mCanceled || !mOwner)
+  if (mCanceled)
     return NS_ERROR_FAILURE;
 
   LOG_SCOPE(gImgLog, "imgRequestProxy::Cancel");
@@ -218,7 +290,8 @@ imgRequestProxy::DoCancel(nsresult status)
 {
   // Passing false to aNotify means that mListener will still get
   // OnStopRequest, if needed.
-  mOwner->RemoveProxy(this, status, PR_FALSE);
+  if (mOwner)
+    mOwner->RemoveProxy(this, status, PR_FALSE);
 
   NullOutListener();
 }
@@ -226,22 +299,104 @@ imgRequestProxy::DoCancel(nsresult status)
 /* void cancelAndForgetObserver (in nsresult aStatus); */
 NS_IMETHODIMP imgRequestProxy::CancelAndForgetObserver(nsresult aStatus)
 {
-  if (mCanceled || !mOwner)
+  if (mCanceled)
     return NS_ERROR_FAILURE;
 
   LOG_SCOPE(gImgLog, "imgRequestProxy::CancelAndForgetObserver");
 
   mCanceled = PR_TRUE;
 
+  // Now cheat and make sure our removal from loadgroup happens async
+  PRBool oldIsInLoadGroup = mIsInLoadGroup;
+  mIsInLoadGroup = PR_FALSE;
+
   // Passing false to aNotify means that mListener will still get
   // OnStopRequest, if needed.
-  mOwner->RemoveProxy(this, aStatus, PR_FALSE);
+  if (mOwner)
+    mOwner->RemoveProxy(this, aStatus, PR_FALSE);
+
+  mIsInLoadGroup = oldIsInLoadGroup;
+
+  if (mIsInLoadGroup) {
+    nsCOMPtr<nsIRunnable> ev =
+      NS_NewRunnableMethod(this, &imgRequestProxy::DoRemoveFromLoadGroup);
+    NS_DispatchToCurrentThread(ev);
+  }
 
   NullOutListener();
 
   return NS_OK;
 }
 
+/* void requestDecode (); */
+NS_IMETHODIMP
+imgRequestProxy::RequestDecode()
+{
+  if (!mOwner)
+    return NS_ERROR_FAILURE;
+
+  // Flag this, so we know to transfer the request if our owner changes
+  mDecodeRequested = PR_TRUE;
+
+  // Forward the request
+  return mOwner->RequestDecode();
+}
+
+/* void lockImage (); */
+NS_IMETHODIMP
+imgRequestProxy::LockImage()
+{
+  mLockCount++;
+  if (mImage)
+    return mImage->LockImage();
+  return NS_OK;
+}
+
+/* void unlockImage (); */
+NS_IMETHODIMP
+imgRequestProxy::UnlockImage()
+{
+  NS_ABORT_IF_FALSE(mLockCount > 0, "calling unlock but no locks!");
+
+  mLockCount--;
+  if (mImage)
+    return mImage->UnlockImage();
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+imgRequestProxy::IncrementAnimationConsumers()
+{
+  mAnimationConsumers++;
+  if (mImage)
+    mImage->IncrementAnimationConsumers();
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+imgRequestProxy::DecrementAnimationConsumers()
+{
+  // We may get here if some responsible code called Increment,
+  // then called us, but we have meanwhile called ClearAnimationConsumers
+  // because we needed to get rid of them earlier (see
+  // imgRequest::RemoveProxy), and hence have nothing left to
+  // decrement. (In such a case we got rid of the animation consumers
+  // early, but not the observer.)
+  if (mAnimationConsumers > 0) {
+    mAnimationConsumers--;
+    if (mImage)
+      mImage->DecrementAnimationConsumers();
+  }
+  return NS_OK;
+}
+
+void
+imgRequestProxy::ClearAnimationConsumers()
+{
+  while (mAnimationConsumers > 0)
+    DecrementAnimationConsumers();
+}
+
 /* void suspend (); */
 NS_IMETHODIMP imgRequestProxy::Suspend()
 {
@@ -283,32 +438,37 @@ NS_IMETHODIMP imgRequestProxy::SetLoadFlags(nsLoadFlags flags)
 /* attribute imgIContainer image; */
 NS_IMETHODIMP imgRequestProxy::GetImage(imgIContainer * *aImage)
 {
-  if (!mOwner)
+  // It's possible that our owner has an image but hasn't notified us of it -
+  // that'll happen if we get Canceled before the owner instantiates its image
+  // (because Canceling unregisters us as a listener on mOwner). If we're
+  // in that situation, just grab the image off of mOwner.
+  imgIContainer* imageToReturn = mImage ? mImage : mOwner->mImage;
+
+  if (!imageToReturn)
     return NS_ERROR_FAILURE;
 
-  mOwner->GetImage(aImage);
+  NS_ADDREF(*aImage = imageToReturn);
+
   return NS_OK;
 }
 
 /* readonly attribute unsigned long imageStatus; */
 NS_IMETHODIMP imgRequestProxy::GetImageStatus(PRUint32 *aStatus)
 {
-  if (!mOwner) {
-    *aStatus = imgIRequest::STATUS_ERROR;
-    return NS_ERROR_FAILURE;
-  }
+  *aStatus = GetStatusTracker().GetImageStatus();
 
-  *aStatus = mOwner->GetImageStatus();
   return NS_OK;
 }
 
 /* readonly attribute nsIURI URI; */
 NS_IMETHODIMP imgRequestProxy::GetURI(nsIURI **aURI)
 {
-  if (!mOwner)
+  if (!mURI)
     return NS_ERROR_FAILURE;
 
-  return mOwner->GetURI(aURI);
+  NS_ADDREF(*aURI = mURI);
+
+  return NS_OK;
 }
 
 /* readonly attribute imgIDecoderObserver decoderObserver; */
@@ -338,12 +498,11 @@ NS_IMETHODIMP imgRequestProxy::Clone(imgIDecoderObserver* aObserver,
                                      imgIRequest** aClone)
 {
   NS_PRECONDITION(aClone, "Null out param");
+
+  LOG_SCOPE(gImgLog, "imgRequestProxy::Clone");
+
   *aClone = nsnull;
-  imgRequestProxy* clone = new imgRequestProxy();
-  if (!clone) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-  NS_ADDREF(clone);
+  nsRefPtr<imgRequestProxy> clone = new imgRequestProxy();
 
   // It is important to call |SetLoadFlags()| before calling |Init()| because
   // |Init()| adds the request to the loadgroup.
@@ -352,30 +511,46 @@ NS_IMETHODIMP imgRequestProxy::Clone(imgIDecoderObserver* aObserver,
   // XXXldb That's not true anymore.  Stuff from imgLoader adds the
   // request to the loadgroup.
   clone->SetLoadFlags(mLoadFlags);
-  nsresult rv = clone->Init(mOwner, mLoadGroup, aObserver);
-  if (NS_FAILED(rv)) {
-    NS_RELEASE(clone);
+  nsresult rv = clone->Init(mOwner, mLoadGroup,
+                            mImage ? mImage : mOwner->mImage,
+                            mURI, aObserver);
+  if (NS_FAILED(rv))
     return rv;
-  }
 
-  // Assign to *aClone before calling NotifyProxyListener so that if
-  // the caller expects to only be notified for requests it's already
-  // holding pointers to it won't be surprised.
-  *aClone = clone;
+  clone->SetPrincipal(mPrincipal);
 
-  // Send the notifications to the clone's observer
-  mOwner->NotifyProxyListener(clone);
+  // Assign to *aClone before calling Notify so that if the caller expects to
+  // only be notified for requests it's already holding pointers to it won't be
+  // surprised.
+  NS_ADDREF(*aClone = clone);
+
+  // This is wrong!!! We need to notify asynchronously, but there's code that
+  // assumes that we don't. This will be fixed in bug 580466.
+  clone->SyncNotifyListener();
 
   return NS_OK;
 }
 
 /* readonly attribute nsIPrincipal imagePrincipal; */
 NS_IMETHODIMP imgRequestProxy::GetImagePrincipal(nsIPrincipal **aPrincipal)
+{
+  if (!mPrincipal)
+    return NS_ERROR_FAILURE;
+
+  NS_ADDREF(*aPrincipal = mPrincipal);
+
+  return NS_OK;
+}
+
+/* readonly attribute PRInt32 CORSMode; */
+NS_IMETHODIMP imgRequestProxy::GetCORSMode(PRInt32* aCorsMode)
 {
   if (!mOwner)
     return NS_ERROR_FAILURE;
 
-  return mOwner->GetPrincipal(aPrincipal);
+  *aCorsMode = mOwner->GetCORSMode();
+
+  return NS_OK;
 }
 
 /** nsISupportsPriority methods **/
@@ -425,7 +600,8 @@ NS_IMETHODIMP imgRequestProxy::GetHasTransferredData(PRBool* hasData)
 
 /** imgIContainerObserver methods **/
 
-void imgRequestProxy::FrameChanged(imgIContainer *container, nsIntRect * dirtyRect)
+void imgRequestProxy::FrameChanged(imgIContainer *container,
+                                   const nsIntRect *dirtyRect)
 {
   LOG_FUNC(gImgLog, "imgRequestProxy::FrameChanged");
 
@@ -453,10 +629,11 @@ void imgRequestProxy::OnStartContainer(imgIContainer *image)
 {
   LOG_FUNC(gImgLog, "imgRequestProxy::OnStartContainer");
 
-  if (mListener && !mCanceled) {
+  if (mListener && !mCanceled && !mSentStartContainer) {
     // Hold a ref to the listener while we call it, just in case.
     nsCOMPtr<imgIDecoderObserver> kungFuDeathGrip(mListener);
     mListener->OnStartContainer(this, image);
+    mSentStartContainer = PR_TRUE;
   }
 }
 
@@ -515,9 +692,18 @@ void imgRequestProxy::OnStopDecode(nsresult status, const PRUnichar *statusArg)
   }
 }
 
+void imgRequestProxy::OnDiscard()
+{
+  LOG_FUNC(gImgLog, "imgRequestProxy::OnDiscard");
 
+  if (mListener && !mCanceled) {
+    // Hold a ref to the listener while we call it, just in case.
+    nsCOMPtr<imgIDecoderObserver> kungFuDeathGrip(mListener);
+    mListener->OnDiscard(this);
+  }
+}
 
-void imgRequestProxy::OnStartRequest(nsIRequest *request, nsISupports *ctxt)
+void imgRequestProxy::OnStartRequest()
 {
 #ifdef PR_LOGGING
   nsCAutoString name;
@@ -534,8 +720,7 @@ void imgRequestProxy::OnStartRequest(nsIRequest *request, nsISupports *ctxt)
   }
 }
 
-void imgRequestProxy::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
-                                    nsresult statusCode, PRBool lastPart)
+void imgRequestProxy::OnStopRequest(PRBool lastPart)
 {
 #ifdef PR_LOGGING
   nsCAutoString name;
@@ -580,6 +765,10 @@ void imgRequestProxy::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
 
 void imgRequestProxy::NullOutListener()
 {
+  // If we have animation consumers, then they don't matter anymore
+  if (mListener)
+    ClearAnimationConsumers();
+
   if (mListenerIsStrongRef) {
     // Releasing could do weird reentery stuff, so just play it super-safe
     nsCOMPtr<imgIDecoderObserver> obs;
@@ -589,3 +778,103 @@ void imgRequestProxy::NullOutListener()
     mListener = nsnull;
   }
 }
+
+NS_IMETHODIMP
+imgRequestProxy::GetStaticRequest(imgIRequest** aReturn)
+{
+  *aReturn = nsnull;
+
+  PRBool animated;
+  if (!mImage || (NS_SUCCEEDED(mImage->GetAnimated(&animated)) && !animated)) {
+    // Early exit - we're not animated, so we don't have to do anything.
+    NS_ADDREF(*aReturn = this);
+    return NS_OK;
+  }
+
+  // We are animated. We need to extract the current frame from this image.
+  PRInt32 w = 0;
+  PRInt32 h = 0;
+  mImage->GetWidth(&w);
+  mImage->GetHeight(&h);
+  nsIntRect rect(0, 0, w, h);
+  nsCOMPtr<imgIContainer> currentFrame;
+  nsresult rv = mImage->ExtractFrame(imgIContainer::FRAME_CURRENT, rect,
+                                     imgIContainer::FLAG_SYNC_DECODE,
+                                     getter_AddRefs(currentFrame));
+  if (NS_FAILED(rv))
+    return rv;
+
+  nsRefPtr<Image> frame = static_cast<Image*>(currentFrame.get());
+
+  // Create a static imgRequestProxy with our new extracted frame.
+  nsRefPtr<imgRequestProxy> req = new imgRequestProxy();
+  req->Init(nsnull, nsnull, frame, mURI, nsnull);
+  req->SetPrincipal(mPrincipal);
+
+  NS_ADDREF(*aReturn = req);
+
+  return NS_OK;
+}
+
+void imgRequestProxy::SetPrincipal(nsIPrincipal *aPrincipal)
+{
+  mPrincipal = aPrincipal;
+}
+
+void imgRequestProxy::NotifyListener()
+{
+  // It would be nice to notify the observer directly in the status tracker
+  // instead of through the proxy, but there are several places we do extra
+  // processing when we receive notifications (like OnStopRequest()), and we
+  // need to check mCanceled everywhere too.
+
+  if (mOwner) {
+    // Send the notifications to our listener asynchronously.
+    GetStatusTracker().Notify(mOwner, this);
+  } else {
+    // We don't have an imgRequest, so we can only notify the clone of our
+    // current state, but we still have to do that asynchronously.
+    NS_ABORT_IF_FALSE(mImage,
+                      "if we have no imgRequest, we should have an Image");
+    mImage->GetStatusTracker().NotifyCurrentState(this);
+  }
+}
+
+void imgRequestProxy::SyncNotifyListener()
+{
+  // It would be nice to notify the observer directly in the status tracker
+  // instead of through the proxy, but there are several places we do extra
+  // processing when we receive notifications (like OnStopRequest()), and we
+  // need to check mCanceled everywhere too.
+
+  GetStatusTracker().SyncNotify(this);
+}
+
+void
+imgRequestProxy::SetImage(Image* aImage)
+{
+  NS_ABORT_IF_FALSE(aImage,  "Setting null image");
+  NS_ABORT_IF_FALSE(!mImage, "Setting image when we already have one");
+
+  mImage = aImage;
+
+  // Apply any locks we have
+  for (PRUint32 i = 0; i < mLockCount; ++i)
+    mImage->LockImage();
+
+  // Apply any animation consumers we have
+  for (PRUint32 i = 0; i < mAnimationConsumers; i++)
+    mImage->IncrementAnimationConsumers();
+}
+
+imgStatusTracker&
+imgRequestProxy::GetStatusTracker()
+{
+  // NOTE: It's possible that our mOwner has an Image that it didn't notify
+  // us about, if we were Canceled before its Image was constructed.
+  // (Canceling removes us as an observer, so mOwner has no way to notify us).
+  // That's why this method uses mOwner->GetStatusTracker() instead of just
+  // mOwner->mStatusTracker -- we might have a null mImage and yet have an
+  // mOwner with a non-null mImage (and a null mStatusTracker pointer).
+  return mImage ? mImage->GetStatusTracker() : mOwner->GetStatusTracker();
+}
