@@ -99,6 +99,9 @@ protected:
      * 
      * @param proxy
      *        The PAC-style proxy string to parse.  This must not be null.
+     * @param aResolveFlags
+     *        The flags passed to Resolve or AsyncResolve that are stored in 
+     *        proxyInfo.
      * @param result
      *        Upon return this points to a newly allocated nsProxyInfo or null
      *        if the proxy string was invalid.
@@ -106,6 +109,7 @@ protected:
      * @return A pointer beyond the parsed proxy string (never null).
      */
     NS_HIDDEN_(const char *) ExtractProxyInfo(const char *proxy,
+                                              PRUint32 aResolveFlags,
                                               nsProxyInfo **result);
 
     /**
@@ -122,10 +126,14 @@ protected:
      *
      * @param pacString
      *        The PAC-style proxy string to parse.  This may be empty.
+     * @param aResolveFlags
+     *        The flags passed to Resolve or AsyncResolve that are stored in 
+     *        proxyInfo.
      * @param result
      *        The resulting list of proxy info objects.
      */
     NS_HIDDEN_(void) ProcessPACString(const nsCString &pacString,
+                                      PRUint32 aResolveFlags,
                                       nsIProxyInfo **result);
 
     /**
@@ -199,6 +207,8 @@ protected:
      *        The failover timeout for this proxy.
      * @param next
      *        The next proxy to try if this one fails.
+     * @param aResolveFlags
+     *        The flags passed to resolve (from nsIProtocolProxyService).
      * @param result
      *        The resulting nsIProxyInfo object.
      */
@@ -208,6 +218,7 @@ protected:
                                                PRUint32 flags,
                                                PRUint32 timeout,
                                                nsIProxyInfo *next,
+                                               PRUint32 aResolveFlags,
                                                nsIProxyInfo **result);
 
     /**
@@ -220,6 +231,8 @@ protected:
      *        The URI to test.
      * @param info
      *        Information about the URI's protocol.
+     * @param flags
+     *        The flags passed to either the resolve or the asyncResolve method.
      * @param usePAC
      *        If this flag is set upon return, then PAC should be queried to
      *        resolve the proxy info.
@@ -228,6 +241,7 @@ protected:
      */
     NS_HIDDEN_(nsresult) Resolve_Internal(nsIURI *uri,
                                           const nsProtocolInfo &info,
+                                          PRUint32 flags,
                                           PRBool *usePAC, 
                                           nsIProxyInfo **result);
 
@@ -306,17 +320,6 @@ public:
         PRUint32 host_len;
     };
 
-    // These values correspond to the integer network.proxy.type preference
-    enum ProxyConfig {
-        eProxyConfig_Direct,
-        eProxyConfig_Manual,
-        eProxyConfig_PAC,
-        eProxyConfig_Direct4x,
-        eProxyConfig_WPAD,
-        eProxyConfig_System, // use system proxy settings if available, otherwise DIRECT
-        eProxyConfig_Last
-    };
-
 protected:
 
     // simplified array of filters defined by this struct
@@ -350,6 +353,9 @@ protected:
       ~FilterLink() { if (next) delete next; }
     };
 
+    // Indicates if local hosts (plain hostnames, no dots) should use the proxy
+    PRBool mFilterLocalHosts;
+
     // Holds an array of HostInfo objects
     nsTArray<nsAutoPtr<HostInfo> > mHostFiltersArray;
 
@@ -357,7 +363,7 @@ protected:
     // of FilterLink objects.
     FilterLink                  *mFilters;
 
-    ProxyConfig                  mProxyConfig;
+    PRUint32                     mProxyConfig;
 
     nsCString                    mHTTPProxyHost;
     PRInt32                      mHTTPProxyPort;
@@ -365,9 +371,6 @@ protected:
     nsCString                    mFTPProxyHost;
     PRInt32                      mFTPProxyPort;
 
-    nsCString                    mGopherProxyHost;
-    PRInt32                      mGopherProxyPort;
-
     nsCString                    mHTTPSProxyHost;
     PRInt32                      mHTTPSProxyPort;
     
