@@ -120,6 +120,16 @@ nsresult nsPartChannel::SendOnStopRequest(nsISupports* aContext,
     return listener->OnStopRequest(this, aContext, aStatus);
 }
 
+void nsPartChannel::SetContentDisposition(const nsACString& aContentDispositionHeader)
+{
+    mContentDispositionHeader = aContentDispositionHeader;
+    nsCOMPtr<nsIURI> uri;
+    GetURI(getter_AddRefs(uri));
+    NS_GetFilenameFromDisposition(mContentDispositionFilename,
+                                  mContentDispositionHeader, uri);
+    mContentDisposition = NS_GetContentDispositionFromHeader(mContentDispositionHeader, this);
+}
+
 //
 // nsISupports implementation...
 //
@@ -337,16 +347,32 @@ nsPartChannel::SetContentLength(PRInt32 aContentLength)
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetContentDisposition(nsACString &aContentDisposition)
+nsPartChannel::GetContentDisposition(PRUint32 *aContentDisposition)
 {
-    aContentDisposition = mContentDisposition;
+    if (mContentDispositionHeader.IsEmpty())
+        return NS_ERROR_NOT_AVAILABLE;
+
+    *aContentDisposition = mContentDisposition;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPartChannel::SetContentDisposition(const nsACString &aContentDisposition)
+nsPartChannel::GetContentDispositionFilename(nsAString &aContentDispositionFilename)
 {
-    mContentDisposition = aContentDisposition;
+    if (mContentDispositionFilename.IsEmpty())
+        return NS_ERROR_NOT_AVAILABLE;
+
+    aContentDispositionFilename = mContentDispositionFilename;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPartChannel::GetContentDispositionHeader(nsACString &aContentDispositionHeader)
+{
+    if (mContentDispositionHeader.IsEmpty())
+        return NS_ERROR_NOT_AVAILABLE;
+
+    aContentDispositionHeader = mContentDispositionHeader;
     return NS_OK;
 }
 
@@ -459,6 +485,8 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
     // fill buffer
     {
         bufLen = count + mBufLen;
+        NS_ENSURE_TRUE((bufLen >= count) && (bufLen >= mBufLen),
+                       NS_ERROR_FAILURE);
         buffer = (char *) malloc(bufLen);
         if (!buffer)
             return NS_ERROR_OUT_OF_MEMORY;
@@ -606,7 +634,7 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
         // have enough info to start a part, go ahead and buffer
         // enough to collect a boundary token.
         if (!mPartChannel || !(cursor[bufLen-1] == nsCRT::LF) )
-            bufAmt = PR_MIN(mTokenLen - 1, bufLen);
+            bufAmt = NS_MIN(mTokenLen - 1, bufLen);
     }
 
     if (bufAmt) {
@@ -799,8 +827,7 @@ nsMultiMixedConv::SendStart(nsIChannel *aChannel) {
     rv = mPartChannel->SetContentLength(mContentLength); // XXX Truncates 64-bit!
     if (NS_FAILED(rv)) return rv;
 
-    rv = mPartChannel->SetContentDisposition(mContentDisposition);
-    if (NS_FAILED(rv)) return rv;
+    mPartChannel->SetContentDisposition(mContentDisposition);
 
     nsLoadFlags loadFlags = 0;
     mPartChannel->GetLoadFlags(&loadFlags);
