@@ -1,28 +1,52 @@
+/*
+ * Copyright (c) 2008-2010 Mozilla Foundation
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a 
+ * copy of this software and associated documentation files (the "Software"), 
+ * to deal in the Software without restriction, including without limitation 
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
+ * and/or sell copies of the Software, and to permit persons to whom the 
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ * DEALINGS IN THE SOFTWARE.
+ */
+
 HTML5_ATOM(emptystring, "")
 HTML5_ATOM(title, "title")
 HTML5_ATOM(textarea, "textarea")
 HTML5_ATOM(style, "style")
-HTML5_ATOM(script, "script")
 HTML5_ATOM(xmp, "xmp")
 HTML5_ATOM(iframe, "iframe")
 HTML5_ATOM(noembed, "noembed")
 HTML5_ATOM(noframes, "noframes")
 HTML5_ATOM(noscript, "noscript")
 HTML5_ATOM(plaintext, "plaintext")
-HTML5_ATOM(table, "table")
+HTML5_ATOM(script, "script")
 HTML5_ATOM(caption, "caption")
 HTML5_ATOM(p, "p")
 HTML5_ATOM(address, "address")
 HTML5_ATOM(div, "div")
 HTML5_ATOM(a, "a")
 HTML5_ATOM(nobr, "nobr")
-HTML5_ATOM(button, "button")
 HTML5_ATOM(input, "input")
 HTML5_ATOM(option, "option")
 HTML5_ATOM(ruby, "ruby")
-HTML5_ATOM(optgroup, "optgroup")
-HTML5_ATOM(frameset, "frameset")
 HTML5_ATOM(select, "select")
+HTML5_ATOM(optgroup, "optgroup")
+HTML5_ATOM(table, "table")
+HTML5_ATOM(frameset, "frameset")
+HTML5_ATOM(button, "button")
+HTML5_ATOM(ul, "ul")
+HTML5_ATOM(ol, "ol")
 HTML5_ATOM(html, "html")
 HTML5_ATOM(td, "td")
 HTML5_ATOM(th, "th")
@@ -434,6 +458,7 @@ HTML5_ATOM(arabic_form, "arabic-form")
 HTML5_ATOM(cellpadding, "cellpadding")
 HTML5_ATOM(cellspacing, "cellspacing")
 HTML5_ATOM(columnwidth, "columnwidth")
+HTML5_ATOM(crossorigin, "crossorigin")
 HTML5_ATOM(columnalign, "columnalign")
 HTML5_ATOM(columnlines, "columnlines")
 HTML5_ATOM(contextmenu, "contextmenu")
@@ -711,12 +736,10 @@ HTML5_ATOM(mi, "mi")
 HTML5_ATOM(mn, "mn")
 HTML5_ATOM(mo, "mo")
 HTML5_ATOM(ms, "ms")
-HTML5_ATOM(ol, "ol")
 HTML5_ATOM(or_, "or")
 HTML5_ATOM(pi, "pi")
 HTML5_ATOM(rp, "rp")
 HTML5_ATOM(tt, "tt")
-HTML5_ATOM(ul, "ul")
 HTML5_ATOM(and_, "and")
 HTML5_ATOM(arg, "arg")
 HTML5_ATOM(abs, "abs")
@@ -823,6 +846,7 @@ HTML5_ATOM(param, "param")
 HTML5_ATOM(power, "power")
 HTML5_ATOM(reals, "reals")
 HTML5_ATOM(small_, "small")
+HTML5_ATOM(track, "track")
 HTML5_ATOM(tspan, "tspan")
 HTML5_ATOM(times, "times")
 HTML5_ATOM(union_, "union")
@@ -841,7 +865,6 @@ HTML5_ATOM(center, "center")
 HTML5_ATOM(canvas, "canvas")
 HTML5_ATOM(divide, "divide")
 HTML5_ATOM(degree, "degree")
-HTML5_ATOM(dialog, "dialog")
 HTML5_ATOM(domain, "domain")
 HTML5_ATOM(exists, "exists")
 HTML5_ATOM(fetile, "fetile")
@@ -849,6 +872,7 @@ HTML5_ATOM(feTile, "feTile")
 HTML5_ATOM(figure, "figure")
 HTML5_ATOM(forall, "forall")
 HTML5_ATOM(footer, "footer")
+HTML5_ATOM(hgroup, "hgroup")
 HTML5_ATOM(header, "header")
 HTML5_ATOM(keygen, "keygen")
 HTML5_ATOM(lambda, "lambda")
@@ -870,7 +894,6 @@ HTML5_ATOM(strike, "strike")
 HTML5_ATOM(strong, "strong")
 HTML5_ATOM(switch_, "switch")
 HTML5_ATOM(symbol, "symbol")
-HTML5_ATOM(spacer, "spacer")
 HTML5_ATOM(subset, "subset")
 HTML5_ATOM(tbreak, "tbreak")
 HTML5_ATOM(vector, "vector")
@@ -930,7 +953,6 @@ HTML5_ATOM(basefont, "basefont")
 HTML5_ATOM(clippath, "clippath")
 HTML5_ATOM(clipPath, "clipPath")
 HTML5_ATOM(codomain, "codomain")
-HTML5_ATOM(datagrid, "datagrid")
 HTML5_ATOM(emptyset, "emptyset")
 HTML5_ATOM(factorof, "factorof")
 HTML5_ATOM(fieldset, "fieldset")
@@ -943,6 +965,7 @@ HTML5_ATOM(listener, "listener")
 HTML5_ATOM(lowlimit, "lowlimit")
 HTML5_ATOM(metadata, "metadata")
 HTML5_ATOM(menclose, "menclose")
+HTML5_ATOM(menuitem, "menuitem")
 HTML5_ATOM(mphantom, "mphantom")
 HTML5_ATOM(polyline, "polyline")
 HTML5_ATOM(prefetch, "prefetch")
@@ -974,6 +997,7 @@ HTML5_ATOM(blockquote, "blockquote")
 HTML5_ATOM(divergence, "divergence")
 HTML5_ATOM(eulergamma, "eulergamma")
 HTML5_ATOM(equivalent, "equivalent")
+HTML5_ATOM(figcaption, "figcaption")
 HTML5_ATOM(imaginaryi, "imaginaryi")
 HTML5_ATOM(malignmark, "malignmark")
 HTML5_ATOM(munderover, "munderover")
@@ -983,7 +1007,6 @@ HTML5_ATOM(solidcolor, "solidcolor")
 HTML5_ATOM(altglyphdef, "altglyphdef")
 HTML5_ATOM(altGlyphDef, "altGlyphDef")
 HTML5_ATOM(determinant, "determinant")
-HTML5_ATOM(eventsource, "eventsource")
 HTML5_ATOM(femergenode, "femergenode")
 HTML5_ATOM(feMergeNode, "feMergeNode")
 HTML5_ATOM(fecomposite, "fecomposite")
