@@ -52,6 +52,9 @@
 #include "nsHtml5Module.h"
 #include "nsHtml5RefPtr.h"
 #include "nsIScriptError.h"
+#include "mozilla/Preferences.h"
+
+using namespace mozilla;
 
 static NS_DEFINE_CID(kCharsetAliasCID, NS_CHARSETALIAS_CID);
 
@@ -62,10 +65,10 @@ PRInt32 nsHtml5StreamParser::sTimerSubsequentDelay = 120;
 void
 nsHtml5StreamParser::InitializeStatics()
 {
-  nsContentUtils::AddIntPrefVarCache("html5.flushtimer.initialdelay",
-                                     &sTimerInitialDelay);
-  nsContentUtils::AddIntPrefVarCache("html5.flushtimer.subsequentdelay",
-                                     &sTimerSubsequentDelay);
+  Preferences::AddIntVarCache(&sTimerInitialDelay,
+                              "html5.flushtimer.initialdelay");
+  Preferences::AddIntVarCache(&sTimerSubsequentDelay,
+                              "html5.flushtimer.subsequentdelay");
 }
 
 /*
@@ -200,12 +203,12 @@ nsHtml5StreamParser::nsHtml5StreamParser(nsHtml5TreeOpExecutor* aExecutor,
   // Chardet is initialized here even if it turns out to be useless
   // to make the chardet refcount its observer (nsHtml5StreamParser)
   // on the main thread.
-  const nsAdoptingString& detectorName = 
-    nsContentUtils::GetLocalizedStringPref("intl.charset.detector");
+  const nsAdoptingCString& detectorName =
+    Preferences::GetLocalizedCString("intl.charset.detector");
   if (!detectorName.IsEmpty()) {
     nsCAutoString detectorContractID;
     detectorContractID.AssignLiteral(NS_CHARSET_DETECTOR_CONTRACTID_BASE);
-    AppendUTF16toUTF8(detectorName, detectorContractID);
+    detectorContractID += detectorName;
     if ((mChardet = do_CreateInstance(detectorContractID.get()))) {
       (void) mChardet->Init(this);
       mFeedChardet = PR_TRUE;
@@ -684,6 +687,7 @@ nsHtml5StreamParser::OnStartRequest(nsIRequest* aRequest, nsISupports* aContext)
       // This is the old Gecko behavior but the HTML5 spec disagrees.
       // Don't reparse on POST.
       mReparseForbidden = PR_TRUE;
+      mFeedChardet = PR_FALSE; // can't restart anyway
     }
   }
 
@@ -901,6 +905,7 @@ nsHtml5StreamParser::internalEncodingDeclaration(nsString* aEncoding)
   }
   if (eq) {
     mCharsetSource = kCharsetFromMetaTag; // become confident
+    mFeedChardet = PR_FALSE; // don't feed chardet when confident
     return PR_FALSE;
   }
   
@@ -926,12 +931,17 @@ nsHtml5StreamParser::internalEncodingDeclaration(nsString* aEncoding)
     return PR_FALSE;
   }
 
+  // Avoid having the chardet ask for another restart after this restart
+  // request.
+  mFeedChardet = PR_FALSE;
   mTreeBuilder->NeedsCharsetSwitchTo(preferred, kCharsetFromMetaTag);
   FlushTreeOpsAndDisarmTimer();
   Interrupt();
   // the tree op executor will cause the stream parser to terminate
   // if the charset switch request is accepted or it'll uninterrupt 
-  // if the request failed.
+  // if the request failed. Note that if the restart request fails,
+  // we don't bother trying to make chardet resume. Might as well
+  // assume that chardet-requested restarts would fail, too.
   return PR_TRUE;
 }
 
