@@ -123,11 +123,13 @@ nsNSSCertificateDB::FindCertByNickname(nsISupports *aToken,
   }
   if (cert) {
     PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("got it\n"));
-    nsCOMPtr<nsIX509Cert> pCert = new nsNSSCertificate(cert);
+    nsCOMPtr<nsIX509Cert> pCert = nsNSSCertificate::Create(cert);
     CERT_DestroyCertificate(cert);
-    *_rvCert = pCert;
-    NS_ADDREF(*_rvCert);
-    return NS_OK;
+    if (pCert) {
+      *_rvCert = pCert;
+      NS_ADDREF(*_rvCert);
+      return NS_OK;
+    }
   }
   *_rvCert = nsnull;
   return NS_ERROR_FAILURE;
@@ -174,7 +176,7 @@ nsNSSCertificateDB::FindCertByDBKey(const char *aDBkey, nsISupports *aToken,
   cert = CERT_FindCertByIssuerAndSN(CERT_GetDefaultCertDB(), &issuerSN);
   PR_FREEIF(keyItem.data);
   if (cert) {
-    nsNSSCertificate *nssCert = new nsNSSCertificate(cert);
+    nsNSSCertificate *nssCert = nsNSSCertificate::Create(cert);
     CERT_DestroyCertificate(cert);
     if (nssCert == nsnull)
       return NS_ERROR_OUT_OF_MEMORY;
@@ -374,7 +376,7 @@ nsNSSCertificateDB::handleCACertDownload(nsIArray *x509Certs,
   der.len = 0;
   
   if (!tmpCert) {
-    NS_ERROR("Couldn't create cert from DER blob\n");
+    NS_ERROR("Couldn't create cert from DER blob");
     return NS_ERROR_FAILURE;
   }
 
@@ -447,7 +449,7 @@ nsNSSCertificateDB::handleCACertDownload(nsIArray *x509Certs,
     der.len = 0;
 
     if (!tmpCert2) {
-      NS_ASSERTION(0, "Couldn't create temp cert from DER blob\n");
+      NS_ERROR("Couldn't create temp cert from DER blob");
       continue;  // Let's try to import the rest of 'em
     }
     
@@ -531,9 +533,20 @@ nsNSSCertificateDB::ImportEmailCertificate(PRUint8 * data, PRUint32 length,
   CERTCertListNode *node;
   PRTime now;
   SECCertUsage certusage;
+  SECCertificateUsage certificateusage;
   SECItem **rawArray;
   int numcerts;
   int i;
+  CERTValOutParam cvout[1];
+  cvout[0].type = cert_po_end;
+
+  nsCOMPtr<nsINSSComponent> inss = do_GetService(kNSSComponentCID, &nsrv);
+  if (!inss)
+    return nsrv;
+  nsRefPtr<nsCERTValInParamWrapper> survivingParams;
+  nsrv = inss->GetDefaultCERTValInParam(survivingParams);
+  if (NS_FAILED(nsrv))
+    return nsrv;
  
   PRArenaPool *arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
   if (!arena)
@@ -547,6 +560,7 @@ nsNSSCertificateDB::ImportEmailCertificate(PRUint8 * data, PRUint32 length,
 
   certdb = CERT_GetDefaultCertDB();
   certusage = certUsageEmailRecipient;
+  certificateusage = certificateUsageEmailRecipient;
 
   numcerts = certCollection->numcerts;
 
@@ -589,6 +603,7 @@ nsNSSCertificateDB::ImportEmailCertificate(PRUint8 * data, PRUint32 length,
    * valid chains, then import them.
    */
   now = PR_Now();
+
   for (node = CERT_LIST_HEAD(certList);
        !CERT_LIST_END(node,certList);
        node = CERT_LIST_NEXT(node)) {
@@ -599,9 +614,19 @@ nsNSSCertificateDB::ImportEmailCertificate(PRUint8 * data, PRUint32 length,
       continue;
     }
 
-    if (CERT_VerifyCert(certdb, node->cert, 
-        PR_TRUE, certusage, now, ctx, NULL) != SECSuccess) {
-      alert_and_skip = true;
+    if (!nsNSSComponent::globalConstFlagUsePKIXVerification) {
+      if (CERT_VerifyCert(certdb, node->cert,
+          PR_TRUE, certusage, now, ctx, NULL) != SECSuccess) {
+        alert_and_skip = true;
+      }
+    }
+    else {
+      if (CERT_PKIXVerifyCert(node->cert, certificateusage,
+                              survivingParams->GetRawPointerForNSS(),
+                              cvout, ctx)
+          != SECSuccess) {
+        alert_and_skip = true;
+      }
     }
 
     CERTCertificateList *certChain = nsnull;
@@ -615,7 +640,7 @@ nsNSSCertificateDB::ImportEmailCertificate(PRUint8 * data, PRUint32 length,
     }
 
     if (alert_and_skip) {    
-      nsCOMPtr<nsIX509Cert> certToShow = new nsNSSCertificate(node->cert);
+      nsCOMPtr<nsIX509Cert> certToShow = nsNSSCertificate::Create(node->cert);
       DisplayCertificateAlert(ctx, "NotImportingUnverifiedCert", certToShow);
       continue;
     }
@@ -772,6 +797,14 @@ nsresult
 nsNSSCertificateDB::ImportValidCACertsInList(CERTCertList *certList, nsIInterfaceRequestor *ctx)
 {
   SECItem **rawArray;
+  nsresult nsrv;
+  nsCOMPtr<nsINSSComponent> inss = do_GetService(kNSSComponentCID, &nsrv);
+  if (!inss)
+    return nsrv;
+  nsRefPtr<nsCERTValInParamWrapper> survivingParams;
+  nsrv = inss->GetDefaultCERTValInParam(survivingParams);
+  if (NS_FAILED(nsrv))
+    return nsrv;
 
   /* filter out the certs we don't want */
   SECStatus srv = CERT_FilterCertListByUsage(certList, certUsageAnyCA, PR_TRUE);
@@ -782,17 +815,29 @@ nsNSSCertificateDB::ImportValidCACertsInList(CERTCertList *certList, nsIInterfac
   /* go down the remaining list of certs and verify that they have
    * valid chains, if yes, then import.
    */
-  PRTime now = PR_Now();
   CERTCertListNode *node;
+  CERTValOutParam cvout[1];
+  cvout[0].type = cert_po_end;
+
   for (node = CERT_LIST_HEAD(certList);
        !CERT_LIST_END(node,certList);
        node = CERT_LIST_NEXT(node)) {
 
     bool alert_and_skip = false;
 
-    if (CERT_VerifyCert(CERT_GetDefaultCertDB(), node->cert, 
-        PR_TRUE, certUsageVerifyCA, now, ctx, NULL) != SECSuccess) {
-      alert_and_skip = true;
+    if (!nsNSSComponent::globalConstFlagUsePKIXVerification) {
+      if (CERT_VerifyCert(CERT_GetDefaultCertDB(), node->cert, 
+          PR_TRUE, certUsageVerifyCA, PR_Now(), ctx, NULL) != SECSuccess) {
+        alert_and_skip = true;
+      }
+    }
+    else {
+      if (CERT_PKIXVerifyCert(node->cert, certificateUsageVerifyCA,
+                              survivingParams->GetRawPointerForNSS(),
+                              cvout, ctx)
+          != SECSuccess) {
+        alert_and_skip = true;
+      }
     }
 
     CERTCertificateList *certChain = nsnull;
@@ -806,7 +851,7 @@ nsNSSCertificateDB::ImportValidCACertsInList(CERTCertList *certList, nsIInterfac
     }
 
     if (alert_and_skip) {    
-      nsCOMPtr<nsIX509Cert> certToShow = new nsNSSCertificate(node->cert);
+      nsCOMPtr<nsIX509Cert> certToShow = nsNSSCertificate::Create(node->cert);
       DisplayCertificateAlert(ctx, "NotImportingUnverifiedCert", certToShow);
       continue;
     }
@@ -912,7 +957,7 @@ nsNSSCertificateDB::ImportUserCertificate(PRUint8 *data, PRUint32 length, nsIInt
 
   slot = PK11_KeyForCertExists(cert, NULL, ctx);
   if ( slot == NULL ) {
-    nsCOMPtr<nsIX509Cert> certToShow = new nsNSSCertificate(cert);
+    nsCOMPtr<nsIX509Cert> certToShow = nsNSSCertificate::Create(cert);
     DisplayCertificateAlert(ctx, "UserCertIgnoredNoPrivateKey", certToShow);
     goto loser;
   }
@@ -940,7 +985,7 @@ nsNSSCertificateDB::ImportUserCertificate(PRUint8 *data, PRUint32 length, nsIInt
   PK11_FreeSlot(slot);
 
   {
-    nsCOMPtr<nsIX509Cert> certToShow = new nsNSSCertificate(cert);
+    nsCOMPtr<nsIX509Cert> certToShow = nsNSSCertificate::Create(cert);
     DisplayCertificateAlert(ctx, "UserCertImported", certToShow);
   }
   rv = NS_OK;
@@ -1341,8 +1386,7 @@ nsNSSCertificateDB::getCertNames(CERTCertList *certList,
           if (sc) *sc = DELIM;
         }
       }
-      if (!namestr) namestr = "";
-      nsAutoString certname = NS_ConvertASCIItoUTF16(namestr);
+      nsAutoString certname = NS_ConvertASCIItoUTF16(namestr ? namestr : "");
       certstr.Append(PRUnichar(DELIM));
       certstr += certname;
       certstr.Append(PRUnichar(DELIM));
@@ -1396,7 +1440,7 @@ nsNSSCertificateDB::FindEmailEncryptionCert(const nsAString &aNickname, nsIX509C
 
   if (!cert) { goto loser; }  
 
-  nssCert = new nsNSSCertificate(cert);
+  nssCert = nsNSSCertificate::Create(cert);
   if (nssCert == nsnull) {
     rv = NS_ERROR_OUT_OF_MEMORY;
   }
@@ -1436,7 +1480,7 @@ nsNSSCertificateDB::FindEmailSigningCert(const nsAString &aNickname, nsIX509Cert
 
   if (!cert) { goto loser; }  
 
-  nssCert = new nsNSSCertificate(cert);
+  nssCert = nsNSSCertificate::Create(cert);
   if (nssCert == nsnull) {
     rv = NS_ERROR_OUT_OF_MEMORY;
   }
@@ -1453,27 +1497,65 @@ NS_IMETHODIMP
 nsNSSCertificateDB::FindCertByEmailAddress(nsISupports *aToken, const char *aEmailAddress, nsIX509Cert **_retval)
 {
   nsNSSShutDownPreventionLock locker;
-  CERTCertificate *any_cert = CERT_FindCertByNicknameOrEmailAddr(CERT_GetDefaultCertDB(), (char*)aEmailAddress);
-  if (!any_cert)
-    return NS_ERROR_FAILURE;
+  
+  nsCOMPtr<nsINSSComponent> inss;
+  nsRefPtr<nsCERTValInParamWrapper> survivingParams;
+  nsresult nsrv;
+  
+  if (nsNSSComponent::globalConstFlagUsePKIXVerification) {
+    inss = do_GetService(kNSSComponentCID, &nsrv);
+    if (!inss)
+      return nsrv;
+    nsrv = inss->GetDefaultCERTValInParam(survivingParams);
+    if (NS_FAILED(nsrv))
+      return nsrv;
+  }
 
-  CERTCertificateCleaner certCleaner(any_cert);
-    
-  // any_cert now contains a cert with the right subject, but it might not have the correct usage
-  CERTCertList *certlist = CERT_CreateSubjectCertList(
-    nsnull, CERT_GetDefaultCertDB(), &any_cert->derSubject, PR_Now(), PR_TRUE);
+  CERTCertList *certlist = PK11_FindCertsFromEmailAddress(aEmailAddress, nsnull);
   if (!certlist)
     return NS_ERROR_FAILURE;  
 
+  // certlist now contains certificates with the right email address,
+  // but they might not have the correct usage or might even be invalid
+
   CERTCertListCleaner listCleaner(certlist);
 
-  if (SECSuccess != CERT_FilterCertListByUsage(certlist, certUsageEmailRecipient, PR_FALSE))
-    return NS_ERROR_FAILURE;
-  
   if (CERT_LIST_END(CERT_LIST_HEAD(certlist), certlist))
+    return NS_ERROR_FAILURE; // no certs found
+
+  CERTCertListNode *node;
+  // search for a valid certificate
+  for (node = CERT_LIST_HEAD(certlist);
+       !CERT_LIST_END(node, certlist);
+       node = CERT_LIST_NEXT(node)) {
+
+    if (!nsNSSComponent::globalConstFlagUsePKIXVerification) {
+      if (CERT_VerifyCert(CERT_GetDefaultCertDB(), node->cert,
+          PR_TRUE, certUsageEmailRecipient, PR_Now(), nsnull, nsnull) == SECSuccess) {
+        // found a valid certificate
+        break;
+      }
+    }
+    else {
+      CERTValOutParam cvout[1];
+      cvout[0].type = cert_po_end;
+      if (CERT_PKIXVerifyCert(node->cert, certificateUsageEmailRecipient,
+                              survivingParams->GetRawPointerForNSS(),
+                              cvout, nsnull)
+          == SECSuccess) {
+        // found a valid certificate
+        break;
+      }
+    }
+  }
+
+  if (CERT_LIST_END(node, certlist)) {
+    // no valid cert found
     return NS_ERROR_FAILURE;
-  
-  nsNSSCertificate *nssCert = new nsNSSCertificate(CERT_LIST_HEAD(certlist)->cert);
+  }
+
+  // node now contains the first valid certificate with correct usage 
+  nsNSSCertificate *nssCert = nsNSSCertificate::Create(node->cert);
   if (!nssCert)
     return NS_ERROR_OUT_OF_MEMORY;
 
@@ -1484,65 +1566,54 @@ nsNSSCertificateDB::FindCertByEmailAddress(nsISupports *aToken, const char *aEma
 
 /* nsIX509Cert constructX509FromBase64 (in string base64); */
 NS_IMETHODIMP
-nsNSSCertificateDB::ConstructX509FromBase64(const char * base64, nsIX509Cert **_retval)
+nsNSSCertificateDB::ConstructX509FromBase64(const char *base64,
+                                            nsIX509Cert **_retval)
 {
-  if (!_retval) {
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_ARG_POINTER(_retval);
+
+  // sure would be nice to have a smart pointer class for PL_ allocations
+  // unfortunately, we cannot distinguish out-of-memory from bad-input here
+  PRUint32 len = PL_strlen(base64);
+  char *certDER = PL_Base64Decode(base64, len, NULL);
+  if (!certDER)
+    return NS_ERROR_ILLEGAL_VALUE;
+  if (!*certDER) {
+    PL_strfree(certDER);
+    return NS_ERROR_ILLEGAL_VALUE;
+  }
+
+  // If we get to this point, we know we had well-formed base64 input;
+  // therefore the input string cannot have been less than two
+  // characters long.  Compute the unpadded length of the decoded data.
+  PRUint32 lengthDER = (len * 3) / 4;
+  if (base64[len-1] == '=') {
+    lengthDER--;
+    if (base64[len-2] == '=')
+      lengthDER--;
   }
 
   nsNSSShutDownPreventionLock locker;
-  PRUint32 len = PL_strlen(base64);
-  int adjust = 0;
+  SECItem secitem_cert;
+  secitem_cert.type = siDERCertBuffer;
+  secitem_cert.data = (unsigned char*)certDER;
+  secitem_cert.len = lengthDER;
 
-  /* Compute length adjustment */
-  if (base64[len-1] == '=') {
-    adjust++;
-    if (base64[len-2] == '=') adjust++;
-  }
+  CERTCertificate *cert =
+    CERT_NewTempCertificate(CERT_GetDefaultCertDB(), &secitem_cert,
+                            nsnull, PR_FALSE, PR_TRUE);
+  PL_strfree(certDER);
 
-  nsresult rv = NS_OK;
-  char *certDER = 0;
-  PRInt32 lengthDER = 0;
+  if (!cert)
+    return (PORT_GetError() == SEC_ERROR_NO_MEMORY)
+      ? NS_ERROR_OUT_OF_MEMORY : NS_ERROR_FAILURE;
 
-  certDER = PL_Base64Decode(base64, len, NULL);
-  if (!certDER || !*certDER) {
-    rv = NS_ERROR_ILLEGAL_VALUE;
-  }
-  else {
-    lengthDER = (len*3)/4 - adjust;
+  nsNSSCertificate *nsNSS = nsNSSCertificate::Create(cert);
+  CERT_DestroyCertificate(cert);
 
-    SECItem secitem_cert;
-    secitem_cert.type = siDERCertBuffer;
-    secitem_cert.data = (unsigned char*)certDER;
-    secitem_cert.len = lengthDER;
+  if (!nsNSS)
+    return NS_ERROR_OUT_OF_MEMORY;
 
-    CERTCertificate *cert = CERT_NewTempCertificate(CERT_GetDefaultCertDB(), &secitem_cert, nsnull, PR_FALSE, PR_TRUE);
-
-    if (!cert) {
-      rv = NS_ERROR_FAILURE;
-    }
-    else {
-      nsNSSCertificate *nsNSS = new nsNSSCertificate(cert);
-      if (!nsNSS) {
-        rv = NS_ERROR_OUT_OF_MEMORY;
-      }
-      else {
-        nsresult rv = nsNSS->QueryInterface(NS_GET_IID(nsIX509Cert), (void**)_retval);
-
-        if (NS_SUCCEEDED(rv) && *_retval) {
-          NS_ADDREF(*_retval);
-        }
-        
-        NS_RELEASE(nsNSS);
-      }
-      CERT_DestroyCertificate(cert);
-    }
-  }
-  
-  if (certDER) {
-    nsCRT::free(certDER);
-  }
-  return rv;
+  return CallQueryInterface(nsNSS, _retval);
 }
 
 void
@@ -1694,7 +1765,7 @@ NS_IMETHODIMP nsNSSCertificateDB::AddCertFromBase64(const char *aBase64, const c
   der.len = 0;
 
   if (!tmpCert) {
-    NS_ASSERTION(0,"Couldn't create cert from DER blob\n");
+    NS_ERROR("Couldn't create cert from DER blob");
     return NS_ERROR_FAILURE;
   }
 
