@@ -130,10 +130,32 @@ struct SEC_PKCS12DecoderContextStr {
     sec_PKCS12MacData		macData;
 
     /* routines for reading back the data to be hmac'd */
+    /* They are called as follows.
+     *
+     * Stage 1: decode the aSafes cinfo into a buffer in dArg,
+     * which p12d.c sometimes refers to as the "temp file".
+     * This occurs during SEC_PKCS12DecoderUpdate calls.
+     *
+     * dOpen(dArg, PR_FALSE)
+     * dWrite(dArg, buf, len)
+     * ...
+     * dWrite(dArg, buf, len)
+     * dClose(dArg, PR_FALSE)
+     *
+     * Stage 2: verify MAC
+     * This occurs SEC_PKCS12DecoderVerify.
+     *
+     * dOpen(dArg, PR_TRUE)
+     * dRead(dArg, buf, IN_BUF_LEN)
+     * ...
+     * dRead(dArg, buf, IN_BUF_LEN)
+     * dClose(dArg, PR_TRUE)
+     */
     digestOpenFn 		dOpen;
     digestCloseFn 		dClose;
     digestIOFn 			dRead, dWrite;
     void 			*dArg;
+    PRBool			dIsOpen;  /* is the temp file created? */
 
     /* helper functions */
     SECKEYGetPasswordKey 	pwfn;
@@ -915,6 +937,8 @@ sec_pkcs12_decode_start_asafes_cinfo(SEC_PKCS12DecoderContext *p12dcx)
 	p12dcx->errorValue = PORT_GetError();
 	goto loser;
     }
+    /* dOpen(dArg, PR_FALSE) creates the temp file */
+    p12dcx->dIsOpen = PR_TRUE;
 
     return SECSuccess;
 
@@ -1161,8 +1185,8 @@ p12u_DigestWrite(void *arg, unsigned char *buf, unsigned long len)
  *	slot - the slot to import the dataa into should multiple slots 
  *		 be supported based on key type and cert type?
  *	dOpen, dClose, dRead, dWrite - digest routines for writing data
- *		 to a file so it could be read back and the hmack recomputed
- *		 and verified.  doesn't seem to be away for both encoding
+ *		 to a file so it could be read back and the hmac recomputed
+ *		 and verified.  doesn't seem to be a way for both encoding
  *		 and decoding to be single pass, thus the need for these
  *		 routines.
  *	dArg - the argument for dOpen, etc.
@@ -1235,6 +1259,7 @@ SEC_PKCS12DecoderStart(SECItem *pwitem, PK11SlotInfo *slot, void *wincx,
     p12dcx->dClose = dClose;
     p12dcx->dRead = dRead;
     p12dcx->dArg = dArg;
+    p12dcx->dIsOpen = PR_FALSE;
     
     p12dcx->keyList = NULL;
     p12dcx->decitem.type = 0;
@@ -1431,6 +1456,7 @@ loser:
     /* close the file and remove it */
     if(p12dcx->dClose) {
 	(*p12dcx->dClose)(p12dcx->dArg, PR_TRUE);
+	p12dcx->dIsOpen = PR_FALSE;
     }
 
     if(pk11cx) {
@@ -1580,6 +1606,11 @@ SEC_PKCS12DecoderFinish(SEC_PKCS12DecoderContext *p12dcx)
 	p12dcx->slot = NULL;
     }
 
+    if(p12dcx->dIsOpen && p12dcx->dClose) {
+	(*p12dcx->dClose)(p12dcx->dArg, PR_TRUE);
+	p12dcx->dIsOpen = PR_FALSE;
+    }
+
     if(p12dcx->arena) {
 	PORT_FreeArena(p12dcx->arena, PR_TRUE);
     }
@@ -1876,8 +1907,7 @@ sec_pkcs12_get_key_info(sec_PKCS12SafeBag *key)
  */
 static SECItem *
 sec_pkcs12_get_nickname_for_cert(sec_PKCS12SafeBag *cert,
-				 sec_PKCS12SafeBag *key, 
-				 void *wincx)
+				 sec_PKCS12SafeBag *key)
 {
     SECItem *nickname;
 
@@ -1908,8 +1938,7 @@ sec_pkcs12_get_nickname_for_cert(sec_PKCS12SafeBag *cert,
 static SECStatus
 sec_pkcs12_set_nickname_for_cert(sec_PKCS12SafeBag *cert, 
 				 sec_PKCS12SafeBag *key, 
-				 SECItem *nickname, 
-				 void *wincx)
+				 SECItem *nickname)
 {
     if(!nickname || !cert) {
 	PORT_SetError(SEC_ERROR_INVALID_ARGS);
@@ -2041,7 +2070,7 @@ gatherNicknames(CERTCertificate *cert, void *arg)
  * If so, return it. 
  */
 static SECItem *
-sec_pkcs12_get_existing_nick_for_dn(sec_PKCS12SafeBag *cert, void *wincx)
+sec_pkcs12_get_existing_nick_for_dn(sec_PKCS12SafeBag *cert)
 {
     struct certNickInfo *nickArg = NULL;
     SECItem *derCert, *returnDn = NULL;
@@ -2160,7 +2189,7 @@ static void
 sec_pkcs12_validate_cert_nickname(sec_PKCS12SafeBag *cert,
 				sec_PKCS12SafeBag *key,
 				SEC_PKCS12NicknameCollisionCallback nicknameCb,
-				void *wincx)
+				CERTCertificate *leafCert)
 {
     SECItem *certNickname, *existingDNNick;
     PRBool setNickname = PR_FALSE, cancel = PR_FALSE;
@@ -2185,8 +2214,8 @@ sec_pkcs12_validate_cert_nickname(sec_PKCS12SafeBag *cert,
 	return;
     }
 
-    certNickname = sec_pkcs12_get_nickname_for_cert(cert, key, wincx);
-    existingDNNick = sec_pkcs12_get_existing_nick_for_dn(cert, wincx);
+    certNickname = sec_pkcs12_get_nickname_for_cert(cert, key);
+    existingDNNick = sec_pkcs12_get_existing_nick_for_dn(cert);
 
     /* nickname is already used w/ this dn, so it is safe to return */
     if(certNickname && existingDNNick &&
@@ -2198,7 +2227,7 @@ sec_pkcs12_validate_cert_nickname(sec_PKCS12SafeBag *cert,
      * this dn.  set the nicks in the p12 bags and finish.
      */
     if(existingDNNick) {
-	sec_pkcs12_set_nickname_for_cert(cert, key, existingDNNick, wincx);
+	sec_pkcs12_set_nickname_for_cert(cert, key, existingDNNick);
 	goto loser;
     }
 
@@ -2226,14 +2255,13 @@ sec_pkcs12_validate_cert_nickname(sec_PKCS12SafeBag *cert,
 	if (certNickname && certNickname->data &&
 	    !sec_pkcs12_certs_for_nickname_exist(certNickname, cert->slot)) {
 	    if (setNickname) {
-		sec_pkcs12_set_nickname_for_cert(cert, key, certNickname,
-				    wincx);
+		sec_pkcs12_set_nickname_for_cert(cert, key, certNickname);
 	    }
 	    break;
 	}
 
 	setNickname = PR_FALSE;
-	newNickname = (*nicknameCb)(certNickname, &cancel, wincx);
+	newNickname = (*nicknameCb)(certNickname, &cancel, leafCert);
 	if(cancel) {
 	    cert->problem = PR_TRUE;
 	    cert->error = SEC_ERROR_USER_CANCELLED;
@@ -2273,8 +2301,7 @@ loser:
 static void 
 sec_pkcs12_validate_cert(sec_PKCS12SafeBag *cert,
 			 sec_PKCS12SafeBag *key,
-			 SEC_PKCS12NicknameCollisionCallback nicknameCb,
-			 void *wincx)
+			 SEC_PKCS12NicknameCollisionCallback nicknameCb)
 {
     CERTCertificate *leafCert;
 
@@ -2314,7 +2341,7 @@ sec_pkcs12_validate_cert(sec_PKCS12SafeBag *cert,
 	return;
     }
 
-    sec_pkcs12_validate_cert_nickname(cert, key, nicknameCb, (void *)leafCert);
+    sec_pkcs12_validate_cert_nickname(cert, key, nicknameCb, leafCert);
 
     CERT_DestroyCertificate(leafCert);
 }
@@ -2717,7 +2744,7 @@ sec_pkcs12_validate_bags(sec_PKCS12SafeBag **safeBags,
 			cert->error   = key->error;
 			continue;
 		    } 
-		    sec_pkcs12_validate_cert(cert, key, nicknameCb, wincx);
+		    sec_pkcs12_validate_cert(cert, key, nicknameCb);
 		    if(cert->problem) {
 			key->problem = cert->problem;
 			key->error   = cert->error;
@@ -2738,7 +2765,7 @@ sec_pkcs12_validate_bags(sec_PKCS12SafeBag **safeBags,
 
 	    switch(bagType) {
 	    case SEC_OID_PKCS12_V1_CERT_BAG_ID:
-		sec_pkcs12_validate_cert(bag, NULL, nicknameCb, wincx);
+		sec_pkcs12_validate_cert(bag, NULL, nicknameCb);
 		break;
 	    case SEC_OID_PKCS12_V1_KEY_BAG_ID:
 	    case SEC_OID_PKCS12_V1_PKCS8_SHROUDED_KEY_BAG_ID:
@@ -2905,8 +2932,7 @@ sec_pkcs12_install_bags(sec_PKCS12SafeBag **safeBags, void *wincx)
 		/* use the cert's nickname, if it has one, else use the 
 		 * key's nickname, else fail.
 		 */
-		nickName = sec_pkcs12_get_nickname_for_cert(certList[0], 
-		                                            key, wincx);
+		nickName = sec_pkcs12_get_nickname_for_cert(certList[0], key);
 	    } else {
 		nickName = sec_pkcs12_get_nickname(key);
 	    }
