@@ -38,6 +38,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
+Components.utils.import("resource://gre/modules/Services.jsm");
 
 ////////////////////////////////////////////////////////////////////////////////
 //// Constants
@@ -284,7 +285,10 @@ function nsPlacesAutoComplete()
   });
 
   XPCOMUtils.defineLazyGetter(this, "_historyQuery", function() {
-    let replacementText = "AND h.visit_count > 0";
+    // Enforce ignoring the visit_count index, since the frecency one is much
+    // faster in this case.  ANALYZE helps the query planner to figure out the
+    // faster path, but it may not have run yet.
+    let replacementText = "AND +h.visit_count > 0";
     return this._db.createAsyncStatement(
       SQL_BASE.replace("{ADDITIONAL_CONDITIONS}", replacementText, "g")
     );
@@ -474,6 +478,7 @@ nsPlacesAutoComplete.prototype = {
       [this._getBoundAdaptiveQuery(), this._getBoundOpenPagesQuery(tokens), query];
 
     // Start executing our queries.
+    this._telemetryStartTime = Date.now();
     this._executeQueries(queries);
 
     // Set up our persistent state for the duration of the search.
@@ -726,6 +731,19 @@ nsPlacesAutoComplete.prototype = {
     result.setSearchResult(Ci.nsIAutoCompleteResult[resultCode]);
     result.setDefaultIndex(result.matchCount ? 0 : -1);
     this._listener.onSearchResult(this, result);
+    if (this._telemetryStartTime) {
+      let elapsed = Date.now() - this._telemetryStartTime;
+      if (elapsed > 50) {
+        try {
+          Services.telemetry
+                  .getHistogramById("PLACES_AUTOCOMPLETE_1ST_RESULT_TIME_MS")
+                  .add(elapsed);
+        } catch (ex) {
+          Components.utils.reportError("Unable to report telemetry.");
+        }
+      }
+      this._telemetryStartTime = null;
+    }
   },
 
   /**
