@@ -17,7 +17,6 @@ XPCOMUtils.defineLazyServiceGetter(this, "gGlobalHistory",
                                    "nsIGlobalHistory2");
 
 const TEST_DOMAIN = "http://mozilla.org/";
-const TOPIC_UPDATEPLACES_COMPLETE = "places-updatePlaces-complete";
 const URI_VISIT_SAVED = "uri-visit-saved";
 const RECENT_EVENT_THRESHOLD = 15 * 60 * 1000000;
 
@@ -86,13 +85,15 @@ function TitleChangedObserver(aURI,
 TitleChangedObserver.prototype = {
   __proto__: NavHistoryObserver.prototype,
   onTitleChanged: function(aURI,
-                           aTitle)
+                           aTitle,
+                           aGUID)
   {
-    do_log_info("onTitleChanged(" + aURI.spec + ", " + aTitle + ")");
+    do_log_info("onTitleChanged(" + aURI.spec + ", " + aTitle + ", " + aGUID + ")");
     if (!this.uri.equals(aURI)) {
       return;
     }
     do_check_eq(aTitle, this.expectedTitle);
+    do_check_guid_for_uri(aURI, aGUID);
     this.callback();
   },
 };
@@ -107,9 +108,11 @@ TitleChangedObserver.prototype = {
  *        being visited.
  */
 function VisitObserver(aURI,
+                       aGUID,
                        aCallback)
 {
   this.uri = aURI;
+  this.guid = aGUID;
   this.callback = aCallback;
 }
 VisitObserver.prototype = {
@@ -119,12 +122,13 @@ VisitObserver.prototype = {
                     aTime,
                     aSessionId,
                     aReferringId,
-                    aTransitionType)
+                    aTransitionType,
+                    aGUID)
   {
     do_log_info("onVisit(" + aURI.spec + ", " + aVisitId + ", " + aTime +
                 ", " + aSessionId + ", " + aReferringId + ", " +
-                aTransitionType + ")");
-    if (!this.uri.equals(aURI)) {
+                aTransitionType + ", " + aGUID + ")"); 
+    if (!this.uri.equals(aURI) || this.guid != aGUID) {
       return;
     }
     this.callback(aTime, aTransitionType);
@@ -154,6 +158,38 @@ function do_check_title_for_uri(aURI,
   stmt.finalize();
 }
 
+/**
+ * Default callback handler throws when success is unexpected.
+ *
+ * @param handleErrorFunc
+ *        The error handling function
+ */
+function expectHandleError(handleErrorFunc)
+{
+  return {
+    handleError: handleErrorFunc,
+    handleResult: function handleResult(aPlaceInfo) {
+      do_throw("Unexpected success.");
+    }
+  };
+}
+/**
+ * Default callback handler throws when failure is unexpected.
+ *
+ * @param handleResultFunc
+ *        The success handling function
+ */
+
+function expectHandleResult(handleResultFunc)
+{
+  return {
+    handleError: function handleError(aResultCode, aPlacesInfo) {
+      do_throw("Unexpected error: " + aResultCode);
+    },
+    handleResult: handleResultFunc
+  };
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 //// Test Functions
 
@@ -417,7 +453,7 @@ function test_non_addable_uri_errors()
   });
 
   let callbackCount = 0;
-  gHistory.updatePlaces(places, function(aResultCode, aPlaceInfo) {
+  gHistory.updatePlaces(places, expectHandleError(function(aResultCode, aPlaceInfo) {
     do_log_info("Checking '" + aPlaceInfo.uri.spec + "'");
     do_check_eq(aResultCode, Cr.NS_ERROR_INVALID_ARG);
     do_check_false(gGlobalHistory.isVisited(aPlaceInfo.uri));
@@ -426,7 +462,7 @@ function test_non_addable_uri_errors()
     if (++callbackCount == places.length) {
       waitForAsyncUpdates(run_next_test);
     }
-  });
+  }));
 }
 
 function test_duplicate_guid_errors()
@@ -441,8 +477,7 @@ function test_duplicate_guid_errors()
   };
 
   do_check_false(gGlobalHistory.isVisited(place.uri));
-  gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
     do_check_true(gGlobalHistory.isVisited(place.uri));
 
     let badPlace = {
@@ -454,13 +489,13 @@ function test_duplicate_guid_errors()
     };
 
     do_check_false(gGlobalHistory.isVisited(badPlace.uri));
-    gHistory.updatePlaces(badPlace, function(aResultCode, aPlaceInfo) {
+    gHistory.updatePlaces(badPlace, expectHandleError(function(aResultCode, aPlaceInfo) {
       do_check_eq(aResultCode, Cr.NS_ERROR_STORAGE_CONSTRAINT);
       do_check_false(gGlobalHistory.isVisited(badPlace.uri));
 
       waitForAsyncUpdates(run_next_test);
-    });
-  });
+    }));
+  }));
 }
 
 function test_invalid_referrerURI_ignored()
@@ -476,8 +511,7 @@ function test_invalid_referrerURI_ignored()
   do_check_false(gGlobalHistory.isVisited(place.uri));
   do_check_false(gGlobalHistory.isVisited(place.visits[0].referrerURI));
 
-  gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
     let uri = aPlaceInfo.uri;
     do_check_true(gGlobalHistory.isVisited(uri));
 
@@ -496,7 +530,7 @@ function test_invalid_referrerURI_ignored()
     stmt.finalize();
 
     waitForAsyncUpdates(run_next_test);
-  });
+  }));
 }
 
 function test_nonnsIURI_referrerURI_ignored()
@@ -511,8 +545,7 @@ function test_nonnsIURI_referrerURI_ignored()
   place.visits[0].referrerURI = place.uri.spec + "_nonnsIURI";
   do_check_false(gGlobalHistory.isVisited(place.uri));
 
-  gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
     let uri = aPlaceInfo.uri;
     do_check_true(gGlobalHistory.isVisited(uri));
 
@@ -528,7 +561,7 @@ function test_nonnsIURI_referrerURI_ignored()
     stmt.finalize();
 
     waitForAsyncUpdates(run_next_test);
-  });
+  }));
 }
 
 function test_invalid_sessionId_ignored()
@@ -543,8 +576,7 @@ function test_invalid_sessionId_ignored()
   place.visits[0].sessionId = -1;
   do_check_false(gGlobalHistory.isVisited(place.uri));
 
-  gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
     let uri = aPlaceInfo.uri;
     do_check_true(gGlobalHistory.isVisited(uri));
 
@@ -564,7 +596,7 @@ function test_invalid_sessionId_ignored()
     stmt.finalize();
 
     waitForAsyncUpdates(run_next_test);
-  });
+  }));
 }
 
 function test_unstored_sessionId_ignored()
@@ -590,8 +622,7 @@ function test_unstored_sessionId_ignored()
   place.visits[0].sessionId = maxSessionId + 10;
   do_check_false(gGlobalHistory.isVisited(place.uri));
 
-  gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
     let uri = aPlaceInfo.uri;
     do_check_true(gGlobalHistory.isVisited(uri));
 
@@ -613,7 +644,7 @@ function test_unstored_sessionId_ignored()
     stmt.finalize();
 
     waitForAsyncUpdates(run_next_test);
-  });
+  }));
 }
 
 
@@ -633,11 +664,10 @@ function test_old_referrer_ignored()
   // First we must add our referrer to the history so that it is not ignored
   // as being invalid.
   do_check_false(gGlobalHistory.isVisited(referrerPlace.uri));
-  gHistory.updatePlaces(referrerPlace, function(aResultCode, aPlaceInfo) {
+  gHistory.updatePlaces(referrerPlace, expectHandleResult(function(aPlaceInfo) {
     // Now that the referrer is added, we can add a page with a valid
     // referrer to determine if the recency of the referrer is taken into
     // account.
-    do_check_true(Components.isSuccessCode(aResultCode));
     do_check_true(gGlobalHistory.isVisited(referrerPlace.uri));
 
     let visitInfo = new VisitInfo();
@@ -650,8 +680,7 @@ function test_old_referrer_ignored()
     };
 
     do_check_false(gGlobalHistory.isVisited(place.uri));
-    gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-      do_check_true(Components.isSuccessCode(aResultCode));
+    gHistory.updatePlaces(place, expectHandleResult (function(aPlaceInfo) {
       do_check_true(gGlobalHistory.isVisited(place.uri));
 
       // Though the visit will not contain the referrer, we must examine the
@@ -669,8 +698,8 @@ function test_old_referrer_ignored()
       stmt.finalize();
 
       waitForAsyncUpdates(run_next_test);
-    });
-  });
+    }));
+  }));
 }
 
 function test_place_id_ignored()
@@ -683,8 +712,7 @@ function test_place_id_ignored()
   };
 
   do_check_false(gGlobalHistory.isVisited(place.uri));
-  gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
     do_check_true(gGlobalHistory.isVisited(place.uri));
 
     let placeId = aPlaceInfo.placeId;
@@ -699,25 +727,28 @@ function test_place_id_ignored()
     };
 
     do_check_false(gGlobalHistory.isVisited(badPlace.uri));
-    gHistory.updatePlaces(badPlace, function(aResultCode, aPlaceInfo) {
-      do_check_true(Components.isSuccessCode(aResultCode));
+    gHistory.updatePlaces(badPlace, {
+      handleResult: function handleResult(aPlaceInfo) {
+        do_check_neq(aPlaceInfo.placeId, placeId);
+        do_check_true(gGlobalHistory.isVisited(badPlace.uri));
 
-      do_check_neq(aPlaceInfo.placeId, placeId);
-      do_check_true(gGlobalHistory.isVisited(badPlace.uri));
-
-      waitForAsyncUpdates(run_next_test);
+        waitForAsyncUpdates(run_next_test);
+      },
+      handleError: function handleError(aResultCode) {
+        do_throw("Unexpected error: " + aResultCode);
+      }
     });
-  });
+  }));
 }
 
-function test_observer_topic_dispatched_when_complete()
+function test_handleCompletion_called_when_complete()
 {
   // We test a normal visit, and embeded visit, and a uri that would fail
   // the canAddURI test to make sure that the notification happens after *all*
   // of them have had a callback.
   let places = [
     { uri: NetUtil.newURI(TEST_DOMAIN +
-                          "test_observer_topic_dispatched_when_complete"),
+                          "test_handleCompletion_called_when_complete"),
       visits: [
         new VisitInfo(),
         new VisitInfo(TRANSITION_EMBED),
@@ -732,26 +763,26 @@ function test_observer_topic_dispatched_when_complete()
   do_check_false(gGlobalHistory.isVisited(places[0].uri));
   do_check_false(gGlobalHistory.isVisited(places[1].uri));
 
-  const EXPECTED_COUNT = 3;
-  let callbackCount = 0;
+  const EXPECTED_COUNT_SUCCESS = 2;
+  const EXPECTED_COUNT_FAILURE = 1;
+  let callbackCountSuccess = 0;
+  let callbackCountFailure = 0;
 
-  gHistory.updatePlaces(places, function(aResultCode, aPlaceInfo) {
-    let checker = PlacesUtils.history.canAddURI(aPlaceInfo.uri) ?
-      do_check_true : do_check_false;
-    checker(Components.isSuccessCode(aResultCode));
-    callbackCount++;
-  });
-
-  let observer = {
-    observe: function(aSubject, aTopic, aData)
-    {
-      do_check_eq(aTopic, TOPIC_UPDATEPLACES_COMPLETE);
-      do_check_eq(callbackCount, EXPECTED_COUNT);
-      Services.obs.removeObserver(observer, TOPIC_UPDATEPLACES_COMPLETE);
+  gHistory.updatePlaces(places, {
+    handleResult: function handleResult(aPlaceInfo) {
+      let checker = PlacesUtils.history.canAddURI(aPlaceInfo.uri) ?
+        do_check_true : do_check_false;
+      callbackCountSuccess++;
+    },
+    handleError: function handleError(aResultCode, aPlaceInfo) {
+      callbackCountFailure++;
+    },
+    handleCompletion: function handleCompletion() {
+      do_check_eq(callbackCountSuccess, EXPECTED_COUNT_SUCCESS);
+      do_check_eq(callbackCountFailure, EXPECTED_COUNT_FAILURE);
       waitForAsyncUpdates(run_next_test);
     },
-  };
-  Services.obs.addObserver(observer, TOPIC_UPDATEPLACES_COMPLETE, false);
+  });
 }
 
 function test_add_visit()
@@ -770,8 +801,7 @@ function test_add_visit()
   do_check_false(gGlobalHistory.isVisited(place.uri));
 
   let callbackCount = 0;
-  gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
     do_check_true(gGlobalHistory.isVisited(place.uri));
 
     // Check mozIPlaceInfo properties.
@@ -785,14 +815,14 @@ function test_add_visit()
     let visit = visits[0];
     do_check_eq(visit.visitDate, VISIT_TIME);
     do_check_true(visit.transitionType >= TRANSITION_LINK &&
-                  visit.transitionType <= TRANSITION_FRAMED_LINK);
+                    visit.transitionType <= TRANSITION_FRAMED_LINK);
     do_check_true(visit.referrerURI === null);
 
     // For TRANSITION_EMBED visits, many properties will always be zero or
     // undefined.
     if (visit.transitionType == TRANSITION_EMBED) {
       // Check mozIPlaceInfo properties.
-      do_check_eq(aPlaceInfo.placeId, 0);
+      do_check_eq(aPlaceInfo.placeId, 0, '//');
       do_check_eq(aPlaceInfo.guid, null);
 
       // Check mozIVisitInfo properties.
@@ -814,7 +844,7 @@ function test_add_visit()
     if (++callbackCount == place.visits.length) {
       waitForAsyncUpdates(run_next_test);
     }
-  });
+  }));
 }
 
 function test_properties_saved()
@@ -837,8 +867,7 @@ function test_properties_saved()
   }
 
   let callbackCount = 0;
-  gHistory.updatePlaces(places, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(places, expectHandleResult(function(aPlaceInfo) {
     let uri = aPlaceInfo.uri;
     do_check_true(gGlobalHistory.isVisited(uri));
     let visit = aPlaceInfo.visits[0];
@@ -910,7 +939,7 @@ function test_properties_saved()
     if (++callbackCount == places.length) {
       waitForAsyncUpdates(run_next_test);
     }
-  });
+  }));
 }
 
 function test_guid_saved()
@@ -925,15 +954,13 @@ function test_guid_saved()
   do_check_valid_places_guid(place.guid);
   do_check_false(gGlobalHistory.isVisited(place.uri));
 
-  gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
     let uri = aPlaceInfo.uri;
     do_check_true(gGlobalHistory.isVisited(uri));
     do_check_eq(aPlaceInfo.guid, place.guid);
     do_check_guid_for_uri(uri, place.guid);
-
     waitForAsyncUpdates(run_next_test);
-  });
+  }));
 }
 
 function test_referrer_saved()
@@ -956,8 +983,7 @@ function test_referrer_saved()
 
   let callbackCount = 0;
   let referrerSessionId;
-  gHistory.updatePlaces(places, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(places, expectHandleResult(function(aPlaceInfo) {
     let uri = aPlaceInfo.uri;
     do_check_true(gGlobalHistory.isVisited(uri));
     let visit = aPlaceInfo.visits[0];
@@ -988,7 +1014,7 @@ function test_referrer_saved()
     stmt.finalize();
 
     waitForAsyncUpdates(run_next_test);
-  });
+  }));
 }
 
 function test_sessionId_saved()
@@ -1002,8 +1028,7 @@ function test_sessionId_saved()
   place.visits[0].sessionId = 3;
   do_check_false(gGlobalHistory.isVisited(place.uri));
 
-  gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
     let uri = aPlaceInfo.uri;
     do_check_true(gGlobalHistory.isVisited(uri));
 
@@ -1023,7 +1048,7 @@ function test_sessionId_saved()
     stmt.finalize();
 
     waitForAsyncUpdates(run_next_test);
-  });
+  }));
 }
 
 function test_guid_change_saved()
@@ -1037,19 +1062,17 @@ function test_guid_change_saved()
   };
   do_check_false(gGlobalHistory.isVisited(place.uri));
 
-  gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
 
     // Then, change the guid with visits.
     place.guid = "_GUIDCHANGE_";
     place.visits = [new VisitInfo()];
-    gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-      do_check_true(Components.isSuccessCode(aResultCode));
+    gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
       do_check_guid_for_uri(place.uri, place.guid);
 
       waitForAsyncUpdates(run_next_test);
-    });
-  });
+    }));
+  }));
 }
 
 function test_title_change_saved()
@@ -1064,35 +1087,31 @@ function test_title_change_saved()
   };
   do_check_false(gGlobalHistory.isVisited(place.uri));
 
-  gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
 
     // Now, make sure the empty string clears the title.
     place.title = "";
     place.visits = [new VisitInfo()];
-    gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-      do_check_true(Components.isSuccessCode(aResultCode));
+    gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
       do_check_title_for_uri(place.uri, null);
 
       // Then, change the title with visits.
       place.title = "title change";
       place.visits = [new VisitInfo()];
-      gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-        do_check_true(Components.isSuccessCode(aResultCode));
+      gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
         do_check_title_for_uri(place.uri, place.title);
 
         // Lastly, check that the title is cleared if we set it to null.
         place.title = null;
         place.visits = [new VisitInfo()];
-        gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-          do_check_true(Components.isSuccessCode(aResultCode));
+        gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
           do_check_title_for_uri(place.uri, place.title);
 
           waitForAsyncUpdates(run_next_test);
-        });
-      });
-    });
-  });
+        }));
+      }));
+    }));
+  }));
 }
 
 function test_no_title_does_not_clear_title()
@@ -1108,19 +1127,16 @@ function test_no_title_does_not_clear_title()
   };
   do_check_false(gGlobalHistory.isVisited(place.uri));
 
-  gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
-
+  gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
     // Now, make sure that not specifying a title does not clear it.
     delete place.title;
     place.visits = [new VisitInfo()];
-    gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-      do_check_true(Components.isSuccessCode(aResultCode));
+    gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
       do_check_title_for_uri(place.uri, TITLE);
 
       waitForAsyncUpdates(run_next_test);
-    });
-  });
+    }));
+  }));
 }
 
 function test_title_change_notifies()
@@ -1173,6 +1189,7 @@ function test_visit_notifies()
   // There are two observers we need to see for each visit.  One is an
   // nsINavHistoryObserver and the other is the uri-visit-saved observer topic.
   let place = {
+    guid: "abcdefghijkl",
     uri: NetUtil.newURI(TEST_DOMAIN + "test_visit_notifies"),
     visits: [
       new VisitInfo(),
@@ -1186,8 +1203,9 @@ function test_visit_notifies()
       waitForAsyncUpdates(run_next_test);
     }
   }
-  let visitObserver = new VisitObserver(place.uri, function(aVisitDate,
-                                                            aTransitionType) {
+  let visitObserver = new VisitObserver(place.uri, place.guid,
+                                        function(aVisitDate,
+                                                 aTransitionType) {
     let visit = place.visits[0];
     do_check_eq(visit.visitDate, aVisitDate);
     do_check_eq(visit.transitionType, aTransitionType);
@@ -1222,8 +1240,7 @@ function test_referrer_sessionId_persists()
   // First we add the referrer visit, and then the main visit with referrer
   // attached. We ensure that the sessionId is maintained across the updates.
   do_check_false(gGlobalHistory.isVisited(referrerPlace.uri));
-  gHistory.updatePlaces(referrerPlace, function(aResultCode, aPlaceInfo) {
-    do_check_true(Components.isSuccessCode(aResultCode));
+  gHistory.updatePlaces(referrerPlace, expectHandleResult(function(aPlaceInfo) {
     do_check_true(gGlobalHistory.isVisited(referrerPlace.uri));
 
     let sessionId = aPlaceInfo.visits[0].sessionId;
@@ -1238,21 +1255,51 @@ function test_referrer_sessionId_persists()
     place.visits[0].referrerURI = referrerPlace.uri;
 
     do_check_false(gGlobalHistory.isVisited(place.uri));
-    gHistory.updatePlaces(place, function(aResultCode, aPlaceInfo) {
-      do_check_true(Components.isSuccessCode(aResultCode));
+    gHistory.updatePlaces(place, expectHandleResult(function(aPlaceInfo) {
       do_check_true(gGlobalHistory.isVisited(place.uri));
 
       do_check_eq(aPlaceInfo.visits[0].sessionId, sessionId);
 
       waitForAsyncUpdates(run_next_test);
-    });
+    }));
+  }));
+}
+
+// test with empty mozIVisitInfoCallback object
+function test_callbacks_not_supplied()
+{
+  const URLS = [
+    "imap://cyrus.andrew.cmu.edu/archive.imap",  // bad URI
+    "http://mozilla.org/" // valid URI
+  ];
+  let places = [];
+  URLS.forEach(function(url) {
+    try {
+      let place = {
+        uri: NetUtil.newURI(url),
+        title: "test for " + url,
+        visits: [
+          new VisitInfo(),
+        ],
+      };
+      places.push(place);
+    }
+    catch (e if e.result === Cr.NS_ERROR_FAILURE) {
+      // NetUtil.newURI() can throw if e.g. our app knows about imap://
+      // but the account is not set up and so the URL is invalid for us.
+      // Note this in the log but ignore as it's not the subject of this test.
+      do_log_info("Could not construct URI for '" + url + "'; ignoring");
+    }
   });
+  
+  gHistory.updatePlaces(places, {});
+  waitForAsyncUpdates(run_next_test);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 //// Test Runner
 
-let gTests = [
+[
   test_interface_exists,
   test_invalid_uri_throws,
   test_invalid_places_throws,
@@ -1271,7 +1318,7 @@ let gTests = [
   test_unstored_sessionId_ignored,
   test_old_referrer_ignored,
   test_place_id_ignored,
-  test_observer_topic_dispatched_when_complete,
+  test_handleCompletion_called_when_complete,
   test_add_visit,
   test_properties_saved,
   test_guid_saved,
@@ -1283,7 +1330,8 @@ let gTests = [
   test_title_change_notifies,
   test_visit_notifies,
   test_referrer_sessionId_persists,
-];
+  test_callbacks_not_supplied,
+].forEach(add_test);
 
 function run_test()
 {
