@@ -26,6 +26,7 @@
  *   Benjamin Smedberg <bsmedberg@covad.net>
  *   Daniel Brooks <db48x@db48x.net>
  *   Taras Glek <tglek@mozilla.com>
+ *   Landry Breuil <landry@openbsd.org>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -45,7 +46,6 @@
 
 #include "nsIAppShellService.h"
 #include "nsPIDOMWindow.h"
-#include "nsIDOMWindowInternal.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsILocalFile.h"
 #include "nsIObserverService.h"
@@ -55,7 +55,6 @@
 #include "nsIPromptService.h"
 #include "nsIStringBundle.h"
 #include "nsISupportsPrimitives.h"
-#include "nsITimelineService.h"
 #include "nsIWebBrowserChrome.h"
 #include "nsIWindowMediator.h"
 #include "nsIWindowWatcher.h"
@@ -90,11 +89,21 @@
 #include <sys/sysctl.h>
 #endif
 
-static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
-#ifdef MOZ_ENABLE_LIBXUL
-extern PRTime gXRE_mainTimestamp;
-extern PRTime gFirstPaintTimestamp;
+#ifdef __OpenBSD__
+#include <sys/param.h>
+#include <sys/sysctl.h>
 #endif
+
+#include "mozilla/Telemetry.h"
+
+static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
+
+using namespace mozilla;
+extern PRTime gXRE_mainTimestamp;
+// The following tracks our overhead between reaching XRE_main and loading any XUL
+extern PRTime gCreateTopLevelWindowTimestamp;// Timestamp of the first call to
+                                             // nsAppShellService::CreateTopLevelWindow
+extern PRTime gFirstPaintTimestamp;
 // mfinklesessionstore-browser-state-restored might be a better choice than the one below
 static PRTime gRestoredTimestamp = 0;       // Timestamp of sessionstore-windows-restored
 static PRTime gProcessCreationTimestamp = 0;// Timestamp of sessionstore-windows-restored
@@ -128,7 +137,8 @@ nsAppStartup::nsAppStartup() :
   mRunning(PR_FALSE),
   mShuttingDown(PR_FALSE),
   mAttemptingQuit(PR_FALSE),
-  mRestart(PR_FALSE)
+  mRestart(PR_FALSE),
+  mInterrupted(PR_FALSE)
 { }
 
 
@@ -336,7 +346,7 @@ nsAppStartup::Quit(PRUint32 aMode)
             ferocity = eAttemptQuit;
             nsCOMPtr<nsISupports> window;
             windowEnumerator->GetNext(getter_AddRefs(window));
-            nsCOMPtr<nsIDOMWindowInternal> domWindow(do_QueryInterface(window));
+            nsCOMPtr<nsIDOMWindow> domWindow = do_QueryInterface(window);
             if (domWindow) {
               PRBool closed = PR_FALSE;
               domWindow->GetClosed(&closed);
@@ -446,6 +456,20 @@ nsAppStartup::GetShuttingDown(PRBool *aResult)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsAppStartup::SetInterrupted(PRBool aInterrupted)
+{
+  mInterrupted = aInterrupted;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsAppStartup::GetInterrupted(PRBool *aInterrupted)
+{
+  *aInterrupted = mInterrupted;
+  return NS_OK;
+}
+
 //
 // nsAppStartup->nsIWindowCreator
 //
@@ -570,11 +594,11 @@ JiffiesSinceBoot(const char *file)
   char *s = strrchr(stat, ')');
   if (!s)
     return 0;
-  sscanf(s + 2,
-         "%*c %*d %*d %*d %*d %*d %*u %*u %*u %*u "
-         "%*u %*u %*u %*u %*u %*d %*d %*d %*d %llu",
-         &starttime);
-  if (!starttime)
+  int ret = sscanf(s + 2,
+                   "%*c %*d %*d %*d %*d %*d %*u %*u %*u %*u "
+                   "%*u %*u %*u %*u %*u %*d %*d %*d %*d %llu",
+                   &starttime);
+  if (ret != 1 || !starttime)
     return 0;
   return starttime;
 }
@@ -591,7 +615,13 @@ ThreadedCalculateProcessCreationTimestamp(void *aClosure)
   char thread_stat[40];
   sprintf(thread_stat, "/proc/self/task/%d/stat", (pid_t) syscall(__NR_gettid));
   
-  PRTime interval = (JiffiesSinceBoot(thread_stat) - JiffiesSinceBoot("/proc/self/stat")) * PR_USEC_PER_SEC / hz;;
+  PRUint64 thread_jiffies = JiffiesSinceBoot(thread_stat);
+  PRUint64 self_jiffies = JiffiesSinceBoot("/proc/self/stat");
+  
+  if (!thread_jiffies || !self_jiffies)
+    return;
+
+  PRTime interval = (thread_jiffies - self_jiffies) * PR_USEC_PER_SEC / hz;
   gProcessCreationTimestamp = now - interval;
 }
 
@@ -624,7 +654,7 @@ CalculateProcessCreationTimestamp()
   timestamp = (timestamp - 116444736000000000LL) / 10LL;
 #else
   timestamp = (timestamp - 116444736000000000i64) / 10i64;
-#endif    
+#endif
   return timestamp;
 }
 #elif defined(XP_MACOSX)
@@ -646,6 +676,25 @@ CalculateProcessCreationTimestamp()
   free(proc);
   return starttime;
 }
+#elif defined(__OpenBSD__)
+static PRTime
+CalculateProcessCreationTimestamp()
+{
+  int mib[6] = { CTL_KERN, KERN_PROC, KERN_PROC_PID, getpid(), sizeof(struct kinfo_proc), 1 };
+  size_t buffer_size;
+  if (sysctl(mib, 6, NULL, &buffer_size, NULL, 0))
+    return 0;
+
+  struct kinfo_proc *proc = (struct kinfo_proc*) malloc(buffer_size);
+  if (sysctl(mib, 6, proc, &buffer_size, NULL, 0)) {
+    free(proc);
+    return 0;
+  }
+  PRTime starttime = static_cast<PRTime>(proc->p_ustart_sec) * PR_USEC_PER_SEC;
+  starttime += proc->p_ustart_usec;
+  free(proc);
+  return starttime;
+}
 #else
 static PRTime
 CalculateProcessCreationTimestamp()
@@ -663,6 +712,14 @@ MaybeDefineProperty(JSContext *cx, JSObject *obj, const char *name, PRTime times
   JS_DefineProperty(cx, obj, name, OBJECT_TO_JSVAL(date), NULL, NULL, JSPROP_ENUMERATE);     
 }
 
+enum {
+  INVALID_PROCESS_CREATION = 0,
+  INVALID_MAIN,
+  INVALID_FIRST_PAINT,
+  INVALID_SESSION_RESTORED,
+  INVALID_CREATE_TOP_LEVEL_WINDOW
+};
+
 NS_IMETHODIMP
 nsAppStartup::GetStartupInfo()
 {
@@ -697,12 +754,33 @@ nsAppStartup::GetStartupInfo()
   } else if (!gProcessCreationTimestamp) {
     gProcessCreationTimestamp = CalculateProcessCreationTimestamp();
   }
+  // Bug 670008: Avoid obviously invalid process creation times
+  if (PR_Now() <= gProcessCreationTimestamp) {
+    gProcessCreationTimestamp = 0;
+    Telemetry::Accumulate(Telemetry::STARTUP_MEASUREMENT_ERRORS, INVALID_PROCESS_CREATION);
+  }
 
   MaybeDefineProperty(cx, obj, "process", gProcessCreationTimestamp);
-#ifdef MOZ_ENABLE_LIBXUL
-  MaybeDefineProperty(cx, obj, "main", gXRE_mainTimestamp);
-  MaybeDefineProperty(cx, obj, "firstPaint", gFirstPaintTimestamp);
-#endif
-  MaybeDefineProperty(cx, obj, "sessionRestored", gRestoredTimestamp);
+
+  if (gXRE_mainTimestamp >= gProcessCreationTimestamp)
+    MaybeDefineProperty(cx, obj, "main", gXRE_mainTimestamp);
+  else
+    Telemetry::Accumulate(Telemetry::STARTUP_MEASUREMENT_ERRORS, INVALID_MAIN);
+
+  if (gCreateTopLevelWindowTimestamp >= gProcessCreationTimestamp)
+    MaybeDefineProperty(cx, obj, "createTopLevelWindow", gCreateTopLevelWindowTimestamp);
+  else
+    Telemetry::Accumulate(Telemetry::STARTUP_MEASUREMENT_ERRORS, INVALID_CREATE_TOP_LEVEL_WINDOW);
+
+  if (gFirstPaintTimestamp >= gXRE_mainTimestamp)
+    MaybeDefineProperty(cx, obj, "firstPaint", gFirstPaintTimestamp);
+  else
+    Telemetry::Accumulate(Telemetry::STARTUP_MEASUREMENT_ERRORS, INVALID_FIRST_PAINT);
+
+  if (gRestoredTimestamp >= gXRE_mainTimestamp)
+    MaybeDefineProperty(cx, obj, "sessionRestored", gRestoredTimestamp);
+  else
+    Telemetry::Accumulate(Telemetry::STARTUP_MEASUREMENT_ERRORS, INVALID_SESSION_RESTORED);
+
   return NS_OK;
 }
