@@ -111,8 +111,6 @@ let gStr = {
   stateBlockedParentalControls: "stateBlocked",
   stateBlockedPolicy: "stateBlockedPolicy",
   stateDirty: "stateDirty",
-  yesterday: "yesterday",
-  monthDate: "monthDate",
   downloadsTitleFiles: "downloadsTitleFiles",
   downloadsTitlePercent: "downloadsTitlePercent",
   fileExecutableSecurityWarningTitle: "fileExecutableSecurityWarningTitle",
@@ -295,7 +293,6 @@ function openDownload(aDownload)
     } catch (e) { }
 
 #ifdef XP_WIN
-#ifndef WINCE
     // On Vista and above, we rely on native security prompting for
     // downloaded content unless it's disabled.
     try {
@@ -306,7 +303,6 @@ function openDownload(aDownload)
         dontAsk = true;
       }
     } catch (ex) { }
-#endif
 #endif
 
     if (!dontAsk) {
@@ -492,12 +488,10 @@ function Startup()
   }, false);
 
 #ifdef XP_WIN
-#ifndef WINCE
   let tempScope = {};
   Cu.import("resource://gre/modules/DownloadTaskbarProgress.jsm", tempScope);
   tempScope.DownloadTaskbarProgress.onDownloadWindowLoad(window);
 #endif
-#endif
 }
 
 function Shutdown()
@@ -1056,51 +1050,10 @@ function updateTime(aItem)
   if (aItem.inProgress)
     return;
 
-  let dts = Cc["@mozilla.org/intl/scriptabledateformat;1"].
-            getService(Ci.nsIScriptableDateFormat);
-
-  // Figure out when today begins
-  let now = new Date();
-  let today = new Date(now.getFullYear(), now.getMonth(), now.getDate());
-
-  // Get the end time to display
   let end = new Date(parseInt(aItem.getAttribute("endTime")));
-
-  // Figure out if the end time is from today, yesterday, this week, etc.
-  let dateTime;
-  if (end >= today) {
-    // Download finished after today started, show the time
-    dateTime = dts.FormatTime("", dts.timeFormatNoSeconds,
-                              end.getHours(), end.getMinutes(), 0);
-  } else if (today - end < (24 * 60 * 60 * 1000)) {
-    // Download finished after yesterday started, show yesterday
-    dateTime = gStr.yesterday;
-  } else if (today - end < (6 * 24 * 60 * 60 * 1000)) {
-    // Download finished after last week started, show day of week
-    dateTime = end.toLocaleFormat("%A");
-  } else {
-    // Download must have been from some time ago.. show month/day
-    let month = end.toLocaleFormat("%B");
-    // Remove leading 0 by converting the date string to a number
-    let date = Number(end.toLocaleFormat("%d"));
-    dateTime = replaceInsert(gStr.monthDate, 1, month);
-    dateTime = replaceInsert(dateTime, 2, date);
-  }
-
-  aItem.setAttribute("dateTime", dateTime);
-
-  // Set the tooltip to be the full date and time
-  let dateTimeTip = dts.FormatDateTime("",
-                                       dts.dateFormatLong,
-                                       dts.timeFormatNoSeconds,
-                                       end.getFullYear(),
-                                       end.getMonth() + 1,
-                                       end.getDate(),
-                                       end.getHours(),
-                                       end.getMinutes(),
-                                       0); 
-
-  aItem.setAttribute("dateTimeTip", dateTimeTip);
+  let [dateCompact, dateComplete] = DownloadUtils.getReadableDates(end);
+  aItem.setAttribute("dateTime", dateCompact);
+  aItem.setAttribute("dateTimeTip", dateComplete);
 }
 
 /**
@@ -1191,11 +1144,11 @@ function buildDownloadList(aForceBuild)
   }
 
   try {
-    gStmt.bindInt32Parameter(0, nsIDM.DOWNLOAD_NOTSTARTED);
-    gStmt.bindInt32Parameter(1, nsIDM.DOWNLOAD_DOWNLOADING);
-    gStmt.bindInt32Parameter(2, nsIDM.DOWNLOAD_PAUSED);
-    gStmt.bindInt32Parameter(3, nsIDM.DOWNLOAD_QUEUED);
-    gStmt.bindInt32Parameter(4, nsIDM.DOWNLOAD_SCANNING);
+    gStmt.bindByIndex(0, nsIDM.DOWNLOAD_NOTSTARTED);
+    gStmt.bindByIndex(1, nsIDM.DOWNLOAD_DOWNLOADING);
+    gStmt.bindByIndex(2, nsIDM.DOWNLOAD_PAUSED);
+    gStmt.bindByIndex(3, nsIDM.DOWNLOAD_QUEUED);
+    gStmt.bindByIndex(4, nsIDM.DOWNLOAD_SCANNING);
   } catch (e) {
     // Something must have gone wrong when binding, so clear and quit
     gStmt.reset();
@@ -1206,7 +1159,6 @@ function buildDownloadList(aForceBuild)
   gBuilder = setTimeout(function() {
     // Start building the list and select the first item
     stepListBuilder(1);
-    gDownloadsView.selectedIndex = 0;
 
     // We just tried to add a single item, so we probably need to enable
     updateClearListButton();
