@@ -40,6 +40,8 @@
 # ***** END LICENSE BLOCK *****
 */
 
+"use strict";
+
 const Cc = Components.classes;
 const Ci = Components.interfaces;
 const Cr = Components.results;
@@ -55,6 +57,7 @@ const KEY_APPDIR                      = "XCurProcD";
 const FILE_BLOCKLIST                  = "blocklist.xml";
 const PREF_BLOCKLIST_LASTUPDATETIME   = "app.update.lastUpdateTime.blocklist-background-update-timer";
 const PREF_BLOCKLIST_URL              = "extensions.blocklist.url";
+const PREF_BLOCKLIST_ITEM_URL         = "extensions.blocklist.itemURL";
 const PREF_BLOCKLIST_ENABLED          = "extensions.blocklist.enabled";
 const PREF_BLOCKLIST_INTERVAL         = "extensions.blocklist.interval";
 const PREF_BLOCKLIST_LEVEL            = "extensions.blocklist.level";
@@ -246,8 +249,8 @@ function getLocale() {
   try {
       // Get the default branch
       var defaultPrefs = gPref.getDefaultBranch(null);
-      return defaultPrefs.getComplexPref(PREF_GENERAL_USERAGENT_LOCALE,
-                                         Ci.nsIPrefLocalizedString).data;
+      return defaultPrefs.getComplexValue(PREF_GENERAL_USERAGENT_LOCALE,
+                                          Ci.nsIPrefLocalizedString).data;
   } catch (e) {}
 
   return gPref.getCharPref(PREF_GENERAL_USERAGENT_LOCALE);
@@ -417,6 +420,28 @@ Blocklist.prototype = {
     return Ci.nsIBlocklistService.STATE_NOT_BLOCKED;
   },
 
+  /* See nsIBlocklistService */
+  getAddonBlocklistURL: function(id, version, appVersion, toolkitVersion) {
+    if (!gBlocklistEnabled)
+      return "";
+
+    if (!this._addonEntries)
+      this._loadBlocklist();
+
+    let blItem = this._addonEntries[id];
+    if (!blItem || !blItem.blockID)
+      return null;
+
+    return this._createBlocklistURL(blItem.blockID);
+  },
+
+  _createBlocklistURL: function(id) {
+    let url = Services.urlFormatter.formatURLPref(PREF_BLOCKLIST_ITEM_URL);
+    url = url.replace(/%blockID%/g, id);
+
+    return url;
+  },
+
   notify: function(aTimer) {
     if (!gBlocklistEnabled)
       return;
@@ -511,6 +536,7 @@ Blocklist.prototype = {
       return;
     }
 
+    LOG("Blocklist::notify: Requesting " + uri.spec);
     var request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
                   createInstance(Ci.nsIXMLHttpRequest);
     request.open("GET", uri.spec, true);
@@ -609,7 +635,7 @@ Blocklist.prototype = {
 #
 #    <blocklist xmlns="http://www.mozilla.org/2006/addons-blocklist">
 #      <emItems>
-#        <emItem id="item_1@domain">
+#        <emItem id="item_1@domain" blockID="i1">
 #          <versionRange minVersion="1.0" maxVersion="2.0.*">
 #            <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
 #              <versionRange minVersion="1.5" maxVersion="1.5.*"/>
@@ -628,7 +654,7 @@ Blocklist.prototype = {
 #            </targetApplication>
 #          </versionRange>
 #        </emItem>
-#        <emItem id="item_2@domain">
+#        <emItem id="item_2@domain" blockID="i2">
 #          <versionRange minVersion="3.1" maxVersion="4.*"/>
 #        </emItem>
 #        <emItem id="item_3@domain">
@@ -638,7 +664,7 @@ Blocklist.prototype = {
 #            </targetApplication>
 #          </versionRange>
 #        </emItem>
-#        <emItem id="item_4@domain">
+#        <emItem id="item_4@domain" blockID="i3">
 #          <versionRange>
 #            <targetApplication>
 #              <versionRange minVersion="1.5" maxVersion="1.5.*"/>
@@ -647,7 +673,7 @@ Blocklist.prototype = {
 #        <emItem id="item_5@domain"/>
 #      </emItems>
 #      <pluginItems>
-#        <pluginItem>
+#        <pluginItem blockID="i4">
 #          <!-- All match tags must match a plugin to blocklist a plugin -->
 #          <match name="name" exp="some plugin"/>
 #          <match name="description" exp="1[.]2[.]3"/>
@@ -742,6 +768,8 @@ Blocklist.prototype = {
     // extension for the current application.
     if (result[id].length == 0)
       result[id].push(new BlocklistItemData(null));
+
+    result[id].blockID = blocklistElement.getAttribute("blockID");
   },
 
   _handlePluginItemNode: function(blocklistElement, result) {
@@ -751,7 +779,8 @@ Blocklist.prototype = {
     var matchNodes = blocklistElement.childNodes;
     var blockEntry = {
       matches: {},
-      versions: []
+      versions: [],
+      blockID: null,
     };
     var hasMatch = false;
     for (var x = 0; x < matchNodes.length; ++x) {
@@ -777,6 +806,9 @@ Blocklist.prototype = {
     // Add a default versionRange if there wasn't one specified
     if (blockEntry.versions.length == 0)
       blockEntry.versions.push(new BlocklistItemData(null));
+
+    blockEntry.blockID = blocklistElement.getAttribute("blockID");
+
     result.push(blockEntry);
   },
 
@@ -843,6 +875,34 @@ Blocklist.prototype = {
     return Ci.nsIBlocklistService.STATE_NOT_BLOCKED;
   },
 
+  /* See nsIBlocklistService */
+  getPluginBlocklistURL: function(plugin) {
+    if (!gBlocklistEnabled)
+      return "";
+
+    if (!this._pluginEntries)
+      this._loadBlocklist();
+
+    for each (let blockEntry in this._pluginEntries) {
+      let matchFailed = false;
+      for (let name in blockEntry.matches) {
+        if (!(name in plugin) ||
+            typeof(plugin[name]) != "string" ||
+            !blockEntry.matches[name].test(plugin[name])) {
+          matchFailed = true;
+          break;
+        }
+      }
+
+      if (!matchFailed) {
+        if(!blockEntry.blockID)
+          return null;
+        else
+          return this._createBlocklistURL(blockEntry.blockID);
+      }
+    }
+  },
+
   _blocklistUpdated: function(oldAddonEntries, oldPluginEntries) {
     var addonList = [];
 
@@ -859,19 +919,23 @@ Blocklist.prototype = {
         LOG("Blocklist state for " + addons[i].id + " changed from " +
             oldState + " to " + state);
 
-        // Don't warn about add-ons becoming unblocked.
-        if (state == 0)
-          continue;
-
         // We don't want to re-warn about add-ons
         if (state == oldState)
           continue;
 
+        // Ensure that softDisabled is false if the add-on is not soft blocked
+        if (state != Ci.nsIBlocklistService.STATE_SOFTBLOCKED)
+          addons[i].softDisabled = false;
+
+        // Don't warn about add-ons becoming unblocked.
+        if (state == Ci.nsIBlocklistService.STATE_NOT_BLOCKED)
+          continue;
+
         // If an add-on has dropped from hard to soft blocked just mark it as
-        // user disabled and don't warn about it.
+        // soft disabled and don't warn about it.
         if (state == Ci.nsIBlocklistService.STATE_SOFTBLOCKED &&
             oldState == Ci.nsIBlocklistService.STATE_BLOCKED) {
-          addons[i].userDisabled = true;
+          addons[i].softDisabled = true;
           continue;
         }
 
@@ -886,7 +950,8 @@ Blocklist.prototype = {
           icon: addons[i].iconURL,
           disable: false,
           blocked: state == Ci.nsIBlocklistService.STATE_BLOCKED,
-          item: addons[i]
+          item: addons[i],
+          url: self.getAddonBlocklistURL(addons[i].id),
         });
       }
 
@@ -922,15 +987,18 @@ Blocklist.prototype = {
               icon: "chrome://mozapps/skin/plugins/pluginGeneric.png",
               disable: false,
               blocked: state == Ci.nsIBlocklistService.STATE_BLOCKED,
-              item: plugins[i]
+              item: plugins[i],
+              url: self.getPluginBlocklistURL(plugins[i]),
             });
           }
         }
         plugins[i].blocklisted = state == Ci.nsIBlocklistService.STATE_BLOCKED;
       }
 
-      if (addonList.length == 0)
+      if (addonList.length == 0) {
+        Services.obs.notifyObservers(self, "blocklist-updated", "");
         return;
+      }
 
       if ("@mozilla.org/addons/blocklist-prompt;1" in Cc) {
         try {
@@ -940,6 +1008,7 @@ Blocklist.prototype = {
         } catch (e) {
           LOG(e);
         }
+        Services.obs.notifyObservers(self, "blocklist-updated", "");
         return;
       }
 
@@ -950,23 +1019,40 @@ Blocklist.prototype = {
       // This lets the dialog get the raw js object
       args.wrappedJSObject = args;
 
-      var ww = Cc["@mozilla.org/embedcomp/window-watcher;1"].
-               getService(Ci.nsIWindowWatcher);
-      ww.openWindow(null, URI_BLOCKLIST_DIALOG, "",
-                    "chrome,centerscreen,dialog,modal,titlebar", args);
+      /*
+        Some tests run without UI, so the async code listens to a message
+        that can be sent programatically
+      */
+      let applyBlocklistChanges = function() {
+        for (let i = 0; i < addonList.length; i++) {
+          if (!addonList[i].disable)
+            continue;
 
-      for (let i = 0; i < addonList.length; i++) {
-        if (!addonList[i].disable)
-          continue;
+          if (addonList[i].item instanceof Ci.nsIPluginTag)
+            addonList[i].item.disabled = true;
+          else
+            addonList[i].item.softDisabled = true;
+        }
 
-        if (addonList[i].item instanceof Ci.nsIPluginTag)
-          addonList[i].item.disabled = true;
-        else
-          addonList[i].item.userDisabled = true;
+        if (args.restart)
+          restartApp();
+
+        Services.obs.notifyObservers(self, "blocklist-updated", "");
+        Services.obs.removeObserver(applyBlocklistChanges, "addon-blocklist-closed");
       }
 
-      if (args.restart)
-        restartApp();
+      Services.obs.addObserver(applyBlocklistChanges, "addon-blocklist-closed", false)
+
+      function blocklistUnloadHandler(event) {
+        if (event.target.location == URI_BLOCKLIST_DIALOG) {
+          applyBlocklistChanges();
+          blocklistWindow.removeEventListener("unload", blocklistUnloadHandler);
+        }
+      }
+
+      let blocklistWindow = Services.ww.openWindow(null, URI_BLOCKLIST_DIALOG, "",
+                              "chrome,centerscreen,dialog,titlebar", args);
+      blocklistWindow.addEventListener("unload", blocklistUnloadHandler, false);
     });
   },
 
