@@ -36,6 +36,8 @@
  * ***** END LICENSE BLOCK *****
  */
 
+Components.utils.import("resource://gre/modules/ctypes.jsm")
+
 /* General URL Construction Tests */
 
 const URL_PREFIX = URL_HOST + URL_PATH + "/";
@@ -141,6 +143,11 @@ function check_test_pt4() {
 
     if (macutils.isUniversalBinary)
       abi += "-u-" + macutils.architecturesInBinary;
+    if (IS_SHARK) {
+      // Disambiguate optimised and shark nightlies
+      abi += "-shark"
+    }
+
   }
 
   do_check_eq(getResult(gRequestURL), gAppInfo.OS + "_" + abi);
@@ -223,12 +230,132 @@ function run_test_pt9() {
   gUpdateChecker.checkForUpdates(updateCheckListener, true);
 }
 
+function getServicePack() {
+  // NOTE: This function is a helper function and not a test.  Thus,
+  // it uses throw() instead of do_throw().  Any tests that use this function
+  // should catch exceptions thrown in this function and deal with them
+  // appropriately (usually by calling do_throw).
+  const BYTE = ctypes.uint8_t;
+  const WORD = ctypes.uint16_t;
+  const DWORD = ctypes.uint32_t;
+  const WCHAR = ctypes.jschar;
+  const BOOL = ctypes.int;
+
+  // This structure is described at:
+  // http://msdn.microsoft.com/en-us/library/ms724833%28v=vs.85%29.aspx
+  const SZCSDVERSIONLENGTH = 128;
+  const OSVERSIONINFOEXW = new ctypes.StructType('OSVERSIONINFOEXW',
+      [
+      {dwOSVersionInfoSize: DWORD},
+      {dwMajorVersion: DWORD},
+      {dwMinorVersion: DWORD},
+      {dwBuildNumber: DWORD},
+      {dwPlatformId: DWORD},
+      {szCSDVersion: ctypes.ArrayType(WCHAR, SZCSDVERSIONLENGTH)},
+      {wServicePackMajor: WORD},
+      {wServicePackMinor: WORD},
+      {wSuiteMask: WORD},
+      {wProductType: BYTE},
+      {wReserved: BYTE}
+      ]);
+
+  let kernel32 = ctypes.open("kernel32");
+  try {
+    let GetVersionEx = kernel32.declare("GetVersionExW",
+                                        ctypes.default_abi,
+                                        BOOL,
+                                        OSVERSIONINFOEXW.ptr);
+    let winVer = OSVERSIONINFOEXW();
+    winVer.dwOSVersionInfoSize = OSVERSIONINFOEXW.size;
+
+    if(0 === GetVersionEx(winVer.address())) {
+      // Using "throw" instead of "do_throw" (see NOTE above)
+      throw("Failure in GetVersionEx (returned 0)");
+    }
+
+    return winVer.wServicePackMajor + "." + winVer.wServicePackMinor;
+  } finally {
+    kernel32.close();
+  }
+}
+
+function getProcArchitecture() {
+  // NOTE: This function is a helper function and not a test.  Thus,
+  // it uses throw() instead of do_throw().  Any tests that use this function
+  // should catch exceptions thrown in this function and deal with them
+  // appropriately (usually by calling do_throw).
+  const WORD = ctypes.uint16_t;
+  const DWORD = ctypes.uint32_t;
+
+  // This structure is described at:
+  // http://msdn.microsoft.com/en-us/library/ms724958%28v=vs.85%29.aspx
+  const SYSTEM_INFO = new ctypes.StructType('SYSTEM_INFO',
+      [
+      {wProcessorArchitecture: WORD},
+      {wReserved: WORD},
+      {dwPageSize: DWORD},
+      {lpMinimumApplicationAddress: ctypes.voidptr_t},
+      {lpMaximumApplicationAddress: ctypes.voidptr_t},
+      {dwActiveProcessorMask: DWORD.ptr},
+      {dwNumberOfProcessors: DWORD},
+      {dwProcessorType: DWORD},
+      {dwAllocationGranularity: DWORD},
+      {wProcessorLevel: WORD},
+      {wProcessorRevision: WORD}
+      ]);
+
+  let kernel32 = ctypes.open("kernel32");
+  try {
+    let GetNativeSystemInfo = kernel32.declare("GetNativeSystemInfo",
+                                               ctypes.default_abi,
+                                               ctypes.void_t,
+                                               SYSTEM_INFO.ptr);
+    let sysInfo = SYSTEM_INFO();
+    // Default to unknown
+    sysInfo.wProcessorArchitecture = 0xffff;
+
+    GetNativeSystemInfo(sysInfo.address());
+    switch(sysInfo.wProcessorArchitecture) {
+      case 9:
+        return "x64";
+      case 6:
+        return "IA64";
+      case 0:
+        return "x86";
+      default:
+        // Using "throw" instead of "do_throw" (see NOTE above)
+        throw("Unknown architecture returned from GetNativeSystemInfo: " + sysInfo.wProcessorArchitecture);
+    }
+  } finally {
+    kernel32.close();
+  }
+}
+
 function check_test_pt9() {
   var osVersion;
   var sysInfo = AUS_Cc["@mozilla.org/system-info;1"].
                 getService(AUS_Ci.nsIPropertyBag2);
   osVersion = sysInfo.getProperty("name") + " " + sysInfo.getProperty("version");
 
+  if(IS_WIN) {
+    try {
+      let servicePack = getServicePack();
+      osVersion += "." + servicePack;
+    } catch (e) {
+      do_throw("Failure obtaining service pack: " + e);
+    }
+
+    if("5.0" === sysInfo.getProperty("version")) { // Win2K
+      osVersion += " (unknown)";
+    } else {
+      try {
+        osVersion += " (" + getProcArchitecture() + ")";
+      } catch (e) {
+        do_throw("Failed to obtain processor architecture: " + e);
+      }
+    }
+  }
+
   if (osVersion) {
     try {
       osVersion += " (" + sysInfo.getProperty("secondaryLibrary") + ")";
@@ -274,11 +401,12 @@ function check_test_pt11() {
   run_test_pt12();
 }
 
-// url constructed that doesn't have a parameter - bug 454357
+// url with force param that doesn't already have a param - bug 454357
 function run_test_pt12() {
   gCheckFunc = check_test_pt12;
   var url = URL_PREFIX;
-  logTestInfo("testing url constructed that doesn't have a parameter - " + url);
+  logTestInfo("testing url with force param that doesn't already have a " +
+              "param - " + url);
   setUpdateURLOverride(url);
   gUpdateChecker.checkForUpdates(updateCheckListener, true);
 }
@@ -288,10 +416,11 @@ function check_test_pt12() {
   run_test_pt13();
 }
 
-// url constructed that has a parameter - bug 454357
+// url with force param that already has a param - bug 454357
 function run_test_pt13() {
   gCheckFunc = check_test_pt13;
   var url = URL_PREFIX + "?extra=param";
+  logTestInfo("testing url with force param that already has a param - " + url);
   logTestInfo("testing url constructed that has a parameter - " + url);
   setUpdateURLOverride(url);
   gUpdateChecker.checkForUpdates(updateCheckListener, true);
@@ -299,5 +428,52 @@ function run_test_pt13() {
 
 function check_test_pt13() {
   do_check_eq(getResult(gRequestURL), "?extra=param&force=1");
+  run_test_pt14();
+}
+
+// url with newchannel param that doesn't already have a param
+function run_test_pt14() {
+  gCheckFunc = check_test_pt14;
+  Services.prefs.setCharPref(PREF_APP_UPDATE_DESIREDCHANNEL, "testchannel");
+  var url = URL_PREFIX;
+  logTestInfo("testing url with newchannel param that doesn't already have a " +
+              "param - " + url);
+  setUpdateURLOverride(url);
+  gUpdateChecker.checkForUpdates(updateCheckListener, false);
+}
+
+function check_test_pt14() {
+  do_check_eq(getResult(gRequestURL), "?newchannel=testchannel");
+  run_test_pt15();
+}
+
+// url with newchannel param that already has a param
+function run_test_pt15() {
+  gCheckFunc = check_test_pt15;
+  Services.prefs.setCharPref(PREF_APP_UPDATE_DESIREDCHANNEL, "testchannel");
+  var url = URL_PREFIX + "?extra=param";
+  logTestInfo("testing url with newchannel param that already has a " +
+              "param - " + url);
+  setUpdateURLOverride(url);
+  gUpdateChecker.checkForUpdates(updateCheckListener, false);
+}
+
+function check_test_pt15() {
+  do_check_eq(getResult(gRequestURL), "?extra=param&newchannel=testchannel");
+  run_test_pt16();
+}
+
+// url with force and newchannel params
+function run_test_pt16() {
+  gCheckFunc = check_test_pt16;
+  Services.prefs.setCharPref(PREF_APP_UPDATE_DESIREDCHANNEL, "testchannel");
+  var url = URL_PREFIX;
+  logTestInfo("testing url with force and newchannel params - " + url);
+  setUpdateURLOverride(url);
+  gUpdateChecker.checkForUpdates(updateCheckListener, true);
+}
+
+function check_test_pt16() {
+  do_check_eq(getResult(gRequestURL), "?newchannel=testchannel&force=1");
   do_test_finished();
 }
