@@ -372,9 +372,6 @@ nsXREDirProvider::GetFile(const char* aProperty, PRBool* aPersistent,
         ensureFilePermissions = PR_TRUE;
       }
     }
-    else if (!strcmp(aProperty, NS_APP_HISTORY_50_FILE)) {
-      rv = file->AppendNative(NS_LITERAL_CSTRING("history.dat"));
-    }
     else if (!strcmp(aProperty, NS_APP_USER_MIMETYPES_50_FILE)) {
       rv = file->AppendNative(NS_LITERAL_CSTRING("mimeTypes.rdf"));
       EnsureProfileFileExists(file);
@@ -454,9 +451,6 @@ LoadDirsIntoArray(nsCOMArray<nsIFile>& aSourceDirs,
   }
 }
 
-static const char *const kAppendChromeManifests[] =
-  { "chrome.manifest", nsnull };
-
 NS_IMETHODIMP
 nsXREDirProvider::GetFiles(const char* aProperty, nsISimpleEnumerator** aResult)
 {
