@@ -42,7 +42,7 @@
 // formal protocols
 #include "mozView.h"
 #ifdef ACCESSIBILITY
-#include "nsIAccessible.h"
+#include "nsAccessible.h"
 #include "mozAccessibleProtocol.h"
 #endif
 
@@ -51,25 +51,79 @@
 #include "nsBaseWidget.h"
 #include "nsIPluginInstanceOwner.h"
 #include "nsIPluginWidget.h"
-#include "nsIScrollableView.h"
 #include "nsWeakPtr.h"
+#include "TextInputHandler.h"
+#include "nsCocoaUtils.h"
 
 #include "nsIAppShell.h"
 
-#include "nsIEventListener.h"
 #include "nsString.h"
 #include "nsIDragService.h"
 
-#include "nsplugindefs.h"
+#include "npapi.h"
 
 #import <Carbon/Carbon.h>
 #import <Cocoa/Cocoa.h>
+#import <AppKit/NSOpenGL.h>
+
+// The header files QuickdrawAPI.h and QDOffscreen.h are missing on OS X 10.7
+// and up (though the QuickDraw APIs defined in them are still present) -- so
+// we need to supply the relevant parts of their contents here.  It's likely
+// that Apple will eventually remove the APIs themselves (probably in OS X
+// 10.8), so we need to make them weak imports, and test for their presence
+// before using them.
+#ifdef __cplusplus
+extern "C" {
+#endif
+  #if !defined(__QUICKDRAWAPI__)
+
+  extern void SetPort(GrafPtr port)
+    __attribute__((weak_import));
+  extern void SetOrigin(short h, short v)
+    __attribute__((weak_import));
+  extern RgnHandle NewRgn(void)
+    __attribute__((weak_import));
+  extern void DisposeRgn(RgnHandle rgn)
+    __attribute__((weak_import));
+  extern void RectRgn(RgnHandle rgn, const Rect * r)
+    __attribute__((weak_import));
+  extern GDHandle GetMainDevice(void)
+    __attribute__((weak_import));
+  extern Boolean IsPortOffscreen(CGrafPtr port)
+    __attribute__((weak_import));
+  extern void SetPortVisibleRegion(CGrafPtr port, RgnHandle visRgn)
+    __attribute__((weak_import));
+  extern void SetPortClipRegion(CGrafPtr port, RgnHandle clipRgn)
+    __attribute__((weak_import));
+  extern CGrafPtr GetQDGlobalsThePort(void)
+    __attribute__((weak_import));
+
+  #endif /* __QUICKDRAWAPI__ */
+
+  #if !defined(__QDOFFSCREEN__)
+
+  extern void GetGWorld(CGrafPtr *  port, GDHandle *  gdh)
+    __attribute__((weak_import));
+  extern void SetGWorld(CGrafPtr port, GDHandle gdh)
+    __attribute__((weak_import));
+
+  #endif /* __QDOFFSCREEN__ */
+#ifdef __cplusplus
+}
+#endif
 
 class gfxASurface;
 class nsChildView;
+class nsCocoaWindow;
 union nsPluginPort;
-class nsITimer;
 
+namespace mozilla {
+namespace gl {
+class TextureImage;
+}
+}
+
+#ifndef NP_NO_CARBON
 enum {
   // Currently focused ChildView (while this TSM document is active).
   // Transient (only set while TSMProcessRawKeyEvent() is processing a key
@@ -90,6 +144,7 @@ enum {
 // (PluginKeyEventsHandler()) to catch these events and pass them to Gecko
 // (which in turn passes them to the plugin).
 extern "C" long TSMProcessRawKeyEvent(EventRef carbonEvent);
+#endif // NP_NO_CARBON
 
 @interface NSEvent (Undocumented)
 
@@ -100,22 +155,72 @@ extern "C" long TSMProcessRawKeyEvent(EventRef carbonEvent);
 
 @end
 
-// Needed to support pixel scrolling.
-// See http://developer.apple.com/qa/qa2005/qa1453.html.
-// kEventMouseScroll is only defined on 10.5+. Using the moz prefix avoids
-// potential symbol conflicts.
-// This should be changed when 10.4 support is dropped.
-enum {
-  mozkEventMouseScroll             = 11
-};
-
 // Support for pixel scroll deltas, not part of NSEvent.h
 // See http://lists.apple.com/archives/cocoa-dev/2007/Feb/msg00050.html
 @interface NSEvent (DeviceDelta)
-  - (float)deviceDeltaX;
-  - (float)deviceDeltaY;
+  - (CGFloat)deviceDeltaX;
+  - (CGFloat)deviceDeltaY;
 @end
 
+// Undocumented scrollPhase flag that lets us discern between real scrolls and
+// automatically firing momentum scroll events.
+@interface NSEvent (ScrollPhase)
+- (long long)_scrollPhase;
+@end
+
+// The following section, required to support fluid swipe tracking on OS X 10.7
+// and up, contains defines/declarations that are only available on 10.7 and up.
+// [NSEvent trackSwipeEventWithOptions:...] also requires that the compiler
+// support "blocks"
+// (http://developer.apple.com/library/mac/#documentation/Cocoa/Conceptual/Blocks/Articles/00_Introduction.html)
+// -- which it does on 10.6 and up (using the 10.6 SDK or higher).
+//
+// MAC_OS_X_VERSION_MAX_ALLOWED "controls which OS functionality, if used,
+// will result in a compiler error because that functionality is not
+// available" (quoting from AvailabilityMacros.h).  The compiler initializes
+// it to the version of the SDK being used.  Its value does *not* prevent the
+// binary from running on higher OS versions.  MAC_OS_X_VERSION_10_7 and
+// friends are defined (in AvailabilityMacros.h) as decimal numbers (not
+// hexadecimal numbers).
+#if !defined(MAC_OS_X_VERSION_10_7) || MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_7
+#ifdef __LP64__
+enum {
+  NSEventPhaseNone        = 0,
+  NSEventPhaseBegan       = 0x1 << 0,
+  NSEventPhaseStationary  = 0x1 << 1,
+  NSEventPhaseChanged     = 0x1 << 2,
+  NSEventPhaseEnded       = 0x1 << 3,
+  NSEventPhaseCancelled   = 0x1 << 4,
+};
+typedef NSUInteger NSEventPhase;
+
+enum {
+  NSEventSwipeTrackingLockDirection = 0x1 << 0,
+  NSEventSwipeTrackingClampGestureAmount = 0x1 << 1
+};
+typedef NSUInteger NSEventSwipeTrackingOptions;
+
+enum {
+  NSEventGestureAxisNone = 0,
+  NSEventGestureAxisHorizontal,
+  NSEventGestureAxisVertical
+};
+typedef NSInteger NSEventGestureAxis;
+
+@interface NSEvent (FluidSwipeTracking)
++ (BOOL)isSwipeTrackingFromScrollEventsEnabled;
+- (BOOL)hasPreciseScrollingDeltas;
+- (CGFloat)scrollingDeltaX;
+- (CGFloat)scrollingDeltaY;
+- (NSEventPhase)phase;
+- (void)trackSwipeEventWithOptions:(NSEventSwipeTrackingOptions)options
+          dampenAmountThresholdMin:(CGFloat)minDampenThreshold
+                               max:(CGFloat)maxDampenThreshold
+                      usingHandler:(void (^)(CGFloat gestureAmount, NSEventPhase phase, BOOL isComplete, BOOL *stop))trackingHandler;
+@end
+#endif // #ifdef __LP64__
+#endif // #if !defined(MAC_OS_X_VERSION_10_7) || MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_7
+
 @interface ChildView : NSView<
 #ifdef ACCESSIBILITY
                               mozAccessible,
@@ -123,44 +228,36 @@ enum {
                               mozView, NSTextInput>
 {
 @private
-  NSWindow* mWindow; // shortcut to the top window, [WEAK]
-  
   // the nsChildView that created the view. It retains this NSView, so
   // the link back to it must be weak.
   nsChildView* mGeckoChild;
-    
-  // Whether we're a plugin view.
+
+  // Text input handler for mGeckoChild and us.  Note that this is a weak
+  // reference.  Ideally, this should be a strong reference but a ChildView
+  // object can live longer than the mGeckoChild that owns it.  And if
+  // mTextInputHandler were a strong reference, this would make it difficult
+  // for Gecko's leak detector to detect leaked TextInputHandler objects.
+  // This is initialized by [mozView installTextInputHandler:aHandler] and
+  // cleared by [mozView uninstallTextInputHandler].
+  mozilla::widget::TextInputHandler* mTextInputHandler;  // [WEAK]
+
   BOOL mIsPluginView;
+  NPEventModel mPluginEventModel;
+  NPDrawingModel mPluginDrawingModel;
 
-  // The following variables are only valid during key down event processing.
-  // Their current usage needs to be fixed to avoid problems with nested event
-  // loops that can confuse them. Once a variable is set during key down event
-  // processing, if an event spawns a nested event loop the previously set value
-  // will be wiped out.
-  NSEvent* mCurKeyEvent;
-  PRBool mKeyDownHandled;
-  // While we process key down events we need to keep track of whether or not
-  // we sent a key press event. This helps us make sure we do send one
-  // eventually.
-  BOOL mKeyPressSent;
-  // Valid when mKeyPressSent is true.
-  PRBool mKeyPressHandled;
-
-  // needed for NSTextInput implementation
-  NSRange mMarkedRange;
-  
   // when mouseDown: is called, we store its event here (strong)
   NSEvent* mLastMouseDownEvent;
-  
+
+  // Whether the last mouse down event was blocked from Gecko.
+  BOOL mBlockedLastMouseDown;
+
+  // when acceptsFirstMouse: is called, we store the event here (strong)
+  NSEvent* mClickThroughMouseDownEvent;
+
   // rects that were invalidated during a draw, so have pending drawing
   NSMutableArray* mPendingDirtyRects;
   BOOL mPendingFullDisplay;
-
-  // All views are always opaque (non-transparent). The only exception is when we're
-  // the content view in a transparent XUL window.
-  BOOL mIsTransparent;
-  PRIntervalTime mLastShadowInvalidation;
-  BOOL mNeedsShadowInvalidation;
+  BOOL mPendingDisplay;
 
   // Holds our drag service across multiple drag calls. The reference to the
   // service is obtained when the mouse enters the view and is released when
@@ -168,11 +265,8 @@ enum {
   // re-establish the connection to the service manager many times per second
   // when handling |draggingUpdated:| messages.
   nsIDragService* mDragService;
-  
-  // For use with plugins, so that we can support IME in them.  We can't use
-  // Cocoa TSM documents (those created and managed by the NSTSMInputContext
-  // class) -- for some reason TSMProcessRawKeyEvent() doesn't work with them.
-  TSMDocumentID mPluginTSMDoc;
+
+  NSOpenGLContext *mGLContext;
 
   // Simple gestures support
   //
@@ -196,6 +290,13 @@ enum {
   } mGestureState;
   float mCumulativeMagnification;
   float mCumulativeRotation;
+
+  BOOL mDidForceRefreshOpenGL;
+
+  // Support for fluid swipe tracking.
+#ifdef __LP64__
+  BOOL *mSwipeAnimationCancelled;
+#endif
 }
 
 // class initialization
@@ -208,11 +309,25 @@ enum {
 // Stop NSView hierarchy being changed during [ChildView drawRect:]
 - (void)delayedTearDown;
 
-- (void)setTransparent:(BOOL)transparent;
-
 - (void)sendFocusEvent:(PRUint32)eventType;
 
-- (void) processPluginKeyEvent:(EventRef)aKeyEvent;
+- (void)handleMouseMoved:(NSEvent*)aEvent;
+
+- (void)drawRect:(NSRect)aRect inTitlebarContext:(CGContextRef)aContext;
+
+- (void)sendMouseEnterOrExitEvent:(NSEvent*)aEvent
+                            enter:(BOOL)aEnter
+                             type:(nsMouseEvent::exitType)aType;
+
+- (void)update;
+- (void)lockFocus;
+- (void) _surfaceNeedsUpdate:(NSNotification*)notification;
+
+- (BOOL)isPluginView;
+
+// Are we processing an NSLeftMouseDown event that will fail to click through?
+// If so, we shouldn't focus or unfocus a plugin.
+- (BOOL)isInFailingLeftClickThrough;
 
 // Simple gestures support
 //
@@ -229,55 +344,34 @@ enum {
 - (void)magnifyWithEvent:(NSEvent *)anEvent;
 - (void)rotateWithEvent:(NSEvent *)anEvent;
 - (void)endGestureWithEvent:(NSEvent *)anEvent;
+
+// Support for fluid swipe tracking.
+#ifdef __LP64__
+- (void)maybeTrackScrollEventAsSwipe:(NSEvent *)anEvent
+                      scrollOverflow:(PRInt32)overflow;
+#endif
 @end
 
+class ChildViewMouseTracker {
 
-
-//-------------------------------------------------------------------------
-//
-// nsTSMManager
-//
-//-------------------------------------------------------------------------
-
-class nsTSMManager {
 public:
-  static PRBool IsComposing() { return sComposingView ? PR_TRUE : PR_FALSE; }
-  static PRBool IsIMEEnabled() { return sIsIMEEnabled; }
-  static PRBool IgnoreCommit() { return sIgnoreCommit; }
 
-  static void OnDestroyView(NSView<mozView>* aDestroyingView);
+  static void MouseMoved(NSEvent* aEvent);
+  static void MouseScrolled(NSEvent* aEvent);
+  static void OnDestroyView(ChildView* aView);
+  static void OnDestroyWindow(NSWindow* aWindow);
+  static BOOL WindowAcceptsEvent(NSWindow* aWindow, NSEvent* aEvent,
+                                 ChildView* aView, BOOL isClickThrough = NO);
+  static void MouseExitedWindow(NSEvent* aEvent);
+  static void MouseEnteredWindow(NSEvent* aEvent);
+  static void ReEvaluateMouseEnterState(NSEvent* aEvent = nil);
+  static void ResendLastMouseMoveEvent();
+  static ChildView* ViewForEvent(NSEvent* aEvent);
 
-  // Note that we cannot get the actual state in TSM. But we can trust this
-  // value. Because nsIMEStateManager reset this at every focus changing.
-  // XXX If plug-ins changed that, we cannot return correct state.
-  static PRBool IsRomanKeyboardsOnly() { return sIsRomanKeyboardsOnly; }
-
-  static PRBool GetIMEOpenState();
-
-  static void InitTSMDocument(NSView<mozView>* aViewForCaret);
-  static void StartComposing(NSView<mozView>* aComposingView);
-  static void UpdateComposing(NSString* aComposingString);
-  static void EndComposing();
-  static void EnableIME(PRBool aEnable);
-  static void SetIMEOpenState(PRBool aOpen);
-  static void SetRomanKeyboardsOnly(PRBool aRomanOnly);
-
-  static void CommitIME();
-  static void CancelIME();
-
-  static void Shutdown();
-private:
-  static PRBool sIsIMEEnabled;
-  static PRBool sIsRomanKeyboardsOnly;
-  static PRBool sIgnoreCommit;
-  static NSView<mozView>* sComposingView;
-  static TSMDocumentID sDocumentID;
-  static NSString* sComposingString;
-  static nsITimer* sSyncKeyScriptTimer;
-
-  static void KillComposing();
-  static void CallKeyScriptAPI();
-  static void SyncKeyScript(nsITimer* aTimer, void* aClosure);
+  static ChildView* sLastMouseEventView;
+  static NSEvent* sLastMouseMoveEvent;
+  static NSWindow* sWindowUnderMouse;
+  static NSPoint sLastScrollEventScreenLocation;
 };
 
 //-------------------------------------------------------------------------
@@ -303,7 +397,7 @@ public:
                                  nsNativeWidget aNativeParent,
                                  const nsIntRect &aRect,
                                  EVENT_CALLBACK aHandleEventFunction,
-                                 nsIDeviceContext *aContext,
+                                 nsDeviceContext *aContext,
                                  nsIAppShell *aAppShell = nsnull,
                                  nsIToolkit *aToolkit = nsnull,
                                  nsWidgetInitData *aInitData = nsnull);
@@ -315,6 +409,7 @@ public:
 
   NS_IMETHOD              SetParent(nsIWidget* aNewParent);
   virtual nsIWidget*      GetParent(void);
+  virtual float           GetDPI();
 
   NS_IMETHOD              ConstrainPosition(PRBool aAllowSlop,
                                             PRInt32 *aX, PRInt32 *aY);
@@ -325,18 +420,12 @@ public:
   NS_IMETHOD              Enable(PRBool aState);
   NS_IMETHOD              IsEnabled(PRBool *aState);
   NS_IMETHOD              SetFocus(PRBool aRaise);
-  NS_IMETHOD              SetBounds(const nsIntRect &aRect);
   NS_IMETHOD              GetBounds(nsIntRect &aRect);
 
-  NS_IMETHOD              Invalidate(PRBool aIsSynchronous);
   NS_IMETHOD              Invalidate(const nsIntRect &aRect, PRBool aIsSynchronous);
-  NS_IMETHOD              Validate();
 
   virtual void*           GetNativeData(PRUint32 aDataType);
   virtual nsresult        ConfigureChildren(const nsTArray<Configuration>& aConfigurations);
-  virtual void            Scroll(const nsIntPoint& aDelta,
-                                 const nsTArray<nsIntRect>& aDestRects,
-                                 const nsTArray<Configuration>& aConfigurations);
   virtual nsIntPoint      WidgetToScreenOffset();
   virtual PRBool          ShowsResizeIndicator(nsIntRect* aResizerRect);
 
@@ -345,11 +434,13 @@ public:
   NS_IMETHOD              DispatchEvent(nsGUIEvent* event, nsEventStatus & aStatus);
 
   NS_IMETHOD              Update();
+  virtual PRBool          GetShouldAccelerate();
 
   NS_IMETHOD        SetCursor(nsCursor aCursor);
   NS_IMETHOD        SetCursor(imgIContainer* aCursor, PRUint32 aHotspotX, PRUint32 aHotspotY);
   
-  NS_IMETHOD        CaptureRollupEvents(nsIRollupListener * aListener, PRBool aDoCapture, PRBool aConsumeRollupEvent);
+  NS_IMETHOD        CaptureRollupEvents(nsIRollupListener * aListener, nsIMenuRollup * aMenuRollup, 
+                                        PRBool aDoCapture, PRBool aConsumeRollupEvent);
   NS_IMETHOD        SetTitle(const nsAString& title);
 
   NS_IMETHOD        GetAttention(PRInt32 aCycleCount);
@@ -362,34 +453,50 @@ public:
   NS_IMETHOD        ResetInputState();
   NS_IMETHOD        SetIMEOpenState(PRBool aState);
   NS_IMETHOD        GetIMEOpenState(PRBool* aState);
-  NS_IMETHOD        SetIMEEnabled(PRUint32 aState);
-  NS_IMETHOD        GetIMEEnabled(PRUint32* aState);
+  NS_IMETHOD        SetInputMode(const IMEContext& aContext);
+  NS_IMETHOD        GetInputMode(IMEContext& aContext);
   NS_IMETHOD        CancelIMEComposition();
   NS_IMETHOD        GetToggledKeyState(PRUint32 aKeyCode,
                                        PRBool* aLEDState);
+  NS_IMETHOD        OnIMEFocusChange(PRBool aFocus);
 
   // nsIPluginWidget
   NS_IMETHOD        GetPluginClipRect(nsIntRect& outClipRect, nsIntPoint& outOrigin, PRBool& outWidgetVisible);
   NS_IMETHOD        StartDrawPlugin();
   NS_IMETHOD        EndDrawPlugin();
   NS_IMETHOD        SetPluginInstanceOwner(nsIPluginInstanceOwner* aInstanceOwner);
-  
+
+  NS_IMETHOD        SetPluginEventModel(int inEventModel);
+  NS_IMETHOD        GetPluginEventModel(int* outEventModel);
+  NS_IMETHOD        SetPluginDrawingModel(int inDrawingModel);
+
+  NS_IMETHOD        StartComplexTextInputForCurrentEvent();
+
   virtual nsTransparencyMode GetTransparencyMode();
   virtual void                SetTransparencyMode(nsTransparencyMode aMode);
-  NS_IMETHOD        SetWindowShadowStyle(PRInt32 aStyle);
+
+  virtual nsresult SynthesizeNativeKeyEvent(PRInt32 aNativeKeyboardLayout,
+                                            PRInt32 aNativeKeyCode,
+                                            PRUint32 aModifierFlags,
+                                            const nsAString& aCharacters,
+                                            const nsAString& aUnmodifiedCharacters);
+
+  virtual nsresult SynthesizeNativeMouseEvent(nsIntPoint aPoint,
+                                              PRUint32 aNativeMessage,
+                                              PRUint32 aModifierFlags);
   
   // Mac specific methods
   
   virtual PRBool    DispatchWindowEvent(nsGUIEvent& event);
   
-  void              LiveResizeStarted();
-  void              LiveResizeEnded();
-  
 #ifdef ACCESSIBILITY
-  void              GetDocumentAccessible(nsIAccessible** aAccessible);
+  already_AddRefed<nsAccessible> GetDocumentAccessible();
 #endif
 
   virtual gfxASurface* GetThebesSurface();
+  virtual void DrawOver(LayerManager* aManager, nsIntRect aRect);
+
+  virtual void UpdateThemeGeometries(const nsTArray<ThemeGeometry>& aThemeGeometries);
 
   NS_IMETHOD BeginSecureKeyboardInput();
   NS_IMETHOD EndSecureKeyboardInput();
@@ -403,9 +510,20 @@ public:
   static PRUint32 GetCurrentInputEventCount();
   static void UpdateCurrentInputEventCount();
 
-  static void ApplyConfiguration(nsIWidget* aExpectedParent,
-                                 const nsIWidget::Configuration& aConfiguration,
-                                 PRBool aRepaint);
+  NSView<mozView>* GetEditorView();
+
+  PRBool IsPluginView() { return (mWindowType == eWindowType_plugin); }
+
+  void PaintQD();
+
+  nsCocoaWindow*    GetXULWindowWidget();
+
+  NS_IMETHOD        ReparentNativeWidget(nsIWidget* aNewParent);
+
+  mozilla::widget::TextInputHandler* GetTextInputHandler()
+  {
+    return mTextInputHandler;
+  }
 
 protected:
 
@@ -413,22 +531,24 @@ protected:
   PRBool            ReportMoveEvent();
   PRBool            ReportSizeEvent();
 
-  virtual PRBool    OnPaint(nsPaintEvent & aEvent);
-
   // override to create different kinds of child views. Autoreleases, so
   // caller must retain.
   virtual NSView*   CreateCocoaView(NSRect inFrame);
   void              TearDownView();
 
-  virtual nsresult SynthesizeNativeKeyEvent(PRInt32 aNativeKeyboardLayout,
-                                            PRInt32 aNativeKeyCode,
-                                            PRUint32 aModifierFlags,
-                                            const nsAString& aCharacters,
-                                            const nsAString& aUnmodifiedCharacters);
+  virtual already_AddRefed<nsIWidget>
+  AllocateChildPopupWidget()
+  {
+    static NS_DEFINE_IID(kCPopUpCID, NS_POPUP_CID);
+    nsCOMPtr<nsIWidget> widget = do_CreateInstance(kCPopUpCID);
+    return widget.forget();
+  }
 
 protected:
 
   NSView<mozView>*      mView;      // my parallel cocoa view (ChildView or NativeScrollbarView), [STRONG]
+  nsRefPtr<mozilla::widget::TextInputHandler> mTextInputHandler;
+  IMEContext            mIMEContext;
 
   NSView<mozView>*      mParentView;
   nsIWidget*            mParentWidget;
@@ -440,14 +560,17 @@ protected:
 #endif
 
   nsRefPtr<gfxASurface> mTempThebesSurface;
+  nsRefPtr<mozilla::gl::TextureImage> mResizerImage;
 
   PRPackedBool          mVisible;
   PRPackedBool          mDrawing;
-  PRPackedBool          mLiveResizeInProgress;
   PRPackedBool          mPluginDrawing;
-  PRPackedBool          mPluginIsCG; // true if this is a CoreGraphics plugin
+  PRPackedBool          mIsDispatchPaint; // Is a paint event being dispatched
 
-  nsPluginPort          mPluginPort;
+  NP_CGContext          mPluginCGContext;
+#ifndef NP_NO_QUICKDRAW
+  NP_Port               mPluginQDPort;
+#endif
   nsIPluginInstanceOwner* mPluginInstanceOwner; // [WEAK]
 
   static PRUint32 sLastInputEventCount;
