@@ -69,6 +69,7 @@
 #include "nsPresContext.h"
 #include "nsIDocument.h"
 #include "nsISelection.h"
+#include "nsIFrame.h"
 
 // This sets how opaque the drag image is
 #define DRAG_IMAGE_ALPHA_LEVEL 0.5
@@ -1568,20 +1569,41 @@ nsDragService::SourceDataGet(GtkWidget        *aWidget,
 
 void nsDragService::SetDragIcon(GdkDragContext* aContext)
 {
+    if (!mHasImage && !mSelection)
+        return;
+
     nsIntRect dragRect;
     nsPresContext* pc;
     nsRefPtr<gfxASurface> surface;
-    if (mHasImage || mSelection) {
-      DrawDrag(mSourceNode, mSourceRegion, mScreenX, mScreenY,
-               &dragRect, getter_AddRefs(surface), &pc);
+    DrawDrag(mSourceNode, mSourceRegion, mScreenX, mScreenY,
+             &dragRect, getter_AddRefs(surface), &pc);
+    if (!pc)
+        return;
+
+    PRInt32 sx = mScreenX, sy = mScreenY;
+    ConvertToUnscaledDevPixels(pc, &sx, &sy);
+
+    PRInt32 offsetX = sx - dragRect.x;
+    PRInt32 offsetY = sy - dragRect.y;
+
+    // If a popup is set as the drag image, use its widget. Otherwise, use
+    // the surface that DrawDrag created.
+    if (mDragPopup) {
+        GtkWidget* gtkWidget = nsnull;
+        nsIFrame* frame = mDragPopup->GetPrimaryFrame();
+        if (frame) {
+            // DrawDrag ensured that this is a popup frame.
+            nsCOMPtr<nsIWidget> widget = frame->GetNearestWidget();
+            if (widget) {
+                gtkWidget = (GtkWidget *)widget->GetNativeData(NS_NATIVE_SHELLWIDGET);
+                if (gtkWidget) {
+                    OpenDragPopup();
+                    gtk_drag_set_icon_widget(aContext, gtkWidget, offsetX, offsetY);
+                }
+            }
+        }
     }
-
-    if (surface) {
-        PRInt32 sx = mScreenX, sy = mScreenY;
-        ConvertToUnscaledDevPixels(pc, &sx, &sy);
-
-        PRInt32 offsetX = sx - dragRect.x;
-        PRInt32 offsetY = sy - dragRect.y;
+    else if (surface) {
         if (!SetAlphaPixmap(surface, aContext, offsetX, offsetY, dragRect)) {
             GdkPixbuf* dragPixbuf =
               nsImageToPixbuf::SurfaceToPixbuf(surface, dragRect.width, dragRect.height);
