@@ -49,14 +49,16 @@
 
 #include "nsGtkIMModule.h"
 #include "nsWindow.h"
+#include "mozilla/Preferences.h"
 
 #ifdef MOZ_PLATFORM_MAEMO
 #include "nsServiceManagerUtils.h"
 #include "nsIObserverService.h"
-#include "nsIPrefService.h"
 #include "mozilla/Services.h"
 #endif
 
+using namespace mozilla;
+
 #ifdef PR_LOGGING
 PRLogModuleInfo* gGtkIMLog = nsnull;
 
@@ -340,7 +342,7 @@ nsGtkIMModule::OnFocusWindow(nsWindow* aWindow)
 
     PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
         ("GtkIMModule(%p): OnFocusWindow, aWindow=%p, mLastFocusedWindow=%p",
-         this, aWindow));
+         this, aWindow, mLastFocusedWindow));
     mLastFocusedWindow = aWindow;
     Focus();
 }
@@ -412,7 +414,7 @@ nsGtkIMModule::OnKeyEvent(nsWindow* aCaller, GdkEventKey* aEvent,
 
     if (mIsComposing && !isFiltered) {
         if (aEvent->type == GDK_KEY_PRESS) {
-            if (!mCompositionString.IsEmpty()) {
+            if (!mDispatchedCompositionString.IsEmpty()) {
                 // If there is composition string, we shouldn't dispatch
                 // any keydown events during composition.
                 filterThisEvent = PR_TRUE;
@@ -499,7 +501,7 @@ nsGtkIMModule::ResetInputState(nsWindow* aCaller)
 
     // XXX We should commit composition ourselves temporary...
     ResetIME();
-    CommitCompositionBy(mCompositionString);
+    CommitCompositionBy(mDispatchedCompositionString);
 
     return NS_OK;
 }
@@ -548,7 +550,8 @@ nsGtkIMModule::SetInputMode(nsWindow* aCaller, const IMEContext* aContext)
 
     PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
         ("GtkIMModule(%p): SetInputMode, aCaller=%p, aState=%s mHTMLInputType=%s",
-         this, aCaller, GetEnabledStateName(aContext->mStatus), aContext->mHTMLInputType.get()));
+         this, aCaller, GetEnabledStateName(aContext->mStatus),
+         NS_ConvertUTF16toUTF8(aContext->mHTMLInputType).get()));
 
     if (aCaller != mLastFocusedWindow) {
         PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
@@ -594,14 +597,11 @@ nsGtkIMModule::SetInputMode(nsWindow* aCaller, const IMEContext* aContext)
             if (mIMEContext.mStatus != nsIWidget::IME_STATUS_DISABLED && 
                 mIMEContext.mStatus != nsIWidget::IME_STATUS_PLUGIN) {
 
-                nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID));
-
-                PRBool useStrictPolicy = PR_FALSE;
-                if (NS_SUCCEEDED(prefs->GetBoolPref("content.ime.strict_policy", &useStrictPolicy))) {
-                    if (useStrictPolicy && !mIMEContext.FocusMovedByUser() && 
-                        mIMEContext.FocusMovedInContentProcess()) {
-                        return NS_OK;
-                    }
+                PRBool useStrictPolicy =
+                    Preferences::GetBool("content.ime.strict_policy", PR_FALSE);
+                if (useStrictPolicy && !mIMEContext.FocusMovedByUser() && 
+                    mIMEContext.FocusMovedInContentProcess()) {
+                    return NS_OK;
                 }
             }
 
@@ -727,8 +727,7 @@ nsGtkIMModule::Focus()
     GtkIMContext *im = GetContext();
     if (!im) {
         PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
-            ("    FAILED, there are no context",
-             this));
+            ("    FAILED, there are no context"));
         return;
     }
 
@@ -835,13 +834,8 @@ nsGtkIMModule::OnEndCompositionNative(GtkIMContext *aContext)
         return;
     }
 
-#ifdef PR_LOGGING
-    if (!mCompositionString.IsEmpty()) {
-        PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
-            ("    WARNING, the composition string is still there"));
-    }
-#endif
-    DispatchCompositionEnd(); // Be aware, widget can be gone
+    // Be aware, widget can be gone
+    DispatchCompositionEnd();
 }
 
 /* static */
@@ -871,12 +865,15 @@ nsGtkIMModule::OnChangeCompositionNative(GtkIMContext *aContext)
         return;
     }
 
-    GetCompositionString(mCompositionString);
-    if (!mIsComposing && mCompositionString.IsEmpty()) {
+    nsAutoString compositionString;
+    GetCompositionString(compositionString);
+    if (!mIsComposing && compositionString.IsEmpty()) {
+        mDispatchedCompositionString.Truncate();
         return; // Don't start the composition with empty string.
     }
 
-    DispatchTextEvent(PR_TRUE); // Be aware, widget can be gone
+    // Be aware, widget can be gone
+    DispatchTextEvent(compositionString, PR_TRUE);
 }
 
 /* static */
@@ -1028,12 +1025,12 @@ PRBool
 nsGtkIMModule::CommitCompositionBy(const nsAString& aString)
 {
     PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
-        ("GtkIMModule(%p): CommitCompositionBy, aString=\"%s\", mCompositionString=\"%s\"",
+        ("GtkIMModule(%p): CommitCompositionBy, aString=\"%s\", "
+         "mDispatchedCompositionString=\"%s\"",
          this, NS_ConvertUTF16toUTF8(aString).get(),
-         NS_ConvertUTF16toUTF8(mCompositionString).get()));
+         NS_ConvertUTF16toUTF8(mDispatchedCompositionString).get()));
 
-    mCompositionString = aString;
-    if (!DispatchTextEvent(PR_FALSE)) {
+    if (!DispatchTextEvent(aString, PR_FALSE)) {
         return PR_FALSE;
     }
     // We should dispatch the compositionend event here because some IMEs
@@ -1094,6 +1091,7 @@ nsGtkIMModule::DispatchCompositionStart()
     }
 
     mCompositionStart = selection.mReply.mOffset;
+    mDispatchedCompositionString.Truncate();
 
     if (mProcessingKeyEvent && !mKeyDownEventWasSent &&
         mProcessingKeyEvent->type == GDK_KEY_PRESS) {
@@ -1120,7 +1118,7 @@ nsGtkIMModule::DispatchCompositionStart()
     }
 
     PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
-        ("    mCompositionStart=%lu", mCompositionStart));
+        ("    mCompositionStart=%u", mCompositionStart));
     mIsComposing = PR_TRUE;
     nsCompositionEvent compEvent(PR_TRUE, NS_COMPOSITION_START,
                                  mLastFocusedWindow);
@@ -1141,7 +1139,9 @@ PRBool
 nsGtkIMModule::DispatchCompositionEnd()
 {
     PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
-        ("GtkIMModule(%p): DispatchCompositionEnd", this));
+        ("GtkIMModule(%p): DispatchCompositionEnd, "
+         "mDispatchedCompositionString=\"%s\"",
+         this, NS_ConvertUTF16toUTF8(mDispatchedCompositionString).get()));
 
     if (!mIsComposing) {
         PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
@@ -1150,6 +1150,7 @@ nsGtkIMModule::DispatchCompositionEnd()
     }
 
     if (!mLastFocusedWindow) {
+        mDispatchedCompositionString.Truncate();
         PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
             ("    FAILED, there are no focused window in this module"));
         return PR_FALSE;
@@ -1158,12 +1159,13 @@ nsGtkIMModule::DispatchCompositionEnd()
     nsCompositionEvent compEvent(PR_TRUE, NS_COMPOSITION_END,
                                  mLastFocusedWindow);
     InitEvent(compEvent);
+    compEvent.data = mDispatchedCompositionString;
     nsEventStatus status;
     nsCOMPtr<nsIWidget> kungFuDeathGrip = mLastFocusedWindow;
     mLastFocusedWindow->DispatchEvent(&compEvent, status);
     mIsComposing = PR_FALSE;
     mCompositionStart = PR_UINT32_MAX;
-    mCompositionString.Truncate();
+    mDispatchedCompositionString.Truncate();
     if (static_cast<nsWindow*>(kungFuDeathGrip.get())->IsDestroyed() ||
         kungFuDeathGrip != mLastFocusedWindow) {
         PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
@@ -1175,7 +1177,8 @@ nsGtkIMModule::DispatchCompositionEnd()
 }
 
 PRBool
-nsGtkIMModule::DispatchTextEvent(PRBool aCheckAttr)
+nsGtkIMModule::DispatchTextEvent(const nsAString &aCompositionString,
+                                 PRBool aCheckAttr)
 {
     PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
         ("GtkIMModule(%p): DispatchTextEvent, aCheckAttr=%s",
@@ -1196,6 +1199,24 @@ nsGtkIMModule::DispatchTextEvent(PRBool aCheckAttr)
         }
     }
 
+    nsEventStatus status;
+    nsRefPtr<nsWindow> lastFocusedWindow = mLastFocusedWindow;
+
+    if (aCompositionString != mDispatchedCompositionString) {
+      nsCompositionEvent compositionUpdate(PR_TRUE, NS_COMPOSITION_UPDATE,
+                                           mLastFocusedWindow);
+      InitEvent(compositionUpdate);
+      compositionUpdate.data = aCompositionString;
+      mDispatchedCompositionString = aCompositionString;
+      mLastFocusedWindow->DispatchEvent(&compositionUpdate, status);
+      if (lastFocusedWindow->IsDestroyed() ||
+          lastFocusedWindow != mLastFocusedWindow) {
+          PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
+              ("    NOTE, the focused widget was destroyed/changed by compositionupdate"));
+          return PR_FALSE;
+      }
+    }
+
     nsTextEvent textEvent(PR_TRUE, NS_TEXT_TEXT, mLastFocusedWindow);
     InitEvent(textEvent);
 
@@ -1203,6 +1224,8 @@ nsGtkIMModule::DispatchTextEvent(PRBool aCheckAttr)
 
     nsAutoTArray<nsTextRange, 4> textRanges;
     if (aCheckAttr) {
+        // NOTE: SetTextRangeList() assumes that mDispatchedCompositionString
+        //       has been updated already.
         SetTextRangeList(textRanges);
         for (PRUint32 i = 0; i < textRanges.Length(); i++) {
             nsTextRange& range = textRanges[i];
@@ -1216,13 +1239,11 @@ nsGtkIMModule::DispatchTextEvent(PRBool aCheckAttr)
 
     textEvent.rangeCount = textRanges.Length();
     textEvent.rangeArray = textRanges.Elements();
-    textEvent.theText = mCompositionString.get();
+    textEvent.theText = mDispatchedCompositionString.get();
 
-    nsEventStatus status;
-    nsCOMPtr<nsIWidget> kungFuDeathGrip = mLastFocusedWindow;
     mLastFocusedWindow->DispatchEvent(&textEvent, status);
-    if (static_cast<nsWindow*>(kungFuDeathGrip.get())->IsDestroyed() ||
-        kungFuDeathGrip != mLastFocusedWindow) {
+    if (lastFocusedWindow->IsDestroyed() ||
+        lastFocusedWindow != mLastFocusedWindow) {
         PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
             ("    NOTE, the focused widget was destroyed/changed by text event"));
         return PR_FALSE;
@@ -1330,7 +1351,7 @@ nsGtkIMModule::SetTextRangeList(nsTArray<nsTextRange> &aTextRangeList)
         aTextRangeList.AppendElement(range);
 
         PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
-            ("    mStartOffset=%lu, mEndOffset=%lu, mRangeType=%s",
+            ("    mStartOffset=%u, mEndOffset=%u, mRangeType=%s",
              range.mStartOffset, range.mEndOffset,
              GetRangeTypeName(range.mRangeType)));
     } while (pango_attr_iterator_next(iter));
@@ -1338,8 +1359,8 @@ nsGtkIMModule::SetTextRangeList(nsTArray<nsTextRange> &aTextRangeList)
     nsTextRange range;
     if (cursor_pos < 0) {
         range.mStartOffset = 0;
-    } else if (PRUint32(cursor_pos) > mCompositionString.Length()) {
-        range.mStartOffset = mCompositionString.Length();
+    } else if (PRUint32(cursor_pos) > mDispatchedCompositionString.Length()) {
+        range.mStartOffset = mDispatchedCompositionString.Length();
     } else {
         range.mStartOffset = PRUint32(cursor_pos);
     }
@@ -1348,7 +1369,7 @@ nsGtkIMModule::SetTextRangeList(nsTArray<nsTextRange> &aTextRangeList)
     aTextRangeList.AppendElement(range);
 
     PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
-        ("    mStartOffset=%lu, mEndOffset=%lu, mRangeType=%s",
+        ("    mStartOffset=%u, mEndOffset=%u, mRangeType=%s",
          range.mStartOffset, range.mEndOffset,
          GetRangeTypeName(range.mRangeType)));
 
@@ -1361,7 +1382,7 @@ void
 nsGtkIMModule::SetCursorPosition(PRUint32 aTargetOffset)
 {
     PR_LOG(gGtkIMLog, PR_LOG_ALWAYS,
-        ("GtkIMModule(%p): SetCursorPosition, aTargetOffset=%lu",
+        ("GtkIMModule(%p): SetCursorPosition, aTargetOffset=%u",
          this, aTargetOffset));
 
     if (aTargetOffset == PR_UINT32_MAX) {
