@@ -38,8 +38,6 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#include "nsIFrame.h"
-
 #include <QApplication>
 #include <QStyle>
 #include <QPalette>
@@ -51,26 +49,29 @@
 #include <QFlags>
 #include <QStyleOptionComboBox>
 
+#include "nsIFrame.h"
+
 #include "nsCoord.h"
 #include "nsNativeThemeQt.h"
-#include "nsIDeviceContext.h"
 #include "nsPresContext.h"
 
 #include "nsRect.h"
 #include "nsSize.h"
 #include "nsTransform2D.h"
 #include "nsThemeConstants.h"
-#include "nsILookAndFeel.h"
 #include "nsIServiceManager.h"
-#include "nsIEventStateManager.h"
 #include "nsIDOMHTMLInputElement.h"
 #include <malloc.h>
 
 
 #include "gfxASurface.h"
 #include "gfxContext.h"
+#include "gfxQtPlatform.h"
 #include "gfxQPainterSurface.h"
-#include "nsIRenderingContext.h"
+#ifdef MOZ_X11
+#include "gfxXlibSurface.h"
+#endif
+#include "nsRenderingContext.h"
 
 nsNativeThemeQt::nsNativeThemeQt()
 {
@@ -82,7 +83,7 @@ nsNativeThemeQt::~nsNativeThemeQt()
 {
 }
 
-NS_IMPL_ISUPPORTS1(nsNativeThemeQt, nsITheme)
+NS_IMPL_ISUPPORTS_INHERITED1(nsNativeThemeQt, nsNativeTheme, nsITheme)
 
 static inline QRect qRectInPixels(const nsRect &aRect,
                                   const PRInt32 p2a)
@@ -93,29 +94,93 @@ static inline QRect qRectInPixels(const nsRect &aRect,
                  NSAppUnitsToIntPixels(aRect.height, p2a));
 }
 
+static inline QImage::Format
+_qimage_from_gfximage_format (gfxASurface::gfxImageFormat aFormat)
+{
+    switch (aFormat) {
+    case gfxASurface::ImageFormatARGB32:
+        return QImage::Format_ARGB32_Premultiplied;
+    case gfxASurface::ImageFormatRGB24:
+        return QImage::Format_RGB32;
+    case gfxASurface::ImageFormatRGB16_565:
+        return QImage::Format_RGB16;
+    case gfxASurface::ImageFormatA8:
+        return QImage::Format_Indexed8;
+    case gfxASurface::ImageFormatA1:
+#ifdef WORDS_BIGENDIAN
+        return QImage::Format_Mono;
+#else
+        return QImage::Format_MonoLSB;
+#endif
+    default:
+        return QImage::Format_Invalid;
+    }
+
+    return QImage::Format_Mono;
+}
+
 NS_IMETHODIMP
-nsNativeThemeQt::DrawWidgetBackground(nsIRenderingContext* aContext,
+nsNativeThemeQt::DrawWidgetBackground(nsRenderingContext* aContext,
                                       nsIFrame* aFrame,
                                       PRUint8 aWidgetType,
                                       const nsRect& aRect,
                                       const nsRect& aClipRect)
+{
+    gfxContext* context = aContext->ThebesContext();
+    nsRefPtr<gfxASurface> surface = context->CurrentSurface();
+
+#ifdef CAIRO_HAS_QT_SURFACE
+    if (surface->GetType() == gfxASurface::SurfaceTypeQPainter) {
+        gfxQPainterSurface* qSurface = (gfxQPainterSurface*) (surface.get());
+        QPainter *painter = qSurface->GetQPainter();
+        NS_ASSERTION(painter, "Where'd my QPainter go?");
+        if (!painter)
+            return NS_ERROR_FAILURE;
+        return DrawWidgetBackground(painter, aContext,
+                                    aFrame, aWidgetType,
+                                    aRect, aClipRect);
+    } else
+#endif
+    if (surface->GetType() == gfxASurface::SurfaceTypeImage) {
+        gfxImageSurface* qSurface = (gfxImageSurface*) (surface.get());
+        QImage tempQImage(qSurface->Data(),
+                          qSurface->Width(),
+                          qSurface->Height(),
+                          qSurface->Stride(),
+                          _qimage_from_gfximage_format(qSurface->Format()));
+        QPainter painter(&tempQImage);
+        return DrawWidgetBackground(&painter, aContext,
+                                    aFrame, aWidgetType,
+                                    aRect, aClipRect);
+    }
+#ifdef MOZ_X11
+    else if (surface->GetType() == gfxASurface::SurfaceTypeXlib) {
+        gfxXlibSurface* qSurface = (gfxXlibSurface*) (surface.get());
+        QPixmap pixmap(QPixmap::fromX11Pixmap(qSurface->XDrawable()));
+        QPainter painter(&pixmap);
+        return DrawWidgetBackground(&painter, aContext,
+                                    aFrame, aWidgetType,
+                                    aRect, aClipRect);
+    }
+#endif
+
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+nsresult
+nsNativeThemeQt::DrawWidgetBackground(QPainter *qPainter,
+                                      nsRenderingContext* aContext,
+                                      nsIFrame* aFrame,
+                                      PRUint8 aWidgetType,
+                                      const nsRect& aRect,
+                                      const nsRect& aClipRect)
+
 {
     gfxContext* context = aContext->ThebesContext();
     nsRefPtr<gfxASurface> surface = context->CurrentSurface();
 
     context->UpdateSurfaceClip();
 
-    if (surface->GetType() != gfxASurface::SurfaceTypeQPainter)
-        return NS_ERROR_NOT_IMPLEMENTED;
-
-    gfxQPainterSurface* qSurface = (gfxQPainterSurface*) (surface.get());
-    QPainter* qPainter = qSurface->GetQPainter();
-
-    NS_ASSERTION(qPainter, "Where'd my QPainter go?");
-
-    if (qPainter == nsnull)
-        return NS_OK;
-
     QStyle* style = qApp->style();
 
     qPainter->save();
@@ -132,7 +197,7 @@ nsNativeThemeQt::DrawWidgetBackground(nsIRenderingContext* aContext,
     QMatrix qctm(ctm.xx, ctm.yx, ctm.xy, ctm.yy, ctm.x0, ctm.y0);
     qPainter->setWorldMatrix(qctm, true);
 
-    PRInt32 p2a = GetAppUnitsPerDevPixel(aContext);
+    PRInt32 p2a = aContext->AppUnitsPerDevPixel();
 
     QRect r = qRectInPixels(aRect, p2a);
     QRect cr = qRectInPixels(aClipRect, p2a);
@@ -246,8 +311,9 @@ nsNativeThemeQt::DrawWidgetBackground(nsIRenderingContext* aContext,
     case NS_THEME_TEXTFIELD_MULTILINE:
     case NS_THEME_LISTBOX: {
         QStyleOptionFrameV2 frameOpt;
-        
-        if (!IsDisabled(aFrame))
+        nsEventStates eventState = GetContentState(aFrame, aWidgetType);
+
+        if (!IsDisabled(aFrame, eventState))
             frameOpt.state |= QStyle::State_Enabled;
 
         frameOpt.rect = r;
@@ -278,7 +344,7 @@ nsNativeThemeQt::DrawWidgetBackground(nsIRenderingContext* aContext,
 }
 
 NS_IMETHODIMP
-nsNativeThemeQt::GetWidgetBorder(nsIDeviceContext* aContext,
+nsNativeThemeQt::GetWidgetBorder(nsDeviceContext* ,
                                  nsIFrame* aFrame,
                                  PRUint8 aWidgetType,
                                  nsIntMargin* aResult)
@@ -301,7 +367,7 @@ nsNativeThemeQt::GetWidgetBorder(nsIDeviceContext* aContext,
 }
 
 PRBool
-nsNativeThemeQt::GetWidgetPadding(nsIDeviceContext* ,
+nsNativeThemeQt::GetWidgetPadding(nsDeviceContext* ,
                                   nsIFrame*, PRUint8 aWidgetType,
                                   nsIntMargin* aResult)
 {
@@ -317,7 +383,7 @@ nsNativeThemeQt::GetWidgetPadding(nsIDeviceContext* ,
 }
 
 NS_IMETHODIMP
-nsNativeThemeQt::GetMinimumWidgetSize(nsIRenderingContext* aContext, nsIFrame* aFrame,
+nsNativeThemeQt::GetMinimumWidgetSize(nsRenderingContext* aContext, nsIFrame* aFrame,
                                       PRUint8 aWidgetType,
                                       nsIntSize* aResult, PRBool* aIsOverridable)
 {
@@ -326,7 +392,7 @@ nsNativeThemeQt::GetMinimumWidgetSize(nsIRenderingContext* aContext, nsIFrame* a
 
     QStyle *s = qApp->style();
 
-    PRInt32 p2a = GetAppUnitsPerDevPixel(aContext);
+    PRInt32 p2a = aContext->AppUnitsPerDevPixel();
 
     switch (aWidgetType) {
     case NS_THEME_RADIO:
@@ -563,32 +629,26 @@ nsNativeThemeQt::ThemeNeedsComboboxDropmarker()
     return PR_TRUE;
 }
 
-nsTransparencyMode
-nsNativeThemeQt::GetWidgetTransparency(PRUint8 aWidgetType)
-{
-  return eTransparencyOpaque;
-}
-
 void
 nsNativeThemeQt::InitButtonStyle(PRUint8 aWidgetType,
                                  nsIFrame* aFrame,
                                  QRect rect,
                                  QStyleOptionButton &opt)
 {
-    PRInt32 eventState = GetContentState(aFrame, aWidgetType);
+    nsEventStates eventState = GetContentState(aFrame, aWidgetType);
 
     opt.rect = rect;
     opt.palette = mNoBackgroundPalette;
 
-    PRBool disabled = IsDisabled(aFrame);
+    PRBool isDisabled = IsDisabled(aFrame, eventState);
 
-    if (!disabled)
+    if (!isDisabled)
         opt.state |= QStyle::State_Enabled;
-    if (eventState & NS_EVENT_STATE_HOVER)
+    if (eventState.HasState(NS_EVENT_STATE_HOVER))
         opt.state |= QStyle::State_MouseOver;
-    if (eventState & NS_EVENT_STATE_FOCUS)
+    if (eventState.HasState(NS_EVENT_STATE_FOCUS))
         opt.state |= QStyle::State_HasFocus;
-    if (!disabled && eventState & NS_EVENT_STATE_ACTIVE)
+    if (!isDisabled && eventState.HasState(NS_EVENT_STATE_ACTIVE))
         // Don't allow sunken when disabled
         opt.state |= QStyle::State_Sunken;
 
@@ -602,7 +662,7 @@ nsNativeThemeQt::InitButtonStyle(PRUint8 aWidgetType,
 
         break;
     default:
-        if (!(eventState & NS_EVENT_STATE_ACTIVE))
+        if (!eventState.HasState(NS_EVENT_STATE_ACTIVE))
             opt.state |= QStyle::State_Raised;
         break;
     }
@@ -615,15 +675,15 @@ nsNativeThemeQt::InitPlainStyle(PRUint8 aWidgetType,
                                 QStyleOption &opt,
                                 QStyle::State extraFlags)
 {
-    PRInt32 eventState = GetContentState(aFrame, aWidgetType);
+    nsEventStates eventState = GetContentState(aFrame, aWidgetType);
 
     opt.rect = rect;
 
-    if (!IsDisabled(aFrame))
+    if (!IsDisabled(aFrame, eventState))
         opt.state |= QStyle::State_Enabled;
-    if (eventState & NS_EVENT_STATE_HOVER)
+    if (eventState.HasState(NS_EVENT_STATE_HOVER))
         opt.state |= QStyle::State_MouseOver;
-    if (eventState & NS_EVENT_STATE_FOCUS)
+    if (eventState.HasState(NS_EVENT_STATE_FOCUS))
         opt.state |= QStyle::State_HasFocus;
 
     opt.state |= extraFlags;
@@ -635,19 +695,18 @@ nsNativeThemeQt::InitComboStyle(PRUint8 aWidgetType,
                                 QRect rect,
                                 QStyleOptionComboBox &opt)
 {
-    PRInt32 eventState = GetContentState(aFrame, aWidgetType);
+    nsEventStates eventState = GetContentState(aFrame, aWidgetType);
+    PRBool isDisabled = IsDisabled(aFrame, eventState);
 
-    PRBool disabled = IsDisabled(aFrame);
-
-    if (!disabled)
+    if (!isDisabled)
         opt.state |= QStyle::State_Enabled;
-    if (eventState & NS_EVENT_STATE_HOVER)
+    if (eventState.HasState(NS_EVENT_STATE_HOVER))
         opt.state |= QStyle::State_MouseOver;
-    if (eventState & NS_EVENT_STATE_FOCUS)
+    if (eventState.HasState(NS_EVENT_STATE_FOCUS))
         opt.state |= QStyle::State_HasFocus;
-    if (!(eventState & NS_EVENT_STATE_ACTIVE))
+    if (!eventState.HasState(NS_EVENT_STATE_ACTIVE))
         opt.state |= QStyle::State_Raised;
-    if (!disabled && eventState & NS_EVENT_STATE_ACTIVE)
+    if (!isDisabled && eventState.HasState(NS_EVENT_STATE_ACTIVE))
         // Don't allow sunken when disabled
         opt.state |= QStyle::State_Sunken;
 
