@@ -42,17 +42,16 @@
 #include "GfxInfo.h"
 #include "GfxInfoWebGL.h"
 #include "nsUnicharUtils.h"
-#include "nsPrintfCString.h"
 #include "mozilla/FunctionTimer.h"
 #include "prenv.h"
 #include "prprf.h"
 #include "GfxDriverInfo.h"
+#include "mozilla/Preferences.h"
 
 #if defined(MOZ_CRASHREPORTER)
 #include "nsExceptionHandler.h"
 #include "nsICrashReporter.h"
 #define NS_CRASHREPORTER_CONTRACTID "@mozilla.org/toolkit/crash-reporter;1"
-#include "nsIPrefService.h"
 #endif
 
 
@@ -78,7 +77,11 @@ static const PRUint32 vendorATI = 0x1002;
 GfxInfo::GfxInfo()
   : mAdapterVendorID(0),
     mAdapterDeviceID(0),
-    mWindowsVersion(0)
+    mAdapterVendorID2(0),
+    mAdapterDeviceID2(0),
+    mWindowsVersion(0),
+    mHasDualGPU(PR_FALSE),
+    mIsGPU2Active(PR_FALSE)
 {
 }
 
@@ -98,6 +101,26 @@ GfxInfo::GetDWriteEnabled(PRBool *aEnabled)
   return NS_OK;
 }
 
+nsresult
+GfxInfo::GetAzureEnabled(PRBool *aEnabled)
+{
+  *aEnabled = PR_FALSE;
+
+  PRBool d2dEnabled = 
+    gfxWindowsPlatform::GetPlatform()->GetRenderMode() == gfxWindowsPlatform::RENDER_DIRECT2D;
+
+  if (d2dEnabled) {
+    PRBool azure = PR_FALSE;
+    nsresult rv = mozilla::Preferences::GetBool("gfx.canvas.azure.enabled", &azure);
+
+    if (NS_SUCCEEDED(rv) && azure) {
+      *aEnabled = PR_TRUE;
+    }
+  }
+
+  return NS_OK;
+}
+
 /* readonly attribute DOMString DWriteVersion; */
 NS_IMETHODIMP
 GfxInfo::GetDWriteVersion(nsAString & aDwriteVersion)
@@ -180,8 +203,6 @@ GfxInfo::GetCleartypeParameters(nsAString & aCleartypeParams)
   return NS_ERROR_FAILURE;
 }
 
-/* XXX: GfxInfo doesn't handle multiple GPUs. We should try to do that. Bug #591057 */
-
 static nsresult GetKeyValue(const WCHAR* keyLocation, const WCHAR* keyName, nsAString& destString, int type)
 {
   HKEY key;
@@ -285,7 +306,21 @@ typedef BOOL (WINAPI*SetupDiDestroyDeviceInfoListFunc)(
   HDEVINFO DeviceInfoSet
 );
 
-
+// The device ID is a string like PCI\VEN_15AD&DEV_0405&SUBSYS_040515AD
+// this function is used to extract the id's out of it
+PRUint32
+ParseIDFromDeviceID(const nsAString &key, const char *prefix, int length)
+{
+  nsAutoString id(key);
+  ToUpperCase(id);
+  PRInt32 start = id.Find(prefix);
+  if (start != -1) {
+    id.Cut(0, start + strlen(prefix));
+    id.Truncate(length);
+  }
+  nsresult err;
+  return id.ToInteger(&err, 16);
+}
 
 /* Other interesting places for info:
  *   IDXGIAdapter::GetDesc()
@@ -354,9 +389,7 @@ GfxInfo::Init()
         setupGetDeviceRegistryProperty &&
         setupDestroyDeviceInfoList) {
       /* create a device information set composed of the current display device */
-      HDEVINFO devinfo = setupGetClassDevs(NULL,
-                                           PromiseFlatString(mDeviceID).get(),
-                                           NULL,
+      HDEVINFO devinfo = setupGetClassDevs(NULL, mDeviceID.get(), NULL,
                                            DIGCF_PRESENT | DIGCF_PROFILE | DIGCF_ALLCLASSES);
 
       if (devinfo != INVALID_HANDLE_VALUE) {
@@ -392,7 +425,40 @@ GfxInfo::Init()
               result = RegQueryValueExW(key, L"DriverDate", NULL, NULL, (LPBYTE)value, &dwcbData);
               if (result == ERROR_SUCCESS)
                 mDriverDate = value;
-              RegCloseKey(key);
+              RegCloseKey(key); 
+
+              // Check for second adapter:
+              //
+              // A second adapter will have the same driver key as the first adapter except for 
+              // the last character, where '1' will be swapped for '0' or vice-versa.
+              // We know driverKey.Length() > 0 since driverKeyPre is a prefix of driverKey.
+              if (driverKey[driverKey.Length()-1] == '0') {
+                driverKey.SetCharAt('1', driverKey.Length()-1);
+              } else {
+                driverKey.SetCharAt('0', driverKey.Length()-1);
+              }
+              result = RegOpenKeyExW(HKEY_LOCAL_MACHINE, driverKey.BeginReading(), 0, KEY_QUERY_VALUE, &key);
+              if (result == ERROR_SUCCESS) {
+                mHasDualGPU = PR_TRUE;
+                mDeviceKey2 = driverKey;
+                dwcbData = sizeof(value);
+                result = RegQueryValueExW(key, L"DriverVersion", NULL, NULL, (LPBYTE)value, &dwcbData);
+                if (result == ERROR_SUCCESS)
+                  mDriverVersion2 = value;
+                dwcbData = sizeof(value);
+                result = RegQueryValueExW(key, L"DriverDate", NULL, NULL, (LPBYTE)value, &dwcbData);
+                if (result == ERROR_SUCCESS)
+                  mDriverDate2 = value;
+                dwcbData = sizeof(value);
+                result = RegQueryValueExW(key, L"Device Description", NULL, NULL, (LPBYTE)value, &dwcbData);
+                if (result == ERROR_SUCCESS)
+                  mDeviceString2 = value;
+                dwcbData = sizeof(value);
+                result = RegQueryValueExW(key, L"MatchingDeviceId", NULL, NULL, (LPBYTE)value, &dwcbData);
+                if (result == ERROR_SUCCESS)
+                  mDeviceID2 = value;
+                RegCloseKey(key);
+              }  
               break;
             }
           }
@@ -405,6 +471,13 @@ GfxInfo::Init()
     FreeLibrary(setupapi);
   }
 
+  mAdapterVendorID  = ParseIDFromDeviceID(mDeviceID,  "VEN_", 4);
+  mAdapterVendorID2 = ParseIDFromDeviceID(mDeviceID2, "VEN_", 4);
+  mAdapterDeviceID  = ParseIDFromDeviceID(mDeviceID,  "&DEV_", 4);
+  mAdapterDeviceID2 = ParseIDFromDeviceID(mDeviceID2, "&DEV_", 4);
+  mAdapterSubsysID  = ParseIDFromDeviceID(mDeviceID,  "&SUBSYS_", 8);
+  mAdapterSubsysID2 = ParseIDFromDeviceID(mDeviceID2, "&SUBSYS_", 8);
+
   const char *spoofedDriverVersionString = PR_GetEnv("MOZ_GFX_SPOOF_DRIVER_VERSION");
   if (spoofedDriverVersionString) {
     mDriverVersion.AssignASCII(spoofedDriverVersionString);
@@ -413,16 +486,6 @@ GfxInfo::Init()
   const char *spoofedVendor = PR_GetEnv("MOZ_GFX_SPOOF_VENDOR_ID");
   if (spoofedVendor) {
      PR_sscanf(spoofedVendor, "%x", &mAdapterVendorID);
-  } else {
-    nsAutoString vendor(mDeviceID);
-    ToUpperCase(vendor);
-    PRInt32 start = vendor.Find(NS_LITERAL_CSTRING("VEN_"));
-    if (start != -1) {
-      vendor.Cut(0, start + strlen("VEN_"));
-      vendor.Truncate(4);
-    }
-    nsresult err;
-    mAdapterVendorID = vendor.ToInteger(&err, 16);
   }
 
   mHasDriverVersionMismatch = PR_FALSE;
@@ -451,16 +514,6 @@ GfxInfo::Init()
   const char *spoofedDevice = PR_GetEnv("MOZ_GFX_SPOOF_DEVICE_ID");
   if (spoofedDevice) {
     PR_sscanf(spoofedDevice, "%x", &mAdapterDeviceID);
-  } else {
-    nsAutoString device(mDeviceID);
-    ToUpperCase(device);
-    PRInt32 start = device.Find(NS_LITERAL_CSTRING("&DEV_"));
-    if (start != -1) {
-      device.Cut(0, start + strlen("&DEV_"));
-      device.Truncate(4);
-    }
-    nsresult err;
-    mAdapterDeviceID = device.ToInteger(&err, 16);
   }
 
   const char *spoofedWindowsVersion = PR_GetEnv("MOZ_GFX_SPOOF_WINDOWS_VERSION");
@@ -483,6 +536,14 @@ GfxInfo::GetAdapterDescription(nsAString & aAdapterDescription)
   return NS_OK;
 }
 
+/* readonly attribute DOMString adapterDescription2; */
+NS_IMETHODIMP
+GfxInfo::GetAdapterDescription2(nsAString & aAdapterDescription)
+{
+  aAdapterDescription = mDeviceString2;
+  return NS_OK;
+}
+
 /* readonly attribute DOMString adapterRAM; */
 NS_IMETHODIMP
 GfxInfo::GetAdapterRAM(nsAString & aAdapterRAM)
@@ -492,6 +553,15 @@ GfxInfo::GetAdapterRAM(nsAString & aAdapterRAM)
   return NS_OK;
 }
 
+/* readonly attribute DOMString adapterRAM2; */
+NS_IMETHODIMP
+GfxInfo::GetAdapterRAM2(nsAString & aAdapterRAM)
+{
+  if (NS_FAILED(GetKeyValue(mDeviceKey2.BeginReading(), L"HardwareInformation.MemorySize", aAdapterRAM, REG_DWORD)))
+    aAdapterRAM = L"Unknown";
+  return NS_OK;
+}
+
 /* readonly attribute DOMString adapterDriver; */
 NS_IMETHODIMP
 GfxInfo::GetAdapterDriver(nsAString & aAdapterDriver)
@@ -501,6 +571,15 @@ GfxInfo::GetAdapterDriver(nsAString & aAdapterDriver)
   return NS_OK;
 }
 
+/* readonly attribute DOMString adapterDriver2; */
+NS_IMETHODIMP
+GfxInfo::GetAdapterDriver2(nsAString & aAdapterDriver)
+{
+  if (NS_FAILED(GetKeyValue(mDeviceKey2.BeginReading(), L"InstalledDisplayDrivers", aAdapterDriver, REG_MULTI_SZ)))
+    aAdapterDriver = L"Unknown";
+  return NS_OK;
+}
+
 /* readonly attribute DOMString adapterDriverVersion; */
 NS_IMETHODIMP
 GfxInfo::GetAdapterDriverVersion(nsAString & aAdapterDriverVersion)
@@ -517,6 +596,22 @@ GfxInfo::GetAdapterDriverDate(nsAString & aAdapterDriverDate)
   return NS_OK;
 }
 
+/* readonly attribute DOMString adapterDriverVersion2; */
+NS_IMETHODIMP
+GfxInfo::GetAdapterDriverVersion2(nsAString & aAdapterDriverVersion)
+{
+  aAdapterDriverVersion = mDriverVersion2;
+  return NS_OK;
+}
+
+/* readonly attribute DOMString adapterDriverDate2; */
+NS_IMETHODIMP
+GfxInfo::GetAdapterDriverDate2(nsAString & aAdapterDriverDate)
+{
+  aAdapterDriverDate = mDriverDate2;
+  return NS_OK;
+}
+
 /* readonly attribute unsigned long adapterVendorID; */
 NS_IMETHODIMP
 GfxInfo::GetAdapterVendorID(PRUint32 *aAdapterVendorID)
@@ -525,6 +620,14 @@ GfxInfo::GetAdapterVendorID(PRUint32 *aAdapterVendorID)
   return NS_OK;
 }
 
+/* readonly attribute unsigned long adapterVendorID2; */
+NS_IMETHODIMP
+GfxInfo::GetAdapterVendorID2(PRUint32 *aAdapterVendorID)
+{
+  *aAdapterVendorID = mAdapterVendorID2;
+  return NS_OK;
+}
+
 /* readonly attribute unsigned long adapterDeviceID; */
 NS_IMETHODIMP
 GfxInfo::GetAdapterDeviceID(PRUint32 *aAdapterDeviceID)
@@ -533,6 +636,22 @@ GfxInfo::GetAdapterDeviceID(PRUint32 *aAdapterDeviceID)
   return NS_OK;
 }
 
+/* readonly attribute unsigned long adapterDeviceID2; */
+NS_IMETHODIMP
+GfxInfo::GetAdapterDeviceID2(PRUint32 *aAdapterDeviceID)
+{
+  *aAdapterDeviceID = mAdapterDeviceID2;
+  return NS_OK;
+}
+
+/* readonly attribute boolean isGPU2Active; */
+NS_IMETHODIMP
+GfxInfo::GetIsGPU2Active(PRBool* aIsGPU2Active)
+{
+  *aIsGPU2Active = mIsGPU2Active;
+  return NS_OK;
+}
+
 #if defined(MOZ_CRASHREPORTER)
 /* Cisco's VPN software can cause corruption of the floating point state.
  * Make a note of this in our crash reports so that some weird crashes
@@ -578,6 +697,7 @@ GfxInfo::AddCrashReportAnnotations()
   /* AppendPrintf only supports 32 character strings, mrghh. */
   note.AppendPrintf("AdapterVendorID: %04x, ", vendorID);
   note.AppendPrintf("AdapterDeviceID: %04x, ", deviceID);
+  note.AppendPrintf("AdapterSubsysID: %08x, ", mAdapterSubsysID);
   note.AppendPrintf("AdapterDriverVersion: ");
   note.Append(NS_LossyConvertUTF16toASCII(adapterDriverVersionString));
 
@@ -590,6 +710,20 @@ GfxInfo::AddCrashReportAnnotations()
   }
   note.Append("\n");
 
+  if (mHasDualGPU) {
+    PRUint32 deviceID2, vendorID2;
+    nsAutoString adapterDriverVersionString2;
+
+    note.Append("Has dual GPUs. GPU #2: ");
+    GetAdapterDeviceID2(&deviceID2);
+    GetAdapterVendorID2(&vendorID2);
+    GetAdapterDriverVersion2(adapterDriverVersionString2);
+    note.AppendPrintf("AdapterVendorID2: %04x, ", vendorID2);
+    note.AppendPrintf("AdapterDeviceID2: %04x, ", deviceID2);
+    note.AppendPrintf("AdapterSubsysID2: %08x, ", mAdapterSubsysID2);
+    note.AppendPrintf("AdapterDriverVersion2: ");
+    note.Append(NS_LossyConvertUTF16toASCII(adapterDriverVersionString2));
+  }
   CrashReporter::AppendAppNotesToCrashReport(note);
 
 #endif
@@ -650,17 +784,6 @@ static const PRUint32 deviceFamilyIntelGMAX3000[] = {
     0
 };
 
-// see bug 595364 comment 10
-static const PRUint32 deviceFamilyIntelBlockDirect2D[] = {
-    0x2982, /* IntelG35_1 */
-    0x2983, /* IntelG35_2 */
-    0x2A02, /* IntelGL960_1 */
-    0x2A03, /* IntelGL960_2 */
-    0x2A12, /* IntelGM965_1 */
-    0x2A13, /* IntelGM965_2 */
-    0
-};
-
 static const PRUint32 deviceFamilyIntelGMAX4500HD[] = {
     0x2A42, /* IntelGMA4500MHD_1 */
     0x2A43, /* IntelGMA4500MHD_2 */
@@ -703,7 +826,6 @@ static const PRUint32 deviceFamilyNvidiaBlockD3D9Layers[] = {
     0x0222, /* NV44 [GeForce 6200 A-LE (TM)] */
     0x0240, /* C51PV [GeForce 6150 (TM)] */
     0x0241, /* C51 [GeForce 6150 LE (TM)] */
-    0x0242, /* C51G [GeForce 6100 (TM)] */
     0x0244, /* C51 [Geforce Go 6150 (TM)] */
     0x0245, /* C51 [Quadro NVS 210S/GeForce 6150LE (TM)] */
     0x0247, /* C51 [GeForce Go 6100 (TM)] */
@@ -782,14 +904,6 @@ static const GfxDriverInfo gDriverInfo[] = {
    * Intel entries
    */
 
-  /*
-   * Implement special Direct2D blocklist from bug 595364
-   */
-  GfxDriverInfo( DRIVER_OS_ALL,
-    vendorIntel, (GfxDeviceFamily) deviceFamilyIntelBlockDirect2D,
-    nsIGfxInfo::FEATURE_DIRECT2D, nsIGfxInfo::FEATURE_BLOCKED_DEVICE,
-    DRIVER_LESS_THAN, allDriverVersions ),
-
   /* implement the blocklist from bug 594877
    * Block all features on any drivers before this, as there's a crash when a MS Hotfix is installed.
    * The crash itself is Direct2D-related, but for safety we block all features.
