@@ -43,16 +43,20 @@
 #include "nscore.h"
 #include "nsWinGesture.h"
 #include "nsUXThemeData.h"
-#include "nsIPrefBranch.h"
-#include "nsIPrefService.h"
-#include "nsIServiceManager.h"
 #include "nsIDOMSimpleGestureEvent.h"
 #include "nsGUIEvent.h"
+#include "mozilla/Preferences.h"
+
+using namespace mozilla;
 
 #ifndef M_PI
 #define M_PI 3.14159265358979323846
 #endif
 
+#ifdef PR_LOGGING
+extern PRLogModuleInfo* gWindowsLog;
+#endif
+
 const PRUnichar nsWinGesture::kGestureLibraryName[] =  L"user32.dll";
 HMODULE nsWinGesture::sLibraryHandle = nsnull;
 nsWinGesture::GetGestureInfoPtr nsWinGesture::getGestureInfo = nsnull;
@@ -86,9 +90,6 @@ nsWinGesture::nsWinGesture() :
 
 PRBool nsWinGesture::InitLibrary()
 {
-#ifdef WINCE
-  return PR_FALSE;
-#else
   if (getGestureInfo) {
     return PR_TRUE;
   } else if (sLibraryHandle) {
@@ -143,20 +144,10 @@ PRBool nsWinGesture::InitLibrary()
 
   // Check to see if we want single finger gesture input. Only do this once
   // for the app so we don't have to look it up on every window create.
-  nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
-  if (prefs) {
-    nsCOMPtr<nsIPrefBranch> prefBranch;
-    prefs->GetBranch(0, getter_AddRefs(prefBranch));
-    if (prefBranch) {
-      PRBool flag;
-      if (NS_SUCCEEDED(prefBranch->GetBoolPref("gestures.enable_single_finger_input", &flag))
-          && flag)
-        gEnableSingleFingerPanEvents = PR_TRUE;
-    }
-  }
+  gEnableSingleFingerPanEvents =
+    Preferences::GetBool("gestures.enable_single_finger_input", PR_FALSE);
 
   return PR_TRUE;
-#endif
 }
 
 #define GCOUNT 5
@@ -487,8 +478,9 @@ nsWinGesture::ProcessPanMessage(HWND hWnd, WPARAM wParam, LPARAM lParam)
 #ifdef DBG_jimm
         PRInt32 deltaX = mPanIntermediate.x - coord.x;
         PRInt32 deltaY = mPanIntermediate.y - coord.y;
-        printf("coordX=%d coordY=%d deltaX=%d deltaY=%d x:%d y:%d\n", coord.x,
-          coord.y, deltaX, deltaY, mXAxisFeedback, mYAxisFeedback);
+        PR_LOG(gWindowsLog, PR_LOG_ALWAYS, 
+               ("coordX=%d coordY=%d deltaX=%d deltaY=%d x:%d y:%d\n", coord.x,
+                coord.y, deltaX, deltaY, mXAxisFeedback, mYAxisFeedback));
 #endif
 
         mPixelScrollDelta.x = mPanIntermediate.x - coord.x;
