@@ -45,7 +45,6 @@
 #include "nsCOMPtr.h"
 #include "nsString.h"
 #include "nsMargin.h"
-#include "nsILookAndFeel.h"
 #include "nsWidgetAtoms.h"
 #include "nsEventStates.h"
 #include "nsTArray.h"
@@ -145,10 +144,17 @@ class nsNativeTheme : public nsITimerCallback
     return CheckBooleanAttr(aFrame, nsWidgetAtoms::checked);
   }
 
+  PRBool IsSelectedButton(nsIFrame* aFrame) {
+    return CheckBooleanAttr(aFrame, nsWidgetAtoms::checked) ||
+           CheckBooleanAttr(aFrame, nsWidgetAtoms::selected);
+  }
+
   PRBool IsOpenButton(nsIFrame* aFrame) {
     return CheckBooleanAttr(aFrame, nsWidgetAtoms::open);
   }
 
+  PRBool IsPressedButton(nsIFrame* aFrame);
+
   // treeheadercell:
   TreeSortDirection GetTreeSortDirection(nsIFrame* aFrame);
   PRBool IsLastTreeHeaderCell(nsIFrame* aFrame);
@@ -156,7 +162,6 @@ class nsNativeTheme : public nsITimerCallback
   // tab:
   PRBool IsBottomTab(nsIFrame* aFrame);
   PRBool IsFirstTab(nsIFrame* aFrame);
-  PRBool IsLastTab(nsIFrame* aFrame);
   
   PRBool IsHorizontal(nsIFrame* aFrame);
 
@@ -187,6 +192,9 @@ class nsNativeTheme : public nsITimerCallback
   PRBool QueueAnimatedContentForRefresh(nsIContent* aContent,
                                         PRUint32 aMinimumFrameRate);
 
+  nsIFrame* GetAdjacentSiblingFrameWithSameAppearance(nsIFrame* aFrame,
+                                                      PRBool aNextSibling);
+
  private:
   PRUint32 mAnimatedContentTimeout;
   nsCOMPtr<nsITimer> mAnimatedContentTimer;
