@@ -47,7 +47,6 @@
 #include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIWindowWatcher.h"
-#include "nsIDOMWindowInternal.h"
 #include "nsISupportsArray.h"
 #include "prprf.h"
 
@@ -160,9 +159,8 @@ nsPrintOptions::ShowPrintSetupDialog(nsIPrintSettings *aPS)
       do_GetService(NS_WINDOWWATCHER_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIDOMWindow> active;
-  wwatch->GetActiveWindow(getter_AddRefs(active));
-  nsCOMPtr<nsIDOMWindowInternal> parent = do_QueryInterface(active);
+  nsCOMPtr<nsIDOMWindow> parent;
+  wwatch->GetActiveWindow(getter_AddRefs(parent));
   // null |parent| is non-fatal
 
   nsCOMPtr<nsIDOMWindow> newWindow;
@@ -308,7 +306,7 @@ nsPrintOptions::ReadPrefs(nsIPrintSettings* aPS, const nsAString& aPrinterName,
 
 #define GETSTRPREF(_prefname, _retval)                  \
   NS_SUCCEEDED(                                         \
-    Preferences::GetChar(                               \
+    Preferences::GetString(                             \
       GetPrefName(_prefname, aPrinterName), _retval     \
     )                                                   \
   )
@@ -623,7 +621,7 @@ nsPrintOptions::WritePrefs(nsIPrintSettings *aPS, const nsAString& aPrinterName,
       DUMP_DBL(kWriteStr, kPrintPaperHeight, height);
       WritePrefDouble(GetPrefName(kPrintPaperHeight, aPrinterName), height);
       DUMP_STR(kWriteStr, kPrintPaperName, name);
-      Preferences::SetChar(GetPrefName(kPrintPaperName, aPrinterName), name);
+      Preferences::SetString(GetPrefName(kPrintPaperName, aPrinterName), name);
     }
   }
 
@@ -652,48 +650,48 @@ nsPrintOptions::WritePrefs(nsIPrintSettings *aPS, const nsAString& aPrinterName,
   if (aFlags & nsIPrintSettings::kInitSaveHeaderLeft) {
     if (NS_SUCCEEDED(aPS->GetHeaderStrLeft(&uStr))) {
       DUMP_STR(kWriteStr, kPrintHeaderStrLeft, uStr);
-      Preferences::SetChar(GetPrefName(kPrintHeaderStrLeft, aPrinterName),
-                           uStr);
+      Preferences::SetString(GetPrefName(kPrintHeaderStrLeft, aPrinterName),
+                             uStr);
     }
   }
 
   if (aFlags & nsIPrintSettings::kInitSaveHeaderCenter) {
     if (NS_SUCCEEDED(aPS->GetHeaderStrCenter(&uStr))) {
       DUMP_STR(kWriteStr, kPrintHeaderStrCenter, uStr);
-      Preferences::SetChar(GetPrefName(kPrintHeaderStrCenter, aPrinterName),
-                           uStr);
+      Preferences::SetString(GetPrefName(kPrintHeaderStrCenter, aPrinterName),
+                             uStr);
     }
   }
 
   if (aFlags & nsIPrintSettings::kInitSaveHeaderRight) {
     if (NS_SUCCEEDED(aPS->GetHeaderStrRight(&uStr))) {
       DUMP_STR(kWriteStr, kPrintHeaderStrRight, uStr);
-      Preferences::SetChar(GetPrefName(kPrintHeaderStrRight, aPrinterName),
-                           uStr);
+      Preferences::SetString(GetPrefName(kPrintHeaderStrRight, aPrinterName),
+                             uStr);
     }
   }
 
   if (aFlags & nsIPrintSettings::kInitSaveFooterLeft) {
     if (NS_SUCCEEDED(aPS->GetFooterStrLeft(&uStr))) {
       DUMP_STR(kWriteStr, kPrintFooterStrLeft, uStr);
-      Preferences::SetChar(GetPrefName(kPrintFooterStrLeft, aPrinterName),
-                           uStr);
+      Preferences::SetString(GetPrefName(kPrintFooterStrLeft, aPrinterName),
+                             uStr);
     }
   }
 
   if (aFlags & nsIPrintSettings::kInitSaveFooterCenter) {
     if (NS_SUCCEEDED(aPS->GetFooterStrCenter(&uStr))) {
       DUMP_STR(kWriteStr, kPrintFooterStrCenter, uStr);
-      Preferences::SetChar(GetPrefName(kPrintFooterStrCenter, aPrinterName),
-                           uStr);
+      Preferences::SetString(GetPrefName(kPrintFooterStrCenter, aPrinterName),
+                             uStr);
     }
   }
 
   if (aFlags & nsIPrintSettings::kInitSaveFooterRight) {
     if (NS_SUCCEEDED(aPS->GetFooterStrRight(&uStr))) {
       DUMP_STR(kWriteStr, kPrintFooterStrRight, uStr);
-      Preferences::SetChar(GetPrefName(kPrintFooterStrRight, aPrinterName),
-                           uStr);
+      Preferences::SetString(GetPrefName(kPrintFooterStrRight, aPrinterName),
+                             uStr);
     }
   }
 
@@ -728,7 +726,7 @@ nsPrintOptions::WritePrefs(nsIPrintSettings *aPS, const nsAString& aPrinterName,
   if (aFlags & nsIPrintSettings::kInitSavePlexName) {
     if (NS_SUCCEEDED(aPS->GetPlexName(&uStr))) {
       DUMP_STR(kWriteStr, kPrintPlexName, uStr);
-      Preferences::SetChar(GetPrefName(kPrintPlexName, aPrinterName), uStr);
+      Preferences::SetString(GetPrefName(kPrintPlexName, aPrinterName), uStr);
     }
   }
 
@@ -743,15 +741,15 @@ nsPrintOptions::WritePrefs(nsIPrintSettings *aPS, const nsAString& aPrinterName,
   if (aFlags & nsIPrintSettings::kInitSaveColorspace) {
     if (NS_SUCCEEDED(aPS->GetColorspace(&uStr))) {
       DUMP_STR(kWriteStr, kPrintColorspace, uStr);
-      Preferences::SetChar(GetPrefName(kPrintColorspace, aPrinterName), uStr);
+      Preferences::SetString(GetPrefName(kPrintColorspace, aPrinterName), uStr);
     }
   }
 
   if (aFlags & nsIPrintSettings::kInitSaveResolutionName) {
     if (NS_SUCCEEDED(aPS->GetResolutionName(&uStr))) {
       DUMP_STR(kWriteStr, kPrintResolutionName, uStr);
-      Preferences::SetChar(GetPrefName(kPrintResolutionName, aPrinterName),
-                           uStr);
+      Preferences::SetString(GetPrefName(kPrintResolutionName, aPrinterName),
+                             uStr);
     }
   }
 
@@ -772,7 +770,7 @@ nsPrintOptions::WritePrefs(nsIPrintSettings *aPS, const nsAString& aPrinterName,
   if (aFlags & nsIPrintSettings::kInitSavePrintCommand) {
     if (NS_SUCCEEDED(aPS->GetPrintCommand(&uStr))) {
       DUMP_STR(kWriteStr, kPrintCommand, uStr);
-      Preferences::SetChar(GetPrefName(kPrintCommand, aPrinterName), uStr);
+      Preferences::SetString(GetPrefName(kPrintCommand, aPrinterName), uStr);
     }
   }
 
@@ -781,7 +779,7 @@ nsPrintOptions::WritePrefs(nsIPrintSettings *aPS, const nsAString& aPrinterName,
       && aPrinterName.IsEmpty()) {
     if (NS_SUCCEEDED(aPS->GetPrinterName(&uStr))) {
       DUMP_STR(kWriteStr, kPrinterName, uStr);
-      Preferences::SetChar(kPrinterName, uStr);
+      Preferences::SetString(kPrinterName, uStr);
     }
   }
 
@@ -795,7 +793,7 @@ nsPrintOptions::WritePrefs(nsIPrintSettings *aPS, const nsAString& aPrinterName,
   if (aFlags & nsIPrintSettings::kInitSaveToFileName) {
     if (NS_SUCCEEDED(aPS->GetToFileName(&uStr))) {
       DUMP_STR(kWriteStr, kPrintToFileName, uStr);
-      Preferences::SetChar(GetPrefName(kPrintToFileName, aPrinterName), uStr);
+      Preferences::SetString(GetPrefName(kPrintToFileName, aPrinterName), uStr);
     }
   }
 
@@ -906,7 +904,7 @@ nsPrintOptions::GetDefaultPrinterName(PRUnichar * *aDefaultPrinterName)
 
   // Look up the printer from the last print job
   nsAutoString lastPrinterName;
-  Preferences::GetChar(kPrinterName, &lastPrinterName);
+  Preferences::GetString(kPrinterName, &lastPrinterName);
   if (!lastPrinterName.IsEmpty()) {
     // Verify it's still a valid printer
     nsCOMPtr<nsIStringEnumerator> printers;
@@ -1096,7 +1094,7 @@ nsPrintOptions::ReadPrefDouble(const char * aPrefId, double& aVal)
   NS_ENSURE_ARG_POINTER(aPrefId);
 
   nsCAutoString str;
-  nsresult rv = Preferences::GetChar(aPrefId, &str);
+  nsresult rv = Preferences::GetCString(aPrefId, &str);
   if (NS_SUCCEEDED(rv) && !str.IsEmpty()) {
     aVal = atof(str.get());
   }
@@ -1111,7 +1109,7 @@ nsPrintOptions::WritePrefDouble(const char * aPrefId, double aVal)
   nsPrintfCString str("%6.2f", aVal);
   NS_ENSURE_TRUE(!str.IsEmpty(), NS_ERROR_FAILURE);
 
-  return Preferences::SetChar(aPrefId, str);
+  return Preferences::SetCString(aPrefId, str);
 }
 
 void
@@ -1119,9 +1117,9 @@ nsPrintOptions::ReadInchesToTwipsPref(const char * aPrefId, PRInt32& aTwips,
                                       const char * aMarginPref)
 {
   nsCAutoString str;
-  nsresult rv = Preferences::GetChar(aPrefId, &str);
+  nsresult rv = Preferences::GetCString(aPrefId, &str);
   if (NS_FAILED(rv) || str.IsEmpty()) {
-    rv = Preferences::GetChar(aMarginPref, &str);
+    rv = Preferences::GetCString(aMarginPref, &str);
   }
   if (NS_SUCCEEDED(rv) && !str.IsEmpty()) {
     nsAutoString justStr;
@@ -1143,7 +1141,7 @@ nsPrintOptions::WriteInchesFromTwipsPref(const char * aPrefId, PRInt32 aTwips)
   nsCAutoString inchesStr;
   inchesStr.AppendFloat(inches);
 
-  Preferences::SetChar(aPrefId, inchesStr);
+  Preferences::SetCString(aPrefId, inchesStr);
 }
 
 void
@@ -1175,7 +1173,7 @@ nsPrintOptions::ReadJustification(const char * aPrefId, PRInt16& aJust,
 {
   aJust = aInitValue;
   nsAutoString justStr;
-  if (NS_SUCCEEDED(Preferences::GetChar(aPrefId, &justStr))) {
+  if (NS_SUCCEEDED(Preferences::GetString(aPrefId, &justStr))) {
     if (justStr.EqualsASCII(kJustRight)) {
       aJust = nsIPrintSettings::kJustRight;
     } else if (justStr.EqualsASCII(kJustCenter)) {
@@ -1192,15 +1190,15 @@ nsPrintOptions::WriteJustification(const char * aPrefId, PRInt16 aJust)
 {
   switch (aJust) {
     case nsIPrintSettings::kJustLeft:
-      Preferences::SetChar(aPrefId, kJustLeft);
+      Preferences::SetCString(aPrefId, kJustLeft);
       break;
 
     case nsIPrintSettings::kJustCenter:
-      Preferences::SetChar(aPrefId, kJustCenter);
+      Preferences::SetCString(aPrefId, kJustCenter);
       break;
 
     case nsIPrintSettings::kJustRight:
-      Preferences::SetChar(aPrefId, kJustRight);
+      Preferences::SetCString(aPrefId, kJustRight);
       break;
   } //switch
 }
