@@ -40,6 +40,10 @@
 
 #include "nsDebugImpl.h"
 #include "nsDebug.h"
+#ifdef MOZ_CRASHREPORTER
+# include "nsExceptionHandler.h"
+#endif
+#include "nsStringGlue.h"
 #include "prprf.h"
 #include "prlog.h"
 #include "prinit.h"
@@ -50,12 +54,11 @@
 #include "prenv.h"
 #include "pratom.h"
 
-#if defined(XP_BEOS)
-/* For DEBUGGER macros */
-#include <Debug.h>
+#ifdef ANDROID
+#include <android/log.h>
 #endif
 
-#if defined(XP_UNIX) || defined(_WIN32) || defined(XP_OS2) || defined(XP_BEOS)
+#if defined(XP_UNIX) || defined(_WIN32) || defined(XP_OS2)
 /* for abort() and getenv() */
 #include <stdlib.h>
 #endif
@@ -72,9 +75,14 @@
 #include "nsString.h"
 #endif
 
+#include "mozilla/mozalloc_abort.h"
+
 static void
 Abort(const char *aMsg);
 
+static void
+RealBreak();
+
 static void
 Break(const char *aMsg);
 
@@ -304,6 +312,10 @@ NS_DebugBreak(PRUint32 aSeverity, const char *aStr, const char *aExpr,
      fprintf(stderr, "\07");
 #endif
 
+#ifdef ANDROID
+   __android_log_print(ANDROID_LOG_INFO, "Gecko", "%s", buf.buffer);
+#endif
+
    // Write the message to stderr
    fprintf(stderr, "%s\n", buf.buffer);
    fflush(stderr);
@@ -316,14 +328,27 @@ NS_DebugBreak(PRUint32 aSeverity, const char *aStr, const char *aExpr,
      Break(buf.buffer);
      return;
 
-   case NS_DEBUG_ABORT:
+   case NS_DEBUG_ABORT: {
+#if defined(MOZ_CRASHREPORTER)
+     nsCString note("xpcom_runtime_abort(");
+     note += buf.buffer;
+     note += ")";
+     CrashReporter::AppendAppNotesToCrashReport(note);
+#endif  // MOZ_CRASHREPORTER
+
+#if defined(DEBUG) && defined(_WIN32)
+     RealBreak();
+#endif
+#ifdef DEBUG
      nsTraceRefcntImpl::WalkTheStack(stderr);
+#endif
      Abort(buf.buffer);
      return;
    }
+   }
 
    // Now we deal with assertions
-   PR_AtomicIncrement(&gAssertionCount);
+   PR_ATOMIC_INCREMENT(&gAssertionCount);
 
    switch (GetAssertBehavior()) {
    case NS_ASSERT_WARN:
@@ -351,41 +376,49 @@ NS_DebugBreak(PRUint32 aSeverity, const char *aStr, const char *aExpr,
      return;
 
    case NS_ASSERT_TRAP:
+   case NS_ASSERT_UNINITIALIZED: // Default to "trap" behavior
      Break(buf.buffer);
+     return;
    }   
 }
 
 static void
 Abort(const char *aMsg)
+{
+  mozalloc_abort(aMsg);
+}
+
+static void
+RealBreak()
 {
 #if defined(_WIN32)
-
-#ifndef WINCE
-  //This should exit us
-  raise(SIGABRT);
+  ::DebugBreak();
+#elif defined(XP_OS2)
+   asm("int $3");
+#elif defined(XP_MACOSX)
+   raise(SIGTRAP);
+#elif defined(__GNUC__) && (defined(__i386__) || defined(__i386) || defined(__x86_64__))
+   asm("int $3");
+#elif defined(__arm__)
+   asm(
+#ifdef __ARM_ARCH_4T__
+/* ARMv4T doesn't support the BKPT instruction, so if the compiler target
+ * is ARMv4T, we want to ensure the assembler will understand that ARMv5T
+ * instruction, while keeping the resulting object tagged as ARMv4T.
+ */
+       ".arch armv5t\n"
+       ".object_arch armv4t\n"
 #endif
-  //If we are ignored exit this way..
-  _exit(3);
-#elif defined(XP_UNIX)
-  PR_Abort();
-#elif defined(XP_BEOS)
-  {
-#ifndef DEBUG_cls
-	DEBUGGER(aMsg);
-#endif
-  }
+       "BKPT #0");
+#elif defined(SOLARIS)
+#if defined(__i386__) || defined(__i386) || defined(__x86_64__)
+   asm("int $3");
 #else
-  // Don't know how to abort on this platform! call Break() instead
-  Break(aMsg);
+   raise(SIGTRAP);
+#endif
+#else
+#warning do not know how to break on this platform
 #endif
-
-  // Still haven't aborted?  Try dereferencing null.
-  // (Written this way to lessen the likelihood of it being optimized away.)
-  gAssertionCount += *((PRInt32 *) 0); // TODO annotation saying we know 
-                                       // this is crazy
-
-  // Still haven't aborted?  Try _exit().
-  PR_ProcessExit(127);
 }
 
 // Abort() calls this function, don't call it!
@@ -393,7 +426,6 @@ static void
 Break(const char *aMsg)
 {
 #if defined(_WIN32)
-#ifndef WINCE // we really just want to crash for now
   static int ignoreDebugger;
   if (!ignoreDebugger) {
     const char *shouldIgnoreDebugger = getenv("XPCOM_DEBUG_DLG");
@@ -448,9 +480,7 @@ Break(const char *aMsg)
     }
   }
 
-  ::DebugBreak();
-
-#endif // WINCE
+  RealBreak();
 #elif defined(XP_OS2)
    char msg[1200];
    PR_snprintf(msg, sizeof(msg),
@@ -472,25 +502,27 @@ Break(const char *aMsg)
    if (( code == MBID_ENTER ) || (code == MBID_ERROR))
      return;
 
-   asm("int $3");
-#elif defined(XP_BEOS)
-   DEBUGGER(aMsg);
+   RealBreak();
 #elif defined(XP_MACOSX)
    /* Note that we put this Mac OS X test above the GNUC/x86 test because the
     * GNUC/x86 test is also true on Intel Mac OS X and we want the PPC/x86
     * impls to be the same.
     */
-   raise(SIGTRAP);
+   RealBreak();
 #elif defined(__GNUC__) && (defined(__i386__) || defined(__i386) || defined(__x86_64__))
-   asm("int $3");
+   RealBreak();
+#elif defined(__arm__)
+   RealBreak();
+#elif defined(SOLARIS)
+   RealBreak();
 #else
-   // don't know how to break on this platform
+#warning do not know how to break on this platform
 #endif
 }
 
 static const nsDebugImpl kImpl;
 
-NS_METHOD
+nsresult
 nsDebugImpl::Create(nsISupports* outer, const nsIID& aIID, void* *aInstancePtr)
 {
   NS_ENSURE_NO_AGGREGATION(outer);
@@ -501,7 +533,7 @@ nsDebugImpl::Create(nsISupports* outer, const nsIID& aIID, void* *aInstancePtr)
 
 ////////////////////////////////////////////////////////////////////////////////
 
-NS_COM nsresult
+nsresult
 NS_ErrorAccordingToNSPR()
 {
     PRErrorCode err = PR_GetError();
@@ -526,3 +558,12 @@ NS_ErrorAccordingToNSPR()
 
 ////////////////////////////////////////////////////////////////////////////////
 
+#ifdef XP_WIN
+PRBool sXPCOMHasLoadedNewDLLs = PR_FALSE;
+
+NS_EXPORT void
+NS_SetHasLoadedNewDLLs()
+{
+  sXPCOMHasLoadedNewDLLs = PR_TRUE;
+}
+#endif
