@@ -20,6 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Philipp Kewisch <mozilla@kewis.ch>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -43,7 +44,7 @@
 #include "prprf.h"
 #if defined(XP_WIN)
 #include <windows.h>
-#elif defined(XP_MACOSX)
+#elif defined(MOZ_WIDGET_COCOA)
 #include <CoreServices/CoreServices.h>
 #elif defined(MOZ_WIDGET_GTK2)
 #include <gtk/gtk.h>
@@ -76,6 +77,8 @@ struct ManifestDirective
 
   bool ischrome;
 
+  bool allowbootstrap;
+
   // The platform/contentaccessible flags only apply to content directives.
   bool contentflags;
 
@@ -92,31 +95,31 @@ struct ManifestDirective
   bool isContract;
 };
 static const ManifestDirective kParsingTable[] = {
-  { "manifest", 1, false, true, false,
+  { "manifest",         1, false, true, true, false,
     &nsComponentManagerImpl::ManifestManifest, NULL },
-  { "binary-component", 1, true, false, false,
+  { "binary-component", 1, true, false, false, false,
     &nsComponentManagerImpl::ManifestBinaryComponent, NULL },
-  { "interfaces",       1, true, false, false,
+  { "interfaces",       1, true, false, false, false,
     &nsComponentManagerImpl::ManifestXPT, NULL },
-  { "component",        2, true, false, false,
+  { "component",        2, true, false, false, false,
     &nsComponentManagerImpl::ManifestComponent, NULL },
-  { "contract",         2, true, false, false,
+  { "contract",         2, true, false, false, false,
     &nsComponentManagerImpl::ManifestContract, NULL, true},
-  { "category",         3, true, false, false,
+  { "category",         3, true, false, false, false,
     &nsComponentManagerImpl::ManifestCategory, NULL },
-  { "content",          2, true, true,  true,
+  { "content",          2, true, true, true,  true,
     NULL, &nsChromeRegistry::ManifestContent },
-  { "locale",           3, true, true,  false,
+  { "locale",           3, true, true, true,  false,
     NULL, &nsChromeRegistry::ManifestLocale },
-  { "skin",             3, false, true,  false,
+  { "skin",             3, false, true, true,  false,
     NULL, &nsChromeRegistry::ManifestSkin },
-  { "overlay",          2, true, true,  false,
+  { "overlay",          2, true, true, false,  false,
     NULL, &nsChromeRegistry::ManifestOverlay },
-  { "style",            2, false, true,  false,
+  { "style",            2, false, true, false,  false,
     NULL, &nsChromeRegistry::ManifestStyle },
-  { "override",         2, true, true,  false,
+  { "override",         2, true, true, true,  false,
     NULL, &nsChromeRegistry::ManifestOverride },
-  { "resource",         2, true, true,  false,
+  { "resource",         2, true, true, false,  false,
     NULL, &nsChromeRegistry::ManifestResource }
 };
 
@@ -428,6 +431,7 @@ ParseManifestCommon(NSLocationType aType, nsILocalFile* aFile,
   NS_NAMED_LITERAL_STRING(kContentAccessible, "contentaccessible");
   NS_NAMED_LITERAL_STRING(kApplication, "application");
   NS_NAMED_LITERAL_STRING(kAppVersion, "appversion");
+  NS_NAMED_LITERAL_STRING(kGeckoVersion, "platformversion");
   NS_NAMED_LITERAL_STRING(kOs, "os");
   NS_NAMED_LITERAL_STRING(kOsVersion, "osversion");
   NS_NAMED_LITERAL_STRING(kABI, "abi");
@@ -437,6 +441,7 @@ ParseManifestCommon(NSLocationType aType, nsILocalFile* aFile,
 
   nsAutoString appID;
   nsAutoString appVersion;
+  nsAutoString geckoVersion;
   nsAutoString osTarget;
   nsAutoString abi;
 
@@ -450,7 +455,11 @@ ParseManifestCommon(NSLocationType aType, nsILocalFile* aFile,
     rv = xapp->GetVersion(s);
     if (NS_SUCCEEDED(rv))
       CopyUTF8toUTF16(s, appVersion);
-    
+
+    rv = xapp->GetPlatformVersion(s);
+    if (NS_SUCCEEDED(rv))
+      CopyUTF8toUTF16(s, geckoVersion);
+
     nsCOMPtr<nsIXULRuntime> xruntime (do_QueryInterface(xapp));
     if (xruntime) {
       rv = xruntime->GetOS(s);
@@ -477,7 +486,7 @@ ParseManifestCommon(NSLocationType aType, nsILocalFile* aFile,
                                          info.dwMajorVersion,
                                          info.dwMinorVersion);
   }
-#elif defined(XP_MACOSX)
+#elif defined(MOZ_WIDGET_COCOA)
   SInt32 majorVersion, minorVersion;
   if ((Gestalt(gestaltSystemVersionMajor, &majorVersion) == noErr) &&
       (Gestalt(gestaltSystemVersionMinor, &minorVersion) == noErr)) {
@@ -538,13 +547,22 @@ ParseManifestCommon(NSLocationType aType, nsILocalFile* aFile,
 	break;
       }
     }
+
     if (!directive) {
       LogMessageWithContext(aFile, aPath, line,
                             "Ignoring unrecognized chrome manifest directive '%s'.",
                             token);
       continue;
     }
-    if (directive->componentonly && NS_COMPONENT_LOCATION != aType) {
+
+    if (!directive->allowbootstrap && NS_BOOTSTRAPPED_LOCATION == aType) {
+      LogMessageWithContext(aFile, aPath, line,
+                            "Bootstrapped manifest not allowed to use '%s' directive.",
+                            token);
+      continue;
+    }
+
+    if (directive->componentonly && NS_SKIN_LOCATION == aType) {
       LogMessageWithContext(aFile, aPath, line,
                             "Skin manifest not allowed to use '%s' directive.",
                             token);
@@ -565,6 +583,7 @@ ParseManifestCommon(NSLocationType aType, nsILocalFile* aFile,
 
     bool ok = true;
     TriState stAppVersion = eUnspecified;
+    TriState stGeckoVersion = eUnspecified;
     TriState stApp = eUnspecified;
     TriState stOsVersion = eUnspecified;
     TriState stOs = eUnspecified;
@@ -580,7 +599,8 @@ ParseManifestCommon(NSLocationType aType, nsILocalFile* aFile,
           CheckStringFlag(kOs, wtoken, osTarget, stOs) ||
           CheckStringFlag(kABI, wtoken, abi, stABI) ||
           CheckVersionFlag(kOsVersion, wtoken, osVersion, stOsVersion) ||
-          CheckVersionFlag(kAppVersion, wtoken, appVersion, stAppVersion))
+          CheckVersionFlag(kAppVersion, wtoken, appVersion, stAppVersion) ||
+          CheckVersionFlag(kGeckoVersion, wtoken, geckoVersion, stGeckoVersion))
         continue;
 
       if (directive->contentflags &&
@@ -605,6 +625,7 @@ ParseManifestCommon(NSLocationType aType, nsILocalFile* aFile,
     if (!ok ||
         stApp == eBad ||
         stAppVersion == eBad ||
+        stGeckoVersion == eBad ||
         stOs == eBad ||
         stOsVersion == eBad ||
         stABI == eBad)
