@@ -1887,7 +1887,7 @@ NS_IMETHODIMP nsVariant::GetAsISupports(nsISupports **_retval)
 }
 
 /* jsval getAsJSVal() */
-NS_IMETHODIMP nsVariant::GetAsJSVal(jsval *_retval)
+NS_IMETHODIMP nsVariant::GetAsJSVal(JS::Value *_retval)
 {
     // Can only get the jsval from an XPCVariant.
     return NS_ERROR_CANNOT_CONVERT_DATA;
