@@ -99,6 +99,22 @@ def paramAttlistToIDL(attlist):
     return '[%s] ' % ', '.join(["%s%s" % (name, value is not None and ' (%s)' % value or '')
                                 for name, value, aloc in sorted])
 
+def unaliasType(t):
+    while t.kind == 'typedef':
+        t = t.realtype
+    assert t is not None
+    return t
+
+def getBuiltinOrNativeTypeName(t):
+    t = unaliasType(t)
+    if t.kind == 'builtin':
+        return t.name
+    elif t.kind == 'native':
+        assert t.specialtype is not None
+        return '[%s]' % t.specialtype
+    else:
+        return None
+
 class BuiltinLocation(object):
     def get(self):
         return "<builtin type>"
@@ -266,6 +282,7 @@ class Include(object):
             self.IDL.resolve(parent.incdirs, parent.parser)
             for type in self.IDL.getNames():
                 parent.setName(type)
+            parent.deps.extend(self.IDL.deps)
             return
 
         raise IDLError("File '%s' not found" % self.filename, self.location)
@@ -273,6 +290,7 @@ class Include(object):
 class IDL(object):
     def __init__(self, productions):
         self.productions = productions
+        self.deps = []
 
     def setName(self, object):
         self.namemap.set(object)
@@ -304,6 +322,12 @@ class IDL(object):
             if p.kind == 'include':
                 yield p
 
+    def needsJSTypes(self):
+        for p in self.productions:
+            if p.kind == 'interface' and p.needsJSTypes():
+                return True
+        return False
+
 class CDATA(object):
     kind = 'cdata'
     _re = re.compile(r'\n+')
@@ -318,6 +342,9 @@ class CDATA(object):
     def __str__(self):
         return "cdata: %s\n\t%r\n" % (self.location.get(), self.data)
 
+    def count(self):
+        return 0
+
 class Typedef(object):
     kind = 'typedef'
 
@@ -389,7 +416,8 @@ class Native(object):
         'domstring': 'nsAString',
         'utf8string': 'nsACString',
         'cstring': 'nsACString',
-        'astring': 'nsAString'
+        'astring': 'nsAString',
+        'jsval': 'JS::Value'
         }
 
     def __init__(self, name, nativename, attlist, location):
@@ -431,6 +459,12 @@ class Native(object):
 
         return self.modifier == 'ref'
 
+    def isPtr(self, calltype):
+        return self.modifier == 'ptr' or (self.modifier == 'ref' and self.specialtype == 'jsval' and calltype == 'out')
+
+    def isRef(self, calltype):
+        return self.modifier == 'ref' and not (self.specialtype == 'jsval' and calltype == 'out')
+
     def nativeType(self, calltype, const=False, shared=False):
         if shared:
             if calltype != 'out':
@@ -440,10 +474,10 @@ class Native(object):
         if self.specialtype is not None and calltype == 'in':
             const = True
 
-        if self.modifier == 'ptr':
-            m = '*' + (calltype != 'in' and '*' or '')
-        elif self.modifier == 'ref':
+        if self.isRef(calltype):
             m = '& '
+        elif self.isPtr(calltype):
+            m = '*' + ((self.modifier == 'ptr' and calltype != 'in') and '*' or '')
         else:
             m = calltype != 'in' and '*' or ''
         return "%s%s %s" % (const and 'const ' or '', self.nativename, m)
@@ -486,7 +520,7 @@ class Interface(object):
         parent.setName(self)
         if self.base is None:
             if self.name != 'nsISupports':
-                print >>sys.stderr, IDLError("interface '%s' not derived from nsISupports",
+                print >>sys.stderr, IDLError("interface '%s' not derived from nsISupports" % self.name,
                                              self.location, warning=True)
         else:
             realbase = parent.getName(self.base, self.location)
@@ -499,6 +533,14 @@ class Interface(object):
         for member in self.members:
             member.resolve(self)
 
+        # The number 250 is NOT arbitrary; this number is the maximum number of
+        # stub entries defined in xpcom/reflect/xptcall/public/genstubs.pl
+        # Do not increase this value without increasing the number in that
+        # location, or you WILL cause otherwise unknown problems!
+        if self.countEntries() > 250 and not self.attributes.builtinclass:
+            raise IDLError("interface '%s' has too many entries" % self.name,
+                self.location)
+
     def isScriptable(self):
         # NOTE: this is not whether *this* interface is scriptable... it's
         # whether, when used as a type, it's scriptable, which is true of all
@@ -523,15 +565,38 @@ class Interface(object):
         return "".join(l)
 
     def getConst(self, name, location):
-        c = self.namemap.get(name, location)
+        # The constant may be in a base class
+        iface = self
+        while name not in iface.namemap and iface is not None:
+            iface = self.idl.getName(self.base, self.location)
+        if iface is None:
+            raise IDLError("cannot find symbol '%s'" % name, c.location)
+        c = iface.namemap.get(name, location)
         if c.kind != 'const':
             raise IDLError("symbol '%s' is not a constant", c.location)
 
         return c.getValue()
 
+    def needsJSTypes(self):
+        for m in self.members:
+            if m.kind == "attribute" and m.type == "jsval":
+                return True
+            if m.kind == "method" and m.needsJSTypes():
+                return True
+        return False
+
+    def countEntries(self):
+        ''' Returns the number of entries in the vtable for this interface. '''
+        total = sum(member.count() for member in self.members)
+        if self.base is not None:
+            realbase = self.idl.getName(self.base, self.location)
+            total += realbase.countEntries()
+        return total
+
 class InterfaceAttributes(object):
     uuid = None
     scriptable = False
+    builtinclass = False
     function = False
     deprecated = False
     noscript = False
@@ -548,12 +613,16 @@ class InterfaceAttributes(object):
     def setnoscript(self):
         self.noscript = True
 
+    def setbuiltinclass(self):
+        self.builtinclass = True
+
     def setdeprecated(self):
         self.deprecated = True
 
     actions = {
         'uuid':       (True, setuuid),
         'scriptable': (False, setscriptable),
+        'builtinclass': (False, setbuiltinclass),
         'function':   (False, setfunction),
         'noscript':   (False, setnoscript),
         'deprecated': (False, setdeprecated),
@@ -588,6 +657,8 @@ class InterfaceAttributes(object):
             l.append("\tuuid: %s\n" % self.uuid)
         if self.scriptable:
             l.append("\tscriptable\n")
+        if self.builtinclass:
+            l.append("\tbuiltinclass\n")
         if self.function:
             l.append("\tfunction\n")
         return "".join(l)
@@ -618,12 +689,20 @@ class ConstMember(object):
     def __str__(self):
         return "\tconst %s %s = %s\n" % (self.type, self.name, self.getValue())
 
+    def count(self):
+        return 0
+
 class Attribute(object):
     kind = 'attribute'
     noscript = False
     notxpcom = False
     readonly = False
+    implicit_jscontext = False
+    nostdcall = False
     binaryname = None
+    null = None
+    undefined = None
+    deprecated = False
 
     def __init__(self, type, name, attlist, readonly, location, doccomments):
         self.type = type
@@ -642,19 +721,54 @@ class Attribute(object):
                 self.binaryname = value
                 continue
 
-            if value is not None:
-                raise IDLError("Unexpected attribute value", aloc)
-
-            if name == 'noscript':
-                self.noscript = True
-            elif name == 'notxpcom':
-                self.notxpcom = True
+            if name == 'Null':
+                if value is None:
+                    raise IDLError("'Null' attribute requires a value", aloc)
+                if readonly:
+                    raise IDLError("'Null' attribute only makes sense for setters",
+                                   aloc);
+                if value not in ('Empty', 'Null', 'Stringify'):
+                    raise IDLError("'Null' attribute value must be 'Empty', 'Null' or 'Stringify'",
+                                   aloc);
+                self.null = value
+            elif name == 'Undefined':
+                if value is None:
+                    raise IDLError("'Undefined' attribute requires a value", aloc)
+                if readonly:
+                    raise IDLError("'Undefined' attribute only makes sense for setters",
+                                   aloc);
+                if value not in ('Empty', 'Null'):
+                    raise IDLError("'Undefined' attribute value must be 'Empty' or 'Null'",
+                                   aloc);
+                self.undefined = value
             else:
-                raise IDLError("Unexpected attribute '%s'", aloc)
+                if value is not None:
+                    raise IDLError("Unexpected attribute value", aloc)
+
+                if name == 'noscript':
+                    self.noscript = True
+                elif name == 'notxpcom':
+                    self.notxpcom = True
+                elif name == 'implicit_jscontext':
+                    self.implicit_jscontext = True
+                elif name == 'deprecated':
+                    self.deprecated = True
+                elif name == 'nostdcall':
+                    self.nostdcall = True
+                else:
+                    raise IDLError("Unexpected attribute '%s'" % name, aloc)
 
     def resolve(self, iface):
         self.iface = iface
         self.realtype = iface.idl.getName(self.type, self.location)
+        if (self.null is not None and
+            getBuiltinOrNativeTypeName(self.realtype) != '[domstring]'):
+            raise IDLError("'Null' attribute can only be used on DOMString",
+                           self.location)
+        if (self.undefined is not None and
+            getBuiltinOrNativeTypeName(self.realtype) != '[domstring]'):
+            raise IDLError("'Undefined' attribute can only be used on DOMString",
+                           self.location)
 
     def toIDL(self):
         attribs = attlistToIDL(self.attlist)
@@ -669,11 +783,18 @@ class Attribute(object):
         return "\t%sattribute %s %s\n" % (self.readonly and 'readonly ' or '',
                                           self.type, self.name)
 
+    def count(self):
+        return self.readonly and 1 or 2
+
 class Method(object):
     kind = 'method'
     noscript = False
     notxpcom = False
     binaryname = None
+    implicit_jscontext = False
+    nostdcall = False
+    optional_argc = False
+    deprecated = False
 
     def __init__(self, type, name, attlist, paramlist, location, doccomments, raises):
         self.type = type
@@ -700,8 +821,16 @@ class Method(object):
                 self.noscript = True
             elif name == 'notxpcom':
                 self.notxpcom = True
+            elif name == 'implicit_jscontext':
+                self.implicit_jscontext = True
+            elif name == 'optional_argc':
+                self.optional_argc = True
+            elif name == 'deprecated':
+                self.deprecated = True
+            elif name == 'nostdcall':
+                self.nostdcall = True
             else:
-                raise IDLError("Unexpected attribute '%s'", aloc)
+                raise IDLError("Unexpected attribute '%s'" % name, aloc)
 
         self.namemap = NameMap()
         for p in paramlist:
@@ -733,6 +862,20 @@ class Method(object):
                                                for p in self.params]),
                                     raises)
 
+    def needsJSTypes(self):
+        if self.implicit_jscontext:
+            return True
+        if self.type == "jsval":
+            return True
+        for p in self.params:
+            t = p.realtype
+            if isinstance(t, Native) and t.specialtype == "jsval":
+                return True
+        return False
+
+    def count(self):
+        return 1
+
 class Param(object):
     size_is = None
     iid_is = None
@@ -741,6 +884,8 @@ class Param(object):
     retval = False
     shared = False
     optional = False
+    null = None
+    undefined = None
 
     def __init__(self, paramtype, type, name, attlist, location, realtype=None):
         self.paramtype = paramtype
@@ -760,6 +905,20 @@ class Param(object):
                 if value is None:
                     raise IDLError("'iid_is' must specify a parameter", aloc)
                 self.iid_is = value
+            elif name == 'Null':
+                if value is None:
+                    raise IDLError("'Null' must specify a parameter", aloc)
+                if value not in ('Empty', 'Null', 'Stringify'):
+                    raise IDLError("'Null' parameter value must be 'Empty', 'Null', or 'Stringify'",
+                                   aloc);
+                self.null = value
+            elif name == 'Undefined':
+                if value is None:
+                    raise IDLError("'Undefined' must specify a parameter", aloc)
+                if value not in ('Empty', 'Null'):
+                    raise IDLError("'Undefined' parameter value must be 'Empty' or 'Null'",
+                                   aloc);
+                self.undefined = value
             else:
                 if value is not None:
                     raise IDLError("Unexpected value for attribute '%s'" % name,
@@ -782,6 +941,14 @@ class Param(object):
         self.realtype = method.iface.idl.getName(self.type, self.location)
         if self.array:
             self.realtype = Array(self.realtype)
+        if (self.null is not None and
+            getBuiltinOrNativeTypeName(self.realtype) != '[domstring]'):
+            raise IDLError("'Null' attribute can only be used on DOMString",
+                           self.location)
+        if (self.undefined is not None and
+            getBuiltinOrNativeTypeName(self.realtype) != '[domstring]'):
+            raise IDLError("'Undefined' attribute can only be used on DOMString",
+                           self.location)
 
     def nativeType(self):
         kwargs = {}
@@ -888,10 +1055,9 @@ class IDLParser(object):
 
     def t_directive(self, t):
         r'\#(?P<directive>[a-zA-Z]+)[^\n]+'
-        print >>sys.stderr, IDLError("Unrecognized directive %s" % t.lexer.lexmatch.group('directive'),
-                                     Location(lexer=self.lexer,
-                                              lineno=self.lexer.lineno,
-                                              lexpos=self.lexer.lexpos))
+        raise IDLError("Unrecognized directive %s" % t.lexer.lexmatch.group('directive'),
+                       Location(lexer=self.lexer, lineno=self.lexer.lineno,
+                                lexpos=self.lexer.lexpos))
 
     def t_newline(self, t):
         r'\n+'
@@ -1206,17 +1372,17 @@ class IDLParser(object):
         location = Location(self.lexer, t.lineno, t.lexpos)
         raise IDLError("invalid syntax", location)
 
-    def __init__(self, outputdir=''):
+    def __init__(self, outputdir='', regen=False):
         self._doccomments = []
         self.lexer = lex.lex(object=self,
                              outputdir=outputdir,
                              lextab='xpidllex',
-                             optimize=1)
+                             optimize=0 if regen else 1)
         self.parser = yacc.yacc(module=self,
                                 outputdir=outputdir,
                                 debugfile='xpidl_debug',
                                 tabmodule='xpidlyacc',
-                                optimize=1)
+                                optimize=0 if regen else 1)
 
     def clearComments(self):
         self._doccomments = []
@@ -1233,7 +1399,10 @@ class IDLParser(object):
             self.lexer.filename = filename
         self.lexer.lineno = 1
         self.lexer.input(data)
-        return self.parser.parse(lexer=self)
+        idl = self.parser.parse(lexer=self)
+        if filename is not None:
+            idl.deps.append(filename)
+        return idl
 
     def getLocation(self, p, i):
         return Location(self.lexer, p.lineno(i), p.lexpos(i))
