@@ -53,14 +53,6 @@
 #include <string.h>
 #include <direct.h>
 
-// These are not defined by VC6. 
-#ifndef CSIDL_LOCAL_APPDATA
-#define CSIDL_LOCAL_APPDATA             0x001C
-#endif
-#ifndef CSIDL_PROGRAM_FILES
-#define CSIDL_PROGRAM_FILES             0x0026
-#endif
-
 #elif defined(XP_OS2)
 
 #define MAX_PATH _MAX_PATH
@@ -76,23 +68,12 @@
 
 #elif defined(XP_UNIX)
 
+#include <limits.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <sys/param.h>
 #include "prenv.h"
 
-#elif defined(XP_BEOS)
-
-#include <FindDirectory.h>
-#include <fs_info.h>
-#include <Path.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <sys/param.h>
-#include <OS.h>
-#include <image.h>
-#include "prenv.h"
-
 #endif
 
 #if defined(VMS)
@@ -100,7 +81,9 @@
 #endif
 
 #ifndef MAXPATHLEN
-#ifdef MAX_PATH
+#ifdef PATH_MAX
+#define MAXPATHLEN PATH_MAX
+#elif defined(MAX_PATH)
 #define MAXPATHLEN MAX_PATH
 #elif defined(_MAX_PATH)
 #define MAXPATHLEN _MAX_PATH
@@ -122,7 +105,7 @@ static nsGetKnownFolderPath gGetKnownFolderPath = NULL;
 static HINSTANCE gShell32DLLInst = NULL;
 #endif
 
-NS_COM void StartupSpecialSystemDirectory()
+void StartupSpecialSystemDirectory()
 {
 #if defined (XP_WIN)
     // SHGetKnownFolderPath is only available on Windows Vista
@@ -130,13 +113,13 @@ NS_COM void StartupSpecialSystemDirectory()
     gShell32DLLInst = LoadLibraryW(L"shell32.dll");
     if(gShell32DLLInst)
     {
-        gGetKnownFolderPath = (nsGetKnownFolderPath) 
+        gGetKnownFolderPath = (nsGetKnownFolderPath)
             GetProcAddress(gShell32DLLInst, "SHGetKnownFolderPath");
     }
 #endif
 }
 
-NS_COM void ShutdownSpecialSystemDirectory()
+void ShutdownSpecialSystemDirectory()
 {
 #if defined (XP_WIN)
     if (gShell32DLLInst)
@@ -161,8 +144,8 @@ static nsresult GetKnownFolder(GUID* guid, nsILocalFile** aFile)
     if (!path)
         return NS_ERROR_FAILURE;
 
-    nsresult rv = NS_NewLocalFile(nsDependentString(path), 
-                                  PR_TRUE, 
+    nsresult rv = NS_NewLocalFile(nsDependentString(path),
+                                  PR_TRUE,
                                   aFile);
 
     CoTaskMemFree(path);
@@ -173,53 +156,62 @@ static nsresult GetKnownFolder(GUID* guid, nsILocalFile** aFile)
 static nsresult GetWindowsFolder(int folder, nsILocalFile** aFile)
 //----------------------------------------------------------------------------------------
 {
-#ifdef WINCE
-#define SHGetSpecialFolderPathW SHGetSpecialFolderPath
-#endif
-    WCHAR path[MAX_PATH + 2];
+    WCHAR path_orig[MAX_PATH + 3];
+    WCHAR *path = path_orig+1;
     HRESULT result = SHGetSpecialFolderPathW(NULL, path, folder, true);
-    
-    if (!SUCCEEDED(result)) 
+
+    if (!SUCCEEDED(result))
         return NS_ERROR_FAILURE;
 
     // Append the trailing slash
     int len = wcslen(path);
-    if (len > 1 && path[len - 1] != L'\\') 
+    if (len > 1 && path[len - 1] != L'\\')
     {
         path[len]   = L'\\';
         path[++len] = L'\0';
     }
 
     return NS_NewLocalFile(nsDependentString(path, len), PR_TRUE, aFile);
-} 
+}
+
+/**
+ * Provides a fallback for getting the path to APPDATA or LOCALAPPDATA by
+ * querying the registry when the call to SHGetSpecialFolderPathW is unable to
+ * provide these paths (Bug 513958).
+ */
+static nsresult GetRegWindowsAppDataFolder(PRBool aLocal, nsILocalFile** aFile)
+{
+    HKEY key;
+    NS_NAMED_LITERAL_STRING(keyName,
+    "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders");
+    DWORD res = ::RegOpenKeyExW(HKEY_CURRENT_USER, keyName.get(), 0, KEY_READ,
+                                &key);
+    if (res != ERROR_SUCCESS)
+        return NS_ERROR_FAILURE;
+
+    WCHAR path[MAX_PATH + 2];
+    DWORD type, size;
+    res = RegQueryValueExW(key, (aLocal ? L"Local AppData" : L"AppData"), NULL,
+                           &type, (LPBYTE)&path, &size);
+    ::RegCloseKey(key);
+    // The call to RegQueryValueExW must succeed, the type must be REG_SZ, the
+    // buffer size must not equal 0, and the buffer size be a multiple of 2.
+    if (res != ERROR_SUCCESS || type != REG_SZ || size == 0 || size % 2 != 0)
+        return NS_ERROR_FAILURE;
+
+    // Append the trailing slash
+    int len = wcslen(path);
+    if (len > 1 && path[len - 1] != L'\\')
+    {
+        path[len]   = L'\\';
+        path[++len] = L'\0';
+    }
+
+    return NS_NewLocalFile(nsDependentString(path, len), PR_TRUE, aFile);
+}
 
 #endif // XP_WIN
 
-#if defined (XP_BEOS)                                            
-static nsresult
-GetBeOSFolder( directory_which which, dev_t volume, nsILocalFile** aFile)
-{
-    char path[MAXPATHLEN];
-    if (volume < 0)
-        return NS_ERROR_FAILURE;
-        
-    status_t result = find_directory(which, volume, false, path, MAXPATHLEN - 2);
-    if (result != B_OK)
-        return NS_ERROR_FAILURE;
-        
-    int len = strlen(path);
-    if (len == 0)
-        return NS_ERROR_FAILURE;
-
-    if (path[len-1] != '/') 
-    {
-        path[len]   = '/';
-        path[len+1] = '\0';            
-    }
-    return NS_NewNativeLocalFile(nsDependentCString(path), PR_TRUE, aFile);
-}
-#endif // XP_BEOS
-
 #if defined(XP_UNIX)
 static nsresult
 GetUnixHomeDir(nsILocalFile** aFile)
@@ -228,16 +220,19 @@ GetUnixHomeDir(nsILocalFile** aFile)
     char *pHome;
     pHome = getenv("HOME");
     if (*pHome == '/') {
-        return NS_NewNativeLocalFile(nsDependentCString(pHome), 
-                                     PR_TRUE, 
+        return NS_NewNativeLocalFile(nsDependentCString(pHome),
+                                     PR_TRUE,
                                      aFile);
     } else {
-        return NS_NewNativeLocalFile(nsDependentCString(decc$translate_vms(pHome)), 
-                                     PR_TRUE, 
+        return NS_NewNativeLocalFile(nsDependentCString(decc$translate_vms(pHome)),
+                                     PR_TRUE,
                                      aFile);
     }
+#elif defined(ANDROID)
+    // XXX no home dir on android; maybe we should return the sdcard if present?
+    return NS_ERROR_FAILURE;
 #else
-    return NS_NewNativeLocalFile(nsDependentCString(PR_GetEnv("HOME")), 
+    return NS_NewNativeLocalFile(nsDependentCString(PR_GetEnv("HOME")),
                                  PR_TRUE, aFile);
 #endif
 }
@@ -278,7 +273,7 @@ xdg_user_dir_lookup (const char *type)
   char *p, *d;
   int len;
   int relative;
-  
+
   home_dir = getenv ("HOME");
 
   if (home_dir == NULL)
@@ -316,11 +311,11 @@ xdg_user_dir_lookup (const char *type)
       len = strlen (buffer);
       if (len > 0 && buffer[len-1] == '\n')
 	buffer[len-1] = 0;
-      
+
       p = buffer;
       while (*p == ' ' || *p == '\t')
 	p++;
-      
+
       if (strncmp (p, "XDG_", 4) != 0)
 	continue;
       p += 4;
@@ -337,14 +332,14 @@ xdg_user_dir_lookup (const char *type)
       if (*p != '=')
 	continue;
       p++;
-      
+
       while (*p == ' ' || *p == '\t')
 	p++;
 
       if (*p != '"')
 	continue;
       p++;
-      
+
       relative = 0;
       if (strncmp (p, "$HOME/", 6) == 0)
 	{
@@ -353,7 +348,7 @@ xdg_user_dir_lookup (const char *type)
 	}
       else if (*p != '/')
 	continue;
-      
+
       if (relative)
 	{
 	  user_dir = (char*) malloc (strlen (home_dir) + 1 + strlen (p) + 1);
@@ -371,7 +366,7 @@ xdg_user_dir_lookup (const char *type)
 
 	  *user_dir = 0;
 	}
-      
+
       d = user_dir + strlen (user_dir);
       while (*p && *p != '"')
 	{
@@ -418,7 +413,7 @@ GetUnixXDGUserDirectory(SystemDirectories aSystemDirectory,
 {
     char *dir = xdg_user_dir_lookup
                     (xdg_user_dirs + xdg_user_dir_offsets[aSystemDirectory -
-                                                          Unix_XDG_Desktop]);
+                                                         Unix_XDG_Desktop]);
 
     nsresult rv;
     nsCOMPtr<nsILocalFile> file;
@@ -434,7 +429,24 @@ GetUnixXDGUserDirectory(SystemDirectories aSystemDirectory,
             return rv;
 
         rv = file->AppendNative(NS_LITERAL_CSTRING("Desktop"));
-    } else {
+    }
+#if defined(MOZ_PLATFORM_MAEMO)
+    // "MYDOCSDIR" is exported to point to "/home/user/MyDocs" in maemo.
+    else if (Unix_XDG_Documents == aSystemDirectory) {
+
+        char *myDocs = PR_GetEnv("MYDOCSDIR");
+        if (!myDocs || !*myDocs)
+            return NS_ERROR_FAILURE;
+
+        rv = NS_NewNativeLocalFile(nsDependentCString(myDocs), PR_TRUE,
+                                   getter_AddRefs(file));
+        if (NS_FAILED(rv))
+            return rv;
+
+        rv = file->AppendNative(NS_LITERAL_CSTRING(".documents"));
+    }
+#endif
+    else {
       // no fallback for the other XDG dirs
       rv = NS_ERROR_FAILURE;
     }
@@ -475,8 +487,8 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
 #if defined(XP_WIN)
             if (!_wgetcwd(path, MAX_PATH))
                 return NS_ERROR_FAILURE;
-            return NS_NewLocalFile(nsDependentString(path), 
-                                   PR_TRUE, 
+            return NS_NewLocalFile(nsDependentString(path),
+                                   PR_TRUE,
                                    aFile);
 #elif defined(XP_OS2)
             if (DosQueryPathInfo( ".", FIL_QUERYFULLNAME, path, MAXPATHLEN))
@@ -487,19 +499,13 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
 #endif
 
 #if !defined(XP_WIN)
-            return NS_NewNativeLocalFile(nsDependentCString(path), 
-                                         PR_TRUE, 
+            return NS_NewNativeLocalFile(nsDependentCString(path),
+                                         PR_TRUE,
                                          aFile);
 #endif
 
         case OS_DriveDirectory:
-#if defined (WINCE)
-        {
-            return NS_NewLocalFile(nsDependentString(L"\\"),
-                                   PR_TRUE, 
-                                   aFile);
-        }
-#elif defined (XP_WIN)
+#if defined (XP_WIN)
         {
             PRInt32 len = ::GetWindowsDirectoryW(path, MAX_PATH);
             if (len == 0)
@@ -508,7 +514,7 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
                 path[3] = 0;
 
             return NS_NewLocalFile(nsDependentString(path),
-                                   PR_TRUE, 
+                                   PR_TRUE,
                                    aFile);
         }
 #elif defined(XP_OS2)
@@ -520,24 +526,24 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
             buffer[0] = 'A' - 1 + ulBootDrive; // duh, 1-based index...
 
             return NS_NewNativeLocalFile(nsDependentCString(buffer),
-                                         PR_TRUE, 
+                                         PR_TRUE,
                                          aFile);
         }
 #else
-        return NS_NewNativeLocalFile(nsDependentCString("/"), 
-                                     PR_TRUE, 
+        return NS_NewNativeLocalFile(nsDependentCString("/"),
+                                     PR_TRUE,
                                      aFile);
 
 #endif
-            
+
         case OS_TemporaryDirectory:
 #if defined (XP_WIN)
             {
             DWORD len = ::GetTempPathW(MAX_PATH, path);
             if (len == 0)
                 break;
-            return NS_NewLocalFile(nsDependentString(path, len), 
-                                   PR_TRUE, 
+            return NS_NewLocalFile(nsDependentString(path, len),
+                                   PR_TRUE,
                                    aFile);
         }
 #elif defined(XP_OS2)
@@ -557,12 +563,12 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
             }
             return NS_NewNativeLocalFile(tString, PR_TRUE, aFile);
         }
-#elif defined(XP_MACOSX)
+#elif defined(MOZ_WIDGET_COCOA)
         {
             return GetOSXFolderType(kUserDomain, kTemporaryFolderType, aFile);
         }
 
-#elif defined(XP_UNIX) || defined(XP_BEOS)
+#elif defined(XP_UNIX)
         {
             static const char *tPath = nsnull;
             if (!tPath) {
@@ -577,63 +583,43 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
                     }
                 }
             }
-            return NS_NewNativeLocalFile(nsDependentCString(tPath), 
-                                         PR_TRUE, 
+            return NS_NewNativeLocalFile(nsDependentCString(tPath),
+                                         PR_TRUE,
                                          aFile);
         }
 #else
         break;
-#endif            
+#endif
 #if defined (XP_WIN)
         case Win_SystemDirectory:
-        {    
-#ifdef WINCE
-            PRUnichar winDirBuf[MAX_PATH];
-            nsAutoString winDir;
-            if (SHGetSpecialFolderPath(NULL, winDirBuf, CSIDL_WINDOWS, PR_TRUE))
-                winDir.Assign(winDirBuf);
-            else
-                winDir.Assign(L"\\Windows");
-            return NS_NewLocalFile(winDir, PR_TRUE, aFile);
-#else
+        {
             PRInt32 len = ::GetSystemDirectoryW(path, MAX_PATH);
-        
+
             // Need enough space to add the trailing backslash
             if (!len || len > MAX_PATH - 2)
                 break;
             path[len]   = L'\\';
             path[++len] = L'\0';
 
-            return NS_NewLocalFile(nsDependentString(path, len), 
-                                   PR_TRUE, 
+            return NS_NewLocalFile(nsDependentString(path, len),
+                                   PR_TRUE,
                                    aFile);
-#endif
         }
 
         case Win_WindowsDirectory:
-        {    
-#ifdef WINCE
-            PRUnichar winDirBuf[MAX_PATH];
-            nsAutoString winDir;
-            if (SHGetSpecialFolderPath(NULL, winDirBuf, CSIDL_WINDOWS, PR_TRUE))
-                winDir.Assign(winDirBuf);
-            else
-                winDir.Assign(L"\\Windows");
-            return NS_NewLocalFile(winDir, PR_TRUE, aFile);
-#else
+        {
             PRInt32 len = ::GetWindowsDirectoryW(path, MAX_PATH);
-            
+
             // Need enough space to add the trailing backslash
             if (!len || len > MAX_PATH - 2)
                 break;
-            
+
             path[len]   = L'\\';
             path[++len] = L'\0';
 
-            return NS_NewLocalFile(nsDependentString(path, len), 
-                                   PR_TRUE, 
+            return NS_NewLocalFile(nsDependentString(path, len),
+                                   PR_TRUE,
                                    aFile);
-#endif
         }
 
         case Win_ProgramFiles:
@@ -642,20 +628,26 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
         }
 
         case Win_HomeDirectory:
-        {    
+        {
+            nsresult rv = GetWindowsFolder(CSIDL_PROFILE, aFile);
+            if (NS_SUCCEEDED(rv))
+                return rv;
+
             PRInt32 len;
             if ((len = ::GetEnvironmentVariableW(L"HOME", path, MAX_PATH)) > 0)
             {
                 // Need enough space to add the trailing backslash
                 if (len > MAX_PATH - 2)
                     break;
-               
+
                 path[len]   = L'\\';
                 path[++len] = L'\0';
 
-                return NS_NewLocalFile(nsDependentString(path, len), 
-                                       PR_TRUE, 
-                                       aFile);
+                rv = NS_NewLocalFile(nsDependentString(path, len),
+                                     PR_TRUE,
+                                     aFile);
+                if (NS_SUCCEEDED(rv))
+                    return rv;
             }
 
             len = ::GetEnvironmentVariableW(L"HOMEDRIVE", path, MAX_PATH);
@@ -665,18 +657,18 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
                 DWORD len2 = ::GetEnvironmentVariableW(L"HOMEPATH", temp, MAX_PATH);
                 if (0 < len2 && len + len2 < MAX_PATH)
                     wcsncat(path, temp, len2);
-        
+
                 len = wcslen(path);
 
                 // Need enough space to add the trailing backslash
                 if (len > MAX_PATH - 2)
                     break;
-            
+
                 path[len]   = L'\\';
                 path[++len] = L'\0';
-                
-                return NS_NewLocalFile(nsDependentString(path, len), 
-                                       PR_TRUE, 
+
+                return NS_NewLocalFile(nsDependentString(path, len),
+                                       PR_TRUE,
                                        aFile);
             }
         }
@@ -764,7 +756,6 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
         {
             return GetWindowsFolder(CSIDL_TEMPLATES, aFile);
         }
-#ifndef WINCE
         case Win_Common_Startmenu:
         {
             return GetWindowsFolder(CSIDL_COMMON_STARTMENU, aFile);
@@ -789,26 +780,30 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
         {
             return GetWindowsFolder(CSIDL_COOKIES, aFile);
         }
-#endif
         case Win_Appdata:
         {
-            return GetWindowsFolder(CSIDL_APPDATA, aFile);
+            nsresult rv = GetWindowsFolder(CSIDL_APPDATA, aFile);
+            if (NS_FAILED(rv))
+                rv = GetRegWindowsAppDataFolder(PR_FALSE, aFile);
+            return rv;
         }
-
         case Win_LocalAppdata:
         {
-            return GetWindowsFolder(CSIDL_LOCAL_APPDATA, aFile);
+            nsresult rv = GetWindowsFolder(CSIDL_LOCAL_APPDATA, aFile);
+            if (NS_FAILED(rv))
+                rv = GetRegWindowsAppDataFolder(PR_TRUE, aFile);
+            return rv;
         }
 #endif  // XP_WIN
 
 #if defined(XP_UNIX)
         case Unix_LocalDirectory:
-            return NS_NewNativeLocalFile(nsDependentCString("/usr/local/netscape/"), 
-                                         PR_TRUE, 
+            return NS_NewNativeLocalFile(nsDependentCString("/usr/local/netscape/"),
+                                         PR_TRUE,
                                          aFile);
         case Unix_LibDirectory:
-            return NS_NewNativeLocalFile(nsDependentCString("/usr/local/lib/netscape/"), 
-                                         PR_TRUE, 
+            return NS_NewNativeLocalFile(nsDependentCString("/usr/local/lib/netscape/"),
+                                         PR_TRUE,
                                          aFile);
 
         case Unix_HomeDirectory:
@@ -825,31 +820,6 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
             return GetUnixXDGUserDirectory(aSystemSystemDirectory, aFile);
 #endif
 
-#ifdef XP_BEOS
-        case BeOS_SettingsDirectory:
-        {
-            return GetBeOSFolder(B_USER_SETTINGS_DIRECTORY,0, aFile);
-        }
-
-        case BeOS_HomeDirectory:
-        {
-            return GetBeOSFolder(B_USER_DIRECTORY,0, aFile);
-        }
-
-        case BeOS_DesktopDirectory:
-        {
-            /* Get the user's desktop folder, which in the future may differ from the boot desktop */
-            char path[MAXPATHLEN];
-            if (find_directory(B_USER_DIRECTORY, 0, false, path, MAXPATHLEN) != B_OK )
-                break;
-            return GetBeOSFolder(B_DESKTOP_DIRECTORY, dev_for_path(path), aFile);
-        }
-
-        case BeOS_SystemDirectory:
-        {
-            return GetBeOSFolder(B_BEOS_DIRECTORY,0, aFile);
-        }
-#endif        
 #ifdef XP_OS2
         case OS2_SystemDirectory:
         {
@@ -859,8 +829,8 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
                              &ulBootDrive, sizeof ulBootDrive);
             buffer[0] = 'A' - 1 + ulBootDrive; // duh, 1-based index...
 
-            return NS_NewNativeLocalFile(nsDependentCString(buffer), 
-                                         PR_TRUE, 
+            return NS_NewNativeLocalFile(nsDependentCString(buffer),
+                                         PR_TRUE,
                                          aFile);
         }
 
@@ -872,8 +842,8 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
                              &ulBootDrive, sizeof ulBootDrive);
             buffer[0] = 'A' - 1 + ulBootDrive; // duh, 1-based index...
 
-            return NS_NewNativeLocalFile(nsDependentCString(buffer), 
-                                         PR_TRUE, 
+            return NS_NewNativeLocalFile(nsDependentCString(buffer),
+                                         PR_TRUE,
                                          aFile);
         }
 
@@ -893,7 +863,7 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
                 tPath = buffer;
             }
             rv = NS_NewNativeLocalFile(nsDependentCString(tPath),
-                                       PR_TRUE, 
+                                       PR_TRUE,
                                        aFile);
 
             PrfWriteProfileString(HINI_USERPROFILE, "Mozilla", "Home", tPath);
@@ -902,7 +872,7 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
 
         case OS2_DesktopDirectory:
         {
-            char szPath[CCHMAXPATH + 1];        
+            char szPath[CCHMAXPATH + 1];
             BOOL fSuccess;
             fSuccess = WinQueryActiveDesktopPathname (szPath, sizeof(szPath));
             if (!fSuccess) {
@@ -910,14 +880,14 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
                 // the Home directory instead
                 return GetSpecialSystemDirectory(OS2_HomeDirectory, aFile);
             }
-            int len = strlen (szPath);   
+            int len = strlen (szPath);
             if (len > CCHMAXPATH -1)
                 break;
-            szPath[len] = '\\';     
+            szPath[len] = '\\';
             szPath[len + 1] = '\0';
 
             return NS_NewNativeLocalFile(nsDependentCString(szPath),
-                                         PR_TRUE, 
+                                         PR_TRUE,
                                          aFile);
         }
 #endif
@@ -927,7 +897,7 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
     return NS_ERROR_NOT_AVAILABLE;
 }
 
-#if defined (XP_MACOSX)
+#if defined (MOZ_WIDGET_COCOA)
 nsresult
 GetOSXFolderType(short aDomain, OSType aFolderType, nsILocalFile **localFile)
 {
@@ -944,6 +914,6 @@ GetOSXFolderType(short aDomain, OSType aFolderType, nsILocalFile **localFile)
             rv = localMacFile->InitWithFSRef(&fsRef);
     }
     return rv;
-}                                                                      
+}
 #endif
 
