@@ -36,14 +36,17 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "mozilla/Mutex.h"
 #include "nsStreamUtils.h"
 #include "nsCOMPtr.h"
 #include "nsIPipe.h"
 #include "nsIEventTarget.h"
 #include "nsIRunnable.h"
-#include "nsAutoLock.h"
+#include "nsISafeOutputStream.h"
 #include "nsString.h"
 
+using namespace mozilla;
+
 //-----------------------------------------------------------------------------
 
 class nsInputStreamReadyEvent : public nsIRunnable
@@ -204,7 +207,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS2(nsOutputStreamReadyEvent, nsIRunnable,
 
 //-----------------------------------------------------------------------------
 
-NS_COM nsresult
+nsresult
 NS_NewInputStreamReadyEvent(nsIInputStreamCallback **event,
                             nsIInputStreamCallback *callback,
                             nsIEventTarget *target)
@@ -218,7 +221,7 @@ NS_NewInputStreamReadyEvent(nsIInputStreamCallback **event,
     return NS_OK;
 }
 
-NS_COM nsresult
+nsresult
 NS_NewOutputStreamReadyEvent(nsIOutputStreamCallback **event,
                              nsIOutputStreamCallback *callback,
                              nsIEventTarget *target)
@@ -244,20 +247,22 @@ public:
     NS_DECL_ISUPPORTS
 
     nsAStreamCopier()
-        : mLock(nsnull)
+        : mLock("nsAStreamCopier.mLock")
         , mCallback(nsnull)
         , mClosure(nsnull)
         , mChunkSize(0)
         , mEventInProcess(PR_FALSE)
         , mEventIsPending(PR_FALSE)
+        , mCloseSource(PR_TRUE)
+        , mCloseSink(PR_TRUE)
+        , mCanceled(PR_FALSE)
+        , mCancelStatus(NS_OK)
     {
     }
 
     // virtual since subclasses call superclass Release()
     virtual ~nsAStreamCopier()
     {
-        if (mLock)
-            PR_DestroyLock(mLock);
     }
 
     // kick off the async copy...
@@ -266,7 +271,9 @@ public:
                    nsIEventTarget *target,
                    nsAsyncCopyCallbackFun callback,
                    void *closure,
-                   PRUint32 chunksize)
+                   PRUint32 chunksize,
+                   PRBool closeSource,
+                   PRBool closeSink)
     {
         mSource = source;
         mSink = sink;
@@ -274,10 +281,8 @@ public:
         mCallback = callback;
         mClosure = closure;
         mChunkSize = chunksize;
-
-        mLock = PR_NewLock();
-        if (!mLock)
-            return NS_ERROR_OUT_OF_MEMORY;
+        mCloseSource = closeSource;
+        mCloseSink = closeSink;
 
         mAsyncSource = do_QueryInterface(mSource);
         mAsyncSink = do_QueryInterface(mSink);
@@ -295,11 +300,31 @@ public:
             return;
 
         nsresult sourceCondition, sinkCondition;
+        nsresult cancelStatus;
+        PRBool canceled;
+        {
+            MutexAutoLock lock(mLock);
+            canceled = mCanceled;
+            cancelStatus = mCancelStatus;
+        }
 
-        // ok, copy data from source to sink.
+        // Copy data from the source to the sink until we hit failure or have
+        // copied all the data.
         for (;;) {
-            PRUint32 n = DoCopy(&sourceCondition, &sinkCondition);
-            if (NS_FAILED(sourceCondition) || NS_FAILED(sinkCondition) || n == 0) {
+            // Note: copyFailed will be true if the source or the sink have
+            //       reported an error, or if we failed to write any bytes
+            //       because we have consumed all of our data.
+            PRBool copyFailed = PR_FALSE;
+            if (!canceled) {
+                PRUint32 n = DoCopy(&sourceCondition, &sinkCondition);
+                copyFailed = NS_FAILED(sourceCondition) ||
+                             NS_FAILED(sinkCondition) || n == 0;
+
+                MutexAutoLock lock(mLock);
+                canceled = mCanceled;
+                cancelStatus = mCancelStatus;
+            }
+            if (copyFailed && !canceled) {
                 if (sourceCondition == NS_BASE_STREAM_WOULD_BLOCK && mAsyncSource) {
                     // need to wait for more data from source.  while waiting for
                     // more source data, be sure to observe failures on output end.
@@ -309,6 +334,7 @@ public:
                         mAsyncSink->AsyncWait(this,
                                               nsIAsyncOutputStream::WAIT_CLOSURE_ONLY,
                                               0, nsnull);
+                    break;
                 }
                 else if (sinkCondition == NS_BASE_STREAM_WOULD_BLOCK && mAsyncSink) {
                     // need to wait for more room in the sink.  while waiting for
@@ -320,39 +346,77 @@ public:
                         mAsyncSource->AsyncWait(this,
                                                 nsIAsyncInputStream::WAIT_CLOSURE_ONLY,
                                                 0, nsnull);
+                    break;
                 }
-                else {
+            }
+            if (copyFailed || canceled) {
+                if (mCloseSource) {
                     // close source
                     if (mAsyncSource)
-                        mAsyncSource->CloseWithStatus(sinkCondition);
+                        mAsyncSource->CloseWithStatus(canceled ? cancelStatus :
+                                                                 sinkCondition);
                     else
                         mSource->Close();
-                    mAsyncSource = nsnull;
-                    mSource = nsnull;
+                }
+                mAsyncSource = nsnull;
+                mSource = nsnull;
 
+                if (mCloseSink) {
                     // close sink
                     if (mAsyncSink)
-                        mAsyncSink->CloseWithStatus(sourceCondition);
-                    else
-                        mSink->Close();
-                    mAsyncSink = nsnull;
-                    mSink = nsnull;
+                        mAsyncSink->CloseWithStatus(canceled ? cancelStatus :
+                                                               sourceCondition);
+                    else {
+                        // If we have an nsISafeOutputStream, and our
+                        // sourceCondition and sinkCondition are not set to a
+                        // failure state, finish writing.
+                        nsCOMPtr<nsISafeOutputStream> sostream =
+                            do_QueryInterface(mSink);
+                        if (sostream && NS_SUCCEEDED(sourceCondition) &&
+                            NS_SUCCEEDED(sinkCondition))
+                            sostream->Finish();
+                        else
+                            mSink->Close();
+                    }
+                }
+                mAsyncSink = nsnull;
+                mSink = nsnull;
 
-                    // notify state complete...
-                    if (mCallback) {
-                        nsresult status = sourceCondition;
+                // notify state complete...
+                if (mCallback) {
+                    nsresult status;
+                    if (!canceled) {
+                        status = sourceCondition;
                         if (NS_SUCCEEDED(status))
                             status = sinkCondition;
                         if (status == NS_BASE_STREAM_CLOSED)
                             status = NS_OK;
-                        mCallback(mClosure, status);
+                    } else {
+                        status = cancelStatus; 
                     }
+                    mCallback(mClosure, status);
                 }
                 break;
             }
         }
     }
 
+    nsresult Cancel(nsresult aReason)
+    {
+        MutexAutoLock lock(mLock);
+        if (mCanceled)
+            return NS_ERROR_FAILURE;
+
+        if (NS_SUCCEEDED(aReason)) {
+            NS_WARNING("cancel with non-failure status code");
+            aReason = NS_BASE_STREAM_CLOSED;
+        }
+
+        mCanceled = PR_TRUE;
+        mCancelStatus = aReason;
+        return NS_OK;
+    }
+
     NS_IMETHOD OnInputStreamReady(nsIAsyncInputStream *source)
     {
         PostContinuationEvent();
@@ -371,7 +435,7 @@ public:
         Process();
 
         // clear "in process" flag and post any pending continuation event
-        nsAutoLock lock(mLock);
+        MutexAutoLock lock(mLock);
         mEventInProcess = PR_FALSE;
         if (mEventIsPending) {
             mEventIsPending = PR_FALSE;
@@ -390,7 +454,7 @@ public:
         // just let that existing event take care of posting the real
         // continuation event.
 
-        nsAutoLock lock(mLock);
+        MutexAutoLock lock(mLock);
         return PostContinuationEvent_Locked();
     }
 
@@ -415,12 +479,16 @@ protected:
     nsCOMPtr<nsIAsyncInputStream>  mAsyncSource;
     nsCOMPtr<nsIAsyncOutputStream> mAsyncSink;
     nsCOMPtr<nsIEventTarget>       mTarget;
-    PRLock                        *mLock;
+    Mutex                          mLock;
     nsAsyncCopyCallbackFun         mCallback;
     void                          *mClosure;
     PRUint32                       mChunkSize;
     PRPackedBool                   mEventInProcess;
     PRPackedBool                   mEventIsPending;
+    PRPackedBool                   mCloseSource;
+    PRPackedBool                   mCloseSink;
+    PRPackedBool                   mCanceled;
+    nsresult                       mCancelStatus;
 };
 
 NS_IMPL_THREADSAFE_ISUPPORTS3(nsAStreamCopier,
@@ -516,14 +584,17 @@ public:
 
 //-----------------------------------------------------------------------------
 
-NS_COM nsresult
+nsresult
 NS_AsyncCopy(nsIInputStream         *source,
              nsIOutputStream        *sink,
              nsIEventTarget         *target,
              nsAsyncCopyMode         mode,
              PRUint32                chunkSize,
              nsAsyncCopyCallbackFun  callback,
-             void                   *closure)
+             void                   *closure,
+             PRBool                  closeSource,
+             PRBool                  closeSink,
+             nsISupports           **aCopierCtx)
 {
     NS_ASSERTION(target, "non-null target required");
 
@@ -540,7 +611,14 @@ NS_AsyncCopy(nsIInputStream         *source,
 
     // Start() takes an owning ref to the copier...
     NS_ADDREF(copier);
-    rv = copier->Start(source, sink, target, callback, closure, chunkSize);
+    rv = copier->Start(source, sink, target, callback, closure, chunkSize,
+                       closeSource, closeSink);
+
+    if (aCopierCtx) {
+        *aCopierCtx = static_cast<nsISupports*>(
+                      static_cast<nsIRunnable*>(copier));
+        NS_ADDREF(*aCopierCtx);
+    }
     NS_RELEASE(copier);
 
     return rv;
@@ -548,7 +626,17 @@ NS_AsyncCopy(nsIInputStream         *source,
 
 //-----------------------------------------------------------------------------
 
-NS_COM nsresult
+nsresult
+NS_CancelAsyncCopy(nsISupports *aCopierCtx, nsresult aReason)
+{
+  nsAStreamCopier *copier = static_cast<nsAStreamCopier *>(
+                            static_cast<nsIRunnable *>(aCopierCtx));
+  return copier->Cancel(aReason);
+}
+
+//-----------------------------------------------------------------------------
+
+nsresult
 NS_ConsumeStream(nsIInputStream *stream, PRUint32 maxCount, nsACString &result)
 {
     nsresult rv = NS_OK;
@@ -603,7 +691,7 @@ TestInputStream(nsIInputStream *inStr,
     return NS_ERROR_ABORT;  // don't call me anymore
 }
 
-NS_COM PRBool
+PRBool
 NS_InputStreamIsBuffered(nsIInputStream *stream)
 {
     PRBool result = PR_FALSE;
@@ -626,7 +714,7 @@ TestOutputStream(nsIOutputStream *outStr,
     return NS_ERROR_ABORT;  // don't call me anymore
 }
 
-NS_COM PRBool
+PRBool
 NS_OutputStreamIsBuffered(nsIOutputStream *stream)
 {
     PRBool result = PR_FALSE;
@@ -637,7 +725,7 @@ NS_OutputStreamIsBuffered(nsIOutputStream *stream)
 
 //-----------------------------------------------------------------------------
 
-NS_COM NS_METHOD
+NS_METHOD
 NS_CopySegmentToStream(nsIInputStream *inStr,
                        void *closure,
                        const char *buffer,
@@ -659,7 +747,7 @@ NS_CopySegmentToStream(nsIInputStream *inStr,
     return NS_OK;
 }
 
-NS_COM NS_METHOD
+NS_METHOD
 NS_CopySegmentToBuffer(nsIInputStream *inStr,
                        void *closure,
                        const char *buffer,
@@ -673,7 +761,7 @@ NS_CopySegmentToBuffer(nsIInputStream *inStr,
     return NS_OK;
 }
 
-NS_COM NS_METHOD
+NS_METHOD
 NS_DiscardSegment(nsIInputStream *inStr,
                   void *closure,
                   const char *buffer,
@@ -687,7 +775,7 @@ NS_DiscardSegment(nsIInputStream *inStr,
 
 //-----------------------------------------------------------------------------
 
-NS_COM NS_METHOD
+NS_METHOD
 NS_WriteSegmentThunk(nsIInputStream *inStr,
                      void *closure,
                      const char *buffer,
