@@ -43,22 +43,21 @@
   /**
    * The base for string comparators
    */
-class NS_COM nsTStringComparator_CharT
+class nsTStringComparator_CharT
   {
     public:
       typedef CharT char_type;
 
       nsTStringComparator_CharT() {}
 
-      virtual int operator()( const char_type*, const char_type*, PRUint32 length ) const = 0;
-      virtual int operator()( char_type, char_type ) const = 0;
+      virtual int operator()( const char_type*, const char_type*, PRUint32, PRUint32 ) const = 0;
   };
 
 
   /**
    * The default string comparator (case-sensitive comparision)
    */
-class NS_COM nsTDefaultStringComparator_CharT
+class nsTDefaultStringComparator_CharT
     : public nsTStringComparator_CharT
   {
     public:
@@ -66,8 +65,7 @@ class NS_COM nsTDefaultStringComparator_CharT
 
       nsTDefaultStringComparator_CharT() {}
 
-      virtual int operator()( const char_type*, const char_type*, PRUint32 length ) const;
-      virtual int operator()( char_type, char_type ) const;
+      virtual int operator()( const char_type*, const char_type*, PRUint32, PRUint32 ) const;
   };
 
   /**
@@ -112,7 +110,7 @@ class nsTSubstring_CharT
     public:
 
         // this acts like a virtual destructor
-      NS_COM NS_CONSTRUCTOR_FASTCALL ~nsTSubstring_CharT();
+      ~nsTSubstring_CharT() { Finalize(); }
 
         /**
          * reading iterators
@@ -254,32 +252,32 @@ class nsTSubstring_CharT
           return mData[mLength - 1];
         }
 
-      NS_COM size_type NS_FASTCALL CountChar( char_type ) const;
-      NS_COM PRInt32 NS_FASTCALL FindChar( char_type, index_type offset = 0 ) const;
+      size_type NS_FASTCALL CountChar( char_type ) const;
+      PRInt32 NS_FASTCALL FindChar( char_type, index_type offset = 0 ) const;
 
 
         /**
          * equality
          */
 
-      NS_COM PRBool NS_FASTCALL Equals( const self_type& ) const;
-      NS_COM PRBool NS_FASTCALL Equals( const self_type&, const comparator_type& ) const;
+      PRBool NS_FASTCALL Equals( const self_type& ) const;
+      PRBool NS_FASTCALL Equals( const self_type&, const comparator_type& ) const;
 
-      NS_COM PRBool NS_FASTCALL Equals( const char_type* data ) const;
-      NS_COM PRBool NS_FASTCALL Equals( const char_type* data, const comparator_type& comp ) const;
+      PRBool NS_FASTCALL Equals( const char_type* data ) const;
+      PRBool NS_FASTCALL Equals( const char_type* data, const comparator_type& comp ) const;
 
         /**
          * An efficient comparison with ASCII that can be used even
          * for wide strings. Call this version when you know the
          * length of 'data'.
          */
-      NS_COM PRBool NS_FASTCALL EqualsASCII( const char* data, size_type len ) const;
+      PRBool NS_FASTCALL EqualsASCII( const char* data, size_type len ) const;
         /**
          * An efficient comparison with ASCII that can be used even
          * for wide strings. Call this version when 'data' is
          * null-terminated.
          */
-      NS_COM PRBool NS_FASTCALL EqualsASCII( const char* data ) const;
+      PRBool NS_FASTCALL EqualsASCII( const char* data ) const;
 
     // EqualsLiteral must ONLY be applied to an actual literal string.
     // Do not attempt to use it with a regular char* pointer, or with a char
@@ -310,8 +308,8 @@ class nsTSubstring_CharT
     // *not* lowercased for you. If you compare to an ASCII or literal
     // string that contains an uppercase character, it is guaranteed to
     // return false. We will throw assertions too.
-      NS_COM PRBool NS_FASTCALL LowerCaseEqualsASCII( const char* data, size_type len ) const;
-      NS_COM PRBool NS_FASTCALL LowerCaseEqualsASCII( const char* data ) const;
+      PRBool NS_FASTCALL LowerCaseEqualsASCII( const char* data, size_type len ) const;
+      PRBool NS_FASTCALL LowerCaseEqualsASCII( const char* data ) const;
 
     // LowerCaseEqualsLiteral must ONLY be applied to an actual
     // literal string.  Do not attempt to use it with a regular char*
@@ -340,13 +338,13 @@ class nsTSubstring_CharT
          * assignment
          */
 
-      NS_COM void NS_FASTCALL Assign( char_type c );
-      NS_COM void NS_FASTCALL Assign( const char_type* data, size_type length = size_type(-1) );
-      NS_COM void NS_FASTCALL Assign( const self_type& );
-      NS_COM void NS_FASTCALL Assign( const substring_tuple_type& );
+      void NS_FASTCALL Assign( char_type c );
+      void NS_FASTCALL Assign( const char_type* data, size_type length = size_type(-1) );
+      void NS_FASTCALL Assign( const self_type& );
+      void NS_FASTCALL Assign( const substring_tuple_type& );
 
-      NS_COM void NS_FASTCALL AssignASCII( const char* data, size_type length );
-      NS_COM void NS_FASTCALL AssignASCII( const char* data );
+      void NS_FASTCALL AssignASCII( const char* data, size_type length );
+      void NS_FASTCALL AssignASCII( const char* data );
 
     // AssignLiteral must ONLY be applied to an actual literal string.
     // Do not attempt to use it with a regular char* pointer, or with a char
@@ -368,19 +366,19 @@ class nsTSubstring_CharT
       self_type& operator=( const self_type& str )                                              { Assign(str);      return *this; }
       self_type& operator=( const substring_tuple_type& tuple )                                 { Assign(tuple);    return *this; }
 
-      NS_COM void NS_FASTCALL Adopt( char_type* data, size_type length = size_type(-1) );
+      void NS_FASTCALL Adopt( char_type* data, size_type length = size_type(-1) );
 
 
         /**
          * buffer manipulation
          */
 
-      NS_COM void NS_FASTCALL Replace( index_type cutStart, size_type cutLength, char_type c );
-      NS_COM void NS_FASTCALL Replace( index_type cutStart, size_type cutLength, const char_type* data, size_type length = size_type(-1) );
+      void NS_FASTCALL Replace( index_type cutStart, size_type cutLength, char_type c );
+      void NS_FASTCALL Replace( index_type cutStart, size_type cutLength, const char_type* data, size_type length = size_type(-1) );
              void Replace( index_type cutStart, size_type cutLength, const self_type& str )      { Replace(cutStart, cutLength, str.Data(), str.Length()); }
-      NS_COM void NS_FASTCALL Replace( index_type cutStart, size_type cutLength, const substring_tuple_type& tuple );
+      void NS_FASTCALL Replace( index_type cutStart, size_type cutLength, const substring_tuple_type& tuple );
 
-      NS_COM void NS_FASTCALL ReplaceASCII( index_type cutStart, size_type cutLength, const char* data, size_type length = size_type(-1) );
+      void NS_FASTCALL ReplaceASCII( index_type cutStart, size_type cutLength, const char* data, size_type length = size_type(-1) );
 
       void Append( char_type c )                                                                 { Replace(mLength, 0, c); }
       void Append( const char_type* data, size_type length = size_type(-1) )                     { Replace(mLength, 0, data, length); }
@@ -389,6 +387,49 @@ class nsTSubstring_CharT
 
       void AppendASCII( const char* data, size_type length = size_type(-1) )                     { ReplaceASCII(mLength, 0, data, length); }
 
+      void AppendPrintf( const char* format, ... );
+      void AppendInt( PRInt32 aInteger )
+                 { AppendPrintf31( "%d", aInteger ); }
+      void AppendInt( PRInt32 aInteger, int aRadix )
+        {
+          const char *fmt = aRadix == 10 ? "%d" : aRadix == 8 ? "%o" : "%x";
+          AppendPrintf31( fmt, aInteger );
+        }
+      void AppendInt( PRUint32 aInteger )
+                 { AppendPrintf31( "%u", aInteger ); }
+      void AppendInt( PRUint32 aInteger, int aRadix )
+        {
+          const char *fmt = aRadix == 10 ? "%u" : aRadix == 8 ? "%o" : "%x";
+          AppendPrintf31( fmt, aInteger );
+        }
+      void AppendInt( PRInt64 aInteger )
+                 { AppendPrintf31( "%lld", aInteger ); }
+      void AppendInt( PRInt64 aInteger, int aRadix )
+        {
+          const char *fmt = aRadix == 10 ? "%lld" : aRadix == 8 ? "%llo" : "%llx";
+          AppendPrintf31( fmt, aInteger );
+        }
+      void AppendInt( PRUint64 aInteger )
+                 { AppendPrintf31( "%llu", aInteger ); }
+      void AppendInt( PRUint64 aInteger, int aRadix )
+        {
+          const char *fmt = aRadix == 10 ? "%llu" : aRadix == 8 ? "%llo" : "%llx";
+          AppendPrintf31( fmt, aInteger );
+        }
+
+      /**
+       * Append the given float to this string 
+       */
+      void AppendFloat( float aFloat )
+                      { DoAppendFloat(aFloat, 6); }
+      void AppendFloat( double aFloat )
+                      { DoAppendFloat(aFloat, 15); }
+  private:
+      void NS_FASTCALL DoAppendFloat( double aFloat, int digits );
+      // AppendPrintf31 truncates output to 31 ASCII characters
+      void AppendPrintf31( const char* format, ... );
+  public:
+
     // AppendLiteral must ONLY be applied to an actual literal string.
     // Do not attempt to use it with a regular char* pointer, or with a char
     // array variable. Use AppendASCII for those.
@@ -429,9 +470,9 @@ class nsTSubstring_CharT
          *          PR_FALSE on out-of-memory, or if requesting a size bigger
          *                   than a string can hold (2^31 chars).
          */
-      NS_COM PRBool NS_FASTCALL SetCapacity( size_type newCapacity );
+      PRBool NS_FASTCALL SetCapacity( size_type newCapacity );
 
-      NS_COM void NS_FASTCALL SetLength( size_type newLength );
+      void NS_FASTCALL SetLength( size_type newLength );
 
       void Truncate( size_type newLength = 0 )
         {
@@ -485,7 +526,7 @@ class nsTSubstring_CharT
          * string will be truncated.  @see nsTSubstring::IsVoid
          */
 
-      NS_COM void NS_FASTCALL SetIsVoid( PRBool );
+      void NS_FASTCALL SetIsVoid( PRBool );
 
         /**
          *  This method is used to remove all occurrences of aChar from this
@@ -495,8 +536,31 @@ class nsTSubstring_CharT
          *  @param  aOffset -- where in this string to start stripping chars
          */
          
-      NS_COM void StripChar( char_type aChar, PRInt32 aOffset=0 );
+      void StripChar( char_type aChar, PRInt32 aOffset=0 );
 
+        /**
+         *  This method is used to remove all occurrences of aChars from this
+         * string.
+         *
+         *  @param  aChars -- chars to be stripped
+         *  @param  aOffset -- where in this string to start stripping chars
+         */
+
+      void StripChars( const char_type* aChars, PRUint32 aOffset=0 );
+
+        /**
+         * If the string uses a shared buffer, this method
+         * clears the pointer without releasing the buffer.
+         */
+      void ForgetSharedBuffer()
+      {
+        if (mFlags & nsSubstring::F_SHARED)
+          {
+            mData = char_traits::sEmptyBuffer;
+            mLength = 0;
+            mFlags = F_TERMINATED;
+          }
+      }
 
     public:
 
@@ -504,7 +568,13 @@ class nsTSubstring_CharT
          * this is public to support automatic conversion of tuple to string
          * base type, which helps avoid converting to nsTAString.
          */
-      NS_COM nsTSubstring_CharT(const substring_tuple_type& tuple);
+      nsTSubstring_CharT(const substring_tuple_type& tuple)
+        : mData(nsnull),
+          mLength(0),
+          mFlags(F_NONE)
+        {
+          Assign(tuple);
+        }
 
         /**
          * allows for direct initialization of a nsTSubstring object. 
@@ -512,11 +582,18 @@ class nsTSubstring_CharT
          * NOTE: this constructor is declared public _only_ for convenience
          * inside the string implementation.
          */
-#ifdef XP_OS2 /* Workaround for GCC 3.3.x bug. */
-       nsTSubstring_CharT( char_type *data, size_type length, PRUint32 flags ) NS_COM;
+        // XXXbz or can I just include nscore.h and use NS_BUILD_REFCNT_LOGGING?
+#if defined(DEBUG) || defined(FORCE_BUILD_REFCNT_LOGGING)
+#define XPCOM_STRING_CONSTRUCTOR_OUT_OF_LINE
+       nsTSubstring_CharT( char_type *data, size_type length, PRUint32 flags );
 #else
-       NS_COM nsTSubstring_CharT( char_type *data, size_type length, PRUint32 flags );
-#endif
+#undef XPCOM_STRING_CONSTRUCTOR_OUT_OF_LINE
+       nsTSubstring_CharT( char_type *data, size_type length, PRUint32 flags )
+         : mData(data),
+           mLength(length),
+           mFlags(flags) {}
+#endif /* DEBUG || FORCE_BUILD_REFCNT_LOGGING */
+
     protected:
 
       friend class nsTObsoleteAStringThunk_CharT;
@@ -530,15 +607,22 @@ class nsTSubstring_CharT
       PRUint32    mFlags;
 
         // default initialization 
-      NS_COM nsTSubstring_CharT();
+      nsTSubstring_CharT()
+        : mData(char_traits::sEmptyBuffer),
+          mLength(0),
+          mFlags(F_TERMINATED) {}
 
         // version of constructor that leaves mData and mLength uninitialized
       explicit
-      NS_COM nsTSubstring_CharT( PRUint32 flags );
+      nsTSubstring_CharT( PRUint32 flags )
+        : mFlags(flags) {}
 
         // copy-constructor, constructs as dependent on given object
         // (NOTE: this is for internal use only)
-      NS_COM nsTSubstring_CharT( const self_type& str );
+      nsTSubstring_CharT( const self_type& str )
+        : mData(str.mData),
+          mLength(str.mLength),
+          mFlags(str.mFlags & (F_TERMINATED | F_VOIDED)) {}
 
         /**
          * this function releases mData and does not change the value of
@@ -587,8 +671,25 @@ class nsTSubstring_CharT
          * this function returns false if is unable to allocate sufficient
          * memory.
          */
-      PRBool NS_FASTCALL ReplacePrep( index_type cutStart, size_type cutLength, size_type newLength );
+      PRBool ReplacePrep(index_type cutStart, size_type cutLength,
+                         size_type newLength)
+      {
+        cutLength = NS_MIN(cutLength, mLength - cutStart);
+        PRUint32 newTotalLen = mLength - cutLength + newLength;
+        if (cutStart == mLength && Capacity() > newTotalLen) {
+          mFlags &= ~F_VOIDED;
+          mData[newTotalLen] = char_type(0);
+          mLength = newTotalLen;
+          return PR_TRUE;
+        }
+        return ReplacePrepInternal(cutStart, cutLength, newLength, newTotalLen);
+      }
 
+      PRBool NS_FASTCALL ReplacePrepInternal(index_type cutStart,
+                                             size_type cutLength,
+                                             size_type newFragLength,
+                                             size_type newTotalLength);
+      
         /**
          * returns the number of writable storage units starting at mData.
          * the value does not include space for the null-terminator character.
@@ -602,7 +703,7 @@ class nsTSubstring_CharT
          * this helper function can be called prior to directly manipulating
          * the contents of mData.  see, for example, BeginWriting.
          */
-      NS_COM PRBool NS_FASTCALL EnsureMutable( size_type newLen = size_type(-1) );
+      PRBool NS_FASTCALL EnsureMutable( size_type newLen = size_type(-1) );
 
         /**
          * returns true if this string overlaps with the given string fragment.
@@ -687,7 +788,6 @@ class nsTSubstring_CharT
       //
   };
 
-NS_COM
 int NS_FASTCALL Compare( const nsTSubstring_CharT::base_string_type& lhs, const nsTSubstring_CharT::base_string_type& rhs, const nsTStringComparator_CharT& = nsTDefaultStringComparator_CharT() );
 
 
