@@ -41,7 +41,6 @@
 #include "nsString.h"
 #include "nsUTF8Utils.h"
 
-NS_COM
 void
 LossyCopyUTF16toASCII( const nsAString& aSource, nsACString& aDest )
   {
@@ -49,7 +48,6 @@ LossyCopyUTF16toASCII( const nsAString& aSource, nsACString& aDest )
     LossyAppendUTF16toASCII(aSource, aDest);
   }
 
-NS_COM
 void
 CopyASCIItoUTF16( const nsACString& aSource, nsAString& aDest )
   {
@@ -57,7 +55,6 @@ CopyASCIItoUTF16( const nsACString& aSource, nsAString& aDest )
     AppendASCIItoUTF16(aSource, aDest);
   }
 
-NS_COM
 void
 LossyCopyUTF16toASCII( const PRUnichar* aSource, nsACString& aDest )
   {
@@ -67,7 +64,6 @@ LossyCopyUTF16toASCII( const PRUnichar* aSource, nsACString& aDest )
     }
   }
 
-NS_COM
 void
 CopyASCIItoUTF16( const char* aSource, nsAString& aDest )
   {
@@ -77,7 +73,6 @@ CopyASCIItoUTF16( const char* aSource, nsAString& aDest )
     }
   }
 
-NS_COM
 void
 CopyUTF16toUTF8( const nsAString& aSource, nsACString& aDest )
   {
@@ -85,7 +80,6 @@ CopyUTF16toUTF8( const nsAString& aSource, nsACString& aDest )
     AppendUTF16toUTF8(aSource, aDest);
   }
 
-NS_COM
 void
 CopyUTF8toUTF16( const nsACString& aSource, nsAString& aDest )
   {
@@ -93,7 +87,6 @@ CopyUTF8toUTF16( const nsACString& aSource, nsAString& aDest )
     AppendUTF8toUTF16(aSource, aDest);
   }
 
-NS_COM
 void
 CopyUTF16toUTF8( const PRUnichar* aSource, nsACString& aDest )
   {
@@ -101,7 +94,6 @@ CopyUTF16toUTF8( const PRUnichar* aSource, nsACString& aDest )
     AppendUTF16toUTF8(aSource, aDest);
   }
 
-NS_COM
 void
 CopyUTF8toUTF16( const char* aSource, nsAString& aDest )
   {
@@ -130,7 +122,6 @@ SetLengthForWritingC(nsACString& aDest, PRUint32 aDesiredLength)
   }
 
 
-NS_COM
 void
 LossyAppendUTF16toASCII( const nsAString& aSource, nsACString& aDest )
   {
@@ -145,13 +136,12 @@ LossyAppendUTF16toASCII( const nsAString& aSource, nsACString& aDest )
 
     dest.advance(old_dest_length);
 
-      // right now, this won't work on multi-fragment destinations
-    LossyConvertEncoding<PRUnichar, char> converter(dest.get());
-    
+    // right now, this won't work on multi-fragment destinations
+    LossyConvertEncoding16to8 converter(dest.get());
+
     copy_string(aSource.BeginReading(fromBegin), aSource.EndReading(fromEnd), converter);
   }
 
-NS_COM
 void
 AppendASCIItoUTF16( const nsACString& aSource, nsAString& aDest )
   {
@@ -167,12 +157,11 @@ AppendASCIItoUTF16( const nsACString& aSource, nsAString& aDest )
     dest.advance(old_dest_length);
 
       // right now, this won't work on multi-fragment destinations
-    LossyConvertEncoding<char, PRUnichar> converter(dest.get());
+    LossyConvertEncoding8to16 converter(dest.get());
 
     copy_string(aSource.BeginReading(fromBegin), aSource.EndReading(fromEnd), converter);
   }
 
-NS_COM
 void
 LossyAppendUTF16toASCII( const PRUnichar* aSource, nsACString& aDest )
   {
@@ -181,7 +170,6 @@ LossyAppendUTF16toASCII( const PRUnichar* aSource, nsACString& aDest )
     }
   }
 
-NS_COM
 void
 AppendASCIItoUTF16( const char* aSource, nsAString& aDest )
   {
@@ -190,7 +178,6 @@ AppendASCIItoUTF16( const char* aSource, nsAString& aDest )
     }
   }
 
-NS_COM
 void
 AppendUTF16toUTF8( const nsAString& aSource, nsACString& aDest )
   {
@@ -209,44 +196,18 @@ AppendUTF16toUTF8( const nsAString& aSource, nsACString& aDest )
         if(!SetLengthForWritingC(aDest, old_dest_length + count))
             return;
 
-        nsACString::iterator dest;
-        aDest.BeginWriting(dest);
+        // All ready? Time to convert
 
-        dest.advance(old_dest_length);
+        ConvertUTF16toUTF8 converter(aDest.BeginWriting() + old_dest_length);
+        copy_string(aSource.BeginReading(source_start),
+                    aSource.EndReading(source_end), converter);
 
-        if (count <= (PRUint32)dest.size_forward())
-          {
-            // aDest has enough room in the fragment just past the end
-            // of its old data that it can hold what we're about to
-            // append. Append using copy_string().
-
-            // All ready? Time to convert
-
-            ConvertUTF16toUTF8 converter(dest.get());
-            copy_string(aSource.BeginReading(source_start),
-                        aSource.EndReading(source_end), converter);
-
-            if (converter.Size() != count)
-              {
-                NS_ERROR("Input invalid or incorrect length was calculated");
-
-                aDest.SetLength(old_dest_length);
-              }
-          }
-        else
-          {
-            // This isn't the fastest way to do this, but it gets
-            // complicated to convert UTF16 into a fragmented UTF8
-            // string, so we'll take the easy way out here in this
-            // rare situation.
-
-            aDest.Replace(old_dest_length, count,
-                          NS_ConvertUTF16toUTF8(aSource));
-          }
+        NS_ASSERTION(converter.Size() == count,
+                     "Unexpected disparity between CalculateUTF8Size and "
+                     "ConvertUTF16toUTF8");
       }
   }
 
-NS_COM
 void
 AppendUTF8toUTF16( const nsACString& aSource, nsAString& aDest )
   {
@@ -257,51 +218,33 @@ AppendUTF8toUTF16( const nsACString& aSource, nsAString& aDest )
 
     PRUint32 count = calculator.Length();
 
+    // Avoid making the string mutable if we're appending an empty string
     if (count)
       {
         PRUint32 old_dest_length = aDest.Length();
 
         // Grow the buffer if we need to.
         if(!SetLengthForWriting(aDest, old_dest_length + count))
-            return;
+          return;
 
-        nsAString::iterator dest;
-        aDest.BeginWriting(dest);
+        // All ready? Time to convert
 
-        dest.advance(old_dest_length);
+        ConvertUTF8toUTF16 converter(aDest.BeginWriting() + old_dest_length);
+        copy_string(aSource.BeginReading(source_start),
+                    aSource.EndReading(source_end), converter);
 
-        if (count <= (PRUint32)dest.size_forward())
+        NS_ASSERTION(converter.ErrorEncountered() ||
+                     converter.Length() == count,
+                     "CalculateUTF8Length produced the wrong length");
+
+        if (converter.ErrorEncountered())
           {
-            // aDest has enough room in the fragment just past the end
-            // of its old data that it can hold what we're about to
-            // append. Append using copy_string().
-
-            // All ready? Time to convert
-
-            ConvertUTF8toUTF16 converter(dest.get());
-            copy_string(aSource.BeginReading(source_start),
-                        aSource.EndReading(source_end), converter);
-
-            if (converter.Length() != count)
-              {
-                NS_ERROR("Input wasn't UTF8 or incorrect length was calculated");
-                aDest.SetLength(old_dest_length);
-              }
-          }
-        else
-          {
-            // This isn't the fastest way to do this, but it gets
-            // complicated to convert parts of a UTF8 string into a
-            // UTF16 string, so we'll take the easy way out here in
-            // this rare situation.
-
-            aDest.Replace(old_dest_length, count,
-                          NS_ConvertUTF8toUTF16(aSource));
+            NS_ERROR("Input wasn't UTF8 or incorrect length was calculated");
+            aDest.SetLength(old_dest_length);
           }
       }
   }
 
-NS_COM
 void
 AppendUTF16toUTF8( const PRUnichar* aSource, nsACString& aDest )
   {
@@ -310,7 +253,6 @@ AppendUTF16toUTF8( const PRUnichar* aSource, nsACString& aDest )
     }
   }
 
-NS_COM
 void
 AppendUTF8toUTF16( const char* aSource, nsAString& aDest )
   {
@@ -336,7 +278,6 @@ AllocateStringCopy( const FromStringT& aSource, ToCharT* )
   }
 
 
-NS_COM
 char*
 ToNewCString( const nsAString& aSource )
   {
@@ -345,12 +286,11 @@ ToNewCString( const nsAString& aSource )
       return nsnull;
 
     nsAString::const_iterator fromBegin, fromEnd;
-    LossyConvertEncoding<PRUnichar, char> converter(result);
+    LossyConvertEncoding16to8 converter(result);
     copy_string(aSource.BeginReading(fromBegin), aSource.EndReading(fromEnd), converter).write_terminator();
     return result;
   }
 
-NS_COM
 char*
 ToNewUTF8String( const nsAString& aSource, PRUint32 *aUTF8Count )
   {
@@ -375,7 +315,6 @@ ToNewUTF8String( const nsAString& aSource, PRUint32 *aUTF8Count )
     return result;
   }
 
-NS_COM
 char*
 ToNewCString( const nsACString& aSource )
   {
@@ -391,7 +330,6 @@ ToNewCString( const nsACString& aSource )
     return result;
   }
 
-NS_COM
 PRUnichar*
 ToNewUnicode( const nsAString& aSource )
   {
@@ -407,7 +345,6 @@ ToNewUnicode( const nsAString& aSource )
     return result;
   }
 
-NS_COM
 PRUnichar*
 ToNewUnicode( const nsACString& aSource )
   {
@@ -416,12 +353,11 @@ ToNewUnicode( const nsACString& aSource )
       return nsnull;
 
     nsACString::const_iterator fromBegin, fromEnd;
-    LossyConvertEncoding<char, PRUnichar> converter(result);
+    LossyConvertEncoding8to16 converter(result);
     copy_string(aSource.BeginReading(fromBegin), aSource.EndReading(fromEnd), converter).write_terminator();
     return result;
   }
 
-NS_COM
 PRUnichar*
 UTF8ToNewUnicode( const nsACString& aSource, PRUint32 *aUTF16Count )
   {
@@ -446,7 +382,6 @@ UTF8ToNewUnicode( const nsACString& aSource, PRUint32 *aUTF16Count )
     return result;
   }
 
-NS_COM
 PRUnichar*
 CopyUnicodeTo( const nsAString& aSource, PRUint32 aSrcOffset, PRUnichar* aDest, PRUint32 aLength )
   {
@@ -456,8 +391,7 @@ CopyUnicodeTo( const nsAString& aSource, PRUint32 aSrcOffset, PRUnichar* aDest,
     return aDest;
   }
 
-NS_COM 
-void 
+void
 CopyUnicodeTo( const nsAString::const_iterator& aSrcStart,
                const nsAString::const_iterator& aSrcEnd,
                nsAString& aDest )
@@ -472,8 +406,7 @@ CopyUnicodeTo( const nsAString::const_iterator& aSrcStart,
     copy_string(fromBegin, aSrcEnd, writer);
   }
 
-NS_COM 
-void 
+void
 AppendUnicodeTo( const nsAString::const_iterator& aSrcStart,
                  const nsAString::const_iterator& aSrcEnd,
                  nsAString& aDest )
@@ -489,7 +422,6 @@ AppendUnicodeTo( const nsAString::const_iterator& aSrcStart,
     copy_string(fromBegin, aSrcEnd, writer);
   }
 
-NS_COM
 PRBool
 IsASCII( const nsAString& aString )
   {
@@ -514,7 +446,6 @@ IsASCII( const nsAString& aString )
     return PR_TRUE;
   }
 
-NS_COM
 PRBool
 IsASCII( const nsACString& aString )
   {
@@ -539,9 +470,8 @@ IsASCII( const nsACString& aString )
     return PR_TRUE;
   }
 
-NS_COM
 PRBool
-IsUTF8( const nsACString& aString )
+IsUTF8( const nsACString& aString, PRBool aRejectNonChar )
   {
     nsReadingIterator<char> done_reading;
     aString.EndReading(done_reading);
@@ -609,20 +539,25 @@ IsUTF8( const nsACString& aString )
               return PR_FALSE; // Not UTF-8 string
           }
           
+        if (nonchar && !aRejectNonChar)
+          nonchar = PR_FALSE;
+
         while ( ptr < end && state )
           {
             c = *ptr++;
             --state;
 
             // non-character : EF BF [BE-BF] or F[0-7] [89AB]F BF [BE-BF]
-            if ( nonchar &&  ( !state &&  c < 0xBE ||
-                  state == 1 && c != 0xBF  ||
-                  state == 2 && 0x0F != (0x0F & c) ))
-                nonchar = PR_FALSE;
+            if ( nonchar &&  
+                 ( ( !state && c < 0xBE ) ||
+                   ( state == 1 && c != 0xBF )  ||
+                   ( state == 2 && 0x0F != (0x0F & c) )))
+              nonchar = PR_FALSE;
 
-            if ( !UTF8traits::isInSeq(c) || overlong && c <= olupper || 
-                  surrogate && slower <= c || nonchar && !state )
+            if ( !UTF8traits::isInSeq(c) || ( overlong && c <= olupper ) || 
+                 ( surrogate && slower <= c ) || ( nonchar && !state ))
               return PR_FALSE; // Not UTF-8 string
+
             overlong = surrogate = PR_FALSE;
           }
         }
@@ -652,7 +587,6 @@ class ConvertToUpperCase
         }
   };
 
-NS_COM
 void
 ToUpperCase( nsCSubstring& aCString )
   {
@@ -677,7 +611,7 @@ class CopyToUpperCase
       PRUint32
       write( const char* aSource, PRUint32 aSourceLength )
         {
-          PRUint32 len = PR_MIN(PRUint32(mIter.size_forward()), aSourceLength);
+          PRUint32 len = NS_MIN(PRUint32(mIter.size_forward()), aSourceLength);
           char* cp = mIter.get();
           const char* end = aSource + len;
           while (aSource != end) {
@@ -697,7 +631,6 @@ class CopyToUpperCase
       nsACString::iterator& mIter;
   };
 
-NS_COM
 void
 ToUpperCase( const nsACString& aSource, nsACString& aDest )
   {
@@ -733,7 +666,6 @@ class ConvertToLowerCase
         }
   };
 
-NS_COM
 void
 ToLowerCase( nsCSubstring& aCString )
   {
@@ -758,7 +690,7 @@ class CopyToLowerCase
       PRUint32
       write( const char* aSource, PRUint32 aSourceLength )
         {
-          PRUint32 len = PR_MIN(PRUint32(mIter.size_forward()), aSourceLength);
+          PRUint32 len = NS_MIN(PRUint32(mIter.size_forward()), aSourceLength);
           char* cp = mIter.get();
           const char* end = aSource + len;
           while (aSource != end) {
@@ -778,7 +710,6 @@ class CopyToLowerCase
       nsACString::iterator& mIter;
   };
 
-NS_COM
 void
 ToLowerCase( const nsACString& aSource, nsACString& aDest )
   {
@@ -791,7 +722,6 @@ ToLowerCase( const nsACString& aSource, nsACString& aDest )
     copy_string(aSource.BeginReading(fromBegin), aSource.EndReading(fromEnd), converter);
   }
 
-NS_COM
 PRBool
 ParseString(const nsACString& aSource, char aDelimiter, 
             nsTArray<nsCString>& aArray)
@@ -844,7 +774,7 @@ FindInReadable_Impl( const StringT& aPattern, IteratorT& aSearchStart, IteratorT
           {
               // fast inner loop (that's what it's called, not what it is) looks for a potential match
             while ( aSearchStart != aSearchEnd &&
-                    compare(*aPatternStart, *aSearchStart) )
+                    compare(aPatternStart.get(), aSearchStart.get(), 1, 1) )
               ++aSearchStart;
 
               // if we broke out of the `fast' loop because we're out of string ... we're done: no match
@@ -881,7 +811,7 @@ FindInReadable_Impl( const StringT& aPattern, IteratorT& aSearchStart, IteratorT
 
                   // else if we mismatched ... it's time to advance to the next search position
                   //  and get back into the `fast' loop
-                if ( compare(*testPattern, *testSearch) )
+                if ( compare(testPattern.get(), testSearch.get(), 1, 1) )
                   {
                     ++aSearchStart;
                     break;
@@ -913,7 +843,7 @@ RFindInReadable_Impl( const StringT& aPattern, IteratorT& aSearchStart, Iterator
         --searchEnd;
     
           // Check last character, if a match, explore further from here
-        if ( compare(*patternEnd, *searchEnd) == 0 )
+        if ( compare(patternEnd.get(), searchEnd.get(), 1, 1) == 0 )
           {  
               // We're at a potential match, let's see if we really hit one
             IteratorT testPattern(patternEnd);
@@ -942,7 +872,7 @@ RFindInReadable_Impl( const StringT& aPattern, IteratorT& aSearchStart, Iterator
                 --testPattern;
                 --testSearch;
               }
-            while ( compare(*testPattern, *testSearch) == 0 );
+            while ( compare(testPattern.get(), testSearch.get(), 1, 1) == 0 );
           }
       }
 
@@ -950,43 +880,37 @@ RFindInReadable_Impl( const StringT& aPattern, IteratorT& aSearchStart, Iterator
     return PR_FALSE;
   }
 
-NS_COM
 PRBool
 FindInReadable( const nsAString& aPattern, nsAString::const_iterator& aSearchStart, nsAString::const_iterator& aSearchEnd, const nsStringComparator& aComparator )
   {
     return FindInReadable_Impl(aPattern, aSearchStart, aSearchEnd, aComparator);
   }
 
-NS_COM
 PRBool
 FindInReadable( const nsACString& aPattern, nsACString::const_iterator& aSearchStart, nsACString::const_iterator& aSearchEnd, const nsCStringComparator& aComparator)
   {
     return FindInReadable_Impl(aPattern, aSearchStart, aSearchEnd, aComparator);
   }
 
-NS_COM
 PRBool
 CaseInsensitiveFindInReadable( const nsACString& aPattern, nsACString::const_iterator& aSearchStart, nsACString::const_iterator& aSearchEnd )
   {
     return FindInReadable_Impl(aPattern, aSearchStart, aSearchEnd, nsCaseInsensitiveCStringComparator());
   }
 
-NS_COM
 PRBool
 RFindInReadable( const nsAString& aPattern, nsAString::const_iterator& aSearchStart, nsAString::const_iterator& aSearchEnd, const nsStringComparator& aComparator)
   {
     return RFindInReadable_Impl(aPattern, aSearchStart, aSearchEnd, aComparator);
   }
 
-NS_COM
 PRBool
 RFindInReadable( const nsACString& aPattern, nsACString::const_iterator& aSearchStart, nsACString::const_iterator& aSearchEnd, const nsCStringComparator& aComparator)
   {
     return RFindInReadable_Impl(aPattern, aSearchStart, aSearchEnd, aComparator);
   }
 
-NS_COM 
-PRBool 
+PRBool
 FindCharInReadable( PRUnichar aChar, nsAString::const_iterator& aSearchStart, const nsAString::const_iterator& aSearchEnd )
   {
     PRInt32 fragmentLength = aSearchEnd.get() - aSearchStart.get();
@@ -1001,8 +925,7 @@ FindCharInReadable( PRUnichar aChar, nsAString::const_iterator& aSearchStart, co
     return PR_FALSE;
   }
 
-NS_COM 
-PRBool 
+PRBool
 FindCharInReadable( char aChar, nsACString::const_iterator& aSearchStart, const nsACString::const_iterator& aSearchEnd )
   {
     PRInt32 fragmentLength = aSearchEnd.get() - aSearchStart.get();
@@ -1015,10 +938,9 @@ FindCharInReadable( char aChar, nsACString::const_iterator& aSearchStart, const
 
     aSearchStart.advance(fragmentLength);
     return PR_FALSE;
-  } 
+  }
 
-NS_COM 
-PRUint32 
+PRUint32
 CountCharInReadable( const nsAString& aStr,
                      PRUnichar aChar )
 {
@@ -1038,8 +960,7 @@ CountCharInReadable( const nsAString& aStr,
   return count;
 }
 
-NS_COM 
-PRUint32 
+PRUint32
 CountCharInReadable( const nsACString& aStr,
                      char aChar )
 {
@@ -1059,7 +980,7 @@ CountCharInReadable( const nsACString& aStr,
   return count;
 }
 
-NS_COM PRBool
+PRBool
 StringBeginsWith( const nsAString& aSource, const nsAString& aSubstring,
                   const nsStringComparator& aComparator )
   {
@@ -1070,7 +991,7 @@ StringBeginsWith( const nsAString& aSource, const nsAString& aSubstring,
     return Substring(aSource, 0, sub_len).Equals(aSubstring, aComparator);
   }
 
-NS_COM PRBool
+PRBool
 StringBeginsWith( const nsACString& aSource, const nsACString& aSubstring,
                   const nsCStringComparator& aComparator )
   {
@@ -1081,7 +1002,7 @@ StringBeginsWith( const nsACString& aSource, const nsACString& aSubstring,
     return Substring(aSource, 0, sub_len).Equals(aSubstring, aComparator);
   }
 
-NS_COM PRBool
+PRBool
 StringEndsWith( const nsAString& aSource, const nsAString& aSubstring,
                 const nsStringComparator& aComparator )
   {
@@ -1093,7 +1014,7 @@ StringEndsWith( const nsAString& aSource, const nsAString& aSubstring,
                                                                  aComparator);
   }
 
-NS_COM PRBool
+PRBool
 StringEndsWith( const nsACString& aSource, const nsACString& aSubstring,
                 const nsCStringComparator& aComparator )
   {
@@ -1109,7 +1030,6 @@ StringEndsWith( const nsACString& aSource, const nsACString& aSubstring,
 
 static const PRUnichar empty_buffer[1] = { '\0' };
 
-NS_COM
 const nsAFlatString&
 EmptyString()
   {
@@ -1118,7 +1038,6 @@ EmptyString()
     return sEmpty;
   }
 
-NS_COM
 const nsAFlatCString&
 EmptyCString()
   {
@@ -1127,7 +1046,7 @@ EmptyCString()
     return sEmpty;
   }
 
-NS_COM PRInt32
+PRInt32
 CompareUTF8toUTF16(const nsASingleFragmentCString& aUTF8String,
                    const nsASingleFragmentString& aUTF16String)
   {
@@ -1204,7 +1123,6 @@ CompareUTF8toUTF16(const nsASingleFragmentCString& aUTF8String,
     return 0;
   }
 
-NS_COM
 void
 AppendUCS4ToUTF16(const PRUint32 aSource, nsAString& aDest)
   {
