@@ -36,16 +36,41 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "mozilla/ReentrantMonitor.h"
 #include "nsThread.h"
 #include "nsThreadManager.h"
 #include "nsIClassInfoImpl.h"
 #include "nsIProgrammingLanguage.h"
-#include "nsAutoLock.h"
 #include "nsAutoPtr.h"
 #include "nsCOMPtr.h"
 #include "prlog.h"
 #include "nsThreadUtilsInternal.h"
 
+#define HAVE_UALARM _BSD_SOURCE || (_XOPEN_SOURCE >= 500 ||                 \
+                      _XOPEN_SOURCE && _XOPEN_SOURCE_EXTENDED) &&           \
+                      !(_POSIX_C_SOURCE >= 200809L || _XOPEN_SOURCE >= 700)
+
+#if defined(XP_UNIX) && !defined(ANDROID) && !defined(DEBUG) && HAVE_UALARM \
+  && defined(_GNU_SOURCE)
+# define MOZ_CANARY
+# include <unistd.h>
+# include <execinfo.h>
+# include <signal.h>
+# include <fcntl.h>
+# include "nsXULAppAPI.h"
+#endif
+
+#include "mozilla/FunctionTimer.h"
+#if defined(NS_FUNCTION_TIMER) && defined(_MSC_VER)
+#include "nsTimerImpl.h"
+#include "nsStackWalk.h"
+#endif
+#ifdef NS_FUNCTION_TIMER
+#include "nsCRT.h"
+#endif
+
+using namespace mozilla;
+
 #ifdef PR_LOGGING
 static PRLogModuleInfo *sLog = PR_NewLogModule("nsThread");
 #endif
@@ -150,12 +175,7 @@ class nsThreadStartupEvent : public nsRunnable {
 public:
   // Create a new thread startup object.
   static nsThreadStartupEvent *Create() {
-    nsThreadStartupEvent *startup = new nsThreadStartupEvent();
-    if (startup && startup->mMon)
-      return startup;
-    // Allocation failure
-    delete startup;
-    return nsnull;
+    return new nsThreadStartupEvent();
   }
 
   // This method does not return until the thread startup object is in the
@@ -163,7 +183,7 @@ public:
   void Wait() {
     if (mInitialized)  // Maybe avoid locking...
       return;
-    nsAutoMonitor mon(mMon);
+    ReentrantMonitorAutoEnter mon(mMon);
     while (!mInitialized)
       mon.Wait();
   }
@@ -171,24 +191,22 @@ public:
   // This method needs to be public to support older compilers (xlC_r on AIX).
   // It should be called directly as this class type is reference counted.
   virtual ~nsThreadStartupEvent() {
-    if (mMon)
-      nsAutoMonitor::DestroyMonitor(mMon);
   }
 
 private:
   NS_IMETHOD Run() {
-    nsAutoMonitor mon(mMon);
+    ReentrantMonitorAutoEnter mon(mMon);
     mInitialized = PR_TRUE;
     mon.Notify();
     return NS_OK;
   }
 
   nsThreadStartupEvent()
-    : mMon(nsAutoMonitor::NewMonitor("xpcom.threadstartup"))
+    : mMon("nsThreadStartupEvent.mMon")
     , mInitialized(PR_FALSE) {
   }
 
-  PRMonitor *mMon;
+  ReentrantMonitor mMon;
   PRBool     mInitialized;
 };
 
@@ -260,7 +278,7 @@ nsThread::ThreadFunc(void *arg)
   // NS_ProcessPendingEvents.
   while (PR_TRUE) {
     {
-      nsAutoLock lock(self->mLock);
+      MutexAutoLock lock(self->mLock);
       if (!self->mEvents->HasPendingEvent()) {
         // No events in the queue, so we will stop now. Don't let any more
         // events be added, since they won't be processed. It is critical
@@ -289,11 +307,25 @@ nsThread::ThreadFunc(void *arg)
 //-----------------------------------------------------------------------------
 
 nsThread::nsThread()
-  : mLock(PR_NewLock())
+  : mLock("nsThread.mLock")
   , mEvents(&mEventsRoot)
   , mPriority(PRIORITY_NORMAL)
   , mThread(nsnull)
   , mRunningEvent(0)
+  , mStackSize(0)
+  , mShutdownContext(nsnull)
+  , mShutdownRequired(PR_FALSE)
+  , mEventsAreDoomed(PR_FALSE)
+{
+}
+
+nsThread::nsThread(PRUint32 aStackSize)
+  : mLock("nsThread.mLock")
+  , mEvents(&mEventsRoot)
+  , mPriority(PRIORITY_NORMAL)
+  , mThread(nsnull)
+  , mRunningEvent(0)
+  , mStackSize(aStackSize)
   , mShutdownContext(nsnull)
   , mShutdownRequired(PR_FALSE)
   , mEventsAreDoomed(PR_FALSE)
@@ -302,15 +334,11 @@ nsThread::nsThread()
 
 nsThread::~nsThread()
 {
-  if (mLock)
-    PR_DestroyLock(mLock);
 }
 
 nsresult
 nsThread::Init()
 {
-  NS_ENSURE_TRUE(mLock, NS_ERROR_OUT_OF_MEMORY);
-
   // spawn thread and wait until it is fully setup
   nsRefPtr<nsThreadStartupEvent> startup = nsThreadStartupEvent::Create();
   NS_ENSURE_TRUE(startup, NS_ERROR_OUT_OF_MEMORY);
@@ -322,7 +350,7 @@ nsThread::Init()
   // ThreadFunc is responsible for setting mThread
   PRThread *thr = PR_CreateThread(PR_USER_THREAD, ThreadFunc, this,
                                   PR_PRIORITY_NORMAL, PR_GLOBAL_THREAD,
-                                  PR_JOINABLE_THREAD, 0);
+                                  PR_JOINABLE_THREAD, mStackSize);
   if (!thr) {
     NS_RELEASE_THIS();
     return NS_ERROR_OUT_OF_MEMORY;
@@ -332,7 +360,7 @@ nsThread::Init()
   // mThread.  By delaying insertion of this event into the queue, we ensure
   // that mThread is set properly.
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     mEvents->PutEvent(startup);
   }
 
@@ -345,8 +373,6 @@ nsThread::Init()
 nsresult
 nsThread::InitCurrentThread()
 {
-  NS_ENSURE_TRUE(mLock, NS_ERROR_OUT_OF_MEMORY);
-
   mThread = PR_GetCurrentThread();
 
   nsThreadManager::get()->RegisterCurrentThread(this);
@@ -357,7 +383,7 @@ nsresult
 nsThread::PutEvent(nsIRunnable *event)
 {
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     if (mEventsAreDoomed) {
       NS_WARNING("An event was posted to a thread that will never run it (rejected)");
       return NS_ERROR_UNEXPECTED;
@@ -402,7 +428,7 @@ nsThread::Dispatch(nsIRunnable *event, PRUint32 flags)
 
     while (wrapper->IsPending())
       NS_ProcessNextEvent(thread);
-    return rv;
+    return wrapper->Result();
   }
 
   NS_ASSERTION(flags == NS_DISPATCH_NORMAL, "unexpected dispatch flags");
@@ -441,7 +467,7 @@ nsThread::Shutdown()
 
   // Prevent multiple calls to this method
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     if (!mShutdownRequired)
       return NS_ERROR_UNEXPECTED;
     mShutdownRequired = PR_FALSE;
@@ -474,7 +500,7 @@ nsThread::Shutdown()
 
 #ifdef DEBUG
   {
-    nsAutoLock lock(mLock);
+    MutexAutoLock lock(mLock);
     NS_ASSERTION(!mObserver, "Should have been cleared at shutdown!");
   }
 #endif
@@ -491,6 +517,67 @@ nsThread::HasPendingEvents(PRBool *result)
   return NS_OK;
 }
 
+#ifdef MOZ_CANARY
+void canary_alarm_handler (int signum);
+
+class Canary {
+//XXX ToDo: support nested loops
+public:
+  Canary() {
+    if (sOutputFD != 0 && EventLatencyIsImportant()) {
+      if (sOutputFD == -1) {
+        const int flags = O_WRONLY | O_APPEND | O_CREAT | O_NONBLOCK;
+        const mode_t mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
+        char* env_var_flag = getenv("MOZ_KILL_CANARIES");
+        sOutputFD = env_var_flag ? (env_var_flag[0] ?
+                                    open(env_var_flag, flags, mode) :
+                                    STDERR_FILENO) : 0;
+        if (sOutputFD == 0)
+          return;
+      }
+      signal(SIGALRM, canary_alarm_handler);
+      ualarm(15000, 0);      
+    }
+  }
+
+  ~Canary() {
+    if (sOutputFD != 0 && EventLatencyIsImportant())
+      ualarm(0, 0);
+  }
+
+  static bool EventLatencyIsImportant() {
+    return NS_IsMainThread() && XRE_GetProcessType() == GeckoProcessType_Default;
+  }
+
+  static int sOutputFD;
+};
+
+int Canary::sOutputFD = -1;
+
+void canary_alarm_handler (int signum)
+{
+  void *array[30];
+  const char msg[29] = "event took too long to run:\n";
+  // use write to be safe in the signal handler
+  write(Canary::sOutputFD, msg, sizeof(msg)); 
+  backtrace_symbols_fd(array, backtrace(array, 30), Canary::sOutputFD);
+}
+
+#endif
+
+#define NOTIFY_EVENT_OBSERVERS(func_, params_)                                 \
+  PR_BEGIN_MACRO                                                               \
+    if (!mEventObservers.IsEmpty()) {                                          \
+      nsAutoTObserverArray<nsCOMPtr<nsIThreadObserver>, 2>::ForwardIterator    \
+        iter_(mEventObservers);                                                \
+      nsCOMPtr<nsIThreadObserver> obs_;                                        \
+      while (iter_.HasMore()) {                                                \
+        obs_ = iter_.GetNext();                                                \
+        obs_ -> func_ params_ ;                                                \
+      }                                                                        \
+    }                                                                          \
+  PR_END_MACRO
+
 NS_IMETHODIMP
 nsThread::ProcessNextEvent(PRBool mayWait, PRBool *result)
 {
@@ -507,8 +594,14 @@ nsThread::ProcessNextEvent(PRBool mayWait, PRBool *result)
   if (obs)
     obs->OnProcessNextEvent(this, mayWait && !ShuttingDown(), mRunningEvent);
 
+  NOTIFY_EVENT_OBSERVERS(OnProcessNextEvent,
+                         (this, mayWait && !ShuttingDown(), mRunningEvent));
+
   ++mRunningEvent;
 
+#ifdef MOZ_CANARY
+  Canary canary;
+#endif
   nsresult rv = NS_OK;
 
   {
@@ -520,6 +613,17 @@ nsThread::ProcessNextEvent(PRBool mayWait, PRBool *result)
     nsCOMPtr<nsIRunnable> event;
     mEvents->GetEvent(mayWait && !ShuttingDown(), getter_AddRefs(event));
 
+#ifdef NS_FUNCTION_TIMER
+    char message[1024] = {'\0'};
+    if (NS_IsMainThread()) {
+        mozilla::FunctionTimer::ft_snprintf(message, sizeof(message), 
+                                            "@ Main Thread Event %p", (void*)event.get());
+    }
+    // If message is empty, it means that we're not on the main thread, and
+    // FunctionTimer won't time this function.
+    NS_TIME_FUNCTION_MIN_FMT(5.0, message);
+#endif
+
     *result = (event.get() != nsnull);
 
     if (event) {
@@ -533,6 +637,9 @@ nsThread::ProcessNextEvent(PRBool mayWait, PRBool *result)
   }
 
   --mRunningEvent;
+
+  NOTIFY_EVENT_OBSERVERS(AfterProcessNextEvent, (this, mRunningEvent));
+
   if (obs)
     obs->AfterProcessNextEvent(this, mRunningEvent);
 
@@ -593,7 +700,7 @@ nsThread::AdjustPriority(PRInt32 delta)
 NS_IMETHODIMP
 nsThread::GetObserver(nsIThreadObserver **obs)
 {
-  nsAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   NS_IF_ADDREF(*obs = mObserver);
   return NS_OK;
 }
@@ -603,7 +710,7 @@ nsThread::SetObserver(nsIThreadObserver *obs)
 {
   NS_ENSURE_STATE(PR_GetCurrentThread() == mThread);
 
-  nsAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   mObserver = obs;
   return NS_OK;
 }
@@ -612,12 +719,8 @@ NS_IMETHODIMP
 nsThread::PushEventQueue(nsIThreadEventFilter *filter)
 {
   nsChainedEventQueue *queue = new nsChainedEventQueue(filter);
-  if (!queue || !queue->IsInitialized()) {
-    delete queue;
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
 
-  nsAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
   queue->mNext = mEvents;
   mEvents = queue;
   return NS_OK;
@@ -626,7 +729,7 @@ nsThread::PushEventQueue(nsIThreadEventFilter *filter)
 NS_IMETHODIMP
 nsThread::PopEventQueue()
 {
-  nsAutoLock lock(mLock);
+  MutexAutoLock lock(mLock);
 
   // Make sure we do not pop too many!
   NS_ENSURE_STATE(mEvents != &mEventsRoot);
@@ -655,13 +758,52 @@ nsThread::nsChainedEventQueue::PutEvent(nsIRunnable *event)
   return val;
 }
 
+NS_IMETHODIMP
+nsThread::GetRecursionDepth(PRUint32 *depth)
+{
+  NS_ENSURE_ARG_POINTER(depth);
+  NS_ENSURE_STATE(PR_GetCurrentThread() == mThread);
+
+  *depth = mRunningEvent;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsThread::AddObserver(nsIThreadObserver *observer)
+{
+  NS_ENSURE_ARG_POINTER(observer);
+  NS_ENSURE_STATE(PR_GetCurrentThread() == mThread);
+
+  NS_WARN_IF_FALSE(!mEventObservers.Contains(observer),
+                   "Adding an observer twice!");
+
+  if (!mEventObservers.AppendElement(observer)) {
+    NS_WARNING("Out of memory!");
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsThread::RemoveObserver(nsIThreadObserver *observer)
+{
+  NS_ENSURE_STATE(PR_GetCurrentThread() == mThread);
+
+  if (observer && !mEventObservers.RemoveElement(observer)) {
+    NS_WARNING("Removing an observer that was never added!");
+  }
+
+  return NS_OK;
+}
+
 //-----------------------------------------------------------------------------
 
 NS_IMETHODIMP
 nsThreadSyncDispatch::Run()
 {
   if (mSyncTask) {
-    mSyncTask->Run();
+    mResult = mSyncTask->Run();
     mSyncTask = nsnull;
     // unblock the origin thread
     mOrigin->Dispatch(this, NS_DISPATCH_NORMAL);
