@@ -40,12 +40,14 @@
 
 #include "nsTimerImpl.h"
 #include "TimerThread.h"
-#include "nsAutoLock.h"
 #include "nsAutoPtr.h"
 #include "nsThreadManager.h"
 #include "nsThreadUtils.h"
 #include "prmem.h"
 
+using mozilla::TimeDuration;
+using mozilla::TimeStamp;
+
 static PRInt32          gGenerator = 0;
 static TimerThread*     gThread = nsnull;
 
@@ -84,14 +86,14 @@ NS_IMETHODIMP_(nsrefcnt) nsTimerImpl::Release(void)
   nsrefcnt count;
 
   NS_PRECONDITION(0 != mRefCnt, "dup release");
-  count = PR_AtomicDecrement((PRInt32 *)&mRefCnt);
+  count = NS_AtomicDecrementRefcnt(mRefCnt);
   NS_LOG_RELEASE(this, count, "nsTimerImpl");
   if (count == 0) {
     mRefCnt = 1; /* stabilize */
 
     /* enable this to find non-threadsafe destructors: */
     /* NS_ASSERT_OWNINGTHREAD(nsTimerImpl); */
-    NS_DELETEXPCOM(this);
+    delete this;
     return 0;
   }
 
@@ -142,18 +144,12 @@ nsTimerImpl::nsTimerImpl() :
   mArmed(PR_FALSE),
   mCanceled(PR_FALSE),
   mGeneration(0),
-  mDelay(0),
-  mTimeout(0)
+  mDelay(0)
 {
   // XXXbsmedberg: shouldn't this be in Init()?
   mEventTarget = static_cast<nsIEventTarget*>(NS_GetCurrentThread());
 
   mCallback.c = nsnull;
-
-#ifdef DEBUG_TIMERS
-  mStart = 0;
-  mStart2 = 0;
-#endif
 }
 
 nsTimerImpl::~nsTimerImpl()
@@ -226,7 +222,7 @@ nsresult nsTimerImpl::InitCommon(PRUint32 aType, PRUint32 aDelay)
   if (mArmed)
     gThread->RemoveTimer(this);
   mCanceled = PR_FALSE;
-  mGeneration = PR_AtomicIncrement(&gGenerator);
+  mGeneration = PR_ATOMIC_INCREMENT(&gGenerator);
 
   mType = (PRUint8)aType;
   SetDelayInternal(aDelay);
@@ -301,8 +297,8 @@ NS_IMETHODIMP nsTimerImpl::SetDelay(PRUint32 aDelay)
 
   // If we're already repeating precisely, update mTimeout now so that the
   // new delay takes effect in the future.
-  if (mTimeout != 0 && mType == TYPE_REPEATING_PRECISE)
-    mTimeout = PR_IntervalNow();
+  if (!mTimeout.IsNull() && mType == TYPE_REPEATING_PRECISE)
+    mTimeout = TimeStamp::Now();
 
   SetDelayInternal(aDelay);
 
@@ -379,31 +375,32 @@ void nsTimerImpl::Fire()
   if (mCanceled)
     return;
 
-  PRIntervalTime now = PR_IntervalNow();
+  TimeStamp now = TimeStamp::Now();
 #ifdef DEBUG_TIMERS
   if (PR_LOG_TEST(gTimerLog, PR_LOG_DEBUG)) {
-    PRIntervalTime a = now - mStart; // actual delay in intervals
-    PRUint32       b = PR_MillisecondsToInterval(mDelay); // expected delay in intervals
-    PRUint32       d = PR_IntervalToMilliseconds((a > b) ? a - b : b - a); // delta in ms
+    TimeDuration   a = now - mStart; // actual delay in intervals
+    TimeDuration   b = TimeDuration::FromMilliseconds(mDelay); // expected delay in intervals
+    TimeDuration   delta = (a > b) ? a - b : b - a;
+    PRUint32       d = delta.ToMilliseconds(); // delta in ms
     sDeltaSum += d;
     sDeltaSumSquared += double(d) * double(d);
     sDeltaNum++;
 
-    PR_LOG(gTimerLog, PR_LOG_DEBUG, ("[this=%p] expected delay time %4dms\n", this, mDelay));
-    PR_LOG(gTimerLog, PR_LOG_DEBUG, ("[this=%p] actual delay time   %4dms\n", this, PR_IntervalToMilliseconds(a)));
+    PR_LOG(gTimerLog, PR_LOG_DEBUG, ("[this=%p] expected delay time %4ums\n", this, mDelay));
+    PR_LOG(gTimerLog, PR_LOG_DEBUG, ("[this=%p] actual delay time   %fms\n", this, a.ToMilliseconds()));
     PR_LOG(gTimerLog, PR_LOG_DEBUG, ("[this=%p] (mType is %d)       -------\n", this, mType));
     PR_LOG(gTimerLog, PR_LOG_DEBUG, ("[this=%p]     delta           %4dms\n", this, (a > b) ? (PRInt32)d : -(PRInt32)d));
 
     mStart = mStart2;
-    mStart2 = 0;
+    mStart2 = TimeStamp();
   }
 #endif
 
-  PRIntervalTime timeout = mTimeout;
-  if (mType == TYPE_REPEATING_PRECISE) {
+  TimeStamp timeout = mTimeout;
+  if (IsRepeatingPrecisely()) {
     // Precise repeating timers advance mTimeout by mDelay without fail before
     // calling Fire().
-    timeout -= PR_MillisecondsToInterval(mDelay);
+    timeout -= TimeDuration::FromMilliseconds(mDelay);
   }
   if (gThread)
     gThread->UpdateFilter(mDelay, timeout, now);
@@ -457,15 +454,19 @@ void nsTimerImpl::Fire()
 #ifdef DEBUG_TIMERS
   if (PR_LOG_TEST(gTimerLog, PR_LOG_DEBUG)) {
     PR_LOG(gTimerLog, PR_LOG_DEBUG,
-           ("[this=%p] Took %dms to fire timer callback\n",
-            this, PR_IntervalToMilliseconds(PR_IntervalNow() - now)));
+           ("[this=%p] Took %fms to fire timer callback\n",
+            this, (TimeStamp::Now() - now).ToMilliseconds()));
   }
 #endif
 
-  // Reschedule REPEATING_SLACK timers, but make sure that we aren't armed
-  // already (which can happen if the callback reinitialized the timer).
-  if (mType == TYPE_REPEATING_SLACK && !mArmed) {
-    SetDelayInternal(mDelay); // force mTimeout to be recomputed.
+  // Reschedule repeating timers, except REPEATING_PRECISE which already did
+  // that in PostTimerEvent, but make sure that we aren't armed already (which
+  // can happen if the callback reinitialized the timer).
+  if (IsRepeating() && mType != TYPE_REPEATING_PRECISE && !mArmed) {
+    if (mType == TYPE_REPEATING_SLACK)
+      SetDelayInternal(mDelay); // force mTimeout to be recomputed.  For
+                                // REPEATING_PRECISE_CAN_SKIP timers this has
+                                // already happened.
     if (gThread)
       gThread->AddTimer(this);
   }
@@ -483,7 +484,7 @@ public:
   }
 
 #ifdef DEBUG_TIMERS
-  PRIntervalTime mInitTime;
+  TimeStamp mInitTime;
 #endif
 
 private:
@@ -509,10 +510,10 @@ NS_IMETHODIMP nsTimerEvent::Run()
 
 #ifdef DEBUG_TIMERS
   if (PR_LOG_TEST(gTimerLog, PR_LOG_DEBUG)) {
-    PRIntervalTime now = PR_IntervalNow();
+    TimeStamp now = TimeStamp::Now();
     PR_LOG(gTimerLog, PR_LOG_DEBUG,
-           ("[this=%p] time between PostTimerEvent() and Fire(): %dms\n",
-            this, PR_IntervalToMilliseconds(now - mInitTime)));
+           ("[this=%p] time between PostTimerEvent() and Fire(): %fms\n",
+            this, (now - mInitTime).ToMilliseconds()));
   }
 #endif
 
@@ -536,15 +537,17 @@ nsresult nsTimerImpl::PostTimerEvent()
 
 #ifdef DEBUG_TIMERS
   if (PR_LOG_TEST(gTimerLog, PR_LOG_DEBUG)) {
-    event->mInitTime = PR_IntervalNow();
+    event->mInitTime = TimeStamp::Now();
   }
 #endif
 
   // If this is a repeating precise timer, we need to calculate the time for
   // the next timer to fire before we make the callback.
-  if (mType == TYPE_REPEATING_PRECISE) {
+  if (IsRepeatingPrecisely()) {
     SetDelayInternal(mDelay);
-    if (gThread) {
+
+    // But only re-arm REPEATING_PRECISE timers.
+    if (gThread && mType == TYPE_REPEATING_PRECISE) {
       nsresult rv = gThread->AddTimer(this);
       if (NS_FAILED(rv))
         return rv;
@@ -559,23 +562,19 @@ nsresult nsTimerImpl::PostTimerEvent()
 
 void nsTimerImpl::SetDelayInternal(PRUint32 aDelay)
 {
-  PRIntervalTime delayInterval = PR_MillisecondsToInterval(aDelay);
-  if (delayInterval > DELAY_INTERVAL_MAX) {
-    delayInterval = DELAY_INTERVAL_MAX;
-    aDelay = PR_IntervalToMilliseconds(delayInterval);
-  }
+  TimeDuration delayInterval = TimeDuration::FromMilliseconds(aDelay);
 
   mDelay = aDelay;
 
-  PRIntervalTime now = PR_IntervalNow();
-  if (mTimeout == 0 || mType != TYPE_REPEATING_PRECISE)
+  TimeStamp now = TimeStamp::Now();
+  if (mTimeout.IsNull() || mType != TYPE_REPEATING_PRECISE)
     mTimeout = now;
 
   mTimeout += delayInterval;
 
 #ifdef DEBUG_TIMERS
   if (PR_LOG_TEST(gTimerLog, PR_LOG_DEBUG)) {
-    if (mStart == 0)
+    if (mStart.IsNull())
       mStart = now;
     else
       mStart2 = now;
