@@ -40,7 +40,6 @@
 #include "nscore.h"
 #include "prprf.h"
 #include "plstr.h"
-#include "nsInt64.h"
 
 #include "nsISupports.h"
 #include "nsIServiceManager.h"
@@ -68,7 +67,7 @@
 #include "nsIWindowMediator.h"
 #include "nsIDocument.h"
 #include "nsIDOMDocument.h"
-#include "nsIDOMWindowInternal.h"
+#include "nsIDOMWindow.h"
 #include "nsDirectoryService.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsAppDirectoryServiceDefs.h"
@@ -199,7 +198,7 @@ nsXPInstallManager::InitManagerWithInstallInfo(nsIXPIInstallInfo* aInstallInfo)
     nsresult rv = aInstallInfo->GetTriggerInfo(&triggers);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCOMPtr<nsIDOMWindowInternal> win;
+    nsCOMPtr<nsIDOMWindow> win;
     rv = aInstallInfo->GetOriginatingWindow(getter_AddRefs(win));
     if (NS_SUCCEEDED(rv))
     {
@@ -218,7 +217,7 @@ nsXPInstallManager::InitManagerWithInstallInfo(nsIXPIInstallInfo* aInstallInfo)
 }
 
 NS_IMETHODIMP
-nsXPInstallManager::InitManager(nsIDOMWindowInternal* aParentWindow, nsXPITriggerInfo* aTriggers, PRUint32 aChromeType)
+nsXPInstallManager::InitManager(nsIDOMWindow* aParentWindow, nsXPITriggerInfo* aTriggers, PRUint32 aChromeType)
 {
     if ( !aTriggers || aTriggers->Size() == 0 )
     {
@@ -369,7 +368,7 @@ nsXPInstallManager::ConfirmInstall(nsIDOMWindow *aParent, const PRUnichar **aPac
 {
     *aRetval = PR_FALSE;
 
-    nsCOMPtr<nsIDOMWindowInternal> parentWindow( do_QueryInterface(aParent) );
+    nsCOMPtr<nsIDOMWindow> parentWindow = aParent;
     nsCOMPtr<nsIDialogParamBlock> params;
     nsresult rv = LoadParams( aCount, aPackageList, getter_AddRefs(params) );
 
@@ -413,7 +412,7 @@ nsXPInstallManager::ConfirmInstall(nsIDOMWindow *aParent, const PRUnichar **aPac
 }
 
 #ifdef ENABLE_SKIN_SIMPLE_INSTALLATION_UI
-PRBool nsXPInstallManager::ConfirmChromeInstall(nsIDOMWindowInternal* aParentWindow, const PRUnichar **aPackage)
+PRBool nsXPInstallManager::ConfirmChromeInstall(nsIDOMWindow* aParentWindow, const PRUnichar **aPackage)
 {
     // get the dialog strings
     nsXPIDLString applyNowText;
@@ -525,7 +524,7 @@ nsXPInstallManager::OpenProgressDialog(const PRUnichar **aPackageList, PRUint32
     if (NS_SUCCEEDED(rv) && !type.IsEmpty()) {
         nsCOMPtr<nsIWindowMediator> wm = do_GetService(NS_WINDOWMEDIATOR_CONTRACTID);
 
-        nsCOMPtr<nsIDOMWindowInternal> recentWindow;
+        nsCOMPtr<nsIDOMWindow> recentWindow;
         wm->GetMostRecentWindow(type.get(), getter_AddRefs(recentWindow));
         if (recentWindow) {
             nsCOMPtr<nsIObserverService> os =
@@ -1230,7 +1229,7 @@ nsXPInstallManager::OnDataAvailable(nsIRequest* request, nsISupports *ctxt,
                                     PRUint32 length)
 {
 #define XPI_ODA_BUFFER_SIZE 8*1024
-    PRUint32 amt = PR_MIN(XPI_ODA_BUFFER_SIZE, length);
+    PRUint32 amt = NS_MIN(XPI_ODA_BUFFER_SIZE, length);
     nsresult err;
     char buffer[XPI_ODA_BUFFER_SIZE];
     PRUint32 writeCount;
@@ -1257,7 +1256,7 @@ nsXPInstallManager::OnDataAvailable(nsIRequest* request, nsISupports *ctxt,
         }
         length -= amt;
 
-        amt = PR_MIN(XPI_ODA_BUFFER_SIZE, length);
+        amt = NS_MIN(XPI_ODA_BUFFER_SIZE, length);
 
     } while (length > 0);
 
